Package: exametrika
Type: Package
Title: Test Data Engineering
Version: 2.0.0
Authors@R: 
    person("Koji Kosugi", , , "kosugitti@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5816-0099"))
Description: Implements comprehensive test data engineering methods as described in 
    Shojima (2022, ISBN:978-9811699856). Provides statistical techniques for 
    engineering and processing test data: Classical Test Theory (CTT) with 
    reliability coefficients for continuous ability assessment; Item Response 
    Theory (IRT) including Rasch, 2PL, and 3PL models with item/test information 
    functions; Latent Class Analysis (LCA) for nominal clustering; Latent Rank 
    Analysis (LRA) for ordinal clustering with automatic determination of cluster 
    numbers; Biclustering methods including infinite relational models for 
    simultaneous clustering of examinees and items without predefined cluster 
    numbers; and Bayesian Network Models (BNM) for visualizing inter-item 
    dependencies. Features local dependence analysis through LRA and biclustering, 
    parameter estimation, dimensionality assessment, and network structure 
    visualization for educational, psychological, and social science research.
License: MIT + file LICENSE
Language: en-US
Encoding: UTF-8
LazyData: true
BuildVignettes: true
URL: https://kosugitti.github.io/exametrika/,
        https://github.com/kosugitti/exametrika
BugReports: https://github.com/kosugitti/exametrika/issues
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: true
Depends: R (>= 4.1.0)
Imports: mvtnorm, igraph, Rcpp (>= 1.0.7)
LinkingTo: Rcpp
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-08-20 01:22:39 UTC; newton
Author: Koji Kosugi [aut, cre] (ORCID: <https://orcid.org/0000-0001-5816-0099>)
Maintainer: Koji Kosugi <kosugitti@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-20 10:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-20 13:20:50 UTC; unix
Archs: exametrika.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(Biclustering,binary)
S3method(Biclustering,default)
S3method(Biclustering,nominal)
S3method(Biclustering,ordinal)
S3method(Biclustering,rated)
S3method(Biclustering_IRM,binary)
S3method(Biclustering_IRM,default)
S3method(Biclustering_IRM,nominal)
S3method(Biclustering_IRM,ordinal)
S3method(Biclustering_IRM,rated)
S3method(CCRR,binary)
S3method(CCRR,default)
S3method(CCRR,nominal)
S3method(Dimensionality,binary)
S3method(Dimensionality,default)
S3method(Dimensionality,ordinal)
S3method(Dimensionality,rated)
S3method(DistractorAnalysis,LRArated)
S3method(DistractorAnalysis,ratedBiclustering)
S3method(ITBiserial,binary)
S3method(ITBiserial,default)
S3method(InterItemAnalysis,binary)
S3method(InterItemAnalysis,default)
S3method(InterItemAnalysis,ordinal)
S3method(ItemEntropy,binary)
S3method(ItemEntropy,default)
S3method(ItemEntropy,ordinal)
S3method(ItemLift,binary)
S3method(ItemLift,default)
S3method(ItemOdds,binary)
S3method(ItemOdds,default)
S3method(ItemStatistics,binary)
S3method(ItemStatistics,default)
S3method(ItemStatistics,ordinal)
S3method(ItemThreshold,binary)
S3method(ItemThreshold,default)
S3method(ItemThreshold,ordinal)
S3method(ItemTotalCorr,binary)
S3method(ItemTotalCorr,default)
S3method(ItemTotalCorr,ordinal)
S3method(JCRR,binary)
S3method(JCRR,default)
S3method(JCRR,nominal)
S3method(JointSampleSize,binary)
S3method(JointSampleSize,default)
S3method(LCA,binary)
S3method(LCA,default)
S3method(LCA,nominal)
S3method(LCA,rated)
S3method(LRA,binary)
S3method(LRA,default)
S3method(LRA,ordinal)
S3method(LRA,rated)
S3method(M2,LRAordinal)
S3method(M2,default)
S3method(M2,nominalBiclustering)
S3method(M2,nominalLCA)
S3method(M2,ordinalBiclustering)
S3method(M2,ratedLCA)
S3method(MutualInformation,binary)
S3method(MutualInformation,default)
S3method(MutualInformation,ordinal)
S3method(PhiCoefficient,binary)
S3method(PhiCoefficient,default)
S3method(PolychoricCorrelationMatrix,default)
S3method(PolychoricCorrelationMatrix,ordinal)
S3method(TestStatistics,binary)
S3method(TestStatistics,default)
S3method(TestStatistics,ordinal)
S3method(TetrachoricCorrelationMatrix,binary)
S3method(TetrachoricCorrelationMatrix,default)
S3method(add_M2,LRAordinal)
S3method(add_M2,default)
S3method(add_M2,nominalBiclustering)
S3method(add_M2,nominalLCA)
S3method(add_M2,ordinalBiclustering)
S3method(add_M2,ratedLCA)
S3method(crr,binary)
S3method(crr,default)
S3method(nrs,binary)
S3method(nrs,default)
S3method(passage,binary)
S3method(passage,default)
S3method(percentile,binary)
S3method(percentile,default)
S3method(plot,DistractorAnalysis)
S3method(plot,exametrika)
S3method(print,DistractorAnalysis)
S3method(print,exametrika)
S3method(sscore,binary)
S3method(sscore,default)
S3method(stanine,binary)
S3method(stanine,default)
export(AlphaCoefficient)
export(BINET)
export(BNM)
export(BNM_GA)
export(BNM_PBIL)
export(Biclustering)
export(Biclustering_IRM)
export(BiserialCorrelation)
export(CCRR)
export(CSR)
export(CTT)
export(Dimensionality)
export(DistractorAnalysis)
export(GRM)
export(Glasso)
export(GridSearch)
export(IIF2PLM)
export(IIF3PLM)
export(IRT)
export(ITBiserial)
export(InterItemAnalysis)
export(ItemEntropy)
export(ItemFit)
export(ItemInformationFunc)
export(ItemLift)
export(ItemOdds)
export(ItemReport)
export(ItemStatistics)
export(ItemThreshold)
export(ItemTotalCorr)
export(JCRR)
export(JSR)
export(JointSampleSize)
export(LCA)
export(LDB)
export(LDLRA)
export(LDLRA_PBIL)
export(LRA)
export(LogisticModel)
export(M2)
export(MutualInformation)
export(OmegaCoefficient)
export(PhiCoefficient)
export(PolychoricCorrelationMatrix)
export(RaschModel)
export(ScoreReport)
export(StudentAnalysis)
export(TestFit)
export(TestFitSaturated)
export(TestInformationFunc)
export(TestResponseFunc)
export(TestStatistics)
export(TetrachoricCorrelationMatrix)
export(ThreePLM)
export(TwoPLM)
export(add_M2)
export(calcFitIndices)
export(chatterjee_matrix)
export(chatterjee_xi)
export(crr)
export(dataFormat)
export(grm_iif)
export(grm_prob)
export(longdataFormat)
export(nrs)
export(passage)
export(percentile)
export(polychoric)
export(polyserial)
export(sscore)
export(stanine)
export(tetrachoric)
export(xi_stable)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,dev.off)
importFrom(grDevices,gray)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,curve)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_data_frame)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_on_grid)
importFrom(igraph,layout_with_fr)
importFrom(igraph,plot.igraph)
importFrom(mvtnorm,pmvnorm)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(exametrika, .registration=TRUE)

<!-- README.md is generated from README.Rmd. Please edit README.Rmd, then run `Rscript -e 'devtools::build_readme()'` and commit both files. Direct edits to README.md will be overwritten. -->

# exametrika <img src="man/figures/logo.png" align="right" height="139" alt="exametrika logo" />

## Overview

The `exametrika` package provides comprehensive Test Data Engineering
tools for analyzing educational test data. Based on the methods
described in Shojima (2022), this package enables researchers and
practitioners to:

- Analyze test response patterns and item characteristics
- Classify respondents using various psychometric models
- Investigate latent structures in test data
- Examine local dependencies between items
- Perform network analysis of item relationships

The package implements both traditional psychometric approaches and
advanced statistical methods, making it suitable for various assessment
and research purposes.

## Features

The package implements various psychometric models and techniques:

#### Classical Methods

- Classical Test Theory (CTT)
  - Item difficulty and discrimination
  - Test reliability and validity
- Item Response Theory (IRT)
  - 2PL, 3PL, and 4PL models
  - Item characteristic curves
  - Test information functions

#### Latent Structure Analysis

- Latent Class Analysis (LCA)
  - Class membership estimation
  - Item response profiles
- Latent Rank Analysis (LRA)
  - Ordered latent classes
  - Rank transition probabilities
- Biclustering and Ranklustering
  - Simultaneous clustering of items and examinees
  - Field-specific response patterns
- Infinite Relational Model (IRM)
  - Optimal class/field determination
  - Nonparametric clustering

#### Advanced Network Models

- Bayesian Network Analysis
  - Structure Learning
    - Genetic Algorithm approach
    - Population-Based Incremental Learning (PBIL)
  - Conditional probability estimation
- Local Dependence Models
  - Local Dependence Latent Rank Analysis (LDLRA)
  - Local Dependence Biclustering (LDB)
  - Bicluster Network Model (BINET)

## Model Overview

### Local Dependence Models

The package implements three complementary approaches to modeling local
dependencies in test data:

1.  **LDLRA (Local Dependence Latent Rank Analysis)**
    - Analyzes how item dependencies change across different proficiency
      ranks
    - Suitable when item relationships are expected to vary by student
      ability level
    - Combines the strengths of LRA and Bayesian Networks
2.  **LDB (Local Dependence Biclustering)**
    - Focuses on relationships between item fields within each rank
    - Optimal when items naturally form groups (fields) with
      hierarchical relationships
    - Integrates biclustering with field-level dependency structures
3.  **BINET (Bicluster Network Model)**
    - Examines class transitions within each field
    - Best for understanding complex patterns of class progression
    - Combines biclustering with class-level network analysis

## Background

Exametrika was originally developed and published as a Mathematica and
Excel Add-in. For additional information about Exametrika, visit:

- [Test Data Engineering
  Website](http://sh0j1ma.stars.ne.jp/tde/index.htm)
- [Package News](NEWS.md)

## Installation

The development version of Exametrika can be installed from
[GitHub](https://github.com/):

``` r
# Install devtools if not already installed
if (!require("devtools")) install.packages("devtools")

# Install Exametrika
devtools::install_github("kosugitti/exametrika")
```

### Dependencies

The package requires:

- R (\>= 4.1.0)
- igraph (for network analysis)
- Other dependencies are automatically installed

## Data Format and Usage

### Basic Usage

``` r
library(exametrika)
```

### Data Requirements

Exametrika accepts both binary and polytomous response data:

- Binary data (0/1)
  - 0: Incorrect answer
  - 1: Correct answer
- Polytomous data
  - Ordinal response categories
  - Multiple score levels
- Missing values
  - NA values supported
  - Custom missing value codes can be specified

### Input Data Specifications

The package accepts data in several formats with the following features:

1.  **Data Structure**
    - Matrix or data.frame format
    - Response data (binary or polytomous)
    - Flexible handling of missing values
    - Support for various data types and structures
2.  **Optional Components**
    - Examinee ID column (default: first column)
    - Item weights (default: all weights = 1)
    - Item labels (default: sequential numbers)
    - Missing value indicator matrix

Note: Some analysis methods may have specific data type requirements.
Please refer to each function’s documentation for detailed requirements.

### Data Formatting

The `dataFormat` function processes input data before analysis:

- **Functions**
  - Extracts and validates ID vectors if present
  - Processes item labels or assigns sequential numbers
  - Creates response data matrix U
  - Generates missing value indicator matrix Z
  - Handles item weights
  - Converts data to appropriate format for analysis

Example:

``` r
# Format raw data for analysis
data <- dataFormat(J15S500) # Using sample dataset
str(data) # View structure of formatted data
```

### Sample Datasets

The package includes various sample datasets from Shojima (2022) for
testing and learning:

- **Naming Convention**: JxxSxxx format
  - J: Number of items (e.g., J15 = 15 items)
  - S: Sample size (e.g., S500 = 500 examinees)

Available datasets:

- J5S10: Very small dataset (5 items, 10 examinees)
  - Useful for quick testing and understanding basic concepts
- J12S5000: Large sample dataset (12 items, 5000 examinees)
  - Suitable for LDLRA and other advanced analyses
- J14S500: Medium dataset (14 items, 500 examinees)
- J15S500: Medium dataset (15 items, 500 examinees)
  - Often used in IRT and LCA examples
- J20S400: Medium dataset (20 items, 400 examinees)
- J35S515: Large item dataset (35 items, 515 examinees)
  - Used in Biclustering and network model examples
- J15S3810: Ordinal scale dataset (15 items with 4-point scale, 3810
  examinees)
  - Used in ordinal latent rank model examples
- J35S500: Ordinal polytomous dataset (35 items with 5 categories, 500
  examinees)
  - Used in ordinal Biclustering examples
- J20S600: Nominal polytomous dataset (20 items with 4 categories, 600
  examinees)
  - Used in nominal Biclustering examples
- J35S5000: Multiple-choice dataset (35 items, 5000 examinees)
  - Includes both response categories and correct answer data
  - Used in nominal scale latent rank model examples

## Examples

### Test Statistics

``` r
TestStatistics(J15S500)
```

### ItemStatistics

``` r
ItemStatistics(J15S500)
```

### CTT

``` r
CTT(J15S500)
```

### IRT

The IRT function estimates the number of parameters using a logistic
model, which can be specified using the `model` option. It supports 2PL,
3PL, and 4PL models.

``` r
result.IRT <- IRT(J15S500, model = 3)
result.IRT
```

The estimated population of subjects is included in the returned object.

``` r
head(result.IRT$ability)
```

The plots offer options for Item Response Function(also known as Item
Characteristic Curves (ICC)),Test Response Function, Item Information
Curves (IIC), and Test Information Curves (TIC), which can be specified
through options. Items can be specified using the `items` argument, and
if not specified, plots will be drawn for all items. The number of rows
and columns for dividing the plotting area can be specified using `nr`
and `nc`, respectively.

``` r
plot(result.IRT, type = "IRF", items = 1:6, nc = 2, nr = 3)
plot(result.IRT, type = "IRF", overlay = TRUE)
plot(result.IRT, type = "IIC", items = 1:6, nc = 2, nr = 3)
plot(result.IRT, type = "TRF")
plot(result.IRT, type = "TIC")
```

### GRM: IRT for Polytomous Cases

The Graded Response Model (Samejima, 1969) can be considered an
extension of IRT to polytomous response models. In this package, it can
be implemented using the GRM function. However, the estimation accuracy
is somewhat inferior to packages such as ltm, so it might be better to
use different packages for more sophisticated analyses.

``` r
result.GRM <- GRM(J5S1000)
result.GRM
```

Similar output to IRT is also possible.

``` r
plot(result.GRM, type = "IRF")
plot(result.GRM, type = "IIF")
plot(result.GRM, type = "TIF")
```

### LCA

Latent Class Analysis requires specifying the dataset and the number of
classes.

``` r
LCA(J15S500, ncls = 5)
```

The returned object contains the Class Membership Matrix, which
indicates which latent class each subject belongs to. The Estimate
includes the one with the highest membership probability.

``` r
result.LCA <- LCA(J15S500, ncls = 5)
head(result.LCA$Students)
```

The plots offer options for IRP, CMP, TRP, and LCD. For more details on
each, please refer to Shojima (2022).

``` r
plot(result.LCA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LCA, type = "CMP", students = 1:9, nc = 3, nr = 3)
plot(result.LCA, type = "TRP")
plot(result.LCA, type = "LCD")
```

### LRA

Latent Rank Analysis requires specifying the dataset and the number of
classes.

``` r
LRA(J15S500, nrank = 6)
```

The estimated subject rank membership probabilities and plots are almost
the same as those in LCA (Latent Class Analysis). Since a ranking is
assumed for the latent classes, rank-up odds and rank-down odds are
calculated.

``` r
result.LRA <- LRA(J15S500, nrank = 6)
head(result.LRA$Students)
```

``` r
plot(result.LRA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LRA, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.LRA, type = "TRP")
plot(result.LRA, type = "LRD")
```

### LRA for ordinal data

LRA can also be applied to ordinal scale data. The sample dataset
J15S3810 contains responses to 15 items on a 4-point scale, which we’ll
classify into 3 ranks. The mic option enforces monotonic increasing
constraints.

``` r
result.LRAord <- LRA(J15S3810, nrank = 3, mic = TRUE)
```

We can visualize the relationship between total scores from the ordinal
scale and estimated ranks. ScoreFreq plots a frequency polygon of scores
with rank thresholds, while ScoreRank shows the relationship between
scores and rank membership probabilities as a heatmap.

``` r
plot(result.LRAord, type = "ScoreFreq")
plot(result.LRAord, type = "ScoreRank")
```

The relationship between items and ranks can be visualized in two
complementary ways using ICBR and ICRP plots. These visualizations help
understand how items function across different ranks:

- ICBR (Item Category Boundary Reference) shows the cumulative
  probability curves for each category threshold. For each item, these
  lines represent the probability of scoring at or above each category
  boundary across ranks.
- ICRP (Item Category Response Profile) displays the probability of
  selecting each response category across ranks. These lines show how
  response patterns change as rank increases.

``` r
plot(result.LRAord, type = "ICBR", items = 1:4, nc = 2, nr = 2)
plot(result.LRAord, type = "ICRP", items = 1:4, nc = 2, nr = 2)
```

Similar to binary data output, we can examine individual examinee
characteristics through rank membership probability plots. This
visualization shows the probability distribution of rank membership for
each examinee, allowing us to understand the certainty of rank
classifications. For the first 15 examinees in the dataset:

``` r
plot(result.LRAord, type = "RMP", students = 1:9, nc = 3, nr = 3)
```

Note: The layout parameters nc = 3 and nr = 5 control the arrangement of
plots in a 3-column by 5-row grid, making it easier to compare multiple
items or examinees simultaneously.

### LRA for rated data

If you have data where respondents select the correct answer from
multiple choices, like in a multiple-choice test (nominal scale level),
you can analyze it using LRA.

``` r
result.LRArated <- LRA(J35S5000, nrank = 10, mic = TRUE)
```

You can visualize the relationship between scores and ranks, just like
with ordinal scale data.

``` r
plot(result.LRArated, type = "ScoreFreq")
plot(result.LRArated, type = "ScoreRank")
```

You can also visualize the relationship between latent ranks and items,
or the probability of subjects belonging to certain ranks.

``` r
plot(result.LRArated, type = "ICRP", items = 1:4, nc = 2, nr = 2)
```

``` r
plot(result.LRAord, type = "RMP", students = 1:9, nc = 3, nr = 3)
```

### Biclustering/Ranklustering

Biclustering and Ranklustering algorithms are almost the same, differing
only in whether they include a filtering matrix or not. The difference
is specified using the `method` option in the `Biclustering()` function.
For more details, please refer to the help documentation.

``` r
## Biclustering
Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
```

``` r
## Ranklustering
result.Ranklustering <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")
plot(result.Ranklustering, type = "Array")
plot(result.Ranklustering, type = "FRP", nc = 2, nr = 3)
plot(result.Ranklustering, type = "RRV")
plot(result.Ranklustering, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.Ranklustering, type = "LRD")
```

### Finding Optimal Number of Classes and Fields

#### Grid Search function

The Grid Search function performs systematic exploration of optimal
numbers of classes and fields based on statistical fit indices. This
approach evaluates multiple parameter combinations and selects the
best-fitting model according to specified criteria such as AIC, BIC, or
other goodness-of-fit measures.

``` r
result <- GridSearch(J35S515, method = "R", max_ncls = 10, max_nfld = 10, index = "BIC")
result$optimal_ncls
result$optimal_nfld
result$optimal_result
```

#### Infinite Relational Model

The Infinite Relational Model uses the Chinese Restaurant Process to
explore the optimal number of fields and classes automatically. This
model is particularly useful when you don’t know the appropriate number
of latent structures beforehand.

**Important notes:** - Execution can be time-consuming, especially with
large datasets - The model tends to detect larger numbers of classes and
fields - Use with caution and consider computational resources

``` r
result.IRM <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result.IRM, type = "Array")
plot(result.IRM, type = "FRP", nc = 3)
plot(result.IRM, type = "TRP")
```

### Biclustering for Polytomous Data

Biclustering and Rankclustering can also be performed on data with
ordinal or nominal polytomous responses. Missing values are displayed in
black in the Array plot. Color palettes can be customized using the
`colors` option to specify different color schemes.

#### Ordinal data

``` r
result.B.ord <- Biclustering(J35S500, ncls = 5, nfld = 5, method = "R")
result.B.ord
plot(result.B.ord, type = "Array")
```

For ordinal polytomous Biclustering, several new plot types are
available. The FRP (Field Reference Profile) shows the expected score
per field across latent ranks. The `stat` parameter controls the summary
statistic: “mean” (default), “median”, or “mode”.

``` r
plot(result.B.ord, type = "FRP", nc = 3, nr = 2)
```

The FCRP (Field Category Response Profile) shows the probability of each
response category across latent ranks. The `style` parameter can be
“line” (default) or “bar”.

``` r
plot(result.B.ord, type = "FCRP", nc = 3, nr = 2)
plot(result.B.ord, type = "FCRP", style = "bar", nc = 3, nr = 2)
```

The FCBR (Field Cumulative Boundary Reference) shows the cumulative
boundary probabilities P(Q \>= q) for each field. This plot type is only
available for ordinal data.

``` r
plot(result.B.ord, type = "FCBR", nc = 3, nr = 2)
```

The ScoreField plot displays a heatmap of expected scores across fields
and latent ranks.

``` r
plot(result.B.ord, type = "ScoreField")
```

The RRV (Rank Reference Vector) is a transposed view of the FRP, with
fields on the x-axis and latent rank lines overlaid for comparison.

``` r
plot(result.B.ord, type = "RRV")
```

#### Nominal data

``` r
result.B.nom <- Biclustering(J20S600, ncls = 5, nfld = 4)
result.B.nom
plot(result.B.nom, type = "Array")
```

Nominal polytomous Biclustering also supports FRP, FCRP, ScoreField, and
RRV plots. Note that FCBR is not available for nominal data because
boundary probabilities only make sense for ordinal categories.

``` r
plot(result.B.nom, type = "FRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", style = "bar", nc = 2, nr = 2)
plot(result.B.nom, type = "ScoreField")
plot(result.B.nom, type = "RRV")
```

### Bayesian Network Model

The Bayesian network model is a model that represents the conditional
probabilities between items in a network format based on the pass rates
of the items. By providing a Directed Acyclic Graph (DAG) between items
externally, it calculates the conditional probabilities based on the
specified graph. The igraph package is used for the analysis and
representation of the network.

There are three ways to specify the graph. You can either pass a
matrix-type DAG to the argument adj_matrix, pass a DAG described in a
CSV file to the argument adj_file, or pass a graph-type object g used in
the igraph package to the argument g.

The methods to create the matrix-type adj_matrix and the graph object g
are as follows:

``` r
library(igraph)
DAG <-
  matrix(
    c(
      "Item01", "Item02",
      "Item02", "Item03",
      "Item02", "Item04",
      "Item03", "Item05",
      "Item04", "Item05"
    ),
    ncol = 2, byrow = T
  )
## graph object
g <- igraph::graph_from_data_frame(DAG)
g
## Adjacency matrix
adj_mat <- as.matrix(igraph::get.adjacency(g))
print(adj_mat)
```

A CSV file with the same information as the graph above in the following
format. The first line contains column names (headers) and will not be
read as data.

While only one specification is sufficient, if multiple specifications
are provided, they will be prioritized in the order of file, matrix, and
graph object.

An example of executing BNM by providing a graph structure (DAG) is as
follows:

``` r
result.BNM <- BNM(J5S10, adj_matrix = adj_mat)
result.BNM
```

#### Structure Learning for Bayesian network with GA

The function searches for a DAG suitable for the data using a genetic
algorithm. A best DAG is not necessarily identified. Instead of
exploring all combinations of nodes and edges, only the space
topologically sorted by the pass rate, namely the upper triangular
matrix of the adjacency matrix, is explored. For interpretability, the
number of parent nodes should be limited. A null model is not proposed.
Utilize the content of the items and the experience of the questioner to
aid in interpreting the results. For more details, please refer to
Section 8.5 of the text(Shojima,2022).

Please note that the GA may take a considerable amount of time,
depending on the number of items and the size of the population.

``` r
BNM_GA(J5S10,
  population = 20, Rs = 0.5, Rm = 0.002, maxParents = 2,
  maxGeneration = 100, crossover = 2, elitism = 2
)
```

The method of Population-Based incremental learning proposed by Fukuda
(2014) can also be used for learning. This method has several variations
for estimating the optimal adjacency matrix at the end, which can be
specified as options. See help or text Section 8.5.2.

``` r
BNM_PBIL(J5S10,
  population = 20, Rs = 0.5, Rm = 0.005, maxParents = 2,
  alpha = 0.05, estimate = 4
)
```

### Local Dependence Latent Rank Analysis

LD-LRA is an analysis that combines LRA and BNM, and it is used to
analyze the network structure among items in the latent rank. In this
function, structural learning is not performed, so you need to provide
item graphs for each rank as separate files.

For each class, it is necessary to specify a graph, and there are three
ways to do so. You can either pass a matrix-type DAG for each class or a
list of graph-type objects used in the igraph package to the arguments
adj_list or g_list, respectively, or you can provide a DAG described in
a CSV file. The way to specify it in a CSV file is as follows.

``` r
DAG_dat <- matrix(c(
  "From", "To", "Rank",
  "Item01", "Item02", 1,
  "Item04", "Item05", 1,
  "Item01", "Item02", 2,
  "Item02", "Item03", 2,
  "Item04", "Item05", 2,
  "Item08", "Item09", 2,
  "Item08", "Item10", 2,
  "Item09", "Item10", 2,
  "Item08", "Item11", 2,
  "Item01", "Item02", 3,
  "Item02", "Item03", 3,
  "Item04", "Item05", 3,
  "Item08", "Item09", 3,
  "Item08", "Item10", 3,
  "Item09", "Item10", 3,
  "Item08", "Item11", 3,
  "Item02", "Item03", 4,
  "Item04", "Item06", 4,
  "Item04", "Item07", 4,
  "Item05", "Item06", 4,
  "Item05", "Item07", 4,
  "Item08", "Item10", 4,
  "Item08", "Item11", 4,
  "Item09", "Item11", 4,
  "Item02", "Item03", 5,
  "Item04", "Item06", 5,
  "Item04", "Item07", 5,
  "Item05", "Item06", 5,
  "Item05", "Item07", 5,
  "Item09", "Item11", 5,
  "Item10", "Item11", 5,
  "Item10", "Item12", 5
), ncol = 3, byrow = TRUE)

# save csv file
edgeFile <- tempfile(fileext = ".csv")
write.csv(DAG_dat, edgeFile, row.names = FALSE, quote = TRUE)
```

Here, it is shown an example of specifying with matrix-type and graph
objects using the aforementioned CSV file. While only one specification
is sufficient, if multiple specifications are provided, they will be
prioritized in the order of file, matrix, and graph object.

``` r
g_csv <- read.csv(edgeFile)
colnames(g_csv) <- c("From", "To", "Rank")
adj_list <- list()
g_list <- list()
for (i in 1:5) {
  adj_R <- g_csv[g_csv$Rank == i, 1:2]
  g_tmp <- igraph::graph_from_data_frame(adj_R)
  adj_tmp <- igraph::get.adjacency(g_tmp)
  g_list[[i]] <- g_tmp
  adj_list[[i]] <- adj_tmp
}
## Example of graph list
g_list
```

``` r
### Example of adjacency list
adj_list
```

The example of running the LDLRA function using this CSV file would look
like this.

``` r
result.LDLRA <- LDLRA(J12S5000,
  ncls = 5,
  adj_file = edgeFile
)
result.LDLRA
```

Of course, it also supports various types of plots.

``` r
plot(result.LDLRA, type = "IRP", nc = 4, nr = 3)
plot(result.LDLRA, type = "TRP")
plot(result.LDLRA, type = "LRD")
```

#### Structure Learning for LDLRA with GA(PBIL)

You can learn item-interaction graphs for each rank using the PBIL
algorithm. In addition to various options, the learning process requires
a very long computation time. It’s also important to note that the
result is merely one of the feasible solutions, and it’s not necessarily
the optimal solution.

``` r
result.LDLRA.PBIL <- LDLRA_PBIL(J35S515,
  seed = 123,
  ncls = 5,
  method = "R",
  elitism = 1,
  successiveLimit = 15
)
result.LDLRA.PBIL
```

### Local Dependence Biclustering

Local Dependence Biclustering combines biclustering and Bayesian network
models. The model requires three main components:

- Number of latent classes/ranks
- Field assignments for items
- Network structure between fields at each rank

Here’s an example implementation:

``` r
# Create field configuration vector (assign items to fields)
conf <- c(1, 6, 6, 8, 9, 9, 4, 7, 7, 7, 5, 8, 9, 10, 10, 9, 9, 10, 10, 10, 2, 2, 3, 3, 5, 5, 6, 9, 9, 10, 1, 1, 7, 9, 10)

# Create edge data for network structure between fields
edges_data <- data.frame(
  "From Field (Parent) >>>" = c(
    6, 4, 5, 1, 1, 4, # Class/Rank 2
    3, 4, 6, 2, 4, 4, # Class/Rank 3
    3, 6, 4, 1, # Class/Rank 4
    7, 9, 6, 7 # Class/Rank 5
  ),
  ">>> To Field (Child)" = c(
    8, 7, 8, 7, 2, 5, # Class/Rank 2
    5, 8, 8, 4, 6, 7, # Class/Rank 3
    5, 8, 5, 8, # Class/Rank 4
    10, 10, 8, 9 # Class/Rank 5
  ),
  "At Class/Rank (Locus)" = c(
    2, 2, 2, 2, 2, 2, # Class/Rank 2
    3, 3, 3, 3, 3, 3, # Class/Rank 3
    4, 4, 4, 4, # Class/Rank 4
    5, 5, 5, 5 # Class/Rank 5
  )
)

# Save edge data to temporary file
edgeFile <- tempfile(fileext = ".csv")
write.csv(edges_data, file = edgeFile, row.names = FALSE)
```

Additionally, as mentioned in the text (Shojima, 2022), it is often the
case that seeking the network structure exploratively does not yield
appropriate results, so it has not been implemented.

``` r
result.LDB <- LDB(U = J35S515, ncls = 5, conf = conf, adj_file = edgeFile)
result.LDB
```

Of course, it also supports various types of plots.

``` r
# Show bicluster structure
plot(result.LDB, type = "Array")
# Test Response Profile
plot(result.LDB, type = "TRP")
# Latent Rank Distribution
plot(result.LDB, type = "LRD")
# Rank Membership Profiles for first 9 students
plot(result.LDB, type = "RMP", students = 1:9, nc = 3, nr = 3)
# Field Reference Profiles
plot(result.LDB, type = "FRP", nc = 3, nr = 2)
```

In this model, you can draw a Field PIRP Profile that visualizes the
correct answer count for each rank and each field.

``` r
plot(result.LDB, type = "FieldPIRP")
```

### Bicluster Network Model

Bicluster Network Model: BINET is a model that combines the Bayesian
network model and Biclustering. BINET is very similar to LDB and LDR.

The most significant difference is that in LDB, the nodes represent the
fields, whereas in BINET, they represent the class. BINET explores the
local dependency structure among latent classes at each latent field,
where each field is a locus.

To execute this analysis, in addition to the dataset, the same field
correspondence file used during exploratory Biclustering is required, as
well as an adjacency matrix between classes.

``` r
# Create field configuration vector for item assignment
conf <- c(1, 5, 5, 5, 9, 9, 6, 6, 6, 6, 2, 7, 7, 11, 11, 7, 7, 12, 12, 12, 2, 2, 3, 3, 4, 4, 4, 8, 8, 12, 1, 1, 6, 10, 10)

# Create edge data for network structure between classes
edges_data <- data.frame(
  "From Class (Parent) >>>" = c(
    1, 2, 3, 4, 5, 7, # Dependencies in various fields
    2, 4, 6, 8, 10,
    6, 6, 11, 8, 9, 12
  ),
  ">>> To Class (Child)" = c(
    2, 4, 5, 5, 6, 11, # Target classes
    3, 7, 9, 12, 12,
    10, 8, 12, 12, 11, 13
  ),
  "At Field (Locus)" = c(
    1, 2, 2, 3, 4, 4, # Field locations
    5, 5, 5, 5, 5,
    7, 8, 8, 9, 9, 12
  )
)

# Save edge data to temporary file
edgeFile <- tempfile(fileext = ".csv")
write.csv(edges_data, file = edgeFile, row.names = FALSE)
```

The model requires three components:

1.  Field assignments for items (vector from configuration file)
2.  Network structure between classes for each field
3.  Number of classes and fields

``` r
# Fit Bicluster Network Model
result.BINET <- BINET(
  U = J35S515,
  ncls = 13, # Maximum class number from edges (13)
  nfld = 12, # Maximum field number from conf (12)
  conf = conf, # Field configuration vector
  adj_file = edgeFile # Network structure file
)

# Display model results
print(result.BINET)
```

Of course, it also supports various types of plots.

``` r
# Show bicluster structure
plot(result.BINET, type = "Array")
# Test Response Profile
plot(result.BINET, type = "TRP")
# Latent Rank Distribution
plot(result.BINET, type = "LRD")
# Rank Membership Profiles for first 9 students
plot(result.BINET, type = "RMP", students = 1:9, nc = 3, nr = 3)
# Field Reference Profiles
plot(result.BINET, type = "FRP", nc = 3, nr = 2)
```

LDPSR plot shows all Passing Student Rates for all locally dependent
classes compared with their respective parents.

``` r
# Locally Dependent Passing Student Rates
plot(result.BINET, type = "LDPSR", nc = 3, nr = 2)
```

### Available Output Types by Model

#### Pattern Analysis

| Model                  | IRP | FRP | TRP | ICRP | C/R RV |
|------------------------|:---:|:---:|:---:|:----:|:------:|
| IRT                    |     |     |     |      |        |
| LCA                    |  ✓  |  ✓  |  ✓  |      |        |
| LRA                    |  ✓  |  ✓  |  ✓  |      |        |
| LRAordinal             |     |     |     |  ✓   |        |
| LRArated               |     |     |     |  ✓   |        |
| Biclustering           |     |  ✓  |  ✓  |      |   ✓    |
| Biclustering (ordinal) |     |  ✓  |     |      |   ✓    |
| Biclustering (nominal) |     |  ✓  |     |      |   ✓    |
| IRM                    |     |  ✓  |  ✓  |      |        |
| LDLRA                  |  ✓  |     |     |      |        |
| LDB                    |     |  ✓  |  ✓  |      |        |
| BINET                  |     |  ✓  |  ✓  |      |        |

#### Diagnostics & Visualization

| Model                  | LCD/LRD | CMP/RMP | Array | Other                      |
|------------------------|:-------:|:-------:|:-----:|----------------------------|
| IRT                    |         |         |       | IIC, ICC, TIC              |
| LCA                    |    ✓    |    ✓    |       |                            |
| LRA                    |    ✓    |    ✓    |       |                            |
| LRAordinal             |         |    ✓    |       | ICBR, ScoreFreq, ScoreRank |
| LRArated               |         |    ✓    |       | ScoreFreq, ScoreRank       |
| Biclustering           |    ✓    |    ✓    |   ✓   |                            |
| Biclustering (ordinal) |    ✓    |    ✓    |   ✓   | FCRP, FCBR, ScoreField     |
| Biclustering (nominal) |    ✓    |    ✓    |   ✓   | FCRP, ScoreField           |
| IRM                    |         |         |   ✓   |                            |
| LDLRA                  |    ✓    |    ✓    |       |                            |
| LDB                    |    ✓    |    ✓    |   ✓   | FieldPIRP                  |
| BINET                  |    ✓    |    ✓    |   ✓   | LDPSR                      |

Note: ✓ indicates available output type for the model.

## Community and Support

We welcome community involvement and feedback to improve `exametrika`.
Here’s how you can participate and get support:

### Reporting Issues

If you encounter bugs or have suggestions for improvements:

- Open an issue on [GitHub
  Issues](https://github.com/kosugitti/exametrika/issues)
- Provide a minimal reproducible example
- Include your R session information (`sessionInfo()`)

### Discussions and Community

Join our [GitHub
Discussions](https://github.com/kosugitti/exametrika/discussions):

- Ask questions
- Share your use cases
- Discuss feature requests
- Exchange tips and tricks
- Get updates about package development

### Contributing

We appreciate contributions from the community:

- Bug reports and feature requests through Issues
- Usage examples and tips through Discussions
- Code improvements through Pull Requests

Please check our existing Issues and Discussions before posting to avoid
duplicates.

## Reference

- Shojima, Kojiro (2022) Test Data Engineering: Latent Rank Analysis,
  Biclustering, and Bayesian Network (Behaviormetrics: Quantitative
  Approaches to Human Behavior, 13),Springer.
- Samejima, F. (1969). Estimation of latent ability using a response
  pattern of graded scores. Psychometrika, 34(S1), 1-97.

Follow our [GitHub repository](https://github.com/kosugitti/exametrika)
and join the
[Discussions](https://github.com/kosugitti/exametrika/discussions) to
stay updated on development progress and provide feedback on desired
features.

## Citation

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.11529926.svg)](https://doi.org/10.5281/zenodo.11529926)

#' @title Structure Learning for BNM by simple GA
#' @description
#' Generating a DAG from data using a genetic algorithm.
#' @details
#' This function generates a DAG from data using a genetic algorithm.
#' Depending on the size of the data and the settings, the computation may
#' take a significant amount of computational time. For details on the
#' settings or algorithm, see Shojima(2022), section 8.5
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param seed seed for random.
#' @param population Population size. The default is 20
#' @param Rs Survival Rate. The default is 0.5
#' @param Rm Mutation Rate. The default is 0.005
#' @param maxParents Maximum number of edges emanating from a single node. The default is 2.
#' @param maxGeneration Maximum number of generations.
#' @param successiveLimit Termination conditions. If the optimal individual does not change
#' for this number of generations, it is considered to have converged.
#' @param crossover Configure crossover using numerical values. Specify 0 for uniform
#' crossover, where bits are randomly copied from both parents. Choose 1 for single-point
#' crossover with one crossover point, and 2 for two-point crossover with two crossover points.
#' The default is 0.
#' @param elitism Number of elites that remain without crossover when transitioning to
#' the next generation.
#' @param filename Specify the filename when saving the generated adjacency matrix in CSV format.
#' The default is null, and no output is written to the file.
#' @param verbose verbose output Flag. default is TRUE
#' @importFrom igraph as_adjacency_matrix
#' @importFrom igraph graph_from_adjacency_matrix
#' @importFrom igraph as_data_frame
#' @importFrom utils write.table
#' @importFrom utils combn
#' @return
#' \describe{
#'  \item{adj}{Optimal adjacency matrix}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{crr}{correct response ratio}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{adj}{Adjacency matrix}
#'  \item{param}{Learned Parameters}
#'  \item{CCRR_table}{Correct Response Rate tables}
#' }
#' @importFrom stats rbinom
#' @examples
#' \donttest{
#' # Perform Structure Learning for Bayesian Network Model using Genetic Algorithm
#' # Parameters are set for balanced exploration and computational efficiency
#' BNM_GA(J5S10,
#'   population = 20, # Size of population in each generation
#'   Rs = 0.5, # 50% survival rate for next generation
#'   Rm = 0.002, # 0.2% mutation rate for genetic diversity
#'   maxParents = 2, # Maximum of 2 parent nodes per item
#'   maxGeneration = 100, # Maximum number of evolutionary steps
#'   crossover = 2, # Use two-point crossover method
#'   elitism = 2 # Keep 2 best solutions in each generation
#' )
#' }
#' @export

BNM_GA <- function(U, Z = NULL, w = NULL, na = NULL,
                   seed = 123,
                   population = 20, Rs = 0.5, Rm = 0.005,
                   maxParents = 2, maxGeneration = 100,
                   successiveLimit = 5, crossover = 0,
                   elitism = 0, filename = NULL,
                   verbose = TRUE) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  # crossover type
  if (crossover < 0 || crossover > 2) {
    stop("Check the crossover type. It should be set as 0, 1 or 2.")
  }

  set.seed(seed)
  crr <- crr(tmp)
  sort_list <- order(crr, decreasing = TRUE)
  adj_sort <- data.frame(item = tmp$ItemLabel, crr = crr)
  adj <- matrix(0, ncol = testlength, nrow = testlength)
  adj[upper.tri(adj)] <- 1
  colnames(adj) <- rownames(adj) <- tmp$ItemLabel[sort_list]
  gene_length <- sum(upper.tri(adj))


  # Initialize ------------------------------------------------------
  RsI <- round(population * Rs)

  adj <- matrix(0, ncol = testlength, nrow = testlength)
  colnames(adj) <- rownames(adj) <- tmp$ItemLabel[sort_list]
  gene_list <- matrix(0, nrow = population, ncol = gene_length)
  for (j in 1:population) {
    adj_gene <- rbinom(gene_length, 1, 0.5)
    gene_list[j, ] <- maxParents_penalty(adj_gene, testlength, maxParents)
  }
  # Elitism check
  if (elitism < 0) {
    elitism <- 0
  } else if (elitism > population * Rs * 0.5) {
    elitism <- round(population * Rs * 0.5)
    warning("Too many elites. Limit to ", elitism)
  }

  bestfit <- 1e+100
  limit_count <- 0
  GA_FLG <- TRUE
  generation <- 0

  # simple GA -------------------------------------------------------

  while (GA_FLG) {
    generation <- generation + 1
    if (verbose) {
      message(
        "\rgen. ", generation, " best BIC ", format(bestfit, digits = 6),
        " limit count ", limit_count,
        appendLF = FALSE
      )
    }
    fitness <- numeric(population)
    for (i in 1:population) {
      # make adj from genes
      adj[upper.tri(adj)] <- gene_list[i, ]
      # Fitness
      ret <- BNM(tmp, adj_matrix = adj)
      fitness[i] <- ret$TestFitIndices$BIC
    }

    # Termination check
    if (generation > maxGeneration) {
      if (verbose) {
        message("The maximum generation has been reached")
      }
      GA_FLG <- FALSE
    }
    if (sort(fitness)[1] == bestfit) {
      limit_count <- limit_count + 1
    } else {
      limit_count <- 0
      bestfit <- sort(fitness)[1]
    }
    if (limit_count >= successiveLimit) {
      if (verbose) {
        message("The BIC has not changed for ", successiveLimit, " times.")
      }
      GA_FLG <- FALSE
    }

    # Selection
    new_gene_list <- matrix(NA, nrow = population, ncol = gene_length)
    survivers <- order(fitness)[1:RsI]
    new_gene_list[1:RsI, ] <- gene_list[survivers, ]

    # elitism
    if (elitism > 0) {
      new_gene_list[(RsI + 1):(RsI + elitism), ] <-
        new_gene_list[1:elitism, ]
    }
    # new gene
    for (i in (RsI + elitism + 1):population) {
      parents_num <- sample(1:(RsI + elitism), size = 2)
      parent1 <- new_gene_list[parents_num[1], ]
      parent2 <- new_gene_list[parents_num[2], ]
      # CrossOver
      if (crossover == 2) {
        # two-points crossover
        combinations <- combn(1:gene_length, 2)
        # Filter out combinations where the two numbers are adjacent
        combinations <- combinations[, abs(combinations[1, ] - combinations[2, ]) > 1]
        # Exclude combinations that start with 1 and 9
        combinations <- combinations[, combinations[1, ] != 1]
        combinations <- combinations[, combinations[2, ] != (gene_length)]
        cut_off <- combinations[, sample(1:ncol(combinations), size = 1)]
        child <- c(
          parent1[1:(cut_off[1] - 1)],
          parent2[cut_off[1]:cut_off[2]],
          parent1[(cut_off[2] + 1):gene_length]
        )
      } else if (crossover == 1) {
        # single point crossover
        cut_off <- sample(2:(gene_length - 1), 1)
        child <- c(parent1[1:cut_off], parent2[(cut_off + 1):gene_length])
      } else {
        # uniform crossover
        inherent <- sample(parents_num, size = gene_length, replace = TRUE)
        child <- numeric(gene_length)
        for (pos in 1:gene_length) {
          child[pos] <- new_gene_list[inherent[pos], pos]
        }
      }
      # Mutation
      Bm <- rbinom(gene_length, 1, Rm)
      child <- (1 - Bm) * child + Bm * (1 - child)

      # Omit Null Model
      if (sum(child) == 0) {
        pos <- sample(1:gene_length, 1)
        child[pos] <- 1
      }

      child <- maxParents_penalty(child, testlength, maxParents)

      ## new_gene
      new_gene_list[i, ] <- child
    }

    gene_list <- new_gene_list
    old_fitness <- fitness
  }
  adj_best <- gene_list[1, ]
  adj[upper.tri(adj)] <- adj_best
  GA_g <- graph_from_adjacency_matrix(adj)

  ret <- BNM(tmp, g = GA_g)

  if (!is.null(filename)) {
    write.table(igraph::as_data_frame(GA_g),
      sep = ",",
      row.names = FALSE,
      col.names = TRUE,
      quote = FALSE,
      file = filename
    )
  }

  return(ret)
}


#' @title Structure Learning for BNM by PBIL
#' @description
#' Generating a DAG from data using a Population-Based Incremental Learning
#' @details
#' This function performs structural learning using the Population-Based
#' Incremental Learning model(PBIL) proposed by Fukuda et al.(2014) within
#' the genetic algorithm framework. Instead of learning the adjacency matrix
#' itself, the 'genes of genes' that generate the adjacency matrix are updated
#' with each generation. For more details, please refer to Fukuda(2014) and Section
#' 8.5.2 of the text(Shojima,2022).
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param seed seed for random.
#' @param population Population size. The default is 20
#' @param Rs Survival Rate. The default is 0.5
#' @param Rm Mutation Rate. The default is 0.002
#' @param maxParents Maximum number of edges emanating from a single node. The default is 2.
#' @param maxGeneration Maximum number of generations.
#' @param successiveLimit Termination conditions. If the optimal individual does not change
#' for this number of generations, it is considered to have converged.
#' @param elitism Number of elites that remain without crossover when transitioning to
#' the next generation.
#' @param alpha Learning rate. The default is 0.05
#' @param estimate In PBIL for estimating the adjacency matrix, specify by number from the
#' following four methods: 1. Optimal adjacency matrix, 2. Rounded average of individuals in
#' the last generation, 3. Rounded average of survivors in the last generation, 4. Rounded
#' generational gene of the last generation. The default is 1.
#' @param filename Specify the filename when saving the generated adjacency matrix in CSV format.
#' The default is null, and no output is written to the file.
#' @param verbose verbose output Flag. default is TRUE
#' @importFrom igraph as_adjacency_matrix
#' @importFrom igraph graph_from_adjacency_matrix
#' @importFrom igraph as_data_frame
#' @importFrom utils write.table
#' @importFrom utils combn
#' @return
#' \describe{
#'  \item{adj}{Optimal adjacency matrix}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{crr}{correct response ratio}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{param}{Learned Parameters}
#'  \item{CCRR_table}{Correct Response Rate tables}
#' }
#' @references Fukuda, S., Yamanaka, Y., & Yoshihiro, T. (2014). A Probability-based evolutionary
#'  algorithm with mutations to learn Bayesian networks. International Journal of Artificial
#'  Intelligence and Interactive Multimedia, 3, 7–13. DOI: 10.9781/ijimai.2014.311
#' @importFrom stats runif rbinom
#' @examples
#' \donttest{
#' # Perform Structure Learning for Bayesian Network Model using PBIL
#' # (Population-Based Incremental Learning)
#' BNM_PBIL(J5S10,
#'   population = 20, # Size of population in each generation
#'   Rs = 0.5, # 50% survival rate for next generation
#'   Rm = 0.005, # 0.5% mutation rate for genetic diversity
#'   maxParents = 2, # Maximum of 2 parent nodes per item
#'   alpha = 0.05, # Learning rate for probability update
#'   estimate = 4 # Use rounded generational gene method
#' )
#' }
#' @export

BNM_PBIL <- function(U, Z = NULL, w = NULL, na = NULL,
                     seed = 123,
                     population = 20, Rs = 0.5, Rm = 0.002,
                     maxParents = 2, maxGeneration = 100,
                     successiveLimit = 5, elitism = 0,
                     alpha = 0.05, estimate = 1,
                     filename = NULL,
                     verbose = TRUE) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  # estimate type
  if (estimate < 1 || estimate > 4) {
    stop("Check the estimate type. It should be set between 1 to 4.")
  }

  RsI <- round(population * Rs)
  # Elitism check
  if (RsI < 0) {
    RsI <- 0
  } else if (RsI > population * Rs * 0.5) {
    RsI <- round(population * Rs * 0.5)
    warning("Too many survivers. Limit to ", RsI)
  }

  # Elitism check
  if (elitism < 0) {
    elitism <- 0
  } else if (elitism > population * Rs * 0.5) {
    elitism <- round(population * Rs * 0.5)
    warning("Too many elites. Limit to ", elitism)
  }

  # Initialize ------------------------------------------------------

  set.seed(seed)
  crr <- crr(tmp)
  sort_list <- order(crr, decreasing = TRUE)
  adj_sort <- data.frame(item = tmp$ItemLabel, crr = crr)
  adj <- matrix(0, ncol = testlength, nrow = testlength)
  adj[upper.tri(adj)] <- 1
  colnames(adj) <- rownames(adj) <- tmp$ItemLabel[sort_list]
  gene_length <- sum(upper.tri(adj))

  gene <- rep(0.5, gene_length)
  adj_t <- matrix(0, nrow = population, ncol = gene_length)

  bestfit <- 1e+100
  limit_count <- 0

  GA_FLG <- TRUE
  generation <- 0
  for (i in 1:population) {
    adj_t[i, ] <- rbinom(gene_length, 1, prob = gene)
  }
  best_individual <- adj_t[1, ]


  # PBIL GA ---------------------------------------------------------
  while (GA_FLG) {
    generation <- generation + 1
    if (verbose) {
      message(
        sprintf(
          "\r%-80s",
          paste0(
            "gen. ", generation, " best BIC ", format(bestfit, digits = 6),
            " limit count ", limit_count
          )
        ),
        appendLF = FALSE
      )
    }
    fitness <- numeric(population)
    for (i in 1:population) {
      # out of elite
      if (i > elitism) {
        vec <- rbinom(gene_length, 1, prob = gene)
        # Omit Null Model
        if (sum(vec) == 0) {
          pos <- sample(1:gene_length, 1)
          vec[pos] <- 1
        }
        adj_t[i, ] <- maxParents_penalty(vec, testlength, maxParents)
      }
      adj[upper.tri(adj)] <- adj_t[i, ]
      # Fitness
      ret <- BNM(tmp, adj_matrix = adj)
      fitness[i] <- ret$TestFitIndices$BIC
    }
    sort_list <- order(fitness)
    adj_t <- adj_t[sort_list, ]

    ### Update Gne
    ar <- round(colMeans(adj_t[1:RsI, ]))
    gene <- gene + (alpha * (ar - gene))

    ### mutation
    u <- runif(gene_length, min = 0, max = 1)
    Bm <- rbinom(gene_length, 1, Rm)
    gene <- (1 - Bm) * gene + Bm * (gene + u) / 2

    # Termination check
    if (generation > maxGeneration) {
      if (verbose) {
        message("The maximum generation has been reached")
      }
      GA_FLG <- FALSE
    }
    if (all(best_individual == adj_t[1, ])) {
      limit_count <- limit_count + 1
    } else {
      fitness <- sort(fitness)
      bestfit <- fitness[1]
      best_individual <- adj_t[1, ]
      limit_count <- 0
    }
    if (limit_count >= successiveLimit) {
      if (verbose) {
        message("The BIC has not changed for ", successiveLimit, " times.")
      }
      GA_FLG <- FALSE
    }
  }

  # Estimate --------------------------------------------------------
  if (estimate == 1) {
    adj[upper.tri(adj)] <- adj_t[1, ]
  } else if (estimate == 2) {
    adj[upper.tri(adj)] <- round(colMeans(adj_t))
  } else if (estimate == 3) {
    adj[upper.tri(adj)] <- round(colMeans(adj_t[1:RsI, ]))
  } else {
    adj[upper.tri(adj)] <- round(gene)
  }

  GA_g <- graph_from_adjacency_matrix(adj)

  ret <- BNM(tmp, g = GA_g)

  if (!is.null(filename)) {
    write.table(igraph::as_data_frame(GA_g),
      sep = ",",
      row.names = FALSE,
      col.names = TRUE,
      quote = FALSE,
      file = filename
    )
  }

  return(ret)
}

#' @title StrLearningGA_BNM (Deprecated)
#' @description
#' This function has been renamed to \code{\link{BNM_GA}}.
#' Please use \code{BNM_GA} instead.
#' @param ... All arguments passed to \code{\link{BNM_GA}}
#' @export
StrLearningGA_BNM <- function(...) {
  .Deprecated("BNM_GA")
  BNM_GA(...)
}

#' @title StrLearningPBIL_BNM (Deprecated)
#' @description
#' This function has been renamed to \code{\link{BNM_PBIL}}.
#' Please use \code{BNM_PBIL} instead.
#' @param ... All arguments passed to \code{\link{BNM_PBIL}}
#' @export
StrLearningPBIL_BNM <- function(...) {
  .Deprecated("BNM_PBIL")
  BNM_PBIL(...)
}

# Polytomous Biclustering plot functions (ordinal / nominal)
# Internal functions called from plot.exametrika()

#' Compute expected score matrix from BCRM (shared helper for FRP / RRV)
#' @noRd
calc_expected_scores <- function(BCRM, stat) {
  nfld <- dim(BCRM)[1]
  ncls <- dim(BCRM)[2]
  maxQ <- dim(BCRM)[3]

  FRP_mat <- matrix(0, nrow = nfld, ncol = ncls)
  for (f in 1:nfld) {
    for (cc in 1:ncls) {
      probs <- BCRM[f, cc, ]
      if (stat == "mean") {
        FRP_mat[f, cc] <- sum((1:maxQ) * probs)
      } else if (stat == "median") {
        cum_probs <- cumsum(probs)
        FRP_mat[f, cc] <- min(which(cum_probs >= 0.5))
      } else if (stat == "mode") {
        FRP_mat[f, cc] <- which.max(probs)
      }
    }
  }
  FRP_mat
}

#' FRP: expected score line plot
#' @noRd
plot_poly_frp <- function(x, stat, nc, nr, dots = list()) {
  BCRM <- x$FRP
  nfld <- dim(BCRM)[1]
  ncls <- dim(BCRM)[2]
  maxQ <- dim(BCRM)[3]
  msg <- x$msg

  FRP_mat <- calc_expected_scores(BCRM, stat)

  for (f in 1:nfld) {
    y <- FRP_mat[f, ]
    call_plot(
      plot,
      list(
        x = 1:ncls, y = y,
        type = "b", pch = 19, lwd = 2,
        xlab = paste("Latent", msg),
        ylab = paste0("Expected Score (", stat, ")"),
        ylim = c(1, maxQ),
        xaxt = "n",
        main = paste("Field", f)
      ),
      dots
    )
    call_plot(graphics::axis, list(side = 1, at = 1:ncls), dots)
    abline(h = 1:maxQ, col = "gray90", lty = 3)
  }
}

#' FCRP: category probability plot
#' @noRd
plot_poly_fcrp <- function(x, style, nc, nr, dots = list()) {
  BCRM <- x$FRP
  nfld <- dim(BCRM)[1]
  ncls <- dim(BCRM)[2]
  maxQ <- dim(BCRM)[3]
  msg <- x$msg
  cols <- get_cb_palette(maxQ)

  setup_legend_layout(nfld, nc)

  for (f in 1:nfld) {
    if (style == "line") {
      call_plot(
        plot,
        list(
          x = 1:ncls, y = BCRM[f, , 1],
          type = "n",
          xlab = paste("Latent", msg),
          ylab = "Category Probability",
          ylim = c(0, 1),
          xaxt = "n",
          main = paste("Field", f, "- Category Response")
        ),
        dots
      )
      call_plot(graphics::axis, list(side = 1, at = 1:ncls), dots)
      for (q in 1:maxQ) {
        call_plot(
          graphics::lines,
          list(
            x = 1:ncls, y = BCRM[f, , q],
            type = "b", pch = q, lty = q, col = cols[q], lwd = 1.5
          ),
          dots
        )
      }
    } else if (style == "bar") {
      bar_data <- t(BCRM[f, , ])
      call_plot(
        graphics::barplot,
        list(
          height = bar_data,
          col = cols[1:maxQ],
          names.arg = paste0(substr(msg, 1, 1), 1:ncls),
          xlab = paste("Latent", msg),
          ylab = "Category Probability",
          ylim = c(0, 1),
          main = paste("Field", f, "- Category Response")
        ),
        dots
      )
    }
  }

  if (style == "line") {
    draw_legend_strip(
      legend = paste("Cat", 1:maxQ),
      col = cols[1:maxQ], lty = 1:maxQ, pch = 1:maxQ,
      cex = 1.0, bty = "n", ncol = min(maxQ, 5)
    )
  } else {
    draw_legend_strip(
      legend = paste("Cat", 1:maxQ),
      fill = cols[1:maxQ],
      cex = 1.0, bty = "n", ncol = min(maxQ, 5)
    )
  }
}

#' FCBR: boundary probability plot (ordinal Biclustering only)
#' @noRd
plot_poly_fcbr <- function(x, nc, nr, dots = list()) {
  BCRM <- x$FRP
  nfld <- dim(BCRM)[1]
  ncls <- dim(BCRM)[2]
  maxQ <- dim(BCRM)[3]
  msg <- x$msg
  n_boundaries <- maxQ - 1
  cols <- get_cb_palette(n_boundaries)

  setup_legend_layout(nfld, nc)

  for (f in 1:nfld) {
    boundary_probs <- matrix(0, nrow = n_boundaries, ncol = ncls)
    for (b in 1:n_boundaries) {
      q_threshold <- b + 1
      for (cc in 1:ncls) {
        boundary_probs[b, cc] <- sum(BCRM[f, cc, q_threshold:maxQ])
      }
    }

    call_plot(
      plot,
      list(
        x = 1:ncls, y = boundary_probs[1, ],
        type = "n",
        xlab = paste("Latent", msg),
        ylab = "Boundary Probability",
        ylim = c(0, 1),
        xaxt = "n",
        main = paste("Field", f, "- Boundary Prob")
      ),
      dots
    )
    call_plot(graphics::axis, list(side = 1, at = 1:ncls), dots)
    # P(Q >= 1) = 1.0 reference line
    call_plot(
      graphics::lines,
      list(
        x = 1:ncls, y = rep(1, ncls),
        type = "b", pch = 0, lty = 1, col = "gray60", lwd = 1
      ),
      dots
    )
    for (b in 1:n_boundaries) {
      call_plot(
        graphics::lines,
        list(
          x = 1:ncls, y = boundary_probs[b, ],
          type = "b", pch = b, lty = b, col = cols[b], lwd = 1.5
        ),
        dots
      )
    }
  }

  draw_legend_strip(
    legend = c("P(Q>=1)", paste0("P(Q>=", 2:maxQ, ")")),
    col = c("gray60", cols[1:n_boundaries]),
    lty = c(1, 1:n_boundaries),
    pch = c(0, 1:n_boundaries),
    cex = 1.0, bty = "n", ncol = min(maxQ, 5)
  )
}

#' ScoreField: expected score heatmap
#' @noRd
plot_scorefield <- function(x, dots = list()) {
  BCRM <- x$FRP
  nfld <- dim(BCRM)[1]
  ncls <- dim(BCRM)[2]
  maxQ <- dim(BCRM)[3]
  msg <- x$msg

  score_mat <- matrix(0, nrow = nfld, ncol = ncls)
  for (f in 1:nfld) {
    for (cc in 1:ncls) {
      score_mat[f, cc] <- sum((1:maxQ) * BCRM[f, cc, ])
    }
  }

  par(mfrow = c(1, 1))
  call_plot(
    graphics::image,
    list(
      x = 1:ncls, y = 1:nfld,
      z = t(score_mat),
      col = hcl.colors(50, "YlOrRd", rev = TRUE),
      xlab = paste("Latent", msg),
      ylab = "Field",
      main = "Score Field Heatmap",
      xaxt = "n", yaxt = "n"
    ),
    dots
  )
  call_plot(graphics::axis, list(side = 1, at = 1:ncls, labels = paste0(substr(msg, 1, 1), 1:ncls)), dots)
  call_plot(graphics::axis, list(side = 2, at = 1:nfld, labels = paste0("F", 1:nfld)), dots)

  for (f in 1:nfld) {
    for (cc in 1:ncls) {
      text(cc, f, sprintf("%.2f", score_mat[f, cc]), cex = 0.8)
    }
  }
}

#' RRV: transposed field-axis version (polytomous Biclustering)
#' @noRd
plot_poly_rrv <- function(x, stat, dots = list()) {
  BCRM <- x$FRP
  nfld <- dim(BCRM)[1]
  ncls <- dim(BCRM)[2]
  maxQ <- dim(BCRM)[3]
  msg <- x$msg

  FRP_mat <- calc_expected_scores(BCRM, stat)
  RRV <- t(FRP_mat)

  par(mfrow = c(1, 1))
  call_plot(
    plot,
    list(
      x = 1:nfld, y = RRV[1, ],
      type = "n",
      ylim = c(1, maxQ),
      xlab = "Field",
      ylab = paste0("Expected Score (", stat, ")"),
      main = paste(msg, "Reference Vector (", stat, ")"),
      xaxt = "n", bty = "n"
    ),
    dots
  )
  call_plot(graphics::axis, list(side = 1, at = 1:nfld, labels = paste0("F", 1:nfld)), dots)
  abline(h = 1:maxQ, col = "gray90", lty = 3)

  cols <- get_cb_palette(ncls)
  for (i in 1:ncls) {
    call_plot(graphics::lines, list(x = 1:nfld, y = RRV[i, ], type = "o", lty = i, col = cols[i], lwd = 1.5), dots)
    for (j in 1:nfld) {
      text(j, RRV[i, j], labels = i, pos = 3, offset = 0.5, cex = 0.8)
    }
  }
  legend("top",
    legend = paste0(substr(msg, 1, 1), 1:ncls),
    lty = 1:ncls, col = cols[1:ncls], lwd = 2,
    ncol = min(ncls, 5), bty = "n"
  )
}

#' @title Soft thresholding operator
#' @param y Input value
#' @param lambda Threshold value
#' @return Soft-thresholded value
#' @noRd

soft_thresholding <- function(y, lambda) {
  sign(y) * pmax(abs(y) - lambda, 0)
}


#' @title Single-lambda Graphical Lasso
#' @description
#' Estimates a precision matrix at a single lambda value via block coordinate
#' descent with cyclical coordinate descent for the inner lasso step.
#' @param S Sample covariance or correlation matrix (p x p)
#' @param lambda Regularization parameter (scalar)
#' @param W_init Initial working covariance matrix. If NULL, initialized as S + diag(lambda, p)
#' @param Beta_init Initial beta cache (p-1 x p matrix). If NULL, initialized as zeros
#' @param eps Convergence tolerance for both inner and outer loops
#' @param max_iter Maximum number of outer iterations
#' @return A list with elements Theta (precision matrix), W (working covariance),
#'   Beta (beta cache), niter (number of outer iterations), converged (logical)
#' @noRd

glasso_one <- function(S, lambda, W_init = NULL, Beta_init = NULL, eps = 1e-6, max_iter = 1000, ...) {
  niter <- 0
  p <- ncol(S)

  if (is.null(W_init)) {
    W_init <- S + diag(lambda, ncol(S))
  }
  if (is.null(Beta_init)) {
    Beta_init <- matrix(0, nrow = nrow(S) - 1, ncol = ncol(S))
  }

  ## Size check

  if (sum(dim(S) != dim(W_init)) != 0) {
    stop("wrong size Winit")
  }
  if (sum(dim(Beta_init) != c(p - 1, p)) != 0) {
    stop("wrong size Beta init")
  }

  W <- W_init
  beta_cache <- Beta_init


  W_old <- W
  W_FLG <- TRUE
  Theta <- matrix(0, nrow = p, ncol = p)
  diverged <- FALSE # numerical divergence flag (NaN/Inf in BCD)

  while (W_FLG) {
    niter <- niter + 1
    for (j in 1:p) {
      W11 <- W[-j, -j]
      s12 <- S[-j, j]
      beta <- beta_cache[, j]
      beta_FLG <- TRUE
      while (beta_FLG) {
        beta_old <- beta
        for (k in 1:(p - 1)) {
          r <- s12[k] - W11[k, ] %*% beta + W11[k, k] * beta[k]
          beta[k] <- soft_thresholding(r, lambda) / W11[k, k]
        }
        if (any(!is.finite(beta))) {
          diverged <- TRUE
          beta_FLG <- FALSE
          break
        }
        diff <- sum(abs(beta - beta_old))
        if (!is.finite(diff)) {
          diverged <- TRUE
          beta_FLG <- FALSE
          break
        }
        if (diff < eps) {
          beta_cache[, j] <- beta
          beta_FLG <- FALSE
        }
      }
      if (diverged) break
      W12 <- W11 %*% beta
      W[-j, j] <- W[j, -j] <- W12

      theta22 <- 1 / (W[j, j] - sum(W12 * beta))
      theta12 <- -beta * as.numeric(theta22)
      Theta[j, j] <- theta22
      Theta[-j, j] <- theta12
      Theta[j, -j] <- theta12
    }
    if (diverged) break
    if (!all(is.finite(W))) {
      diverged <- TRUE
      break
    }
    if (max(abs(W - W_old)) < eps) {
      W_FLG <- FALSE
    } else {
      W_old <- W
    }
    if (niter > max_iter) {
      break
    }
  }

  return(list(
    Theta = Theta,
    W = W,
    Beta = beta_cache,
    niter = niter,
    converged = (niter <= max_iter) && !diverged
  ))
}

#' @title Extended Bayesian Information Criterion for Graphical Lasso
#' @description
#' Computes the EBIC of Foygel and Drton (2010) for a given precision matrix estimate.
#' @param S Sample covariance or correlation matrix
#' @param Theta Estimated precision matrix
#' @param n Sample size
#' @param p Number of variables
#' @param gamma EBIC tuning parameter (default 0.5)
#' @return Scalar EBIC value
#' @noRd

compute_EBIC_glasso <- function(S, Theta, n, p, gamma = 0.5) {
  term1 <- n * sum(S * Theta) - n * as.numeric(determinant(Theta, logarithm = TRUE)$modulus)
  E <- sum(abs(Theta[upper.tri(Theta)]) > 1e-6)
  term2 <- E * log(n)
  term3 <- 4 * gamma * E * log(p)
  EBIC <- term1 + term2 + term3
  return(EBIC)
}


#' @title Graphical Lasso for Gaussian Graphical Models
#' @description
#' Estimates a sparse precision matrix from ordinal item response data via
#' the Graphical Lasso (Friedman, Hastie, Tibshirani 2008). The polychoric
#' correlation matrix is used as input. The optimal regularization parameter
#' lambda is selected by the Extended Bayesian Information Criterion
#' (Foygel and Drton 2010).
#' @details
#' The Graphical Lasso estimates the precision matrix Theta by maximizing
#' the penalized log-likelihood:
#'
#'   log det(Theta) - tr(S Theta) - lambda * ||Theta||_1
#'
#' subject to Theta being positive definite. Optimization is performed by
#' block coordinate descent (Algorithm 17.2 of Hastie, Tibshirani, Friedman
#' 2009) with cyclical coordinate descent for the inner lasso step.
#'
#' Lambda is searched on a log-scale grid from lambda_max (the maximum
#' absolute off-diagonal of S) down to lambda_max * lambda_ratio. For each
#' lambda, the EBIC is computed and the lambda minimizing EBIC is returned.
#' Warm-starting (W and beta cache from the previous lambda) is used to
#' accelerate the search.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#'   it is converted to the exametrika class with the [dataFormat] function.
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param gamma EBIC tuning parameter in `[0, 1]`. Default is 0.5.
#' @param n_lambda Number of lambda values in the search grid. Default is 50.
#' @param lambda_ratio Ratio of lambda_min to lambda_max. Default is 0.01.
#' @param penalize_diagonal Logical. If TRUE, the L1 penalty is applied to
#'   the diagonal of Theta. Default is TRUE.
#' @param max_iter Maximum number of outer iterations for the block
#'   coordinate descent algorithm. Default is 100.
#' @param eps Convergence tolerance. Default is 1e-6.
#' @param edge_tol Threshold below which an off-diagonal element of Theta is
#'   considered zero (no edge). Default is 1e-8.
#' @param verbose Logical. If TRUE, progress messages are displayed.
#' @param ... Additional arguments (currently unused; reserved for future extensions).
#' @return
#' \describe{
#'   \item{theta}{Estimated precision matrix at the optimal lambda.}
#'   \item{W}{Working covariance matrix at the optimal lambda.}
#'   \item{lambda_opt}{Selected lambda value.}
#'   \item{gamma}{EBIC tuning parameter used.}
#'   \item{ebic_opt}{Minimum EBIC value.}
#'   \item{n_edge}{Number of edges in the selected model.}
#'   \item{path}{Data frame with columns lambda, ebic, n_edge over the search grid.}
#' }
#' @references
#' Friedman, J., Hastie, T., and Tibshirani, R. (2008). Sparse inverse
#' covariance estimation with the graphical lasso. Biostatistics, 9(3), 432-441.
#'
#' Foygel, R., and Drton, M. (2010). Extended Bayesian Information Criteria
#' for Gaussian Graphical Models. Advances in Neural Information Processing
#' Systems 23.
#' @seealso [PolychoricCorrelationMatrix]
#' @examples
#' \donttest{
#' # Estimate a sparse precision matrix from ordinal data
#' result.Glasso <- Glasso(J15S3810)
#' result.Glasso$lambda_opt
#' result.Glasso$n_edge
#' }
#' @export

Glasso <- function(U,
                   na = NULL,
                   Z = NULL,
                   w = NULL,
                   gamma = 0.5,
                   n_lambda = 50,
                   lambda_ratio = 0.01,
                   penalize_diagonal = TRUE,
                   max_iter = 100,
                   eps = 1e-6,
                   edge_tol = 1e-8,
                   verbose = FALSE,
                   ...) {
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (tmp$response.type != "ordinal") {
    response_type_error(tmp$response.type, "Glasso")
  }

  p <- NCOL(tmp$Q)
  nobs <- NROW(tmp$Q)
  if (verbose) message("Computing polychoric correlation matrix...")
  S <- PolychoricCorrelationMatrix(tmp)

  lambda_max <- max(abs(S - diag(diag(S))))
  lambda_min <- lambda_max * lambda_ratio
  lambda_set <- exp(seq(log(lambda_max), log(lambda_min), length.out = n_lambda))

  if (verbose) message("Searching optimal lambda by EBIC...")

  best_ebic <- Inf
  best_Theta <- NULL
  best_W <- NULL
  best_lambda <- NA
  best_index <- NA

  # Initialise with NA so that lambda values skipped by an early break
  # remain visible as NA in the returned path data.frame.
  EBICs <- rep(NA_real_, n_lambda)
  n_edges <- rep(NA_integer_, n_lambda)

  W_prev <- NULL
  Beta_prev <- NULL
  for (k in seq_along(lambda_set)) {
    l <- lambda_set[k]
    if (k == 1) {
      if (penalize_diagonal) {
        W_init <- S + diag(l, p)
      } else {
        W_init <- S
      }
      Beta_init <- matrix(0, p - 1, p)
    } else {
      W_init <- W_prev
      if (penalize_diagonal) {
        diag(W_init) <- diag(S) + l
      } else {
        diag(W_init) <- diag(S)
      }
      Beta_init <- Beta_prev
    }

    res <- glasso_one(S,
      lambda = l, W_init = W_init, Beta_init = Beta_init,
      eps = eps, max_iter = max_iter
    )
    if (!res$converged) {
      if (is.null(best_Theta)) {
        stop(sprintf(
          "Glasso could not be computed at the first lambda (%.4f). The polychoric correlation matrix may be severely ill-conditioned (e.g. N < p). Try increasing max_iter or raising lambda_ratio.",
          l
        ))
      } else {
        warning(sprintf(
          "Glasso diverged at lambda = %.4f (k=%d of %d). Stopping lambda search; returning the best solution found so far (lambda = %.4f, EBIC = %.2f).",
          l, k, n_lambda, best_lambda, best_ebic
        ))
        break
      }
    }
    ebic_k <- tryCatch(
      compute_EBIC_glasso(S, res$Theta, n = nobs, p = p, gamma = gamma),
      error = function(e) NA_real_
    )
    if (!is.finite(ebic_k)) {
      if (is.null(best_Theta)) {
        stop(sprintf(
          "EBIC could not be computed at the first lambda (%.4f).", l
        ))
      } else {
        warning(sprintf(
          "EBIC could not be computed at lambda = %.4f. Stopping lambda search; returning the best solution found so far (lambda = %.4f).",
          l, best_lambda
        ))
        break
      }
    }
    edges_k <- sum(abs(res$Theta[upper.tri(res$Theta)]) > edge_tol)

    EBICs[k] <- ebic_k
    n_edges[k] <- edges_k

    if (ebic_k < best_ebic) {
      best_ebic <- ebic_k
      best_Theta <- res$Theta
      best_W <- res$W
      best_lambda <- l
      best_index <- k
    }


    W_prev <- res$W
    Beta_prev <- res$Beta
  }

  ret <- structure(
    list(
      theta = best_Theta,
      W = best_W,
      lambda_opt = best_lambda,
      gamma = gamma,
      ebic_opt = best_ebic,
      n_edge = n_edges[best_index],
      path = data.frame(
        lambda = lambda_set,
        ebic = EBICs,
        n_edge = n_edges
      )
    ),
    class = c("exametrika", "Glasso")
  )

  return(ret)
}

# Edge case tests for dataFormat function
# Tests for unusual/abnormal input patterns

# ============================================================
# Phase 1: ID column identification
# ============================================================

test_that("id=NULL auto-detects numeric consecutive IDs (1:N)", {
  set.seed(201)
  data <- data.frame(
    id = 1:15,
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(ncol(result$Q), 2)
  expect_equal(result$ID, as.character(1:15))
})

test_that("id=1 explicitly treats column 1 as ID", {
  set.seed(202)
  data <- data.frame(
    id = 1:15,
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  result <- dataFormat(data, id = 1)
  expect_equal(ncol(result$Q), 2)
  expect_equal(result$ID, as.character(1:15))
})

test_that("id=NULL auto-detects string IDs in first column", {
  set.seed(203)
  data <- data.frame(
    student = paste0("S", 1:15),
    item1 = sample(0:1, 15, replace = TRUE),
    item2 = sample(0:1, 15, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(ncol(result$U), 2)
  expect_equal(result$ID, paste0("S", 1:15))
})

test_that("id=NULL generates auto IDs when first column is response data + message", {
  set.seed(204)
  # First column has duplicates and looks like response data
  data <- data.frame(
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE),
    item3 = sample(1:4, 15, replace = TRUE)
  )
  expect_message(result <- dataFormat(data), "No ID column detected")
  expect_equal(ncol(result$Q), 3)
  expect_equal(result$ID[1], "Student1")
})

test_that("explicit id for non-first column works", {
  set.seed(205)
  data <- data.frame(
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE),
    student = paste0("S", 1:15)
  )
  result <- dataFormat(data, id = 3)
  expect_equal(ncol(result$Q), 2)
  expect_equal(result$ID, paste0("S", 1:15))
})

test_that("NA in ID column causes error (duplicated NAs)", {
  data <- data.frame(
    id = c(paste0("S", 1:13), NA, NA),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  expect_error(dataFormat(data, id = 1), "Duplicated IDs found")
})

test_that("id column number exceeds ncol causes error", {
  data <- data.frame(
    item1 = sample(0:1, 15, replace = TRUE),
    item2 = sample(0:1, 15, replace = TRUE)
  )
  expect_error(dataFormat(data, id = 5), "ID column number exceeds")
})

test_that("rownames lost by unclass: auto-generated IDs used instead", {
  # NOTE: as.data.frame(unclass(data)) on line 61 resets rownames to 1:n.
  # The rownames-as-ID path is effectively dead code in the current implementation.
  set.seed(206)
  data <- data.frame(
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  rownames(data) <- paste0("Subj", 1:15)
  expect_message(result <- dataFormat(data), "No ID column detected")
  # Rownames are lost, so auto-generated IDs are used
  expect_equal(result$ID[1], "Student1")
})

# ============================================================
# Phase 2: Response type detection
# ============================================================

test_that("CA provided without response.type → auto-detect as rated", {
  set.seed(211)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(1:4, 20, replace = TRUE),
    item2 = sample(1:4, 20, replace = TRUE),
    item3 = sample(1:4, 20, replace = TRUE)
  )
  result <- dataFormat(data, CA = c(2, 3, 1))
  expect_equal(result$response.type, "rated")
  expect_true(!is.null(result$U))
  expect_true(!is.null(result$Q))
  expect_equal(result$CA, c(2, 3, 1))
})

test_that("CA with numeric ID (1:N) → rated with correct ID detection", {
  data <- data.frame(
    id = 1:15,
    item1 = c(1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3),
    item2 = c(2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3, 1)
  )
  result <- dataFormat(data, CA = c(3, 2))
  expect_equal(result$response.type, "rated")
  expect_equal(ncol(result$U), 2)
  expect_equal(ncol(result$Q), 2)
})

test_that("rated U matrix correctly scores against CA", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 2, 3, 1, 2, 3, 1, 2, 3, 1),
    item2 = c(3, 3, 3, 1, 1, 1, 2, 2, 2, 3)
  )
  result <- dataFormat(data, CA = c(2, 3))
  # item1: CA=2 → S2,S5,S8 correct
  expect_equal(result$U[, 1], c(0, 1, 0, 0, 1, 0, 0, 1, 0, 0))
  # item2: CA=3 → S1,S2,S3,S10 correct
  expect_equal(result$U[, 2], c(1, 1, 1, 0, 0, 0, 0, 0, 0, 1))
})

test_that("rated U matrix preserves missing as -1", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, NA, 3, 1, 2, 3, 1, 2, 3, 1),
    item2 = c(3, 3, NA, 1, 1, 1, 2, 2, 2, 3)
  )
  result <- dataFormat(data, CA = c(2, 3))
  expect_equal(result$U[2, 1], -1) # NA in item1 → -1 in U
  expect_equal(result$U[3, 2], -1) # NA in item2 → -1 in U
  # Non-missing values still scored correctly
  expect_equal(result$U[1, 1], 0) # 1 != CA(2)
  expect_equal(result$U[1, 2], 1) # 3 == CA(3)
})

test_that("CA length mismatch causes error", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  expect_error(dataFormat(data, CA = c(1, 2, 3)), "length of CA must match")
})

test_that("CA value not in data warns (was error pre-v1.13.1)", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE)
  )
  expect_warning(
    dataFormat(data, CA = c(5, 1)),
    "not observed in responses"
  )
})

test_that("2-category non-binary (1/2) without response.type causes error", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:2, 15, replace = TRUE),
    item2 = sample(1:2, 15, replace = TRUE)
  )
  expect_error(dataFormat(data), "2-category data with non-binary values")
})

test_that("2-category non-binary with explicit response.type works", {
  set.seed(213)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:2, 15, replace = TRUE),
    item2 = sample(1:2, 15, replace = TRUE)
  )
  result <- dataFormat(data, response.type = "ordinal")
  expect_equal(result$response.type, "ordinal")
})

test_that(">=20 categories without response.type causes error", {
  set.seed(214)
  data <- data.frame(
    id = paste0("S", 1:100),
    item1 = sample(1:25, 100, replace = TRUE),
    item2 = sample(1:25, 100, replace = TRUE)
  )
  expect_error(dataFormat(data), "Too many categories")
})

test_that(">=20 categories with explicit response.type works", {
  set.seed(215)
  data <- data.frame(
    id = paste0("S", 1:100),
    item1 = sample(1:25, 100, replace = TRUE),
    item2 = sample(1:25, 100, replace = TRUE)
  )
  result <- dataFormat(data, response.type = "ordinal")
  expect_equal(result$response.type, "ordinal")
})

test_that("binary 0/1 auto-detected correctly", {
  set.seed(216)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(0:1, 20, replace = TRUE),
    item2 = sample(0:1, 20, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(result$response.type, "binary")
  expect_true(!is.null(result$U))
})

test_that("binary data with CA still becomes rated", {
  set.seed(217)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(0:1, 20, replace = TRUE),
    item2 = sample(0:1, 20, replace = TRUE)
  )
  result <- dataFormat(data, CA = c(1, 1))
  expect_equal(result$response.type, "binary")
  # Binary check runs first, CA check only for non-binary
})

test_that("explicit response.type overrides auto-detection", {
  set.seed(218)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(1:5, 20, replace = TRUE),
    item2 = sample(1:5, 20, replace = TRUE)
  )
  result <- dataFormat(data, response.type = "nominal")
  expect_equal(result$response.type, "nominal")
})

test_that("invalid response.type causes error", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(0:1, 15, replace = TRUE)
  )
  expect_error(dataFormat(data, response.type = "invalid"))
})

# ============================================================
# Size anomalies
# ============================================================

test_that("fewer than 10 rows causes error", {
  data <- data.frame(
    id = paste0("S", 1:5),
    item1 = c(0, 1, 0, 1, 0),
    item2 = c(1, 0, 1, 0, 1)
  )
  expect_error(dataFormat(data), "at least 10 rows")
})

test_that("exactly 10 rows works (boundary)", {
  set.seed(221)
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = sample(0:1, 10, replace = TRUE),
    item2 = sample(0:1, 10, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
  expect_equal(nrow(result$U), 10)
})

test_that("0 rows causes error", {
  data <- data.frame(
    id = character(0),
    item1 = numeric(0),
    item2 = numeric(0)
  )
  expect_error(dataFormat(data), "at least 10 rows")
})

test_that("single item after ID removal works", {
  set.seed(222)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:4, 15, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 1)
})

# ============================================================
# Variance anomalies (sd = NA or 0)
# ============================================================

test_that("all items constant value → warns zero variance", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = rep(3, 15),
    item2 = rep(5, 15),
    item3 = rep(1, 15)
  )
  expect_message(result <- dataFormat(data), "zero variance")
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 3) # Not excluded, but warned
})

test_that("all-NA column → warns all missing values", {
  set.seed(231)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = rep(NA, 15),
    item2 = sample(1:4, 15, replace = TRUE),
    item3 = sample(1:4, 15, replace = TRUE)
  )
  expect_message(result <- dataFormat(data), "all missing values")
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 3) # All 3 columns remain
  expect_true(all(result$Q[, 1] == -1)) # item1 is all -1
})

test_that("all items all NA → warns all missing + all students missing", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = rep(NA, 15),
    item2 = rep(NA, 15)
  )
  expect_message(
    expect_message(result <- dataFormat(data), "all missing values"),
    "all missing responses"
  )
  expect_s3_class(result, "exametrika")
  expect_true(all(result$Q == -1))
  expect_true(all(result$Z == 0))
})

test_that("column with Inf triggers sd=NA → excluded", {
  set.seed(232)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(Inf, sample(1:4, 14, replace = TRUE)), # sd=NA due to Inf
    item2 = sample(1:4, 15, replace = TRUE),
    item3 = sample(1:4, 15, replace = TRUE)
  )
  expect_message(result <- dataFormat(data, response.type = "nominal"), "no variance")
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 2) # item1 excluded
  expect_false("item1" %in% result$ItemLabel)
})

# ============================================================
# Missing value handling
# ============================================================

test_that("na=NA is handled (treated as NULL)", {
  set.seed(241)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(1, 2, 3, NA), 15, replace = TRUE),
    item2 = sample(c(1, 2, 3, NA), 15, replace = TRUE)
  )
  result <- dataFormat(data, na = NA)
  expect_s3_class(result, "exametrika")
})

test_that("custom na value converted to -1 in response matrix", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 2, 99, 1, 2, 3, 1, 2, 3, 1),
    item2 = c(3, 99, 2, 1, 2, 3, 1, 99, 3, 2)
  )
  result <- dataFormat(data, na = 99)
  expect_equal(unname(result$Q[1, 1]), 1)
  expect_equal(unname(result$Q[3, 1]), -1) # 99 → -1
  expect_equal(unname(result$Z[3, 1]), 0) # missing indicator
})

test_that("Z matrix overrides response data for missing", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 2, 3, 1, 2, 3, 1, 2, 3, 1),
    item2 = c(3, 3, 2, 1, 2, 3, 1, 2, 3, 2)
  )
  Z <- matrix(1, nrow = 10, ncol = 2)
  Z[1, 1] <- 0 # Force item1 of student 1 to missing
  Z[5, 2] <- 0 # Force item2 of student 5 to missing
  result <- dataFormat(data, Z = Z)
  expect_equal(unname(result$Q[1, 1]), -1)
  expect_equal(unname(result$Q[5, 2]), -1)
  expect_equal(unname(result$Z[1, 1]), 0)
  expect_equal(unname(result$Z[5, 2]), 0)
})

test_that("Z matrix with non-0/1 values causes error", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = sample(1:3, 10, replace = TRUE),
    item2 = sample(1:3, 10, replace = TRUE)
  )
  Z_bad <- matrix(c(0, 1, 2, 1), nrow = 10, ncol = 2)
  expect_error(dataFormat(data, Z = Z_bad), "must contain only 0 or 1")
})

# ============================================================
# rated without response.type (regression test for the bug fix)
# ============================================================

test_that("rated auto-detection: ordinal-looking data + CA → rated", {
  set.seed(251)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(1:5, 20, replace = TRUE),
    item2 = sample(1:5, 20, replace = TRUE),
    item3 = sample(1:5, 20, replace = TRUE)
  )
  result <- dataFormat(data, CA = c(3, 1, 5))
  expect_equal(result$response.type, "rated")
  expect_true(!is.null(result$U))
  expect_true(all(result$U[result$Q != -1] %in% c(0, 1)))
})

test_that("rated auto-detection: nominal-looking data + CA → rated", {
  set.seed(252)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(c(1, 5, 10), 20, replace = TRUE),
    item2 = sample(c(2, 7, 15), 20, replace = TRUE)
  )
  result <- dataFormat(data, CA = c(5, 7))
  expect_equal(result$response.type, "rated")
})

# ============================================================
# Weight vector anomalies
# ============================================================

test_that("default weight is all 1s matching number of items", {
  set.seed(261)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE),
    item3 = sample(1:3, 15, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(result$w, c(1, 1, 1))
})

test_that("custom weight vector applied correctly", {
  set.seed(262)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE)
  )
  result <- dataFormat(data, w = c(2, 0.5))
  expect_equal(result$w, c(2, 0.5))
})

# ============================================================
# Factor / CategoryLabel handling
# ============================================================

test_that("factor response columns converted to numeric with labels preserved", {
  set.seed(271)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = factor(sample(c("A", "B", "C"), 15, replace = TRUE)),
    item2 = factor(sample(c("X", "Y", "Z"), 15, replace = TRUE))
  )
  result <- dataFormat(data, response.type = "nominal")
  expect_s3_class(result, "exametrika")
  expect_true(!is.null(result$CategoryLabel))
  expect_true(all(result$Q[result$Q != -1] > 0)) # numeric, not character
})

test_that("mixed factor and numeric columns handled", {
  set.seed(272)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = factor(sample(c("Low", "Mid", "High"), 15, replace = TRUE),
      levels = c("Low", "Mid", "High")
    ),
    item2 = sample(1:3, 15, replace = TRUE)
  )
  result <- dataFormat(data, response.type = "ordinal")
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 2)
})

# ============================================================
# Matrix input (no column names, no ID)
# ============================================================

test_that("matrix without colnames gets V1/V2/V3 labels from as.data.frame", {
  set.seed(281)
  mat <- matrix(sample(0:1, 15 * 3, replace = TRUE), nrow = 15, ncol = 3)
  expect_message(result <- dataFormat(mat), "No ID column detected")
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")
  # as.data.frame(unclass()) on unnamed matrix gives "V1", "V2", ...
  expect_equal(result$ItemLabel, c("V1", "V2", "V3"))
  expect_equal(result$ID[1], "Student1")
})

test_that("matrix with colnames preserves item labels", {
  set.seed(282)
  mat <- matrix(sample(1:4, 20 * 2, replace = TRUE), nrow = 20, ncol = 2)
  colnames(mat) <- c("Q1", "Q2")
  result <- dataFormat(mat)
  expect_s3_class(result, "exametrika")
  expect_equal(result$ItemLabel, c("Q1", "Q2"))
})

# ============================================================
# Already formatted data (early return)
# ============================================================

test_that("already formatted exametrika object returned as-is", {
  set.seed(291)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(0:1, 15, replace = TRUE),
    item2 = sample(0:1, 15, replace = TRUE)
  )
  result1 <- dataFormat(data)
  result2 <- dataFormat(result1)
  expect_identical(result1, result2)
})

# ============================================================
# Regression: id default change (NULL instead of 1)
# ============================================================

test_that("existing code without id arg still works (backward compat)", {
  set.seed(301)
  # String ID in first column - should auto-detect
  data1 <- data.frame(
    student = paste0("S", 1:20),
    item1 = sample(0:1, 20, replace = TRUE),
    item2 = sample(0:1, 20, replace = TRUE)
  )
  result1 <- dataFormat(data1)
  expect_equal(ncol(result1$U), 2)
  expect_equal(result1$ID, paste0("S", 1:20))

  # Matrix input - should auto-detect no ID
  set.seed(302)
  mat <- matrix(sample(0:1, 15 * 4, replace = TRUE), nrow = 15, ncol = 4)
  result2 <- dataFormat(mat)
  expect_equal(ncol(result2$U), 4)

  # Non-first column ID - should work with explicit id
  set.seed(303)
  data3 <- data.frame(
    item1 = sample(1:3, 15, replace = TRUE),
    student = paste0("S", 1:15),
    item2 = sample(1:3, 15, replace = TRUE)
  )
  result3 <- dataFormat(data3, id = 2)
  expect_equal(ncol(result3$Q), 2)
  expect_equal(result3$ID, paste0("S", 1:15))
})

# ============================================================
# Non-consecutive numeric IDs (e.g., 101, 205, 300)
# ============================================================

test_that("non-consecutive numeric IDs auto-detected", {
  set.seed(311)
  data <- data.frame(
    id = seq(101, 115),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(ncol(result$Q), 2)
  expect_equal(result$ID, as.character(101:115))
})

# ============================================================
# categories field correctness
# ============================================================

test_that("categories counts unique non-missing values per item", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 2, 3, 4, 1, 2, 3, 4, 1, 2), # 4 categories
    item2 = c(1, 1, 2, 2, 3, 3, 1, 1, 2, 2) # 3 categories
  )
  result <- dataFormat(data)
  expect_equal(unname(result$categories[1]), 4)
  expect_equal(unname(result$categories[2]), 3)
})

test_that("categories ignores missing values", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 2, NA, 3, 1, 2, NA, 3, 1, 2), # 3 non-missing categories (1, 2, 3)
    item2 = c(1, 2, 1, 3, 3, 2, 1, 1, 3, NA) # 3 non-missing categories (1, 2, 3)
  )
  result <- dataFormat(data)
  # Note: NA→-1, so categories includes -1 as a "category" in current implementation
  # categories counts unique(x[x != -1]) so -1 is excluded
  expect_equal(unname(result$categories[1]), 3)
  expect_equal(unname(result$categories[2]), 3)
})

# ============================================================
# Absurd / adversarial input patterns
# ============================================================

# --- Wrong data types passed to data parameter ---

test_that("list input coerced to data.frame (as.data.frame handles it)", {
  # list with 10-element vectors becomes a 10x2 data.frame
  result <- dataFormat(list(
    a = sample(0:1, 10, replace = TRUE),
    b = sample(0:1, 10, replace = TRUE)
  ))
  expect_s3_class(result, "exametrika")
})

test_that("NULL input causes error", {
  expect_error(dataFormat(NULL))
})

test_that("single vector coerced to 1-column data.frame", {
  # A 10-element vector becomes a 10x1 data.frame
  result <- dataFormat(c(1, 0, 1, 0, 1, 0, 1, 0, 1, 0))
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")
})

test_that("truly invalid input: string scalar", {
  expect_error(dataFormat("not a dataset"))
})

# --- Weird values in response data ---

test_that("negative values in response data", {
  set.seed(401)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(-2, -1, 0, 1, 2), 15, replace = TRUE),
    item2 = sample(c(-2, -1, 0, 1, 2), 15, replace = TRUE)
  )
  # Negative values: auto-detect should fail or classify as nominal
  # -1 is the missing indicator internally, so this is problematic
  result <- dataFormat(data, response.type = "nominal")
  expect_s3_class(result, "exametrika")
})

test_that("decimal/float values in response data", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(1.5, 2.7, 3.1, 1.5, 2.7, 3.1, 1.5, 2.7, 3.1, 1.5, 2.7, 3.1, 1.5, 2.7, 3.1),
    item2 = c(0.1, 0.2, 0.3, 0.1, 0.2, 0.3, 0.1, 0.2, 0.3, 0.1, 0.2, 0.3, 0.1, 0.2, 0.3)
  )
  # Non-integer values: auto ordinal detection should fail (not integers)
  # Should fall through to nominal
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "nominal")
})

test_that("Inf values in response data → item excluded by sd=NA", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(1, 2, Inf, 1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3),
    item2 = c(3, 2, 1, 3, 2, 1, 3, 2, 1, 3, 2, 1, 3, 2, 1)
  )
  # Inf causes sd=NA → item excluded with message
  expect_message(result <- dataFormat(data, response.type = "nominal"), "no variance")
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 1) # item1 excluded
})

test_that("NaN values in response data treated as NA", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(1, 2, NaN, 1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3),
    item2 = c(3, 2, 1, 3, 2, 1, 3, 2, 1, 3, 2, 1, 3, 2, 1)
  )
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
  expect_equal(unname(result$Z[3, 1]), 0) # NaN → NA → missing
})

test_that("very large numbers in response data", {
  set.seed(402)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(1000000, 2000000, 3000000), 15, replace = TRUE),
    item2 = sample(c(1000000, 2000000, 3000000), 15, replace = TRUE)
  )
  result <- dataFormat(data, response.type = "nominal")
  expect_s3_class(result, "exametrika")
})

# --- ID column weirdness ---

test_that("ID column with all NAs (auto-detect) → auto-ID + warns all missing", {
  data <- data.frame(
    id = rep(NA, 15),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  # All NA in first column: not consecutive, not string → auto-ID generated
  # The all-NA column also triggers "all missing values" warning
  expect_message(result <- dataFormat(data), "No ID column detected")
  expect_s3_class(result, "exametrika")
})

test_that("ID column with empty strings", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE),
    stringsAsFactors = FALSE
  )
  data$id[5] <- ""
  result <- dataFormat(data, id = 1)
  expect_s3_class(result, "exametrika")
  expect_equal(result$ID[5], "")
})

test_that("ID column with whitespace strings", {
  data <- data.frame(
    id = c(paste0("S", 1:13), "  ", " "),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE),
    stringsAsFactors = FALSE
  )
  # "  " and " " are different strings, no duplicate
  result <- dataFormat(data, id = 1)
  expect_s3_class(result, "exametrika")
})

test_that("all same ID value causes duplicate error with row numbers", {
  data <- data.frame(
    id = rep("SAME", 15),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  expect_error(dataFormat(data, id = 1), "Duplicated IDs found")
  expect_error(dataFormat(data, id = 1), "rows") # Row numbers shown
})

test_that("duplicate IDs error shows which rows are duplicated", {
  data <- data.frame(
    id = c(
      "A", "B", "C", "A", "D", "E", "B", "F", "G", "H",
      "I", "J", "K", "L", "M"
    ),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  expect_error(dataFormat(data, id = 1), "'A' \\(rows 1,4\\)")
  expect_error(dataFormat(data, id = 1), "'B' \\(rows 2,7\\)")
})

test_that("first column with many unique values triggers ID hint", {
  set.seed(420)
  # First column has 12 unique values out of 15 rows (80%) but has duplicates
  # → not consecutive, not detected as ID → auto-generate
  # But 80% unique is suspicious, so a hint message is shown
  data <- data.frame(
    subject = c(
      101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
      111, 112, 101, 102, 103
    ), # 12 unique out of 15
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  expect_message(dataFormat(data), "No ID column detected")
  expect_message(dataFormat(data), "first column.*unique values")
})

test_that("factor ID with few levels (< 20) auto-detected as response", {
  set.seed(403)
  data <- data.frame(
    group = factor(sample(c("A", "B", "C"), 15, replace = TRUE)),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  # Factor with < 20 unique values: not detected as ID
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 3) # All 3 columns as response
})

# --- CA parameter weirdness ---

test_that("CA as character vector causes error or is coerced", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE)
  )
  # CA should be numeric; character "2" should be coerced
  result <- dataFormat(data, CA = c("2", "1"))
  expect_equal(result$response.type, "rated")
})

test_that("CA=NULL with response.type='rated' causes error", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:3, 15, replace = TRUE),
    item2 = sample(1:3, 15, replace = TRUE)
  )
  expect_error(
    dataFormat(data, response.type = "rated", CA = NULL),
    "Correct Answer must be specified"
  )
})

test_that("CA with named vector works", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3),
    item2 = c(3, 2, 1, 4, 3, 2, 1, 4, 3, 2, 1, 4, 3, 2, 1)
  )
  result <- dataFormat(data, CA = c(item1 = 2, item2 = 3))
  expect_equal(result$response.type, "rated")
  expect_equal(as.numeric(result$CA), c(2, 3))
})

# --- na parameter weirdness ---

test_that("na=0 treats 0 as missing (ambiguous with binary)", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2),
    item2 = c(3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1, 0, 3, 2, 1)
  )
  result <- dataFormat(data, na = 0)
  # 0 should be treated as missing
  expect_equal(unname(result$Z[1, 1]), 0) # item1[1] = 0 → missing
  expect_equal(unname(result$Q[1, 1]), -1)
})

test_that("na=-999 custom sentinel value", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 2, -999, 3, 1, 2, -999, 3, 1, 2),
    item2 = c(3, -999, 1, 2, 3, -999, 1, 2, 3, 1)
  )
  result <- dataFormat(data, na = -999)
  expect_equal(unname(result$Q[3, 1]), -1)
  expect_equal(unname(result$Q[2, 2]), -1)
  expect_equal(unname(result$Z[3, 1]), 0)
})

# --- Data with more columns than rows ---

test_that("wide data (more items than students)", {
  set.seed(405)
  n_items <- 30
  n_students <- 12
  mat <- matrix(sample(0:1, n_students * n_items, replace = TRUE),
    nrow = n_students, ncol = n_items
  )
  colnames(mat) <- paste0("Q", 1:n_items)
  result <- dataFormat(mat)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$U), n_items)
  expect_equal(nrow(result$U), n_students)
})

# --- Extremely sparse data ---

test_that("mostly NA data (>90% missing) → NA becomes -1, still works", {
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = c(1, rep(NA, 19)),
    item2 = c(rep(NA, 19), 2),
    item3 = c(NA, 3, rep(NA, 18))
  )
  # After NA→-1, each column has {value, -1, -1, ...}: sd > 0, not excluded
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 3)
})

test_that("sparse but enough variance survives", {
  set.seed(407)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = c(1, 2, 3, rep(NA, 12)), # 3 non-NA, has variance
    item2 = c(rep(NA, 12), 3, 4, 5), # 3 non-NA, has variance
    item3 = sample(1:3, 15, replace = TRUE) # fully observed
  )
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
})

# --- Duplicate column names ---

test_that("duplicate column names in data.frame", {
  set.seed(408)
  data <- data.frame(
    id = paste0("S", 1:15),
    x = sample(1:3, 15, replace = TRUE),
    x = sample(1:3, 15, replace = TRUE),
    check.names = FALSE
  )
  # R allows duplicate column names with check.names=FALSE
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
})

# --- Column names with special characters ---

test_that("column names with spaces and special chars → make.names applied", {
  set.seed(409)
  data <- data.frame(
    id = paste0("S", 1:15),
    `item 1` = sample(1:4, 15, replace = TRUE),
    `item-2` = sample(1:4, 15, replace = TRUE),
    check.names = FALSE
  )
  result <- dataFormat(data, id = 1)
  expect_s3_class(result, "exametrika")
  # as.data.frame(unclass()) applies make.names: "item 1" → "item.1"
  expect_true("item.1" %in% result$ItemLabel)
})

# --- Mixed binary and ordinal columns ---

test_that("mix of binary and ordinal items → ordinal overall", {
  set.seed(410)
  data <- data.frame(
    id = paste0("S", 1:20),
    item1 = sample(0:1, 20, replace = TRUE), # binary
    item2 = sample(0:1, 20, replace = TRUE), # binary
    item3 = sample(1:5, 20, replace = TRUE), # ordinal
    item4 = sample(1:5, 20, replace = TRUE) # ordinal
  )
  result <- dataFormat(data)
  # Not all binary → should be ordinal or nominal
  expect_true(result$response.type %in% c("ordinal", "nominal"))
})

# --- Boolean/logical data ---

test_that("logical TRUE/FALSE data detected as binary", {
  set.seed(411)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(TRUE, FALSE), 15, replace = TRUE),
    item2 = sample(c(TRUE, FALSE), 15, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(result$response.type, "binary")
  expect_true(all(result$U[result$U != -1] %in% c(0, 1)))
})

# --- Factor ID column with many levels (>= 20) looks like ID ---

test_that("factor ID with >= 20 levels detected as ID", {
  set.seed(412)
  data <- data.frame(
    student = factor(paste0("Student_", 1:25)),
    item1 = sample(1:4, 25, replace = TRUE),
    item2 = sample(1:4, 25, replace = TRUE)
  )
  result <- dataFormat(data)
  expect_equal(ncol(result$Q), 2)
  expect_equal(length(result$ID), 25)
})

# --- Factor with unused levels ---

test_that("factor with unused levels handled correctly", {
  set.seed(413)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = factor(sample(c("A", "B"), 15, replace = TRUE),
      levels = c("A", "B", "C", "D")
    ), # C and D unused
    item2 = factor(sample(c("X", "Y"), 15, replace = TRUE),
      levels = c("X", "Y", "Z")
    ) # Z unused
  )
  result <- dataFormat(data, response.type = "nominal")
  expect_s3_class(result, "exametrika")
})

# --- Tibble-like input (data.frame subclass) ---

test_that("data.frame subclass still works", {
  set.seed(414)
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(0:1, 15, replace = TRUE),
    item2 = sample(0:1, 15, replace = TRUE)
  )
  class(data) <- c("tbl_df", "tbl", "data.frame")
  result <- dataFormat(data)
  expect_s3_class(result, "exametrika")
})

# --- rated: all correct or all incorrect ---

test_that("rated: all students answer correctly → U all 1s, warns zero variance", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = rep(3, 10),
    item2 = rep(2, 10)
  )
  expect_message(result <- dataFormat(data, CA = c(3, 2)), "zero variance")
  expect_equal(result$response.type, "rated")
  expect_true(all(result$U == 1))
})

test_that("rated: almost no student answers correctly", {
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = c(1, 1, 1, 1, 1, 1, 1, 1, 1, 2), # CA=2, only S10 correct
    item2 = c(3, 3, 3, 3, 3, 3, 3, 3, 3, 1) # CA=1, only S10 correct
  )
  result <- dataFormat(data, CA = c(2, 1))
  expect_equal(result$response.type, "rated")
  expect_equal(sum(result$U[, 1] == 1), 1) # Only 1 correct for item1
  expect_equal(sum(result$U[, 2] == 1), 1) # Only 1 correct for item2
})

# --- Z matrix edge cases ---

test_that("Z matrix all zeros → all -1, warns all missing", {
  set.seed(415)
  data <- data.frame(
    id = paste0("S", 1:10),
    item1 = sample(1:3, 10, replace = TRUE),
    item2 = sample(1:3, 10, replace = TRUE)
  )
  Z_all_zero <- matrix(0, nrow = 10, ncol = 2)
  expect_message(result <- dataFormat(data, Z = Z_all_zero), "all missing")
  expect_s3_class(result, "exametrika")
  expect_true(all(result$Q == -1))
  expect_true(all(result$Z == 0))
})

# --- response.type explicitly set to binary but data is not ---

test_that("binary response.type with non-binary data causes error", {
  data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:4, 15, replace = TRUE),
    item2 = sample(1:4, 15, replace = TRUE)
  )
  expect_error(
    dataFormat(data, response.type = "binary"),
    "can only contain the values 0, 1"
  )
})

# --- Multiple issues combined ---

test_that("numeric ID + CA + NA all at once", {
  data <- data.frame(
    id = 1:15,
    item1 = c(1, NA, 3, 2, 1, 3, 2, NA, 1, 3, 2, 1, 3, 2, 1),
    item2 = c(NA, 2, 1, 3, 2, 1, NA, 3, 2, 1, 3, 2, 1, 3, 2),
    item3 = c(2, 3, 1, NA, 2, 3, 1, 2, NA, 3, 1, 2, 3, 1, 2)
  )
  result <- dataFormat(data, CA = c(2, 3, 1))
  expect_equal(result$response.type, "rated")
  expect_equal(ncol(result$U), 3)
  expect_equal(ncol(result$Q), 3)
  # Check missing preserved in U
  expect_equal(result$U[2, 1], -1) # NA → -1
  expect_equal(result$U[1, 2], -1) # NA → -1
  # Check correct scoring
  expect_equal(result$U[1, 1], 0) # 1 != CA(2)
  expect_equal(result$U[3, 2], 0) # 1 != CA(3)
  expect_equal(result$U[4, 1], 1) # 2 == CA(2)
})

test_that("custom na + CA + explicit id", {
  data <- data.frame(
    item1 = c(1, 99, 3, 2, 1, 3, 2, 99, 1, 3, 2, 1, 3, 2, 1),
    student = paste0("S", 1:15),
    item2 = c(99, 2, 1, 3, 2, 1, 99, 3, 2, 1, 3, 2, 1, 3, 2)
  )
  result <- dataFormat(data, na = 99, id = 2, CA = c(2, 3))
  expect_equal(result$response.type, "rated")
  expect_equal(result$ID, paste0("S", 1:15))
  expect_equal(unname(result$Q[2, 1]), -1) # 99 → -1
  expect_equal(unname(result$U[2, 1]), -1) # missing preserved in U
})

# --- Regression: sample datasets still work ---

test_that("package dataset J5S10 loads and formats correctly", {
  data(J5S10, package = "exametrika")
  result <- dataFormat(J5S10)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")
})

test_that("package dataset J35S500 loads and formats correctly", {
  data(J35S500, package = "exametrika")
  result <- dataFormat(J35S500)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")
})

test_that("package dataset J20S600 loads and formats correctly", {
  data(J20S600, package = "exametrika")
  result <- dataFormat(J20S600)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "nominal")
})

# Tests for Graphical Lasso

### Setup (full J15S3810 for numerical stability)
tmp <- J15S3810
S_test <- PolychoricCorrelationMatrix(tmp)


test_that("soft_thresholding works correctly", {
  expect_equal(soft_thresholding(0.5, 0.3), 0.2)
  expect_equal(soft_thresholding(-0.5, 0.3), -0.2)
  expect_equal(soft_thresholding(0.2, 0.3), 0)
  expect_equal(soft_thresholding(c(0.5, -0.5, 0.2), 0.3), c(0.2, -0.2, 0))
})


test_that("glasso_one at lambda = 0 matches solve(S)", {
  res <- glasso_one(S_test, lambda = 0, eps = 1e-8, max_iter = 200)
  expect_true(res$converged)
  expect_equal(res$Theta, unname(solve(S_test)), tolerance = 1e-4)
})


test_that("glasso_one at large lambda yields diagonal Theta", {
  big_lambda <- max(abs(S_test - diag(diag(S_test)))) * 2
  res <- glasso_one(S_test, lambda = big_lambda)
  off_diag <- res$Theta[upper.tri(res$Theta)]
  expect_true(all(abs(off_diag) < 1e-6))
})


test_that("glasso_one returns expected structure", {
  res <- glasso_one(S_test, lambda = 0.1)
  expect_named(res, c("Theta", "W", "Beta", "niter", "converged"))
  expect_true(is.matrix(res$Theta))
  expect_true(is.logical(res$converged))
})


test_that("compute_EBIC_glasso returns scalar", {
  Theta_hat <- glasso_one(S_test, lambda = 0.1)$Theta
  ebic <- compute_EBIC_glasso(S_test, Theta_hat, n = 200, p = 15, gamma = 0.5)
  expect_length(ebic, 1)
  expect_true(is.numeric(ebic))
})


test_that("Glasso returns expected structure", {
  res <- Glasso(tmp, n_lambda = 10)
  expect_s3_class(res, "exametrika")
  expect_named(res, c("theta", "W", "lambda_opt", "gamma", "ebic_opt", "n_edge", "path"))
  expect_true(is.matrix(res$theta))
  expect_equal(dim(res$theta), c(15, 15))
})


test_that("Glasso theta is symmetric", {
  res <- Glasso(tmp, n_lambda = 10)
  expect_lt(max(abs(res$theta - t(res$theta))), 1e-8)
})


test_that("Glasso path has the expected length", {
  res <- Glasso(tmp, n_lambda = 10)
  expect_equal(nrow(res$path), 10)
  expect_named(res$path, c("lambda", "ebic", "n_edge"))
})


test_that("Glasso rejects non-ordinal data", {
  binary_data <- J5S10
  expect_error(Glasso(binary_data))
})


# --- Divergence handling (v1.14.0) ------------------------------
# When the BCD inner loop produces NaN/Inf (typical for N << p with very
# small lambda), glasso_one() should return converged = FALSE rather than
# crashing on `if (diff < eps)`, and Glasso() should warn + break the
# lambda search while returning the best solution found so far.

test_that("glasso_one returns converged = FALSE when BCD diverges", {
  # Inject NaN into S directly to force BCD to produce NaN values.
  # Previously this crashed at `if (diff < eps)` with
  # "missing value where TRUE/FALSE needed".
  S_bad <- S_test
  S_bad[1, 2] <- NaN
  S_bad[2, 1] <- NaN
  res <- glasso_one(S_bad, lambda = 0.1, max_iter = 20)
  expect_named(res, c("Theta", "W", "Beta", "niter", "converged"))
  expect_false(res$converged)
})


test_that("Glasso early-break wiring: warn + best solution + NA path", {
  # End-to-end check that the divergence machinery on the Glasso() side
  # is wired up: when `glasso_one()` reports `converged = FALSE` mid-grid,
  # Glasso() must (a) emit a warning, (b) break the lambda loop, and
  # (c) preserve NA in `path$ebic` for the skipped tail.
  #
  # We exercise this through the public API by monkey-patching
  # `glasso_one` to flip `converged = FALSE` on the third lambda.
  ns <- asNamespace("exametrika")
  orig <- ns$glasso_one
  call_count <- 0L
  patched <- function(...) {
    call_count <<- call_count + 1L
    out <- orig(...)
    if (call_count == 3L) out$converged <- FALSE
    out
  }
  unlockBinding("glasso_one", ns)
  on.exit({
    assign("glasso_one", orig, envir = ns)
    lockBinding("glasso_one", ns)
  })
  assign("glasso_one", patched, envir = ns)

  expect_warning(
    res <- Glasso(J15S3810, n_lambda = 10),
    "diverged"
  )
  # A valid best solution must still be returned.
  expect_false(is.null(res$theta))
  expect_true(is.finite(res$ebic_opt))
  expect_true(is.finite(res$lambda_opt))
  # The path tail past the breakpoint must be NA.
  expect_true(any(is.na(res$path$ebic)))
})

# Network model plot functions (LDLRA / LDB / BINET)
# Internal functions called from plot.exametrika()

#' FieldPIRP plot (LDB only)
#' @noRd
plot_field_pirp <- function(x, dots = list()) {
  target <- x$IRP
  ## rank x field x nrs
  Nrank <- dim(target)[1]
  nfld <- dim(target)[2]
  nrs <- dim(target)[3]
  for (i in 1:Nrank) {
    mat <- target[i, , ]
    mat[mat == 0] <- NA
    xvals <- 0:nrs
    call_plot(
      plot,
      list(
        x = xvals,
        y = runif(length(xvals)), type = "n",
        xlim = c(0, nrs), ylim = c(0, 1),
        xlab = "PIRP(Number-Right Score) in Parent Field(s)",
        ylab = "Correct Response Rate"
      ),
      dots
    )
    for (j in 1:NROW(mat)) {
      y <- as.vector(na.omit(mat[j, ]))
      xvals_j <- seq(0, nrs)[1:length(y)]
      labels <- rep(as.character(j), length(y))
      call_plot(graphics::lines, list(x = xvals_j, y = y, type = "l", lwd = 2), dots)
      text(xvals_j, y, labels = labels, pos = 1)
    }
    title(main = paste("Rank", i))
  }
}

#' LDPSR plot (BINET only)
#' @noRd
plot_ldpsr <- function(x, dots = list()) {
  for (i in 1:length(x$params)) {
    target <- x$params[[i]]
    ln <- length(target$fld)
    lb <- names(target$chap)
    y1 <- target$pap
    y2 <- target$chap
    call_plot(
      plot,
      list(
        x = 1:ln, y = y1,
        type = "b", xaxt = "n",
        col = 3, lwd = 2, ylim = c(0, 1),
        ylab = "Probability",
        xlab = "",
        main = paste("Field", i, "items")
      ),
      dots
    )
    call_plot(graphics::lines, list(x = 1:ln, y = y2, col = 2, lwd = 2, type = "b"), dots)
    call_plot(graphics::axis, list(side = 1, at = 1:ln, labels = lb), dots)
    posx <- length(1:ln)
    text(
      x = posx, y = y1[posx], labels = paste("C", target$parent),
      col = 1, pos = 3, lwd = 2
    )
    text(
      x = posx, y = y2[posx], labels = paste("C", target$child),
      col = 1, pos = 3, lwd = 2
    )
  }
}

#' @title Print Method for Exametrika Objects
#' @description
#' S3 method for printing objects of class "exametrika". This function formats and displays
#' appropriate summary information based on the specific subclass of the exametrika object.
#' Different types of analysis results (IRT, LCA, network models, etc.) are presented
#' with customized formatting to highlight the most relevant information.
#'
#' @param x An object of class "exametrika" with various possible subclasses
#' @param digits Integer indicating the number of decimal places to display. Default is 3.
#' @param ... Additional arguments passed to print methods (not currently used)
#'
#' @details
#' The function identifies the specific subclass of the exametrika object and tailors the
#' output accordingly. For most analysis types, the function displays:
#'
#' * Basic model description and parameters
#' * Estimation results (e.g., item parameters, latent class profiles)
#' * Model fit statistics and diagnostics
#' * Visual representations where appropriate (e.g., graphs for network models, scree plots
#'   for dimensionality analysis)
#'
#' When printing network-based models (LDLRA, LDB, BINET), this function visualizes
#' the network structure using graphs, which can help in interpreting complex relationships
#' between items or latent variables.
#'
#' @return
#' Prints a formatted summary of the exametrika object to the console, with content
#' varying by object subclass:
#'
#' \describe{
#'   \item{TestStatistics}{Basic descriptive statistics of the test}
#'   \item{Dimensionality}{Eigenvalue analysis results with scree plot}
#'   \item{ItemStatistics}{Item-level statistics and psychometric properties}
#'   \item{QitemStatistics}{Item statistics for polytomous items}
#'   \item{exametrikaData}{Data structure details including response patterns and weights}
#'   \item{IIAnalysis}{Item-item relationship measures (tetrachoric correlations, etc.)}
#'   \item{CTT}{Classical Test Theory reliability measures}
#'   \item{IRT/GRM}{Item parameters, ability estimates, and fit indices}
#'   \item{LCA/LRA}{Class/Rank profiles, distribution information, and model fit statistics}
#'   \item{Biclustering/Biclustering_IRM}{Cluster profiles, field distributions, and model diagnostics}
#'   \item{LDLRA/LDB/BINET}{Network visualizations, parameter estimates, and conditional probabilities}
#' }
#'
#' @examples
#' \donttest{
#' # Print IRT analysis results with 4 decimal places
#' result <- IRT(J15S500)
#' print(result, digits = 4)
#'
#' # Print Latent Class Analysis results
#' result_lca <- LCA(J15S500, ncls = 3)
#' print(result_lca)
#' }
#'
#' @importFrom utils tail
#' @importFrom igraph plot.igraph layout_on_grid layout_with_fr E
#' @export
#'

print.exametrika <- function(x, digits = 3, ...) {
  value <- if (length(class(x)) > 1) tail(class(x), 1) else "all"

  switch(value,
    TestStatistics = {
      cat("Test Statistics\n")
      tmp <- as.data.frame(unlist(x))
      colnames(tmp) <- "value"
      print(tmp)
    },
    Dimensionality = {
      cat("Dimensionality Analyeis\n")
      cat("Eigenvalues\n")
      print(x$Eigenvalue)
      cat("Percentage Of Variance\n")
      print(x$PerOfVar)
      cat("Cummurative Percentage\n")
      print(x$CumOfPer)
      xdim <- x$Component
      ydim <- x$Eigenvalue
      plot(xdim, ydim,
        xlab = "Number of Eigenvalues",
        ylab = "Eigenvalue", type = "b"
      )
    },
    ItemStatistics = {
      cat("Item Statistics\n")
      tmp <- as.data.frame(unclass(x))
      rownames(tmp) <- NULL
      print(tmp, digits = digits)
    },
    QitemStatistics = {
      cat("Item Statistics\n")
      tmp <- as.data.frame(unclass(x))
      rownames(tmp) <- NULL
      print(tmp, digits = digits)
    },
    exametrikaData = {
      cat("Response Type:", x$response.type, "\n")
      if (x$response.type == "binary") {
        cat("Binary Response Pattern\n")
        print(x$U)
      } else {
        cat("Polytomous Response Pattern (", x$response.type, ")\n")
        print(x$Q)
        if (x$response.type == "rated") {
          cat("\nCorrect Answers\n")
          print(x$CA)
        }
      }
      cat("\nMissing Pattern\n")
      print(x$Z)
      cat("\nWeight\n")
      print(x$w)
      if (!is.null(x$factor_labels)) {
        cat("\nFactor Labels\n")
        print(x$factor_labels)
      }
    },
    IIAnalysis = {
      cat("Joint Sample Size\n")
      print(x$JSS, digits = digits)
      cat("\nJoint Correct Response Ratio\n")
      print(x$JCRR, digits = digits)
      cat("\nConditonal Correct Response Ratio\n")
      print(x$CCRR, digits = digits)
      cat("\nItem Lift\n")
      print(x$IL, digits = digits)
      cat("\nMutual Information\n")
      print(x$MI, digits = digits)
      cat("\nPhi coefficient\n")
      print(x$Phi, digits = digits)
      cat("\nCorrelation Matrix\n")
      print(x$Tetrachoric, digits = digits)
    },
    IIAnalysis.ordinal = {
      cat("Joint Sample Size\n")
      print(x$JSS, digits = digits)
      cat("\nJoint Selection Rateio\n")
      print(x$JSR, digits = digits)
      cat("\nConditonal Selection Ratio\n")
      print(x$JSR, digits = digits)
      cat("\nMutual Information\n")
      print(x$MI, digits = digits)
      cat("\nCorrelation Matrix\n")
      print(x$Polychoric, digits = digits)
    },
    CTT = {
      cat("Realiability\n")
      print(x$Reliability, digits = digits)
      cat("\nReliability Excluding Item\n")
      print(x$ReliabilityExcludingItem, digits = digits)
    },
    IRT_EAP_PSD = {
      cat("Ability")
      print(x$EAP, digits = digits)
    },
    IRT = {
      cat("Item Parameters\n")
      y <- cbind(x$params, x$itemPSD)
      print(y, digits = digits)
      cat("\nItem Fit Indices\n")
      y <- unclass(x$ItemFitIndices)
      y <- as.data.frame(y)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    GRM = {
      cat("Item Parameter\n")
      print(x$params, digits = digits)
      cat("\nItem Fit Indices\n")
      y <- unclass(x$ItemFitIndices)
      y <- as.data.frame(y)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    LCA = {
      cat("\nItem Reference Profile\n")
      print(x$IRP, digits = digits)
      cat("\nTest Profile\n")
      y <- rbind(x$TRP, x$LCD, x$CMD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent Class Ditribution",
        "Class Membership Distribution"
      )
      colnames(y) <- paste("Class", 1:x$Nclass)
      print(round(y, digits))
      cat("\nItem Fit Indices\n")
      y <- unclass(x$ItemFitIndices)
      y <- as.data.frame(y)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent class:", x$Nclass))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    LRA = {
      cat(paste("estimating method is ", x$method, "\n"))
      if (x$mic) {
        cat("\n Monotonic increasing IRP option is TRUE.\n")
      }
      cat("Item Reference Profile\n")
      print(x$IRP, digits = digits)
      cat("\nItem Reference Profile Indices\n")
      print(x$IRPIndex, digits = digits)
      cat("\nTest Profile\n")
      y <- rbind(x$TRP, x$LRD, x$RMD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent Rank Ditribution",
        "Rank Membership Distribution"
      )
      colnames(y) <- paste("Rank", 1:x$Nrank)
      print(round(y, digits))
      cat("\nItem Fit Indices\n")
      y <- unclass(x$ItemFitIndices)
      y <- as.data.frame(y)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent rank:", x$Nrank))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    LRAordinal = {
      if (x$mic) {
        cat("\n Monotonic increasing IRP option is TRUE.\n")
      }
      print(x$ScoreReport, digits = digits)
      print(x$ItemReport, digits = digits)
      cat("Item Category Reference Profile\n")
      print(x$ICRP, digits = digits)
      cat("\nTest Profile\n")
      y <- rbind(x$TRP, x$LRD, x$RMD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent Rank Ditribution",
        "Rank Membership Distribution"
      )
      colnames(y) <- paste("Rank", 1:x$Nrank)
      print(round(y, digits))
      cat("\nItem Fit Indices\n")
      y <- unclass(x$ItemFitIndices)
      y <- as.data.frame(y)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent rank:", x$Nrank))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    LRArated = {
      if (x$mic) {
        cat("\n Monotonic increasing IRP option is TRUE.\n")
      }
      print(x$ScoreReport, digits = digits)
      print(x$ItemReport, digits = digits)
      cat("Item Quantile Reference Matrix\n")
      print(x$ItemQuantileRef, digits = digits)
      cat("Item Category Reference Profile\n")
      print(x$ICRP, digits = digits)
      cat("\nTest Profile\n")
      y <- rbind(x$TRP, x$LRD, x$RMD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent Rank Ditribution",
        "Rank Membership Distribution"
      )
      colnames(y) <- paste("Rank", 1:x$Nrank)
      print(round(y, digits))
      cat("\nItem Fit Indices\n")
      y <- unclass(x$ItemFitIndices)
      y <- as.data.frame(y)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent rank:", x$Nrank))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    Biclustering = {
      model_name <- ifelse(x$model == 1, "Biclustering", "Ranklustering")
      mic_suffix <- if (x$mic) " [MIC]" else ""

      cat(sprintf("%s Analysis%s\n\n", model_name, mic_suffix))

      cat(paste(model_name, "Reference Matrix Profile\n"))
      print(x$FRP, digits = digits)

      cat("\nField Reference Profile Indices\n")
      print(x$FRPIndex, digits = digits)
      cat("\n")

      y <- rbind(x$TRP, x$LRD, x$CMD)
      rownames(y) <- c(
        "Test Reference Profile",
        paste("Latent", x$msg, "Ditribution"),
        paste(x$msg, "Membership Distribution")
      )
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      print(round(y, digits))

      cat("\nField Membership Profile\n")
      y <- as.data.frame(x$FieldAnalysis)
      y$LFE <- as.integer(y$LFE)
      num_cols <- setdiff(names(y), "LFE")
      y[num_cols] <- lapply(y[num_cols], function(v) format(round(v, digits), nsmall = digits))
      print(y)

      cat("Latent Field Distribution\n")
      y <- matrix(x$LFD, byrow = T, nrow = 1)
      rownames(y) <- "N of Items"
      colnames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))
      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent", x$msg, ":", x$Nclass))
      cat(paste("\nNumber of Latent Field:", x$Nfield))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
      if (x$SOACflg) {
        cat("Strongly Ordinal Alignment Condition is Satisfied.\n")
      }
      if (x$WOACflg) {
        cat("Weakly Ordinal Alignment Condition is Satisfied.\n")
      }
    },
    ratedBiclustering = {
      model_name <- ifelse(x$model == 1, "Biclustering", "Ranklustering")
      mic_suffix <- if (x$mic) " [MIC]" else ""

      cat(sprintf("%s Analysis (Rated)%s\n\n", model_name, mic_suffix))

      cat(paste(model_name, "Reference Matrix Profile (Nominal)\n"))
      for (q in 1:dim(x$FRP)[3]) {
        cat(paste("For category", q, "\n"))
        y <- x$FRP[, , q]
        colnames(y) <- paste(x$msg, 1:x$Nclass)
        rownames(y) <- paste("Field", 1:x$Nfield)
        print(y, digits = digits)
      }

      cat(paste("\nField Reference Profile (Binary)\n"))
      y <- x$FieldFRP
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      rownames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))

      cat("\nField Reference Profile Indices\n")
      print(x$FRPIndex, digits = digits)
      cat("\n")

      y <- rbind(x$TRP, x$LRD, x$CMD)
      rownames(y) <- c(
        "Test Reference Profile",
        paste("Latent", x$msg, "Ditribution"),
        paste(x$msg, "Membership Distribution")
      )
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      print(round(y, digits))

      cat("\nField Membership Profile\n")
      y <- as.data.frame(x$FieldAnalysis)
      y$LFE <- as.integer(y$LFE)
      num_cols <- setdiff(names(y), "LFE")
      y[num_cols] <- lapply(y[num_cols], function(v) format(round(v, digits), nsmall = digits))
      print(y)

      cat("Latent Field Distribution\n")
      y <- matrix(x$LFD, byrow = TRUE, nrow = 1)
      rownames(y) <- "N of Items"
      colnames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))

      # Layer 1: Binary fit indices
      cat("\nModel Fit Indices (Binary)\n")
      cat(paste("Number of Latent", x$msg, ":", x$Nclass))
      cat(paste("\nNumber of Latent Field:", x$Nfield))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))

      # Layer 2: Nominal fit indices (AIC/BIC/CAIC/LogLik only)
      cat("\nModel Fit Indices (Nominal)\n")
      nom <- x$TestFitIndices_nominal
      y <- data.frame(value = c(
        nom$model_log_like,
        nom$AIC,
        nom$CAIC,
        nom$BIC
      ))
      rownames(y) <- c("model_log_like", "AIC", "CAIC", "BIC")
      print(round(y, digits))

      if (x$SOACflg) {
        cat("Strongly Ordinal Alignment Condition is Satisfied.\n")
      }
      if (x$WOACflg) {
        cat("Weakly Ordinal Alignment Condition is Satisfied.\n")
      }
    },
    nominalBiclustering = {
      cat(paste("Biclustering Reference Matrix Profile\n"))
      for (q in 1:dim(x$FRP)[3]) {
        cat(paste("For category", q, "\n"))
        y <- x$FRP[, , q]
        colnames(y) <- paste(x$msg, 1:x$Nclass)
        rownames(y) <- paste("Field", 1:x$Nfield)
        print(y, digits = digits)
      }

      y <- rbind(x$LRD, x$CMD)
      rownames(y) <- c(
        paste("Latent", x$msg, "Ditribution"),
        paste(x$msg, "Membership Distribution")
      )
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      print(round(y, digits))

      cat("Latent Field Distribution\n")
      y <- matrix(x$LFD, byrow = T, nrow = 1)
      rownames(y) <- "N of Items"
      colnames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))

      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent", x$msg, ":", x$Nclass))
      cat(paste("\nNumber of Latent Field:", x$Nfield))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y$LogLik <- x$LogLik
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    ordinalBiclustering = {
      model_name <- ifelse(x$model == 1, "Biclustering", "Ranklustering")
      mic_suffix <- if (x$mic) " [MIC]" else ""

      cat(sprintf("%s Analysis%s\n\n", model_name, mic_suffix))

      cat(paste(model_name, "Reference Matrix Profile\n"))
      for (q in 1:dim(x$FRP)[3]) {
        cat(paste("For category", q, "\n"))
        y <- x$FRP[, , q]
        colnames(y) <- paste(x$msg, 1:x$Nclass)
        rownames(y) <- paste("Field", 1:x$Nfield)
        print(y, digits = digits)
      }
      y <- rbind(x$TRP, x$LRD, x$CMD)
      rownames(y) <- c(
        "Test Reference Profile",
        paste("Latent", x$msg, "Ditribution"),
        paste(x$msg, "Membership Distribution")
      )
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      print(round(y, digits))
      cat("Latent Field Distribution\n")
      y <- matrix(x$LFD, byrow = T, nrow = 1)
      rownames(y) <- "N of Items"
      colnames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))

      cat("Boundary field reference profile\n")
      cat("Weighted\n")
      y <- x$BFRP$Weighted
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      rownames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))
      cat("Observed\n")
      y <- x$BFRP$Observed
      colnames(y) <- paste(x$msg, 1:x$Nclass)
      rownames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))

      cat("\nField Reference Profile Indices\n")
      cat("(Based on normalized expected scores: (E[score]-1)/(maxQ-1))\n")
      print(x$FRPIndex, digits = digits)

      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent", x$msg, ":", x$Nclass))
      cat(paste("\nNumber of Latent Field:", x$Nfield))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y$LogLik <- x$LogLik
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
      if (x$WOACflg) {
        cat("Weakly Ordinal Alignment Condition is Satisfied.\n")
      }
    },
    IRM = {
      cat(paste("Bicluster Reference Matrix\n"))
      print(x$FRP, digits = digits)

      y <- rbind(x$TRP, x$LCD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent class Ditribution"
      )
      colnames(y) <- paste("class", 1:x$Nclass)
      print(round(y, digits))

      cat("Latent Field Distribution\n")
      y <- matrix(x$LFD, byrow = T, nrow = 1)
      rownames(y) <- "N of Items"
      colnames(y) <- paste("Field", 1:x$Nfield)
      print(round(y, digits))

      cat("\nModel Fit Indices\n")
      cat(paste("Number of Latent Class :", x$Nclass))
      cat(paste("\nNumber of Latent Field:", x$Nfield))
      cat(paste("\nNumber of EM cycle:", x$N_Cycle, "\n"))
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    BNM = {
      cat("Adjacency Matrix\n")
      print(x$adj)
      if (x$acyclicFLG == 1) {
        print("Your graph is an acyclic graph.")
      }
      if (x$connectedFLG == 1) {
        print("Your graph is connected DAG.")
      }
      lay.tree <- layout_on_grid(x$g)
      xcoord <- lay.tree[, 1]
      ycoord <- lay.tree[, 2]
      xcoord <- xcoord[order(rowSums(x$adj))]
      ycoord <- ycoord[order(x$crr, decreasing = TRUE)]

      plot.igraph(x$g, layout = cbind(xcoord, ycoord))

      cat("\nParameter Learning\n")
      p_table <- x$param
      rownames(p_table) <- x$ItemLabel
      colnames(p_table) <- paste("PIRP", 1:ncol(p_table))
      print(p_table, na.print = "", digits = digits)

      cat("\nConditional Correct Response Rate\n")
      for (j in 1:NROW(x$CCRR_table)) {
        if (is.nan(x$CCRR_table[j, 5])) {
          x$CCRR_table[j, 5] <- "NaN(0/0)"
        } else {
          x$CCRR_tabje[j, 5] <- sprintf(
            paste0("%.", digits, "f"),
            as.numeric(x$CCRR_tabje[j, 5])
          )
        }
      }
      print(x$CCRR_table)

      cat("\nModel Fit Indices\n")
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    LDLRA = {
      if (x$model == 1) {
        msg <- "Class"
      } else {
        msg <- "Rank"
      }
      cat("Adjacency Matrix\n")
      print(x$adj_list)
      lay.tree <- layout_on_grid(x$g_list[[1]])
      xcoord <- lay.tree[, 1]
      ycoord <- lay.tree[, 2]
      xcoord <- xcoord[order(rowSums(x$adj_list[[1]]))]
      ycoord <- ycoord[order(x$crr, decreasing = TRUE)]
      for (i in 1:x$Nclass) {
        plot.igraph(x$g_list[[i]],
          layout = cbind(xcoord, ycoord),
          main = paste("Graph of ", msg, i)
        )
      }

      cat("\nParameter Learning\n")
      y <- x$Estimation_table
      numeric_cols <- which(sapply(y[, 3:NCOL(y)], is.numeric))
      for (j in numeric_cols) {
        y[, j + 2] <- ifelse(is.na(y[, j + 2]), "",
          sprintf(paste0("%.", digits, "f"), y[, j + 2])
        )
      }
      print(y)

      cat("\nConditional Correct Response Rate\n")
      for (j in 1:NROW(x$CCRR_table)) {
        if (is.nan(x$CCRR_table[j, 6])) {
          x$CCRR_table[j, 6] <- "NaN(0/0)"
        } else {
          x$CCRR_tabje[j, 6] <- sprintf(
            paste0("%.", digits, "f"),
            as.numeric(x$CCRR_tabje[j, 6])
          )
        }
      }
      print(x$CCRR_table, digits = digits)
      cat("\nMarginal Item Reference Profile\n")
      print(x$IRP, na.print = "", digits = digits)
      cat("\nIRP Indices\n")
      print(x$IRPIndex)
      if (x$SOACflg) {
        print("Strongly ordinal alignment condition was satisfied.")
      }

      cat("\nTest reference Profile and Latent Rank Distribution\n")
      y <- rbind(x$TRP, x$LRD, x$RMD)
      rownames(y) <- c(
        "Test Reference Profile",
        paste("Latent", msg, "Ditribution"),
        paste(msg, "Membership Distribution")
      )
      colnames(y) <- paste(msg, 1:x$Nclass)
      print(round(y, digits))

      if (x$WOACflg) {
        print("Weakly ordinal alignment condition was satisfied.")
      }

      cat("\nModel Fit Indices\n")
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))
    },
    LDB = {
      cat("Adjacency Matrix\n")
      print(x$adj_list)
      lay.tree <- layout_on_grid(x$g_list[[1]])
      xcoord <- lay.tree[, 1]
      ycoord <- lay.tree[, 2]
      xcoord <- xcoord[order(rowSums(x$adj_list[[1]]))]
      ycoord <- ycoord[order(colSums(x$adj_list[[1]]))]
      for (i in 1:x$Nrank) {
        plot.igraph(x$g_list[[i]],
          layout = cbind(xcoord, ycoord),
          main = paste("Graph at Rank", i)
        )
      }

      cat("\nParameter Learning\n")
      for (i in 1:x$Nrank) {
        tbl <- x$IRP[i, , ]
        tbl[tbl == 0] <- NA
        rownames(tbl) <- x$FieldLabel
        colnames(tbl) <- paste("PIRP", 0:(NCOL(tbl) - 1))
        cat(paste("Rank", i, "\n"))
        print(tbl, na.print = "", digits = digits)
      }

      cat("\nMarginal Rankluster Reference Matrix\n")
      print(x$FRP, digits = digits)
      cat("\nIRP Indices\n")
      print(x$FRPIndex)

      y <- rbind(x$TRP, x$LRD, x$RMD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent Rank Ditribution",
        "Rank Membership Dsitribution"
      )
      colnames(y) <- paste("Rank", 1:x$Nrank)
      print(round(y, digits))

      cat("\nLatent Field Distribution\n")
      y <- matrix(x$LFD, nrow = 1)
      rownames(y) <- "N of Items"
      colnames(y) <- paste("Field", 1:x$Nfield)
      print(y)

      cat("\nModel Fit Indices\n")
      y <- unclass(x$TestFitIndices)
      y <- t(as.data.frame(y))
      colnames(y) <- "value"
      print(round(y, digits))

      if (x$SOACflg & x$WOACflg) {
        message("Strongly ordinal alignment condition was satisfied.")
      }
      if (!x$SOACflg & x$WOACflg) {
        message("Weakly ordinal alignment condition was satisfied.")
      }
    },
    BINET = {
      cat("Total Graph\n")
      print(x$all_adj)
      layout_fr <- layout_with_fr(x$all_g)
      plot.igraph(x$all_g,
        layout = layout_fr,
        edge.label = E(x$all_g)$Field
      )
      cat("Estimation of Parameter set\n")
      for (i in 1:length(x$PSRP)) {
        cat("Field", i, "\n")
        print(x$PSRP[[i]], na.print = "", digits = digits)
      }
      cat("Local Dependence Passing Student Rate\n")
      df <- x$LDPSR
      for (j in 1:NCOL(df)) {
        if (is.numeric(df[, j])) {
          df[, j] <- sprintf(paste0("%.", digits, "f"), df[, j])
        }
        flg <- df[, j] == "NA"
        df[, j][flg] <- ""
      }
      print(df, na.print = "")

      cat("Marginal Bicluster Reference Matrix\n")
      print(round(x$FRP, digits))

      y <- rbind(x$TRP, x$LCD, x$CMD)
      rownames(y) <- c(
        "Test Reference Profile",
        "Latent Class Ditribution",
        "Class Membership Dsitribution"
      )
      colnames(y) <- paste("Class", 1:x$Nclass)
      print(round(y, digits))

      cat("\nModel Fit Indices\n")
      y1 <- unclass(x$MG_FitIndices)
      y2 <- unclass(x$SM_FitIndices)
      y <- t(as.data.frame(rbind(y1, y2)))
      colnames(y) <- c("Multigroup Model", "Saturated Moodel")
      print(y)
    },
    ModelFit = {
      tmp <- data.frame(unclass(x))
      print(tmp)
    },
    Glasso = {
      cat("Graphical Lasso\n")
      cat(sprintf("Optimal lambda: %.4f\n", x$lambda_opt))
      cat(sprintf("EBIC (gamma = %.2f): %.2f\n", x$gamma, x$ebic_opt))
      cat(sprintf("Number of edges: %d\n", x$n_edge))
      cat("\nPrecision matrix:\n")
      print(round(x$theta, digits))
    },
    matrix = {
      class(x) <- "matrix"
      print(x, digits = digits)
    },
    all = {
      class(x) <- "list"
      print(x, digits = digits)
    },
  )
}

#' Generate Error Message for Invalid Response Type
#'
#' Internal function to generate standardized error messages when a function
#' is called with an incompatible response type.
#'
#' @param response_type character. One of "binary", "rated", "ordinal", or "nominal"
#' @param fun_name character. Name of the calling function
#'
#' @return Never returns; always stops with an error message
#'
#' @keywords internal
response_type_error <- function(response_type, fun_name) {
  switch(response_type,
    "binary" = stop(sprintf("Function %s is not applicable to binary response data.", fun_name)),
    "rated" = stop(sprintf("Function %s is not applicable to rated response data.", fun_name)),
    "ordinal" = stop(sprintf("Function %s is not applicable to ordinal response data.", fun_name)),
    "nominal" = stop(sprintf("Function %s is not applicable to nominal response data.", fun_name))
  )
}

#' @rdname Biclustering
#' @param ncls Number of latent classes/ranks to identify (between 2 and 20).
#' @param nfld Number of latent fields (item clusters) to identify.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#' observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @param method Analysis method to use (character string):
#'   * "B" or "Biclustering": Standard biclustering (default)
#'   * "R" or "Ranklustering": Ranklustering with ordered class structure
#' @param conf Confirmatory parameter for pre-specified field assignments. Can be either:
#'   * A vector with items and corresponding fields in sequence
#'   * A field membership profile matrix (items × fields) with 0/1 values
#'   * NULL (default) for exploratory analysis where field memberships are estimated
#' @param mic Logical; if TRUE, forces Field Reference Profiles to be monotonically
#' increasing. Default is FALSE.
#' @param maxiter Maximum number of EM algorithm iterations. Default is 100.
#' @param verbose Logical; if TRUE, displays progress during estimation. Default is TRUE.
#' @param alpha Dirichlet distribution concentration parameter for prior density of field reference probabilities. Default is 1.
#' @param ... Additional arguments passed to specific methods.
#' @export
Biclustering.ordinal <- function(U,
                                 ncls = 2, nfld = 2,
                                 method = "B",
                                 conf = NULL,
                                 conf_class = NULL,
                                 mic = FALSE,
                                 maxiter = 100,
                                 verbose = TRUE,
                                 alpha = 1, ...) {
  tmp <- U
  maxQ <- max(tmp$Q)
  nobs <- NROW(tmp$Q)
  nitems <- NCOL(tmp$Q)
  ncat <- tmp$categories
  const <- exp(-nitems)
  test_log_lik <- -1 / const
  old_test_log_lik <- -2 / const
  emt <- 0
  maxemt <- maxiter
  ncat <- as.vector(tmp$categories)
  # if (length(unique(ncat)) > 1) {
  #   stop("Error: Variables have different numbers of categories. Nominal data processing requires the same number of categories for all variables.")
  # }

  if (method == "B" | method == "Biclustering") {
    if (verbose) {
      message("Biclustering is chosen.")
    }
    model <- 1
  } else if (method == "R" | method == "Ranklustering") {
    if (verbose) {
      message("Ranklustering is chosen.")
    }
    model <- 2
  } else {
    stop("The method must be selected as either Biclustering or Ranklustering.")
  }

  if (alpha <= 0) {
    stop("alpha must be positive (alpha > 0)")
  }

  ## confirmatory
  # set conf_mat for confirmatory clustering
  if (!is.null(conf)) {
    if (verbose) {
      message("Confirmatory Clustering is chosen.")
    }
    if (is.vector(conf)) {
      # check size
      if (length(conf) != NCOL(U$Q)) {
        stop("conf vector size does NOT match with data.")
      }
      conf_mat <- matrix(0, nrow = NCOL(U$Q), ncol = max(conf))
      for (i in 1:NROW(conf_mat)) {
        conf_mat[i, conf[i]] <- 1
      }
    } else if (is.matrix(conf) | is.data.frame(conf)) {
      if (NROW(conf) != NCOL(U$Q)) {
        stop("conf matrix size does NOT match with data.")
      }
      if (any(!conf %in% c(0, 1))) {
        stop("The conf matrix should only contain 0s and 1s.")
      }
      if (any(rowSums(conf) > 1)) {
        stop("The row sums of the conf matrix must be equal to 1.")
      }
      conf_mat <- as.matrix(conf)
    } else {
      stop("conf matrix is not set properly.")
    }
    ###
    nfld <- NCOL(conf_mat)
  } else {
    conf_mat <- NULL
  }

  # set conf_class_mat for class-side confirmatory clustering
  if (!is.null(conf_class)) {
    if (verbose) {
      message("Class-side Confirmatory Clustering is chosen.")
    }
    if (is.vector(conf_class)) {
      if (length(conf_class) != nobs) {
        stop("conf_class vector size does NOT match with the number of respondents.")
      }
      conf_class_mat <- matrix(0, nrow = nobs, ncol = max(conf_class))
      for (i in 1:NROW(conf_class_mat)) {
        conf_class_mat[i, conf_class[i]] <- 1
      }
    } else if (is.matrix(conf_class) | is.data.frame(conf_class)) {
      if (NROW(conf_class) != nobs) {
        stop("conf_class matrix size does NOT match with the number of respondents.")
      }
      if (any(!conf_class %in% c(0, 1))) {
        stop("The conf_class matrix should only contain 0s and 1s.")
      }
      if (any(rowSums(conf_class) > 1)) {
        stop("The row sums of the conf_class matrix must be equal to 1.")
      }
      conf_class_mat <- as.matrix(conf_class)
    } else {
      stop("conf_class is not set properly.")
    }
    ncls <- NCOL(conf_class_mat)
  } else {
    conf_class_mat <- NULL
  }

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  ##
  fld0 <- pmin(ceiling(1:nitems / (nitems / nfld)), nfld)
  med_order <- order(apply(tmp$Q, 2, median), decreasing = TRUE)
  fld <- fld0[match(1:nitems, med_order)]
  fldmemb <- matrix(0, nrow = nitems, ncol = nfld)
  for (i in 1:nitems) {
    fldmemb[i, fld[i]] <- 1
  }
  ## Confirmatory Biclustering
  if (!any(is.null(conf_mat))) {
    fldmemb <- conf_mat
  }

  # Bicluster Category Reference Matrix(FxCxQ)
  BCRM <- array(NA, dim = c(nfld, ncls, maxQ))
  for (i in 1:nfld) {
    for (j in 1:ncls) {
      class_effect <- j / ncls
      field_effect <- (nfld - i + 1) / nfld
      BCRM[i, j, ] <- init_field_membership_probs(maxQ, class_effect, field_effect)
    }
  }

  # Bicluter Boundary Reference Matrix(FxCxQ+1)
  BBRM <- array(NA, dim = c(nfld, ncls, maxQ + 1))
  BBRM[, , 1] <- 1
  BBRM[, , maxQ + 1] <- 0
  for (q in maxQ:2) {
    BBRM[, , q] <- BCRM[, , q] + BBRM[, , q + 1]
  }

  # One-hot encode tmp$Q into Uq[i, j, tmp$Q[i,j]] = 1 using matrix indexing.
  # Replaces a nobs*nitems R-level double loop with a single C-level write
  # into the flat backing storage of the 3-D array.
  # Missing entries (tmp$Z == 0, flagged as tmp$Q == -1 by dataFormat) are
  # left at zero; every downstream use of Uq is masked by tmp$Z so the
  # missing-cell values are never read.
  Uq <- array(0, dim = c(nobs, nitems, maxQ))
  valid <- as.vector(tmp$Z) == 1
  Uq[cbind(
    rep(seq_len(nobs), times = nitems)[valid],
    rep(seq_len(nitems), each = nobs)[valid],
    as.vector(tmp$Q)[valid]
  )] <- 1

  # Precompute Z * Uq[,,q] once; neither Z nor Uq changes after this point.
  # Uq is stored column-major with dim1 (nobs) varying fastest, so multiplying
  # by as.vector(Z) recycles Z across the maxQ slices, giving ZU[i,j,q] =
  # Z[i,j] * Uq[i,j,q]. This replaces repeated elementwise products inside
  # the EM loop and the post-EM fit blocks.
  ZU <- Uq * as.vector(tmp$Z)

  if (model != 2) {
    Fil <- diag(rep(1, ncls))
  } else {
    Fil <- create_filter_matrix(ncls)
  }
  # Initialize smoothed_memb so that it is defined even when the EM loop
  # exits before reaching the smoothing step (e.g. immediate convergence).
  smoothed_memb <- matrix(0, nrow = nobs, ncol = ncls)
  # iteration start ---------------------------------------------------------
  converge <- TRUE
  FLG <- TRUE
  while (FLG) {
    if (!is.finite(test_log_lik) ||
      test_log_lik - old_test_log_lik < 1e-8 * abs(old_test_log_lik)) {
      if (!is.finite(test_log_lik)) converge <- FALSE
      FLG <- FALSE
      break
    }
    if (emt == maxemt) {
      converge <- FALSE
      FLG <- FALSE
      message("\nReached the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
    }

    emt <- emt + 1
    old_test_log_lik <- test_log_lik

    # Precompute log(BBRM[,,q] - BBRM[,,q+1] + const) once per EM iteration.
    # Both E-steps (class and field) use the same values across all q; the old
    # code recomputed this 2*maxQ times per iteration. Using drop=FALSE on
    # array slicing keeps the 3D shape even when nfld == 1.
    log_delta <- log(BBRM[, , seq_len(maxQ), drop = FALSE] -
      BBRM[, , seq_len(maxQ) + 1, drop = FALSE] + const)

    ## Msc <- Pi, Mjf
    tmpL <- matrix(0, nrow = nobs, ncol = ncls)
    for (q in 1:maxQ) {
      log_probs <- matrix(log_delta[, , q], nrow = nfld, ncol = ncls)
      tmpL <- tmpL + ZU[, , q] %*% fldmemb %*% log_probs
    }
    # Row-wise min via C-level pmin.int instead of apply (one R-level call per row):
    # do.call(pmin.int, as.data.frame(X)) passes each column as a separate argument,
    # and pmin.int(col1, col2, ...) returns the element-wise min across columns.
    minllsr <- do.call(pmin.int, as.data.frame(tmpL))
    expllsr <- exp(pmin(tmpL - minllsr, 700))
    clsmemb <- round(expllsr / rowSums(expllsr), 1e8)

    if (!is.null(conf_class_mat)) {
      clsmemb <- conf_class_mat
      smoothed_memb <- clsmemb
    } else {
      # For Ranklustering
      smoothed_memb <- clsmemb %*% Fil
    }

    ## Mjf <- Pi, Msc
    tmpH <- matrix(0, nrow = nitems, ncol = nfld)
    for (q in 1:maxQ) {
      log_probs <- matrix(log_delta[, , q], nrow = nfld, ncol = ncls)
      tmpH <- tmpH + (t(ZU[, , q]) %*% clsmemb) %*% t(log_probs)
    }

    minllsr <- do.call(pmin.int, as.data.frame(tmpH))
    expllsr <- exp(pmin(tmpH - minllsr, 700)) # 700 is approx upper limit for exp()
    fldmemb <- round(expllsr / rowSums(expllsr), 1e8)

    if (!any(is.null(conf_mat))) {
      fldmemb <- conf_mat
    }

    ## Maximization
    oldBCRM <- BCRM
    Ufcq <- array(0, dim = c(nfld, ncls, maxQ))
    cUfcq <- array(0, dim = c(nfld, ncls, maxQ))
    for (q in 1:maxQ) {
      Ufcq[, , q] <- (t(fldmemb) %*% t(ZU[, , q])) %*% clsmemb
    }
    # Apply Dirichlet prior (alpha parameter)
    Ufcq_prior <- Ufcq + alpha - 1
    Ufcq_prior <- pmax(Ufcq_prior, 1e-10)
    # Reverse cumulative sum along the 3rd axis:
    # cUfcq[, , q] = sum over k >= q of Ufcq_prior[, , k].
    # Equivalent to aperm(apply(., c(1,2), function(x) rev(cumsum(rev(x)))), c(2,3,1))
    # but vectorized: O(maxQ) array additions instead of nfld*ncls R-level calls.
    cUfcq <- Ufcq_prior
    for (q in rev(seq_len(maxQ - 1))) {
      cUfcq[, , q] <- cUfcq[, , q] + cUfcq[, , q + 1]
    }

    for (q in 1:maxQ) {
      BBRM[, , q] <- cUfcq[, , q] / cUfcq[, , 1]
    }

    ## Forced Ordering
    if (mic) {
      overall_order <- array(0, dim = ncls)
      for (i in 1:ncls) {
        total_expected <- 0
        for (j in 1:nfld) {
          field_expected <- sum(BBRM[j, i, 1:maxQ])
          total_expected <- total_expected + field_expected
        }
        overall_order[i] <- total_expected
      }
      overall_order <- order(overall_order)
      BBRM <- BBRM[, overall_order, ]
    }
    for (q in 1:maxQ) {
      BCRM[, , q] <- BBRM[, , q] - BBRM[, , q + 1]
    }

    test_log_lik <- 0
    for (q in 1:maxQ) {
      observed_mask <- ZU[, , q] == 1
      prob_leq_q1 <- t(fldmemb %*% BBRM[, , q] %*% t(clsmemb))
      prob_leq_q2 <- t(fldmemb %*% BBRM[, , q + 1] %*% t(clsmemb))
      prob_exact <- prob_leq_q1 - prob_leq_q2
      test_log_lik <- test_log_lik + sum(log(pmax(prob_exact[observed_mask], const)))
    }

    if (verbose) {
      message(
        sprintf(
          "\r%-80s",
          paste0(
            "iter ", emt, " log_lik ", format(test_log_lik, digits = 6)
          )
        ),
        appendLF = FALSE
      )
    }

    if (!is.finite(test_log_lik) || test_log_lik - old_test_log_lik <= 0) {
      BCRM <- oldBCRM
      if (!is.finite(test_log_lik)) converge <- FALSE
      break
    }
  }


  cls <- max.col(clsmemb, ties.method = "first")
  fld <- max.col(fldmemb, ties.method = "first")


  ### Model Fit
  testell <- 0
  for (q in 1:maxQ) {
    pred_prob <- t(fldmemb %*% BCRM[, , q] %*% t(clsmemb))
    observed_mask <- ZU[, , q] == 1
    testell <- testell + sum(log(pmax(pred_prob[observed_mask], const)))
  }

  if (model == 2) {
    nparam <- sum(diag(Fil)) * nfld * (maxQ - 1)
  } else {
    nparam <- ncls * nfld * (maxQ - 1)
  }

  ptn <- apply(tmp$Q, 1, function(x) paste(x, collapse = ""))
  benchG <- as.numeric(as.factor(ptn))

  fullG <- length(unique(benchG))
  benchmemb <- matrix(0, nrow = nobs, ncol = fullG)
  for (i in 1:nobs) {
    benchmemb[i, benchG[i]] <- 1
  }

  BenchFRQ <- array(NA, dim = c(nitems, fullG, maxQ))
  Bfcq <- array(0, dim = c(nitems, fullG, maxQ))
  for (q in 1:maxQ) {
    Bfcq[, , q] <- t(ZU[, , q]) %*% benchmemb
  }

  # rowSums(X, dims=2) sums over the 3rd dim, replacing apply(X, c(1,2), sum)
  BenchFRQ <- Bfcq / array(rowSums(Bfcq, dims = 2), dim = dim(BenchFRQ))
  BenchFRQ[is.nan(BenchFRQ)] <- const

  ell_B <- 0
  for (q in 1:maxQ) {
    ell_B <- ell_B + sum(t(ZU[, , q]) %*% benchmemb * log(BenchFRQ[, , q] + const))
  }
  bench_nparam <- nitems * fullG

  # Zrep * Uq == ZU elementwise; avoid the replicate() allocation by reusing ZU.
  # colSums(X, dims=1) sums over the 1st dim, replacing apply(X, c(2,3), sum).
  ZU_col_sums <- colSums(ZU, dims = 1)
  NullFRQ <- ZU_col_sums / colSums(tmp$Z)
  ell_N <- sum(ZU_col_sums * log(NullFRQ + const))
  null_nparam <- nitems


  df_B <- bench_nparam - null_nparam
  chi_B <- 2 * (ell_B - ell_N)
  # Analysis model
  chi_A <- 2 * (ell_B - test_log_lik)
  df_A <- bench_nparam - nparam
  FitIndices <- structure(
    c(list(
      model_log_like = test_log_lik,
      bench_log_like = ell_B,
      null_log_like = ell_N,
      model_Chi_sq = chi_A,
      null_Chi_sq = chi_B,
      model_df = df_A,
      null_df = df_B
    ), calcFitIndices(chi_A, chi_B, df_A, df_B, nobs)),
    class = c("exametrika", "ModelFit")
  )

  # output ----------------------------------------------------------
  cls <- max.col(clsmemb, ties.method = "first")
  fld <- max.col(fldmemb, ties.method = "first")
  check_empty_fields(fld, nfld)
  fldmemb01 <- sign(fldmemb - do.call(pmax.int, as.data.frame(fldmemb))) + 1
  flddist <- colSums(fldmemb01)
  clsmemb01 <- sign(clsmemb - do.call(pmax.int, as.data.frame(clsmemb))) + 1
  clsdist <- colSums(clsmemb01)
  StudentRank <- cbind(clsmemb, Estimate = cls)
  rownames(StudentRank) <- tmp$ID
  colnames(StudentRank) <- c(paste("Membership", 1:ncls), "Estimate")
  ## Expected score
  BFRP1 <- BFRP2 <- matrix(0, nrow = nfld, ncol = ncls)
  weights <- matrix(0, nrow = nfld, ncol = ncls)

  for (q in 1:maxQ) {
    contrib <- (t(fldmemb) %*% t(ZU[, , q]) %*% clsmemb) * (BCRM[, , q])
    BFRP1 <- BFRP1 + q * contrib
    weights <- weights + contrib
  }
  BFRP1 <- BFRP1 / weights

  for (i in 1:ncls) {
    for (j in 1:nfld) {
      BFRP2[j, i] <- mean(tmp$Z[cls == i, fld == j] * tmp$Q[cls == i, fld == j])
    }
  }

  TRP <- colSums(BFRP1)
  TRPlag <- TRP[2:ncls]
  TRPmic <- sum(TRPlag[1:(ncls - 1)] - TRP[1:(ncls - 1)] < 0, na.rm = TRUE)

  model_esp <- matrix(0, nrow = nfld, ncol = ncls)
  for (f in 1:nfld) {
    for (cc in 1:ncls) {
      model_esp[f, cc] <- sum((1:maxQ) * BCRM[f, cc, ])
    }
  }
  norm_frp <- (model_esp - 1) / (maxQ - 1)
  FRPIndex <- IRPindex(norm_frp)
  FRPmic <- sum(abs(FRPIndex$C))

  SOACflg <- WOACflg <- FALSE
  if (TRPmic == 0) {
    WOACflg <- TRUE
    if (FRPmic == 0) {
      SOACflg <- TRUE
    }
  }
  if (verbose) {
    if (SOACflg & WOACflg) {
      message("Strongly ordinal alignment condition was satisfied.")
    }
    if (!SOACflg & WOACflg) {
      message("Weakly ordinal alignment condition was satisfied.")
    }
  }


  msg <- ifelse(model == 1, "Class", "Rank")
  ret <- structure(list(
    Q = tmp$Q,
    Z = tmp$Z,
    testlength = nitems,
    msg = msg,
    model = model,
    mic = mic,
    converge = converge,
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    n_cycle = emt,
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = emt,
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = BCRM,
    FRPIndex = FRPIndex,
    TRP = TRP,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    SmoothedMembership = smoothed_memb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    BFRP = list(Weighted = BFRP1, Observed = BFRP2),
    TestFitIndices = FitIndices,
    log_lik = test_log_lik, # New naming convention
    SOACflg = SOACflg,
    WOACflg = WOACflg,
    # Deprecated fields (for backward compatibility)
    LogLik = test_log_lik
  ), class = c("exametrika", "ordinalBiclustering"))

  return(ret)
}

#' @title J5S10
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 5 students and 10 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J5S10"

#' @title J15S500
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 500 students and 15 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J15S500"

#' @title J20S400
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 400 students and 20 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J20S400"

#' @title J35S515
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 515 students and 35 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J35S515"

#' @title J12S5000
#' @description A binary response dataset for test analysis
#' @format An exametrika class object with 5000 students and 12 items containing binary (0/1) responses
#' @source \url{http://sh0j1ma.stars.ne.jp/exmk/}
"J12S5000"

#' @title J15S3810
#' @description An ordinal response dataset for test analysis
#' @format An exametrika class object with 3810 students and 15 items containing ordinal responses with 4 categories
"J15S3810"

#' @title J35S5000
#' @description A rated response dataset for test analysis
#' @format An exametrika class object with 5000 students and 35 items containing polytomous responses with correct answers
"J35S5000"

#' @title J5S1000
#' @description A simulated ordinal dataset for test analysis. This is a synthetic dataset
#'   generated using random number generation for demonstration and testing purposes.
#' @format An exametrika class object with 1000 students and 5 items containing ordinal responses
"J5S1000"

#' @title J50S100
#' @description A simulated binary dataset for test analysis. This is a synthetic dataset
#'   generated using random number generation for demonstration and testing purposes.
#' @format An exametrika class object with 100 students and 50 items containing binary responses
"J50S100"

#' @title J35S500
#' @description A simulated ordinal dataset for polytomous Biclustering analysis.
#'   This is a synthetic dataset generated using random number generation
#'   with a cumulative staircase pattern (5 latent classes, 5 fields,
#'   5 response categories) for demonstration and testing purposes.
#'   Contains approximately 0.5\% missing values.
#' @format An exametrika class object with 500 students and 35 items containing ordinal responses with 5 categories
"J35S500"

#' @title J20S600
#' @description A simulated nominal dataset for polytomous Biclustering analysis.
#'   This is a synthetic dataset generated using random number generation
#'   with a cyclic category preference pattern (5 latent classes, 4 fields,
#'   4 response categories) for demonstration and testing purposes.
#'   Contains approximately 0.5\% missing values.
#' @format An exametrika class object with 600 students and 20 items containing nominal responses with 4 categories
"J20S600"

#' @title J21S300
#' @description A simulated rated dataset for Rated IRM and Biclustering analysis.
#'   This is a synthetic dataset generated with a known structure
#'   (5 latent classes, 3 fields, 4 response categories with correct answers)
#'   for demonstration and testing purposes. Each field contains 7 items.
#' @format An exametrika class object with 300 students and 21 items containing
#'   rated responses (multiple-choice with correct answers) with 4 categories
"J21S300"

#' @title Alpha Coefficient
#' @description
#' This function computes Tau-Equivalent Measurement, also known as Cronbach's alpha coefficient, for a given data set.
#' @param x This should be a data matrix or a Covariance/Phi/Tetrachoric matrix.
#' @param Z This parameter represents a missing indicator matrix. It is only needed if 'x' is a data matrix.
#' @param w This parameter is an item weight vector. It is only required if 'x' is a data matrix.
#' @param na This parameter identifies the numbers or characters that should be treated as missing values when 'x' is a data matrix.
#' @return For a correlation/covariance matrix input, returns a single numeric value
#'   representing the alpha coefficient. For a data matrix input, returns a list with
#'   three components:
#'   \describe{
#'     \item{AlphaCov}{Alpha coefficient calculated from covariance matrix}
#'     \item{AlphaPhi}{Alpha coefficient calculated from phi coefficient matrix}
#'     \item{AlphaTetrachoric}{Alpha coefficient calculated from tetrachoric correlation matrix}
#'   }
#' @references Cronbach, L. J. (1951). Coefficient alpha and the internal structure of a test. Psychometrika, 16,297–334.
#' @importFrom stats cov
#' @export

AlphaCoefficient <- function(x, na = NULL, Z = NULL, w = NULL) {
  calcs <- function(V) {
    J <- NCOL(V)
    alphaConst <- J / (J - 1)
    tr <- sum(diag(V))
    alpha <- (sum(V) - tr) / (sum(V)) * alphaConst
    return(alpha)
  }

  if (NROW(x) != NCOL(x)) {
    if (!inherits(x, "exametrika")) {
      tmp <- dataFormat(data = x, na = na, Z = Z, w = w)
    } else {
      tmp <- x
    }
    V2 <- exametrika::PhiCoefficient(tmp)
    V3 <- exametrika::TetrachoricCorrelationMatrix(tmp)
    tmp$U[tmp$Z == 0] <- NA
    V1 <- stats::cov(tmp$U, use = "pairwise")
    alpha1 <- calcs(V1)
    alpha2 <- calcs(V2)
    alpha3 <- calcs(V3)
    return(list(
      AlphaCov = alpha1,
      AlphaPhi = alpha2,
      AlphaTetrachoric = alpha3
    ))
  } else {
    return(calcs(x))
  }
}

#' @title Alpha Coefficient if Item removed
#' @description
#' This function returns the alpha coefficient when the specified item is excluded.
#' @param x This should be a data matrix or a Covariance/Phi/Tetrachoric matrix.
#' @param Z This parameter represents a missing indicator matrix. It is only needed if 'x' is a data matrix.
#' @param w This parameter is an item weight vector. It is only required if 'x' is a data matrix.
#' @param na This parameter identifies the numbers or characters that should be treated as missing values when 'x' is a data matrix.
#' @param delItem Specify the item to be deleted. If NULL, calculations are performed for all cases.
#' @importFrom stats cov

AlphaIfDel <- function(x, delItem = NULL, na = NULL, Z = NULL, w = NULL) {
  calcs <- function(V, delItem) {
    J <- NCOL(V)
    if (is.null(delItem)) {
      alphaIfDel <- vector(length = J)
      for (j in 1:J) {
        mat <- V[-j, -j]
        alphaIfDel[j] <- AlphaCoefficient(mat)
      }
      return(alphaIfDel)
    } else {
      mat <- V[-delItem, -delItem]
      ret <- AlphaCoefficient(mat)
      return(ret)
    }
  }

  if (!inherits(x, "exametrika")) {
    tmp <- dataFormat(data = x, na = na, Z = Z, w = w)
  } else {
    tmp <- x
  }

  if (NROW(x) != NCOL(x)) {
    V2 <- exametrika::PhiCoefficient(tmp)
    V3 <- exametrika::TetrachoricCorrelationMatrix(tmp)
    tmp$U[tmp$Z == 0] <- NA
    V1 <- stats::cov(tmp$U, use = "pairwise")
    alpha1 <- calcs(V1, delItem)
    alpha2 <- calcs(V2, delItem)
    alpha3 <- calcs(V3, delItem)
    return(list(
      AlphaCov = alpha1,
      AlphaPhi = alpha2,
      AlphaTetrachoric = alpha3
    ))
  }
}

#' @title Omega Coefficient
#' @description
#' This function computes Tau-Congeneric Measurement, also known as McDonald's tau coefficient, for a given data set.
#' @references McDonald, R. P. (1999). Test theory: A unified treatment. Erlbaum.
#' @param x This should be a data matrix or a Covariance/Phi/Tetrachoric matrix.
#' @param Z This parameter represents a missing indicator matrix. It is only needed if 'x' is a data matrix.
#' @param w This parameter is an item weight vector. It is only required if 'x' is a data matrix.
#' @param na This parameter identifies the numbers or characters that should be treated as missing values when 'x' is a data matrix.
#' @return For a correlation/covariance matrix input, returns a single numeric value
#'   representing the omega coefficient. For a data matrix input, returns a list with
#'   three components:
#'   \describe{
#'     \item{OmegaCov}{Omega coefficient calculated from covariance matrix}
#'     \item{OmegaPhi}{Omega coefficient calculated from phi coefficient matrix}
#'     \item{OmegaTetrachoric}{Omega coefficient calculated from tetrachoric correlation matrix}
#'   }
#' @importFrom stats cov runif
#' @export

OmegaCoefficient <- function(x, na = NULL, Z = NULL, w = NULL) {
  calcs <- function(V) {
    J <- NCOL(V)
    offdiagonal <- matrix(1, ncol = ncol(V), nrow = nrow(V)) - diag(nrow = nrow(V))
    lam <- runif(J)
    # Define the function to minimize
    objective_function <- function(lam) {
      lammat <- outer(lam, lam)
      sum(offdiagonal * (lammat - V)^2)
    }
    # optimize
    result <- optim(lam, objective_function, method = "BFGS")
    # Error check
    if (result$convergence != 0) {
      stop(result$message)
    }
    lamest <- result$par
    Numerator <- sum(lamest)^2
    Denominator <- Numerator + sum(diag(V - outer(lamest, lamest)))
    omega <- Numerator / Denominator
    return(omega)
  }

  if (NROW(x) != NCOL(x)) {
    if (!inherits(x, "exametrika")) {
      tmp <- dataFormat(data = x, na = na, Z = Z, w = w)
    } else {
      tmp <- x
    }
    V2 <- exametrika::PhiCoefficient(tmp)
    V3 <- exametrika::TetrachoricCorrelationMatrix(tmp)
    tmp$U[tmp$Z == 0] <- NA
    V1 <- stats::cov(tmp$U, use = "pairwise")
    omega1 <- calcs(V1)
    omega2 <- calcs(V2)
    omega3 <- calcs(V3)
    return(list(
      OmegaCov = omega1,
      OmegaPhi = omega2,
      OmegaTetrachoric = omega3
    ))
  } else {
    return(calcs(x))
  }
}


#' @title Classical Test Theory
#' @description
#' This function calculates the overall alpha and omega coefficients for
#' the given data matrix. It also computes the alpha coefficient for
#' each item, assuming that item is excluded.
#' @param U U is a data matrix of the type matrix or data.frame.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @return Returns a list of class c("exametrika", "CTT") containing two data frames:
#'   \describe{
#'     \item{Reliability}{A data frame with overall reliability coefficients
#'       (Alpha and Omega) calculated using different correlation matrices
#'       (Covariance, Phi, and Tetrachoric)}
#'     \item{ReliabilityExcludingItem}{A data frame showing alpha coefficients
#'       when each item is excluded, calculated using different correlation matrices}
#'   }
#' @examples
#' \donttest{
#' # using sample dataset
#' CTT(J15S500)
#' }
#' @export

CTT <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    tmp <- dataFormat(U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (tmp$response.type != "binary") {
    response_type_error(U$response.type, "CTT")
  }


  alphaAll <- AlphaCoefficient(tmp)
  omegaAll <- OmegaCoefficient(tmp)
  eachAlpha <- AlphaIfDel(tmp)

  vec1 <- as.vector(unlist(alphaAll))
  vec2 <- as.vector(unlist(omegaAll))
  df <- data.frame(list(
    name = c(
      "Alpha(Covariance)",
      "Alpha(Phi)",
      "Alpha(Tetrachoric)",
      "Omega(Covariance)",
      "Omega(Phi)",
      "Omega(Tetrachoric)"
    ),
    value = c(vec1, vec2)
  ))
  df2 <- data.frame(list(
    IfDeleted = tmp$ItemLabel,
    "Alpha.Covariance" = eachAlpha$AlphaCov,
    "Alpha.Phi" = eachAlpha$AlphaPhi,
    "Alpha.Tetrachoric" = eachAlpha$AlphaTetrachoric
  ))
  ret <- list(Reliability = df, ReliabilityExcludingItem = df2)
  structure(ret, class = c("exametrika", "CTT"))
}

# Prepare Section ---------------------------------------------------------
library(exametrika)

Ch03Tests <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter03CTT_Test.csv"),
  check.names = FALSE
)

SimpleStatistics <- Ch03Tests[1:23, 1:2]
names(SimpleStatistics) <- c("name", "value")
SimpleStatistics$value <- as.numeric(SimpleStatistics$value)

Ch03Tests2 <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter03CTT_Test.csv"),
  check.names = FALSE
)

Dimensionality_ref <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter03CTT_Dimensionality.csv"),
  check.names = FALSE
)

CTTexpect <- Ch03Tests[24:29, ]

Ch03Items <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter03CTT_Item.csv"),
  check.names = FALSE
)
Ch03Items <- Ch03Items[, sapply(Ch03Items, is.numeric), drop = FALSE]

## read raw data (J20S400 package data is pre-formatted as nominal)
U <- read.csv(
  test_path("fixtures", "sample_data", "J20S400.csv"),
  check.names = FALSE
)
dat <- dataFormat(U, na = -99, id = 1)

# Test Section ------------------------------------------------------------

test_that("Simple Test Statistics", {
  result <- TestStatistics(dat)
  expect <- Ch03Tests2[1:23, 2] |>
    unlist() |>
    as.vector()
  result <- unclass(result) |>
    unlist() |>
    as.vector()
  expect_equal(object = result, expected = expect)
})

test_that("Dimensionality Analysis", {
  result <- Dimensionality(dat) |>
    unclass() |>
    unlist() |>
    matrix(ncol = 4)
  expect <- Dimensionality_ref |>
    as.matrix() |>
    unname()
  expect_equal(object = result[, 2], expected = expect[, 2], tolerance = 1e-4)
  expect_equal(object = result[, 3] / 100, expected = expect[, 3], tolerance = 1e-4)
  expect_equal(object = result[, 4] / 100, expected = expect[, 4], tolerance = 1e-4)
})


test_that("Reliability", {
  result <- CTT(dat)
  result <- result$Reliability[, 2] |> as.matrix()
  expect <- CTTexpect[, 2] |>
    as.matrix() |>
    unname()
  expect_equal(object = result, expected = expect, tolerance = 1e-4)
})

test_that("Item Del Reliability", {
  result <- CTT(dat)
  result <- result$ReliabilityExcludingItem[, -1] |> unname()
  expect <- Ch03Items[, 8:10] |> unname()
  expect_equal(object = result, expected = expect, tolerance = 1e-4)
})

#' @title Generate category labels for response data
#'
#' @param data_column A vector containing response data for a single item
#' @param item_name Character string of the item name
#' @return A character vector of category labels
#' @details
#' If the input is a factor, returns its levels.
#' Otherwise, generates labels in the format "Item name-Category-N"
#' @keywords internal
generate_category_labels <- function(data_column, item_name) {
  # For factor type data, preserve existing levels
  if (is.factor(data_column)) {
    return(levels(data_column))
  }

  # Get unique values excluding missing values (-1)
  unique_values <- sort(unique(data_column[data_column != -1]))
  n_categories <- length(unique_values)

  # Generate automatic category names
  paste0(item_name, "-Cat", seq_len(n_categories))
}

#' @title bivariate normal CDF
#' @description
#' Calculates the cumulative distribution function (CDF) of a bivariate normal distribution.
#' This function computes P(X <= a, Y <= b) where X and Y follow a bivariate normal
#' distribution with correlation coefficient rho.
#'
#' @param a Numeric value, the upper limit for the first variable.
#' @param b Numeric value, the upper limit for the second variable.
#' @param rho Numeric value between -1 and 1, the correlation coefficient.
#'
#' @return The probability P(X <= a, Y <= b), a value between 0 and 1.
#'
#' @details
#' The implementation uses numerical integration with Gauss-Legendre quadrature
#' for accurate computation. Special cases for infinite bounds are handled separately.
#'
#' @keywords internal

qBiNormal <- function(a, b, rho) {
  if (is.na(a) || is.na(b) || is.na(rho)) {
    warning("NA values detected in qBiNormal inputs")
    return(NA)
  }
  # support function
  f <- function(x, y, aprime, bprime, rho) {
    exp(aprime * (2 * x - aprime) + bprime * (2 * y - bprime) +
      2 * rho * (x - aprime) * (y - bprime))
  }

  # special case : Inf/-Inf
  if (is.infinite(a) && is.infinite(b)) {
    if (a > 0 && b > 0) {
      return(1)
    }
    if (a < 0 && b < 0) {
      return(0)
    }
    return(0)
  }

  # othre special case of Inf/-Inf
  if (is.infinite(a) || is.infinite(b)) {
    if (any(c(a, b) == -Inf)) {
      return(0)
    }
    #
    if (is.infinite(a)) {
      return(pnorm(b))
    } else if (is.infinite(b)) {
      return(pnorm(a))
    }
  }

  # main loop
  if (a <= 0 && b <= 0 && rho <= 0) {
    aprime <- a / sqrt(2 * (1 - rho^2))
    bprime <- b / sqrt(2 * (1 - rho^2))

    # const
    A <- c(
      5.54433663102343E-02, 1.24027738987730E-01, 1.75290943892075E-01,
      1.91488340747342E-01, 1.63473797144070E-01, 1.05937637278492E-01,
      5.00270211534535E-02, 1.64429690052673E-02, 3.57320421428311E-03,
      4.82896509305201E-04, 3.74908650266318E-05, 1.49368411589636E-06,
      2.55270496934465E-08, 1.34217679136316E-10, 9.56227446736465E-14
    )

    B <- c(
      2.16869474675590E-02, 1.12684220347775E-01, 2.70492671421899E-01,
      4.86902370381935E-01, 7.53043683072978E-01, 1.06093100362236E+00,
      1.40425495820363E+00, 1.77864637941183E+00, 2.18170813144494E+00,
      2.61306084533352E+00, 3.07461811380851E+00, 3.57140815113714E+00,
      4.11373608977209E+00, 4.72351306243148E+00, 5.46048893578335E+00
    )

    # double summation
    sum <- 0
    for (i in 1:15) {
      for (j in 1:15) {
        sum <- sum + A[i] * A[j] * f(B[i], B[j], aprime, bprime, rho)
      }
    }

    return(sum * sqrt(1 - rho^2) / pi)
  }

  if (a * b * rho <= 0) {
    if (a <= 0 && b >= 0 && rho >= 0) {
      return(pnorm(a) - qBiNormal(a, -b, -rho))
    } else if (a >= 0 && b <= 0 && rho >= 0) {
      return(pnorm(b) - qBiNormal(-a, b, -rho))
    } else if (a >= 0 && b >= 0 && rho <= 0) {
      return(pnorm(a) + pnorm(b) - 1 + qBiNormal(-a, -b, rho))
    }
  }

  if (a * b * rho >= 0) {
    denum <- sqrt(a^2 - 2 * rho * a * b + b^2)
    rho1 <- ((rho * a - b) * sign(a)) / denum
    rho2 <- ((rho * b - a) * sign(b)) / denum
    delta <- (1 - sign(a) * sign(b)) / 4

    return(qBiNormal(a, 0, rho1) + qBiNormal(b, 0, rho2) - delta)
  }

  # error case
  return(-99.9)
}

#' @title Calculate Polychoric Correlation Likelihood
#'
#' @description
#' Calculates the negative log-likelihood for estimating polychoric correlation
#' from a contingency table of two ordinal variables.
#'
#' @param rho Numeric value between -1 and 1, the correlation coefficient
#' @param mat A contingency table matrix for two ordinal variables
#'
#' @return The negative log-likelihood value for the given correlation coefficient
#'
#' @details
#' The function estimates thresholds from the marginal distributions and
#' calculates the expected probabilities based on a bivariate normal distribution.
#' It then computes the log-likelihood by comparing observed and expected frequencies.
#'
#' @keywords internal
#'
polychoric_likelihood <- function(rho, mat) {
  nr <- nrow(mat)
  nc <- ncol(mat)
  th_r <- rowSums(mat) / sum(mat)
  th_c <- colSums(mat) / sum(mat)
  a <- qnorm(pmin(cumsum(th_r), 1))
  b <- qnorm(pmin(cumsum(th_c), 1))

  exp_probs <- matrix(0, nrow = nr, ncol = nc)
  for (i in 1:nr) {
    for (j in 1:nc) {
      a_low <- if (i > 1) {
        a[i - 1]
      } else {
        -Inf
      }
      a_high <- a[i]
      b_low <- if (j > 1) {
        b[j - 1]
      } else {
        (-Inf)
      }
      b_high <- b[j]

      exp_probs[i, j] <- qBiNormal(a_high, b_high, rho) -
        qBiNormal(a_high, b_low, rho) -
        qBiNormal(a_low, b_high, rho) +
        qBiNormal(a_low, b_low, rho)
    }
  }

  log_lik <- sum(mat * log(pmax(exp_probs, 1e-10)))
  return(-log_lik)
}

#' @title Generate Score Report for Non-Binary Test Data
#'
#' @description
#' Calculates comprehensive descriptive statistics for a test, including measures of
#' central tendency, variability, distribution shape, and reliability.
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#' it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#' observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item
#' @param na Values to be treated as missing values
#'
#' @return An object of class "exametrika" and "TestStatistics" containing:
#' \describe{
#'   \item{TestLength}{Number of items included in the test}
#'   \item{SampleSize}{Number of examinees (rows) in the dataset}
#'   \item{Mean}{Average score across all examinees}
#'   \item{Median}{Median score}
#'   \item{SD}{Standard deviation of test scores}
#'   \item{Variance}{Variance of test scores}
#'   \item{Skewness}{Skewness of the score distribution (measure of asymmetry)}
#'   \item{Kurtosis}{Kurtosis of the score distribution (measure of tail extremity)}
#'   \item{Min}{Minimum score obtained}
#'   \item{Max}{Maximum score obtained}
#'   \item{Range}{Difference between maximum and minimum scores}
#'   \item{Alpha}{Cronbach's alpha coefficient, a measure of internal consistency reliability}
#' }
#'
#' @details
#' This function is intended for non-binary (ordinal or rated) response data. It calculates
#' descriptive statistics for the overall test performance. If binary data is provided,
#' an error message will be displayed.
#'
#' @importFrom stats median
#' @examples
#' \donttest{
#' # Generate score report for sample ordinal data
#' ScoreReport(J15S3810)
#'
#' # Example with rated data
#' ScoreReport(J35S5000)
#' }
#'
#' @export
ScoreReport <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  if (tmp$response.type == "binary") {
    response_type_error(U$response.type, "ScoreReport")
  }
  if (U$response.type == "rated") {
    tmp$Q <- tmp$U
  }

  nobs <- nrow(tmp$Q)
  nitems <- ncol(tmp$Q)
  ## score dist
  score <- rowSums(tmp$Z * tmp$Q)
  score.max <- max(score)
  score.median <- median(score)
  score.mean <- mean(score)
  score.min <- min(score)
  score.sd <- sd(score)
  score.range <- score.max - score.min

  m3 <- sum((score - mean(score))^3) / nobs
  s3 <- (sum((score - mean(score))^2) / nobs)^(3 / 2)
  score.skew <- m3 / s3
  m4 <- sum((score - mean(score))^4) / nobs
  s4 <- (sum((score - mean(score))^2) / nobs)^2
  score.kurt <- m4 / s4 - 3

  ##
  zzzTotal <- apply(tmp$Z, 2, sum)
  zzzMean <- apply(tmp$Z, 2, mean)
  itemMean <- apply(tmp$Z * tmp$Q, 2, sum) / zzzTotal
  itemSD <- sqrt(apply((tmp$Z * tmp$Q^2), 2, sum) / zzzTotal - itemMean^2)
  zzzScoreMean <- colSums(tmp$Z * score) / colSums(tmp$Z)
  zzzScoreSD <- sqrt(colSums(tmp$Z * score^2) / colSums(tmp$Z) - zzzScoreMean^2)
  cronbach01 <- (1 - (sum(itemSD^2) / var(score))) * nitems / (nitems - 1)

  ret <-
    structure(list(
      SampleSize = nobs,
      TestLength = nitems,
      Median = score.median,
      Max = score.max,
      Min = score.min,
      Range = score.range,
      Mean = score.mean,
      SD = score.sd,
      Skewness = score.skew,
      Kurtosis = score.kurt,
      Alpha = cronbach01
    ), class = c("exametrika", "TestStatistics"))
  return(ret)
}

#' @title Generate Item Report for Non-Binary Test Data
#'
#' @description
#' Calculates item-level statistics for non-binary test data, including response rates,
#' basic descriptive statistics, and item-total correlations.
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#' it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#' observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item
#' @param na Values to be treated as missing values
#'
#' @return An object of class "exametrika" and "QitemStatistics" containing:
#' \describe{
#'   \item{ItemLabel}{Labels identifying each item}
#'   \item{Obs}{Number of valid responses for each item}
#'   \item{ObsRatio}{Proportion of valid responses for each item (range: 0-1)}
#'   \item{ItemMean}{Mean score of each item}
#'   \item{ItemSD}{Standard deviation of each item score}
#'   \item{ItemCORR}{Item-total correlation coefficients - correlation between
#'         item scores and total test scores}
#'   \item{ItemCORR_R}{Corrected item-total correlation coefficients - correlation between
#'         item scores and total test scores excluding the target item}
#' }
#'
#' @details
#' This function is intended for non-binary (ordinal or rated) response data. It provides
#' detailed statistics for each item in the test, focusing on response patterns and
#' the relationship between individual items and overall test performance.
#' If binary data is provided, an error message will be displayed.
#'
#' @examples
#' \donttest{
#' # Generate item report for sample ordinal data
#' item_stats <- ItemReport(J15S3810)
#'
#' # View first few rows of the item report
#' head(item_stats)
#'
#' # Example with rated data including custom missing value indicator
#' item_stats2 <- ItemReport(J35S5000, na = -99)
#' }
#'
#' @export
ItemReport <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    dat <- dataFormat(U, na = na, Z = Z, w = w)
  } else {
    dat <- U
  }
  if (U$response.type == "binary") {
    response_type_error(dat$response.type, "ScoreReport")
  }
  if (U$response.type == "rated") {
    dat$Q <- dat$U
  }
  score <- rowSums(dat$Z * dat$Q)
  maxscore <- sum(apply(dat$Q, 2, max))
  score.dist <- table(factor(score, levels = 0:maxscore))

  zzzTotal <- apply(dat$Z, 2, sum)
  zzzMean <- apply(dat$Z, 2, mean)
  itemMean <- apply(dat$Z * dat$Q, 2, sum) / zzzTotal
  itemSD <- sqrt(apply((dat$Z * dat$Q^2), 2, sum) / zzzTotal - itemMean^2)
  zzzScoreMean <- colSums(dat$Z * score) / colSums(dat$Z)
  zzzScoreSD <- sqrt(colSums(dat$Z * score^2) / colSums(dat$Z) - zzzScoreMean^2)
  itemScoreCorr <- (colSums(dat$Z * dat$Q * score) / colSums(dat$Z) - itemMean * zzzScoreMean) / (zzzScoreSD * itemSD)
  scoreRemainder <- matrix(0, nrow = nrow(dat$Q), ncol = ncol(dat$Q))
  for (j in 1:ncol(dat$Q)) {
    scoreRemainder[, j] <- dat$Z[, j] * (score - dat$Q[, j])
  }
  zzzScoreReminderMean <- colSums(scoreRemainder) / zzzTotal
  zzzScoreRemainderSD <- sqrt(colSums(scoreRemainder^2) / zzzTotal - zzzScoreReminderMean^2)

  itemScoreRemainderCorr <- rep(NA, ncol(dat$Q))
  for (j in 1:ncol(dat$Q)) {
    term1 <- sum((dat$Z[, j] * dat$Q[, j] * scoreRemainder[, j]) / zzzTotal[j])
    term2 <- itemMean[j] * sum((scoreRemainder[, j]) / zzzTotal[j])
    itemScoreRemainderCorr[j] <- (term1 - term2) / (zzzScoreRemainderSD[j] * itemSD[j])
  }
  ret <-
    structure(list(
      ItemLabel = dat$ItemLabel,
      Obs = zzzTotal,
      ObsRatio = zzzMean,
      ItemMean = itemMean,
      ItemSD = itemSD,
      ItemCORR = itemScoreCorr,
      ItemCORR_R = itemScoreRemainderCorr
    ), class = c("exametrika", "QitemStatistics"))
  return(ret)
}

#' Grid Search for Optimal Parameters
#'
#' Performs a grid search to find optimal parameters for different
#' analysis methods.
#' Supports Biclustering, LCA (Latent Class Analysis),
#' and LRA (Latent Rank Analysis).
#'
#' @param obj Input data matrix or object to be analyzed
#' @param max_ncls Maximum number of classes/clusters to test (default: 10)
#' @param max_nfld Maximum number of fields to test for
#' Biclustering (default: 10)
#' @param fun Function name to use for analysis.
#' Options: "Biclustering", "LCA", "LRA" (default: "Biclustering")
#' @param index Fit index to optimize from TestFitIndices returned by
#'  each function. Valid options: "BIC" (default), "AIC", "CAIC",
#'  "model_log_like", "model_Chi_sq", "RMSEA", "NFI", "RFI", "IFI",
#'  "TLI", "CFI". Aliases are also accepted: "loglik", "log_lik",
#'  "LogLik", "LL" (all map to "model_log_like"), "Chi_sq", "chi_sq"
#'  (map to "model_Chi_sq").
#' @param verbose Logical; if TRUE, displays detailed progress messages during grid search. Default is TRUE.
#' @param ... Additional arguments passed to the analysis function
#'
#' @return A list containing:
#'   For Biclustering:
#'   \item{index_matrix}{Matrix of fit indices for each
#'    ncls/nfld combination}
#'   \item{optimal_ncls}{Optimal number of classes/clusters}
#'   \item{optimal_nfld}{Optimal number of fields}
#'   \item{optimal_result}{Analysis result using optimal parameters}
#'   \item{failed_settings}{List of parameter combinations that failed to converge}
#'
#'   For LCA/LRA:
#'   \item{index_vec}{Vector of fit indices for each ncls}
#'   \item{optimal_ncls}{Optimal number of classes/clusters}
#'   \item{optimal_result}{Analysis result using optimal parameters}
#'   \item{failed_settings}{List of parameter combinations that failed to converge}
#'
#' @examples
#' \dontrun{
#' # Grid search for Biclustering
#' result <- grid_serch(data_matrix, max_ncls = 5, max_nfld = 5)
#'
#' # Grid search for LCA
#' result <- grid_serch(data_matrix, max_ncls = 8, fun = "LCA")
#' }
#'
#' @export
#'
GridSearch <- function(
  obj,
  max_ncls = 10,
  max_nfld = 10,
  fun = "Biclustering",
  index = "BIC",
  verbose = TRUE,
  ...
) {
  obj <- dataFormat(obj)
  # Handle different data types for testlength calculation
  if (!is.null(obj$U)) {
    testlength <- NCOL(obj$U) # binary data
    nobs <- NROW(obj$U)
  } else {
    testlength <- NCOL(obj$Q) # ordinal data
    nobs <- NROW(obj$Q)
  }

  # Index alias mapping (normalize user-friendly names to internal field names)
  index_aliases <- c(
    "loglik" = "model_log_like",
    "log_lik" = "model_log_like",
    "LogLik" = "model_log_like",
    "LL" = "model_log_like",
    "Chi_sq" = "model_Chi_sq",
    "chi_sq" = "model_Chi_sq"
  )
  if (index %in% names(index_aliases)) {
    original_index <- index
    index <- index_aliases[[index]]
    message(sprintf("Note: index '%s' mapped to '%s'", original_index, index))
  }

  # Valid indices and their optimization direction
  minimize_indices <- c("model_Chi_sq", "RMSEA", "AIC", "CAIC", "BIC")
  maximize_indices <- c("model_log_like", "NFI", "RFI", "IFI", "TLI", "CFI")
  valid_indices <- c(minimize_indices, maximize_indices)

  if (!(index %in% valid_indices)) {
    stop(
      "Unknown index: ", index, ". Please specify one of: ",
      paste(valid_indices, collapse = ", "),
      "\nAliases also accepted: ",
      paste(names(index_aliases), collapse = ", ")
    )
  }

  # ------------------------------------------ Biclustering
  if (fun == "Biclustering") {
    if (max_ncls >= nobs) {
      max_ncls <- nobs - 1
    }
    if (max_nfld >= testlength) {
      max_nfld <- testlength - 1
    }
    ret <- matrix(NA, nrow = length(2:max_ncls), ncol = length(2:max_nfld))
    colnames(ret) <- paste("nfld", 2:max_nfld)
    rownames(ret) <- paste("ncls", 2:max_ncls)
    args_list <- list()
    extra_args <- list(...)
    failed_settings <- list()

    for (ncls in 2:max_ncls) {
      if (verbose) {
        message(sprintf("ncls = %d: ", ncls), appendLF = FALSE)
      }
      for (nfld in 2:max_nfld) {
        # Display progress
        if (verbose) {
          message(sprintf("nfld=%d ", nfld), appendLF = FALSE)
        }

        args_list <- c(
          list(
            U = obj,
            ncls = ncls,
            nfld = nfld,
            verbose = FALSE
          ),
          extra_args
        )
        # Tolerate per-cell errors: a single Biclustering failure (e.g.
        # empty-cluster edge cases at grid corners) should not abort the
        # entire grid search. Treat errors the same as non-convergence.
        result <- tryCatch(do.call(fun, args_list),
          error = function(e) NULL
        )
        if (is.null(result) ||
          (!is.null(result$converge) && isFALSE(result$converge))) {
          ret[ncls - 1, nfld - 1] <- NA
          failed_settings <- append(
            failed_settings,
            list(list(ncls = ncls, nfld = nfld))
          )
        } else {
          ret[ncls - 1, nfld - 1] <- result$TestFitIndices[[index]]
        }
      }
      if (verbose) {
        message("") # New line after each ncls
      }
    }

    # Check if all parameters failed to converge
    if (all(is.na(ret))) {
      message("Error: All parameter combinations failed to converge.")
      message("Grid search cannot find optimal parameters.")
      message("Consider:")
      message("  - Increasing maxiter parameter")
      message("  - Using different parameter ranges")
      message("  - Checking data quality")
      stop("Grid search terminated due to convergence failure in all combinations.")
    }

    if (index %in% minimize_indices) {
      optimal_idx <- which(ret == min(ret, na.rm = TRUE), arr.ind = TRUE)
    } else {
      optimal_idx <- which(ret == max(ret, na.rm = TRUE), arr.ind = TRUE)
    }
    optimal_ncls <- optimal_idx[1] + 1
    optimal_nfld <- optimal_idx[2] + 1

    # Display warning for failed convergence
    if (length(failed_settings) > 0) {
      message("\nWarning: The following settings may have failed to converge:")
      for (i in 1:length(failed_settings)) {
        setting <- failed_settings[[i]]
        message(sprintf("  ncls=%d, nfld=%d", setting$ncls, setting$nfld))
      }
    }

    message(paste(
      "\nOptimal ncls/nrank is ",
      optimal_ncls, "and",
      "Optimal nfld is",
      optimal_nfld
    ))

    # Run analysis with optimal parameters
    message("Running analysis with optimal parameters...")
    optimal_args <- c(
      list(
        U = obj,
        ncls = optimal_ncls,
        nfld = optimal_nfld
      ),
      extra_args
    )
    optimal_result <- do.call(fun, optimal_args)

    ret_list <- structure(list(
      index_matrix = ret,
      optimal_ncls = optimal_ncls,
      optimal_nfld = optimal_nfld,
      optimal_result = optimal_result,
      failed_settings = failed_settings
    ), class = c("exametrika", "GridSearch"))
    # ------------------------------------------ LCA / LRA
  } else if (fun == "LCA" || fun == "LRA") {
    if (max_ncls >= nobs) {
      max_ncls <- nobs - 1
    }
    ret <- vector(length = length(2:max_ncls))
    names(ret) <- paste0("ncls", 2:max_ncls)
    extra_args <- list(...)
    failed_settings <- list()

    for (i in seq_along(2:max_ncls)) {
      ncls <- (2:max_ncls)[i]
      # Display progress
      if (verbose) {
        message(sprintf("Executing ncls = %d...", ncls))
      }

      args_list <- c(list(
        U = obj,
        ncls = ncls,
        verbose = FALSE
      ), extra_args)
      # Tolerate per-cell errors (see Biclustering branch above for rationale).
      result <- tryCatch(do.call(fun, args_list),
        error = function(e) NULL
      )
      if (is.null(result) ||
        (!is.null(result$converge) && isFALSE(result$converge))) {
        ret[i] <- NA
        failed_settings <- append(
          failed_settings,
          list(list(ncls = ncls))
        )
      } else {
        ret[i] <- result$TestFitIndices[[index]]
      }
    }

    # Check if all parameters failed to converge
    if (all(is.na(ret))) {
      message("Error: All parameter combinations failed to converge.")
      message("Grid search cannot find optimal parameters.")
      message("Consider:")
      message("  - Increasing maxiter parameter")
      message("  - Using different parameter ranges")
      message("  - Checking data quality")
      stop("Grid search terminated due to convergence failure in all combinations.")
    }

    if (index %in% minimize_indices) {
      optimal_ncls <- which.min(ret) + 1
    } else {
      optimal_ncls <- which.max(ret) + 1
    }

    # Display warning for failed convergence
    if (length(failed_settings) > 0) {
      message("\nWarning: The following settings may have failed to converge:")
      for (i in 1:length(failed_settings)) {
        setting <- failed_settings[[i]]
        message(sprintf("  ncls=%d", setting$ncls))
      }
    }

    message("\nOptimal ncls/nrank is ", optimal_ncls)

    # Run analysis with optimal parameters
    message("Running analysis with optimal parameters...")
    optimal_args <- c(
      list(
        U = obj,
        ncls = optimal_ncls
      ),
      extra_args
    )
    optimal_result <- do.call(fun, optimal_args)

    ret_list <- structure(list(
      index_vec = ret,
      optimal_ncls = optimal_ncls,
      optimal_result = optimal_result,
      failed_settings = failed_settings
    ), class = c("exametrika", "GridSearch"))
  }
  return(ret_list)
}

library(exametrika)

### GridSearch Tests

### Setup - run Biclustering GridSearch once and share across tests
tmp_gs <- dataFormat(J35S515)
result_gs_bic <- GridSearch(tmp_gs,
  max_ncls = 3, max_nfld = 3,
  fun = "Biclustering", index = "BIC", verbose = FALSE
)

test_that("GridSearch Basic Execution (Biclustering)", {
  # Basic structure checks
  expect_s3_class(result_gs_bic, "exametrika")
  expect_true("GridSearch" %in% class(result_gs_bic))

  # Result components exist
  expect_true(!is.null(result_gs_bic$index_matrix))
  expect_true(!is.null(result_gs_bic$optimal_ncls))
  expect_true(!is.null(result_gs_bic$optimal_nfld))
  expect_true(!is.null(result_gs_bic$optimal_result))

  # index_matrix should be a matrix
  expect_true(is.matrix(result_gs_bic$index_matrix))

  # Optimal values should be within search range
  expect_true(result_gs_bic$optimal_ncls >= 2 && result_gs_bic$optimal_ncls <= 3)
  expect_true(result_gs_bic$optimal_nfld >= 2 && result_gs_bic$optimal_nfld <= 3)
})

test_that("GridSearch Optimal Result Has Fit Indices", {
  # Optimal result should have TestFitIndices
  opt <- result_gs_bic$optimal_result
  expect_true(!is.null(opt$TestFitIndices))
  expect_true(!is.null(opt$TestFitIndices$BIC))

  # BIC should be finite
  expect_true(is.finite(opt$TestFitIndices$BIC))
})

test_that("GridSearch LCA Method", {
  tmp <- dataFormat(J15S500)
  result <- GridSearch(tmp,
    max_ncls = 4,
    fun = "LCA", index = "BIC", verbose = FALSE
  )

  # Basic structure
  expect_s3_class(result, "exametrika")
  expect_true("GridSearch" %in% class(result))

  # LCA returns index_vec instead of index_matrix (no nfld dimension)
  expect_true(!is.null(result$index_vec))
  expect_true(!is.null(result$optimal_ncls))
  expect_true(!is.null(result$optimal_result))

  # Optimal ncls should be within range
  expect_true(result$optimal_ncls >= 2 && result$optimal_ncls <= 4)
})

#' @title Utility function for searching DAG
#' @description
#' Function to limit the number of parent nodes
#' @details
#' When generating an adjacency matrix using GA, the number of edges coming
#' from a single node should be limited to 2 or 3. This is because if
#' there are too many edges, it becomes difficult to interpret in practical
#' applications. This function works to adjust the sampling of the randomly
#' generated adjacency matrix so that the column sum of the upper triangular
#' elements fits within the set limit.
#' @param vec gene Vector corresponding to the upper triangular of the adjacency matrix
#' @param testlength test length. In this context it means a number of nodes.
#' @param maxParents Upper limit of number of nodes.

maxParents_penalty <- function(vec, testlength, maxParents) {
  adj_check <- matrix(0, ncol = testlength, nrow = testlength)
  adj_check[upper.tri(adj_check)] <- vec
  for (i in 2:testlength) {
    vec <- adj_check[, i]
    if (sum(vec) > maxParents) {
      ones_index <- which(vec == 1)
      selection <- sample(ones_index, maxParents)
      vec[-selection] <- 0
      adj_check[, i] <- vec
    }
  }
  vec <- adj_check[upper.tri(adj_check)]
  return(vec)
}

#' @title Model Fit Functions for Items
#' @description
#' A general function that returns the model fit indices.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param ell_A log likelihood of this model
#' @param nparam number of parameters for this model
#' @return
#' \describe{
#' \item{model_log_like}{log likelihood of analysis model}
#' \item{bench_log_like}{log likelihood of benchmark model}
#' \item{null_log_like}{log likelihood of null model}
#' \item{model_Chi_sq}{Chi-Square statistics for analysis model}
#' \item{null_Chi_sq}{Chi-Square statistics for null model}
#' \item{model_df}{degrees of freedom of analysis model}
#' \item{null_df}{degrees of freedom of null model}
#' \item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
#' \item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
#' \item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
#' \item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
#' }
#' @export

ItemFit <- function(U, Z, ell_A, nparam) {
  ## Item Fit Indices
  testlength <- ncol(U)
  nobs <- NROW(U)
  nrs <- colSums(Z)
  crr <- colSums(Z * U) / nrs
  const <- exp(-testlength)
  # Null model
  ell_N <- nobs * crr * log(crr + const) + nobs * (1 - crr) * log(1 - crr + const)
  # Benchmark model
  total <- rowSums(Z * U)
  totalList <- sort(unique(total))
  totalDist <- as.vector(table(total))
  ntotal <- length(totalList)
  ## Group Membership Profile Matrix
  MsG <- matrix(0, ncol = ntotal, nrow = nobs)
  for (i in 1:nobs) {
    MsG[i, which(totalList == total[i])] <- 1
  }
  ## PjG
  U1gj <- t(MsG) %*% (Z * U)
  U0gj <- replicate(testlength, totalDist, simplify = "matrix") - U1gj
  PjG <- U1gj / totalDist
  ell_B <- colSums(U1gj * log(PjG + const) + U0gj * log(1 - PjG + const))

  # chisquares
  chi_A <- 2 * (ell_B - ell_A)
  chi_B <- 2 * (ell_B - ell_N)

  # dfs
  df_B <- rep(ntotal - 1, testlength)
  df_A <- rep(ntotal - nparam, testlength)

  ItemFitIndices <- calcFitIndices(
    chi_A,
    chi_B,
    df_A,
    df_B,
    nobs
  )
  ItemFitIndices <- structure(
    c(list(
      model_log_like = ell_A,
      bench_log_like = ell_B,
      null_log_like = ell_N,
      model_Chi_sq = chi_A,
      null_Chi_sq = chi_B,
      model_df = df_A,
      null_df = df_B
    ), ItemFitIndices),
    class = c("exametrika", "ModelFit")
  )

  TestFitIndices <- calcFitIndices(
    sum(chi_A),
    sum(chi_B),
    sum(df_A),
    sum(df_B),
    nobs
  )
  TestFitIndices <- structure(
    c(list(
      model_log_like = sum(ell_A),
      bench_log_like = sum(ell_B),
      null_log_like = sum(ell_N),
      model_Chi_sq = sum(chi_A),
      null_Chi_sq = sum(chi_B),
      model_df = sum(df_A),
      null_df = sum(df_B)
    ), TestFitIndices),
    class = c("exametrika", "ModelFit")
  )

  ret <- list(
    item = ItemFitIndices,
    test = TestFitIndices
  )
  return(ret)
}

#' @title Model Fit Functions for test whole
#' @description
#' A general function that returns the model fit indices.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param ell_A log likelihood of this model
#' @param nparam number of parameters for this model
#' @return
#' \describe{
#' \item{model_log_like}{log likelihood of analysis model}
#' \item{bench_log_like}{log likelihood of benchmark model}
#' \item{null_log_like}{log likelihood of null model}
#' \item{model_Chi_sq}{Chi-Square statistics for analysis model}
#' \item{null_Chi_sq}{Chi-Square statistics for null model}
#' \item{model_df}{degrees of freedom of analysis model}
#' \item{null_df}{degrees of freedom of null model}
#' \item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
#' \item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
#' \item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
#' \item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
#' }
#' @export

TestFit <- function(U, Z, ell_A, nparam) {
  nres <- colSums(Z)
  nitem <- NCOL(U)
  nobs <- NROW(U)
  crr <- colSums(Z * U) / nres
  const <- exp(-nitem)
  # Benchmark model
  total <- rowSums(Z * U)
  totalList <- sort(unique(total))
  totalDist <- as.vector(table(total))
  ntotal <- length(totalList)
  ## Group Membership Profile Matrix
  MsG <- matrix(0, ncol = nobs, nrow = ntotal)
  for (i in 1:nobs) {
    MsG[which(total[i] == totalList), i] <- 1
  }
  ## PjG
  U1gj <- MsG %*% (Z * U)
  U0gj <- replicate(nitem, totalDist, simplify = "matrix") - U1gj
  PjG <- U1gj / totalDist
  ell_B <- colSums(U1gj * log(PjG + const) + U0gj * log(1 - PjG + const))
  ell_B <- sum(ell_B)
  bench_nparm <- ntotal * nitem
  # Null model
  ell_N <- nobs * crr * log(crr + const) + nobs * (1 - crr) * log(1 - crr + const)
  ell_N <- sum(ell_N)
  null_nparam <- nitem
  df_B <- bench_nparm - null_nparam
  chi_B <- 2 * (ell_B - ell_N)
  # Analysis model
  chi_A <- 2 * (ell_B - ell_A)
  df_A <- bench_nparm - nparam

  indices <- calcFitIndices(chi_A, chi_B, df_A, df_B, nobs)
  TestFitIndices <- structure(
    c(list(
      model_log_like = ell_A,
      bench_log_like = ell_B,
      null_log_like = ell_N,
      model_Chi_sq = chi_A,
      null_Chi_sq = chi_B,
      model_df = df_A,
      null_df = df_B
    ), indices),
    class = c("exametrika", "ModelFit")
  )
}

#' @title Model Fit Functions for saturated model
#' @description
#' A general function that returns the model fit indices.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param ell_A log likelihood of this model
#' @param nparam number of parameters for this model
#' @return
#' \describe{
#' \item{model_log_like}{log likelihood of analysis model}
#' \item{bench_log_like}{log likelihood of benchmark model}
#' \item{null_log_like}{log likelihood of null model}
#' \item{model_Chi_sq}{Chi-Square statistics for analysis model}
#' \item{null_Chi_sq}{Chi-Square statistics for null model}
#' \item{model_df}{degrees of freedom of analysis model}
#' \item{null_df}{degrees of freedom of null model}
#' \item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
#' \item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
#' \item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
#' \item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
#' }
#' @export

TestFitSaturated <- function(U, Z, ell_A, nparam) {
  nres <- colSums(Z)
  nitem <- NCOL(U)
  nobs <- NROW(U)
  crr <- colSums(U) / nres
  const <- exp(-nitem)
  # Benchmark model
  ell_B <- 0
  npattern <- length(as.numeric(table(apply(U, 1, paste, collapse = ""))))
  bench_nparm <- npattern * nitem
  # Null model
  ell_N <- nobs * crr * log(crr + const) + nobs * (1 - crr) * log(1 - crr + const)
  ell_N <- sum(ell_N)
  null_nparam <- nitem
  df_B <- bench_nparm - null_nparam
  chi_B <- 2 * (ell_B - ell_N)
  # Analysis model
  chi_A <- 2 * (ell_B - ell_A)
  df_A <- bench_nparm - nparam

  indices <- calcFitIndices(chi_A, chi_B, df_A, df_B, nobs)
  TestFitIndices <- structure(
    c(list(
      model_log_like = ell_A,
      bench_log_like = ell_B,
      null_log_like = ell_N,
      model_Chi_sq = chi_A,
      null_Chi_sq = chi_B,
      model_df = df_A,
      null_df = df_B
    ), indices),
    class = c("exametrika", "ModelFit")
  )
}

#' @title calc Fit Indices
#' @description
#' A general function that returns the model fit indices.
#' @param chi_A chi-squares for this model
#' @param chi_B chi-squares for compared model
#' @param df_A degrees of freedom for this model
#' @param df_B degrees of freedom for compared model
#' @param nobs number of observations for Information criteria
#' @return
#' \describe{
#' \item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
#' \item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
#' \item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
#' \item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
#' \item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
#' }
#' @export
#'

calcFitIndices <- function(chi_A, chi_B, df_A, df_B, nobs) {
  corrected_values <- pmax(chi_A - df_A, 0)
  RMSEA <- sqrt(corrected_values / (df_A * (nobs - 1)))

  AIC <- chi_A - 2 * df_A
  CAIC <- chi_A - df_A * (log(nobs) + 1)
  BIC <- chi_A - df_A * log(nobs)

  NFI <- (chi_A / chi_B)
  RFI <- ((chi_A / df_A) / (chi_B / df_B))
  IFI <- ((chi_A - df_A) / (chi_B - df_A))
  TLI <- ((chi_A / df_A) - 1) / ((chi_B / df_B) - 1)
  CFI <- ((chi_A - df_A) / (chi_B - df_B))

  ## Clip Function
  vars <- list(NFI, RFI, IFI, TLI, CFI)
  names <- c("NFI", "RFI", "IFI", "TLI", "CFI")

  for (i in 1:length(vars)) {
    assign(names[i], 1 - pmin(pmax(vars[[i]], 0), 1))
  }

  for (i in 1:length(chi_A)) {
    if (df_A[i] <= 0) {
      RFI[i] <- NaN
      TLI[i] <- NaN
      RMSEA[i] <- NaN
    }
  }

  return(list(
    NFI = NFI, RFI = RFI, IFI = IFI, TLI = TLI, CFI = CFI,
    RMSEA = RMSEA, AIC = AIC, CAIC = CAIC, BIC = BIC
  ))
}

#' @title Data Frame DAG to adj
#' @description inner function of return full length adj from data.frame
#' @param g igraph object
#' @param ItemLabel Labels for all items. Information should held by the
#' exametrika class data.
#' @importFrom igraph as_adjacency_matrix
#' @return adjacency matrix
#' @noRd

fill_adj <- function(g, ItemLabel) {
  testlength <- length(ItemLabel)
  tmp_adj <- as.matrix(igraph::as_adjacency_matrix(g))
  adj <- matrix(0, ncol = testlength, nrow = testlength)
  colnames(adj) <- rownames(adj) <- ItemLabel
  tmp_col_names <- colnames(tmp_adj)
  for (col_name in colnames(adj)) {
    if (col_name %in% tmp_col_names) {
      for (tmp_col in tmp_col_names) {
        adj[tmp_col, col_name] <- tmp_adj[tmp_col, col_name]
      }
    }
  }
  adj <- adj[sort(rownames(adj)), sort(colnames(adj))]
  return(adj)
}

#' @title Bayesian Network Model
#' @description
#' performs Bayesian Network Model with specified graph structure
#' @details
#' This function performs a Bayesian network analysis on the relationships
#' between items. This corresponds to Chapter 8 of the text. It uses the igraph
#' package for graph visualization and checking the adjacency matrix.
#' You need to provide either a graph object or a CSV file where the graph
#'  structure is specified.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param g Specify a graph object suitable for the igraph class.
#' @param adj_file specify CSV file where the graph structure is specified.
#' @param adj_matrix specify adjacency matrix.
#' @param beta1 Beta distribution parameter 1 (for correct responses). Default is 1.
#' @param beta2 Beta distribution parameter 2 (for incorrect responses). Default is 1. Note: referred to as beta0 internally.
#' @importFrom igraph graph_from_data_frame
#' @importFrom igraph as_adjacency_matrix
#' @importFrom utils read.csv
#' @importFrom igraph V
#' @return
#' \describe{
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{crr}{correct response ratio}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{adj}{Adjacency matrix}\
#'  \item{param}{Learned Parameters}
#'  \item{CCRR_table}{Correct Response Rate tables}
#' }
#' @examples
#' \donttest{
#' # Create a Directed Acyclic Graph (DAG) structure for item relationships
#' # Each row represents a directed edge from one item to another
#' DAG <-
#'   matrix(
#'     c(
#'       "Item01", "Item02", # Item01 influences Item02
#'       "Item02", "Item03", # Item02 influences Item03
#'       "Item02", "Item04", # Item02 influences Item04
#'       "Item03", "Item05", # Item03 influences Item05
#'       "Item04", "Item05" # Item04 influences Item05
#'     ),
#'     ncol = 2, byrow = TRUE
#'   )
#'
#' # Convert the DAG matrix to an igraph object for network analysis
#' g <- igraph::graph_from_data_frame(DAG)
#' g
#'
#' # Create adjacency matrix from the graph
#' # Shows direct connections between items (1 for connection, 0 for no connection)
#' adj_mat <- as.matrix(igraph::as_adjacency_matrix(g))
#' print(adj_mat)
#'
#' # Fit Bayesian Network Model using the specified adjacency matrix
#' # Analyzes probabilistic relationships between items based on the graph structure
#' result.BNM <- BNM(J5S10, adj_matrix = adj_mat)
#' result.BNM
#' }
#'
#' @export

BNM <- function(U, Z = NULL, w = NULL, na = NULL,
                g = NULL, adj_file = NULL, adj_matrix = NULL,
                beta1 = 1, beta2 = 1) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (U$response.type != "binary") {
    response_type_error(U$response.type, "BNM")
  }


  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  # graph check
  if (is.null(g) && is.null(adj_file) && is.null(adj_matrix)) {
    stop("Specify the graph in either matrix form, CSV file, or as a graph object.")
  }
  if (!is.null(g)) {
    value <- class(g)[1]
    if (value != "igraph") {
      stop("The provided graph is not compatible with the igraph class.")
    }
  }
  if (!is.null(adj_matrix)) {
    g <- igraph::graph_from_adjacency_matrix(adj_matrix)
  }
  if (!is.null(adj_file)) {
    DAG <- read.csv(adj_file, header = TRUE)
    g <- igraph::graph_from_data_frame(DAG)
  }

  graph_label <- V(g)$name
  if (!any(graph_label %in% tmp$ItemLabel)) {
    stop("Some labels in the graph do not align with the item labels.")
  }

  # get Adj matrix
  adj <- fill_adj(g, tmp$ItemLabel)

  ### Adj mat check
  adjU <- adj + t(adj)
  simpleFLG <- ifelse(max(adjU) <= 1, 1, 0)
  testlength <- ncol(adj)
  acyclicFLG <- 0
  connectedFLG <- 0
  for (i in 1:(testlength - 1)) {
    acyclicFLG <- acyclicFLG + sum(diag(adj^i))
    connectedFLG <- connectedFLG + min(sum(adjU^i))
  }
  acyclicFLG <- ifelse(acyclicFLG == 0, 1, 0)
  connectedFLG <- ifelse(connectedFLG > 0, 1, 0)
  dag <- simpleFLG * acyclicFLG
  cdag <- dag * connectedFLG

  # Initialize
  beta0 <- beta2
  npa <- colSums(adj)

  pir <- lapply(1:length(npa), function(i) {
    if (npa[i] > 0) {
      mat <- as.matrix(tmp$U[, adj[, i] == 1])
      colnames(mat) <- colnames(tmp$U)[adj[, i] == 1]
      return(mat)
    } else {
      mat <- as.matrix(rep(0, nobs))
      colnames(mat) <- "No Parents"
      return(mat)
    }
  })

  PIRP_mat <- matrix(nrow = nobs, ncol = testlength)
  for (s in 1:nobs) {
    for (j in 1:testlength) {
      # For each element of the matrix, calculate the decimal value from
      # the binary representation represented by the row of the pir list
      # at the corresponding index rev() reverses the binary vector,
      # which() identifies the indices of 1s,
      # and sum(2^(indices - 1)) converts binary to decimal
      PIRP_mat[s, j] <- sum(2^(which(rev(pir[[j]][s, ]) == 1) - 1)) + 1
    }
  }

  item_pattern_max <- 2^max(npa)
  PIRP_array <- array(0, dim = c(nobs, testlength, item_pattern_max))
  for (s in 1:nobs) {
    for (j in 1:testlength) {
      PIRP_array[s, j, PIRP_mat[s, j]] <- 1
    }
  }

  n_PIRP_1 <- matrix(nrow = testlength, ncol = item_pattern_max)
  n_PIRP_0 <- matrix(nrow = testlength, ncol = item_pattern_max)
  for (i in 1:item_pattern_max) {
    n_PIRP_1[, i] <- colSums(tmp$U * PIRP_array[, , i])
    n_PIRP_0[, i] <- colSums(tmp$Z * (1 - tmp$U) * PIRP_array[, , i])
  }

  deno <- n_PIRP_0 + n_PIRP_1 + beta0 + beta1 - 2
  denom0 <- sign(n_PIRP_1 + n_PIRP_0)

  param <- (n_PIRP_1 + beta1 - 1) / deno
  rownames(param) <- tmp$ItemLabel
  colnames(param) <- paste("PIRP", 1:ncol(param))

  # Model Fit
  const <- pmin(exp(-testlength), 1e-10)
  bounded <- pmax(pmin((param * denom0 + const), 1 - const), const)
  model_loglike <- sum(n_PIRP_1 * log(bounded) + n_PIRP_0 * log(1 - bounded), na.rm = T)
  model_nparam <- sum(denom0)
  FitIndices <- TestFit(tmp$U, tmp$Z, model_loglike, model_nparam)

  ## for output
  item_ptn <- 2^colSums(adj)
  for (j in 1:testlength) {
    for (i in 1:item_pattern_max) {
      if (i > item_ptn[j]) {
        param[j, i] <- NA
      }
    }
  }

  CCRR_table <- as.data.frame(matrix(NA, nrow = sum(item_ptn), ncol = 5))
  colnames(CCRR_table) <- c("Child Item", "N of Parents", "Parent Items", "PIRP", "Conditional CRR")
  CCRR_table[, 1] <- rep(tmp$ItemLabel, item_ptn)
  CCRR_table[, 2] <- rep(colSums(adj), item_ptn)
  parent_items <- lapply(pir, function(mat) {
    paste(colnames(mat), collapse = ", ")
  })
  CCRR_table[, 3] <- rep(unlist(parent_items), item_ptn)

  CCRR_table[, 4] <- unlist(sapply(colSums(adj), BitRespPtn))

  vec <- numeric(sum(item_ptn))
  cnt <- 0
  for (j in 1:testlength) {
    for (i in 1:item_ptn[j]) {
      cnt <- cnt + 1
      vec[cnt] <- sprintf("%.7f", as.numeric(param[j, i]))
      if (is.nan(param[j, i])) {
        vec[cnt] <- "NaN(0/0)"
      }
    }
  }
  CCRR_table[, 5] <- vec

  ret <- structure(list(
    U = U,
    testlength = testlength,
    nobs = nobs,
    crr = crr(U),
    ItemLabel = tmp$ItemLabel,
    adj = adj,
    g = g,
    acyclicFLG = acyclicFLG,
    connectedFLG = connectedFLG,
    param = param,
    TestFitIndices = FitIndices,
    log_lik = model_loglike,
    CCRR_table = CCRR_table
  ), class = c("exametrika", "BNM"))
  return(ret)
}

#' @title Binary pattern maker
#' @param n decimal numbers
#' @return binary patterns
#' @details
#' if n <- 1, return 0,1
#' if n <- 2, return 00,01,10,11
#' and so on.
#'

BitRespPtn <- function(n) {
  if (n == 0) {
    ptn <- "No Pattern"
  } else {
    ptn <- sapply(0:(2^n - 1), function(x) {
      binary_str <- as.integer(intToBits(x)[1:n])
      paste0(rev(binary_str), collapse = "")
    })
  }
  return(ptn)
}

library(exametrika)

### GOALS - Mathematica reference data
Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter08BNM_Test.csv"),
  check.names = FALSE
)
Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter08BNM_Item.csv"),
  check.names = FALSE
)
CCRR <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter08BNM_CCRR.csv"),
  check.names = FALSE
)
Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter08BNM_Student.csv"),
  check.names = FALSE
)

### Setup
tmp <- dataFormat(J5S10)
library(igraph)
DAG <-
  matrix(
    c(
      "Item01", "Item02",
      "Item02", "Item03",
      "Item02", "Item04",
      "Item03", "Item05",
      "Item04", "Item05"
    ),
    ncol = 2, byrow = TRUE
  )
## graph object
g <- igraph::graph_from_data_frame(DAG)
## Adj mmatrix
adj_mat <- as.matrix(igraph::as_adjacency_matrix(g))
tgt <- BNM(tmp, adj_matrix = adj_mat)


# test1 Test ------------------------------------------------------

test_that("Test Info", {
  expect <- Test[12:27, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- tgt$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Item ------------------------------------------------------------

test_that("Item Info", {
  expect <- Item[1:5, 6:9] |>
    unlist() |>
    unname()
  expect <- expect[-10] |>
    as.numeric() |>
    na.omit() |>
    as.vector()
  result <- tgt$param |>
    as.numeric() |>
    na.omit() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})


# CCRR ------------------------------------------------------------

test_that("Conditional Correct Response Rate", {
  expect <- CCRR[, 1] |>
    unlist() |>
    unname()
  result <- tgt$CCRR[, 1]
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 2] |>
    unlist() |>
    unname()
  result <- tgt$CCRR[, 2]
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 3] |>
    unlist() |>
    unname()
  result <- tgt$CCRR[, 3]
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 4] |>
    unlist() |>
    unname()
  result <- tgt$CCRR[, 4]
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 5] |>
    unlist() |>
    unname()
  expect <- expect[-9] |>
    as.numeric()
  result <- tgt$CCRR[, 5]
  result <- result[-9] |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

#' @rdname Biclustering
#' @param ncls Number of latent classes/ranks to identify (between 2 and 20).
#' @param nfld Number of latent fields (item clusters) to identify.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#' observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @param method Analysis method to use (character string):
#'   * For nominal data, only "Biclustering" is available.
#' @param conf Confirmatory parameter for pre-specified field assignments. Can be either:
#'   * A vector with items and corresponding fields in sequence
#'   * A field membership profile matrix (items × fields) with 0/1 values
#'   * NULL (default) for exploratory analysis where field memberships are estimated
#' @param maxiter Maximum number of EM algorithm iterations. Default is 100.
#' @param verbose Logical; if TRUE, displays progress during estimation. Default is TRUE.
#' @param alpha Dirichlet distribution concentration parameter for prior density of field reference probabilities. Default is 1.
#' @param ... Additional arguments passed to specific methods.
#' #'
#' @examples
#' \donttest{
#' # Perform Biclustering for nominal sample data()
#' # Analyze data with 5 fields and 6 classes
#' result.Bi <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
#' }
#' @export
Biclustering.nominal <- function(U,
                                 ncls = 2, nfld = 2,
                                 conf = NULL,
                                 conf_class = NULL,
                                 mic = FALSE,
                                 maxiter = 100,
                                 verbose = TRUE,
                                 alpha = 1, ...) {
  tmp <- U
  maxQ <- max(tmp$Q)
  nobs <- NROW(tmp$Q)
  nitems <- NCOL(tmp$Q)
  const <- exp(-nitems)
  test_log_lik <- -1 / const
  old_test_log_lik <- -2 / const
  emt <- 0
  maxemt <- maxiter

  ncat <- as.vector(tmp$categories)

  ## confirmatory
  # set conf_mat for confirmatory clustering
  if (!is.null(conf)) {
    if (verbose) {
      message("Confirmatory Clustering is chosen.")
    }
    if (is.vector(conf)) {
      # check size
      if (length(conf) != NCOL(U$Q)) {
        stop("conf vector size does NOT match with data.")
      }
      conf_mat <- matrix(0, nrow = NCOL(U$Q), ncol = max(conf))
      for (i in 1:NROW(conf_mat)) {
        conf_mat[i, conf[i]] <- 1
      }
    } else if (is.matrix(conf) | is.data.frame(conf)) {
      if (NROW(conf) != NCOL(U$Q)) {
        stop("conf matrix size does NOT match with data.")
      }
      if (any(!conf %in% c(0, 1))) {
        stop("The conf matrix should only contain 0s and 1s.")
      }
      if (any(rowSums(conf) > 1)) {
        stop("The row sums of the conf matrix must be equal to 1.")
      }
      conf_mat <- as.matrix(conf)
    } else {
      stop("conf matrix is not set properly.")
    }
    ###
    nfld <- NCOL(conf_mat)
  } else {
    conf_mat <- NULL
  }

  # set conf_class_mat for class-side confirmatory clustering
  if (!is.null(conf_class)) {
    if (verbose) {
      message("Class-side Confirmatory Clustering is chosen.")
    }
    if (is.vector(conf_class)) {
      if (length(conf_class) != nobs) {
        stop("conf_class vector size does NOT match with the number of respondents.")
      }
      conf_class_mat <- matrix(0, nrow = nobs, ncol = max(conf_class))
      for (i in 1:NROW(conf_class_mat)) {
        conf_class_mat[i, conf_class[i]] <- 1
      }
    } else if (is.matrix(conf_class) | is.data.frame(conf_class)) {
      if (NROW(conf_class) != nobs) {
        stop("conf_class matrix size does NOT match with the number of respondents.")
      }
      if (any(!conf_class %in% c(0, 1))) {
        stop("The conf_class matrix should only contain 0s and 1s.")
      }
      if (any(rowSums(conf_class) > 1)) {
        stop("The row sums of the conf_class matrix must be equal to 1.")
      }
      conf_class_mat <- as.matrix(conf_class)
    } else {
      stop("conf_class is not set properly.")
    }
    ncls <- NCOL(conf_class_mat)
  } else {
    conf_class_mat <- NULL
  }

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  if (alpha <= 0) {
    stop("alpha must be positive (alpha > 0)")
  }

  ##
  fld0 <- pmin(ceiling(1:nitems / (nitems / nfld)), nfld)
  med_order <- order(apply(tmp$Q, 2, median), decreasing = TRUE)
  fld <- fld0[match(1:nitems, med_order)]
  fldmemb <- matrix(0, nrow = nitems, ncol = nfld)
  for (i in 1:nitems) {
    fldmemb[i, fld[i]] <- 1
  }
  ## Confirmatory Biclustering
  if (!any(is.null(conf_mat))) {
    fldmemb <- conf_mat
  }

  ## 3-dimensiona allay(nfld x ncls x Q)
  BCRM <- array(NA, dim = c(nfld, ncls, maxQ))
  for (i in 1:nfld) {
    for (j in 1:ncls) {
      class_effect <- j / ncls
      field_effect <- (nfld - i + 1) / nfld
      BCRM[i, j, ] <- init_field_membership_probs(maxQ, class_effect, field_effect)
    }
  }

  # One-hot encode tmp$Q into Uq[i, j, tmp$Q[i,j]] = 1 using matrix indexing.
  # Replaces a nobs*nitems R-level double loop with a single C-level write
  # into the flat backing storage of the 3-D array.
  # Missing entries (tmp$Z == 0) are left at zero; every downstream use of
  # Uq is masked by tmp$Z so the missing-cell values are never read.
  Uq <- array(0, dim = c(nobs, nitems, maxQ))
  valid <- as.vector(tmp$Z) == 1
  Uq[cbind(
    rep(seq_len(nobs), times = nitems)[valid],
    rep(seq_len(nitems), each = nobs)[valid],
    as.vector(tmp$Q)[valid]
  )] <- 1

  # iteration -------------------------------------------------------
  converge <- TRUE
  FLG <- TRUE
  while (FLG) {
    if (!is.finite(test_log_lik) ||
      test_log_lik - old_test_log_lik < 1e-8 * abs(old_test_log_lik)) {
      if (!is.finite(test_log_lik)) converge <- FALSE
      FLG <- FALSE
      break
    }
    if (emt == maxemt) {
      message("\nReached the maximum number of iterations (", maxemt, ").")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
      converge <- FALSE
      FLG <- FALSE
    }

    emt <- emt + 1
    old_test_log_lik <- test_log_lik

    tmpL <- matrix(0, nrow = nobs, ncol = ncls)
    for (q in 1:maxQ) {
      tmpL <- tmpL + (tmp$Z * Uq[, , q]) %*% fldmemb %*% log(BCRM[, , q] + const)
    }

    minllsr <- apply(tmpL, 1, min)
    expllsr <- exp(pmin(tmpL - minllsr, 700))
    clsmemb <- round(expllsr / rowSums(expllsr), 1e8)

    if (!is.null(conf_class_mat)) {
      clsmemb <- conf_class_mat
    }

    tmpH <- matrix(0, nrow = nitems, ncol = nfld)
    for (q in 1:maxQ) {
      tmpH <- tmpH + (t(tmp$Z * Uq[, , q]) %*% clsmemb) %*% t(log(BCRM[, , q] + const))
    }

    minllsr <- apply(tmpH, 1, min)
    expllsr <- exp(pmin(tmpH - minllsr, 700))
    fldmemb <- round(expllsr / rowSums(expllsr), 1e8)

    if (!any(is.null(conf_mat))) {
      fldmemb <- conf_mat
    }

    ## Maximization
    oldBCRM <- BCRM
    Ufcq <- array(0, dim = c(nfld, ncls, maxQ))
    for (q in 1:maxQ) {
      Ufcq[, , q] <- (t(fldmemb) %*% t(tmp$Z * Uq[, , q])) %*% clsmemb
    }

    # Apply Dirichlet prior (alpha parameter).
    # rowSums(Ufcq, dims=2) sums over the 3rd dim (categories), replacing
    # apply(Ufcq, c(1,2), sum) at C-level instead of per-cell apply.
    BCRM <- (Ufcq + alpha - 1) / array(rowSums(Ufcq, dims = 2) + maxQ * alpha - maxQ, dim = dim(BCRM))

    test_log_lik <- 0
    for (q in 1:maxQ) {
      pred_prob <- t(fldmemb %*% BCRM[, , q] %*% t(clsmemb))
      observed_mask <- (tmp$Z * Uq[, , q]) == 1
      test_log_lik <- test_log_lik + sum(log(pmax(pred_prob[observed_mask], const)))
    }

    if (verbose) {
      message(
        sprintf(
          "\r%-80s",
          paste0(
            "iter ", emt, " log_lik ", format(test_log_lik, digits = 6)
          )
        ),
        appendLF = FALSE
      )
    }

    if (!is.finite(test_log_lik) || test_log_lik - old_test_log_lik <= 0) {
      BCRM <- oldBCRM
      if (!is.finite(test_log_lik)) converge <- FALSE
      break
    }
  }


  # output ----------------------------------------------------------
  cls <- apply(clsmemb, 1, which.max)
  fld <- apply(fldmemb, 1, which.max)
  check_empty_fields(fld, nfld)
  fldmemb01 <- sign(fldmemb - apply(fldmemb, 1, max)) + 1
  flddist <- colSums(fldmemb01)
  clsmemb01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  clsdist <- colSums(clsmemb01)
  StudentRank <- cbind(clsmemb, Estimate = cls)
  rownames(StudentRank) <- tmp$ID
  colnames(StudentRank) <- c(paste("Membership", 1:ncls), "Estimate")


  # model fit -------------------------------------------------------
  testell <- 0
  for (q in 1:maxQ) {
    pred_prob <- t(fldmemb %*% BCRM[, , q] %*% t(clsmemb))
    observed_mask <- (tmp$Z * Uq[, , q]) == 1
    testell <- testell + sum(log(pmax(pred_prob[observed_mask], const)))
  }
  nparam <- ncls * nfld * (maxQ - 1)

  # Null model.
  # Zrep * Uq = Z * Uq[,,q] elementwise; compute via column-major recycling
  # (Uq * as.vector(Z)) which avoids the replicate() allocation.
  # colSums(X, dims=1) sums over the 1st dim, replacing apply(X, c(2,3), sum).
  ZU <- Uq * as.vector(tmp$Z)
  ZU_col_sums <- colSums(ZU, dims = 1)
  NullFRQ <- ZU_col_sums / colSums(tmp$Z)
  ell_N <- sum(ZU_col_sums * log(NullFRQ + const))

  # Fit indices: For nominal data, no meaningful benchmark (saturated) model exists
  # because response patterns are almost always unique with many items and categories.
  # Only information criteria (AIC, BIC, CAIC) are reported.
  AIC <- -2 * testell + 2 * nparam
  CAIC <- -2 * testell + nparam * (log(nobs) + 1)
  BIC <- -2 * testell + nparam * log(nobs)

  FitIndices <- structure(
    list(
      model_log_like = testell,
      bench_log_like = NA,
      null_log_like = ell_N,
      model_Chi_sq = NA,
      null_Chi_sq = NA,
      model_df = NA,
      null_df = NA,
      NFI = NA,
      RFI = NA,
      IFI = NA,
      TLI = NA,
      CFI = NA,
      RMSEA = NA,
      AIC = AIC,
      CAIC = CAIC,
      BIC = BIC
    ),
    class = c("exametrika", "ModelFit")
  )

  msg <- "Class"
  ret <- structure(list(
    Q = tmp$Q,
    Z = tmp$Z,
    testlength = nitems,
    msg = msg,
    converge = converge,
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    n_cycle = emt,
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = emt,
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = BCRM,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    TestFitIndices = FitIndices,
    log_lik = testell, # New naming convention
    # Deprecated fields (for backward compatibility)
    LogLik = testell
  ), class = c("exametrika", "nominalBiclustering"))

  return(ret)
}

#' @title log-likelihood function for ability parameter
#' @description An internal function to calculate the log-likelihood
#' @param theta target unknown parameter
#' @param Lambda Item parameters matrix
#' @param U data matrix
#' @param Z missing indicator
#' @noRd
LLtheta_mat <- function(theta, Lambda, U, Z) {
  f <- function(x) {
    p <- Lambda[, 3] + (Lambda[, 4] - Lambda[, 3]) / (1 + exp(-Lambda[, 1] * (x - Lambda[, 2])))
    ll <- sum(U * log(p) + (1 - U) * log(1 - p)) - x^2 / 2
    ll <- U %*% log(p) + (1 - U) %*% log(1 - p) - x^2 / 2
    return(exp(ll))
  }
  f_vec <- Vectorize(f)
  return(f_vec(theta))
}


#' @title Ability Parameter Estimation Function
#' @description
#' A function that returns the Expected A Posteriori (EAP) of the
#' ability parameter and its posterior standard deviation.
#' @param Lambda Item parameters matrix
#' @param U U is a data matrix of the type matrix or data.frame.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @noRd

EAP_PSD <- function(Lambda, U, Z) {
  dev <- 0.01
  xdev <- seq(-5, 5, dev)
  N <- length(xdev)

  tmp1 <- LLtheta_mat(xdev[1:N - 1], Lambda, U, Z)
  tmp2 <- LLtheta_mat(xdev[2:N], Lambda, U, Z)

  denominator <- rowSums((tmp1 + tmp2) * dev / 2)
  numerator <- rowSums((tmp1 %*% diag(xdev[1:N - 1]) + tmp2 %*% diag(xdev[2:N])) * dev / 2)
  EAP <- numerator / denominator

  weight <- ((EAP %*% t(rep(1, length(xdev)))) - ((rep(1, length(EAP))) %*% t(xdev)))^2
  numerator <- rowSums((tmp1 * weight[, 1:N - 1] + tmp2 * weight[, 2:N]) * dev / 2)
  PSDs <- sqrt(numerator / denominator)

  ret <- list(EAP = EAP, PSDs = PSDs)
  ret <- structure(ret, class = c("exametrika", "IRT_EAP_PSD"))
  return(ret)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)

## ----library, message=FALSE, warning=FALSE------------------------------------
library(exametrika)

## ----model-irt, message=FALSE, warning=FALSE----------------------------------
result.IRT <- IRT(J15S500, model = 3)
result.IRT

## ----results-irt-ability, message=FALSE, warning=FALSE------------------------
head(result.IRT$ability)

## ----plot-irt-irf, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot(result.IRT, type = "IRF", items = 1:6, nc = 2, nr = 3)

## ----plot-irt-overlay, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot(result.IRT, type = "IRF", overlay = TRUE)

## ----plot-irt-iic, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot(result.IRT, type = "IIC", items = 1:6, nc = 2, nr = 3)

## ----plot-irt-trf, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot(result.IRT, type = "TRF")

## ----plot-irt-tic, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot(result.IRT, type = "TIC")

## ----grm, message=FALSE, warning=FALSE----------------------------------------
result.GRM <- GRM(J5S1000)
result.GRM

## ----grm-irf, fig.width=7, fig.height=8, message=FALSE, warning=FALSE---------
plot(result.GRM, type = "IRF", nc = 2)

## ----grm-iif, fig.width=7, fig.height=8, message=FALSE, warning=FALSE---------
plot(result.GRM, type = "IIF", nc = 2)

## ----grm-tif, fig.width=7, fig.height=5, message=FALSE, warning=FALSE---------
plot(result.GRM, type = "TIF")

#' @title Plot Method for Objects of Class "exametrika"
#' @description
#' Creates visualizations for objects with class "exametrika".
#' The calculation results of the exametrika package have an exametrika class attribute,
#' along with the specific analysis model class (IRT, GRM, LCA, LRA, Biclustering, Biclustering_IRM, LDLRA, LDB,
#' BINET). Each model has its own compatible plot types, accessible by specifying the 'type' parameter.
#'
#' @param x An object of class "exametrika"
#' @param type Character string specifying the plot type. Available types vary by model:
#'   \describe{
#'     \item{IRF, ICC}{Item Response Function. Also known as 'ICC' (Item Characteristic Curve).}
#'     \item{TRF}{Test Response Function.}
#'     \item{IIF, IIC}{Item Information Function. Also known as 'IIC' (Item Information Curve).}
#'     \item{TIF, TIC}{Test Information Function. Also known as 'TIC' (Test Information Curve).}
#'     \item{IRP}{Item Reference Profile. Line graph with items and latent classes/ranks
#'       on the horizontal axis, and membership probability on the vertical axis.}
#'     \item{CRV, RRV}{Class/Rank Reference Vector. Plots correct answer rates for each class or rank,
#'       with fields on the horizontal axis and correct answer rates on the vertical axis.}
#'     \item{TRP}{Test Reference Profile. Shows latent classes/ranks on the horizontal axis,
#'       displaying members per class/rank as a bar graph and expected test scores as a line graph.}
#'     \item{LCD}{Latent Class Distribution. Displays latent classes on the horizontal axis,
#'       showing members per class as a bar graph and cumulative membership probability as a line.}
#'     \item{LRD}{Latent Rank Distribution. Similar to LCD but with ranks instead of classes on the horizontal axis.}
#'     \item{CMP}{Class Membership Profile. Line graph showing class membership probabilities of students.}
#'     \item{RMP}{Rank Membership Profile. Similar to CMP but with ranks instead of classes.}
#'     \item{ScoreFreq}{Frequency polygon of score distribution with rank thresholds.}
#'     \item{ScoreRank}{Heatmap of score membership probabilities for each rank.}
#'     \item{ICRP}{Visualizes ranks (x-axis) versus category response probabilities (y-axis).}
#'     \item{ICBR}{Visualizes ranks (x-axis) versus cumulative category probabilities (y-axis).}
#'     \item{FRP}{Field Reference Profile. Shows correspondence between fields and latent classes/ranks.}
#'     \item{Array}{Array plot for Biclustering/Ranklustering. Colored matrix cells where darker cells
#'       indicate larger values.}
#'     \item{FieldPIRP}{Shows correct response rates by number of correct answers in parent fields.
#'       Only available for LDB model.}
#'     \item{LDPSR}{Latent Dependence Passing Student Rate. Compares passing rates of parent and child classes.}
#'     \item{FCRP}{Field Category Response Profile. Category probability plot per field for polytomous Biclustering.
#'       Use \code{style} parameter to choose between "line" and "bar" display.}
#'     \item{FCBR}{Field Cumulative Boundary Reference. Boundary probability plot per field
#'       (ordinal Biclustering only).}
#'     \item{ScoreField}{Heatmap of expected scores across fields and latent classes/ranks
#'       for polytomous Biclustering.}
#'   }
#' @param items Numeric vector specifying which items to plot. If NULL, all items are included.
#'   When type is "IIF"/"IIC", specifying 0 will produce a TIF/TIC for the entire test.
#' @param students Numeric vector specifying which students to plot. If NULL, all students are included.
#' @param nc Integer specifying the number of columns for multiple plots. Default is 1.
#' @param nr Integer specifying the number of rows for multiple plots. Default is 1.
#' @param overlay Logical. If TRUE, elements such as IRFs will be overlaid on a single plot. Default is FALSE.
#' @param colors Character vector specifying custom color palette. If NULL, default colorblind-friendly palette is used.
#' For array plots, the first color should be for missing values, followed by response category colors.
#' @param cell_width Numeric value specifying the width of each cell in array plots. Default is 3.
#' @param cell_height Numeric value specifying the height of each cell in array plots. Default is 1.
#' @param filename Character string specifying output filename. If NULL, plot is displayed on screen.
#' Supported formats: png, pdf, jpeg, jpg. Format determined by file extension.
#' @param width Numeric value specifying plot width in pixels (for png/jpeg) or inches (for pdf). Default is 800.
#' @param height Numeric value specifying plot height in pixels (for png/jpeg) or inches (for pdf). Default is 600.
#' @param dpi Numeric value specifying resolution in dots per inch for raster formats (png/jpeg). Default is 300.
#' @param stat Character string specifying the summary statistic for polytomous FRP and RRV plots.
#' One of "mean" (default), "median", or "mode".
#' @param style Character string specifying the display style for FCRP plots.
#' One of "line" (default) or "bar" (stacked bar chart).
#' @param ... Additional graphical parameters passed to the underlying base R
#'   plotting functions. Standard parameters such as \code{pch}, \code{las},
#'   \code{cex}, \code{col}, \code{lty}, and \code{lwd} are forwarded
#'   consistently to all plot types (including manually drawn axes), so they
#'   can be used to customise point symbols, axis-label orientation, text
#'   sizes, and other graphical elements. User-supplied values take precedence
#'   over the package defaults (e.g. \code{xlab}, \code{ylab}, \code{main} can
#'   be overridden).
#'
#' @details
#' Each model class supports specific plot types:
#'
#' \describe{
#'   \item{IRT}{Supports "IRF"/"ICC", "TRF", "IIF"/"IIC", "TIF"/"TIC"}
#'   \item{GRM}{Supports "IRF"/"ICC", "IIF"/"IIC", "TIF"/"TIC"}
#'   \item{LCA}{Supports "IRP", "FRP", "TRP", "LCD", "CMP"}
#'   \item{LRA}{Supports "IRP", "FRP", "TRP", "LRD", "RMP"}
#'   \item{LRAordinal}{Supports "ScoreFreq", "ScoreRank", "ICRP", "ICBR", "RMP"}
#'   \item{LRArated}{Supports "ScoreFreq", "ScoreRank", "ICRP", "RMP"}
#'   \item{Biclustering}{Supports "FRP", "TRP", "LCD", "LRD", "CMP", "RMP", "CRV", "RRV", "Array"}
#'   \item{ordinalBiclustering}{Supports "FRP", "FCRP", "FCBR", "LCD", "LRD", "CMP", "RMP", "Array", "ScoreField", "RRV"}
#'   \item{nominalBiclustering}{Supports "FRP", "FCRP", "LCD", "LRD", "CMP", "Array", "ScoreField", "RRV"}
#'   \item{Biclustering_IRM}{Supports "FRP", "TRP", "Array"}
#'   \item{LDLRA}{Supports "IRP", "TRP", "LRD", "RMP"}
#'   \item{LDB}{Supports "FRP", "TRP", "LRD", "RMP", "Array", "FieldPIRP"}
#'   \item{BINET}{Supports "FRP", "TRP", "LRD", "RMP", "Array", "LDPSR"}
#' }
#'
#' @return
#' Produces visualizations based on the model class and specified type:
#'
#' \describe{
#'   \item{IRT models}{IRF (Item Response Function), TRF (Test Response Function),
#'     IIF (Item Information Function), TIF (Test Information Function)}
#'   \item{LCA/LRA models}{IRP (Item Reference Profile), TRP (Test Reference Profile),
#'     LCD/LRD (Latent Class/Rank Distribution), CMP/RMP (Class/Rank Membership Profile)}
#'   \item{Biclustering/Biclustering_IRM models}{Array plots showing clustering patterns, FRP, TRP, etc.}
#'   \item{LDLRA/LDB/BINET models}{Network and profile plots specific to each model}
#' }
#'
#' @importFrom graphics curve title axis barplot mtext par text lines rect legend abline image layout plot.new
#' @importFrom grDevices gray dev.off jpeg pdf png hcl.colors rainbow
#' @importFrom stats density runif
#' @importFrom utils tail
#'
#' @examples
#' \dontrun{
#' # IRT model example
#' irt_result <- exametrika::IRT(J15S500)
#' plot(irt_result, type = "IRF", items = 1:5)
#' plot(irt_result, type = "TIF")
#'
#' # LCA model example
#' lca_result <- exametrika::LCA(U)
#' plot(lca_result, type = "IRP")
#' plot(lca_result, type = "LCD")
#' }
#'
#' # Array plot with custom output
#' biclustering_result <- exametrika::Biclustering(J35S515)
#' # Custom colors and file output
#' my_colors <- c("#404040", "#E69F00", "#56B4E9", "#009E73", "#F0E442")
#' plot(biclustering_result, type = "Array", colors = my_colors)
#'
#' @export
#'
plot.exametrika <- function(x,
                            type = c(
                              "IRF", "TRF", "IIF", "TIF", "IIC", "ICC", "TIC",
                              "IRP", "TRP", "LCD", "CMP",
                              "FRP", "RMP", "LRD", "Array", "CRV", "RRV",
                              "FCRP", "FCBR", "ScoreField",
                              "FieldPIRP", "LDPSR",
                              "ScoreFreq", "ScoreRank", "ICRP", "ICBR"
                            ),
                            items = NULL,
                            students = NULL,
                            nc = 1,
                            nr = 1,
                            overlay = FALSE,
                            colors = NULL,
                            cell_width = 3,
                            cell_height = 1,
                            filename = NULL,
                            width = 800,
                            height = 600,
                            dpi = 300,
                            stat = "mean",
                            style = "line",
                            ...) {
  value <- if (length(class(x)) > 1) tail(class(x), 1) else "None"

  if (!is.null(filename)) {
    ext <- tolower(tools::file_ext(filename))
    if (ext == "") ext <- "png"
    switch(ext,
      "png" = png(filename, width = width, height = height, res = dpi),
      "pdf" = pdf(filename, width = width / 72, height = height / 72),
      "jpeg" = jpeg(filename, width = width, height = height, res = dpi),
      "jpg" = jpeg(filename, width = width, height = height, res = dpi),
      stop("Supported formats: png, pdf, jpeg, jpg")
    )
  }

  old_par <- par(no.readonly = TRUE)
  on.exit({
    # Safely restore parameters, excluding problematic read-only ones
    restore_par <- old_par
    restore_par[c("pin", "fin", "plt", "usr")] <- NULL
    suppressWarnings(par(restore_par))
    if (!is.null(filename)) dev.off()
  })

  # Skip mfrow for plot types that use layout() internally (FCRP, FCBR)
  uses_layout <- type %in% c("FCRP", "FCBR")
  if (!uses_layout) {
    par(mfrow = c(nr, nc))
  }
  testlength <- x$testlength
  nobs <- x$nobs

  valid_types <- list(
    IRT = c("IRF", "TRF", "IIF", "TIF", "IIC", "ICC", "TIC"),
    GRM = c("IRF", "IIF", "TIF", "IIC", "TIC"),
    LCA = c("IRP", "TRP", "LCD", "CMP"),
    LRA = c("IRP", "TRP", "LRD", "RMP"),
    LRAordinal = c("ScoreFreq", "ScoreRank", "ICRP", "ICBR", "RMP"),
    LRArated = c("ScoreFreq", "ScoreRank", "ICRP", "RMP"),
    Biclustering = c("FRP", "TRP", "LCD", "LRD", "CMP", "RMP", "CRV", "RRV", "Array"),
    nominalBiclustering = c("FRP", "FCRP", "LCD", "LRD", "CMP", "Array", "ScoreField", "RRV"),
    ratedBiclustering = c("FRP", "FCRP", "LCD", "LRD", "CMP", "RMP", "Array", "ScoreField", "RRV"),
    ordinalBiclustering = c("FRP", "FCRP", "FCBR", "LCD", "LRD", "CMP", "RMP", "Array", "ScoreField", "RRV"),
    IRM = c("FRP", "TRP", "Array"),
    LDLRA = c("IRP", "TRP", "LRD", "RMP"),
    LDB = c("FRP", "TRP", "LRD", "RMP", "Array", "FieldPIRP"),
    BINET = c("FRP", "TRP", "LRD", "RMP", "Array", "LDPSR")
  )

  if (missing(type)) {
    stop("The 'type' argument must be specified.")
  }

  if (!type %in% valid_types[[value]]) {
    stop(paste("Warning: The type", type, "does not correspond to the value", value))
  }

  plotItemID <- if (!is.null(items)) {
    if (!is.numeric(items) || length(items) > testlength || any(items < 0 | items > testlength)) {
      stop("'items' must be a numeric vector of length at most ", testlength, " and contain numbers between 0 and ", testlength)
    }
    items
  } else {
    1:testlength
  }

  plotStudentID <- if (!is.null(students)) {
    if (!is.numeric(students) || length(students) > nobs || any(students < 0 | students > nobs)) {
      stop("'students' must be a numeric vector of length at most ", nobs, " and contain numbers between 0 and ", nobs)
    }
    students
  } else {
    1:nobs
  }

  # Additional graphical parameters supplied via ... are forwarded to the
  # underlying base graphics calls in every model-specific plot function,
  # so standard parameters (pch, las, cex, col, lty, lwd, ...) work
  # consistently across all plot types.
  dots <- list(...)

  # Dispatch to model-specific plot functions
  switch(value,
    IRT = plot_irt_model(x, type, plotItemID, nc, nr, overlay, colors, dots),
    GRM = plot_grm_model(x, type, plotItemID, nc, nr, colors, dots),
    LCA = ,
    LRA = ,
    LDLRA = plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots),
    LRAordinal = ,
    LRArated = {
      if (type == "RMP") {
        plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots)
      } else {
        plot_lra_ordinal(x, type, plotItemID, dots)
      }
    },
    Biclustering = {
      if (type == "Array") {
        plot_array(x, cell_width, cell_height, colors, dots)
      } else {
        plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots)
      }
    },
    ordinalBiclustering = ,
    ratedBiclustering = ,
    nominalBiclustering = {
      if (type == "Array") {
        plot_array(x, cell_width, cell_height, colors, dots)
      } else if (type %in% c("LCD", "LRD", "CMP", "RMP")) {
        plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots)
      } else if (type == "FRP") {
        plot_poly_frp(x, stat, nc, nr, dots)
      } else if (type == "FCRP") {
        plot_poly_fcrp(x, style, nc, nr, dots)
      } else if (type == "FCBR") {
        plot_poly_fcbr(x, nc, nr, dots)
      } else if (type == "ScoreField") {
        plot_scorefield(x, dots)
      } else if (type == "RRV") {
        plot_poly_rrv(x, stat, dots)
      }
    },
    IRM = {
      if (type == "Array") {
        plot_array(x, cell_width, cell_height, colors, dots)
      } else {
        plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots)
      }
    },
    LDB = {
      if (type == "FieldPIRP") {
        plot_field_pirp(x, dots)
      } else if (type == "Array") {
        plot_array(x, cell_width, cell_height, colors, dots)
      } else {
        plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots)
      }
    },
    BINET = {
      if (type == "LDPSR") {
        plot_ldpsr(x, dots)
      } else if (type == "Array") {
        plot_array(x, cell_width, cell_height, colors, dots)
      } else {
        plot_common_profiles(x, type, value, plotItemID, plotStudentID, testlength, dots)
      }
    },
    none = {
      stop("This object cannot be plotted. The object must be of class 'exametrika' with a valid model type.")
    }
  )
}

#' @title Latent Class Analysis
#' @description
#' Performs Latent Class Analysis (LCA) on binary response data using the Expectation-Maximization (EM) algorithm.
#' LCA identifies unobserved (latent) subgroups of examinees with similar response patterns,
#' and estimates both the class characteristics and individual membership probabilities.
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#' it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param ncls Number of latent classes to identify (between 2 and 20). Default is 2.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#' observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @param maxiter Maximum number of EM algorithm iterations. Default is 100.
#' @param verbose Logical; if TRUE, displays progress during estimation. Default is TRUE.
#' @param beta1 Beta distribution parameter 1 for prior density of class reference matrix. Default is 1.
#' @param beta2 Beta distribution parameter 2 for prior density of class reference matrix. Default is 1.
#' @param conf Confirmatory IRP matrix (items x ncls) for test equating.
#'   Same format as the IRP output. Non-NA values are fixed throughout estimation,
#'   NA values are freely estimated. Fixed values must be in the open interval (0, 1).
#'   When row names are present, items are matched by label; otherwise by position.
#'   Default is NULL (fully exploratory).
#'
#' @return
#' An object of class "exametrika" and "LCA" containing:
#' \describe{
#'  \item{msg}{A character string indicating the model type. }
#'  \item{testlength}{Length of the test (number of items).}
#'  \item{nobs}{Sample size (number of rows in the dataset).}
#'  \item{Nclass}{Number of latent classes specified.}
#'  \item{N_Cycle}{Number of EM algorithm iterations performed.}
#'  \item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
#'  \item{TRP}{Test Reference Profile vector showing expected scores for each latent class.
#'    Calculated as the column sum of the estimated class reference matrix.}
#'  \item{LCD}{Latent Class Distribution vector showing the number of examinees assigned to each latent class.}
#'  \item{CMD}{Class Membership Distribution vector showing the sum of membership probabilities for each latent class.}
#'  \item{Students}{Class Membership Profile matrix showing the posterior probability of each examinee
#'    belonging to each latent class. The last column ("Estimate") indicates the most likely class assignment.}
#'  \item{IRP}{Item Reference Profile matrix where each row represents an item and each column
#'    represents a latent class. Values indicate the probability of a correct response
#'    for members of that class.}
#'  \item{ItemFitIndices}{Fit indices for each item. See also \code{\link{ItemFit}}.}
#'  \item{TestFitIndices}{Overall fit indices for the test. See also \code{\link{TestFit}}.}
#' }
#'
#' @details
#' Latent Class Analysis is a statistical method for identifying unobserved subgroups within
#' a population based on observed response patterns. It assumes that examinees belong to one
#' of several distinct latent classes, and that the probability of a correct response to each
#' item depends on class membership.
#'
#' The algorithm proceeds by:
#' 1. Initializing class reference probabilities
#' 2. Computing posterior class membership probabilities for each examinee (E-step)
#' 3. Re-estimating class reference probabilities based on these memberships (M-step)
#' 4. Iterating until convergence or reaching the maximum number of iterations
#'
#' Unlike Item Response Theory (IRT), LCA treats latent variables as categorical rather than
#' continuous, identifying distinct profiles rather than positions on a continuum.
#'
#' @references
#' Goodman, L. A. (1974). Exploratory latent structure analysis using both identifiable and
#' unidentifiable models. Biometrika, 61(2), 215-231.
#'
#' Lazarsfeld, P. F., & Henry, N. W. (1968). Latent structure analysis.
#' Boston: Houghton Mifflin.
#'
#' @examples
#' \donttest{
#' # Fit a Latent Class Analysis model with 5 classes to the sample dataset
#' result.LCA <- LCA(J15S500, ncls = 5)
#'
#' # Display the first few rows of student class membership probabilities
#' head(result.LCA$Students)
#'
#' # Plot Item Response Profiles (IRP) for items 1-6 in a 2x3 grid
#' # Shows probability of correct response for each item across classes
#' plot(result.LCA, type = "IRP", items = 1:6, nc = 2, nr = 3)
#'
#' # Plot Class Membership Probabilities (CMP) for students 1-9 in a 3x3 grid
#' # Shows probability distribution of class membership for each student
#' plot(result.LCA, type = "CMP", students = 1:9, nc = 3, nr = 3)
#'
#' # Plot Test Response Profile (TRP) showing expected scores for each class
#' plot(result.LCA, type = "TRP")
#'
#' # Plot Latent Class Distribution (LCD) showing class sizes
#' plot(result.LCA, type = "LCD")
#'
#' # Compare models with different numbers of classes
#' # (In practice, you might try more class counts)
#' lca2 <- LCA(J15S500, ncls = 2)
#' lca3 <- LCA(J15S500, ncls = 3)
#' lca4 <- LCA(J15S500, ncls = 4)
#' lca5 <- LCA(J15S500, ncls = 5)
#'
#' # Compare BIC values to select optimal number of classes
#' # (Lower BIC indicates better fit)
#' data.frame(
#'   Classes = 2:5,
#'   BIC = c(
#'     lca2$TestFitIndices$BIC,
#'     lca3$TestFitIndices$BIC,
#'     lca4$TestFitIndices$BIC,
#'     lca5$TestFitIndices$BIC
#'   )
#' )
#' }
#'
#' @export
LCA <- function(U, ncls = 2, na = NULL, Z = NULL, w = NULL, maxiter = 100,
                verbose = TRUE, beta1 = 1, beta2 = 1, conf = NULL) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (tmp$response.type != "binary") {
    response_type_error(tmp$response.type, "LCA")
  }

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  # Validate and align confirmatory IRP matrix
  if (!is.null(conf)) {
    conf <- validate_conf(conf, ncls, colnames(tmp$U))
  }

  fit <- emclus(tmp$U, tmp$Z, ncls,
    Fil = diag(rep(1, ncls)),
    beta1 = beta1, beta2 = beta2, maxiter = maxiter,
    mic = FALSE,
    verbose = verbose,
    conf = conf
  )

  ## Returns
  #### Class Information
  TRP <- fit$classRefMat %*% tmp$w
  bMax <- matrix(rep(apply(fit$postDist, 1, max), ncls), ncol = ncls)
  clsNum <- apply(fit$postDist, 1, which.max)
  cls01 <- sign(fit$postDist - bMax) + 1
  LCD <- colSums(cls01)
  CMD <- colSums(fit$postDist)
  StudentClass <- cbind(fit$postDist, clsNum)
  colnames(StudentClass) <- c(paste("Membership", 1:ncls), "Estimate")
  rownames(StudentClass) <- tmp$ID
  ### Item Information
  IRP <- t(fit$classRefMat)
  colnames(IRP) <- paste0("IRP", 1:ncls)

  ### Model Fit
  # each Items
  ell_A <- item_log_lik(tmp$U, tmp$Z, fit$postDist, fit$classRefMat)
  FitIndices <- ItemFit(tmp$U, tmp$Z, ell_A, ncls)

  ret <- structure(list(
    msg = "Class",
    testlength = testlength <- NCOL(tmp$U),
    nobs = NROW(tmp$U),
    n_class = ncls, # New naming convention
    n_cycle = fit$iter, # New naming convention
    converge = fit$converge,
    TRP = as.vector(TRP),
    LCD = as.vector(LCD),
    CMD = as.vector(CMD),
    Students = StudentClass,
    IRP = IRP,
    ItemFitIndices = FitIndices$item,
    TestFitIndices = FitIndices$test,
    log_lik = FitIndices$test$model_log_like,
    # Deprecated fields (for backward compatibility)
    Nclass = ncls,
    N_Cycle = fit$iter
  ), class = c("exametrika", "LCA"))
  return(ret)
}

library(exametrika)

### GOALS - Mathematica reference data
test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter05LCA_Test.csv"),
  check.names = FALSE
)
class <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter05LCA_Class.csv"),
  check.names = FALSE
)
items <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter05LCA_Item.csv"),
  check.names = FALSE
)
student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter05LCA_Student.csv"),
  check.names = FALSE
)


### Setup
tmp <- dataFormat(J15S500, na = -99)
model <- LCA(tmp, ncls = 5)

### Tests
test_that("LCA Test Info", {
  expect <- test[14:29, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- model$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("LCA Class Info", {
  ## TRP
  expect <- class[1, 2:6] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$TRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## LCD
  expect <- class[2, 2:6] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$LCD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## CMD
  expect <- class[3, 2:6] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$CMD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


test_that("LCA Item Info", {
  ## IRP
  expect <- items[, 6:10] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$IRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## FitIndex
  expect <- items[, c(15, 11, 12, 16, 13, 17, 14, 18:26)] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$ItemFitIndices |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("LCA Students", {
  ## Membership
  expect <- student[, 6:11] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$Students |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

#' @title IRP/FRP Profile Shape Indices (Kumagai, 2007)
#' @description
#' Computes six indices that quantitatively describe the shape of an
#' IRP (Item Reference Profile) or FRP (Field Reference Profile).
#' Used for both binary and polytomous Biclustering.
#'
#' ## Input matrix specification
#' - Rows: fields (or items), Columns: latent classes/ranks
#' - Cell values in \[0, 1\] (probabilities or normalized expected scores)
#'
#' ## Six indices
#'
#' ### Location parameters
#' - **Beta**: Class number whose value is closest to 0.5; the "center" of the profile.
#'   `Beta_f = argmin_c |FRP(f,c) - 0.5|`
#' - **B**: Profile value at the Beta position. B close to 0.5 indicates ideal location.
#'   `B_f = FRP(f, Beta_f)`
#'
#' ### Slope parameters
#' - **Alpha**: Class transition where the lag (adjacent-class difference) is largest;
#'   the steepest ascent point. `Alpha_f = argmax_c lag(f,c) - 1`
#'   (Alpha=1 means the steepest rise is from class 1 to class 2.)
#' - **A**: Maximum lag value; magnitude of the steepest slope.
#'   `A_f = max_c lag(f,c)` where `lag(f,c) = FRP(f,c) - FRP(f,c-1)`
#'
#' ### Monotonicity indices
#' - **C**: Sum of negative lags; amount of monotonicity violation.
#'   C = 0 means the profile is perfectly monotone increasing.
#'   `C_f = sum of lag(f,c)` (only terms where lag < 0)
#' - **Gamma**: Proportion of non-monotone transitions. Positive only when C != 0.
#'   `Gamma_f = (number of negative lags - 1) / (ncls - 1)`
#'
#' ## Relationship to Ordinal Alignment Conditions
#' - **SOAC** (Strongly Ordinal Alignment Condition): C = 0 for all fields
#'   and TRP is monotone increasing; all profiles are perfectly monotone.
#' - **WOAC** (Weakly Ordinal Alignment Condition): TRP is monotone increasing
#'   but some fields have C != 0; monotone overall with local reversals.
#'
#' ## Use with polytomous Biclustering
#' For polytomous data (ordinal Biclustering), the 3D category probability
#' array BCRM is converted to expected scores and normalized to \[0,1\].
#' See the FRPIndex computation in \code{Biclustering.ordinal} for details.
#'
#' @param IRP Numeric matrix. Rows = fields (or items), Columns = classes/ranks.
#'   Values in \[0,1\]. For binary Biclustering, pass correct-response probabilities;
#'   for polytomous Biclustering, pass normalized expected scores
#'   `(E\[score\] - 1) / (maxQ - 1)`.
#' @return data.frame with rows for fields and columns Alpha, A, Beta, B, Gamma, C.
#'
#' @references
#' Kumagai, K. (2007). IRP indices for understanding profile shapes.
#'
#' @noRd

IRPindex <- function(IRP) {
  # Beta: class position closest to 0.5
  Beta <- apply(abs(IRP - 0.5), 1, which.min)
  NR <- NROW(IRP)
  NC <- NCOL(IRP)
  # B: profile value at the Beta position
  B <- IRP[cbind(1:NR, Beta)]
  A <- Alpha <- rep(NA, NR)
  C <- Gamma <- rep(0, NR)
  for (i in 1:NR) {
    vec <- IRP[i, ]
    # lag: difference between adjacent classes (c minus c-1)
    lags <- vec - c(NA, vec[1:(NC - 1)])
    # A: maximum lag (steepest ascent)
    A[i] <- max(lags, na.rm = T)
    # Alpha: position of maximum lag (source class number)
    Alpha[i] <- which.max(lags) - 1
    # C: sum of negative lags (monotonicity violation; 0 = perfectly monotone)
    C[i] <- sum(lags[lags < 0], na.rm = T)
    if (C[i] != 0) {
      # Gamma: proportion of non-monotone transitions
      Gamma[i] <- (length(lags[lags < 0]) - 1) / (NC - 1)
    }
  }
  ret <- as.data.frame(cbind(Alpha, A, Beta, B, Gamma, C))
  return(ret)
}

# LCA / LRA / LRAordinal / LRArated plot functions
# Internal functions called from plot.exametrika()

#' Common profile plots (IRP / FRP / TRP / LCD / LRD / CMP / RMP / CRV / RRV)
#' @noRd
plot_common_profiles <- function(x, type, value, plotItemID, plotStudentID, testlength, dots = list()) {
  if (type == "IRP") {
    # Item Reference Profile ----------------------------------------
    msg <- x$msg
    params <- x$IRP[plotItemID, ]
    if (is.null(x$Nclass)) {
      steps <- x$Nrank
    } else {
      steps <- x$Nclass
    }
    for (i in 1:nrow(params)) {
      y <- params[i, ]
      call_plot(
        plot,
        list(
          x = y,
          type = "b",
          ylab = "Correct Response Rate",
          xlab = paste("Latent", msg),
          ylim = c(0, 1),
          xaxt = "n",
          main = paste("Item", i)
        ),
        dots
      )
      call_plot(graphics::axis, list(side = 1, at = 1:steps), dots)
    }
  }
  if (type == "FRP") {
    # Field Reference Profile ----------------------------------------
    params <- x$FRP
    msg <- x$msg
    for (i in 1:nrow(params)) {
      y <- params[i, ]
      call_plot(
        plot,
        list(
          x = y,
          type = "b",
          ylab = "Correct Response Rate",
          xlab = paste("Latent", msg),
          ylim = c(0, 1),
          main = paste("Field", i)
        ),
        dots
      )
    }
  }
  if (type == "CRV" | type == "RRV") {
    # Rank Reference Vector -------------------------------------------
    msg <- x$msg
    RRV <- t(x$FRP)
    call_plot(
      plot,
      list(
        x = 1:x$Nfield, y = RRV[1, ],
        type = "n",
        ylim = c(0, 1.1),
        xlab = "Field",
        ylab = "Correct Response Rate",
        main = paste(msg, "Reference Vector"),
        xaxt = "n", bty = "n"
      ),
      dots
    )
    call_plot(graphics::axis, list(side = 1, at = 1:x$Nfield, labels = colnames(RRV)), dots)
    for (i in 1:x$Nclass) {
      call_plot(graphics::lines, list(x = 1:x$Nfield, y = RRV[i, ], type = "o", lty = i), dots)
      for (j in 1:x$Nfield) {
        text(j, RRV[i, j], labels = i, pos = 3, offset = 0.5, cex = 0.8)
      }
    }
    legend("top",
      legend = rownames(RRV),
      lty = 1:x$Nclass,
      lwd = 2,
      ncol = x$Nclass,
      bty = "n"
    )
  }

  if (type == "TRP") {
    # Test Reference Profile ----------------------------------------
    old_par <- par(no.readonly = TRUE)
    on.exit(
      {
        restore_par <- old_par
        restore_par[c("pin", "fin", "plt", "usr")] <- NULL
        suppressWarnings(par(restore_par))
      },
      add = TRUE
    )
    par(mar = c(5, 4, 4, 4) + 0.1)
    if (value == "LCA" | value == "IRM" | value == "BINET") {
      target <- x$LCD
    } else if (value == "LRA" | value == "LDLRA" | value == "LDB") {
      target <- x$LRD
    } else if (value == "Biclustering" | value == "ordinalBiclustering" | value == "nominalBiclustering" |
      value == "ratedBiclustering") {
      target <- x$LRD
      if (is.null(target)) {
        target <- x$LCD
      }
    }
    msg <- x$msg

    if (is.null(x$Nclass)) {
      steps <- x$Nrank
    } else {
      steps <- x$Nclass
    }
    names.arg <- 1:steps

    bp <- call_plot(
      graphics::barplot,
      list(
        height = target,
        width = .9,
        ylim = c(0, max(target) + 10),
        xlim = c(0, steps + 1),
        xlab = paste("Latent", msg),
        ylab = "Number of Students"
      ),
      dots
    )
    text(x = bp, y = target, label = target, pos = 1, cex = 1.2)
    par(new = TRUE)
    call_plot(
      plot,
      list(
        x = bp, y = x$TRP,
        type = "b", pch = 19, lty = 1,
        axes = FALSE, xaxt = "n", xlab = "", ylab = "",
        bty = "n",
        ylim = c(0, testlength),
        xlim = c(0, steps + 1)
      ),
      dots
    )
    call_plot(graphics::axis, list(side = 4, at = pretty(range(0, testlength))), dots)
    mtext("Expected Score", side = 4, line = 3)
  }
  if (type == "LCD" | type == "LRD") {
    # Latent Class Distribution ----------------------------------------
    old_par <- par(no.readonly = TRUE)
    on.exit(
      {
        restore_par <- old_par
        restore_par[c("pin", "fin", "plt", "usr")] <- NULL
        suppressWarnings(par(restore_par))
      },
      add = TRUE
    )
    par(mar = c(5, 4, 4, 4) + 0.1)
    if (value == "LCA" | value == "BINET") {
      target1 <- x$LCD
      target2 <- x$CMD
    } else if (value == "Biclustering" | value == "ordinalBiclustering" | value == "nominalBiclustering" |
      value == "ratedBiclustering" | value == "LRA" | value == "LDLRA" | value == "LDB") {
      target1 <- x$LRD
      target2 <- x$RMD
    }
    msg <- x$msg
    if (is.null(x$Nclass)) {
      steps <- x$Nrank
    } else {
      steps <- x$Nclass
    }
    bp <- call_plot(
      graphics::barplot,
      list(
        height = target1,
        names.arg = 1:steps,
        width = .9,
        ylim = c(0, max(target1) + 10),
        xlim = c(0, steps + 1),
        xlab = paste("Latent", msg),
        ylab = "Number of Students"
      ),
      dots
    )
    text(x = bp, y = target1, label = target1, pos = 1, cex = 1.2)
    par(new = TRUE)
    call_plot(
      plot,
      list(
        x = bp, y = target2,
        type = "b", pch = 19, lty = 1,
        axes = FALSE, xaxt = "n", xlab = "", ylab = "",
        bty = "n",
        ylim = c(0, max(target1) + 10),
        xlim = c(0, steps + 1)
      ),
      dots
    )
    call_plot(graphics::axis, list(side = 4, at = pretty(range(0, max(target2) + 10))), dots)
    mtext("Frequency", side = 4, line = 3)
  }
  if (type == "CMP" | type == "RMP") {
    # Class Membership Profile ----------------------------------------
    msg <- x$msg
    if (is.null(x$Nclass)) {
      steps <- x$Nrank
    } else {
      steps <- x$Nclass
    }
    params <- x$Students[plotStudentID, 1:steps, drop = FALSE]
    for (i in 1:NROW(params)) {
      y <- params[i, ]
      call_plot(
        plot,
        list(
          x = y,
          type = "b",
          xlab = paste("Latent", msg),
          ylab = "Membership",
          ylim = c(0, 1),
          main = paste("Student", plotStudentID[i])
        ),
        dots
      )
    }
  }
}

#' LRA ordinal/rated plot dispatch
#' @noRd
plot_lra_ordinal <- function(x, type, plotItemID, dots = list()) {
  if (type == "ScoreFreq") {
    score_freq_plot(x, dots)
  } else if (type == "ScoreRank") {
    score_rank_plot(x, dots)
  } else if (type == "ICRP" || type == "ICBR") {
    IC_RP_BR_plot(x, type, plotItemID, dots)
  }
}

#' ScoreFreq: score distribution frequency polygon
#' @noRd
score_freq_plot <- function(x, dots = list()) {
  tmp <- as.data.frame(x$Students)
  sc <- tmp$Score
  rank <- tmp$Estimate
  thresholds <- numeric(length(unique(rank)) - 1)
  for (i in 1:(length(unique(rank)) - 1)) {
    max_rank_i <- max(sc[rank == i])
    min_rank_next <- min(sc[rank == (i + 1)])
    thresholds[i] <- (max_rank_i + min_rank_next) / 2
  }
  call_plot(
    plot,
    list(
      x = density(sc),
      xlab = "Score",
      ylab = "Frequency",
      main = "Latent Rank"
    ),
    dots
  )
  abline(
    v = thresholds,
    col = "red",
    lty = 2
  )
}

#' ScoreRank: score-membership probability heatmap
#' @noRd
score_rank_plot <- function(x, dots = list()) {
  score_rank_matrix <- x$ScoreRank
  call_plot(
    graphics::image,
    list(
      x = 1:ncol(score_rank_matrix),
      y = as.numeric(rownames(score_rank_matrix)),
      z = t(score_rank_matrix),
      col = gray(seq(1, 0, length.out = 100)),
      xlab = "Latent Rank",
      ylab = "Score",
      main = "Score-Rank Distribution",
      xaxt = "n"
    ),
    dots
  )
  call_plot(graphics::axis, list(side = 1, at = 1:ncol(score_rank_matrix)), dots)
}

#' ICRP / ICBR plot
#' @noRd
IC_RP_BR_plot <- function(x, type, plotItemID, dots = list()) {
  label <- x$U$ItemLabel[plotItemID]
  if (type == "ICRP") {
    tmp <- x$ICRP[x$ICRP$ItemLabel %in% label, ]
  } else if (type == "ICBR") {
    tmp <- x$ICBR[x$ICBR$ItemLabel %in% label, ]
  }
  yRange <- range(tmp[, -c(1:2)])
  for (i in 1:length(plotItemID)) {
    slice <- tmp[tmp$ItemLabel == label[i], ]
    slice <- unname(as.matrix(slice[, -c(1:2)]))
    call_plot(
      plot,
      list(
        x = slice[1, ],
        type = "l", lty = 1, col = 1, ylim = yRange,
        xlab = "Rank", ylab = "Probability", main = label[i],
        xaxt = "n"
      ),
      dots
    )
    call_plot(graphics::axis, list(side = 1, at = 1:ncol(slice), labels = 1:x$Nrank), dots)
    for (j in 1:nrow(slice)) {
      call_plot(graphics::lines, list(x = slice[j, ], lty = j), dots)
      text(
        x = ncol(slice), y = slice[j, ncol(slice)],
        labels = j, cex = 0.8, ylim = yRange
      )
    }
  }
}

#' @title Simple Test Statistics
#' @description
#' Calculates descriptive statistics for test scores, providing a comprehensive
#' summary of central tendency, variability, and distribution shape.
#' Different statistics are calculated based on the data type (binary, ordinal, rated, or nominal).
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#'
#' @return
#' The returned object depends on the data type:
#'
#' For binary data, a list of class c("exametrika", "TestStatistics") containing:
#' \describe{
#' \item{TestLength}{Length of the test. The number of items included in the test.}
#' \item{SampleSize}{Sample size. The number of rows in the dataset.}
#' \item{Mean}{Average number of correct answers.}
#' \item{SEofMean}{Standard error of mean.}
#' \item{Variance}{Variance of test scores.}
#' \item{SD}{Standard Deviation of test scores.}
#' \item{Skewness}{Skewness of score distribution (measure of asymmetry).}
#' \item{Kurtosis}{Kurtosis of score distribution (measure of tail extremity).}
#' \item{Min}{Minimum score.}
#' \item{Max}{Maximum score.}
#' \item{Range}{Range of scores (Max - Min).}
#' \item{Q1}{First quartile. Same as the 25th percentile.}
#' \item{Median}{Median. Same as the 50th percentile.}
#' \item{Q3}{Third quartile. Same as the 75th percentile.}
#' \item{IQR}{Interquartile range. Calculated by subtracting Q1 from Q3.}
#' \item{Stanine}{Stanine score boundaries, see \code{\link{stanine}}.}
#' }
#'
#' For ordinal and rated data, the function calls \code{\link{ScoreReport}} and returns
#' its result. See \code{\link{ScoreReport}} for details of the returned object.
#'
#' For nominal data, an error is returned as this function does not support nominal data.
#'
#' @importFrom stats sd var quantile
#' @examples
#' # Basic usage
#' stats <- TestStatistics(J15S500)
#' print(stats)
#'
#' # Extract specific statistics
#' cat("Mean score:", stats$Mean, "\n")
#' cat("Standard deviation:", stats$SD, "\n")
#'
#' # View score distribution summary
#' summary_stats <- data.frame(
#'   Min = stats$Min,
#'   Q1 = stats$Q1,
#'   Median = stats$Median,
#'   Mean = stats$Mean,
#'   Q3 = stats$Q3,
#'   Max = stats$Max
#' )
#' print(summary_stats)
#'
#' # Stem-and-leaf plot of score distribution
#' stem(nrs(dataFormat(J15S500)))
#' @export
#'
TestStatistics <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("TestStatistics")
}

#' @rdname TestStatistics
#' @export
TestStatistics.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = TestStatistics.binary(U, na, Z, w),
      "rated" = TestStatistics.ordinal(U, na, Z, w),
      "ordinal" = TestStatistics.ordinal(U, na, Z, w),
      "nominal" = response_type_error(U$response.type, "TestStatistics")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    TestStatistics(U)
  }
}

#' @rdname TestStatistics
#' @export
TestStatistics.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  tW <- nrs(tmp)
  TestLength <- NCOL(tmp$Z)
  SampleSize <- NROW(tmp$Z)
  Mean <- mean(tW)
  SEofMean <- sd(tW) / sqrt(SampleSize)
  Variance <- var(tW)
  SD <- sd(tW)
  SDs <- sqrt(mean((tW - Mean)^2))
  tmpZ <- (tW - Mean) / SDs
  Skewness <- mean(tmpZ^3)
  Kurtosis <- mean(tmpZ^4) - 3
  Min <- min(tW)
  Max <- max(tW)
  Range <- Max - Min
  Q1 <- quantile(tW, probs = 0.25, na.rm = TRUE)
  Median <- quantile(tW, probs = 0.5, na.rm = TRUE)
  Q3 <- quantile(tW, probs = 0.75, na.rm = TRUE)
  IQR <- Q3 - Q1
  names(IQR) <- ""
  Stanine <- stanine(tmp)
  ret <-
    structure(list(
      TestLength = TestLength,
      SampleSize = SampleSize,
      Mean = Mean,
      SEofMean = SEofMean,
      Variance = Variance,
      SD = SD,
      Skewness = Skewness,
      Kurtosis = Kurtosis,
      Min = Min,
      Max = Max,
      Range = Range,
      Q1 = Q1,
      Median = Median,
      Q3 = Q3,
      IQR = IQR,
      Stanine = Stanine$stanine
    ), class = c("exametrika", "TestStatistics"))
  return(ret)
}

#' @rdname TestStatistics
#' @export
TestStatistics.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  # ordinal / rated common functions
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  ScoreReport(tmp)
}

#' @title Inter-Item Analysis for Psychometric Data
#' @description
#' Calculates various relationship metrics between pairs of items in test data. This analysis
#' helps identify item interdependencies, content overlaps, and potential local dependence.
#' For binary data, metrics include joint response rates, conditional probabilities,
#' and several correlation measures. For ordinal/rated data, appropriate correlation
#' measures are calculated.
#'
#' The following metrics are calculated for binary data:
#' \itemize{
#'   \item JSS: Joint Sample Size - number of examinees responding to both items
#'   \item JCRR: Joint Correct Response Rate - proportion of examinees answering both items correctly
#'   \item CCRR: Conditional Correct Response Rate - probability of answering one item correctly
#'     given a correct response to another item
#'   \item IL: Item Lift - ratio of joint correct response rate to the product of marginal rates
#'   \item MI: Mutual Information - measure of mutual dependence between items
#'   \item Phi: Phi Coefficient - correlation coefficient for binary variables
#'   \item Tetrachoric: Tetrachoric Correlation - estimate of Pearson correlation for underlying
#'     continuous variables
#' }
#'
#' For ordinal/rated data, the function calculates:
#' \itemize{
#'   \item JSS: Joint Sample Size
#'   \item JSR: Joint Selection Rate
#'   \item CSR: Conditional Selection Rate
#'   \item MI: Mutual Information
#'   \item Polychoric: Polychoric Correlation - extension of tetrachoric correlation for ordinal data
#' }
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#'
#' @return
#' For binary data, an object of class "exametrika" and "IIAnalysis" containing:
#' \describe{
#'   \item{JSS}{Joint Sample Size matrix - N(i,j) shows number of examinees who responded to both items i and j}
#'   \item{JCRR}{Joint Correct Response Rate matrix - P(Xi=1, Xj=1) shows probability of correct responses to both items}
#'   \item{CCRR}{Conditional Correct Response Rate matrix - P(Xi=1|Xj=1) shows probability of correct response to item i
#'     given correct response to item j}
#'   \item{IL}{Item Lift matrix - P(Xi=1, Xj=1)/(P(Xi=1)*P(Xj=1)) measures association strength}
#'   \item{MI}{Mutual Information matrix - measures information shared between items}
#'   \item{Phi}{Phi Coefficient matrix - correlation coefficient between binary variables}
#'   \item{Tetrachoric}{Tetrachoric Correlation matrix - correlation between underlying continuous variables}
#' }
#'
#' For ordinal/rated data, an object of class "exametrika" and "IIAnalysis.ordinal" containing:
#' \describe{
#'   \item{JSS}{Joint Sample Size matrix}
#'   \item{JSR}{Joint Selection Rate matrix - frequencies of joint category selections}
#'   \item{CSR}{Conditional Selection Rate matrix - probabilities of response categories conditional on other items}
#'   \item{MI}{Mutual Information matrix}
#'   \item{Polychoric}{Polychoric Correlation matrix - correlations between underlying continuous variables}
#' }
#'
#' @details
#' This function automatically detects the data type and applies appropriate analysis methods:
#' \itemize{
#'   \item For binary data: Calculates tetrachoric correlations and related statistics
#'   \item For ordinal/rated data: Calculates polychoric correlations and related statistics
#'   \item For nominal data: Returns an error (not supported)
#' }
#'
#' Inter-item analysis is useful for:
#' \itemize{
#'   \item Identifying groups of highly related items
#'   \item Detecting local dependence between items
#'   \item Evaluating test dimensionality
#'   \item Informing item selection and test construction
#' }
#'
#' @examples
#' \donttest{
#' # Basic usage with binary data
#' ii_analysis <- InterItemAnalysis(J15S500)
#'
#' # View joint sample sizes
#' head(ii_analysis$JSS)
#'
#' # View tetrachoric correlations
#' head(ii_analysis$Tetrachoric)
#'
#' # Find pairs of items with high mutual information (potential local dependence)
#' high_MI <- which(ii_analysis$MI > 0.2 & upper.tri(ii_analysis$MI), arr.ind = TRUE)
#' if (nrow(high_MI) > 0) {
#'   print("Item pairs with high mutual information:")
#'   print(high_MI)
#' }
#'
#' # Example with ordinal data
#' ordinal_analysis <- InterItemAnalysis(J15S3810)
#'
#' # View polychoric correlations for ordinal data
#' head(ordinal_analysis$Polychoric)
#' }
#'
#' @seealso \code{\link{dataFormat}} for data preparation, \code{\link{CTT}} for
#' Classical Test Theory analysis
#'
#' @export

InterItemAnalysis <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("InterItemAnalysis")
}

#' @export
InterItemAnalysis.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = InterItemAnalysis.binary(U, na, Z, w),
      "rated" = InterItemAnalysis.ordinal(U, na, Z, w),
      "ordinal" = InterItemAnalysis.ordinal(U, na, Z, w),
      "nominal" = response_type_error(U$response.type, "InterItemAnalysis")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    InterItemAnalysis(U)
  }
}
#' @export
InterItemAnalysis.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate all matrices
  JSS <- JointSampleSize(U)
  JCRR <- JCRR(U)
  CCRR <- CCRR(U)
  IL <- ItemLift(U)
  MI <- MutualInformation(U)
  Phi <- PhiCoefficient(U)
  Tet <- TetrachoricCorrelationMatrix(U)

  # Create return structure
  structure(
    list(
      JSS = JSS,
      JCRR = JCRR,
      CCRR = CCRR,
      IL = IL,
      MI = MI,
      Phi = Phi,
      Tetrachoric = Tet
    ),
    class = c("exametrika", "IIAnalysis")
  )
}

#' @export
InterItemAnalysis.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  ret <- structure(
    list(
      JSS = JointSampleSize(tmp),
      JSR = JSR(tmp),
      CSR = CSR(tmp),
      MI = MutualInformation(tmp),
      Polychoric = PolychoricCorrelationMatrix(tmp)
    ),
    class = c("exametrika", "IIAnalysis.ordinal")
  )
  return(ret)
}

#' @title LDparam set
#' @description
#' A function that extracts only the estimation of graph parameters
#' after the rank estimation is completed.
#' @param tmp tmp
#' @param adj_list adj_list
#' @param classRefMat values returned from emclus
#' @param ncls ncls
#' @param smoothpost smoothpost
#' @param beta1 Beta distribution parameter 1 for prior density. Default is 2.
#' @param beta2 Beta distribution parameter 2 for prior density. Default is 2.
#'
LD_param_est <- function(tmp, adj_list, classRefMat, ncls, smoothpost, beta1 = 2, beta2 = 2) {
  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  maxnpa <- 4
  parent <- list()
  nparent <- array(NA, dim = c(testlength, ncls))
  for (i in 1:ncls) {
    adj <- adj_list[[i]]
    npa <- colSums(adj)
    parent[[i]] <- list()
    for (j in 1:testlength) {
      nparent[j, i] <- npa[j]
      if (npa[j] == 0) {
        parent[[i]][[j]] <- ""
      } else {
        parent[[i]][[j]] <- tmp$ItemLabel[adj[, j] == 1]
      }
    }
  }

  if (max(nparent) > maxnpa) {
    warning("[Caution!] The maximum number of parents per item is ", maxnpa, ". Please check.")
  }

  # PIRP  --------------------------------------------------------------
  npapat <- 2^max(nparent)
  #### JxRxPattern
  refmat <- array(NA, dim = c(testlength, ncls, npapat))
  for (i in 1:npapat) {
    refmat[, , i] <- classRefMat
  }
  refmat <- array(replicate(npapat, classRefMat), dim = c(testlength, ncls, npapat))

  ### SxJxRxPattern
  pat01 <- array(0, dim = c(nobs, testlength, ncls, npapat))
  for (s in 1:nobs) {
    for (j in 1:testlength) {
      for (cls in 1:ncls) {
        if (parent[[cls]][[j]][[1]] == "") {
          # No parents
          pos <- 1
        } else {
          pos.tmp <- tmp$U[s, parent[[cls]][[j]]]
          pos <- sum(2^(which(rev(pos.tmp) == 1) - 1)) + 1
        }
        pat01[s, j, cls, pos] <- 1
      }
    }
  }

  # Estimation --------------------------------------------------------------
  const <- exp(-testlength)

  n_correct <- array(0, dim = c(nobs, testlength, ncls, npapat))
  n_incorrect <- array(0, dim = c(nobs, testlength, ncls, npapat))

  # Expand dimensions for vectorized operations
  U_expanded <- array(replicate(ncls * npapat, tmp$U), dim = dim(pat01))
  Z_expanded <- array(replicate(ncls * npapat, tmp$Z), dim = dim(pat01))

  U_ezpanded <- array(rep(tmp$U, times = ncls * npapat), dim = c(nobs, testlength, ncls, npapat))
  Z_ezpanded <- array(rep(tmp$Z, times = ncls * npapat), dim = c(nobs, testlength, ncls, npapat))

  # Expand smoothpost
  # Using aperm and array to expand smoothpost
  smoothpost_expanded <- aperm(
    array(rep(smoothpost, times = testlength * npapat),
      dim = c(nobs, ncls, testlength, npapat)
    ),
    perm = c(1, 3, 2, 4)
  )
  # Vectorized calculations
  n_correct <- colSums(U_expanded * pat01 * smoothpost_expanded)
  n_incorrect <- colSums(Z_expanded * (1 - U_expanded) * pat01 * smoothpost_expanded)

  refmat <- (n_correct + beta1 - 1) / (n_correct + n_incorrect + beta1 + beta2 - 2)
  item_ell <- n_correct * log(refmat + const) + n_incorrect * log(1 - refmat + const)
  item_ell <- rowSums(apply(item_ell, 1:2, sum))
  test_ell <- sum(item_ell)

  refmat_expanded <- array(rep(refmat, nobs), dim = c(testlength, ncls, npapat, nobs))
  term1 <- aperm(U_expanded * pat01, perm = c(2, 3, 4, 1)) * log(refmat_expanded + const)
  term2 <- aperm(Z_expanded * (1 - U_expanded) * pat01, perm = c(2, 3, 4, 1)) * log(1 - refmat_expanded + const)
  llmat <- aperm(term1 + term2, perm = c(1, 3, 4, 2))
  llmat <- apply(llmat, 3:4, sum)
  exp_llmat <- exp(llmat)
  postdist <- exp_llmat / rowSums(exp_llmat)

  postdist_expand <- array(rep(postdist, testlength * npapat), dim = c(nobs, ncls, testlength, npapat))
  irp_tmp <- postdist_expand * aperm(pat01, perm = c(1, 3, 2, 4))
  irp_tmp <- apply(irp_tmp, 2:4, sum) / colSums(postdist)
  irp_tmp <- aperm(irp_tmp, perm = c(2, 1, 3))
  irp_tmp <- irp_tmp * refmat
  irp <- apply(irp_tmp, 1:2, sum)

  # Model Fit -------------------------------------------------------

  model_nparam <- sum(apply(sign(n_correct + n_incorrect), 1:2, sum))
  FitIndices <- TestFit(tmp$U, tmp$Z, test_ell, model_nparam)

  return(list(
    irp = irp,
    parent = parent,
    FitIndices = FitIndices,
    npapat = npapat,
    postdist = postdist,
    refmat = refmat
  ))
}

#' @title Local Dependence Latent Rank Analysis
#' @description
#' performs local dependence latent lank analysis(LD_LRA) by Shojima(2011)
#' @details
#' This function is intended to perform LD-LRA. LD-LRA is an analysis that
#' combines LRA and BNM, and it is used to analyze the network structure among
#' items in the latent rank. In this function, structural learning is not
#' performed, so you need to provide item graphs for each rank as separate files.
#' The file format for this is plain text CSV that includes edges (From, To) and
#' rank numbers.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param ncls number of latent class(rank). The default is 2.
#' @param method specify the model to analyze the data.Local dependence latent
#' class model is set to "C", latent rank model is set "R". The default is "R".
#' @param g_list A list compiling graph-type objects for each rank/class.
#' @param adj_list A list compiling matrix-type adjacency matrices for each rank/class.
#' @param adj_file A file detailing the relationships of the graph for each rank/class,
#' listed in the order of starting point, ending point, and rank(class).
#' @param verbose verbose output Flag. default is TRUE
#' @param beta1 Beta distribution parameter 1 for prior density of rank reference matrix. Default is 2.
#' @param beta2 Beta distribution parameter 2 for prior density of rank reference matrix. Default is 2.
#' @importFrom igraph as_adjacency_matrix
#' @importFrom igraph graph_from_data_frame
#' @importFrom igraph graph_from_adjacency_matrix
#' @importFrom utils read.csv
#' @importFrom igraph V
#' @return
#' \describe{
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{msg}{A character string indicating the model type. }
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{crr}{correct response ratio}
#'  \item{adj_list}{adjacency matrix list}
#'  \item{g_list}{graph list}
#'  \item{referenceMatrix}{Learned Parameters.A three-dimensional array of patterns where
#'  item x rank x pattern.}
#'  \item{IRP}{Marginal Item Reference Matrix}
#'  \item{IRPIndex}{IRP Indices which include Alpha, Beta, Gamma.}
#'  \item{TRP}{Test Reference Profile matrix.}
#'  \item{LRD}{latent Rank/Class Distribution}
#'  \item{RMD}{Rank/Class Membership Distribution}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{Estimation_table}{Estimated parameters tables.}
#'  \item{CCRR_table}{Correct Response Rate tables}
#'  \item{Studens}{Student information. It includes estimated class
#'  membership, probability of class membership, RUO, and RDO.}
#' }
#' @examples
#' \donttest{
#' # Create sample DAG structure with different rank levels
#' # Format: From, To, Rank
#' DAG_dat <- matrix(c(
#'   "From", "To", "Rank",
#'   "Item01", "Item02", "1", # Simple structure for Rank 1
#'   "Item01", "Item02", "2", # More complex structure for Rank 2
#'   "Item02", "Item03", "2",
#'   "Item01", "Item02", "3", # Additional connections for Rank 3
#'   "Item02", "Item03", "3",
#'   "Item03", "Item04", "3"
#' ), ncol = 3, byrow = TRUE)
#'
#' # Method 1: Directly use graph and adjacency lists
#' g_list <- list()
#' adj_list <- list()
#'
#' for (i in 1:3) {
#'   adj_R <- DAG_dat[DAG_dat[, 3] == as.character(i), 1:2, drop = FALSE]
#'   g_tmp <- igraph::graph_from_data_frame(
#'     d = data.frame(
#'       From = adj_R[, 1],
#'       To = adj_R[, 2]
#'     ),
#'     directed = TRUE
#'   )
#'   adj_tmp <- igraph::as_adjacency_matrix(g_tmp)
#'   g_list[[i]] <- g_tmp
#'   adj_list[[i]] <- adj_tmp
#' }
#'
#' # Fit Local Dependence Latent Rank Analysis
#' result.LDLRA1 <- LDLRA(J12S5000,
#'   ncls = 3,
#'   g_list = g_list,
#'   adj_list = adj_list
#' )
#'
#' # Plot Item Reference Profiles (IRP) in a 4x3 grid
#' # Shows the probability patterns of correct responses for each item across ranks
#' plot(result.LDLRA1, type = "IRP", nc = 4, nr = 3)
#'
#' # Plot Test Reference Profile (TRP)
#' # Displays the overall pattern of correct response probabilities across ranks
#' plot(result.LDLRA1, type = "TRP")
#'
#' # Plot Latent Rank Distribution (LRD)
#' # Shows the distribution of students across different ranks
#' plot(result.LDLRA1, type = "LRD")
#' }
#'
#' @export
#'

LDLRA <- function(U, Z = NULL, w = NULL, na = NULL,
                  ncls = 2, method = "R",
                  g_list = NULL, adj_list = NULL, adj_file = NULL,
                  verbose = FALSE, beta1 = 2, beta2 = 2) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }


  if (U$response.type != "binary") {
    response_type_error(U$response.type, "LDLRA")
  }


  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  if (method == "C" | method == "Class") {
    if (verbose) {
      message("local dependence latent Class model is chosen.")
    }
    model <- 1
  } else if (method == "R" | method == "Rank") {
    if (verbose) {
      message("local dependence latent Rank model is chosen.")
    }
    model <- 2
  } else {
    stop("The method must be selected as either LD-LCA or LD-LRA.")
  }

  # graph check
  if (is.null(g_list) && is.null(adj_list) && is.null(adj_file)) {
    stop("Specify the graph in either matrix form, CSV file, or as a graph object.")
  }
  # g_list check
  if (!is.null(g_list)) {
    if (length(g_list) != ncls) {
      stop("The number of classes does not match the length of the list.
           Please specify a graph for all classes.")
    }
    adj_list <- list()
    for (j in 1:ncls) {
      if (!inherits(g_list[[1]], "igraph")) {
        stop("Some items in g_list are not recognized as graph objects.")
      }
      adj_list[[j]] <- fill_adj(g_list[[j]], tmp$ItemLabel)
    }
  }
  # adj_list check
  if (!is.null(adj_list)) {
    if (length(adj_list) != ncls) {
      stop("The number of classes does not match the length of the list.
           Please specify a graph for all classes.")
    }
    for (j in 1:ncls) {
      g <- igraph::graph_from_adjacency_matrix(adj_list[[j]])
      adj_list[[j]] <- fill_adj(g, tmp$ItemLabel)
    }
  }
  # adj_file check
  if (!is.null(adj_file)) {
    g_csv <- read.csv(adj_file)
    colnames(g_csv) <- c("From", "To", "Rank")
    adj_list <- list()
    for (i in 1:ncls) {
      adj_R <- g_csv[g_csv$Rank == i, 1:2]
      g_tmp <- igraph::graph_from_data_frame(adj_R)
      adj_list[[i]] <- fill_adj(g_tmp, tmp$ItemLabel)
    }
  }

  # Set filter --------------------------------------------------------------
  if (model == 1) {
    filmat <- diag(1, ncol = ncls, nrow = ncls)
  } else {
    fil0 <- (1 / 3 - 0.4) / 5 * (ncls - 5) + 0.4
    fil1 <- (1 / 3 - 0.2) / 5 * (ncls - 5) + 0.2
    fil2 <- (1 - fil0 - 2 * fil1) / 2
    filmat0 <- diag(fil0, ncls)
    filmat1 <- diag(fil1, (ncls + 1))[2:(ncls + 1), 1:ncls]
    filmat2 <- diag(fil2, (ncls + 2))[3:(ncls + 2), 1:ncls]
    filmat <- filmat0 + filmat1 + filmat2 + t(filmat1) + t(filmat2)
    filmat <- filmat / (rep(1, ncls) %*% t(rep(1, ncls)) %*% filmat)
  }

  ret.emclus <- emclus(tmp$U, tmp$Z, ncls, Fil = filmat, beta1 = beta1, beta2 = beta2)
  smoothpost <- ret.emclus$postDist %*% filmat
  ret.LDparam <- LD_param_est(tmp, adj_list, ret.emclus$classRefMat, ncls, smoothpost, beta1, beta2)
  irp <- ret.LDparam$irp
  npapat <- ret.LDparam$npapat
  FitIndices <- ret.LDparam$FitIndices
  postdist <- ret.LDparam$postdist
  refmat <- ret.LDparam$refmat
  parent <- ret.LDparam$parent

  # output ----------------------------------------------------------

  g_list <- list()
  for (i in 1:ncls) {
    g_list[[i]] <- igraph::graph_from_adjacency_matrix(adj_list[[i]])
  }

  ### refmat; Item x Rank x Pattern
  Estimation_table <- as.data.frame(matrix(NA, nrow = testlength * ncls, ncol = npapat + 2))
  model <- 2
  if (model == 1) {
    model_msg <- "Class"
  } else {
    model_msg <- "Rank"
  }

  colnames(Estimation_table) <- c("Item", model_msg, paste("PIRP", 1:npapat))
  Estimation_table[, 1] <- rep(tmp$ItemLabel, ncls)
  Estimation_table[, 2] <- rep(1:ncls, each = testlength)

  parent_vec <- unlist(lapply(adj_list, colSums))
  CCRR_table <- as.data.frame(matrix(NA, nrow = sum(2^parent_vec), ncol = 6))
  colnames(CCRR_table) <- c("Child Item", model_msg, "N of Parents", "Parent Items", "PIRP", "Conditional CRR")
  CCRR_table[, 1] <- rep(rep(tmp$ItemLabel, ncls), times = 2^parent_vec)
  CCRR_table[, 2] <- rep(rep(1:ncls, each = testlength), times = 2^parent_vec)
  CCRR_table[, 3] <- rep(parent_vec, times = 2^parent_vec)
  parent_items <- character(sum(2^parent_vec))
  CRR_vec <- numeric(sum(2^parent_vec))

  l1 <- 0
  l2 <- 0
  l3 <- 0
  for (i in 1:ncls) {
    for (j in 1:testlength) {
      l1 <- l1 + 1
      if (length(parent[[i]][[j]]) == 1 && parent[[i]][[j]] == "") {
        max_k <- 1
        l2 <- l2 + 1
        parent_items[l2] <- "No Parents"
      } else {
        max_k <- 2^length(parent[[i]][[j]])
        word <- paste(unlist(parent[[i]][[j]]), collapse = ", ")
        rep_times <- 2^length(parent[[i]][[j]])
        for (k in 1:rep_times) {
          l2 <- l2 + 1
          parent_items[l2] <- word
        }
      }
      for (k in 1:max_k) {
        l3 <- l3 + 1
        Estimation_table[l1, k + 2] <- refmat[j, i, k]
        CRR_vec[l3] <- refmat[j, i, k]
      }
    }
  }

  CCRR_table[, 4] <- parent_items
  CCRR_table[, 5] <- unlist(sapply(parent_vec, BitRespPtn))
  CCRR_table[, 6] <- CRR_vec

  # IRP index -------------------------------------------------------

  rownames(irp) <- tmp$ItemLabel
  colnames(irp) <- paste(model_msg, 1:ncls)
  IRPIndex <- IRPindex(irp)
  if (sum(IRPIndex$C) == 0) {
    SOACflg <- TRUE
  } else {
    SOACflg <- FALSE
  }

  # TRP -------------------------------------------------------------
  TRP <- colSums(irp)
  dif <- diff(TRP)
  WOACsum <- sum(dif > 0) + 1
  if (WOACsum == ncls) {
    WOACflg <- TRUE
  } else {
    WOACflg <- FALSE
  }

  # Student information ---------------------------------------------
  clsNum <- apply(postdist, 1, which.max)
  bMax <- matrix(rep(apply(postdist, 1, max), ncls), ncol = ncls)
  cls01 <- sign(postdist - bMax) + 1
  LRD <- colSums(cls01)
  RMD <- colSums(postdist)
  StudentClass <- postdist
  RU <- ifelse(clsNum + 1 > ncls, NA, clsNum + 1)
  RD <- ifelse(clsNum - 1 < 1, NA, clsNum - 1)
  RUO <- StudentClass[cbind(1:nobs, RU)] / StudentClass[cbind(1:nobs, clsNum)]
  RDO <- StudentClass[cbind(1:nobs, RD)] / StudentClass[cbind(1:nobs, clsNum)]
  StudentClass <- cbind(StudentClass, clsNum, RUO, RDO)
  colnames(StudentClass) <- c(
    paste("Membership", 1:ncls), "Estimate",
    "Rank-Up Odds", "Rank-Down Odds"
  )
  rownames(StudentClass) <- tmp$ID

  ret <- structure(list(
    U = U,
    testlength = testlength,
    msg = "Rank",
    nobs = nobs,
    n_class = ncls,
    Nclass = ncls,
    model = model,
    crr = crr(U),
    ItemLabel = tmp$ItemLabel,
    adj_list = adj_list,
    g_list = g_list,
    referenceMatrix = refmat,
    IRP = irp,
    IRPIndex = IRPIndex,
    TRP = as.vector(TRP),
    LRD = as.vector(LRD),
    RMD = as.vector(RMD),
    Students = StudentClass,
    TestFitIndices = FitIndices,
    log_lik = FitIndices$model_log_like,
    Estimation_table = Estimation_table,
    CCRR_table = CCRR_table,
    SOACflg = SOACflg,
    WOACflg = WOACflg
  ), class = c("exametrika", "LDLRA"))
  return(ret)
}

library(exametrika)

### GOALS - Mathematica reference data
Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter09LDLRA_Test.csv"),
  check.names = FALSE
)
Rank <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter09LDLRA_Rank.csv"),
  check.names = FALSE
)
Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter09LDLRA_Item.csv"),
  check.names = FALSE
)
CCRR <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter09LDLRA_CCRR.csv"),
  check.names = FALSE
)
Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter09LDLRA_Student.csv"),
  check.names = FALSE
)

### Target
tgt <- LDLRA(J12S5000,
  ncls = 5,
  adj_file = test_path("fixtures", "auxiliary_data", "DAG_file.csv")
)

# Test ------------------------------------------------------------

test_that("Test Info", {
  expect <- Test[14:29, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- tgt$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Rank ------------------------------------------------------------


test_that("Rank Info", {
  expect <- Rank[1, 2:6] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$TRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- Rank[2, 2:6] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$LRD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- Rank[3, 2:6] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$RMD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Item ------------------------------------------------------------
test_that("Item Info", {
  R1 <- Item[, 6:9]
  R2 <- Item[, 10:13]
  R3 <- Item[, 14:17]
  R4 <- Item[, 18:21]
  R5 <- Item[, 22:25]
  colnames(R1) <- colnames(R2) <- colnames(R3) <- colnames(R4) <- colnames(R5)
  expect <- rbind(R1, R2, R3, R4, R5) |>
    unlist() |>
    as.vector()
  result <- tgt$Estimation_table[, 3:6] |>
    unlist() |>
    as.vector()
  colnames(expect) <- colnames(result)
  expect_equal(result, expect, tolerance = 1e-4)

  expect <- Item[, 26:30] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$IRP |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- Item[, 31:36] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$IRPIndex |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})

# CCRR ------------------------------------------------------------

test_that("Conditional Correct Response Rate", {
  expect <- CCRR[, 1] |>
    unlist() |>
    as.vector()
  result <- tgt$CCRR_table$`Child Item` |> unclass()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 2] |>
    unlist() |>
    as.vector()
  result <- tgt$CCRR_table$Rank |> unclass()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 3] |>
    unlist() |>
    as.vector()
  result <- tgt$CCRR_table$`N of Parents` |> unclass()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 4] |>
    unlist() |>
    as.vector()
  result <- tgt$CCRR_table$`Parent Items` |> unclass()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 5] |>
    unlist() |>
    as.vector()
  result <- tgt$CCRR_table$PIRP |> unclass()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- CCRR[, 6] |>
    unlist() |>
    as.vector()
  result <- tgt$CCRR_table$`Conditional CRR` |> unclass()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Student ---------------------------------------------------------

test_that("Students", {
  expect <- Student[, 6:13] |>
    unlist() |>
    as.vector()
  result <- tgt$Students |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})

#' @title emclus function
#' @description
#' This function is used for both LCA (Latent Class Analysis) and
#' LRA (Latent Rank Analysis). LCA is considered a special case of LRA,
#' in which the filtering matrix is reduced to the identity matrix.
#' This function takes a dataset, the number of classes, and a filtering
#' matrix as input and returns the latent rank.
#' When a confirmatory IRP matrix (conf) is provided, non-NA cells are
#' held fixed throughout estimation and only NA cells are freely estimated.
#' @param U response matrix U of the examData class.
#' @param Z missing indicator matrix Z of the examData class.
#' @param Fil Filter matrix
#' @param ncls number of latent class
#' @param beta1 beta distribution parameter1 as prior density of rank reference matrix
#' @param beta2 beta distribution parameter2 as prior density of rank reference matrix
#' @param maxiter Maximum number of iterations.
#' @param mic Monotonic increasing IRP option
#' @param verbose verbose output Flag. default is FALSE
#' @param conf Confirmatory IRP matrix (ncls x testlength). Non-NA values are
#'   fixed, NA values are freely estimated. NULL means fully exploratory.
#' @noRd

emclus <- function(U, Z, ncls, Fil, beta1, beta2, maxiter = 100, mic = FALSE,
                   verbose = FALSE, conf = NULL) {
  # Initialize
  testlength <- NCOL(U)
  const <- exp(-testlength)
  test_log_lik <- -1 / const
  old_test_log_lik <- -2 / const
  item_log_lik <- rep(test_log_lik / testlength, testlength)
  classRefMat <- matrix(rep(1:ncls / (ncls + 1), testlength), ncol = testlength)

  # Prepare confirmatory constraints
  if (!is.null(conf)) {
    fixed <- !is.na(conf)
    classRefMat[fixed] <- conf[fixed]
  }

  ## EM algorithm
  emt <- 0
  converge <- TRUE
  FLG <- TRUE

  while (FLG) {
    emt <- emt + 1
    old_test_log_lik <- test_log_lik

    llmat <- U %*% t(log(classRefMat + const)) + (Z * (1 - U)) %*% t(log(1 - classRefMat + const))
    exp_llmat <- exp(llmat)
    postDist <- exp_llmat / rowSums(exp_llmat)

    smoothPost <- postDist %*% Fil
    correct_cls <- t(smoothPost) %*% U
    incorrect_cls <- t(smoothPost) %*% (Z * (1 - U))

    old_classRefMat <- classRefMat
    classRefMat <- (correct_cls + beta1 - 1) / (correct_cls + incorrect_cls + beta1 + beta2 - 2)
    classRefMat <- pmax(pmin(classRefMat, 1 - const), const)

    # Apply confirmatory constraints after M-step, before mic sort
    if (!is.null(conf)) {
      classRefMat[fixed] <- conf[fixed]
    }

    if (mic) {
      classRefMat <- apply(classRefMat, 2, sort)
    }

    item_log_lik <- colSums(correct_cls * log(classRefMat + const) + incorrect_cls * log(1 - classRefMat + const))
    test_log_lik <- sum(item_log_lik)
    if (verbose) {
      message(
        sprintf(
          "\r%-80s",
          paste0(
            "iter ", emt, " log_lik ", format(test_log_lik, digits = 6)
          )
        ),
        appendLF = FALSE
      )
    }
    if (test_log_lik - old_test_log_lik <= 0) {
      classRefMat <- old_classRefMat
      FLG <- FALSE
    }
    if ((test_log_lik - old_test_log_lik) <= 0.0001 * abs(old_test_log_lik)) {
      FLG <- FALSE
    }
    if (emt == maxiter) {
      message("\nReached the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
      converge <- FALSE
      FLG <- FALSE
    }
  }

  ret <- list(
    iter = emt,
    converge = converge,
    postDist = postDist,
    classRefMat = classRefMat
  )
  return(ret)
}


#' @title SOM-based Latent Rank Estimation
#' @description
#' Estimates latent ranks using Self-Organizing Maps (SOM).
#' This function is used internally by LRA.binary when method = "SOM".
#' Unlike the GTM method (emclus), SOM uses online learning with
#' neighborhood functions to update the rank reference matrix.
#' @param U response matrix U of the examData class.
#' @param Z missing indicator matrix Z of the examData class.
#' @param ncls number of latent ranks
#' @param mic Monotonic increasing IRP option
#' @param maxiter Maximum number of iterations.
#' @param BIC.check If TRUE, convergence is checked using BIC values.
#' @param seed Random seed for reproducibility.
#' @param verbose verbose output Flag. default is FALSE
#' @param conf Confirmatory IRP matrix (ncls x testlength). Non-NA values are
#'   fixed, NA values are freely estimated. NULL means fully exploratory.
#' @return A list with iter, converge, postDist, classRefMat (same structure as emclus)
#' @noRd

somclus <- function(U, Z, ncls, mic = FALSE, maxiter = 100,
                    BIC.check = FALSE, seed = NULL, verbose = FALSE,
                    conf = NULL) {
  testlength <- NCOL(U)
  samplesize <- NROW(U)
  const <- exp(-testlength)

  # Prepare confirmatory constraint (transposed to match RefMat: testlength x ncls)
  if (!is.null(conf)) {
    conf_t <- t(conf)
    fixed_t <- !is.na(conf_t)
  }

  somt <- 0
  alpha1 <- 1
  alphaT <- 0.01
  sigma1 <- 1
  sigmaT <- 0.12

  alpha_list <- ((maxiter - 1:maxiter) * alpha1 + (1:maxiter - 1) * alphaT) / (maxiter - 1)
  sigma_list <- ((maxiter - 1:maxiter) * sigma1 + (1:maxiter - 1) * sigmaT) / (maxiter - 1)

  kappa1 <- 0.01
  kappaT <- 0.0001

  kappa_list <- ((maxiter - 1:maxiter) * kappa1 + (1:maxiter - 1) * kappaT) / (maxiter - 1)

  prior_list <- rep(1 / ncls, ncls)

  r_list <- seq(-ncls + 1, ncls - 1)
  hhhmat <- array(NA, c(maxiter, length(r_list)))
  for (t in 1:maxiter) {
    hhhmat[t, ] <- alpha_list[t] * ncls / samplesize * exp(-(r_list)^2 / (2 * ncls^2 * sigma_list[t]^2))
  }

  clsRefMat <- matrix(rep(1:ncls / (ncls + 1), testlength), ncol = testlength)
  RefMat <- t(clsRefMat)

  # Apply confirmatory constraints to initial values
  if (!is.null(conf)) {
    RefMat[fixed_t] <- conf_t[fixed_t]
  }

  oldBIC <- 1e5
  converge <- TRUE
  FLG <- TRUE
  while (FLG) {
    somt <- somt + 1

    if (somt <= maxiter) {
      h_count <- somt
    } else {
      h_count <- maxiter
    }

    loglike <- 0

    if (is.null(seed)) {
      set.seed(sum(U) + somt)
    } else {
      set.seed(seed)
    }

    is <- order(runif(samplesize, 1, 100))

    for (s in 1:samplesize) {
      ss <- is[s]
      mlrank <- U[ss, ] %*% log(RefMat + const) + (1 - U[ss, ]) %*% log(1 - RefMat + const) + log(prior_list)
      winner <- which.max(mlrank)
      loglike <- loglike + mlrank[winner]
      hhh <- matrix(rep(hhhmat[h_count, (ncls + 1 - winner):(2 * ncls - winner)], testlength),
        nrow = testlength, byrow = T
      )
      RefMat <- RefMat + hhh * (U[ss, ] - RefMat)
      prior_list <- prior_list + (kappa_list[h_count] / ncls)
      prior_list[winner] <- prior_list[winner] - kappa_list[h_count]
      prior_list[prior_list > 1] <- 1
      prior_list[prior_list < const] <- const
    }

    # Apply confirmatory constraints before mic sort
    if (!is.null(conf)) {
      RefMat[fixed_t] <- conf_t[fixed_t]
    }

    if (mic) {
      RefMat <- t(apply(RefMat, 1, sort))
    }
    llmat <- U %*% t(log(t(RefMat) + const)) + (Z * (1 - U)) %*%
      t(log(1 - t(RefMat) + const))
    expllmat <- exp(llmat)
    postdist <- expllmat / rowSums(expllmat)
    item_ell <- item_log_lik(U, Z, postdist, t(RefMat))
    if (BIC.check) {
      if (somt > maxiter * 10) {
        message("\nReached ten times the maximum number of iterations.")
        message("Warning: Algorithm may not have converged. Interpret results with caution.")
        converge <- FALSE
        FLG <- FALSE
        break
      }
      FI <- ItemFit(U, Z, item_ell, ncls)
      diff <- abs(oldBIC - FI$test$BIC)
      oldBIC <- FI$test$BIC
      if (diff < 1e-4) {
        message("\nConverged before reaching maximum iterations.")
        FLG <- FALSE
        break
      }
    } else {
      if (somt == maxiter) {
        message("\nReached the maximum number of iterations.")
        message("Warning: Algorithm may not have converged. Interpret results with caution.")
        converge <- FALSE
        FLG <- FALSE
      }
    }
  }

  ret <- list(
    iter = somt,
    converge = converge,
    postDist = postdist,
    classRefMat = t(RefMat)
  )
  return(ret)
}


#' @title calc final item log-likelihood
#' @description
#' Using the original data, class membership matrix, and class reference matrix,
#'  the log-likelihood for each item is calculated.
#' @param U response matrix U of the examData class.
#' @param Z missing indicator matrix Z of the examData class.
#' @param postDist class membership matrix
#' @param classRefMat class reference matrix
#' @noRd

#' @title Validate and align confirmatory IRP matrix
#' @description
#' Validates the confirmatory IRP matrix (conf) for use in LCA/LRA.
#' The user-facing format is items (rows) x ncls (columns), matching
#' the IRP output format. When conf has row names, matches them against
#' the item labels of the dataset and reorders/expands accordingly.
#' Items in the data but not in conf are set to NA (freely estimated).
#' When conf has no row names, validates dimension match by position.
#' Returns the transposed matrix (ncls x testlength) for internal use.
#' @param conf Confirmatory IRP matrix (items x ncls). Non-NA values are
#'   fixed, NA values are freely estimated. Row names, if present,
#'   are used for label-based matching against item_labels.
#' @param ncls Number of latent classes/ranks.
#' @param item_labels Character vector of item labels from the dataset.
#' @return The validated (and possibly reordered/expanded) conf matrix
#'   transposed to dimension ncls x length(item_labels) for internal use.
#' @noRd

validate_conf <- function(conf, ncls, item_labels) {
  testlength <- length(item_labels)

  if (!is.matrix(conf)) {
    stop("conf must be a matrix.")
  }
  if (ncol(conf) != ncls) {
    stop(sprintf(
      "conf must have %d columns (ncls/nrank), but got %d.",
      ncls, ncol(conf)
    ))
  }

  # Label-based matching when conf has row names
  if (!is.null(rownames(conf))) {
    conf_labels <- rownames(conf)
    # Check for unknown labels
    unknown <- setdiff(conf_labels, item_labels)
    if (length(unknown) > 0) {
      stop(sprintf(
        "conf contains item labels not found in the data: %s",
        paste(unknown, collapse = ", ")
      ))
    }
    # Build aligned matrix (NA for items not in conf)
    aligned <- matrix(NA, nrow = testlength, ncol = ncls)
    rownames(aligned) <- item_labels
    matched <- match(conf_labels, item_labels)
    aligned[matched, ] <- conf
    conf <- aligned
  } else {
    # Position-based: require exact dimension match
    if (nrow(conf) != testlength) {
      stop(sprintf(
        "conf must have %d rows (testlength), but got %d. Set row names for label-based matching.",
        testlength, nrow(conf)
      ))
    }
  }

  # Value range check
  non_na <- conf[!is.na(conf)]
  if (length(non_na) > 0 && any(non_na <= 0 | non_na >= 1)) {
    stop("Non-NA values in conf must be in the open interval (0, 1).")
  }

  # Transpose to ncls x testlength for internal use
  return(t(conf))
}


item_log_lik <- function(U, Z, postDist, classRefMat) {
  const <- exp(-NCOL(U))
  correct_cls <- t(postDist) %*% U
  incorrect_cls <- t(postDist) %*% (Z * (1 - U))
  item_ll <- colSums(correct_cls * log(classRefMat + const) + incorrect_cls * log(1 - classRefMat + const))
  return(item_ll)
}

#' @title Structure Learning for LDLRA by PBIL algorithm
#' @description
#' Generating DAG list from data using Population-Based Incremental learning
#' @details
#' This function performs structural learning for each classes by using
#' the Population-Based Incremental Learning model(PBIL) proposed by
#' Fukuda et al.(2014) within the genetic algorithm framework.
#' Instead of learning the adjacency matrix itself, the 'genes of genes'
#' that generate the adjacency matrix are updated with each generation.
#' For more details, please refer to Fukuda(2014) and Section
#' 9.4.3 of the text(Shojima,2022).
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param ncls number of latent class(rank). The default is 2.
#' @param seed seed for random.
#' @param method specify the model to analyze the data.Local dependence latent
#' class model is set to "C", latent rank model is set "R". The default is "R".
#' @param population Population size. The default is 20
#' @param Rs Survival Rate. The default is 0.5
#' @param Rm Mutation Rate. The default is 0.002
#' @param maxParents Maximum number of edges emanating from a single node. The default is 2.
#' @param maxGeneration Maximum number of generations.
#' @param successiveLimit Termination conditions. If the optimal individual does not change
#' for this number of generations, it is considered to have converged.
#' @param elitism Number of elites that remain without crossover when transitioning to
#' the next generation.
#' @param alpha Learning rate. The default is 0.05
#' @param estimate In PBIL for estimating the adjacency matrix, specify by number from the
#' following four methods: 1. Optimal adjacency matrix, 2. Rounded average of individuals in
#' the last generation, 3. Rounded average of survivors in the last generation, 4. Rounded
#' generational gene of the last generation. The default is 1.
#' @param filename Specify the filename when saving the generated adjacency matrix in CSV format.
#' The default is null, and no output is written to the file.
#' @param verbose verbose output Flag. default is TRUE
#' @param beta1 Beta distribution parameter 1 for prior density. Default is 2.
#' @param beta2 Beta distribution parameter 2 for prior density. Default is 2.
#' @return
#' \describe{
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{crr}{correct response ratio}
#'  \item{adj_list}{adjacency matrix list}
#'  \item{g_list}{graph list}
#'  \item{referenceMatrix}{Learned Parameters.A three-dimensional array of patterns where
#'  item x rank x pattern.}
#'  \item{IRP}{Marginal Item Reference Matrix}
#'  \item{IRPIndex}{IRP Indices which include Alpha, Beta, Gamma.}
#'  \item{TRP}{Test Reference Profile matrix.}
#'  \item{LRD}{latent Rank/Class Distribution}
#'  \item{RMD}{Rank/Class Membership Distribution}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#'  \item{Estimation_table}{Estimated parameters tables.}
#'  \item{CCRR_table}{Correct Response Rate tables}
#'  \item{Studens}{Student information. It includes estimated class
#'  membership, probability of class membership, RUO, and RDO.}
#' }
#' @references Fukuda, S., Yamanaka, Y., & Yoshihiro, T. (2014). A Probability-based evolutionary
#'  algorithm with mutations to learn Bayesian networks. International Journal of Artificial
#'  Intelligence and Interactive Multimedia, 3, 7–13. DOI: 10.9781/ijimai.2014.311
#' @importFrom stats runif rbinom
#' @examples
#' \donttest{
#' # Perform Structure Learning for LDLRA using PBIL algorithm
#' # This process may take considerable time due to evolutionary optimization
#' result.LDLRA.PBIL <- LDLRA_PBIL(J35S515,
#'   seed = 123, # Set random seed for reproducibility
#'   ncls = 5, # Number of latent ranks
#'   maxGeneration = 10,
#'   method = "R", # Use rank model (vs. class model)
#'   elitism = 1, # Keep best solution in each generation
#'   successiveLimit = 15 # Convergence criterion
#' )
#'
#' # Examine the learned network structure
#' # Plot Item Response Profiles showing item patterns across ranks
#' plot(result.LDLRA.PBIL, type = "IRP", nc = 4, nr = 3)
#'
#' # Plot Test Response Profile showing overall response patterns
#' plot(result.LDLRA.PBIL, type = "TRP")
#'
#' # Plot Latent Rank Distribution showing student distribution
#' plot(result.LDLRA.PBIL, type = "LRD")
#' }
#' @export
#'

LDLRA_PBIL <- function(U, Z = NULL, w = NULL, na = NULL,
                       seed = 123, ncls = 2, method = "R",
                       population = 20, Rs = 0.5, Rm = 0.002,
                       maxParents = 2, maxGeneration = 100,
                       successiveLimit = 5, elitism = 0,
                       alpha = 0.05, estimate = 1,
                       filename = NULL,
                       verbose = TRUE, beta1 = 2, beta2 = 2) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  if (method == "C" | method == "Class") {
    if (verbose) message("local dependence latent Class model is chosen.")
    model <- 1
  } else if (method == "R" | method == "Rank") {
    if (verbose) message("local dependence latent Rank model is chosen.")
    model <- 2
  }

  # estimate type
  if (estimate < 1 || estimate > 4) {
    stop("Check the estimate type. It should be set between 1 to 4.")
  }

  RsI <- round(population * Rs)
  # Elitism check
  if (RsI < 0) {
    RsI <- 0
  } else if (RsI > population * Rs * 0.5) {
    RsI <- round(population * Rs * 0.5)
    warning("Too many survivers. Limit to ", RsI)
  }

  # Elitism check
  if (elitism < 0) {
    elitism <- 0
  } else if (elitism > population * Rs * 0.5) {
    elitism <- round(population * Rs * 0.5)
    warning("Too many elites. Limit to ", elitism)
  }

  # Initialize ------------------------------------------------------

  set.seed(seed)
  crr <- crr(tmp)
  sort_list <- order(crr, decreasing = TRUE)
  adj_sort <- data.frame(item = tmp$ItemLabel, crr = crr)
  adj <- matrix(0, ncol = testlength, nrow = testlength)
  adj[upper.tri(adj)] <- 1
  colnames(adj) <- rownames(adj) <- tmp$ItemLabel[sort_list]
  gene_length <- sum(upper.tri(adj))

  gene <- array(0.5, dim = c(ncls, gene_length))
  adj_t <- array(0, dim = c(population, ncls, gene_length))

  if (model == 1) {
    filmat <- diag(1, ncol = ncls, nrow = ncls)
  } else {
    fil0 <- (1 / 3 - 0.4) / 5 * (ncls - 5) + 0.4
    fil1 <- (1 / 3 - 0.2) / 5 * (ncls - 5) + 0.2
    fil2 <- (1 - fil0 - 2 * fil1) / 2
    filmat0 <- diag(fil0, ncls)
    filmat1 <- diag(fil1, (ncls + 1))[2:(ncls + 1), 1:ncls]
    filmat2 <- diag(fil2, (ncls + 2))[3:(ncls + 2), 1:ncls]
    filmat <- filmat0 + filmat1 + filmat2 + t(filmat1) + t(filmat2)
    filmat <- filmat / (rep(1, ncls) %*% t(rep(1, ncls)) %*% filmat)
  }

  ret.emclus <- emclus(tmp$U, tmp$Z, ncls, Fil = filmat, beta1 = beta1, beta2 = beta2)
  smoothpost <- ret.emclus$postDist %*% filmat

  fitness <- numeric(population)

  limit_count <- 0
  adj_list <- list()

  GA_FLG <- TRUE
  generation <- 0
  bestfit <- 1e+100

  # gen0
  for (i in 1:population) {
    for (j in 1:ncls) {
      vec <- rbinom(gene_length, 1, prob = gene[j, ])
      # Omit Null Model
      if (sum(vec) == 0) {
        pos <- sample(1:gene_length, 1)
        vec[pos] <- 1
      }
      adj_t[i, j, ] <- maxParents_penalty(vec, testlength, maxParents)
      adj[upper.tri(adj)] <- adj_t[i, j, ]
      adj_list[[j]] <- adj
    }
    # fitness
    ret.LDparam <- LD_param_est(tmp, adj_list, ret.emclus$classRefMat, ncls, smoothpost, beta1, beta2)
    fitness[i] <- ret.LDparam$FitIndices$BIC
  }

  sort_list <- order(fitness)
  adj_t <- adj_t[sort_list, , ]
  best_individual <- adj_t[1, , ]
  fitness <- sort(fitness)
  bestfit <- fitness[1]

  while (GA_FLG) {
    generation <- generation + 1
    for (i in 1:population) {
      if (i > elitism) {
        for (j in 1:ncls) {
          vec <- rbinom(gene_length, 1, prob = gene[j, ])
          # Omit Null Model
          if (sum(vec) == 0) {
            pos <- sample(1:gene_length, 1)
            vec[pos] <- 1
          }
          adj_t[i, j, ] <- maxParents_penalty(vec, testlength, maxParents)
          adj[upper.tri(adj)] <- adj_t[i, j, ]
          adj_list[[j]] <- adj
        }
        # fitness
        ret.LDparam <- LD_param_est(tmp, adj_list, ret.emclus$classRefMat, ncls, smoothpost, beta1, beta2)
        if (verbose) {
          message(
            sprintf(
              "\r%-80s",
              paste0(
                "Gen ", generation, " ID.", i,
                " BIC ", format(round(ret.LDparam$FitIndices$BIC, 3), nsmall = 3),
                " BEST ", format(round(bestfit, 3), nsmall = 3),
                " limit count ", limit_count
              )
            ),
            appendLF = FALSE
          )
        }
        fitness[i] <- ret.LDparam$FitIndices$BIC
      }
    }

    sort_list <- order(fitness)
    adj_t <- adj_t[sort_list, , ]

    ## update Gene
    ar <- round(colMeans(adj_t[1:RsI, , ]))
    gene <- gene + (alpha * (ar - gene))
    ## mutation
    u <- matrix(runif(gene_length * ncls, min = 0, max = 1), nrow = ncls)
    Bm <- matrix(rbinom(gene_length * ncls, 1, Rm), nrow = ncls)
    gene <- (1 - Bm) * gene + Bm * (gene + u) / 2

    # Termination check
    if (generation > maxGeneration) {
      if (verbose) message("The maximum generation has been reached")
      GA_FLG <- FALSE
    }
    if (all(best_individual == adj_t[1, , ])) {
      limit_count <- limit_count + 1
    } else {
      limit_count <- 0
      fitness <- sort(fitness)
      bestfit <- fitness[1]
      best_individual <- adj_t[1, , ]
    }
    if (limit_count >= successiveLimit) {
      if (verbose) message("The BIC has not changed for ", successiveLimit, " times.")
      GA_FLG <- FALSE
    }
  }

  # Estimate --------------------------------------------------------
  if (estimate == 1) {
    for (i in 1:ncls) {
      adj[upper.tri(adj)] <- adj_t[1, i, ]
      adj_list[[i]] <- adj
    }
  } else if (estimate == 2) {
    for (i in 1:ncls) {
      adj[upper.tri(adj)] <- round(colMeans(adj_t[, i, ]))
      adj_list[[i]] <- adj
    }
  } else if (estimate == 3) {
    for (i in 1:ncls) {
      adj[upper.tri(adj)] <- round(colMeans(adj_t[1:RsI, i, ]))
      adj_list[[i]] <- adj
    }
  } else {
    for (i in 1:ncls) {
      adj[upper.tri(adj)] <- round(gene[i, ])
      adj_list[[i]] <- adj
    }
  }

  # output file -----------------------------------------------------
  if (!is.null(filename)) {
    g.df <- data.frame()
    for (i in 1:ncls) {
      g <- igraph::graph_from_adjacency_matrix(adj_list[[i]])
      tmp <- igraph::as_data_frame(g)
      tmp$Rank <- i
      g.df <- rbind(g.df, tmp)
    }
    write.table(g.df,
      sep = ",",
      row.names = FALSE,
      col.names = TRUE,
      quote = FALSE,
      file = filename
    )
  }


  ret <- LDLRA(tmp, ncls = ncls, adj_list = adj_list)
  return(ret)
}

#' @title StrLearningPBIL_LDLRA (Deprecated)
#' @description
#' This function has been renamed to \code{\link{LDLRA_PBIL}}.
#' Please use \code{LDLRA_PBIL} instead.
#' @param ... All arguments passed to \code{\link{LDLRA_PBIL}}
#' @export
StrLearningPBIL_LDLRA <- function(...) {
  .Deprecated("LDLRA_PBIL")
  LDLRA_PBIL(...)
}

# Biclustering plot functions (binary Array + shared color palette)
# Internal functions called from plot.exametrika()

#' Merge default plotting arguments with user-supplied graphical parameters
#'
#' User-supplied arguments collected by \code{...} in \code{plot.exametrika()}
#' (passed down as \code{dots}) take precedence over the package defaults, so
#' callers can override \code{xlab}, \code{ylab}, \code{main}, etc., and add
#' standard graphical parameters such as \code{pch}, \code{las}, \code{cex}.
#' Base graphics functions (\code{plot}, \code{barplot}, \code{image},
#' \code{axis}, \code{lines}, \code{curve}) silently ignore graphical
#' parameters that do not apply, so the merged list can be forwarded verbatim.
#' @noRd
merge_plot_dots <- function(defaults, dots) {
  if (length(dots)) utils::modifyList(defaults, dots) else defaults
}

#' Call a base graphics function with defaults overridable by user dots
#' @noRd
call_plot <- function(.fun, defaults, dots = list()) {
  do.call(.fun, merge_plot_dots(defaults, dots))
}

#' Draw a curve by grid evaluation, honouring user graphical parameters
#'
#' Replacement for \code{curve()} that evaluates \code{fn} (a vectorised
#' function of the abscissa) on a grid and draws it via \code{plot}/\code{lines}
#' through \code{call_plot()}, so user-supplied \code{...} arguments are
#' applied. Using grid evaluation avoids the non-standard evaluation of
#' \code{curve()}'s first argument.
#' @noRd
draw_curve <- function(fn, from = -4, to = 4, n = 201, add = FALSE,
                       defaults = list(), dots = list()) {
  xx <- seq(from, to, length.out = n)
  yy <- fn(xx)
  if (add) {
    call_plot(graphics::lines, c(list(x = xx, y = yy), defaults), dots)
  } else {
    call_plot(plot, c(list(x = xx, y = yy, type = "l"), defaults), dots)
  }
}

#' Colorblind-friendly palette (Paul Tol Vibrant + Bright extension)
#' @noRd
get_cb_palette <- function(n) {
  base <- c(
    "#0077BB", # blue
    "#EE7733", # orange
    "#009988", # teal
    "#EE3377", # magenta
    "#CC3311", # red
    "#33BBEE", # cyan
    "#AA3377", # purple
    "#DDCC77", # sand
    "#332288", # indigo
    "#117733" # forest green
  )
  if (n <= length(base)) {
    return(base[1:n])
  }
  return(c(base, rainbow(n - length(base))))
}

#' Set up panel grid + legend strip layout
#' Overrides mfrow, reserving a thin legend area in the bottom row
#' @noRd
setup_legend_layout <- function(n_panels, nc) {
  if (n_panels <= 1) {
    layout(matrix(c(1, 2), nrow = 2), heights = c(1, 0.2))
  } else {
    n_rows <- ceiling(n_panels / nc)
    layout_mat <- matrix(0, nrow = n_rows + 1, ncol = nc)
    for (i in seq_len(n_panels)) {
      r <- ceiling(i / nc)
      cc <- ((i - 1) %% nc) + 1
      layout_mat[r, cc] <- i
    }
    layout_mat[n_rows + 1, ] <- n_panels + 1
    layout(layout_mat, heights = c(rep(1, n_rows), 0.2))
  }
}

#' Draw legend in the bottom row of the layout
#' Call after setup_legend_layout() and all panels are drawn
#' @noRd
draw_legend_strip <- function(...) {
  par(mar = c(0, 0, 0, 0))
  plot.new()
  legend("center", ...)
}

#' Array plot (shared by Biclustering / IRM / LDB / BINET)
#' @noRd
plot_array <- function(x, cell_width, cell_height, colors, dots = list()) {
  cell_w <- cell_width
  cell_h <- cell_height
  old_par <- par(no.readonly = TRUE)
  on.exit({
    restore_par <- old_par
    restore_par[c("pin", "fin", "plt", "usr")] <- NULL
    suppressWarnings(par(restore_par))
  })
  # Reduce margins to maximize plot area
  par(mfrow = c(1, 2), mar = c(1, 1, 2, 1), oma = c(0, 0, 0, 0))

  nrows <- x$nobs
  ncols <- x$testlength

  # Sort so that higher class numbers (higher correct response rates) appear at bottom
  case_order <- order(x$ClassEstimated, decreasing = FALSE)
  field_order <- order(x$FieldEstimated, decreasing = FALSE)
  # For polytomous models, use Q (polytomous responses) instead of U (binary)
  if (!is.null(x$Q) && inherits(x, c("nominalBiclustering", "ordinalBiclustering", "ratedBiclustering"))) {
    raw_data <- x$Q
  } else {
    raw_data <- x$U
    if (is.null(raw_data)) raw_data <- x$Q
  }

  clustered_data <- raw_data[case_order, field_order]

  sorted_class <- x$ClassEstimated[case_order]
  sorted_field <- x$FieldEstimated[field_order]

  class_breaks <- cumsum(table(sorted_class))
  field_breaks <- cumsum(table(sorted_field))

  class_lines <- (nrows - class_breaks[-length(class_breaks)]) * cell_h
  field_lines <- field_breaks[-length(field_breaks)] * cell_w

  ## colors
  # Use sort(unique(...)) to ensure consistent ordering: 0 (white), 1 (black)
  # Exclude missing values (NA and -1) from category colors
  all_values <- sort(unique(as.vector(as.matrix(raw_data))))
  all_values <- all_values[!is.na(all_values) & all_values != -1]
  n_categories <- length(all_values)

  if (is.null(colors)) {
    if (n_categories == 2) {
      colors <- c("#FFFFFF", "#000000")
    } else {
      colors <- c(
        "#E69F00", "#0173B2", "#DE8F05", "#029E73", "#CC78BC",
        "#CA9161", "#FBAFE4", "#949494", "#ECE133", "#56B4E9"
      )
    }
  }
  if (length(colors) < n_categories) {
    additional_colors <- c(
      "#D55E00", "#F0E442", "#009E73", "#CC79A7", "#0072B2",
      "#E8601C", "#7CAE00", "#C77CFF", "#00BFC4", "#F8766D"
    )
    colors <- c(colors, additional_colors)
    colors <- colors[1:n_categories]
  }

  # Missing value color: gray for binary (to distinguish from white/black), black for polytomous
  missing_color <- if (n_categories == 2) "#808080" else "#000000"

  # Plot area
  plot_width <- ncols * cell_w
  plot_height <- nrows * cell_h

  # original data
  call_plot(
    plot,
    list(
      x = 0, y = 0,
      type = "n",
      xlim = c(0, plot_width), ylim = c(0, plot_height),
      xlab = "", ylab = "", xaxt = "n", yaxt = "n",
      main = "Original Data", frame.plot = TRUE
    ),
    dots
  )
  for (i in 1:nrows) {
    for (j in 1:ncols) {
      val <- raw_data[i, j]
      x1 <- (j - 1) * cell_w
      y1 <- (nrows - i) * cell_h
      x2 <- j * cell_w
      y2 <- (nrows - i + 1) * cell_h

      if (is.na(val) || val == -1) {
        fill_color <- missing_color
      } else {
        color_index <- match(val, all_values)
        fill_color <- colors[color_index]
      }

      rect(x1, y1, x2, y2, col = fill_color, border = "white", lwd = 0.1)
    }
  }

  ## Clustered Plot
  call_plot(
    plot,
    list(
      x = 0, y = 0,
      type = "n",
      xlim = c(0, plot_width), ylim = c(0, plot_height),
      xlab = "", ylab = "", xaxt = "n", yaxt = "n",
      main = "Clustered Data", frame.plot = TRUE
    ),
    dots
  )
  for (i in 1:nrows) {
    for (j in 1:ncols) {
      val <- clustered_data[i, j]
      x1 <- (j - 1) * cell_w
      y1 <- (nrows - i) * cell_h
      x2 <- j * cell_w
      y2 <- (nrows - i + 1) * cell_h

      if (is.na(val) || val == -1) {
        fill_color <- missing_color
      } else {
        color_index <- match(val, all_values)
        fill_color <- colors[color_index]
      }

      rect(x1, y1, x2, y2, col = fill_color, border = "white", lwd = 0.1)
    }
  }
  for (line_y in class_lines) {
    lines(c(0, plot_width), c(line_y, line_y),
      col = "red", lwd = 1
    )
  }
  for (line_x in field_lines) {
    lines(c(line_x, line_x), c(0, plot_height),
      col = "red", lwd = 1
    )
  }
}

# Tests for forwarding of graphical parameters supplied via ... to the
# underlying base R plotting functions (R Journal reviewer request, v1.14.0).

test_that("merge_plot_dots lets user dots override and extend defaults", {
  # user value overrides a default
  expect_equal(merge_plot_dots(list(pch = 1, type = "b"), list(pch = 16))$pch, 16)
  # user value is added when not present in defaults
  expect_equal(merge_plot_dots(list(type = "b"), list(las = 2))$las, 2)
  # defaults preserved
  expect_equal(merge_plot_dots(list(type = "b"), list(las = 2))$type, "b")
  # empty dots leaves defaults untouched
  expect_identical(merge_plot_dots(list(type = "b"), list()), list(type = "b"))
})

test_that("call_plot forwards merged arguments to the target function", {
  spy <- function(...) list(...)
  res <- call_plot(spy, list(side = 1, at = 1:3), list(las = 2, pch = 16))
  expect_equal(res$side, 1)
  expect_equal(res$at, 1:3)
  expect_equal(res$las, 2) # graphical parameter reaches the call
  expect_equal(res$pch, 16)
})

test_that("call_plot lets dots override a default passed by the package", {
  spy <- function(...) list(...)
  res <- call_plot(spy, list(xlab = "default", main = "m"), list(xlab = "user"))
  expect_equal(res$xlab, "user")
  expect_equal(res$main, "m")
})

test_that("draw_curve forwards graphical parameters to plot and lines", {
  captured <- new.env()
  captured$plot_args <- NULL
  captured$line_args <- NULL
  local_mocked_bindings(
    call_plot = function(.fun, defaults, dots = list()) {
      merged <- merge_plot_dots(defaults, dots)
      if (identical(.fun, graphics::lines)) {
        captured$line_args <- merged
      } else {
        captured$plot_args <- merged
      }
      invisible(NULL)
    }
  )
  draw_curve(function(theta) theta, from = -1, to = 1, dots = list(las = 2))
  expect_equal(captured$plot_args$las, 2)
  draw_curve(function(theta) theta, from = -1, to = 1, add = TRUE, dots = list(pch = 16))
  expect_equal(captured$line_args$pch, 16)
})

test_that("plot.exametrika accepts the reviewer's example without error", {
  skip_on_cran()
  pdf(tempfile(fileext = ".pdf"))
  on.exit(dev.off())
  res_lra <- LRA(J15S500, nrank = 4)
  # exact call from the R Journal review
  expect_no_error(
    plot(res_lra, type = "IRP", items = 1:4, nc = 2, nr = 2, las = 2, pch = 16)
  )
})

test_that("graphical parameters propagate across LRA / IRT / GRM plot types", {
  skip_on_cran()
  pdf(tempfile(fileext = ".pdf"))
  on.exit(dev.off())

  res_lra <- LRA(J15S500, nrank = 4)
  expect_no_error(plot(res_lra, type = "IRP", items = 1:2, las = 2, pch = 16))
  expect_no_error(plot(res_lra, type = "TRP", las = 2))
  expect_no_error(plot(res_lra, type = "LRD", cex.axis = 1.2))
  expect_no_error(plot(res_lra, type = "RMP", students = 1:2, pch = 17))

  res_irt <- IRT(J15S500)
  expect_no_error(plot(res_irt, type = "IRF", items = 1:3, pch = 16, las = 2))
  expect_no_error(plot(res_irt, type = "IRF", items = 1:3, overlay = TRUE, lwd = 3))
  expect_no_error(plot(res_irt, type = "TRF", las = 2))
  expect_no_error(plot(res_irt, type = "TIF", col = "blue"))

  res_grm <- GRM(J5S1000)
  expect_no_error(plot(res_grm, type = "IRF", items = 1:2, las = 2))
  expect_no_error(plot(res_grm, type = "TIF", lwd = 2))
})

test_that("graphical parameters propagate across Biclustering plot types", {
  skip_on_cran()
  pdf(tempfile(fileext = ".pdf"))
  on.exit(dev.off())

  bic <- Biclustering(J35S515, nfld = 5, ncls = 4, method = "B", verbose = FALSE)
  expect_no_error(plot(bic, type = "Array", las = 2))
  expect_no_error(plot(bic, type = "FRP", pch = 16))
  expect_no_error(plot(bic, type = "CRV", las = 2, pch = 15))
  expect_no_error(plot(bic, type = "TRP", las = 2))
})

#' @rdname Biclustering
#' @param ncls Number of latent classes/ranks to identify (between 2 and 20).
#' @param nfld Number of latent fields (item clusters) to identify.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#' observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @param method Analysis method to use (character string):
#'   * "B" or "Biclustering": Standard biclustering (default)
#'   * "R" or "Ranklustering": Ranklustering with ordered class structure
#' @param conf Confirmatory parameter for pre-specified field assignments. Can be either:
#'   * A vector with items and corresponding fields in sequence
#'   * A field membership profile matrix (items x fields) with 0/1 values
#'   * NULL (default) for exploratory analysis where field memberships are estimated
#' @param maxiter Maximum number of EM algorithm iterations. Default is 100.
#' @param verbose Logical; if TRUE, displays progress during estimation. Default is TRUE.
#' @param alpha Dirichlet distribution concentration parameter for prior density of field reference probabilities. Default is 1.
#' @param ... Additional arguments passed to specific methods.
#'
#' @examples
#' \donttest{
#' # Perform Biclustering for rated sample data
#' # Analyze data with 5 fields and 6 classes
#' result.Bi <- Biclustering(J35S5000, nfld = 5, ncls = 6, method = "R")
#' }
#' @export
Biclustering.rated <- function(U,
                               ncls = 2, nfld = 2,
                               method = "R",
                               conf = NULL,
                               maxiter = 100,
                               verbose = TRUE,
                               alpha = 1, ...) {
  tmp <- U
  nobs <- NROW(tmp$Q)
  nitems <- NCOL(tmp$Q)
  const <- exp(-nitems)

  # --- Step 1: Run nominal Biclustering internally ---
  # Reuse the already-formatted exametrika object as nominal (avoid re-calling dataFormat)
  dat_nom <- tmp
  dat_nom$response.type <- "nominal"

  ret_nom <- Biclustering.nominal(
    dat_nom,
    ncls = ncls,
    nfld = nfld,
    conf = conf,
    maxiter = maxiter,
    verbose = verbose,
    alpha = alpha,
    ...
  )

  # --- Step 2: Retrieve nominal results ---
  fld <- ret_nom$FieldEstimated
  fldmemb <- ret_nom$FieldMembership
  fldmemb01 <- sign(fldmemb - apply(fldmemb, 1, max)) + 1
  flddist <- colSums(fldmemb01)
  BCRM <- ret_nom$FRP

  # --- Step 3: Sort classes by correct response rate (Ranklustering) ---
  if (method == "R" || method == "Ranklustering") {
    model <- 2
    mic <- TRUE
    msg <- "Rank"
  } else {
    model <- 1
    mic <- FALSE
    msg <- "Class"
  }

  clsmemb <- ret_nom$ClassMembership
  ord <- order(colSums(t(tmp$U) %*% clsmemb) / colSums(clsmemb))
  clsmemb <- clsmemb[, ord]
  BCRM <- BCRM[, ord, ]
  cls <- apply(clsmemb, 1, which.max)
  clsmemb01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  clsdist <- colSums(clsmemb01)

  # --- Step 4: Student rank membership ---
  StudentRank <- clsmemb
  rownames(StudentRank) <- tmp$ID
  if (model == 2) {
    RU <- ifelse(cls + 1 > ncls, NA, cls + 1)
    RD <- ifelse(cls - 1 < 1, NA, cls - 1)
    RUO <- StudentRank[cbind(1:nobs, RU)] / StudentRank[cbind(1:nobs, cls)]
    RDO <- StudentRank[cbind(1:nobs, RD)] / StudentRank[cbind(1:nobs, cls)]
    StudentRank <- cbind(StudentRank, cls, RUO, RDO)
    colnames(StudentRank) <- c(
      paste("Membership", 1:ncls), "Estimate",
      "Rank-Up Odds", "Rank-Down Odds"
    )
  } else {
    StudentRank <- cbind(StudentRank, Estimate = cls)
    colnames(StudentRank) <- c(paste("Membership", 1:ncls), "Estimate")
  }

  # --- Step 5: Binary fit indices (Layer 1) ---
  # Compute item correct response rate per class (no field constraint)
  PiFR <- matrix(0, nrow = nitems, ncol = ncls)
  for (c in 1:ncls) {
    students_c <- which(cls == c)
    if (length(students_c) == 1) {
      denom <- tmp$Z[students_c, ]
      PiFR[, c] <- (tmp$U[students_c, ] * tmp$Z[students_c, ]) /
        pmax(denom, const)
    } else if (length(students_c) > 1) {
      denom <- colSums(tmp$Z[students_c, ])
      PiFR[, c] <- colSums(tmp$U[students_c, ] * tmp$Z[students_c, ]) /
        pmax(denom, const)
    }
  }

  P_correct <- clsmemb %*% t(PiFR)
  test_log_lik_binary <- sum(tmp$Z * (tmp$U * log(pmax(P_correct, const)) +
    (1 - tmp$U) * log(pmax(1 - P_correct, const))))
  nparam_binary <- nitems * ncls
  FitIndices_binary <- TestFit(tmp$U, tmp$Z, test_log_lik_binary, nparam_binary)

  # --- Step 6: Nominal fit indices (Layer 2) --- from ret_nom directly
  FitIndices_nominal <- ret_nom$TestFitIndices

  # --- Step 7: FRP labels ---
  FRP <- BCRM
  colnames(FRP) <- paste0(msg, 1:ncls)
  rownames(FRP) <- paste0("Field", 1:nfld)
  colnames(fldmemb) <- paste0("Field", 1:nfld)
  rownames(clsmemb) <- tmp$ID
  colnames(clsmemb) <- paste0(msg, 1:ncls)

  # --- Step 8: TRP and FRPIndex (binary-based) ---
  # quasiFRP: item-level correct response rate per class (J x C, no field constraint)
  # "quasi" because field assignments come from nominal analysis, not binary
  quasiFRP <- PiFR
  rownames(quasiFRP) <- colnames(tmp$U)
  colnames(quasiFRP) <- paste0(msg, 1:ncls)

  FieldFRP <- matrix(0, nrow = nfld, ncol = ncls)
  for (f in 1:nfld) {
    items_f <- which(fld == f)
    if (length(items_f) == 0) {
      FieldFRP[f, ] <- NA
    } else if (length(items_f) == 1) {
      FieldFRP[f, ] <- PiFR[items_f, ]
    } else {
      FieldFRP[f, ] <- colMeans(PiFR[items_f, ])
    }
  }
  rownames(FieldFRP) <- paste0("Field", 1:nfld)
  colnames(FieldFRP) <- paste0(msg, 1:ncls)

  TRP <- colSums(FieldFRP * flddist, na.rm = TRUE)
  # IRPindex only on non-empty fields
  valid_fields <- which(!is.na(FieldFRP[, 1]))
  if (length(valid_fields) > 0) {
    FRPIndex <- IRPindex(FieldFRP[valid_fields, , drop = FALSE])
  } else {
    FRPIndex <- data.frame(
      Alpha = numeric(0), A = numeric(0),
      Beta = numeric(0), B = numeric(0),
      Gamma = numeric(0), C = numeric(0)
    )
  }

  TRPlag <- TRP[2:ncls]
  TRPmic <- sum(TRPlag[1:(ncls - 1)] - TRP[1:(ncls - 1)] < 0, na.rm = TRUE)
  FRPmic <- sum(abs(FRPIndex$C))
  SOACflg <- WOACflg <- FALSE
  if (TRPmic == 0) {
    WOACflg <- TRUE
    if (FRPmic == 0) {
      SOACflg <- TRUE
    }
  }
  if (verbose) {
    if (SOACflg & WOACflg) {
      message("\nStrongly ordinal alignment condition was satisfied.")
    }
    if (!SOACflg & WOACflg) {
      message("\nWeakly ordinal alignment condition was satisfied.")
    }
  }

  # --- Step 9: Field Analysis ---
  tmp_binary <- tmp
  tmp_binary$response.type <- "binary"
  crr_vals <- crr(tmp_binary)
  fieldAnalysis <- as.data.frame(fldmemb)
  fieldAnalysis <- cbind(crr_vals, fld, fieldAnalysis)
  colnames(fieldAnalysis) <- c("CRR", "LFE", paste0("Field", 1:nfld))
  fieldAnalysis <- fieldAnalysis[order(fieldAnalysis$CRR, decreasing = TRUE), ]
  fieldAnalysis <- fieldAnalysis[order(fieldAnalysis$LFE), ]
  rownames_tmp <- rownames(fieldAnalysis)
  fieldAnalysis <- matrix(
    as.numeric(as.matrix(fieldAnalysis)),
    ncol = NCOL(fieldAnalysis), nrow = NROW(fieldAnalysis)
  )
  colnames(fieldAnalysis) <- c("CRR", "LFE", paste0("Field", 1:nfld))
  rownames(fieldAnalysis) <- rownames_tmp

  # --- Return ---
  ret <- structure(list(
    model = model,
    mic = mic,
    msg = msg,
    converge = ret_nom$converge,
    Q = tmp$Q,
    U = tmp$U,
    Z = tmp$Z,
    testlength = nitems,
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    n_cycle = ret_nom$n_cycle,
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = FRP,
    FieldFRP = FieldFRP,
    quasiFRP = quasiFRP,
    FRPIndex = FRPIndex,
    TRP = TRP,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    FieldAnalysis = fieldAnalysis,
    TestFitIndices = FitIndices_binary,
    TestFitIndices_nominal = FitIndices_nominal,
    log_lik = test_log_lik_binary,
    log_lik_nominal = ret_nom$log_lik,
    SOACflg = SOACflg,
    WOACflg = WOACflg,
    # Deprecated fields (for backward compatibility)
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = ret_nom$n_cycle,
    LogLik = test_log_lik_binary
  ), class = c("exametrika", "ratedBiclustering"))

  return(ret)
}

#' @title softmax function
#' @description
#' to avoid overflow
#' @param x numeric vector
#'

softmax <- function(x) {
  x_max <- max(x)
  x <- x - x_max
  return(exp(x) / sum(exp(x)))
}

#' @title Biclustering and Ranklustering Analysis
#' @description
#' Performs biclustering, ranklustering, or their confirmatory variants on binary response data.
#' These methods simultaneously cluster both examinees and items into homogeneous groups
#' (or ordered ranks for ranklustering). The analysis reveals latent structures and patterns
#' in the data by creating a matrix with rows and columns arranged to highlight block structures.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#' it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param ... Additional arguments passed to specific methods.
#'
#' @return An object of class "exametrika" and "Biclustering" containing:
#' \describe{
#'  \item{model}{Model type indicator (1 for biclustering, 2 for ranklustering)}
#'  \item{msg}{A character string indicating the model type. }
#'  \item{mic}{Logical value indicating whether monotonicity constraint was applied}
#'  \item{testlength}{Number of items in the test}
#'  \item{nobs}{Number of examinees in the dataset}
#'  \item{Nclass}{Number of latent classes/ranks specified}
#'  \item{Nfield}{Number of latent fields specified}
#'  \item{N_Cycle}{Number of EM iterations performed}
#'  \item{converge}{Logical value indicating wheter the algorithm converged within maxiter iterasions}
#'  \item{LFD}{Latent Field Distribution - counts of items assigned to each field}
#'  \item{LRD/LCD}{Latent Rank/Class Distribution - counts of examinees assigned to each class/rank}
#'  \item{FRP}{Field Reference Profile matrix - probability of correct response for each field-class combination}
#'  \item{FRPIndex}{Field Reference Profile indices (Kumagai, 2007) — data.frame with 6 columns:
#'    \describe{
#'      \item{Alpha}{Maximum-slope location: class transition where the largest increase occurs}
#'      \item{A}{Maximum slope: largest consecutive-class difference in the profile}
#'      \item{Beta}{Location parameter: class whose FRP value is closest to 0.5}
#'      \item{B}{FRP value at the Beta position}
#'      \item{Gamma}{Non-monotonicity ratio: proportion of class transitions that decrease}
#'      \item{C}{Monotonicity violation: sum of negative consecutive-class differences (C=0 means perfectly monotone)}
#'    }
#'    For binary data, computed directly from the FRP matrix (correct response rates).
#'    For ordinal data, computed from normalized expected scores: (E\[score\]-1)/(maxQ-1),
#'    which maps expected scores from \[1, maxQ\] to \[0, 1\] for comparability with the binary case.
#'    Not computed for nominal data (no natural category ordering).}
#'  \item{TRP}{Test Reference Profile - expected score for examinees in each class/rank}
#'  \item{CMD/RMD}{Class/Rank Membership Distribution - sum of membership probabilities across examinees}
#'  \item{FieldMembership}{Matrix showing the probabilities of each item belonging to each field}
#'  \item{ClassMembership}{Matrix showing the probabilities of each examinee belonging to each class/rank}
#'  \item{SmoothedMembership}{Matrix of smoothed class membership probabilities after filtering}
#'  \item{FieldEstimated}{Vector of the most likely field assignments for each item}
#'  \item{ClassEstimated}{Vector of the most likely class/rank assignments for each examinee}
#'  \item{Students}{Data frame containing membership probabilities and classification information for each examinee}
#'  \item{FieldAnalysis}{Matrix showing field analysis results with item-level information}
#'  \item{TestFitIndices}{Model fit indices for evaluating the quality of the clustering solution}
#'  \item{SOACflg}{Logical flag indicating whether Strongly Ordinal Alignment Condition is satisfied}
#'  \item{WOACflg}{Logical flag indicating whether Weakly Ordinal Alignment Condition is satisfied}
#' }
#'
#' @details
#' Biclustering simultaneously clusters both rows (examinees) and columns (items) of a data matrix.
#' Unlike traditional clustering that groups either rows or columns, biclustering identifies
#' submatrices with similar patterns. Ranklustering is a variant that imposes an ordinal
#' structure on the classes, making it suitable for proficiency scaling.
#'
#' The algorithm uses an Expectation-Maximization approach to iteratively estimate:
#' 1. Field membership of items (which items belong to which fields)
#' 2. Class/rank membership of examinees (which examinees belong to which classes)
#' 3. Field Reference Profiles (probability patterns for each field-class combination)
#'
#' The confirmatory option allows for pre-specified field assignments, which is useful
#' when there is prior knowledge about item groupings or for testing hypothesized structures.
#'
#' @references
#' Shojima, K. (2012). Biclustering of binary data matrices using bilinear models.
#' Behaviormetrika, 39(2), 161-178.
#'
#' @export

Biclustering <- function(U, ...) {
  UseMethod("Biclustering")
}

#' @rdname Biclustering
#' @param na Values to be treated as missing values.
#' @param Z Missing indicator matrix of type matrix or data.frame. 1 indicates observed values, 0 indicates missing values.
#' @param w Item weight vector.
#' @param ... Additional arguments passed to specific methods.
#'
#' @export
#'
Biclustering.default <- function(U, na = NULL, Z = NULL, w = NULL, ...) {
  if (inherits(U, "exametrika")) {
    if (U$response.type == "binary") {
      return(Biclustering.binary(U, ...))
    } else if (U$response.type == "ordinal") {
      return(Biclustering.ordinal(U, ...))
    } else if (U$response.type == "rated") {
      return(Biclustering.rated(U, ...))
    } else if (U$response.type == "nominal") {
      return(Biclustering.nominal(U, ...))
    }
  }

  U <- dataFormat(U, na = na, Z = Z, w = w)
  Biclustering(U, ...)
}


#' @rdname Biclustering
#' @param ncls Number of latent classes/ranks to identify (between 2 and 20).
#' @param nfld Number of latent fields (item clusters) to identify.
#' @param method Analysis method to use (character string):
#'   * "B" or "Biclustering": Standard biclustering (default)
#'   * "R" or "Ranklustering": Ranklustering with ordered class structure
#' @param conf Confirmatory parameter for pre-specified field assignments. Can be either:
#'   * A vector with items and corresponding fields in sequence
#'   * A field membership profile matrix (items × fields) with 0/1 values
#'   * NULL (default) for exploratory analysis where field memberships are estimated
#' @param conf_class Confirmatory parameter for pre-specified class assignments
#'   on the respondent side. Mirrors `conf` but operates on classes. Can be:
#'   * A vector with respondents and corresponding class indices
#'   * A class membership profile matrix (respondents × classes) with 0/1 values
#'   * NULL (default) for exploratory class estimation
#'   When supplied, class memberships are held fixed during EM. For Ranklustering
#'   (`method = "R"`), the neighbour-smoothing step is skipped because smoothing
#'   pre-specified labels would defeat the purpose of fixing them.
#' @param mic Logical; if TRUE, forces Field Reference Profiles to be monotonically
#' increasing. Default is FALSE.
#' @param maxiter Maximum number of EM algorithm iterations. Default is 100.
#' @param verbose Logical; if TRUE, displays progress during estimation. Default is TRUE.
#' @param beta1 Beta distribution parameter 1 for prior density of field reference matrix. Default is 1.
#' @param beta2 Beta distribution parameter 2 for prior density of field reference matrix. Default is 1.
#' @param ... Additional arguments passed to specific methods.
#'
#' @examples
#' \donttest{
#' # Perform Biclustering with Binary method (B)
#' # Analyze data with 5 fields and 6 classes
#' result.Bi <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
#'
#' # Perform Biclustering with Rank method (R)
#' # Store results for further analysis and visualization
#' result.Rank <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")
#'
#' # Display the Bicluster Reference Matrix (BRM) as a heatmap
#' plot(result.Rank, type = "Array")
#'
#' # Plot Field Reference Profiles (FRP) in a 2x3 grid
#' # Shows the probability patterns for each field
#' plot(result.Rank, type = "FRP", nc = 2, nr = 3)
#'
#' # Plot Rank Membership Profiles (RMP) for students 1-9 in a 3x3 grid
#' # Shows posterior probability distribution of rank membership
#' plot(result.Rank, type = "RMP", students = 1:9, nc = 3, nr = 3)
#'
#' # Example of confirmatory analysis with pre-specified fields
#' # Assign items 1-10 to field 1, 11-20 to field 2, etc.
#' field_assignments <- c(rep(1, 10), rep(2, 10), rep(3, 15))
#' result.Conf <- Biclustering(J35S515, nfld = 3, ncls = 5, conf = field_assignments)
#' }
#' @export
Biclustering.binary <- function(U,
                                ncls = 2, nfld = 2,
                                method = "B",
                                conf = NULL,
                                conf_class = NULL,
                                mic = FALSE,
                                maxiter = 100,
                                verbose = TRUE,
                                beta1 = 1,
                                beta2 = 1, ...) {
  tmp <- U
  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)
  const <- .Machine$double.eps
  ret.TS <- TestStatistics(tmp)

  if (method == "B" | method == "Biclustering") {
    if (verbose) {
      message("Biclustering is chosen.")
    }
    model <- 1
  } else if (method == "R" | method == "Ranklustering") {
    if (verbose) {
      message("Ranklustering is chosen.")
    }
    model <- 2
  } else if (method == "BINET") {
    if (verbose) {
      message("BINET is chosen.")
    }
    model <- 3
  } else {
    stop("The method must be selected as either Biclustering or Ranklustering.")
  }

  # set conf_mat for confirmatory clustering
  if (!is.null(conf)) {
    if (verbose) {
      message("Confirmatory Clustering is chosen.")
    }
    if (is.vector(conf)) {
      # check size
      if (length(conf) != NCOL(tmp$U)) {
        stop("conf vector size does NOT match with data.")
      }
      conf_mat <- matrix(0, nrow = NCOL(tmp$U), ncol = max(conf))
      for (i in 1:NROW(conf_mat)) {
        conf_mat[i, conf[i]] <- 1
      }
    } else if (is.matrix(conf) | is.data.frame(conf)) {
      if (NROW(conf) != NCOL(tmp$U)) {
        stop("conf matrix size does NOT match with data.")
      }
      if (any(!conf %in% c(0, 1))) {
        stop("The conf matrix should only contain 0s and 1s.")
      }
      if (any(rowSums(conf) > 1)) {
        stop("The row sums of the conf matrix must be equal to 1.")
      }
      conf_mat <- as.matrix(conf)
    } else {
      stop("conf matrix is not set properly.")
    }
    ###
    nfld <- NCOL(conf_mat)
  } else {
    conf_mat <- NULL
  }

  # set conf_class_mat for class-side confirmatory clustering
  if (!is.null(conf_class)) {
    if (verbose) {
      message("Class-side Confirmatory Clustering is chosen.")
    }
    if (is.vector(conf_class)) {
      if (length(conf_class) != nobs) {
        stop("conf_class vector size does NOT match with the number of respondents.")
      }
      conf_class_mat <- matrix(0, nrow = nobs, ncol = max(conf_class))
      for (i in 1:NROW(conf_class_mat)) {
        conf_class_mat[i, conf_class[i]] <- 1
      }
    } else if (is.matrix(conf_class) | is.data.frame(conf_class)) {
      if (NROW(conf_class) != nobs) {
        stop("conf_class matrix size does NOT match with the number of respondents.")
      }
      if (any(!conf_class %in% c(0, 1))) {
        stop("The conf_class matrix should only contain 0s and 1s.")
      }
      if (any(rowSums(conf_class) > 1)) {
        stop("The row sums of the conf_class matrix must be equal to 1.")
      }
      conf_class_mat <- as.matrix(conf_class)
    } else {
      stop("conf_class is not set properly.")
    }
    ncls <- NCOL(conf_class_mat)
  } else {
    conf_class_mat <- NULL
  }

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  if (model == 3) {
    zero_scorer <- ifelse(ret.TS$Min == 0, 1, 0)
    full_scorer <- ifelse(ret.TS$Max == testlength, 1, 0)
    if (ncls < zero_scorer + full_scorer + 1) {
      stop(paste(
        "The number of class must be more than ",
        zero_scorer + full_scorer + 1, "."
      ))
    }
  }

  ### Algorithm
  test_log_lik <- -1 / const
  old_test_log_lik <- -2 / const
  emt <- 0
  maxemt <- 100

  fld0 <- pmin(ceiling(1:testlength / (testlength / nfld)), nfld)
  crr_order <- order(crr(tmp), decreasing = TRUE)
  fld <- fld0[match(1:testlength, crr_order)]
  fldmemb <- matrix(0, nrow = testlength, ncol = nfld)
  for (i in 1:testlength) {
    fldmemb[i, fld[i]] <- 1
  }
  ## Confirmatory Biclustering
  if (!any(is.null(conf_mat))) {
    fldmemb <- conf_mat
  }

  PiFR <- matrix(NA, nrow = nfld, ncol = ncls)
  for (i in 1:nfld) {
    for (j in 1:ncls) {
      PiFR[i, j] <- (nfld - i + j) / (nfld + ncls)
    }
  }
  # For BINET
  if (model == 3) {
    if (zero_scorer == 1) {
      PiFR[, 1] <- 0
    }
    if (full_scorer == 1) {
      PiFR[, ncls] <- 1
    }
  }

  if (model != 2) {
    Fil <- diag(rep(1, ncls))
  } else {
    Fil <- create_filter_matrix(ncls)
  }

  ## Algorithm
  FLG <- TRUE
  converge <- TRUE
  while (FLG) {
    if (test_log_lik - old_test_log_lik < 1e-4 * abs(old_test_log_lik)) {
      FLG <- FALSE
      break
    }
    if (emt == maxemt) {
      message("\nReached ten times the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
      FLG <- FALSE
      converge <- FALSE
    }
    emt <- emt + 1
    old_test_log_lik <- test_log_lik
    csr <- (tmp$Z * tmp$U) %*% fldmemb
    fsr <- (tmp$Z * (1 - tmp$U)) %*% fldmemb
    # Apply pmax only when necessary to avoid NaN/Inf
    if (any(is.nan(csr)) || any(is.infinite(csr)) || any(csr <= 0)) {
      csr <- pmax(csr, const)
    }
    if (any(is.nan(fsr)) || any(is.infinite(fsr)) || any(fsr <= 0)) {
      fsr <- pmax(fsr, const)
    }
    llsr <- csr %*% log(PiFR + const) + fsr %*% log(1 - PiFR + const)
    # minllsr <- apply(llsr, 1, min)
    # expllsr <- exp(llsr - minllsr)
    # clsmemb <- round(expllsr / rowSums(expllsr), 1e8)
    clsmemb <- t(apply(llsr, 1, softmax))

    if (!is.null(conf_class_mat)) {
      clsmemb <- conf_class_mat
      smoothed_memb <- clsmemb
    } else {
      smoothed_memb <- clsmemb %*% Fil
    }

    cjr <- t(tmp$Z * tmp$U) %*% smoothed_memb
    fjr <- t(tmp$Z * (1 - tmp$U)) %*% smoothed_memb
    # Apply pmax only when necessary to avoid NaN/Inf
    if (any(is.nan(cjr)) || any(is.infinite(cjr)) || any(cjr <= 0)) {
      cjr <- pmax(cjr, const)
    }
    if (any(is.nan(fjr)) || any(is.infinite(fjr)) || any(fjr <= 0)) {
      fjr <- pmax(fjr, const)
    }
    lljf <- cjr %*% log(t(PiFR) + const) + fjr %*% log(t(1 - PiFR) + const)

    # max_log_lljf <- apply(lljf, 1, max)
    # log_lljf_adj <- lljf - max_log_lljf
    # log_fldmemb <- log_lljf_adj - log(rowSums(exp(log_lljf_adj)))
    fldmemb <- t(apply(lljf, 1, softmax))

    if (!any(is.null(conf_mat))) {
      fldmemb <- conf_mat
    }

    cfr <- t(fldmemb) %*% t(tmp$Z * tmp$U) %*% smoothed_memb
    ffr <- t(fldmemb) %*% t(tmp$Z * (1 - tmp$U)) %*% smoothed_memb
    # Apply pmax only when necessary to avoid NaN/Inf
    if (any(is.nan(cfr)) || any(is.infinite(cfr)) || any(cfr <= 0)) {
      cfr <- pmax(cfr, const)
    }
    if (any(is.nan(ffr)) || any(is.infinite(ffr)) || any(ffr <= 0)) {
      ffr <- pmax(ffr, const)
    }
    oldPiFR <- PiFR
    PiFR <- (cfr + beta1 - 1) / (cfr + ffr + beta1 + beta2 - 2)
    PiFR[is.nan(PiFR)] <- 1e-16
    PiFR[is.infinite(PiFR)] <- 1e-16
    PiFR <- pmax(pmin(PiFR, 1 - 1e-16), 1e-16)
    if (model == 3) {
      if (zero_scorer == 1) {
        PiFR[, 1] <- 0
      }
      if (full_scorer == 1) {
        PiFR[, ncls] <- 1
      }
    }
    if (mic) {
      PiFR <- t(apply(PiFR, 1, sort))
    }
    if (any(is.nan(cfr))) {
      stop("The calculation diverged during the process. Please adjust your settings appropriately")
    }

    test_log_lik <- sum(cfr * log(PiFR + const) + ffr * log(1 - PiFR + const))
    if (verbose) {
      message(
        sprintf(
          "\r%-80s",
          paste0(
            "iter ", emt, " log_lik ", format(test_log_lik, digits = 6)
          )
        ),
        appendLF = FALSE
      )
    }
    if (test_log_lik - old_test_log_lik <= 0) {
      PiFR <- oldPiFR
      break
    }
  }

  # Ensure proper line termination after verbose output
  if (verbose) {
    message("")
  }

  #### OUTPUT

  cls <- apply(clsmemb, 1, which.max)
  fld <- apply(fldmemb, 1, which.max)
  check_empty_fields(fld, nfld)
  fldmemb01 <- sign(fldmemb - apply(fldmemb, 1, max)) + 1
  flddist <- colSums(fldmemb01)
  clsmemb01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  clsdist <- colSums(clsmemb01)
  TRP <- colSums(PiFR * flddist)
  StudentRank <- clsmemb
  rownames(StudentRank) <- tmp$ID
  if (model == 2) {
    RU <- ifelse(cls + 1 > ncls, NA, cls + 1)
    RD <- ifelse(cls - 1 < 1, NA, cls - 1)
    RUO <- StudentRank[cbind(1:nobs, RU)] / StudentRank[cbind(1:nobs, cls)]
    RDO <- StudentRank[cbind(1:nobs, RD)] / StudentRank[cbind(1:nobs, cls)]
    StudentRank <- cbind(StudentRank, cls, RUO, RDO)
    colnames(StudentRank) <- c(
      paste("Membership", 1:ncls), "Estimate",
      "Rank-Up Odds", "Rank-Down Odds"
    )
  } else {
    StudentRank <- cbind(StudentRank, cls)
    colnames(StudentRank) <- c(
      paste("Membership", 1:ncls), "Estimate"
    )
  }

  if (model == 2) {
    msg1 <- "Rank"
  } else {
    msg1 <- "Class"
  }
  FRP <- PiFR
  colnames(FRP) <- paste0(msg1, 1:ncls)
  rownames(FRP) <- paste0("Field", 1:nfld)
  colnames(fldmemb) <- paste0("Field", 1:nfld)
  rownames(clsmemb) <- tmp$ID
  colnames(clsmemb) <- paste0(msg1, 1:ncls)

  FRPIndex <- IRPindex(FRP)

  TRPlag <- TRP[2:ncls]
  TRPmic <- sum(TRPlag[1:(ncls - 1)] - TRP[1:(ncls - 1)] < 0, na.rm = TRUE)
  FRPmic <- sum(abs(FRPIndex$C))
  SOACflg <- WOACflg <- FALSE
  if (TRPmic == 0) {
    WOACflg <- TRUE
    if (FRPmic == 0) {
      SOACflg <- TRUE
    }
  }
  if (verbose) {
    if (SOACflg & WOACflg) {
      message("\nStrongly ordinal alignment condition was satisfied.")
    }
    if (!SOACflg & WOACflg) {
      message("\nWeakly ordinal alignment condition was satisfied.")
    }
  }

  ### Model Fit
  cfr <- t(fldmemb) %*% t(tmp$Z * tmp$U) %*% clsmemb
  ffr <- t(fldmemb) %*% t(tmp$Z * (1 - tmp$U)) %*% clsmemb
  test_log_lik <- sum(cfr * log(PiFR + const) + ffr * log(1 - PiFR + const))
  nparam <- ifelse(model == 1, ncls * nfld, sum(diag(Fil)) * nfld)
  FitIndices <- TestFit(tmp$U, tmp$Z, test_log_lik, nparam)

  ### Field Analysis
  crr <- crr(tmp$Z * tmp$U)
  fieldAnalysis <- as.data.frame(fldmemb)
  fieldAnalysis <- cbind(crr, fld, fieldAnalysis)
  colnames(fieldAnalysis) <- c("CRR", "LFE", paste0("Field", 1:nfld))
  fieldAnalysis <- fieldAnalysis[order(fieldAnalysis$CRR, decreasing = TRUE), ]
  fieldAnalysis <- fieldAnalysis[order(fieldAnalysis$LFE), ]
  rownames_tmp <- rownames(fieldAnalysis)
  fieldAnalysis <- matrix(as.numeric(as.matrix(fieldAnalysis)), ncol = NCOL(fieldAnalysis), nrow = NROW(fieldAnalysis))
  colnames(fieldAnalysis) <- c("CRR", "LFE", paste0("Field", 1:nfld))
  rownames(fieldAnalysis) <- rownames_tmp

  msg <- ifelse(model == 1, "Class", "Rank")
  ret <- structure(list(
    model = model,
    mic = mic,
    msg = msg,
    converge = converge,
    U = U,
    testlength = testlength,
    nobs = nobs,
    n_class = ncls, # New naming convention
    n_field = nfld, # New naming convention
    n_cycle = emt, # New naming convention
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = FRP,
    FRPIndex = FRPIndex,
    TRP = TRP,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    SmoothedMembership = smoothed_memb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    FieldAnalysis = fieldAnalysis,
    TestFitIndices = FitIndices,
    log_lik = test_log_lik,
    SOACflg = SOACflg,
    WOACflg = WOACflg,
    # Deprecated fields (for backward compatibility)
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = emt
  ), class = c("exametrika", "Biclustering"))
  return(ret)
}

library(exametrika)

### GOALS - Mathematica reference data (Biclustering)
test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Biclustering_Test.csv"),
  check.names = FALSE
)
Bicluster <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Biclustering_Bicluster.csv"),
  check.names = FALSE
)
items <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Biclustering_Item.csv"),
  check.names = FALSE
)
student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Biclustering_Student.csv"),
  check.names = FALSE
)


### Setup (Biclustering)
tmp <- dataFormat(J35S515)
Bic <- Biclustering(tmp, ncls = 6, nfld = 5, method = "B", mic = TRUE)

### Tests (Biclustering)
test_that("Biclustering Test Fit", {
  expect <- test[15:30, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- Bic$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


test_that("Biclustering Class Info", {
  ## FRP
  expect <- Bicluster[1:5, 2:7] |>
    unlist() |>
    unname() |>
    as.vector()
  result <- Bic$FRP |>
    unname() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  ## TRP
  expect <- Bicluster[6, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$TRP
  expect_equal(result, expect, tolerance = 1e-4)
  ## LCD
  expect <- Bicluster[7, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$LRD
  expect_equal(result, expect, tolerance = 1e-4)
  ## CMD
  expect <- Bicluster[8, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$ClassMembership |>
    colSums() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})


test_that("Biclustering Item Info", {
  ## IRP
  expect <- items[, 6:10] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$FieldMembership |>
    unlist() |>
    unname() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## Estimated
  expect <- items[, 11] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$FieldEstimated |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("Biclustering Students", {
  ## Membership
  expect <- student[, 6:11] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$Students[, 1:6] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Ranklustering ---------------------------------------------------

test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Ranklustering_Test.csv"),
  check.names = FALSE
)
Rankluster <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Ranklustering_Rankluster.csv"),
  check.names = FALSE
)
items <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Ranklustering_Item.csv"),
  check.names = FALSE
)
student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter07Ranklustering_Student.csv"),
  check.names = FALSE
)


### Setup (Ranklustering)
tmp <- dataFormat(J35S515)
Bic <- Biclustering(tmp, ncls = 6, nfld = 5, method = "R", mic = TRUE)

### Tests (Ranklustering)
test_that("Ranklustering Test Fit", {
  expect <- test[15:30, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- Bic$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


test_that("Ranklustering Class Info", {
  ## FRP
  expect <- Rankluster[1:5, 2:7] |>
    unlist() |>
    unname() |>
    as.vector()
  result <- Bic$FRP |>
    unname() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  ## TRP
  expect <- Rankluster[6, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$TRP
  expect_equal(result, expect, tolerance = 1e-4)
  ## LCD
  expect <- Rankluster[7, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$LRD
  expect_equal(result, expect, tolerance = 1e-4)
  ## CMD
  expect <- Rankluster[8, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$ClassMembership |>
    colSums() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  ## Index
  expect <- Rankluster[1:5, 9:14] |>
    unlist() |>
    unname() |>
    as.vector()
  result <- Bic$FRPIndex |>
    unlist() |>
    unname() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("Ranklustering Item Info", {
  ## IRP
  expect <- items[, 6:10] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$FieldMembership |>
    unlist() |>
    unname() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## Estimated
  expect <- items[, 11] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$FieldEstimated |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("Ranklustering Students", {
  ## Membership
  expect <- student[, 6:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- Bic$Students[, 1:9] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


### Confirmatory Biclustering (binary)
conf_b <- c(rep(1, 7), rep(2, 7), rep(3, 7), rep(4, 7), rep(5, 7))
Bic_conf <- Biclustering(J35S515, ncls = 5, nfld = 5, method = "B", conf = conf_b, verbose = FALSE)

test_that("binary confirmatory respects field assignments", {
  expect_equal(as.numeric(Bic_conf$FieldEstimated), conf_b)
})

test_that("binary confirmatory rejects wrong-length conf vector", {
  expect_error(
    Biclustering(J35S515,
      ncls = 5, nfld = 5, method = "B",
      conf = rep(1:5, each = 6), verbose = FALSE
    ),
    "conf vector size does NOT match"
  )
})

test_that("binary confirmatory accepts membership matrix", {
  conf_mat <- matrix(0, nrow = 35, ncol = 5)
  for (i in seq_len(35)) conf_mat[i, conf_b[i]] <- 1
  res <- Biclustering(J35S515,
    ncls = 5, nfld = 5, method = "B",
    conf = conf_mat, verbose = FALSE
  )
  expect_equal(as.numeric(res$FieldEstimated), conf_b)
})

test_that("binary confirmatory rejects wrong-row matrix", {
  bad <- matrix(0, nrow = 30, ncol = 5)
  bad[cbind(seq_len(30), rep(1:5, each = 6))] <- 1
  expect_error(
    Biclustering(J35S515,
      ncls = 5, nfld = 5, method = "B",
      conf = bad, verbose = FALSE
    ),
    "conf matrix size does NOT match"
  )
})


### Class-side Confirmatory Biclustering (binary)
nobs_b <- NROW(dataFormat(J35S515)$U)
conf_class_b <- ((seq_len(nobs_b) - 1) %% 5) + 1

test_that("binary B class-side confirmatory respects class assignments", {
  res <- Biclustering(J35S515,
    ncls = 5, nfld = 5, method = "B",
    conf_class = conf_class_b, verbose = FALSE
  )
  est <- apply(res$ClassMembership, 1, which.max)
  expect_equal(as.numeric(est), conf_class_b)
})

test_that("binary R class-side confirmatory respects class assignments and skips smoothing", {
  res <- Biclustering(J35S515,
    ncls = 5, nfld = 5, method = "R",
    conf_class = conf_class_b, verbose = FALSE
  )
  est <- apply(res$ClassMembership, 1, which.max)
  expect_equal(as.numeric(est), conf_class_b)
  expect_equal(
    as.numeric(res$SmoothedMembership),
    as.numeric(res$ClassMembership)
  )
})

test_that("binary class-side confirmatory rejects wrong-length conf_class", {
  expect_error(
    Biclustering(J35S515,
      ncls = 5, nfld = 5, method = "B",
      conf_class = 1:10, verbose = FALSE
    ),
    "conf_class vector size does NOT match"
  )
})

test_that("binary class-side confirmatory accepts membership matrix", {
  cm <- matrix(0, nrow = nobs_b, ncol = 5)
  for (i in seq_len(nobs_b)) cm[i, conf_class_b[i]] <- 1
  res <- Biclustering(J35S515,
    ncls = 5, nfld = 5, method = "B",
    conf_class = cm, verbose = FALSE
  )
  est <- apply(res$ClassMembership, 1, which.max)
  expect_equal(as.numeric(est), conf_class_b)
})

test_that("binary conf and conf_class can be combined", {
  res <- Biclustering(J35S515,
    ncls = 5, nfld = 5, method = "B",
    conf = conf_b,
    conf_class = conf_class_b, verbose = FALSE
  )
  est <- apply(res$ClassMembership, 1, which.max)
  expect_equal(as.numeric(res$FieldEstimated), conf_b)
  expect_equal(as.numeric(est), conf_class_b)
})

library(exametrika)

### DistractorAnalysis Tests
### Data: J21S300 (rated, 21 items, 300 students, 4 categories)

### Setup - run models once and share across tests
result_lra <- LRA(J21S300, nrank = 5, mic = TRUE, verbose = FALSE)
da_lra <- DistractorAnalysis(result_lra)

result_bic <- Biclustering(J21S300,
  ncls = 5, nfld = 3, method = "R",
  maxiter = 300, verbose = FALSE
)
da_bic <- DistractorAnalysis(result_bic)

# === LRA.rated tests =====================================================

test_that("DistractorAnalysis.LRArated Basic Structure", {
  expect_s3_class(da_lra, "DistractorAnalysis")
  expect_true("exametrika" %in% class(da_lra))

  expect_true(!is.null(da_lra$freq_table))
  expect_true(!is.null(da_lra$prop_table))
  expect_true(!is.null(da_lra$chisq_table))
  expect_true(!is.null(da_lra$pvalue_table))
  expect_true(!is.null(da_lra$cramersv_table))
  expect_true(!is.null(da_lra$CA))
  expect_true(!is.null(da_lra$n_rank))
  expect_true(!is.null(da_lra$maxQ))
  expect_true(!is.null(da_lra$nitems))
  expect_true(!is.null(da_lra$ItemLabel))
  expect_true(!is.null(da_lra$msg))

  # LRA should NOT have field info
  expect_null(da_lra$n_field)
  expect_null(da_lra$FieldEstimated)
  expect_null(da_lra$field_items)
})

test_that("DistractorAnalysis.LRArated Dimensions", {
  expect_equal(da_lra$nitems, 21)
  expect_equal(da_lra$n_rank, 5)
  expect_equal(da_lra$maxQ, 4)

  expect_equal(length(da_lra$freq_table), 21)
  expect_equal(length(da_lra$prop_table), 21)
  expect_equal(dim(da_lra$chisq_table), c(21, 5))
  expect_equal(dim(da_lra$pvalue_table), c(21, 5))
  expect_equal(dim(da_lra$cramersv_table), c(21, 5))
  expect_equal(length(da_lra$CA), 21)
  expect_equal(length(da_lra$ItemLabel), 21)
})

test_that("DistractorAnalysis.LRArated Frequency Table Structure", {
  for (j in 1:da_lra$nitems) {
    mat <- da_lra$freq_table[[j]]
    expect_equal(dim(mat), c(5, 4))
    expect_true(all(mat >= 0))
    expect_true(all(mat == round(mat))) # integers
  }
})

test_that("DistractorAnalysis.LRArated Proportion Table Sums to 1", {
  for (j in 1:da_lra$nitems) {
    row_sums <- unname(rowSums(da_lra$prop_table[[j]]))
    for (r in seq_along(row_sums)) {
      if (sum(da_lra$freq_table[[j]][r, ]) > 0) {
        expect_equal(row_sums[r], 1.0, tolerance = 1e-10)
      }
    }
  }
})

test_that("DistractorAnalysis.LRArated Proportions in Valid Range", {
  for (j in 1:da_lra$nitems) {
    expect_true(all(da_lra$prop_table[[j]] >= 0))
    expect_true(all(da_lra$prop_table[[j]] <= 1))
  }
})

test_that("DistractorAnalysis.LRArated Chi-square Values Are Non-negative", {
  expect_true(all(da_lra$chisq_table >= 0, na.rm = TRUE))
})

test_that("DistractorAnalysis.LRArated P-values in Valid Range", {
  expect_true(all(da_lra$pvalue_table >= 0, na.rm = TRUE))
  expect_true(all(da_lra$pvalue_table <= 1, na.rm = TRUE))
})

test_that("DistractorAnalysis.LRArated Cramer's V in Valid Range", {
  expect_true(all(da_lra$cramersv_table >= 0, na.rm = TRUE))
  expect_true(all(da_lra$cramersv_table <= 1, na.rm = TRUE))
})

test_that("DistractorAnalysis.LRArated CA Values Are Valid", {
  expect_true(all(da_lra$CA >= 1))
  expect_true(all(da_lra$CA <= da_lra$maxQ))
})

# === ratedBiclustering tests =============================================

test_that("DistractorAnalysis.ratedBiclustering Basic Structure", {
  expect_s3_class(da_bic, "DistractorAnalysis")
  expect_true("exametrika" %in% class(da_bic))

  # Should have field info
  expect_true(!is.null(da_bic$n_field))
  expect_true(!is.null(da_bic$FieldEstimated))
  expect_true(!is.null(da_bic$field_items))
})

test_that("DistractorAnalysis.ratedBiclustering Dimensions", {
  expect_equal(da_bic$nitems, 21)
  expect_equal(da_bic$maxQ, 4)

  expect_equal(length(da_bic$freq_table), 21)
  expect_equal(length(da_bic$prop_table), 21)
  expect_equal(dim(da_bic$chisq_table), c(21, da_bic$n_rank))
  expect_equal(dim(da_bic$pvalue_table), c(21, da_bic$n_rank))
  expect_equal(dim(da_bic$cramersv_table), c(21, da_bic$n_rank))
})

test_that("DistractorAnalysis.ratedBiclustering Field Info", {
  nfld <- da_bic$n_field
  expect_true(nfld >= 1)
  expect_equal(length(da_bic$field_items), nfld)
  expect_equal(length(da_bic$FieldEstimated), 21)

  # All items covered by fields
  all_items <- sort(unname(unlist(da_bic$field_items)))
  expect_equal(all_items, 1:21)

  # FieldEstimated values match field_items
  for (f in 1:nfld) {
    expect_true(all(da_bic$FieldEstimated[da_bic$field_items[[f]]] == f))
  }
})

test_that("DistractorAnalysis.ratedBiclustering Frequency Table Structure", {
  ncls <- da_bic$n_rank
  maxQ <- da_bic$maxQ
  for (j in 1:da_bic$nitems) {
    mat <- da_bic$freq_table[[j]]
    expect_equal(dim(mat), c(ncls, maxQ))
    expect_true(all(mat >= 0))
    expect_true(all(mat == round(mat)))
  }
})

test_that("DistractorAnalysis.ratedBiclustering Proportion Table Sums to 1", {
  for (j in 1:da_bic$nitems) {
    row_sums <- unname(rowSums(da_bic$prop_table[[j]]))
    for (r in seq_along(row_sums)) {
      if (sum(da_bic$freq_table[[j]][r, ]) > 0) {
        expect_equal(row_sums[r], 1.0, tolerance = 1e-10)
      }
    }
  }
})

test_that("DistractorAnalysis.ratedBiclustering Statistics in Valid Range", {
  expect_true(all(da_bic$chisq_table >= 0, na.rm = TRUE))
  expect_true(all(da_bic$pvalue_table >= 0, na.rm = TRUE))
  expect_true(all(da_bic$pvalue_table <= 1, na.rm = TRUE))
  expect_true(all(da_bic$cramersv_table >= 0, na.rm = TRUE))
  expect_true(all(da_bic$cramersv_table <= 1, na.rm = TRUE))
})

# === Print filtering tests ===============================================

test_that("DistractorAnalysis print with items filter works", {
  expect_output(print(da_lra, items = 1:3), "Item01")
  expect_output(print(da_lra, items = 1:3), "Item03")
})

test_that("DistractorAnalysis print with ranks filter works", {
  expect_output(print(da_lra, ranks = c(1, 5)), "Rank1")
  expect_output(print(da_lra, ranks = c(1, 5)), "Rank5")
})

test_that("DistractorAnalysis print with both filters works", {
  expect_output(print(da_lra, items = 10, ranks = c(1, 5)), "Item10")
})

test_that("DistractorAnalysis print shows field grouping for Biclustering", {
  expect_output(print(da_bic), "Field Grouping")
  expect_output(print(da_bic), "Field 1")
})

test_that("DistractorAnalysis print does not show field grouping for LRA", {
  output <- capture.output(print(da_lra, items = 1))
  expect_false(any(grepl("Field Grouping", output)))
})

# === Plot tests ==========================================================

test_that("DistractorAnalysis plot runs without error", {
  expect_no_error(plot(da_lra, items = 1))
  expect_no_error(plot(da_bic, items = 1))
})

test_that("DistractorAnalysis plot with ranks filter works", {
  expect_no_error(plot(da_lra, items = 1, ranks = c(1, 5)))
  expect_no_error(plot(da_bic, items = 1:4, ranks = c(1, 3, 5), nc = 2, nr = 2))
})

# === Biclustering_IRM.rated test =========================================

test_that("DistractorAnalysis works with Biclustering_IRM.rated", {
  result_irm <- Biclustering_IRM(J21S300,
    gamma_c = 1, gamma_f = 1,
    seed = 123, verbose = FALSE
  )
  da_irm <- DistractorAnalysis(result_irm)

  expect_s3_class(da_irm, "DistractorAnalysis")
  expect_true(!is.null(da_irm$n_field))
  expect_true(!is.null(da_irm$field_items))
  expect_equal(da_irm$nitems, 21)
  expect_equal(da_irm$maxQ, 4)
  expect_equal(length(da_irm$CA), 21)
})

# === LRA.rated with different nrank ======================================

test_that("DistractorAnalysis adapts to nrank=4 LRA results", {
  result_lra4 <- LRA(J21S300, nrank = 4, mic = TRUE, verbose = FALSE)
  da4 <- DistractorAnalysis(result_lra4)
  expect_equal(da4$n_rank, 4)
  expect_equal(dim(da4$chisq_table), c(21, 4))
  expect_equal(dim(da4$pvalue_table), c(21, 4))
  expect_equal(dim(da4$cramersv_table), c(21, 4))
  for (j in 1:da4$nitems) {
    expect_equal(dim(da4$freq_table[[j]]), c(4, 4))
  }
})

library(exametrika)

# ================================================================
# J35S500: ordinal Biclustering (Ranklustering)
# 理論構造: 累積パターン, 5cls × 5fld × 5cat
# ================================================================

result_ord <- Biclustering(J35S500, ncls = 5, nfld = 5, method = "R", mic = TRUE, maxiter = 200)

test_that("ordinal Biclustering converges", {
  expect_true(result_ord$converge)
})

test_that("ordinal Biclustering FRP dimensions", {
  expect_equal(dim(result_ord$FRP), c(5, 5, 5))
})

test_that("ordinal Biclustering FRP probabilities sum to 1", {
  nfld <- dim(result_ord$FRP)[1]
  ncls <- dim(result_ord$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_ord$FRP[f, c, ]), 1.0, tolerance = 1e-10)
    }
  }
})

test_that("ordinal Biclustering expected scores in valid range", {
  nfld <- dim(result_ord$FRP)[1]
  ncls <- dim(result_ord$FRP)[2]
  maxQ <- dim(result_ord$FRP)[3]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      esp <- sum((1:maxQ) * result_ord$FRP[f, c, ])
      expect_gte(esp, 1.0)
      expect_lte(esp, maxQ)
    }
  }
})

test_that("ordinal Biclustering shows score differentiation per field", {
  # 各フィールドにおいて、クラス間で期待得点に差がある（max - min > 0.5）
  nfld <- dim(result_ord$FRP)[1]
  ncls <- dim(result_ord$FRP)[2]
  maxQ <- dim(result_ord$FRP)[3]
  for (f in 1:nfld) {
    esp <- numeric(ncls)
    for (c in 1:ncls) {
      esp[c] <- sum((1:maxQ) * result_ord$FRP[f, c, ])
    }
    expect_gt(max(esp) - min(esp), 0.5)
  }
})

test_that("ordinal Biclustering field/class counts", {
  expect_equal(sum(result_ord$LFD), 35)
  expect_equal(sum(result_ord$LCD), 500)
  expect_equal(length(result_ord$LFD), 5)
  expect_equal(length(result_ord$LCD), 5)
})

test_that("ordinal Biclustering fit indices", {
  expect_equal(result_ord$TestFitIndices$RMSEA, 0.0533156, tolerance = 1e-3)
  expect_equal(result_ord$TestFitIndices$AIC, 7287.635, tolerance = 1)
  expect_equal(result_ord$TestFitIndices$BIC, -66115.85, tolerance = 1)
})


# ================================================================
# J20S600: nominal Biclustering
# 理論構造: 循環パターン, 5cls × 4fld × 4cat
# ================================================================

result_nom <- Biclustering(J20S600, ncls = 5, nfld = 4, maxiter = 200)

test_that("nominal Biclustering converges", {
  expect_true(result_nom$converge)
})

test_that("nominal Biclustering FRP dimensions", {
  expect_equal(dim(result_nom$FRP), c(4, 5, 4))
})

test_that("nominal Biclustering FRP probabilities sum to 1", {
  nfld <- dim(result_nom$FRP)[1]
  ncls <- dim(result_nom$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_nom$FRP[f, c, ]), 1.0, tolerance = 1e-10)
    }
  }
})

test_that("nominal Biclustering field distribution is uniform", {
  # 理論値: 各フィールドに5項目ずつ
  expect_equal(result_nom$LFD, rep(5, 4))
})

test_that("nominal Biclustering class count sums to nobs", {
  expect_equal(sum(result_nom$LCD), 600)
  expect_equal(length(result_nom$LCD), 5)
})

test_that("nominal Biclustering shows cyclic mode pattern", {
  # 各フィールドで、クラス間の最頻カテゴリが多様（少なくとも3種類以上）
  nfld <- dim(result_nom$FRP)[1]
  ncls <- dim(result_nom$FRP)[2]
  for (f in 1:nfld) {
    modes <- integer(ncls)
    for (c in 1:ncls) {
      modes[c] <- which.max(result_nom$FRP[f, c, ])
    }
    expect_gte(length(unique(modes)), 3)
  }
})

test_that("nominal Biclustering fit indices", {
  # Nominal data has no benchmark model; chi-square based indices are NA
  expect_true(is.na(result_nom$TestFitIndices$bench_log_like))
  expect_true(is.na(result_nom$TestFitIndices$RMSEA))
  expect_true(is.na(result_nom$TestFitIndices$NFI))
  expect_true(is.na(result_nom$TestFitIndices$CFI))
  # Information criteria are computed directly from log-likelihood
  expect_equal(result_nom$TestFitIndices$AIC, 27943.98, tolerance = 1)
  expect_equal(result_nom$TestFitIndices$BIC, 28260.56, tolerance = 1)
})


# ================================================================
# ordinal Biclustering: FRPIndex テスト
# ================================================================

test_that("ordinal Biclustering FRPIndex exists and has correct dimensions", {
  expect_false(is.null(result_ord$FRPIndex))
  # FRPIndex: nfld rows × 6 columns (Alpha, A, Beta, B, Gamma, C)
  expect_equal(nrow(result_ord$FRPIndex), 5)
  expect_equal(ncol(result_ord$FRPIndex), 6)
  expect_equal(names(result_ord$FRPIndex), c("Alpha", "A", "Beta", "B", "Gamma", "C"))
})

test_that("ordinal Biclustering FRPIndex values are valid", {
  # Alpha: location of steepest ascent (integer, 0 to ncls-1)
  expect_true(all(result_ord$FRPIndex$Alpha >= 0))
  expect_true(all(result_ord$FRPIndex$Alpha < 5))
  # A: maximum slope (positive)
  expect_true(all(result_ord$FRPIndex$A > 0))
  # B: normalized expected score at Beta (0 to 1)
  expect_true(all(result_ord$FRPIndex$B >= 0))
  expect_true(all(result_ord$FRPIndex$B <= 1))
  # Gamma: non-monotonicity index (0 to 1)
  expect_true(all(result_ord$FRPIndex$Gamma >= 0))
  expect_true(all(result_ord$FRPIndex$Gamma <= 1))
})

test_that("ordinal Biclustering SOACflg exists", {
  expect_true(is.logical(result_ord$SOACflg))
})


# ================================================================
# ordinal Biclustering: プロットタイプテスト
# ================================================================

test_that("ordinal Biclustering FRP plot works", {
  expect_no_error(plot(result_ord, type = "FRP", nc = 3, nr = 2))
})

test_that("ordinal Biclustering FRP plot with stat=median works", {
  expect_no_error(plot(result_ord, type = "FRP", nc = 3, nr = 2, stat = "median"))
})

test_that("ordinal Biclustering FRP plot with stat=mode works", {
  expect_no_error(plot(result_ord, type = "FRP", nc = 3, nr = 2, stat = "mode"))
})

test_that("ordinal Biclustering FCRP plot (line) works", {
  expect_no_error(plot(result_ord, type = "FCRP", nc = 3, nr = 2))
})

test_that("ordinal Biclustering FCRP plot (bar) works", {
  expect_no_error(plot(result_ord, type = "FCRP", nc = 3, nr = 2, style = "bar"))
})

test_that("ordinal Biclustering FCBR plot works", {
  expect_no_error(plot(result_ord, type = "FCBR", nc = 3, nr = 2))
})

test_that("ordinal Biclustering ScoreField plot works", {
  expect_no_error(plot(result_ord, type = "ScoreField"))
})

test_that("ordinal Biclustering RRV plot works", {
  expect_no_error(plot(result_ord, type = "RRV"))
})

test_that("ordinal Biclustering RRV plot with stat=median works", {
  expect_no_error(plot(result_ord, type = "RRV", stat = "median"))
})

test_that("ordinal Biclustering LCD plot works", {
  expect_no_error(plot(result_ord, type = "LCD"))
})

test_that("ordinal Biclustering RMP plot works", {
  expect_no_error(plot(result_ord, type = "RMP", students = 1:3, nc = 3))
})


# ================================================================
# nominal Biclustering: プロットタイプテスト
# ================================================================

test_that("nominal Biclustering FRP plot works", {
  expect_no_error(plot(result_nom, type = "FRP", nc = 2, nr = 2))
})

test_that("nominal Biclustering FCRP plot (line) works", {
  expect_no_error(plot(result_nom, type = "FCRP", nc = 2, nr = 2))
})

test_that("nominal Biclustering FCRP plot (bar) works", {
  expect_no_error(plot(result_nom, type = "FCRP", nc = 2, nr = 2, style = "bar"))
})

test_that("nominal Biclustering ScoreField plot works", {
  expect_no_error(plot(result_nom, type = "ScoreField"))
})

test_that("nominal Biclustering RRV plot works", {
  expect_no_error(plot(result_nom, type = "RRV"))
})

test_that("nominal Biclustering LCD plot works", {
  expect_no_error(plot(result_nom, type = "LCD"))
})

test_that("nominal Biclustering FCBR is not allowed", {
  expect_error(plot(result_nom, type = "FCBR"))
})


# ================================================================
# J35S5000: rated Biclustering (Ranklustering)
# rated = nominal推定 + 正答率によるクラスソート + 二値/名義二層適合度
# ================================================================

result_rated <- Biclustering(J35S5000, ncls = 3, nfld = 3, method = "R", maxiter = 200)

test_that("rated Biclustering returns correct class", {
  expect_s3_class(result_rated, "exametrika")
  expect_true("ratedBiclustering" %in% class(result_rated))
})

test_that("rated Biclustering converges", {
  expect_true(result_rated$converge)
})

test_that("rated Biclustering FRP dimensions (nominal 3D array)", {
  expect_equal(length(dim(result_rated$FRP)), 3)
  expect_equal(dim(result_rated$FRP)[1], 3) # nfld
  expect_equal(dim(result_rated$FRP)[2], 3) # ncls
})

test_that("rated Biclustering FRP probabilities sum to 1", {
  nfld <- dim(result_rated$FRP)[1]
  ncls <- dim(result_rated$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_rated$FRP[f, c, ]), 1.0, tolerance = 1e-6)
    }
  }
})

test_that("rated Biclustering has both binary and nominal fit indices", {
  # Binary layer
  expect_false(is.null(result_rated$TestFitIndices))
  expect_s3_class(result_rated$TestFitIndices, "ModelFit")
  expect_false(is.na(result_rated$TestFitIndices$model_log_like))
  expect_false(is.na(result_rated$TestFitIndices$AIC))
  expect_false(is.na(result_rated$TestFitIndices$BIC))
  # Nominal layer
  expect_false(is.null(result_rated$TestFitIndices_nominal))
  expect_s3_class(result_rated$TestFitIndices_nominal, "ModelFit")
  expect_false(is.na(result_rated$TestFitIndices_nominal$AIC))
  expect_false(is.na(result_rated$TestFitIndices_nominal$BIC))
  # Nominal layer has no benchmark
  expect_true(is.na(result_rated$TestFitIndices_nominal$bench_log_like))
  expect_true(is.na(result_rated$TestFitIndices_nominal$RMSEA))
})

test_that("rated Biclustering field/class counts", {
  # LFD sum may exceed nitems when field memberships have ties
  expect_equal(sum(result_rated$LCD), 5000)
  expect_equal(length(result_rated$LFD), 3)
  expect_equal(length(result_rated$LCD), 3)
})

test_that("rated Biclustering classes are sorted by correct rate", {
  # Classes are sorted by correct rate; TRP should generally increase
  trp <- result_rated$TRP
  # Last class should have higher TRP than first
  expect_gt(trp[length(trp)], trp[1])
})

test_that("rated Biclustering has quasiFRP and FieldFRP", {
  expect_equal(dim(result_rated$quasiFRP), c(35, 3))
  # FieldFRP: some fields may be NA if empty
  expect_equal(nrow(result_rated$FieldFRP), 3)
  expect_equal(ncol(result_rated$FieldFRP), 3)
})

test_that("rated Biclustering quasiFRP values in [0, 1]", {
  valid <- result_rated$quasiFRP[!is.na(result_rated$quasiFRP)]
  expect_true(all(valid >= 0 & valid <= 1))
})

test_that("rated Biclustering has FRPIndex", {
  expect_false(is.null(result_rated$FRPIndex))
  expect_equal(ncol(result_rated$FRPIndex), 6)
  expect_equal(names(result_rated$FRPIndex), c("Alpha", "A", "Beta", "B", "Gamma", "C"))
})

test_that("rated Biclustering Students table has rank-up/down odds", {
  expect_true("Rank-Up Odds" %in% colnames(result_rated$Students))
  expect_true("Rank-Down Odds" %in% colnames(result_rated$Students))
  expect_true("Estimate" %in% colnames(result_rated$Students))
  expect_equal(nrow(result_rated$Students), 5000)
})

test_that("rated Biclustering has FieldAnalysis", {
  expect_false(is.null(result_rated$FieldAnalysis))
  expect_equal(nrow(result_rated$FieldAnalysis), 35)
  expect_true("CRR" %in% colnames(result_rated$FieldAnalysis))
  expect_true("LFE" %in% colnames(result_rated$FieldAnalysis))
})

test_that("rated Biclustering has SOAC/WOAC flags", {
  expect_true(is.logical(result_rated$SOACflg))
  expect_true(is.logical(result_rated$WOACflg))
})

test_that("rated Biclustering log_lik fields exist", {
  expect_false(is.na(result_rated$log_lik))
  expect_false(is.na(result_rated$log_lik_nominal))
  # Binary log_lik should be different from nominal
  expect_false(result_rated$log_lik == result_rated$log_lik_nominal)
})

test_that("rated Biclustering backward compatibility fields", {
  expect_equal(result_rated$Nclass, result_rated$n_class)
  expect_equal(result_rated$Nfield, result_rated$n_field)
  expect_equal(result_rated$N_Cycle, result_rated$n_cycle)
  expect_equal(result_rated$LogLik, result_rated$log_lik)
})

test_that("rated Biclustering print works", {
  expect_no_error(capture.output(print(result_rated)))
})


# ================================================================
# Confirmatory Biclustering (nominal / ordinal)
# ================================================================

# nominal: J20S600 = 20項目 → 4フィールド × 5項目
conf_nom <- rep(1:4, each = 5)
result_nom_conf <- Biclustering(J20S600,
  ncls = 5, nfld = 4,
  conf = conf_nom, verbose = FALSE
)

test_that("nominal confirmatory respects field assignments", {
  expect_equal(as.numeric(result_nom_conf$FieldEstimated), conf_nom)
})

test_that("nominal confirmatory rejects wrong-length conf vector", {
  expect_error(
    Biclustering(J20S600,
      ncls = 5, nfld = 4,
      conf = rep(1:4, each = 4), verbose = FALSE
    ),
    "conf vector size does NOT match"
  )
})

test_that("nominal confirmatory accepts membership matrix", {
  conf_mat <- matrix(0, nrow = 20, ncol = 4)
  for (i in seq_len(20)) conf_mat[i, conf_nom[i]] <- 1
  res <- Biclustering(J20S600,
    ncls = 5, nfld = 4,
    conf = conf_mat, verbose = FALSE
  )
  expect_equal(as.numeric(res$FieldEstimated), conf_nom)
})

test_that("nominal confirmatory rejects wrong-row matrix", {
  bad <- matrix(0, nrow = 18, ncol = 4)
  bad[cbind(seq_len(18), rep(1:4, length.out = 18))] <- 1
  expect_error(
    Biclustering(J20S600,
      ncls = 5, nfld = 4,
      conf = bad, verbose = FALSE
    ),
    "conf matrix size does NOT match"
  )
})

# ordinal: J35S500 = 35項目 → 5フィールド × 7項目
conf_ord <- c(rep(1, 7), rep(2, 7), rep(3, 7), rep(4, 7), rep(5, 7))
result_ord_conf <- Biclustering(J35S500,
  ncls = 5, nfld = 5, method = "R",
  conf = conf_ord, verbose = FALSE
)

test_that("ordinal confirmatory respects field assignments", {
  expect_equal(as.numeric(result_ord_conf$FieldEstimated), conf_ord)
})

test_that("ordinal confirmatory rejects wrong-length conf vector", {
  expect_error(
    Biclustering(J35S500,
      ncls = 5, nfld = 5, method = "R",
      conf = rep(1:5, each = 6), verbose = FALSE
    ),
    "conf vector size does NOT match"
  )
})


# ================================================================
# Class-side Confirmatory Biclustering (conf_class)
# ================================================================

# nominal: J20S600, 600 respondents -> 5 classes
nobs_nom <- NROW(dataFormat(J20S600)$Q)
conf_class_nom <- ((seq_len(nobs_nom) - 1) %% 5) + 1
result_nom_cc <- Biclustering(J20S600,
  ncls = 5, nfld = 4,
  conf_class = conf_class_nom, verbose = FALSE
)

test_that("nominal class-side confirmatory respects class assignments", {
  est <- apply(result_nom_cc$ClassMembership, 1, which.max)
  expect_equal(as.numeric(est), conf_class_nom)
})

test_that("nominal class-side confirmatory rejects wrong-length conf_class", {
  expect_error(
    Biclustering(J20S600,
      ncls = 5, nfld = 4,
      conf_class = 1:10, verbose = FALSE
    ),
    "conf_class vector size does NOT match"
  )
})

test_that("nominal class-side confirmatory accepts membership matrix", {
  cm <- matrix(0, nrow = nobs_nom, ncol = 5)
  for (i in seq_len(nobs_nom)) cm[i, conf_class_nom[i]] <- 1
  res <- Biclustering(J20S600,
    ncls = 5, nfld = 4,
    conf_class = cm, verbose = FALSE
  )
  est <- apply(res$ClassMembership, 1, which.max)
  expect_equal(as.numeric(est), conf_class_nom)
})

# ordinal: J35S500, 500 respondents -> 5 classes
nobs_ord <- NROW(dataFormat(J35S500)$Q)
conf_class_ord <- ((seq_len(nobs_ord) - 1) %% 5) + 1
result_ord_cc <- Biclustering(J35S500,
  ncls = 5, nfld = 5, method = "R",
  conf_class = conf_class_ord, verbose = FALSE
)

test_that("ordinal class-side confirmatory respects class assignments", {
  est <- apply(result_ord_cc$ClassMembership, 1, which.max)
  expect_equal(as.numeric(est), conf_class_ord)
})

test_that("ordinal Ranklustering with conf_class skips smoothing", {
  # When conf_class fixes the class memberships, smoothed_memb should equal
  # clsmemb (no neighbour averaging applied).
  expect_equal(
    as.numeric(result_ord_cc$SmoothedMembership),
    as.numeric(result_ord_cc$ClassMembership)
  )
})

# rated: J21S300, 300 respondents -> 4 classes (post-processing reorders classes)
nobs_rat <- NROW(dataFormat(J21S300)$Q)
conf_class_rat <- ((seq_len(nobs_rat) - 1) %% 4) + 1
result_rat_cc <- Biclustering(J21S300,
  ncls = 4, nfld = 3,
  conf_class = conf_class_rat, verbose = FALSE
)

test_that("rated class-side confirmatory keeps individuals in a single class (up to relabeling)", {
  # rated reorders classes by correct rate after estimation, so the input
  # labels do not necessarily match output labels. The invariant we test is
  # that each input class maps bijectively to one output class.
  est <- apply(result_rat_cc$ClassMembership, 1, which.max)
  cross <- table(conf_class_rat, est)
  # Every input class should land in exactly one output class.
  expect_true(all(rowSums(cross > 0) == 1))
  # And every output class should come from exactly one input class.
  expect_true(all(colSums(cross > 0) == 1))
})

# combined: conf + conf_class together (binary-style on ordinal data)
test_that("conf and conf_class can be combined (ordinal)", {
  conf_field_ord <- c(rep(1, 7), rep(2, 7), rep(3, 7), rep(4, 7), rep(5, 7))
  res <- Biclustering(J35S500,
    ncls = 5, nfld = 5, method = "R",
    conf = conf_field_ord,
    conf_class = conf_class_ord, verbose = FALSE
  )
  est <- apply(res$ClassMembership, 1, which.max)
  expect_equal(as.numeric(res$FieldEstimated), conf_field_ord)
  expect_equal(as.numeric(est), conf_class_ord)
})

# IRT / GRM plot functions
# Internal functions called from plot.exametrika()

#' Plot IRT model (IRF / TRF / IIF / TIF)
#' @noRd
plot_irt_model <- function(x, type, plotItemID, nc, nr, overlay, colors, dots = list()) {
  testlength <- x$testlength

  # rename type option
  type <- switch(type,
    "ICC" = "IRF",
    "IIC" = "IIF",
    "TIC" = "TIF",
    type
  )
  if ((type %in% c("IIF", "TIF") && any(plotItemID == 0))) {
    type <- "TIF"
    plotItemID <- 1:testlength
  } else if (type == "IRF" && any(plotItemID == 0)) {
    type <- "TRF"
    plotItemID <- 1:testlength
  }
  if (length(plotItemID) == 0) {
    plotItemID <- 1:testlength
  }

  ### IRT curve function
  plotIRTCurve <- function(params, curveFunc, titleBase, ylab, overlay) {
    if (overlay) {
      setup_legend_layout(1, nc)
      call_plot(
        plot,
        list(
          x = NULL,
          xlim = c(-4, 4),
          ylim = c(0, 1),
          xlab = "ability",
          ylab = ylab,
          main = titleBase
        ),
        dots
      )
      for (i in 1:nrow(params)) {
        a <- params[i, 1]
        b <- params[i, 2]
        c <- if (x$model > 2) params[i, 3] else 0
        d <- if (x$model > 3) params[i, 4] else 1
        draw_curve(
          function(theta) curveFunc(a, b, c, d, theta = theta),
          from = -4, to = 4, add = TRUE,
          defaults = list(lty = i, col = i, lwd = 2),
          dots = dots
        )
      }
      draw_legend_strip(
        legend = paste("Item", plotItemID),
        lty = 1:nrow(params),
        col = 1:nrow(params),
        lwd = 2, bty = "n",
        ncol = min(nrow(params), 5)
      )
    } else {
      for (i in 1:nrow(params)) {
        a <- params[i, 1]
        b <- params[i, 2]
        c <- if (x$model > 2) params[i, 3] else 0
        d <- if (x$model > 3) params[i, 4] else 1
        title <- paste0(titleBase, ", item ", plotItemID[i])
        draw_curve(
          function(theta) curveFunc(a, b, c, d, theta = theta),
          from = -4, to = 4,
          defaults = list(xlab = "ability", ylab = ylab, main = title),
          dots = dots
        )
      }
    }
  }

  params <- x$params[plotItemID, , drop = FALSE]
  if (type == "IRF") {
    plotIRTCurve(
      params, exametrika::LogisticModel,
      "Item Response Function", "probability",
      overlay
    )
  }
  if (type == "TRF") {
    draw_curve(
      function(theta) exametrika::TestResponseFunc(params, theta = theta),
      from = -4, to = 4,
      defaults = list(xlab = "ability", ylab = "probability", main = "Test Response Function"),
      dots = dots
    )
  }
  if (type == "IIF") {
    plotIRTCurve(
      params, exametrika::ItemInformationFunc,
      "Item Information Function", "information",
      overlay
    )
  }
  if (type == "TIF") {
    draw_curve(
      function(theta) exametrika::TestInformationFunc(params, theta = theta),
      from = -4, to = 4,
      defaults = list(xlab = "ability", ylab = "Information", main = "Test Information Function"),
      dots = dots
    )
  }
}

#' Plot GRM model (IRF / IIF / TIF)
#' @noRd
plot_grm_model <- function(x, type, plotItemID, nc, nr, colors, dots = list()) {
  testlength <- x$testlength

  # rename type option
  type <- switch(type,
    "ICC" = "IRF",
    "IIC" = "IIF",
    "TIC" = "TIF",
    type
  )
  if ((type %in% c("IIF", "TIF") && any(plotItemID == 0))) {
    type <- "TIF"
    plotItemID <- 1:testlength
  } else if (type == "IRF" && any(plotItemID == 0)) {
    type <- "TRF"
    plotItemID <- 1:testlength
  }
  if (length(plotItemID) == 0) {
    plotItemID <- 1:testlength
  }

  ### GRM curve function
  grm_IRF <- function(a, b, title) {
    thetas <- seq(-4, 4, 0.01)
    n_theta <- length(thetas)
    K <- length(b) + 1
    grm_colors <- if (!is.null(colors)) {
      colors[-1]
    } else {
      c("#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7", "#8B4513")
    }
    probs <- matrix(0, nrow = n_theta, ncol = K)
    for (i in 1:n_theta) {
      probs[i, ] <- grm_prob(thetas[i], a, b)
    }
    call_plot(
      plot,
      list(
        x = thetas, y = probs[, 1],
        type = "l",
        col = grm_colors[1],
        ylim = c(0, 1),
        xlab = expression(theta),
        main = title
      ),
      dots
    )
    for (k in 2:K) {
      call_plot(graphics::lines, list(x = thetas, y = probs[, k], col = grm_colors[k]), dots)
    }
  }

  grm_IIC <- function(a, b, title) {
    thetas <- seq(-4, 4, 0.01)
    I <- sapply(thetas, function(theta) grm_iif(theta, a, b))

    call_plot(
      plot,
      list(
        x = thetas, y = I,
        type = "l",
        xlab = expression(theta),
        ylab = "Item Information",
        main = title
      ),
      dots
    )
  }

  params <- x$params[plotItemID, , drop = FALSE]
  if (type == "IRF") {
    setup_legend_layout(nrow(params), nc)
    max_K <- 0
    for (j in 1:nrow(params)) {
      a <- params[j, 1]
      b <- params[j, -1]
      b <- b[!is.na(b)]
      K <- length(b) + 1
      if (K > max_K) max_K <- K
      title <- paste("Item Category Response Function for item", plotItemID[j])
      grm_IRF(a, b, title)
    }
    grm_colors <- if (!is.null(colors)) {
      colors[-1]
    } else {
      c("#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7", "#8B4513")
    }
    draw_legend_strip(
      legend = paste("Category", 1:max_K),
      col = grm_colors[1:max_K], lty = 1,
      cex = 1.0, bty = "n", ncol = min(max_K, 5)
    )
  }
  if (type == "IIF") {
    for (j in 1:nrow(params)) {
      a <- params[j, 1]
      b <- params[j, -1]
      b <- b[!is.na(b)]
      title <- paste("Item Information Function for item", plotItemID[j])
      grm_IIC(a, b, title)
    }
  }
  if (type == "TIF") {
    thetas <- seq(-4, 4, 0.01)
    I <- matrix(nrow = testlength, ncol = length(thetas))
    for (j in 1:NROW(params)) {
      a <- params[j, 1]
      b <- params[j, -1]
      b <- b[!is.na(b)]
      I[j, ] <- sapply(thetas, function(theta) grm_iif(theta, a, b))
    }
    TestInfo <- colSums(I, na.rm = T)
    call_plot(
      plot,
      list(
        x = thetas, y = TestInfo,
        type = "l",
        xlab = expression(theta),
        ylab = "Information",
        main = "Test Information Function"
      ),
      dots
    )
  }
}

# IRM Gibbs Sampler Core and Helper Functions
#
# Internal functions shared by Biclustering_IRM.nominal and Biclustering_IRM.ordinal.
# These are not exported.

# Helper functions -------------------------------------------------------

#' Compute U_fcq: sufficient statistics (nfld x ncls x maxQ)
#' @noRd
irm_calc_Ufcq <- function(Uq, Z, cls01, fld01, maxQ) {
  nfld_l <- ncol(fld01)
  ncls_l <- ncol(cls01)
  U_fcq <- array(0, dim = c(nfld_l, ncls_l, maxQ))
  for (q in 1:maxQ) {
    U_fcq[, , q] <- t(fld01) %*% t(Z * Uq[, , q]) %*% cls01
  }
  return(U_fcq)
}

#' Log multivariate Beta function
#' @noRd
irm_lmvbeta <- function(a) {
  sum(lgamma(a)) - lgamma(sum(a))
}

#' Log-sum-exp trick for numerical stability
#' @noRd
irm_log_sum_exp <- function(x) {
  m <- max(x)
  m + log(sum(exp(x - m)))
}

#' Convert log-probabilities to probabilities
#' @noRd
irm_log_to_prob <- function(log_p) {
  exp(log_p - irm_log_sum_exp(log_p))
}

#' Expand 3D array by appending one zero-slice along dimension 1 (field axis)
#' @noRd
irm_expand_dim1 <- function(A) {
  d <- dim(A)
  result <- array(0, dim = c(d[1] + 1, d[2], d[3]))
  result[1:d[1], , ] <- A
  result
}

#' Expand 3D array by appending one zero-slice along dimension 2 (class axis)
#' @noRd
irm_expand_dim2 <- function(A) {
  d <- dim(A)
  result <- array(0, dim = c(d[1], d[2] + 1, d[3]))
  result[, 1:d[2], ] <- A
  result
}

#' BIC calculation for IRM model selection
#' @noRd
irm_bic_calc <- function(Uq, Z, fld01, cls01, BCRM, maxQ, const) {
  ncls_l <- ncol(cls01)
  nfld_l <- ncol(fld01)
  nobs_l <- nrow(cls01)
  testell <- 0
  for (q in 1:maxQ) {
    pred_prob <- t(fld01 %*% BCRM[, , q] %*% t(cls01))
    observed_mask <- (Z * Uq[, , q]) == 1
    testell <- testell + sum(log(pmax(pred_prob[observed_mask], const)))
  }
  nparam <- ncls_l * nfld_l * (maxQ - 1)
  bic <- -2 * testell + nparam * log(nobs_l)
  return(bic)
}

# Gibbs Sampler Core -----------------------------------------------------

#' Collapsed Gibbs Sampler for IRM (Chinese Restaurant Process)
#'
#' Shared core for nominal and ordinal IRM. Performs CRP-based collapsed
#' Gibbs sampling with Dirichlet-Multinomial likelihood. Data type
#' (nominal/ordinal) does not affect the Gibbs phase.
#'
#' @param Uq One-hot encoded response array (nobs x nitems x maxQ).
#' @param Z Missing indicator matrix (nobs x nitems).
#' @param cls01 Initial class membership matrix (nobs x ncls).
#' @param fld01 Initial field membership matrix (nitems x nfld).
#' @param gamma_c CRP concentration parameter for classes.
#' @param gamma_f CRP concentration parameter for fields.
#' @param alpha_vec Dirichlet prior parameter vector (length maxQ).
#' @param max_iter Maximum number of Gibbs iterations.
#' @param stable_limit Number of consecutive stable iterations for convergence.
#' @param verbose If TRUE, display progress messages.
#' @param use_cpp If TRUE (default), dispatch to the Rcpp implementation
#'   `irm_gibbs_core_cpp()`. With the same `set.seed()` it produces
#'   bit-identical output to the R reference implementation but runs
#'   roughly 4-5x faster. Set FALSE only for cross-checking.
#' @return A list with:
#'   \describe{
#'     \item{cls01}{Final class membership matrix (nobs x ncls).}
#'     \item{fld01}{Final field membership matrix (nitems x nfld).}
#'     \item{U_fcq}{Final sufficient statistics array (nfld x ncls x maxQ).}
#'     \item{ncls}{Number of classes.}
#'     \item{nfld}{Number of fields.}
#'     \item{cls}{Class assignment vector (length nobs).}
#'     \item{fld}{Field assignment vector (length nitems).}
#'     \item{Nc}{Class sizes (length ncls).}
#'     \item{Nf}{Field sizes (length nfld).}
#'     \item{n_gibbs_iter}{Number of Gibbs iterations performed.}
#'   }
#' @noRd
irm_gibbs_core <- function(Uq, Z, cls01, fld01,
                           gamma_c, gamma_f, alpha_vec,
                           max_iter, stable_limit, verbose,
                           use_cpp = TRUE) {
  if (isTRUE(use_cpp)) {
    out <- irm_gibbs_core_cpp(
      Uq_vec = as.numeric(Uq),
      Uq_dim = as.integer(dim(Uq)),
      Z = Z,
      cls01_init = cls01,
      fld01_init = fld01,
      gamma_c = gamma_c,
      gamma_f = gamma_f,
      alpha_vec_R = alpha_vec,
      max_iter = max_iter,
      stable_limit = stable_limit,
      verbose = verbose
    )
    rownames(out$cls01) <- rownames(cls01)
    colnames(out$cls01) <- paste("Rank", seq_len(out$ncls))
    colnames(out$fld01) <- paste("Field", seq_len(out$nfld))
    return(out)
  }

  nobs <- nrow(cls01)
  nitems <- nrow(fld01)
  ncls <- ncol(cls01)
  nfld <- ncol(fld01)
  maxQ <- length(alpha_vec)
  const <- exp(-nitems)

  cls <- as.vector(cls01 %*% (1:ncls))
  fld <- as.vector(fld01 %*% (1:nfld))
  Nc <- colSums(cls01)
  Nf <- colSums(fld01)

  limit_count <- 0
  iter <- 1

  IRM_FLG <- TRUE
  while (IRM_FLG) {
    iRand <- sample(1:nobs, nobs, replace = FALSE)

    # Compute U_fcq once before the class-side loop
    U_fcq <- irm_calc_Ufcq(Uq, Z, cls01, fld01, maxQ)

    ## Gibbs sampler for Class c
    for (target in iRand) {
      # Compute v_fq: contribution of student "target" (class-independent)
      v_fq <- matrix(0, nrow = nfld, ncol = maxQ)
      for (q in 1:maxQ) {
        v_fq[, q] <- t(fld01) %*% (Z[target, ] * Uq[target, , q])
      }

      # Subtract target's contribution: U_fcq becomes U_fcq^{-s*}
      old_cls <- cls[target]
      for (q in 1:maxQ) {
        U_fcq[, old_cls, q] <- U_fcq[, old_cls, q] - v_fq[, q]
      }
      Nc[old_cls] <- Nc[old_cls] - 1

      # If the class disappeared
      if (Nc[old_cls] == 0) {
        ncls <- ncls - 1
        cls01 <- cls01[, -old_cls, drop = FALSE]
        cls <- as.vector(cls01 %*% (1:ncls))
        Nc <- Nc[-old_cls]
        U_fcq <- U_fcq[, -old_cls, , drop = FALSE]
      }

      # CRP likelihood for existing classes
      exist_tab <- numeric(ncls)
      for (c in 1:ncls) {
        exist_tab[c] <- log(Nc[c] / (nobs - 1 + gamma_c) + const)
        for (f in 1:nfld) {
          nume <- U_fcq[f, c, ] + v_fq[f, ] + alpha_vec
          deno <- U_fcq[f, c, ] + alpha_vec
          exist_tab[c] <- exist_tab[c] + irm_lmvbeta(nume) - irm_lmvbeta(deno)
        }
      }

      # CRP likelihood for new class
      new_tab1 <- log(gamma_c / (nobs - 1 + gamma_c))
      new_tab2 <- 0
      for (f in 1:nfld) {
        new_tab2 <- new_tab2 + irm_lmvbeta(v_fq[f, ] + alpha_vec) - irm_lmvbeta(alpha_vec)
      }
      new_tab <- new_tab1 + new_tab2

      # Sample class assignment
      ptab <- irm_log_to_prob(c(exist_tab, new_tab))
      sampled_value <- rmultinom(1, 1, ptab)
      delpos <- sampled_value[-(ncls + 1)]
      if (sampled_value[ncls + 1] == 0) {
        # Student s belongs to an existing class
        new_cls <- which.max(delpos)
        cls01[target, ] <- delpos
        cls[target] <- new_cls
      } else {
        # Student s belongs to a new class
        ncls <- ncls + 1
        cls01 <- cbind(cls01, 0)
        cls01[target, ] <- 0
        cls01[target, ncls] <- 1
        cls <- as.vector(cls01 %*% (1:ncls))
        new_cls <- ncls
        U_fcq <- irm_expand_dim2(U_fcq)
      }

      # Add back target's contribution to new class
      for (q in 1:maxQ) {
        U_fcq[, new_cls, q] <- U_fcq[, new_cls, q] + v_fq[, q]
      }
      Nc <- colSums(cls01)
    }

    ## Gibbs sampler for Field f
    oldfld <- fld
    jRand <- sample(1:nitems, nitems, replace = FALSE)

    # U_fcq is already up-to-date from class-side loop

    for (target in jRand) {
      # Compute v_cq: contribution of item "target" (field-independent)
      v_cq <- matrix(0, nrow = ncls, ncol = maxQ)
      for (q in 1:maxQ) {
        v_cq[, q] <- t(cls01) %*% (Z[, target] * Uq[, target, q])
      }

      # Subtract target's contribution: U_fcq becomes U_fcq^{-j*}
      old_fld <- fld[target]
      for (q in 1:maxQ) {
        U_fcq[old_fld, , q] <- U_fcq[old_fld, , q] - v_cq[, q]
      }
      Nf[old_fld] <- Nf[old_fld] - 1

      # If the field disappeared
      if (Nf[old_fld] == 0) {
        nfld <- nfld - 1
        fld01 <- fld01[, -old_fld, drop = FALSE]
        fld <- as.vector(fld01 %*% (1:nfld))
        Nf <- Nf[-old_fld]
        U_fcq <- U_fcq[-old_fld, , , drop = FALSE]
      }

      # CRP likelihood for existing fields
      exist_tab <- numeric(nfld)
      for (f in 1:nfld) {
        exist_tab[f] <- log(Nf[f] / (nitems - 1 + gamma_f) + const)
        for (c in 1:ncls) {
          nume <- U_fcq[f, c, ] + v_cq[c, ] + alpha_vec
          deno <- U_fcq[f, c, ] + alpha_vec
          exist_tab[f] <- exist_tab[f] + irm_lmvbeta(nume) - irm_lmvbeta(deno)
        }
      }

      # CRP likelihood for new field
      new_tab1 <- log(gamma_f / (nitems - 1 + gamma_f))
      new_tab2 <- 0
      for (c in 1:ncls) {
        new_tab2 <- new_tab2 + irm_lmvbeta(v_cq[c, ] + alpha_vec) - irm_lmvbeta(alpha_vec)
      }
      new_tab <- new_tab1 + new_tab2

      # Sample field assignment
      ptab <- irm_log_to_prob(c(exist_tab, new_tab))
      sampled_value <- rmultinom(1, 1, ptab)
      delpos <- sampled_value[-(nfld + 1)]
      if (sampled_value[nfld + 1] == 0) {
        # Item j belongs to an existing field
        new_fld <- which.max(delpos)
        fld01[target, ] <- delpos
        fld[target] <- new_fld
      } else {
        # Item j belongs to a new field
        nfld <- nfld + 1
        fld01 <- cbind(fld01, 0)
        fld01[target, ] <- 0
        fld01[target, nfld] <- 1
        fld <- as.vector(fld01 %*% (1:nfld))
        new_fld <- nfld
        U_fcq <- irm_expand_dim1(U_fcq)
      }

      # Add back target's contribution to new field
      for (q in 1:maxQ) {
        U_fcq[new_fld, , q] <- U_fcq[new_fld, , q] + v_cq[, q]
      }
      Nf <- colSums(fld01)
    }

    # Check convergence: field assignment stability (matching Mathematica original)
    limit_count <- if (sum(abs(oldfld - fld)) == 0) {
      limit_count + 1
    } else {
      0
    }

    if (verbose) {
      message(
        sprintf(
          "iter %d: match=%d nfld=%d ncls=%d",
          iter, limit_count, nfld, ncls
        )
      )
    }

    if (limit_count == stable_limit || iter == max_iter) {
      IRM_FLG <- FALSE
    } else {
      iter <- iter + 1
    }
  }

  colnames(cls01) <- paste("Rank", 1:ncls)
  colnames(fld01) <- paste("Field", 1:nfld)

  list(
    cls01 = cls01, fld01 = fld01,
    U_fcq = U_fcq,
    ncls = ncls, nfld = nfld,
    cls = cls, fld = fld,
    Nc = Nc, Nf = Nf,
    n_gibbs_iter = iter
  )
}

library(exametrika)

# ================================================================
# IRM Gibbs sampler: C++ vs R reference implementation
# ================================================================
#
# The Rcpp implementation `irm_gibbs_core_cpp()` is a direct translation
# of the R reference `irm_gibbs_core(..., use_cpp = FALSE)`. RNG calls
# go through R-level sample.int() and rmultinom() so the unif_rand()
# consumption order matches base R.
#
# Bit-identical agreement holds for short runs (the cases tested in this
# file: <= 10 iterations on the bundled J20S600 / J35S500 datasets). On
# real polytomous data with many iterations, sub-LSB floating-point
# ordering differences inside `lmvbeta()` accumulate through the CRP
# likelihood and can flip a single `rmultinom()` outcome, after which
# the two chains follow different sample paths. The marginal posterior
# distributions remain statistically indistinguishable between the two
# paths (verified empirically; see NEWS.md v1.13.0).
#
# TODO: add a long-iteration parity test on real-data-like configurations
# to characterise where (and how reliably) the two paths diverge.

setup_state <- function(dat) {
  nitems <- NCOL(dat$Q)
  nobs <- NROW(dat$Q)
  maxQ <- max(dat$categories)
  Uq <- array(0, dim = c(nobs, nitems, maxQ))
  for (s in seq_len(nobs)) {
    for (j in seq_len(nitems)) {
      Uq[s, j, dat$Q[s, j]] <- 1
    }
  }
  mode_cat <- apply(dat$Q, 1, function(x) {
    tab <- table(x)
    as.integer(names(which.max(tab)))
  })
  ncls0 <- length(unique(mode_cat))
  cls01 <- matrix(0, nobs, ncls0)
  for (i in seq_len(nobs)) cls01[i, mode_cat[i]] <- 1
  fld01 <- diag(nitems)
  list(
    Uq = Uq, Z = dat$Z, cls01 = cls01, fld01 = fld01,
    maxQ = maxQ, nobs = nobs, nitems = nitems
  )
}

run_R <- function(s, max_iter, seed = 42) {
  set.seed(seed)
  exametrika:::irm_gibbs_core(
    Uq = s$Uq, Z = s$Z, cls01 = s$cls01, fld01 = s$fld01,
    gamma_c = 1, gamma_f = 1, alpha_vec = rep(1, s$maxQ),
    max_iter = max_iter, stable_limit = 9999, verbose = FALSE,
    use_cpp = FALSE
  )
}

run_C <- function(s, max_iter, seed = 42) {
  set.seed(seed)
  exametrika:::irm_gibbs_core(
    Uq = s$Uq, Z = s$Z, cls01 = s$cls01, fld01 = s$fld01,
    gamma_c = 1, gamma_f = 1, alpha_vec = rep(1, s$maxQ),
    max_iter = max_iter, stable_limit = 9999, verbose = FALSE,
    use_cpp = TRUE
  )
}

# nominal data
state_nom <- setup_state(dataFormat(J20S600))

test_that("nominal IRM Gibbs: C++ matches R after 1 iteration", {
  rR <- run_R(state_nom, max_iter = 1)
  rC <- run_C(state_nom, max_iter = 1)
  expect_equal(rC$ncls, rR$ncls)
  expect_equal(rC$nfld, rR$nfld)
  expect_equal(rC$cls, rR$cls)
  expect_equal(rC$fld, rR$fld)
  expect_equal(rC$Nc, rR$Nc)
  expect_equal(rC$Nf, rR$Nf)
  expect_equal(unname(as.matrix(rC$cls01)), unname(as.matrix(rR$cls01)))
  expect_equal(unname(as.matrix(rC$fld01)), unname(as.matrix(rR$fld01)))
  expect_equal(as.numeric(rC$U_fcq), as.numeric(rR$U_fcq))
})

test_that("nominal IRM Gibbs: C++ matches R after 10 iterations", {
  rR <- run_R(state_nom, max_iter = 10)
  rC <- run_C(state_nom, max_iter = 10)
  expect_equal(rC$ncls, rR$ncls)
  expect_equal(rC$nfld, rR$nfld)
  expect_equal(rC$cls, rR$cls)
  expect_equal(rC$fld, rR$fld)
  expect_equal(as.numeric(rC$U_fcq), as.numeric(rR$U_fcq))
})

test_that("nominal IRM Gibbs: C++ matches R with different seed", {
  rR <- run_R(state_nom, max_iter = 5, seed = 777)
  rC <- run_C(state_nom, max_iter = 5, seed = 777)
  expect_equal(rC$cls, rR$cls)
  expect_equal(rC$fld, rR$fld)
})

# ordinal data
state_ord <- setup_state(dataFormat(J35S500))

test_that("ordinal IRM Gibbs: C++ matches R after 1 iteration", {
  rR <- run_R(state_ord, max_iter = 1)
  rC <- run_C(state_ord, max_iter = 1)
  expect_equal(rC$ncls, rR$ncls)
  expect_equal(rC$nfld, rR$nfld)
  expect_equal(rC$cls, rR$cls)
  expect_equal(rC$fld, rR$fld)
  expect_equal(as.numeric(rC$U_fcq), as.numeric(rR$U_fcq))
})

test_that("ordinal IRM Gibbs: C++ matches R after 5 iterations", {
  rR <- run_R(state_ord, max_iter = 5)
  rC <- run_C(state_ord, max_iter = 5)
  expect_equal(rC$cls, rR$cls)
  expect_equal(rC$fld, rR$fld)
})

test_that("Biclustering_IRM nominal end-to-end (C++ default) is reproducible", {
  set.seed(42)
  r1 <- Biclustering_IRM(J20S600,
    gamma_c = 1, gamma_f = 1,
    max_iter = 30, verbose = FALSE
  )
  set.seed(42)
  r2 <- Biclustering_IRM(J20S600,
    gamma_c = 1, gamma_f = 1,
    max_iter = 30, verbose = FALSE
  )
  expect_equal(r1$n_class, r2$n_class)
  expect_equal(r1$n_field, r2$n_field)
  expect_equal(as.numeric(r1$ClassMembership), as.numeric(r2$ClassMembership))
  expect_equal(as.numeric(r1$FieldMembership), as.numeric(r2$FieldMembership))
})

#' @title Latent Rank Analysis
#' @description
#' A general function for estimating Latent Rank Analysis across different response types.
#' This function automatically dispatches to the appropriate method based on the response type:
#'
#' \itemize{
#'   \item For binary data (\code{LRA.binary}): Analysis using either SOM or GTM method
#'   \item For ordinal data (\code{LRA.ordinal}): Analysis using the GTM method with category thresholds
#'   \item For rated data (\code{LRA.rated}): Analysis using the GTM method with rating categories
#' }
#'
#' Latent Rank Analysis identifies underlying rank structures in test data and assigns
#' examinees to these ranks based on their response patterns.
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#' it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param ... Additional arguments passed to specific methods.
#'
#' @return
#' A list of class "exametrika" and the specific subclass (e.g., "LRA", "LRAordinal", "LRArated")
#' containing the following common elements:
#' \describe{
#'  \item{msg}{A character string indicating the model type. }
#'  \item{testlength}{Length of the test (number of items).}
#'  \item{nobs}{Sample size (number of rows in the dataset).}
#'  \item{Nrank}{Number of latent ranks specified.}
#'  \item{N_Cycle}{Number of EM algorithm iterations performed.}
#'  \item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
#'  \item{TRP}{Test Reference Profile vector showing expected scores at each rank.}
#'  \item{LRD}{Latent Rank Distribution vector showing the number of examinees at each rank.}
#'  \item{RMD}{Rank Membership Distribution vector showing the sum of probabilities for each rank.}
#'  \item{Students}{Rank Membership Profile matrix showing the posterior probabilities of
#'       examinees belonging to each rank, along with their estimated ranks and odds ratios.}
#'  \item{ItemFitIndices}{Fit indices for each item. See also \code{\link{ItemFit}}.}
#'  \item{TestFitIndices}{Overall fit indices for the test. See also \code{\link{TestFit}}.}
#' }
#'
#' Each subclass returns additional specific elements, detailed in their respective documentation.
#'
#' @seealso \code{\link{plot.exametrika}} for visualizing LRA results.
#' @export
#'
LRA <- function(U, ...) {
  UseMethod("LRA")
}

#' @rdname LRA
#' @param na Values to be treated as missing values.
#' @param Z Missing indicator matrix of type matrix or data.frame. 1 indicates observed values, 0 indicates missing values.
#' @param w Item weight vector.
#' @export
LRA.default <- function(U, na = NULL, Z = NULL, w = NULL, ...) {
  if (inherits(U, "exametrika")) {
    if (U$response.type == "binary") {
      return(LRA.binary(U, ...))
    } else if (U$response.type == "ordinal") {
      return(LRA.ordinal(U, ...))
    } else if (U$response.type == "rated") {
      return(LRA.rated(U, ...))
    } else {
      response_type_error(U$response.type, "LRA")
    }
  }

  U <- dataFormat(U, na = na, Z = Z, w = w)
  LRA(U, ...)
}

#' @rdname LRA
#' @section Binary Data Method:
#' \code{LRA.binary} analyzes dichotomous (0/1) response data using either Self-Organizing Maps (SOM)
#' or Gaussian Topographic Mapping (GTM).
#'
#' @param nrank Number of latent ranks to estimate. Must be between 2 and 20.
#' @param method For binary data only. Either "SOM" (Self-Organizing Maps) or "GTM" (Gaussian Topographic Mapping). Default is "GTM".
#' @param mic Logical; if TRUE, forces Item Reference Profiles to be monotonically increasing. Default is FALSE.
#' @param maxiter Maximum number of iterations for estimation. Default is 100.
#' @param BIC.check For binary data with SOM method only. If TRUE, convergence is checked using BIC values. Default is FALSE.
#' @param seed For binary data with SOM method only. Random seed for reproducibility.
#' @param verbose Logical; if TRUE, displays detailed progress during estimation. Default is TRUE.
#' @param beta1 Beta distribution parameter 1 for prior density of rank reference matrix (GTM method only). Default is 1.
#' @param beta2 Beta distribution parameter 2 for prior density of rank reference matrix (GTM method only). Default is 1.
#' @param conf Confirmatory IRP matrix (items x nrank) for test equating.
#'   Same format as the IRP output. Non-NA values are fixed throughout estimation,
#'   NA values are freely estimated. Fixed values must be in the open interval (0, 1).
#'   When row names are present, items are matched by label; otherwise by position.
#'   Default is NULL (fully exploratory).
#'
#' @return
#' For binary data (\code{LRA.binary}), the returned list additionally includes:
#' \describe{
#'  \item{IRP}{Item Reference Profile matrix showing the probability of correct response for each item across different ranks.}
#'  \item{IRPIndex}{Item Response Profile indices including the location parameters B and Beta,
#'    slope parameters A and Alpha, and monotonicity indices C and Gamma.}
#' }
#'
#' @examples
#' \donttest{
#' # Binary data example
#' # Fit a Latent Rank Analysis model with 6 ranks to binary data
#' result.LRA <- LRA(J15S500, nrank = 6)
#'
#' # Display the first few rows of student rank membership profiles
#' head(result.LRA$Students)
#'
#' # Plot Item Reference Profiles (IRP) for the first 6 items
#' plot(result.LRA, type = "IRP", items = 1:6, nc = 2, nr = 3)
#'
#' # Plot Test Reference Profile (TRP) showing expected scores at each rank
#' plot(result.LRA, type = "TRP")
#' }
#'
#' @importFrom stats runif
#' @export
LRA.binary <- function(U,
                       nrank = 2,
                       method = "GTM",
                       mic = FALSE,
                       maxiter = 100,
                       BIC.check = FALSE,
                       seed = NULL,
                       verbose = FALSE,
                       beta1 = 1,
                       beta2 = 1,
                       conf = NULL, ...) {
  tmp <- U
  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  samplesize <- NROW(tmp$U)
  const <- exp(-testlength)
  ncls <- nrank

  if (method != "SOM" & method != "GTM") {
    stop("The method must be selected as either SOM or GTM.")
  }

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  # Validate and align confirmatory IRP matrix
  if (!is.null(conf)) {
    conf <- validate_conf(conf, ncls, tmp$ItemLabel)
  }

  if (method == "SOM") {
    fit <- somclus(tmp$U, tmp$Z, ncls,
      mic = mic, maxiter = maxiter,
      BIC.check = BIC.check, seed = seed, verbose = verbose,
      conf = conf
    )
  } else {
    # GTM.
    Filter <- create_filter_matrix(ncls)

    fit <- emclus(tmp$U, tmp$Z,
      ncls = ncls,
      Fil = Filter, beta1, beta2, mic = mic,
      conf = conf
    )
  }

  ## Returns
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)
  #### Class Information
  TRP <- fit$classRefMat %*% tmp$w
  bMax <- matrix(rep(apply(fit$postDist, 1, max), ncls), ncol = ncls)
  clsNum <- apply(fit$postDist, 1, which.max)
  cls01 <- sign(fit$postDist - bMax) + 1
  LRD <- colSums(cls01)
  RMD <- colSums(fit$postDist)
  StudentClass <- fit$postDist
  RU <- ifelse(clsNum + 1 > ncls, NA, clsNum + 1)
  RD <- ifelse(clsNum - 1 < 1, NA, clsNum - 1)
  RUO <- StudentClass[cbind(1:nobs, RU)] / StudentClass[cbind(1:nobs, clsNum)]
  RDO <- StudentClass[cbind(1:nobs, RD)] / StudentClass[cbind(1:nobs, clsNum)]
  StudentClass <- cbind(StudentClass, clsNum, RUO, RDO)
  colnames(StudentClass) <- c(
    paste("Membership", 1:ncls), "Estimate",
    "Rank-Up Odds", "Rank-Down Odds"
  )
  rownames(StudentClass) <- tmp$ID
  ### Item Information
  IRP <- t(fit$classRefMat)
  colnames(IRP) <- paste0("IRP", 1:ncls)
  rownames(IRP) <- tmp$ItemLabel

  IRPIndex <- IRPindex(IRP)

  if (sum(IRPIndex$C) == 0) {
    if (verbose) {
      message("Strongly ordinal alignment condition was satisfied.")
    }
  }

  ### Model Fit
  # each Items
  ell_A <- item_log_lik(tmp$U, tmp$Z, fit$postDist, fit$classRefMat)
  if (method == "GTM") {
    nparam <- sum(diag(Filter))
  } else {
    nparam <- ncls
  }
  FitIndices <- ItemFit(tmp$U, tmp$Z, ell_A, nparam)

  ret <- structure(list(
    method = method,
    mic = mic,
    msg = "Rank",
    converge = fit$converge,
    testlength = testlength,
    nobs = nobs,
    n_rank = ncls, # New naming convention
    n_cycle = fit$iter, # New naming convention
    TRP = as.vector(TRP),
    LRD = as.vector(LRD),
    RMD = as.vector(RMD),
    Students = StudentClass,
    IRP = IRP,
    IRPIndex = IRPIndex,
    ItemFitIndices = FitIndices$item,
    TestFitIndices = FitIndices$test,
    log_lik = FitIndices$test$model_log_like,
    # Deprecated fields (for backward compatibility)
    Nrank = ncls,
    N_Cycle = fit$iter
  ), class = c("exametrika", "LRA"))
  return(ret)
}

library(exametrika)

### GOALS - Mathematica reference data
test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter06LRA_GTMmic0_Test.csv"),
  check.names = FALSE
)
class <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter06LRA_GTMmic0_Rank.csv"),
  check.names = FALSE
)
items <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter06LRA_GTMmic0_Item.csv"),
  check.names = FALSE
)
student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter06LRA_GTMmic0_Student.csv"),
  check.names = FALSE
)


### Setup
tmp <- dataFormat(J15S500, na = -99)
model <- LRA(tmp, nrank = 6)

### Tests
test_that("LRA Test Fit", {
  expect <- test[15:30, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- model$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("LRA Class Info", {
  ## TRP
  expect <- class[1, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$TRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## LCD
  expect <- class[2, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$LRD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## CMD
  expect <- class[3, 2:7] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$RMD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("LRA Item Info", {
  ## IRP
  expect <- items[, 6:11] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$IRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## IRP index
  expect <- items[, 12:17] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$IRPIndex |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## FitIndex
  expect <- items[, c(22, 18, 19, 23, 20, 24, 21, 25:33)] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$ItemFitIndices |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("LRA Student Info", {
  ## Membership probabilities
  expect <- student[, 6:11] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$Students[, 1:6] |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## Latent Rank Estimate
  expect <- student[, 12] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$Students[, 7] |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## Rank-Up/Down Odds
  expect <- student[, 13:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$Students[, 8:9] |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})


test_that("LRA Students", {
  ## Membership
  expect <- student[, 6:12] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$Students[, 1:7] |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  ## RUO/RDO
  expect <- student[, 13:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- model$Students[, 8:9] |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

library(exametrika)

### GOALS - Mathematica reference data
testReport <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_ScoreReport.csv"),
  check.names = FALSE
)
ItemReport <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_ItemReport.csv"),
  check.names = FALSE
)
catQuant <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_ItemCategoryQuantileRepo.csv"),
  check.names = FALSE
)
catIRPreport <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_CatIRPreport.csv"),
  check.names = FALSE
)
RankScoreMat <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_RankScoremat.csv"),
  check.names = FALSE
)
ScoreMemb <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_ScoreMemb.csv"),
  check.names = FALSE
)
RankQuantMat <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_RankQuantMat.csv"),
  check.names = FALSE
)
MembQuantMat <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_MembQuantMat.csv"),
  check.names = FALSE
)
ItemFit1 <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_ItemFit1.csv"),
  check.names = FALSE
)
ItemFit2 <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_ItemFit2.csv"),
  check.names = FALSE
)
TestFit <- read.csv(
  test_path("fixtures", "mathematica_reference", "13NNT_TestFit.csv"),
  check.names = FALSE
)


result <- LRA(J35S5000, nrank = 10, mic = TRUE, verbose = F)

test_that("Test Info", {
  expect <- testReport[, 2] |>
    unlist() |>
    as.numeric()
  actual <- result$ScoreReport |>
    as.matrix() |>
    unlist() |>
    as.numeric()
  expect_equal(actual[-6], expect, tolerance = 1e-4)
})

test_that("Item Info", {
  # Exclude CORR column (col 5) which contains Mathematica formula strings
  expect <- ItemReport[, c(2:4, 6)] |> as.matrix()
  storage.mode(expect) <- "double"
  actual <- result$ItemReport |>
    unclass() |>
    as.data.frame()
  actual <- actual[, names(actual) != "ItemLabel", drop = FALSE]
  actual <- as.matrix(actual)
  # Remove ItemSD (col 4) and ItemCORR (col 5); keep Obs, ObsRatio, ItemMean, ItemCORR_R
  actual <- actual[, -c(4, 5)]
  rownames(actual) <- NULL
  colnames(actual) <- colnames(expect) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("Category Quantile", {
  expect <- catQuant[, 2:12] |> as.matrix()
  actual <- result$CatQuant |>
    unclass() |>
    as.data.frame()
  actual <- actual[, !names(actual) %in% c("Item", "Category"), drop = FALSE]
  actual <- as.matrix(actual)
  rownames(actual) <- NULL
  colnames(actual) <- colnames(expect) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("Category IRP Report", {
  expect <- catIRPreport[, -1] |> as.matrix()
  actual <- result$ICRP[, -c(1, 2)] |> as.matrix()
  rownames(actual) <- NULL
  colnames(actual) <- colnames(expect) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("Rank Score Matrix", {
  # CSV from col_names=FALSE sheet; convert to numeric matrix
  expect <- apply(as.matrix(RankScoreMat), 2, as.numeric)
  actual <- result$ScoreRank |> as.matrix()
  # actual[,1:10] <- actual[,10:1]
  actual[1:36, ] <- actual[36:1, ]
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("Score Membership", {
  # CSV from col_names=FALSE sheet; convert to numeric matrix
  expect <- apply(as.matrix(ScoreMemb), 2, as.numeric)
  actual <- result$ScoreMembership |> as.matrix()
  actual[1:36, ] <- round(actual[36:1, ], 8)
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("Item Fit", {
  expect <- ItemFit2[, -1] |> as.matrix()
  actual <- result$ItemFitIndices |>
    unclass() |>
    as.data.frame() |>
    as.matrix()
  actual <- actual[, -c(1, 2, 3, 5, 7)]
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-3)
})

test_that("Test Fit", {
  expect <- TestFit[, 3] |>
    unlist() |>
    as.vector()
  actual <- result$TestFitIndices |>
    unlist() |>
    as.vector()
  expect <- expect[c(1, 2, 4:10)]
  actual <- actual[c(4, 6, 8:14)]
  expect_equal(actual, expect, tolerance = 1e-3)
})

library(exametrika)

### GOALS - Mathematica reference data
testReport <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_TestReport.csv"),
  check.names = FALSE
)
itemReport <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_ItemReport.csv"),
  check.names = FALSE
)
catQuant <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_CatQuantReport.csv"),
  check.names = FALSE
)
cumRatio <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_CumRatio.csv"),
  check.names = FALSE
)
testRefProf <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_TestRefProf.csv"),
  check.names = FALSE
)
TRP_ref <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_TRP.csv"),
  check.names = FALSE
)
RankProf <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_RankProf.csv"),
  check.names = FALSE
)
ScoreRank <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_ScoreRank.csv"),
  check.names = FALSE
)
TesFit1 <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_TestFit1.csv"),
  check.names = FALSE
)
TesFit2 <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_TestFit2.csv"),
  check.names = FALSE
)
TesFit3 <- read.csv(
  test_path("fixtures", "mathematica_reference", "12GNT_TestFit3.csv"),
  check.names = FALSE
)


result <- LRA(J15S3810, mic = TRUE, nrank = 3)

test_that("Test Info", {
  expect <- testReport[, 2] |>
    unlist() |>
    as.numeric()
  actual <- result$ScoreReport |>
    as.matrix() |>
    unlist() |>
    as.numeric()
  expect_equal(actual, expect, tolerance = 1e-4)
})

test_that("Item Info", {
  expect <- itemReport[, -1] |> as.matrix()
  actual <- result$ItemReport |>
    unclass() |>
    as.data.frame()
  actual <- actual[, names(actual) != "ItemLabel", drop = FALSE]
  actual <- as.matrix(actual)
  rownames(actual) <- NULL
  colnames(actual) <- colnames(expect) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("CatQuant Ref Mat", {
  expect <- catQuant[, 2:5] |> as.matrix()
  actual <- result$CatQuant[, 3:6] |> as.matrix()
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("IC Boundary", {
  expect <- cumRatio[, 2:4] |> as.matrix()
  actual <- result$ICBR[, 3:5] |> as.matrix()
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-4)
})

test_that("IC Reference Profile", {
  expect <- testRefProf[, 2:4] |> as.matrix()
  actual <- result$ICRP[, 3:5] |> as.matrix()
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-6)
})

test_that("Test Reference Profile", {
  TRP_mat <- TRP_ref[, 2:4] |>
    as.matrix() |>
    unname()
  expect1 <- TRP_mat[1, ] |> as.vector()
  expect2 <- TRP_mat[2, ] |> as.vector()
  actual1 <- result$TRP |> as.vector()
  actual2 <- result$LRD |> as.vector()
  expect_equal(actual1, expect1, tolerance = 1e-6)
  expect_equal(actual2, expect2, tolerance = 1e-6)
})

test_that("Rank Profile", {
  expect <- RankProf[, -1] |> as.matrix()
  actual <- result$Students[, c(1, 2, 3, 5, 4)] |> as.matrix()
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-4)
})

test_that("Score Rank", {
  expect <- ScoreRank[, -1] |> as.data.frame()
  actual <- result$ScoreMembership |>
    as.matrix() |>
    as.data.frame()
  actual[1:28, ] <- actual[28:1, ]
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  expect_equal(actual, expect, tolerance = 1e-4)
})

test_that("Item Fit", {
  # Expected: TesFit2 columns = Chi-sq, df, NFI, RFI, IFI, TLI, CFI, RMSEA, AIC, CAIC, BIC
  expect <- TesFit2[, -c(1, 4)] |> as.data.frame()
  # Actual: ItemFitIndices after removing log-lik and null fields
  actual <- result$ItemFitIndices |>
    unclass() |>
    as.data.frame()
  actual <- actual[, -c(1, 2, 3, 5, 7)]
  rownames(expect) <- rownames(actual) <- NULL
  colnames(expect) <- colnames(actual) <- NULL
  names(expect) <- names(actual) <- NULL
  expect <- as.matrix(expect)
  actual <- as.matrix(actual)
  # Compare Chi-sq(1), df(2), NFI(3), RFI(4), IFI(5), TLI(6), CFI(7), AIC(9)
  # Known discrepancies in RMSEA(8), CAIC(10), BIC(11):
  #   - CAIC formula is now unified: both R and Mathematica use log(n)+1 (Bozdogan 1987)
  #   - However, nobs handling differs: R uses per-item nobs (colSums(Z)),
  #     Mathematica uses total nobs (3810)
  #   - This causes RMSEA, CAIC, BIC to differ (J15S3810 has significant missing data)
  # TODO: Investigate whether to unify nobs handling
  cols_to_test <- c(1:7, 9)
  expect_equal(actual[, cols_to_test], expect[, cols_to_test], tolerance = 1e-4)
})

test_that("Test Fit", {
  # Expected: TesFit3 column 3 (RMP-Indices), excluding Chi-p (row 3)
  # = Chi-sq, df, NFI, RFI, IFI, TLI, CFI, RMSEA, AIC, CAIC, BIC (11 values)
  expect <- TesFit3[-3, 3] |> as.numeric()
  # Actual: TestFitIndices (16 fields), removing non-comparable fields
  # Remove: model_log_like(1), bench_log_like(2), null_log_like(3),
  #         null_Chi_sq(5), null_df(7)
  actual <- result$TestFitIndices |>
    unclass() |>
    unlist() |>
    as.numeric()
  actual <- actual[-c(1, 2, 3, 5, 7)]
  # Compare Chi-sq(1), df(2), NFI(3), RFI(4), IFI(5), TLI(6), CFI(7)
  # Known discrepancies in RMSEA(8), AIC(9), CAIC(10), BIC(11):
  #   - CAIC formula is now unified: both R and Mathematica use log(n)+1 (Bozdogan 1987)
  #   - However, R uses sum(colSums(Z))=34290 as nobs, Mathematica uses 3810
  #   - This causes RMSEA, AIC, CAIC, BIC to differ
  # TODO: Investigate test-level nobs handling for ordinal LRA
  cols_to_test <- 1:7
  expect_equal(actual[cols_to_test], expect[cols_to_test], tolerance = 1e-4)
})

#' @title Prior distribution function with respect to the slope.
#' @param a slope coefficient
#' @param m prior parameter to be set
#' @param s prior parameter to be set
#' @param const A very small constant
#'
slopeprior <- function(a, m, s, const = 1e-15) {
  -(log(max(a, const)) - m)^2 / (2 * s^2) - log(max(a, const)) - log(s)
}

#' @title Prior distribution function with guessing parameter
#' @param c guessing parameter
#' @param alp prior to be set
#' @param bet prior to be set
#'
asymprior <- function(c, alp, bet) {
  (alp - 1) * log(c) + (bet - 1) * log(1 - c)
}

#' @title Log-likelihood function used in the Maximization Step (M-Step).
#' @param model 2,3,or 4 PL
#' @param lambda item parameter vector
#' @param qjtrue correct resp pattern
#' @param qjfalse incorrect resp pattern
#' @param quadrature Pattern of a segmented normal distribution.

objective_function_IRT <- function(lambda, model, qjtrue, qjfalse, quadrature) {
  a <- lambda[1]
  b <- lambda[2]
  c <- ifelse(model > 2, lambda[3], 0)
  d <- ifelse(model > 3, lambda[4], 1)

  exloglike <- sum(
    qjtrue * log(LogisticModel(a = a, b = b, c = c, d = d, theta = quadrature)) +
      qjfalse * log(1 - LogisticModel(a = a, b = b, c = c, d = d, theta = quadrature))
  )

  exloglike <- exloglike - ((b / 2)^2 / 2) + slopeprior(a, 0, 0.5)

  if (model >= 3) {
    exloglike <- exloglike + asymprior(c, 2, 5)
  }

  if (model == 4) {
    exloglike <- exloglike + asymprior(d, 10, 2)
  }

  return(exloglike)
}

#' @title internal functions for PSD of Item parameters
#' @param model 2,3,or 4PL
#' @param Lambda item parameters Matrix
#' @param quadrature quads
#' @param marginal_posttheta marginal post theta

PSD_item_params <- function(model, Lambda, quadrature, marginal_posttheta) {
  J <- NROW(Lambda)
  ret <- array(NA, dim = c(J, model))
  for (j in 1:J) {
    a <- Lambda[j, 1]
    b <- Lambda[j, 2]
    c <- Lambda[j, 3]
    d <- Lambda[j, 4]
    ### I_pr
    I_pr_lambda <- diag(rep(NA, model))
    I_pr_lambda[1, 1] <- (1 - 0.5^2 - log(a)) / (a^2 * 0.5^2)
    I_pr_lambda[2, 2] <- 1 / 2^2
    if (model > 2) {
      I_pr_lambda[3, 3] <- 1 / c^2 + 4 / (1 - c)^2
    }
    if (model > 3) {
      I_pr_lambda[4, 4] <- 9 / d^2 + 1 / (1 - d)^2
    }
    ## I_F
    p <- LogisticModel(a = a, b = b, c = c, d = d, theta = quadrature)
    q <- 1 - p
    I_F_lambda <- matrix(rep(NA, model * model), ncol = model)
    den <- (d - c)^2 * p * q
    ## aa
    num <- (quadrature - b)^2 * (p - c)^2 * (d - p)^2
    I_F_lambda[1, 1] <- sum((num / den) * marginal_posttheta)
    ## ba
    num <- a * (quadrature - b) * (p - c)^2 * (d - p)^2
    I_F_lambda[1, 2] <- I_F_lambda[2, 1] <- -1 * sum(num / den * marginal_posttheta)
    ## bb
    num <- a^2 * (p - c)^2 * (d - p)^2
    I_F_lambda[2, 2] <- sum((num / den) * marginal_posttheta)

    if (model > 2) {
      ## ca
      num <- (quadrature - b) * (p - c) * (d - p)^2
      I_F_lambda[1, 3] <- I_F_lambda[3, 1] <- sum((num / den) * marginal_posttheta)
      ## cb
      num <- a * (p - c) * (d - p)^2
      I_F_lambda[2, 3] <- I_F_lambda[3, 2] <- -1 * sum((num / den) * marginal_posttheta)
      ## cc
      num <- (d - p)^2
      I_F_lambda[3, 3] <- sum(num / den * marginal_posttheta)
    }

    if (model > 3) {
      ## da
      num <- (quadrature - b) * (p - c)^2 * (d - p)
      I_F_lambda[1, 4] <- I_F_lambda[4, 1] <- sum((num / den) * marginal_posttheta)
      ## db
      num <- a * (p - c)^2 * (d - p)
      I_F_lambda[2, 4] <- I_F_lambda[4, 2] <- -1 * sum((num / den) * marginal_posttheta)
      ## dc
      num <- (p - c) * (d - p)
      I_F_lambda[3, 4] <- I_F_lambda[4, 3] <- sum((num / den) * marginal_posttheta)
      ## dd
      num <- (p - c)^2
      I_F_lambda[4, 4] <- sum((num / den) * marginal_posttheta)
    }

    ##
    Ij <- I_F_lambda + I_pr_lambda
    ret[j, ] <- sqrt(diag(solve(Ij)))
    ## Return
  }
  return(PSD = ret)
}


#' @title Estimating Item parameters using EM algorithm
#' @description
#' A function for estimating item parameters using the EM algorithm.
#' @param model This argument takes the number of item parameters to be
#' estimated in the logistic model. It is limited to values 2, 3, or 4.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param verbose logical; if TRUE, shows progress of iterations (default: TRUE)
#' @importFrom stats optim
#' @details
#' Apply the 2, 3, and 4 parameter logistic models to estimate the item and subject populations.
#' The 4PL model can be described as follows.
#' \deqn{P(\theta,a_j,b_j,c_j,d_j)= c_j + \frac{d_j -c_j}{1+exp\{-a_j(\theta - b_j)\}}}
#' \eqn{a_j, b_j, c_j}, and \eqn{d_j} are parameters related to item j, and are parameters that
#' adjust the logistic curve.
#' \eqn{a_j} is called the slope parameter, \eqn{b_j} is the location, \eqn{c_j} is the lower asymptote,
#' and \eqn{d_j} is the upper asymptote parameter.
#' The model includes lower models, and among the 4PL models, the case where \eqn{d=1} is the 3PL model,
#' and among the 3PL models, the case where \eqn{c=0} is the 2PL model.
#' @return
#' \describe{
#' \item{model}{number of item parameters you set.}
#' \item{testlength}{Length of the test. The number of items included in the test.}
#' \item{nobs}{Sample size. The number of rows in the dataset.}
#' \item{params}{Matrix containing the estimated item parameters}
#' \item{Q3mat}{Q3-matrix developed by Yen(1984)}
#' \item{itemPSD}{Posterior standard deviation of the item parameters}
#' \item{ability}{Estimated parameters of students ability}
#' \item{ItemFitIndices}{Fit index for each item.See also \code{\link{ItemFit}}}
#' \item{TestFitIndices}{Overall fit index for the test.See also \code{\link{TestFit}}}
#' }
#' @references Yen, W. M. (1984) Applied Psychological Measurement, 8, 125-145.
#' @importFrom stats rnorm
#' @examples
#' \donttest{
#' # Fit a 3-parameter IRT model to the sample dataset
#' result.IRT <- IRT(J15S500, model = 3)
#'
#' # Display the first few rows of estimated student abilities
#' head(result.IRT$ability)
#'
#' # Plot Item Response Function (IRF) for items 1-6 in a 2x3 grid
#' plot(result.IRT, type = "IRF", items = 1:6, nc = 2, nr = 3)
#'
#' # Plot Item Information Function (IIF) for items 1-6 in a 2x3 grid
#' plot(result.IRT, type = "IIF", items = 1:6, nc = 2, nr = 3)
#'
#' # Plot the Test Information Function (TIF) for all items
#' plot(result.IRT, type = "TIF")
#' }
#' @export
#'

IRT <- function(U, model = 2, na = NULL, Z = NULL, w = NULL, verbose = TRUE) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (U$response.type != "binary") {
    response_type_error(U$response.type, "IRT")
  }

  U <- tmp$U * tmp$Z

  rho <- exametrika::ItemTotalCorr(U)
  tau <- exametrika::ItemThreshold(U)

  # initialize
  testlength <- NCOL(U)
  nobs <- NROW(U)
  slope <- 2 * rho
  loc <- 2 * tau
  if (model >= 3) {
    loasym <- rep(0.05, testlength)
  } else {
    loasym <- rep(0, testlength)
  }
  if (model >= 4) {
    upasym <- rep(0.95, testlength)
  } else {
    upasym <- rep(1, testlength)
  }

  paramset <- matrix(c(slope, loc, loasym, upasym), ncol = 4)
  quadrature <- seq(-3.2, 3.2, 0.4)
  const <- exp(-testlength)
  loglike <- -1 / const
  oldloglike <- -2 / const
  itemloglike <- rep(loglike / testlength, testlength)

  # EM algorithm
  emt <- 0
  maxemt <- 25
  FLG <- TRUE
  itemloglike <- array(NA, testlength)

  while (FLG) {
    if (abs(loglike - oldloglike) < 0.00001 * abs(oldloglike)) {
      FLG <- FALSE
    }
    emt <- emt + 1
    oldloglike <- loglike
    if (emt > maxemt) {
      FLG <- FALSE
    }
    ### Expectation
    lpj <- matrix(NA, nrow = testlength, ncol = length(quadrature))
    for (j in 1:testlength) {
      lpj[j, ] <- log(LogisticModel(
        a = paramset[j, 1],
        b = paramset[j, 2],
        c = paramset[j, 3],
        d = paramset[j, 4],
        theta = quadrature
      ) + const)
    }

    lqj <- matrix(NA, nrow = testlength, ncol = length(quadrature))
    for (j in 1:testlength) {
      lqj[j, ] <- log(1 - LogisticModel(
        a = paramset[j, 1],
        b = paramset[j, 2],
        c = paramset[j, 3],
        d = paramset[j, 4],
        theta = quadrature
      ) + const)
    }

    posttheta_numerator <- exp(
      (tmp$Z * tmp$U) %*% lpj +
        (tmp$Z * (1 - tmp$U)) %*% lqj -
        matrix(rep(quadrature^2 / 2, NROW(tmp$U)), nrow = NROW(tmp$U), byrow = T)
    )

    post_theta <- posttheta_numerator / rowSums(posttheta_numerator)

    marginal_posttheta <- colSums(post_theta)
    #
    qjtrue <- t(tmp$Z * tmp$U) %*% post_theta
    qjfalse <- t(tmp$Z * (1 - tmp$U)) %*% post_theta


    ### Maximize
    totalLogLike <- 0
    for (j in 1:testlength) {
      ## Initial perturbation and setting of upper and lower limits.
      initial_values <- paramset[j, 1:model] + rnorm(model, 0, 0.01)
      lowers <- c(-Inf, -5, 1e-10, 1e-10)[1:model]
      uppers <- c(Inf, 5, 1 - 1e-10, 1 - 1e-10)[1:model]
      ## optimization
      optim_flg <- FALSE
      warning_flg <- FALSE
      max_attempt <- 100
      attempt <- 0
      while (!optim_flg & attempt < max_attempt) {
        attempt <- attempt + 1
        tryCatch(
          {
            result <- optim(
              par = initial_values,
              fn = objective_function_IRT,
              method = "L-BFGS-B",
              lower = lowers,
              upper = uppers,
              control = list(fnscale = -1, factr = 1e-10),
              ### args for objective_function_IRT
              model = model,
              qjtrue = qjtrue[j, ],
              qjfalse = qjfalse[j, ],
              quadrature = quadrature
            )
            optim_flg <- TRUE
          },
          error = function(e) {
            initial_values <<- paramset[j, 1:model] + rnorm(model, 0, 0.1)
          }
        )
      }
      itemloglike[j] <- result$value
      totalLogLike <- totalLogLike + itemloglike[j]
      if (model == 2) {
        newparams <- c(result$par, 0, 1)
      } else if (model == 3) {
        newparams <- c(result$par, 1)
      } else if (model == 4) {
        newparams <- result$par
      }
      paramset[j, ] <- newparams
    }
    loglike <- totalLogLike
    if (verbose) {
      message(
        sprintf(
          "\r%-80s",
          paste0(
            "iter ", emt, " LogLik ", format(totalLogLike, digits = 6)
          )
        ),
        appendLF = FALSE
      )
    }
  }

  #### Warning
  if (sum(paramset[, 1] > 10) > 0) {
    warning("Some items have a discrimination parameter that exceeds 10. Please exercise caution in interpreting the model.")
  }

  ## Returns
  #### Item information
  item_model_loglike <- itemloglike + (paramset[, 2] / 2)^2 / 2 - slopeprior(paramset[, 1], 0, 0.5)
  if (model > 2) {
    item_model_loglike <- item_model_loglike - asymprior(paramset[, 3], 2, 5)
  }
  if (model > 3) {
    item_model_loglike <- item_model_loglike - asymprior(paramset[, 4], 10, 2)
  }
  itemPSD <- PSD_item_params(model, paramset, quadrature, marginal_posttheta)

  ### Ability Scores
  EAP <- post_theta %*% quadrature
  tmpA <- matrix(rep(quadrature, NROW(tmp$U)), nrow = NROW(tmp$U), byrow = T)
  tmpB <- matrix(rep(EAP, length(quadrature)), nrow = NROW(tmp$U), byrow = F)
  PSD <- sqrt(diag(post_theta %*% t((tmpA - tmpB)^2)))

  ### Model fit
  ell_A <- item_model_loglike
  FitIndices <- ItemFit(tmp$U, tmp$Z, ell_A, model)

  ### Q3mat
  pij <- matrix(nrow = nobs, ncol = testlength)
  for (j in 1:testlength) {
    pij[, j] <-
      LogisticModel(
        a = paramset[j, 1],
        b = paramset[j, 2],
        c = paramset[j, 3],
        d = paramset[j, 4],
        theta = EAP
      )
  }
  eij <- tmp$Z * (tmp$U - pij)
  eij_mean <- colSums(eij) / colSums(tmp$Z)
  eij_dev <- tmp$Z * (eij - eij_mean)
  eij_var <- colSums(eij_dev^2) / (colSums(tmp$Z) - 1)
  eij_sd <- sqrt(eij_var)
  eij_cov <- t(eij_dev) %*% eij_dev / (JointSampleSize(tmp) - 1)
  Q3mat <- eij_cov / (eij_sd %*% t(eij_sd))


  ## Formatting
  paramset <- as.data.frame(paramset)[1:model]
  itemPSD <- as.data.frame(itemPSD)[1:model]
  colnames(paramset) <- c("slope", "location", "lowerAsym", "upperAsym")[1:model]
  colnames(itemPSD) <- c("PSD(slope)", "PSD(location)", "PSD(lowerAsym)", "PSD(upperAsym)")[1:model]
  rownames(paramset) <- rownames(itemPSD) <- tmp$ItemLabel

  names(item_model_loglike) <- tmp$ItemLabel
  EAP <- data.frame(EAP)
  PSD <- data.frame(PSD)
  theta <- cbind(tmp$ID, EAP, PSD)
  colnames(theta) <- c("ID", "EAP", "PSD")

  ret <- structure(list(
    model = model,
    testlength = testlength,
    nobs = nobs,
    params = paramset,
    itemPSD = itemPSD,
    Q3mat = Q3mat,
    ability = theta,
    ItemFitIndices = FitIndices$item,
    TestFitIndices = FitIndices$test,
    log_lik = FitIndices$test$model_log_like
  ), class = c("exametrika", "IRT"))
  return(ret)
}

library(exametrika)

### model2
pl2Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT2pl_Test.csv"),
  check.names = FALSE
)
pl2Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT2pl_Item.csv"),
  check.names = FALSE
)
pl2Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT2pl_Student.csv"),
  check.names = FALSE
)
pl2Q3mat <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT2pl_Q3.csv"),
  check.names = FALSE
)

tmp <- dataFormat(J15S500, na = -99)
result2 <- IRT(model = 2, tmp)

test_that("2PL model Test Info", {
  expect <- pl2Test[13:28, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8, 9, 10, 11, 12, 13, 14, 15, 16)]
  result <- result2$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("2PL model Item Params Info", {
  expect <- pl2Item[, 7:6] |>
    unlist() |>
    as.numeric()
  result <- result2$params |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl2Item$`PSD(Slope)` |>
    unlist() |>
    as.numeric()
  result <- result2$itemPSD[, 1] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl2Item$`PSD(Loc)` |>
    unlist() |>
    as.numeric()
  result <- result2$itemPSD[, 2] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("2PL model Item FitIndices", {
  expect <- pl2Item$`Log-Likelihood(Benchmark Model)` |>
    unlist() |>
    as.numeric()
  result <- result2$ItemFitIndices$bench_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl2Item$`Log-Likelihood(Null Model)` |>
    unlist() |>
    as.numeric()
  result <- result2$ItemFitIndices$null_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl2Item$`Log-Likelihood(Analysis Model)` |>
    unlist() |>
    as.numeric()
  result <- result2$ItemFitIndices$model_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl2Item[, 18:26] |>
    unlist() |>
    as.numeric()
  result <- result2$ItemFitIndices |>
    unclass() |>
    as.data.frame()
  result <- result[, 8:16] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})


test_that("2PL model Students", {
  expect <- pl2Student$`EAP Score` |>
    unlist() |>
    as.numeric()
  result <- result2$ability$EAP |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl2Student$`Posterior SD` |>
    unlist() |>
    as.numeric()
  result <- result2$ability$PSD |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("2PL Q3mat", {
  expect <- pl2Q3mat[, -1] |>
    unlist() |>
    as.numeric()
  result <- result2$Q3mat |>
    unlist() |>
    as.numeric()
  # Q3 residual correlations include values near zero where relative
  # tolerance is unreliable. Use absolute difference comparison instead.
  max_abs_diff <- max(abs(result - expect))
  expect_true(max_abs_diff < 0.005,
    info = sprintf(
      "Max absolute difference in Q3 matrix: %.6f (threshold: 0.005)",
      max_abs_diff
    )
  )
})


# model3 ------------------------------------------------------------------
pl3Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT3pl_Test.csv"),
  check.names = FALSE
)
pl3Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT3pl_Item.csv"),
  check.names = FALSE
)
pl3Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT3pl_Student.csv"),
  check.names = FALSE
)


tmp <- dataFormat(J15S500, na = -99)
result3 <- IRT(model = 3, tmp)

test_that("3PL model Test Info", {
  expect <- pl3Test[13:28, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8, 9, 10, 11, 12, 13, 14, 15, 16)]
  result <- result3$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

test_that("3PL model Item Params Info", {
  expect <- pl3Item[, 7:9] |>
    unlist() |>
    as.numeric()
  result <- result3$params |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl3Item[, 10:12] |>
    unlist() |>
    as.numeric()
  result <- result3$itemPSD |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("3PL model Item FitIndices", {
  expect <- pl3Item$`Log-Likelihood(Benchmark Model)` |>
    unlist() |>
    as.numeric()
  result <- result3$ItemFitIndices$bench_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl3Item$`Log-Likelihood(Null Model)` |>
    unlist() |>
    as.numeric()
  result <- result3$ItemFitIndices$null_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl3Item$`Log-Likelihood(Analysis Model)` |>
    unlist() |>
    as.numeric()
  result <- result3$ItemFitIndices$model_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl3Item[, 20:28] |>
    unlist() |>
    as.numeric()
  result <- result3$ItemFitIndices |>
    unclass() |>
    as.data.frame()
  result <- result[, 8:16] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})


test_that("3PL model Students", {
  expect <- pl3Student$`EAP Score` |>
    unlist() |>
    as.numeric()
  result <- result3$ability$EAP |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl3Student$`Posterior SD` |>
    unlist() |>
    as.numeric()
  result <- result3$ability$PSD |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})


# model4 ------------------------------------------------------------------
pl4Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT4pl_Test.csv"),
  check.names = FALSE
)
pl4Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT4pl_Item.csv"),
  check.names = FALSE
)
pl4Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter04IRT4pl_Student.csv"),
  check.names = FALSE
)


tmp <- dataFormat(J15S500, na = -99)
result4 <- IRT(model = 4, tmp)

test_that("4PL model Test Info", {
  expect <- pl4Test[13:28, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8, 9, 10, 11, 12, 13, 14, 15, 16)]
  result <- result4$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


test_that("4PL model Item Params", {
  expect <- pl4Item[, 7:10] |>
    unlist() |>
    as.numeric()
  result <- result4$params |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("4PL model Item PSD", {
  expect <- pl4Item[, 11:14] |>
    unlist() |>
    as.numeric()
  result <- result4$itemPSD |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("4PL log-lik Bench", {
  expect <- pl4Item$`Log-Likelihood(Benchmark Model)` |>
    unlist() |>
    as.numeric()
  result <- result4$ItemFitIndices$bench_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("4PL log-lik Null", {
  expect <- pl4Item$`Log-Likelihood(Null Model)` |>
    unlist() |>
    as.numeric()
  result <- result4$ItemFitIndices$null_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})
test_that("4PL log-lik Analysis", {
  expect <- pl4Item$`Log-Likelihood(Analysis Model)` |>
    unlist() |>
    as.numeric()
  result <- result4$ItemFitIndices$model_log_like |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

test_that("4PL Standardized FIdx", {
  expect <- pl4Item[, 22:30] |>
    unlist() |>
    as.numeric()
  result <- result4$ItemFitIndices |>
    unclass() |>
    as.data.frame()
  result <- result[, 8:16] |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})


test_that("4PL model Students", {
  expect <- pl4Student$`EAP Score` |>
    unlist() |>
    as.numeric()
  result <- result4$ability$EAP |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)

  expect <- pl4Student$`Posterior SD` |>
    unlist() |>
    as.numeric()
  result <- result4$ability$PSD |>
    unlist() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-3)
})

#' @title dataFormat
#' @description
#' This function serves the role of formatting the data prior to the analysis.
#' @param data is a data matrix of the type matrix or data.frame.
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param id id indicates the column number containing the examinee ID.
#' If NULL (default), the first column is auto-detected as ID or response data.
#' If a column number is specified, that column is always used as the ID column.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param response.type Character string specifying the type of response data:
#'   "binary" for dichotomous data,
#'   "ordinal" for ordered polytomous data,
#'   "rated" for polytomous data with correct answers,
#'   "nominal" for unordered polytomous data.
#'   If NULL (default), the type is automatically detected.
#' @param CA A numeric vector specifying the correct answers for rated polytomous data.
#' Required when response.type is "rated".
#' @return
#' \describe{
#' \item{U}{For binary response data. A matrix with rows representing the sample size and columns
#'  representing the number of items, where elements are either 0 or 1. \eqn{u_{ij}=1} indicates
#'  that student i correctly answered item j, while \eqn{u_{ij}=0} means that student i answered
#'  item j incorrectly.}
#' \item{Q}{For polytomous response data. A matrix with rows representing the sample size and columns
#'  representing the number of items, where elements are non-negative integers. When input data is
#'  in factor format, the factor levels are converted to consecutive integers starting from 1.}
#' \item{ID}{The ID label given by the designated column or function.}
#' \item{ItemLabel}{The item names given by the provided column names or function.}
#' \item{Z}{Missing indicator matrix. \eqn{z_{ij}=1} indicates that item j is presented to Student i,
#' while \eqn{z_{ij}=0} indicates item j is NOT presented to Student i.
#' If the data contains NA values, -1 is assigned.}
#' \item{w}{Item weight vector}
#' \item{response.type}{Character string indicating the type of response data:
#'  "binary", "ordinal", "rated", or "nominal"}
#' \item{CategoryLabel}{List containing the original factor labels when polytomous responses
#'  are provided as factors. NULL if no factor data is present.}
#' \item{categories}{Numeric vector containing the number of response categories for each item.}
#' \item{CA}{For rated polytomous data, a numeric vector of correct answers. NULL for other types.}
#' }
#' @importFrom stats sd
#' @export
dataFormat <- function(data, na = NULL, id = NULL, Z = NULL, w = NULL,
                       response.type = NULL, CA = NULL) {
  # Check if the object is already formatted
  if (inherits(data, "exametrika")) {
    return(data)
  }
  if (!is.null(na) && is.na(na)) {
    na <- NULL
  }

  # Store original response type for later auto-detection
  original_response_type <- response.type

  # Validate manually specified response type (only if provided)
  if (!is.null(response.type) && !response.type %in% c("binary", "ordinal", "rated", "nominal")) {
    stop("response.type must be one of: binary, ordinal, rated, nominal")
  }

  data <- as.data.frame(unclass(data))
  data[data == na] <- NA

  # Check if U is either a matrix or a dataframe, otherwise stop the execution
  if (!is.matrix(data) && !is.data.frame(data)) {
    stop("Data must be matrix or data.frame")
  }

  # Check minimum number of rows (cases)
  if (nrow(data) < 10) {
    stop("Data must have at least 10 rows (cases) for reliable analysis")
  }

  # Helper function to check if a vector represents consecutive numbers
  is_consecutive_numbers <- function(x) {
    if (all(is.na(x))) {
      return(FALSE)
    }
    x_clean <- x[!is.na(x)]
    if (length(x_clean) <= 1) {
      return(TRUE)
    }

    x_num <- suppressWarnings(as.numeric(x_clean))
    if (any(is.na(x_num))) {
      return(FALSE)
    }

    x_sorted <- sort(unique(x_num))
    all(diff(x_sorted) == 1) && length(x_sorted) == length(x_num)
  }

  # Helper function to check if data looks like response data (has < 20 unique values)
  looks_like_response_data <- function(x) {
    if (all(is.na(x))) {
      return(FALSE)
    }
    x_clean <- x[!is.na(x)]
    if (length(x_clean) == 0) {
      return(FALSE)
    }

    x_num <- suppressWarnings(as.numeric(x_clean))
    if (any(is.na(x_num))) {
      return(FALSE)
    }

    unique_vals <- unique(x_num)
    length(unique_vals) < 20 && all(unique_vals >= 0) && all(unique_vals == round(unique_vals))
  }

  # Phase 1: ID column identification and response matrix extraction
  if (!is.null(id)) {
    # Explicit ID column specified
    if (!is.numeric(id) || length(id) != 1) {
      stop(
        "id must be a single integer specifying the column number (e.g., id = 1). ",
        "To pass ID labels, use a data.frame with an ID column instead."
      )
    }
    if (id > ncol(data)) {
      stop("ID column number exceeds the number of columns in data")
    }
    ID <- as.character(data[, id])
    response.matrix <- data[, -id, drop = FALSE]
  } else {
    # Auto-detect: analyze first column to determine if it's an ID column
    first_col <- data[, 1]

    if (is.factor(first_col)) {
      is_string_or_factor <- length(unique(first_col)) >= 20
    } else {
      is_string_or_factor <- is.character(first_col)
    }
    is_consecutive <- is_consecutive_numbers(first_col)

    if (is_string_or_factor || is_consecutive) {
      # First column is ID (strings, or unique consecutive integers)
      ID <- as.character(first_col)
      response.matrix <- data[, -1, drop = FALSE]
    } else {
      # First column treated as response data → auto-generate IDs
      ID <- paste0("Student", seq_len(nrow(data)))
      response.matrix <- data
      message(
        "No ID column detected. All columns treated as response data. ",
        "Sequential IDs (Student1, Student2, ...) were generated. ",
        "Use id= parameter to specify the ID column explicitly."
      )
      # Warn if first column has many unique values (might be an undetected ID column)
      n_unique_first <- length(unique(first_col[!is.na(first_col)]))
      if (n_unique_first > nrow(data) * 0.5 && n_unique_first < nrow(data)) {
        message(
          "Note: The first column ('", colnames(data)[1],
          "') has ", n_unique_first, " unique values out of ", nrow(data),
          " rows. If this is an ID column, specify id=1."
        )
      }
    }
  }

  # Check for duplicate IDs
  if (any(duplicated(ID))) {
    dup_ids <- unique(ID[duplicated(ID)])
    dup_details <- vapply(dup_ids, function(id) {
      rows <- which(ID == id)
      paste0("'", id, "' (rows ", paste(rows, collapse = ","), ")")
    }, character(1))
    stop(
      "Duplicated IDs found. Each ID must be unique.\n",
      paste(dup_details, collapse = "\n")
    )
  }

  # Store factor labels and convert factors to numeric (response columns only)
  CategoryLabel <- list()
  if (is.data.frame(response.matrix)) {
    for (col in names(response.matrix)) {
      if (is.factor(response.matrix[[col]])) {
        CategoryLabel[[col]] <- levels(response.matrix[[col]])
        response.matrix[[col]] <- as.numeric(response.matrix[[col]])
      }
    }
  }

  # Get Item-labels
  ItemLabel <- colnames(response.matrix)
  if (is.null(ItemLabel)) {
    ItemLabel <- paste0("Item", seq(1, NCOL(response.matrix)))
  }

  response.matrix <- as.matrix(response.matrix)
  response.matrix[is.na(response.matrix)] <- -1

  # Check if Z is indicator matrix
  if (!is.null(Z)) {
    if (!all(Z %in% c(0, 1))) {
      stop("The missing indicator matrix must contain only 0 or 1")
    }
    response.matrix <- ifelse(Z == 0, -1, response.matrix)
  }

  # Automatic response type detection (now moved to before validation)
  if (is.null(original_response_type)) {
    # Function to check if data is binary
    is_binary <- function(x, na_value = NULL) {
      # Remove both NA and specified na values
      x_clean <- x[!is.na(x) & x != -1] # Exclude missing indicator
      if (!is.null(na_value)) {
        x_clean <- x_clean[x_clean != na_value]
      }

      if (length(x_clean) == 0) {
        return(TRUE)
      } # Empty data considered as binary

      x_clean <- suppressWarnings(as.numeric(x_clean))
      all(x_clean %in% c(0, 1))
    }

    # Check if data appears to be ordinal (ordered categories)
    is_ordinal <- function(x, na_value = NULL) {
      x_clean <- x[!is.na(x) & x != -1] # Exclude missing indicator
      if (!is.null(na_value)) {
        x_clean <- x_clean[x_clean != na_value]
      }
      if (length(x_clean) == 0) {
        return(FALSE)
      }

      x_clean <- suppressWarnings(as.numeric(x_clean))
      if (any(is.na(x_clean))) {
        return(FALSE)
      }

      # Check if values are integers
      if (!all(x_clean == round(x_clean))) {
        return(FALSE)
      }

      unique_vals <- sort(unique(x_clean))

      # 2-category case: if not 0/1, stop with an error
      if (length(unique_vals) == 2) {
        if (!all(unique_vals %in% c(0, 1))) {
          stop("2-category data with non-binary values detected. Please specify response.type explicitly.")
        }
        return(FALSE) # 0/1 values are treated as binary
      }

      # Too many categories (>= 20)
      if (length(unique_vals) >= 20) {
        stop("Too many categories (>=20) for ordinal data. Please specify response.type explicitly.")
      }

      # 3 or more categories -> ordinal
      return(length(unique_vals) >= 3)
    }

    # Apply binary check to all response columns
    is_all_binary <- all(apply(response.matrix, 2, is_binary, na_value = na))

    if (is_all_binary) {
      response.type <- "binary"
    } else if (!is.null(CA)) {
      response.type <- "rated"
    } else {
      is_all_ordinal <- all(apply(response.matrix, 2, is_ordinal, na_value = na))

      if (is_all_ordinal) {
        response.type <- "ordinal"
      } else {
        response.type <- "nominal"
      }
    }
  } else {
    response.type <- original_response_type
  }

  if (response.type == "binary") {
    if (!all(response.matrix[response.matrix != -1] %in% c(0, 1))) {
      stop("For binary response type, data matrix can only contain the values 0, 1, NA, and the specified missing value")
    }
  } else if (response.type == "polytgomous") {
    # Check if all non-missing values are non-negative integers
    if (!all(response.matrix[response.matrix != -1] == floor(response.matrix[response.matrix != -1])) || any(response.matrix[response.matrix != -1] < 0)) {
      stop("For polytomous response type, data matrix must contain only non-negative integers, NA, and the specified missing value")
    }
  }

  ### This function finally makes each matrix as follow:
  # U is a matrix composed solely of 0s,1s and -1.
  # Z is the missing identifier matrix composed solely of 0s and 1s.
  if (!is.null(na)) {
    ## na value specified
    response.matrix <- ifelse(response.matrix == na, -1, response.matrix)
  }

  Z <- ifelse(response.matrix == -1, 0, 1)

  # If w is not specified, create a vector of 1s with length equal to the number of columns in U
  if (is.null(w)) {
    w <- rep(1, NCOL(response.matrix))
  }

  # check sd for each items
  sd.check <- apply(response.matrix, 2, function(x) sd(x, na.rm = TRUE))
  if (sum(is.na(sd.check)) != 0) {
    excluded_items <- ItemLabel[is.na(sd.check)]
    message(
      "The following items with no variance.Excluded from the data:\n",
      paste(excluded_items, collapse = ", "), "\n"
    )
    response.matrix <- response.matrix[, !is.na(sd.check), drop = FALSE]
    Z <- Z[, !is.na(sd.check), drop = FALSE]
    ItemLabel <- ItemLabel[!is.na(sd.check)]
    w <- w[!is.na(sd.check)]

    # Check if all items were excluded
    if (ncol(response.matrix) == 0) {
      stop("All items have no variance and were excluded. No valid response data remains.")
    }
  }

  # Warn about items with all missing values
  all_missing_cols <- which(apply(response.matrix, 2, function(x) all(x == -1)))
  if (length(all_missing_cols) > 0) {
    message(
      "Warning: The following items have all missing values: ",
      paste(ItemLabel[all_missing_cols], collapse = ", ")
    )
  }

  # Warn about items with zero variance (constant values, excluding all-missing)
  zero_var_cols <- which(!is.na(sd.check[seq_len(ncol(response.matrix))]) &
    sd.check[seq_len(ncol(response.matrix))] == 0)
  zero_var_cols <- setdiff(zero_var_cols, all_missing_cols)
  if (length(zero_var_cols) > 0) {
    message(
      "Warning: The following items have zero variance (constant values): ",
      paste(ItemLabel[zero_var_cols], collapse = ", ")
    )
  }

  # Warn about students with all missing responses
  all_missing_rows <- which(apply(response.matrix, 1, function(x) all(x == -1)))
  if (length(all_missing_rows) > 0) {
    message(
      "Warning: The following students have all missing responses: ",
      paste(ID[all_missing_rows], collapse = ", ")
    )
  }

  # Add category information
  categories <- apply(response.matrix, 2, function(x) length(unique(x[x != -1])))

  # Process category labels for each item
  for (i in seq_len(ncol(response.matrix))) {
    item_name <- ItemLabel[i]
    column_data <- response.matrix[, i]
    if (!item_name %in% names(CategoryLabel)) { # If not already processed as factor
      CategoryLabel[[item_name]] <- generate_category_labels(column_data, item_name)
    }
  }
  CA <- as.vector(unlist(CA))
  # check correct_answer
  if (response.type == "rated") {
    if (is.null(CA)) {
      stop("Correct Answer must be specified when polytype is rated")
    }
    if (length(CA) != ncol(response.matrix)) {
      stop("length of CA must match number of items")
    }

    unsupported <- integer(0)
    for (i in 1:length(CA)) {
      resp_cats <- unique(response.matrix[, i][response.matrix[, i] != -1])
      if (!CA[i] %in% resp_cats) unsupported <- c(unsupported, i)
    }
    if (length(unsupported) > 0) {
      warning(sprintf(
        "CA for item(s) %s not observed in responses (no correct responders); treated as all-incorrect.",
        paste(unsupported, collapse = ", ")
      ))
    }
    U <- matrix(NA, nrow = nrow(response.matrix), ncol = ncol(response.matrix))
    for (i in 1:nrow(response.matrix)) {
      U[i, ] <- ifelse(response.matrix[i, ] == -1, -1,
        ifelse(response.matrix[i, ] == CA, 1, 0)
      )
    }
  }


  # Create return list with appropriate matrix name based on response type
  ret.list <- list(
    ID = ID,
    ItemLabel = ItemLabel,
    Z = Z,
    w = w,
    response.type = response.type,
    categories = categories
  )
  # Add response matrix with appropriate name
  if (response.type == "binary") {
    ret.list$U <- response.matrix
  } else {
    ret.list$Q <- response.matrix
    ret.list$CategoryLabel <- CategoryLabel
    ret.list$CA <- CA
  }
  if (response.type == "rated") {
    ret.list$U <- U
  }

  # Return with appropriate class structure
  ret <- structure(ret.list, class = c("exametrika", "exametrikaData"))
  return(ret)
}

#' @title Long Format Data Conversion
#' @description
#' A function to reshape long data into a dataset suitable for exametrika.
#' @param data is a data matrix of the type matrix or data.frame. This must
#' contain at least three columns to identify the student, the item, and
#' the response. Additionally, it can include a column for the weight of
#' the items.
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param Sid Specify the column number containing the student ID label vector.
#' @param Qid Specify the column number containing the Question label vector.
#' @param Resp Specify the column number containing the Response value vector.
#' @param w Specify the column number containing the weight vector.
#' @param response.type Character string specifying the type of response data:
#'   "binary" for dichotomous data,
#'   "ordinal" for ordered polytomous data,
#'   "rated" for polytomous data with correct answers,
#'   "nominal" for unordered polytomous data.
#'   If NULL (default), the type is automatically detected.
#' @param CA A numeric vector specifying the correct answers for rated polytomous data.
#' Required when response.type is "rated".
#' @return
#' \describe{
#' \item{U}{For binary response data. A matrix with rows representing the sample size and columns
#'  representing the number of items, where elements are either 0 or 1. \eqn{u_{ij}=1} indicates
#'  that student i correctly answered item j, while \eqn{u_{ij}=0} means that student i answered
#'  item j incorrectly.}
#' \item{Q}{For polytomous response data. A matrix with rows representing the sample size and columns
#'  representing the number of items, where elements are non-negative integers. When input data is
#'  in factor format, the factor levels are converted to consecutive integers starting from 1.}
#' \item{ID}{The ID label given by the designated column or function.}
#' \item{ItemLabel}{The item names given by the provided column names or function.}
#' \item{Z}{Missing indicator matrix. \eqn{z_{ij}=1} indicates that item j is presented to Student i,
#' while \eqn{z_{ij}=0} indicates item j is NOT presented to Student i.}
#' \item{w}{Item weight vector}
#' \item{response.type}{Character string indicating the type of response data:
#'  "binary", "ordinal", "rated", or "nominal"}
#' \item{CategoryLabel}{List containing the original factor labels when polytomous responses
#'  are provided as factors. NULL if no factor data is present.}
#' \item{categories}{Numeric vector containing the number of response categories for each item.}
#' \item{CA}{For rated polytomous data, a numeric vector of correct answers. NULL for other types.}
#' }
#' @export
#'
longdataFormat <- function(data, na = NULL,
                           Sid = NULL, Qid = NULL,
                           Resp = NULL, w = NULL,
                           response.type = NULL, CA = NULL) {
  # Check if already formatted
  if (inherits(data, "exametrika")) {
    return(data)
  }

  # Basic input checks
  if (!is.matrix(data) && !is.data.frame(data)) {
    stop("Data must be matrix or data.frame")
  }
  if (is.null(Sid)) {
    stop("Column number for identifier for Student must be specified.")
  }
  if (is.null(Qid)) {
    stop("Column number for identifier for Questions must be specified.")
  }
  if (is.null(Resp)) {
    stop("Column number for response pattern must be specified.")
  }

  # Extract vectors based on data type
  if (is.data.frame(data)) {
    Sid_vec <- data[[Sid]]
    Qid_vec <- data[[Qid]]
    Resp_vec <- data[[Resp]]
  } else {
    Sid_vec <- data[, Sid]
    Qid_vec <- data[, Qid]
    Resp_vec <- data[, Resp]
  }

  # Process Student IDs
  if (!is.numeric(Sid_vec)) {
    Sid_vec <- as.factor(Sid_vec)
    Sid_label <- unique(levels(Sid_vec))
    Sid_num <- as.numeric(Sid_vec)
  } else {
    Sid_num <- Sid_vec
    Sid_label <- unique(paste0("Student", Sid_num))
  }
  if (any(duplicated(Sid_vec))) {
    duplicated_ids <- Sid_vec[duplicated(Sid_vec)]
    stop(paste("Duplicated IDs found:", paste(unique(duplicated_ids), collapse = ",")))
  }

  # Process Question IDs
  if (!is.numeric(Qid_vec)) {
    Qid_vec <- as.factor(Qid_vec)
    Qid_label <- unique(levels(Qid_vec))
    Qid_num <- as.numeric(Qid_vec)
  } else {
    Qid_num <- Qid_vec
    Qid_label <- unique(paste0("Q", Qid_vec))
  }

  # Process Response vector and determine response type if not specified
  CategoryLabel <- NULL
  if (is.factor(Resp_vec)) {
    CategoryLabel <- levels(Resp_vec)
    Resp_vec <- as.numeric(Resp_vec)
  } else {
    Resp_vec <- as.numeric(Resp_vec)
  }

  if (is.null(response.type)) {
    if (all(Resp_vec[!is.na(Resp_vec)] %in% c(0, 1))) {
      response.type <- "binary"
    } else {
      response.type <- if (!is.null(CA)) "rated" else "nominal"
    }
  } else {
    response.type <- match.arg(
      response.type,
      c("binary", "ordinal", "rated", "nominal")
    )
  }

  # Validate responses based on response_type
  if (response.type == "binary" &&
    !all(Resp_vec[!is.na(Resp_vec)] %in% c(0, 1))) {
    stop("Binary response type specified but data contains non-binary values")
  }

  if (response.type == "rated") {
    if (is.null(CA)) {
      stop("CA must be specified when response.type is rated")
    }
    # Verify CA length matches number of unique items
    if (length(CA) != length(unique(Qid_vec))) {
      stop("Length of CA must match number of items")
    }
    # Warn if a CA category is not observed (e.g., no correct responders);
    # such items are still valid (treated as all-incorrect downstream).
    unsupported <- integer(0)
    for (i in seq_along(CA)) {
      item_responses <- unique(Resp_vec[Qid_num == i])
      item_responses <- item_responses[!is.na(item_responses)]
      if (!CA[i] %in% item_responses) unsupported <- c(unsupported, i)
    }
    if (length(unsupported) > 0) {
      warning(sprintf(
        "CA for item(s) %s not observed in responses (no correct responders); treated as all-incorrect.",
        paste(unsupported, collapse = ", ")
      ))
    }
  }

  # Create response matrix
  response_matrix <- matrix(NA, ncol = max(Qid_num), nrow = max(Sid_num))
  for (i in 1:length(Resp_vec)) {
    response_matrix[Sid_num[i], Qid_num[i]] <- Resp_vec[i]
  }

  # Handle NA values
  if (!is.null(na)) {
    response_matrix[response_matrix == na] <- -1
  }
  response_matrix[is.na(response_matrix)] <- -1

  # Create missing indicator matrix
  Z <- ifelse(response_matrix == -1, 0, 1)

  # Process weights
  if (is.null(w)) {
    w <- rep(1, NCOL(response_matrix))
  } else {
    if (is.data.frame(data)) {
      w_vec <- data[[w]]
    } else {
      w_vec <- data[, w]
    }
    w <- w_vec[unique(Qid_num)]
  }

  # Calculate categories for each item
  categories <- apply(response_matrix, 2, function(x) length(unique(x[x != -1])))

  # Create return list
  ret_list <- list(
    ID = Sid_label,
    ItemLabel = Qid_label,
    Z = Z,
    w = w,
    response.type = response.type,
    categories = categories
  )

  # Add response matrix with appropriate name
  if (response.type == "binary") {
    ret_list$U <- response_matrix
  } else {
    ret_list$Q <- response_matrix
    if (!is.null(CategoryLabel)) {
      ret_list$CategoryLabel <- list(response = CategoryLabel)
    }
    if (!is.null(CA)) {
      ret_list$CA <- CA
    }
  }

  # Return with appropriate class structure
  ret <- structure(ret_list,
    class = c("exametrika", "exametrikaData")
  )
  return(ret)
}

# Comprehensive Tests for dataFormat Function

test_that("dataFormat handles basic input types correctly", {
  # Matrix input with 25 rows
  set.seed(123)
  matrix_data <- matrix(sample(c(0, 1), 25 * 4, replace = TRUE), nrow = 25, ncol = 4)
  result <- dataFormat(matrix_data)
  expect_s3_class(result, "exametrika")

  # Data.frame input with 25 rows
  set.seed(124)
  df_data <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(0, 1), 25, replace = TRUE),
    item2 = sample(c(0, 1), 25, replace = TRUE)
  )
  result <- dataFormat(df_data)
  expect_s3_class(result, "exametrika")

  # Already formatted exametrika object (early return)
  result2 <- dataFormat(result)
  expect_identical(result, result2)
})

test_that("dataFormat handles missing values correctly", {
  # NA missing values with 25 rows
  set.seed(125)
  data_na <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(1, 2, 3, NA), 25, replace = TRUE),
    item2 = sample(c(1, 2, 3, NA), 25, replace = TRUE)
  )
  result <- dataFormat(data_na)
  expect_s3_class(result, "exametrika")
  expect_true(any(result$Z == 0)) # Should have missing indicators

  # Specific missing value (99) with 25 rows
  set.seed(126)
  data_99 <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(1, 2, 3, 99), 25, replace = TRUE),
    item2 = sample(c(1, 2, 3, 99), 25, replace = TRUE)
  )
  result <- dataFormat(data_99, na = 99)
  expect_s3_class(result, "exametrika")
  expect_true(any(result$Z == 0))

  # Multiple missing value types with 25 rows
  set.seed(127)
  data_mixed <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(1, 2, 3, NA, 99), 25, replace = TRUE),
    item2 = sample(c(1, 2, 3, NA, 99), 25, replace = TRUE)
  )
  result <- dataFormat(data_mixed, na = 99)
  expect_s3_class(result, "exametrika")

  # All same values in one column (zero variance) with 25 rows
  data_all_missing <- data.frame(
    id = paste0("S", 1:25),
    item1 = rep(1, 25), # All same value = zero variance
    item2 = sample(1:5, 25, replace = TRUE)
  )
  result <- dataFormat(data_all_missing)
  expect_s3_class(result, "exametrika")
})

test_that("dataFormat handles ID columns correctly", {
  # No ID column specified (default id=1)
  # Case 1: First column has consecutive unique IDs -> used as ID
  set.seed(128)
  data_unique_first <- data.frame(
    student_id = 1:25,
    item1 = sample(1:5, 25, replace = TRUE),
    item2 = sample(1:5, 25, replace = TRUE)
  )
  result <- dataFormat(data_unique_first)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 2) # Should have 2 response columns

  # Case 2: First column looks like response data -> all columns are response data
  set.seed(129)
  data_dup_first <- data.frame(
    response1 = sample(1:4, 25, replace = TRUE), # Response-like data
    item1 = sample(1:4, 25, replace = TRUE),
    item2 = sample(1:4, 25, replace = TRUE)
  )
  result <- dataFormat(data_dup_first)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 3) # Should have 3 response columns

  # Explicit ID column specification
  set.seed(130)
  data_explicit_id <- data.frame(
    item1 = sample(1:5, 25, replace = TRUE),
    item2 = sample(1:5, 25, replace = TRUE),
    student_id = paste0("S", 1:25)
  )
  result <- dataFormat(data_explicit_id, id = 3)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 2)
  expect_equal(length(result$ID), 25)

  # Numeric ID
  set.seed(133)
  data_numeric_id <- data.frame(
    student_id = 101:115,
    item1 = sample(1:5, 15, replace = TRUE),
    item2 = sample(1:5, 15, replace = TRUE)
  )
  result <- dataFormat(data_numeric_id)
  expect_s3_class(result, "exametrika")

  # Factor ID
  set.seed(134)
  data_factor_id <- data.frame(
    student_id = factor(paste0("Student", 1:15)),
    item1 = sample(1:5, 15, replace = TRUE),
    item2 = sample(1:5, 15, replace = TRUE)
  )
  result <- dataFormat(data_factor_id)
  expect_s3_class(result, "exametrika")

  # ID column out of range -> auto-generate IDs
  set.seed(135)
  data_no_id <- data.frame(
    item1 = sample(1:5, 15, replace = TRUE),
    item2 = sample(1:5, 15, replace = TRUE)
  )
  expect_error(dataFormat(data_no_id, id = 5), "ID column number exceeds the number of columns in data") # Column 5 doesn't exist
})

test_that("dataFormat handles binary data correctly", {
  # Perfect 0/1 binary with 25 rows
  set.seed(140)
  binary_data <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(0, 1), 25, replace = TRUE),
    item2 = sample(c(0, 1), 25, replace = TRUE),
    item3 = sample(c(0, 1), 25, replace = TRUE)
  )
  result <- dataFormat(binary_data)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")
  expect_true("U" %in% names(result))

  # Binary with missing values (25 rows)
  set.seed(141)
  binary_na <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(0, 1, NA), 25, replace = TRUE),
    item2 = sample(c(0, 1, NA), 25, replace = TRUE)
  )
  result <- dataFormat(binary_na)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")

  # Logical input (TRUE/FALSE)
  set.seed(142)
  logical_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(TRUE, FALSE), 15, replace = TRUE),
    item2 = sample(c(TRUE, FALSE), 15, replace = TRUE)
  )
  result <- dataFormat(logical_data)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")
})

test_that("dataFormat handles ordinal data correctly", {
  # Consecutive integers starting from 1 (25 rows)
  set.seed(142)
  ordinal_1234 <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(1:4, 25, replace = TRUE),
    item2 = sample(1:4, 25, replace = TRUE),
    item3 = sample(1:4, 25, replace = TRUE)
  )
  result <- dataFormat(ordinal_1234)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")
  expect_true("Q" %in% names(result))

  # Non-consecutive integers (missing categories) (25 rows)
  set.seed(143)
  ordinal_gaps <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(c(1, 2, 4, 5), 25, replace = TRUE), # Missing 3
    item2 = sample(c(1, 2, 4, 5), 25, replace = TRUE),
    item3 = sample(c(1, 2, 4, 5), 25, replace = TRUE)
  )
  result <- dataFormat(ordinal_gaps)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")

  # Starting from 0
  set.seed(143)
  ordinal_zero <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(0:2, 15, replace = TRUE),
    item2 = sample(0:2, 15, replace = TRUE),
    item3 = sample(0:2, 15, replace = TRUE)
  )
  result <- dataFormat(ordinal_zero)
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")

  # Explicit ordinal specification
  set.seed(144)
  mixed_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(1, 3, 5), 15, replace = TRUE), # Might be classified as nominal without explicit type
    item2 = sample(c(2, 4, 6), 15, replace = TRUE)
  )
  result <- dataFormat(mixed_data, response.type = "ordinal")
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")
})

test_that("dataFormat handles rated data correctly", {
  # Rated data with correct answers (25 rows)
  set.seed(144)
  rated_data <- data.frame(
    id = paste0("S", 1:25),
    item1 = sample(1:4, 25, replace = TRUE),
    item2 = sample(1:4, 25, replace = TRUE),
    item3 = sample(1:4, 25, replace = TRUE)
  )
  result <- dataFormat(rated_data, response.type = "rated", CA = c(3, 2, 4))
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "rated")
  expect_true("Q" %in% names(result))
  expect_true("U" %in% names(result))
  expect_equal(result$CA, c(3, 2, 4))

  # Check that U matrix is correctly generated
  expect_true(all(result$U %in% c(0, 1, -1)))
})

test_that("dataFormat handles nominal data correctly", {
  # Mixed non-ordinal data that will cause variance issue (25 rows)
  set.seed(145)
  nominal_data <- data.frame(
    id = paste0("S", 1:25),
    item1 = rep("A", 25), # All same value - no variance
    item2 = rep("X", 25), # All same value - no variance
    item3 = rep("P", 25) # All same value - no variance
  )
  suppressWarnings(expect_error(dataFormat(nominal_data, response.type = "nominal"), "All items have no variance and were excluded"))

  # Large gaps in numeric data (should be nominal)
  set.seed(146)
  large_gaps <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(c(1, 10, 100), 15, replace = TRUE),
    item2 = sample(c(5, 50, 500), 15, replace = TRUE)
  )
  result <- dataFormat(large_gaps)
  expect_s3_class(result, "exametrika")
  # Should be classified as nominal due to large gaps
})

test_that("dataFormat handles Factor inputs correctly", {
  # Ordered factor
  set.seed(147)
  ordered_factor_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = ordered(sample(c("Low", "Med", "High"), 15, replace = TRUE), levels = c("Low", "Med", "High")),
    item2 = ordered(sample(c("Low", "Med", "High"), 15, replace = TRUE), levels = c("Low", "Med", "High"))
  )
  result <- dataFormat(ordered_factor_data, response.type = "ordinal")
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")
  expect_true(!is.null(result$CategoryLabel))

  # Unordered factor
  set.seed(148)
  factor_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = factor(sample(c("Red", "Blue", "Green"), 15, replace = TRUE)),
    item2 = factor(sample(c("Cat", "Dog", "Bird"), 15, replace = TRUE))
  )
  result <- dataFormat(factor_data, response.type = "nominal")
  expect_s3_class(result, "exametrika")
  expect_true(!is.null(result$CategoryLabel))

  # Numeric factor labels
  set.seed(149)
  numeric_factor <- data.frame(
    id = paste0("S", 1:15),
    item1 = factor(sample(c("1", "2", "3"), 15, replace = TRUE)),
    item2 = factor(sample(c("2", "3", "1"), 15, replace = TRUE))
  )
  result <- dataFormat(numeric_factor)
  expect_s3_class(result, "exametrika")
})

test_that("dataFormat handles weight vectors correctly", {
  set.seed(150)
  test_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:5, 15, replace = TRUE),
    item2 = sample(1:5, 15, replace = TRUE)
  )

  # Default weights (should be all 1s)
  result <- dataFormat(test_data)
  expect_s3_class(result, "exametrika")
  expect_equal(result$w, c(1, 1))

  # Custom weights
  result <- dataFormat(test_data, w = c(2, 3))
  expect_s3_class(result, "exametrika")
  expect_equal(result$w, c(2, 3))
})

test_that("dataFormat handles missing indicator matrix Z correctly", {
  set.seed(151)
  test_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:5, 15, replace = TRUE),
    item2 = sample(1:5, 15, replace = TRUE)
  )

  # Custom Z matrix
  Z_matrix <- matrix(sample(c(0, 1), 15 * 2, replace = TRUE), nrow = 15, ncol = 2)
  result <- dataFormat(test_data, Z = Z_matrix)
  expect_s3_class(result, "exametrika")
  expect_true(any(result$Z == 0))
})

test_that("dataFormat handles mixed column types", {
  # Some binary, some ordinal columns
  set.seed(152)
  mixed_data <- data.frame(
    id = paste0("S", 1:15),
    binary1 = sample(c(0, 1), 15, replace = TRUE), # Binary
    binary2 = sample(c(0, 1), 15, replace = TRUE), # Binary
    ordinal1 = sample(1:4, 15, replace = TRUE), # Ordinal
    ordinal2 = sample(1:4, 15, replace = TRUE) # Ordinal
  )

  # Should be classified based on majority or most restrictive
  result <- dataFormat(mixed_data)
  expect_s3_class(result, "exametrika")
  # The exact classification depends on the algorithm, but should be valid
  expect_true(result$response.type %in% c("binary", "ordinal", "nominal"))
})

test_that("dataFormat automatic detection works correctly", {
  # Clear binary case (25 rows)
  set.seed(150)
  binary_auto <- matrix(sample(c(0, 1), 25 * 3, replace = TRUE), nrow = 25, ncol = 3)
  result <- dataFormat(binary_auto)
  expect_equal(result$response.type, "binary")

  # Clear ordinal case (25 rows)
  set.seed(151)
  ordinal_auto <- matrix(sample(1:4, 25 * 4, replace = TRUE), nrow = 25, ncol = 4)
  result <- dataFormat(ordinal_auto)
  expect_equal(result$response.type, "ordinal")

  # Boundary case: insufficient data should error
  two_cat <- matrix(c(1, 2, 2, 1, 1, 2), nrow = 2, ncol = 3)
  expect_error(dataFormat(two_cat), "Data must have at least 10 rows")
})

test_that("dataFormat handles edge cases correctly", {
  # Single row should error due to insufficient data
  single_row <- data.frame(
    id = 1,
    item1 = 1,
    item2 = 2
  )
  expect_error(dataFormat(single_row), "Data must have at least 10 rows")

  # Single column (after ID processing)
  set.seed(153)
  single_col <- data.frame(
    id = paste0("S", 1:15),
    item1 = sample(1:5, 15, replace = TRUE)
  )
  result <- dataFormat(single_col)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 1)

  # All same values (zero variance) - should be excluded
  set.seed(156)
  zero_var <- data.frame(
    id = paste0("S", 1:15),
    item1 = rep(1, 15), # Zero variance
    item2 = sample(1:5, 15, replace = TRUE) # Normal variance
  )
  result <- dataFormat(zero_var)
  expect_s3_class(result, "exametrika")
  expect_equal(ncol(result$Q), 2) # Both columns remain (zero variance might not be excluded)
})

test_that("dataFormat error handling works correctly", {
  # Invalid response.type
  test_data <- data.frame(id = 1:3, item1 = c(1, 2, 3))
  expect_error(dataFormat(test_data, response.type = "invalid"))

  # Rated without CA
  expect_error(dataFormat(test_data, response.type = "rated"))

  # CA length mismatch
  expect_error(dataFormat(test_data, response.type = "rated", CA = c(1, 2, 3, 4)))

  # CA category not observed → now warns (was error before v1.13.1)
  test_data10 <- data.frame(id = 1:10, item1 = sample(1:3, 10, replace = TRUE))
  expect_warning(
    dataFormat(test_data10, response.type = "rated", CA = c(5)),
    "not observed in responses"
  )

  # Duplicated IDs (25 rows with duplicates)
  dup_id_data <- data.frame(
    id = c(rep("S1", 5), rep("S2", 5), paste0("S", 3:17)), # 5 duplicates of S1 and S2, total 25 rows
    item1 = sample(1:5, 25, replace = TRUE)
  )
  expect_error(dataFormat(dup_id_data), "Duplicated IDs found")

  # Invalid Z matrix
  expect_error(dataFormat(test_data, Z = matrix(c(1, 2, 3, 4, 5, 6), nrow = 3)))

  # Invalid data type
  expect_error(dataFormat(list(a = 1, b = 2)))
})

test_that("dataFormat preserves data integrity", {
  # Test that output maintains input relationships
  set.seed(154)
  test_data <- data.frame(
    id = paste0("S", 1:15),
    math = sample(1:5, 15, replace = TRUE),
    reading = sample(1:5, 15, replace = TRUE),
    science = sample(1:5, 15, replace = TRUE)
  )
  result <- dataFormat(test_data)

  # Check dimensions
  expect_equal(nrow(result$Q), 15)
  expect_equal(ncol(result$Q), 3)
  expect_equal(length(result$ID), 15)
  expect_equal(length(result$ItemLabel), 3)
  expect_equal(length(result$categories), 3)

  # Check that categories are correctly calculated
  expect_true(all(result$categories >= 1))

  # Check Z matrix dimensions and values
  expect_equal(dim(result$Z), dim(result$Q))
  expect_true(all(result$Z %in% c(0, 1)))
})

test_that("dataFormat works with various ordinal data patterns", {
  # Simulate ordinal data (Likert-like)
  set.seed(42)
  ordinal_data <- matrix(sample(1:5, 100, replace = TRUE), nrow = 20, ncol = 5)
  colnames(ordinal_data) <- paste0("Item", 1:5)
  result <- dataFormat(ordinal_data, response.type = "ordinal")
  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")

  # Simulate larger dataset with mixed response patterns
  set.seed(123)
  larger_data <- matrix(sample(1:4, 500, replace = TRUE), nrow = 50, ncol = 10)
  colnames(larger_data) <- paste0("Q", 1:10)
  result2 <- dataFormat(larger_data, response.type = "ordinal")
  expect_s3_class(result2, "exametrika")
  expect_equal(ncol(result2$Q), 10)
})

test_that("dataFormat handles Japanese/Unicode labels", {
  # Japanese factor labels
  set.seed(155)
  japanese_data <- data.frame(
    id = paste0("S", 1:15),
    item1 = factor(sample(c("はい", "いいえ", "わからない"), 15, replace = TRUE)),
    item2 = factor(sample(c("良い", "普通", "悪い"), 15, replace = TRUE))
  )
  result <- dataFormat(japanese_data, response.type = "nominal")
  expect_s3_class(result, "exametrika")
  expect_true(!is.null(result$CategoryLabel))
})

test_that("rated dataFormat warns (not errors) when CA category is unobserved", {
  # 30 obs × 5 items × ncat=5
  set.seed(42)
  Q <- matrix(sample(1:5, 30 * 5, replace = TRUE), nrow = 30, ncol = 5)
  # item 3 を 1〜4 に制限し、CA[3]=5 を未観測にする
  Q[, 3] <- sample(1:4, 30, replace = TRUE)
  CA <- c(1, 2, 5, 3, 4) # CA[3]=5 だが Q[,3] には 5 が無い
  colnames(Q) <- paste0("Q", 1:5)

  # warning が出るが処理は続行
  expect_warning(
    res <- dataFormat(Q, response.type = "rated", CA = CA),
    "item.*3.*not observed"
  )
  expect_s3_class(res, "exametrika")
  expect_equal(res$response.type, "rated")
  # item 3 列は全 0（誰も正解していない）
  expect_equal(sum(res$U[, 3]), 0)
  # 他の item は通常通り
  expect_true(any(res$U[, 1] == 1))

  # 複数 item で未観測の場合、まとめて warning
  Q2 <- Q
  Q2[, 1] <- sample(1:4, 30, replace = TRUE) # item 1 も
  CA2 <- c(5, 2, 5, 3, 4)
  expect_warning(
    dataFormat(Q2, response.type = "rated", CA = CA2),
    "item.*1.*3"
  )
})

# Note: longdataFormat 経路も同じ修正 (line 555 周辺) を適用済み。
# 機構は matrix 経路と同一なので上記テストでカバー。

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)

## ----library, message=FALSE, warning=FALSE------------------------------------
library(exametrika)

## ----model-lca, message=FALSE, warning=FALSE----------------------------------
LCA(J15S500, ncls = 5)

## ----results-lca-membership, message=FALSE, warning=FALSE---------------------
result.LCA <- LCA(J15S500, ncls = 5)
head(result.LCA$Students)

## ----plot-lca, fig.width=7, fig.height=5, message=FALSE, warning=FALSE--------
plot(result.LCA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LCA, type = "CMP", students = 1:9, nc = 3, nr = 3)
plot(result.LCA, type = "TRP")
plot(result.LCA, type = "LCD")

## ----model-lra, message=FALSE, warning=FALSE----------------------------------
LRA(J15S500, nrank = 6)

## ----results-lra-membership, message=FALSE, warning=FALSE---------------------
result.LRA <- LRA(J15S500, nrank = 6)
head(result.LRA$Students)

## ----plot-lra, fig.width=7, fig.height=5, message=FALSE, warning=FALSE--------
plot(result.LRA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LRA, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.LRA, type = "TRP")
plot(result.LRA, type = "LRD")

## ----model-lra-ordinal, eval=FALSE--------------------------------------------
# result.LRAord <- LRA(J15S3810, nrank = 3, mic = TRUE)

## ----plot-lra-ordinal-score, eval=FALSE---------------------------------------
# plot(result.LRAord, type = "ScoreFreq")
# plot(result.LRAord, type = "ScoreRank")

## ----plot-lra-ordinal-icbr, eval=FALSE----------------------------------------
# plot(result.LRAord, type = "ICBR", items = 1:4, nc = 2, nr = 2)
# plot(result.LRAord, type = "ICRP", items = 1:4, nc = 2, nr = 2)

## ----plot-lra-ordinal-rmp, eval=FALSE-----------------------------------------
# plot(result.LRAord, type = "RMP", students = 1:9, nc = 3, nr = 3)

## ----lra-rated, eval=FALSE----------------------------------------------------
# result.LRArated <- LRA(J35S5000, nrank = 10, mic = TRUE)

## ----plot-lra-rated-score, eval=FALSE-----------------------------------------
# plot(result.LRArated, type = "ScoreFreq")
# plot(result.LRArated, type = "ScoreRank")

## ----plot-lra-rated-icrp, eval=FALSE------------------------------------------
# plot(result.LRArated, type = "ICRP", items = 1:4, nc = 2, nr = 2)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)

## ----install, eval=FALSE------------------------------------------------------
# # Install from CRAN
# install.packages("exametrika")
# 
# # Or install the development version from GitHub
# if (!require("devtools")) install.packages("devtools")
# devtools::install_github("kosugitti/exametrika")

## ----library, message=FALSE, warning=FALSE------------------------------------
library(exametrika)

## ----example-data-format------------------------------------------------------
# Format raw data for analysis
data <- dataFormat(J15S500)
str(data)

## ----test-statistics, message=FALSE, warning=FALSE----------------------------
TestStatistics(J15S500)

## ----item-statistics, message=FALSE, warning=FALSE----------------------------
ItemStatistics(J15S500)

## ----ctt, message=FALSE, warning=FALSE----------------------------------------
CTT(J15S500)

#' @rdname LRA
#' @section Ordinal Data Method:
#' \code{LRA.ordinal} analyzes ordered categorical data with multiple thresholds,
#' such as Likert-scale responses or graded items.
#'
#' @param trapezoidal Specifies the height of both tails when using a trapezoidal
#' prior distribution. Must be less than 1/nrank. The default value is 0, which
#' results in a uniform prior distribution.
#' @param eps Convergence threshold for parameter updates. Default is 1e-4.
#'
#' @return
#' For ordinal data (\code{LRA.ordinal}), the returned list additionally includes:
#' \describe{
#' \item{msg}{A character string indicating the model type. }
#' \item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
#' \item{ScoreReport}{Descriptive statistics of test performance, including sample size,
#'   test length, central tendency, variability, distribution characteristics, and reliability.}
#' \item{ItemReport}{Basic statistics for each item including category proportions and item-total correlations.}
#' \item{ICBR}{Item Category Boundary Reference matrix showing cumulative probabilities for rank-category combinations.}
#' \item{ICRP}{Item Category Reference Profile matrix showing probability of response in each category by rank.}
#' \item{ScoreRankCorr}{Spearman's correlation between test scores and estimated ranks.}
#' \item{RankQuantCorr}{Spearman's correlation between estimated ranks and quantile groups.}
#' \item{ScoreRank}{Contingency table of raw scores by estimated ranks.}
#' \item{ScoreMembership}{Expected rank memberships for each raw score.}
#' \item{RankQuantile}{Cross-tabulation of rank frequencies and quantile groups.}
#' \item{MembQuantile}{Cross-tabulation of rank membership probabilities and quantile groups.}
#' \item{CatQuant}{Response patterns across item categories and quantile groups.}
#' }
#'
#' @examples
#' \donttest{
#' # Ordinal data example
#' # Fit a Latent Rank Analysis model with 3 ranks to ordinal data
#' result.LRAord <- LRA(J15S3810, nrank = 3, mic = TRUE)
#'
#' # Plot score distributions
#' plot(result.LRAord, type = "ScoreFreq")
#' plot(result.LRAord, type = "ScoreRank")
#'
#' # Plot category response patterns for items 1-6
#' plot(result.LRAord, type = "ICBR", items = 1:6, nc = 3, nr = 2)
#' plot(result.LRAord, type = "ICRP", items = 1:6, nc = 3, nr = 2)
#' }
#'
#' @importFrom stats cor
#' @export
#'
LRA.ordinal <- function(U,
                        nrank = 2,
                        mic = FALSE,
                        maxiter = 100,
                        trapezoidal = 0,
                        eps = 1e-4,
                        verbose = FALSE, ...) {
  ## check trapezoidal prior
  if (trapezoidal > 0) {
    if (trapezoidal > 1 / nrank) {
      stop("Trapezoidal prior hem value must be smaller than 1/nrank")
    }
  }

  ScoreReport <- ScoreReport(U)
  ItemReport <- ItemReport(U)
  score <- rowSums(U$Z * U$Q)
  maxscore <- max(score)
  scoredist <- table(factor(score, levels = 0:maxscore))
  nobs <- nrow(U$Q)
  nitems <- ncol(U$Q)
  nquan <- nrank
  const <- 1e-6

  # Prepare ---------------------------------------------------------
  score <- rowSums(U$Z * U$Q)
  zzzTotal <- apply(U$Z, 2, sum)


  # Calc Frequency
  # NOTE: Use lapply(seq_len, ...) instead of lapply(apply(..., 2, unique), ...)
  # because apply() returns a matrix (not a list) when all columns have the
  # same number of unique values, causing lapply to iterate over individual
  # elements rather than per-column vectors.
  category999 <- lapply(seq_len(nitems), function(j) sort(unique(U$Q[, j])))
  # Calc Frequency excluding missing
  category <- lapply(
    category999,
    function(x) x[x != -1]
  )
  # number of categories
  ncat999 <- sapply(category999, length)
  # number of categories excluding missing
  ncat <- sapply(category, length)

  ## Check for mixed category counts
  ## LRA.ordinal uses fixed-stride matrix indexing (nitems * max(ncat)) that

  ## assumes all items have the same number of categories. Mixed category counts
  ## cause dimension mismatches in the saturation/restricted model matrices.
  if (length(unique(ncat)) > 1) {
    stop(
      "LRA.ordinal does not support items with different numbers of categories.\n",
      "  Found category counts: ", paste(unique(ncat), collapse = ", "),
      " across items.\n",
      "  All items must have the same number of response categories.\n",
      "  Note: LRA.rated supports mixed category counts via its list-based design.\n",
      "  Consider using Biclustering.ordinal, which also supports mixed categories."
    )
  }

  # Frequency table of categories
  catfreq999 <- lapply(seq_len(nitems), function(j) table(U$Q[, j]))
  catfreq <- lapply(catfreq999, function(x) x[names(x) != "-1"])


  UU <- array(NA, dim = c(nobs, nitems, max(ncat)))
  YY <- array(0, dim = c(nobs, nitems, max(ncat) - 1))
  uuMat <- matrix(NA, nrow = nobs, ncol = nitems * max(ncat))
  for (i in 1:nobs) {
    for (j in 1:nitems) {
      for (k in 1:ncat[j]) {
        UU[i, j, k] <- ifelse(U$Q[i, j] == category[[j]][k], 1, 0)
      }
      for (k in 2:ncat[j]) {
        YY[i, j, k - 1] <- ifelse((U$Z[i, j] * U$Q[i, j]) >= category[[j]][k], 1, 0)
      }
    }
    uuMat[i, ] <- as.vector(t(UU[i, , ]))
  }


  quantileScore <- quantile(score, probs = (1:(nquan - 1)) / nquan)
  quantileRank <- rowSums(outer(score, quantileScore, ">")) + 1


  quanRefmat <- array(NA, dim = c(nitems, max(ncat) - 1, nquan))
  catquanRefmat <- array(NA, dim = c(nitems, max(ncat), nquan))

  for (j in 1:nitems) {
    for (q in 1:nquan) {
      quanRefmat[j, , q] <- apply(YY[quantileRank == q, j, ], 2, sum) / apply(U$Z[quantileRank == q, ], 2, sum)[j]
      catquanRefmat[j, , q] <- apply(UU[quantileRank == q, j, ], 2, sum) / apply(U$Z[quantileRank == q, ], 2, sum)[j]
    }
  }


  ## Category Quantile Report

  SelectRatio <- NULL
  ref_mat <- matrix(NA, nrow = sum(ncat), ncol = nquan)
  colnames(ref_mat) <- paste0("Q", 1:nquan)
  for (i in 1:nitems) {
    item_name <- U$ItemLabel[i]
    categories <- dimnames(catfreq[[i]])[[1]]
    ratios <- as.vector(catfreq[[i]] / zzzTotal[i])

    ref_mat[((i - 1) * ncat[i] + 1):((i * ncat[i])), ] <- catquanRefmat[i, , ]

    item_data <- data.frame(
      item = item_name,
      category = categories,
      selectRatio = ratios
    )

    SelectRatio <- rbind(SelectRatio, item_data)
  }

  SelectRatioTable <- cbind(SelectRatio, ref_mat)

  # Algorithm -------------------------------------------------------

  ## Filter
  Fil <- create_filter_matrix(nrank)

  ## Prior
  if (trapezoidal > 0) {
    log_prior <- c(
      trapezoidal,
      rep((1 - 2 * trapezoidal) / (nrank - 2), nrank - 2),
      trapezoidal
    )
  } else {
    log_prior <- rep(1, nrank)
  }
  logprior_NQmat <- matrix(rep(log(log_prior), nobs), byrow = TRUE, nrow = nobs)

  ## Design vector/Matrix
  designfunc <- function(x, y) 1

  create_upper_diagonal_matrix <- function(func, n) {
    matrix <- matrix(0, n, n)
    for (i in 1:n) {
      for (j in 1:n) {
        if (i <= j) matrix[i, j] <- func(i, j)
      }
    }
    return(matrix)
  }

  design0 <- sapply(1:nitems, function(j) sum(ncat[1:j]))
  design1 <- cbind(design0 - ncat[1] + 1, design0)

  design2 <- lapply(1:nitems, function(j) {
    seq(design1[j, 1], design1[j, 2])
  })

  design3 <- do.call(rbind, lapply(1:nitems, function(j) {
    cbind(rep(j, ncat[j]), design2[[j]])
  }))

  design4 <- matrix(0, nrow = nitems, ncol = sum(ncat))
  for (i in 1:nrow(design3)) {
    design4[design3[i, 1], design3[i, 2]] <- 1
  }

  design5 <- do.call(rbind, lapply(1:nitems, function(j) {
    matrix(rep(design4[j, ], ncat[j]), nrow = ncat[j], byrow = TRUE)
  }))

  design6_matrix <- create_upper_diagonal_matrix(designfunc, sum(ncat))
  design6 <- design5 * design6_matrix


  # Saturation Model -------------------------------------------------
  ij_log_lik_satu <- -10
  iter_satu <- 0
  refbox_satu <- array(0, dim = c(nitems, max(ncat) - 1, nitems))
  for (j in 1:nitems) {
    for (k in 1:(ncat[j] - 1)) {
      for (q in 1:nitems) {
        refbox_satu[j, k, q] <- 0.1 + 0.8 / (1 + exp(-q + nitems * k / ncat[j]))
      }
    }
  }

  refbox111_satu <- array(1, dim = c(nitems, max(ncat), nitems))
  refbox000_satu <- array(0, dim = c(nitems, max(ncat), nitems))
  refbox111_satu[, 2:max(ncat), ] <- refbox_satu
  refbox000_satu[, 1:(max(ncat) - 1), ] <- refbox_satu
  catrefbox_satu <- refbox111_satu - refbox000_satu
  catRefMat_satu <- matrix(0, nrow = (nitems * max(ncat)), ncol = nitems)
  for (i in 1:nitems) {
    l <- (i - 1) * ncat[i] + 1
    catRefMat_satu[l:(l + ncat[i] - 1), ] <- catrefbox_satu[i, , ]
  }
  ## EM Algorithm
  if (verbose) {
    message("Starting EM estimation for Saturation Model...")
  }
  converge_satu <- TRUE
  FLG <- TRUE
  while (FLG) {
    old_log_like_satu <- ij_log_lik_satu
    ## Estep
    nume_satu <- exp(uuMat %*% log(catRefMat_satu + const))
    denom_satu <- rowSums(nume_satu)
    rankProf_satu <- nume_satu / denom_satu

    ## Mstep
    refMatcore_satu <- t(uuMat) %*% rankProf_satu
    refMat111_satu <- design6 %*% refMatcore_satu / design5 %*% refMatcore_satu
    delete_rows <- design0 - ncat[1] + 1
    refMat_satu <- refMat111_satu[-delete_rows, ]
    refMat000_satu <- rbind(refMat111_satu[-1, ], refMat111_satu[1, ])
    refMat000_satu[design0, ] <- 0
    catRefMat_satu <- refMat111_satu - refMat000_satu
    ### Log Lik for Saturation Model
    log_lik_satu <- sum(rankProf_satu * log(nume_satu + const))
    ij_log_lik_satu <- log_lik_satu / nitems / nobs

    iter_satu <- iter_satu + 1

    if (verbose) {
      message(
        sprintf(
          "Saturation Model - iter %d: log_lik=%.6f",
          iter_satu, ij_log_lik_satu
        )
      )
    }

    if (abs(old_log_like_satu - ij_log_lik_satu) < (eps * abs(old_log_like_satu))) {
      FLG <- FALSE
    }
    if (iter_satu > maxiter) {
      converge_satu <- FALSE
      FLG <- FALSE
      message("\nReached the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
    }
  }

  # monotonic increasing check
  col_sums <- colSums(refMat111_satu)
  if (col_sums[1] > col_sums[ncol(refMat111_satu)]) {
    refMat111_satu <- refMat111_satu[, ncol(refMat111_satu):1]
  }
  if (mic) {
    refMat111_satu <- t(apply(refMat111_satu, 1, sort))
  }

  delete_rows <- design0 - ncat[1] + 1
  refMat_satu <- refMat111_satu[-delete_rows, ]

  refMat000_satu <- rbind(refMat111_satu[-1, ], refMat111_satu[1, ])
  refMat000_satu[design0, ] <- 0

  catRefMat_satu <- refMat111_satu - refMat000_satu


  # Restricted Model ------------------------------------------------
  ij_log_lik <- -10
  iter <- 0
  refBox <- array(0, dim = c(nitems, max(ncat) - 1, nrank))

  for (j in 1:nitems) {
    for (k in 1:(ncat[j] - 1)) {
      refBox[j, k, ] <- sort(quanRefmat[j, k, ])
    }
  }


  refBox111 <- array(1, dim = c(nitems, max(ncat), nrank))
  refBox000 <- array(0, dim = c(nitems, max(ncat), nrank))
  refBox111[, 2:(max(ncat)), ] <- refBox
  refBox000[, 1:(max(ncat) - 1), ] <- refBox
  catRefBox <- refBox111 - refBox000

  refMat <- matrix(0, nrow = nitems * (max(ncat) - 1), nrank)
  refMat111 <- matrix(0, nrow = nitems * (max(ncat)), nrank)
  refMat000 <- matrix(0, nrow = nitems * (max(ncat)), nrank)
  catRefMat <- matrix(0, nrow = nitems * (max(ncat)), nrank)
  for (i in 1:nitems) {
    l <- (i - 1) * (max(ncat) - 1) + 1
    refMat[l:(l + max(ncat) - 2), ] <- refBox[i, , ]
    m <- (i - 1) * (max(ncat)) + 1
    refMat111[m:(m + max(ncat) - 1), ] <- refBox111[i, , ]
    refMat000[m:(m + max(ncat) - 1), ] <- refBox000[i, , ]
    catRefMat[m:(m + max(ncat) - 1), ] <- catRefBox[i, , ]
  }

  ## EM Algorithm
  if (verbose) {
    message("\nStarting EM estimation for Restricted Model...")
  }
  converge <- TRUE
  FLG <- TRUE
  while (FLG) {
    old_log_like <- ij_log_lik
    ## Estep
    nume <- exp(uuMat %*% log(catRefMat + const) + logprior_NQmat)
    denom <- rowSums(nume)
    rankProf <- nume / denom

    # Filtering
    refMatcore <- t(uuMat) %*% rankProf %*% Fil
    refMat111 <- design6 %*% refMatcore / design5 %*% refMatcore

    if (sum(refMat111[1, ]) > sum(refMat111[nrank, ])) {
      refMat111 <- refMat111[, ncol(refMat111):1]
    }

    if (mic == 1) {
      refMat111 <- t(apply(refMat111, 1, sort))
    }

    delete_rows <- sapply(1:nitems, function(j) design0[j] - ncat[1] + 1)
    refMat <- refMat111[-delete_rows, ]

    refMat000 <- rbind(refMat111[2:nrow(refMat111), ], rep(0, nrank))
    refMat000[design0, ] <- 0
    catRefMat <- refMat111 - refMat000

    log_lik <- sum(rankProf * log(nume))
    ij_log_lik <- log_lik / nitems / nobs

    iter <- iter + 1
    if (verbose) {
      message(
        sprintf(
          "Restricted Model - iter %d: log_lik=%.6f",
          iter, ij_log_lik
        )
      )
    }
    if (abs(old_log_like - ij_log_lik) < (eps * abs(old_log_like))) {
      FLG <- FALSE
    }
    if (iter > maxiter) {
      converge <- FALSE
      FLG <- FALSE
      message("\nReached the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
    }
  }


  # results ---------------------------------------------------------
  nume <- exp(uuMat %*% log(catRefMat + const) + logprior_NQmat)
  denom <- rowSums(nume)
  rankProf <- nume / denom

  ## Item - Prob report
  boundary_report <- as.data.frame(refMat111)
  colnames(boundary_report) <- paste0("rank", 1:nrank)
  boundary_report$ItemLabel <- rep(U$ItemLabel, U$categories)
  boundary_report$CategoryLabel <- unlist(U$CategoryLabel)
  ## Item-Category Boundary Profile
  ICBR <- boundary_report[, c("ItemLabel", "CategoryLabel", paste0("rank", 1:nrank))]

  category_report <- as.data.frame(catRefMat)
  colnames(category_report) <- paste0("rank", 1:nrank)
  category_report$ItemLabel <- rep(U$ItemLabel, U$categories)
  category_report$CategoryLabel <- unlist(U$CategoryLabel)
  ## Item-Category Reference Profile
  ICRP <- category_report[, c("ItemLabel", "CategoryLabel", paste0("rank", 1:nrank))]

  testRefVec <- t(catRefMat) %*% unlist(category)

  rankmemb <- apply(rankProf, 1, which.max)
  rankmemb01 <- sign(rankProf - apply(rankProf, 1, max)) + 1
  rankdist <- colSums(rankmemb01)
  RMD <- colSums(rankProf)

  StudentRank <- rankProf
  RU <- ifelse(rankmemb + 1 > nrank, NA, rankmemb + 1)
  RD <- ifelse(rankmemb - 1 < 1, NA, rankmemb - 1)
  RUO <- StudentRank[cbind(1:nobs, RU)] / StudentRank[cbind(1:nobs, rankmemb)]
  RDO <- StudentRank[cbind(1:nobs, RD)] / StudentRank[cbind(1:nobs, rankmemb)]
  StudentRank <- cbind(StudentRank, score, rankmemb, quantileRank, RUO, RDO)
  colnames(StudentRank) <- c(
    paste("Membership", 1:nrank), "Score", "Estimate",
    "Quantile Rank",
    "Rank-Up Odds", "Rank-Down Odds"
  )
  rownames(StudentRank) <- U$ID


  rho1 <- cor(score, rankmemb, method = "spearman")
  ## score-rank dist
  scoreRankDist <- matrix(0, nrow = maxscore + 1, ncol = nrank)
  scoreMembDist <- matrix(0, nrow = maxscore + 1, ncol = nrank)
  rownames(scoreRankDist) <- rownames(scoreMembDist) <- 0:maxscore
  colnames(scoreRankDist) <- colnames(scoreMembDist) <- paste0("Rank", 1:nrank)
  for (s in 0:maxscore) {
    if (scoredist[s + 1] > 0) {
      ranks_s <- rankmemb[score == s]
      scoreRankDist[s + 1, ] <- sapply(1:nrank, function(r) sum(ranks_s == r))
      scoreMembDist[s + 1, ] <- colSums(rankProf[score == s, , drop = FALSE])
    }
  }

  rankQuanDist <- unname(table(rankmemb, quantileRank))
  membQuanDist <- matrix(0, nrow = nrank, ncol = nquan)
  rho2 <- cor(rankmemb, quantileRank, method = "spearman")
  for (q in 1:nquan) {
    membQuanDist[, q] <- colSums(rankProf[quantileRank == q, , drop = FALSE])
  }
  rownames(rankQuanDist) <- rownames(membQuanDist) <- paste0("Rank", 1:nrank)
  colnames(rankQuanDist) <- colnames(membQuanDist) <- paste0("Quantile", 1:nrank)

  # Fit Indices -----------------------------------------------------

  itemdf <- (ncat - 1) * (nitems - sum(diag(Fil)))
  testdf <- sum(itemdf)
  null_itemdf <- (ncat - 1) * (nitems - 1)
  null_testdf <- sum(null_itemdf)

  rankProf_satu_num <- exp(uuMat %*% log(catRefMat_satu + const))
  rankProf_satu_denom <- rowSums(rankProf_satu_num)
  rankProf_satu <- rankProf_satu_num / rankProf_satu_denom
  Rank_satu <- apply(rankProf_satu, 1, which.max)
  Rank_satu01 <- sign(rankProf_satu - apply(rankProf_satu, 1, max)) + 1


  # modelfff_jq1 <- t(dat$Z) %*% rankmemb01
  # modelggg_jq1 <- t(uuMat) %*% rankmemb01
  # satufff_jq1 <- t(dat$Z) %*% Rank_satu01
  # satuggg_jq1 <- t(uuMat) %*% Rank_satu01
  modelfff_jq2 <- t(U$Z) %*% rankProf
  modelggg_jq2 <- t(uuMat) %*% rankProf
  satufff_jq2 <- t(U$Z) %*% rankProf_satu
  satuggg_jq2 <- t(uuMat) %*% rankProf_satu

  # Model item log-likelihood
  # model_itemll1 <- design4 %*% colSums(t(modelggg_jq1 * log(catRefMat + const)))
  model_itemll2 <- design4 %*% colSums(t(modelggg_jq2 * log(catRefMat + const)))

  # Saturated item log-likelihood
  # satu_itemll1 <- design4 %*% colSums(t(satuggg_jq1 * log(catRefMat_satu + const)))
  satu_itemll2 <- design4 %*% colSums(t(satuggg_jq2 * log(catRefMat_satu + const)))

  # Null item log-likelihood
  catfreqMat <- matrix(unlist(catfreq), ncol = ncat, byrow = T)
  null_itemll <- colSums(t(catfreqMat * log(catfreqMat / zzzTotal + const)))

  # Model chi-square
  # model_itemchisq1 <- pmax(0.000001, pmin(2 * (satu_itemll1 - model_itemll1), 1000000000))
  model_itemchisq2 <- pmax(0.000001, pmin(2 * (satu_itemll2 - model_itemll2), 1000000000))

  # Null chi-square
  # null_itemchisq1 <- pmax(0.000001, pmin(2 * (satu_itemll1 - null_itemll), 1000000000)) + 0.000001
  null_itemchisq2 <- pmax(0.000001, pmin(2 * (satu_itemll2 - null_itemll), 1000000000)) + 0.000001


  # Item Fit Indices
  # ItemFitIndices1 <- calcFitIndices(model_itemchisq1, null_itemchisq1, itemdf, null_itemdf, colSums(dat$Z))
  ItemFits <- calcFitIndices(model_itemchisq2, null_itemchisq2, itemdf, null_itemdf, colSums(U$Z))
  ItemFitIndices <- structure(
    c(list(
      model_log_like = model_itemll2,
      bench_log_like = satu_itemll2,
      null_log_like = null_itemll,
      model_Chi_sq = model_itemchisq2,
      null_Chi_sq = null_itemchisq2,
      model_df = itemdf,
      null_df = null_itemdf
    ), ItemFits),
    class = c("exametrika", "ModelFit")
  )

  # Test Fit Indices
  # TestFitindices1 <- calcFitIndices(
  #   sum(model_itemchisq1),
  #   sum(null_itemchisq1) + const,
  #   sum(itemdf),
  #   sum(null_itemdf),
  #   sum(colSums(dat$Z))
  # )

  TestFits <- calcFitIndices(
    sum(model_itemchisq2),
    sum(null_itemchisq2) + const,
    sum(itemdf),
    sum(null_itemdf),
    sum(colSums(U$Z))
  )

  TestFitIndices <- structure(
    c(list(
      model_log_like = log_lik,
      bench_log_like = sum(satu_itemll2),
      null_log_like = sum(null_itemll),
      model_Chi_sq = sum(model_itemchisq2),
      null_Chi_sq = sum(null_itemchisq2),
      model_df = sum(itemdf),
      null_df = sum(null_itemdf)
    ), TestFits),
    class = c("exametrika", "ModelFit")
  )


  # output ----------------------------------------------------------

  ret <- structure(list(
    U = U,
    mic = mic,
    testlength = NCOL(U$Q),
    msg = "Rank",
    converge = converge,
    nobs = NROW(U$Q),
    n_rank = nrank, # New naming convention
    n_cycle = iter, # New naming convention
    TRP = as.vector(testRefVec),
    LRD = rankdist,
    RMD = RMD,
    ICBR = ICBR,
    ICRP = ICRP,
    ScoreRankCorr = rho1,
    RankQuantCorr = rho2,
    Students = StudentRank,
    ScoreRank = scoreRankDist,
    ScoreMembership = scoreMembDist,
    RankQuantile = rankQuanDist,
    MembQuantile = membQuanDist,
    ItemFitIndices = ItemFitIndices,
    TestFitIndices = TestFitIndices,
    ScoreReport = ScoreReport,
    ItemReport = ItemReport,
    CatQuant = SelectRatioTable,
    # Deprecated fields (for backward compatibility)
    Nrank = nrank,
    N_Cycle = iter
  ), class = c("exametrika", "LRAordinal"))
  return(ret)
}

#' @rdname Biclustering_IRM
#' @param alpha Dirichlet distribution concentration parameter for the prior density of
#' field reference probabilities (nominal IRM only). Must be positive. The default is 1.
#' @return
#' For nominal data, the returned list includes:
#' \describe{
#'  \item{Q}{Response matrix.}
#'  \item{Z}{Missing indicator matrix.}
#'  \item{testlength}{Number of items.}
#'  \item{nobs}{Sample size.}
#'  \item{n_class}{Optimal number of classes.}
#'  \item{n_field}{Optimal number of fields.}
#'  \item{n_cycle}{Number of EM algorithm iterations.}
#'  \item{FRP}{Field Reference Profile, a 3D array (nfld x ncls x maxQ).}
#'  \item{LFD}{Latent Field Distribution.}
#'  \item{LCD}{Latent Class Distribution.}
#'  \item{FieldMembership}{Field membership probability matrix.}
#'  \item{ClassMembership}{Class membership probability matrix.}
#'  \item{FieldEstimated}{Estimated field assignment for each item.}
#'  \item{ClassEstimated}{Estimated class assignment for each student.}
#'  \item{Students}{Rank Membership Profile matrix with estimated class.}
#'  \item{TestFitIndices}{Overall fit index for the test.}
#'  \item{log_lik}{Log-likelihood of the model.}
#' }
#' @examples
#' \donttest{
#' # Fit a nominal Biclustering IRM model
#' result <- Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, verbose = TRUE)
#' plot(result, type = "Array")
#' }
#' @export
Biclustering_IRM.nominal <- function(U,
                                     gamma_c = 1, gamma_f = 1, alpha = 1,
                                     max_iter = 100, stable_limit = 5,
                                     minSize = 20, EM_limit = 20,
                                     seed = 123, verbose = TRUE, ...) {
  tmp <- U

  nitems <- NCOL(tmp$Q)
  nobs <- NROW(tmp$Q)
  const <- exp(-nitems)
  maxQ <- max(tmp$categories)
  alpha_vec <- rep(alpha, maxQ)

  if (alpha <= 0) {
    stop("alpha must be positive (alpha > 0)")
  }

  # One-hot encoding of response matrix: Uq[s, j, q] = 1 if student s chose category q on item j
  Uq <- array(0, dim = c(nobs, nitems, maxQ))
  for (s in 1:nobs) {
    for (j in 1:nitems) {
      Uq[s, j, tmp$Q[s, j]] <- 1
    }
  }

  # Initialize -------------------------------------------------------------
  if (!is.null(seed)) {
    set.seed(seed)
  }

  ## Initial Class: assign by mode category
  mode_cat <- apply(tmp$Q, 1, function(x) {
    tab <- table(x)
    as.integer(names(which.max(tab)))
  })
  unique_vec <- unique(mode_cat)
  ncls <- length(unique_vec)
  cls01 <- matrix(0, ncol = ncls, nrow = nobs)
  for (i in 1:nobs) {
    cls01[i, mode_cat[i]] <- 1
  }
  colnames(cls01) <- paste("Rank", 1:ncls)
  rownames(cls01) <- tmp$ID

  ## Initial Field: one item per field
  nfld <- nitems
  fld01 <- matrix(0, nrow = nitems, ncol = nfld)
  for (i in 1:nitems) {
    fld01[i, i] <- 1
  }

  # Phase 1: Collapsed Gibbs Sampler (shared core) --------------------------
  gibbs <- irm_gibbs_core(
    Uq = Uq, Z = tmp$Z, cls01 = cls01, fld01 = fld01,
    gamma_c = gamma_c, gamma_f = gamma_f, alpha_vec = alpha_vec,
    max_iter = max_iter, stable_limit = stable_limit, verbose = verbose
  )

  cls01 <- gibbs$cls01
  fld01 <- gibbs$fld01
  ncls <- gibbs$ncls
  nfld <- gibbs$nfld

  # Phase 2: BCRM estimation (direct normalization) -------------------------
  U_fcq <- irm_calc_Ufcq(Uq, tmp$Z, cls01, fld01, maxQ)
  BCRM <- sweep(U_fcq, 3, alpha_vec, "+")
  denom <- apply(BCRM, c(1, 2), sum)
  BCRM <- sweep(BCRM, c(1, 2), denom, "/")

  # Reorganizing small-sized classes ----------------------------------------
  bic <- irm_bic_calc(Uq, tmp$Z, fld01, cls01, BCRM, maxQ, const)
  best_BCRM <- BCRM
  best_cls01 <- cls01
  EMt <- 0

  DelRepFLG <- TRUE
  while (DelRepFLG) {
    bestfit <- 10^10
    Nc <- colSums(cls01)
    NcTable <- matrix(Nc)
    minclass <- NcTable[order(NcTable[, 1]), ]
    if (minclass[1] < minSize) {
      if (verbose) {
        message(
          sprintf(
            "Adjusting classes: BIC=%.1f ncls=%d (min size < %d)",
            bic, ncls, minSize
          )
        )
      }
    } else {
      DelRepFLG <- FALSE
      break
    }

    ncls <- ncls - 1
    del_class <- which.min(NcTable)
    BCRM <- BCRM[, -del_class, , drop = FALSE]

    EMrepFLG <- TRUE
    while (EMrepFLG) {
      EMt <- EMt + 1
      # E-step
      V_scq <- array(0, dim = c(nobs, ncls, maxQ))
      for (q in 1:maxQ) {
        V_scq[, , q] <- (Uq[, , q] %*% fld01) %*% log(BCRM[, , q])
      }
      log_S <- apply(V_scq, c(1, 2), sum)
      prob_S <- matrix(0, nrow = nobs, ncol = ncls)
      for (i in seq_len(nobs)) {
        prob_S[i, ] <- irm_log_to_prob(log_S[i, ])
      }
      cls <- apply(prob_S, 1, which.max)
      cls01 <- matrix(0, ncol = ncls, nrow = nobs)
      for (i in 1:nobs) {
        cls01[i, cls[i]] <- 1
      }

      # M-step
      U_fcq <- irm_calc_Ufcq(Uq, tmp$Z, cls01, fld01, maxQ)
      BCRM <- sweep(U_fcq, 3, alpha_vec, "+")
      denom <- apply(BCRM, c(1, 2), sum)
      BCRM <- sweep(BCRM, c(1, 2), denom, "/")

      # Model fit
      bic <- irm_bic_calc(Uq, tmp$Z, fld01, cls01, BCRM, maxQ, const)

      # Converge check
      if (bic < bestfit) {
        bestfit <- bic
        best_BCRM <- BCRM
        best_cls01 <- cls01
      } else {
        BCRM <- best_BCRM
        cls01 <- best_cls01
        EMrepFLG <- FALSE
      }

      if (EMt >= EM_limit) {
        EMrepFLG <- FALSE
      }
    }
  }

  # Output -----------------------------------------------------------------

  ## Class membership
  V_scq <- array(0, dim = c(nobs, ncls, maxQ))
  for (q in 1:maxQ) {
    V_scq[, , q] <- (Uq[, , q] %*% fld01) %*% log(BCRM[, , q])
  }
  log_S <- apply(V_scq, c(1, 2), sum)
  clsmemb <- matrix(0, nrow = nobs, ncol = ncls)
  for (i in seq_len(nobs)) {
    clsmemb[i, ] <- irm_log_to_prob(log_S[i, ])
  }
  cls <- apply(clsmemb, 1, which.max)
  cls01 <- matrix(0, ncol = ncls, nrow = nobs)
  for (i in 1:nobs) {
    cls01[i, cls[i]] <- 1
  }

  ## Field membership
  V_jfq <- array(0, dim = c(nitems, nfld, maxQ))
  for (q in 1:maxQ) {
    V_jfq[, , q] <- (t(Uq[, , q]) %*% cls01) %*% log(t(BCRM[, , q]))
  }
  log_J <- apply(V_jfq, c(1, 2), sum)
  fldmemb <- matrix(0, nrow = nitems, ncol = nfld)
  for (i in seq_len(nitems)) {
    fldmemb[i, ] <- irm_log_to_prob(log_J[i, ])
  }
  fld <- apply(fldmemb, 1, which.max)
  check_empty_fields(fld, nfld)
  fld01 <- matrix(0, ncol = nfld, nrow = nitems)
  for (i in 1:nitems) {
    fld01[i, fld[i]] <- 1
  }

  fldmemb01 <- sign(fldmemb - apply(fldmemb, 1, max)) + 1
  flddist <- colSums(fldmemb01)
  clsmemb01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  clsdist <- colSums(clsmemb01)
  StudentRank <- cbind(clsmemb, Estimate = cls)
  rownames(StudentRank) <- tmp$ID
  colnames(StudentRank) <- c(paste("Membership", 1:ncls), "Estimate")

  # Model fit ---------------------------------------------------------------
  testell <- 0
  for (q in 1:maxQ) {
    pred_prob <- t(fldmemb %*% BCRM[, , q] %*% t(clsmemb))
    observed_mask <- (tmp$Z * Uq[, , q]) == 1
    testell <- testell + sum(log(pmax(pred_prob[observed_mask], const)))
  }
  nparam <- ncls * nfld * (maxQ - 1)

  # Null model
  Zrep <- replicate(maxQ, tmp$Z)
  NullFRQ <- apply(Zrep * Uq, c(2, 3), sum) / apply(tmp$Z, 2, sum)
  ell_N <- sum(apply(Zrep * Uq, c(2, 3), sum) * log(NullFRQ + const))

  # Fit indices: For nominal data, no meaningful benchmark (saturated) model exists
  # because response patterns are almost always unique with many items and categories.
  # Only information criteria (AIC, BIC, CAIC) are reported.
  AIC <- -2 * testell + 2 * nparam
  CAIC <- -2 * testell + nparam * (log(nobs) + 1)
  BIC <- -2 * testell + nparam * log(nobs)

  FitIndices <- structure(
    list(
      model_log_like = testell,
      bench_log_like = NA,
      null_log_like = ell_N,
      model_Chi_sq = NA,
      null_Chi_sq = NA,
      model_df = NA,
      null_df = NA,
      NFI = NA,
      RFI = NA,
      IFI = NA,
      TLI = NA,
      CFI = NA,
      RMSEA = NA,
      AIC = AIC,
      CAIC = CAIC,
      BIC = BIC
    ),
    class = c("exametrika", "ModelFit")
  )

  # Return ------------------------------------------------------------------
  msg <- "Class"
  ret <- structure(list(
    Q = tmp$Q,
    Z = tmp$Z,
    testlength = nitems,
    msg = msg,
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    n_cycle = EMt,
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = EMt,
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = BCRM,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    TestFitIndices = FitIndices,
    log_lik = testell,
    # Deprecated fields (for backward compatibility)
    LogLik = testell
  ), class = c("exametrika", "nominalBiclustering"))

  return(ret)
}

#' @title Distractor Analysis
#' @description
#' Performs distractor analysis for rated (multiple-choice) models.
#' For each item and rank/class, computes observed category frequency tables,
#' chi-square tests against chance level (uniform distribution), and
#' Cramer's V as an effect size measure.
#'
#' This function works with results from \code{\link{LRA}} (rated data) and
#' \code{\link{Biclustering}} / \code{\link{Biclustering_IRM}} (rated data).
#'
#' @param x A result object from a rated model (class \code{LRArated} or
#'   \code{ratedBiclustering}).
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class \code{c("exametrika", "DistractorAnalysis")} containing:
#' \describe{
#'   \item{freq_table}{List of frequency matrices (nrank x maxQ), one per item.}
#'   \item{prop_table}{List of proportion matrices (nrank x maxQ), one per item.}
#'   \item{chisq_table}{Matrix (nitems x nrank) of chi-square statistics.}
#'   \item{pvalue_table}{Matrix (nitems x nrank) of p-values.}
#'   \item{cramersv_table}{Matrix (nitems x nrank) of Cramer's V effect sizes.}
#'   \item{CA}{Correct answer vector.}
#'   \item{n_rank}{Number of ranks/classes.}
#'   \item{maxQ}{Number of response categories.}
#'   \item{nitems}{Number of items.}
#'   \item{ItemLabel}{Item label vector.}
#'   \item{n_field}{Number of fields (Biclustering only).}
#'   \item{FieldEstimated}{Field assignment vector (Biclustering only).}
#'   \item{field_items}{List of item indices per field (Biclustering only).}
#' }
#'
#' @examples
#' \donttest{
#' # LRA.rated example
#' result_lra <- LRA(J21S300, nrank = 5, mic = TRUE)
#' da <- DistractorAnalysis(result_lra)
#' print(da)
#' print(da, items = 1:3)
#' print(da, ranks = c(1, 5))
#' plot(da)
#' plot(da, items = 1:6, nc = 3, nr = 2)
#'
#' # Biclustering.rated example
#' result_bic <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R")
#' da_bic <- DistractorAnalysis(result_bic)
#' print(da_bic, items = 1:7)
#' plot(da_bic, items = 1:6, nc = 3, nr = 2)
#' }
#'
#' @importFrom stats pchisq
#' @export
DistractorAnalysis <- function(x, ...) {
  UseMethod("DistractorAnalysis")
}

#' @rdname DistractorAnalysis
#' @export
DistractorAnalysis.LRArated <- function(x, ...) {
  tmp <- x$U
  da <- distractor_core(
    Q = tmp$Q, Z = tmp$Z,
    rank_est = as.numeric(x$Students[, "Estimate"]),
    CA = tmp$CA,
    nrank = x$n_rank,
    maxQ = max(tmp$categories),
    item_labels = tmp$ItemLabel
  )
  da$msg <- "Rank"
  return(da)
}

#' @rdname DistractorAnalysis
#' @export
DistractorAnalysis.ratedBiclustering <- function(x, ...) {
  da <- distractor_core(
    Q = x$Q, Z = x$Z,
    rank_est = x$ClassEstimated,
    CA = if (!is.null(x$U)) {
      # Recover CA from U and Q: CA[j] = category q where U[s,j]==1 for any observed s
      apply(x$Q * x$U * x$Z, 2, function(col) {
        vals <- col[col > 0]
        if (length(vals) > 0) vals[1] else NA
      })
    } else {
      stop("Cannot determine correct answers from result object")
    },
    nrank = x$n_class,
    maxQ = dim(x$FRP)[3],
    item_labels = colnames(x$Q)
  )
  da$msg <- x$msg

  # Attach field metadata
  nfld <- x$n_field
  da$n_field <- nfld
  da$FieldEstimated <- x$FieldEstimated
  da$field_items <- lapply(1:nfld, function(f) which(x$FieldEstimated == f))
  names(da$field_items) <- paste0("Field", 1:nfld)

  return(da)
}


# --- Internal core computation -------------------------------------------

#' @noRd
distractor_core <- function(Q, Z, rank_est, CA, nrank, maxQ, item_labels) {
  nitems <- NCOL(Q)

  # Frequency table: item x rank x category
  freq_table <- list()
  for (j in 1:nitems) {
    mat <- matrix(0, nrow = nrank, ncol = maxQ)
    for (r in 1:nrank) {
      students_r <- which(rank_est == r)
      for (q in 1:maxQ) {
        mat[r, q] <- sum(Q[students_r, j] == q & Z[students_r, j] == 1)
      }
    }
    rownames(mat) <- paste0("Rank", 1:nrank)
    colnames(mat) <- paste0("Cat", 1:maxQ)
    freq_table[[j]] <- mat
  }
  names(freq_table) <- item_labels

  # Proportion table
  prop_table <- list()
  for (j in 1:nitems) {
    row_sums <- rowSums(freq_table[[j]])
    row_sums[row_sums == 0] <- 1
    prop_table[[j]] <- freq_table[[j]] / row_sums
  }
  names(prop_table) <- item_labels

  # Chi-square, p-value, Cramer's V
  chisq_table <- matrix(NA, nrow = nitems, ncol = nrank)
  pvalue_table <- matrix(NA, nrow = nitems, ncol = nrank)
  cramersv_table <- matrix(NA, nrow = nitems, ncol = nrank)
  rownames(chisq_table) <- rownames(pvalue_table) <- rownames(cramersv_table) <- item_labels
  colnames(chisq_table) <- colnames(pvalue_table) <- colnames(cramersv_table) <- paste0("Rank", 1:nrank)

  for (j in 1:nitems) {
    for (r in 1:nrank) {
      observed <- freq_table[[j]][r, ]
      n_total <- sum(observed)
      if (n_total > 0) {
        expected <- rep(n_total / maxQ, maxQ)
        chisq_val <- sum((observed - expected)^2 / expected)
        df <- maxQ - 1
        pvalue_table[j, r] <- pchisq(chisq_val, df, lower.tail = FALSE)
        chisq_table[j, r] <- chisq_val
        cramersv_table[j, r] <- sqrt(chisq_val / (n_total * df))
      }
    }
  }

  structure(list(
    freq_table = freq_table,
    prop_table = prop_table,
    chisq_table = chisq_table,
    pvalue_table = pvalue_table,
    cramersv_table = cramersv_table,
    CA = CA,
    n_rank = nrank,
    maxQ = maxQ,
    nitems = nitems,
    ItemLabel = item_labels
  ), class = c("DistractorAnalysis", "exametrika"))
}


# --- Print method --------------------------------------------------------

#' @param items Integer vector of item indices to display. NULL for all items.
#' @param ranks Integer vector of rank/class indices to display. NULL for all ranks.
#' @param digits Number of digits for rounding. Default is 4.
#' @rdname DistractorAnalysis
#' @export
print.DistractorAnalysis <- function(x, items = NULL, ranks = NULL, digits = 4, ...) {
  if (is.null(items)) items <- 1:x$nitems
  if (is.null(ranks)) ranks <- 1:x$n_rank

  rank_cols <- paste0("Rank", ranks)
  msg <- if (!is.null(x$msg)) x$msg else "Rank"

  cat("Distractor Analysis\n\n")

  # Field info (Biclustering only)
  if (!is.null(x$n_field)) {
    cat("Field Grouping:\n")
    for (f in 1:x$n_field) {
      fi <- x$field_items[[f]]
      cat(sprintf("  Field %d: %s\n", f, paste(x$ItemLabel[fi], collapse = ", ")))
    }
    cat("\n")
  }

  # Group items by field (or single group for LRA)
  if (!is.null(x$n_field)) {
    field_groups <- list()
    for (f in 1:x$n_field) {
      field_groups[[f]] <- intersect(x$field_items[[f]], items)
    }
    names(field_groups) <- paste0("Field ", 1:x$n_field)
  } else {
    field_groups <- list(items)
    names(field_groups) <- NULL
  }

  for (g in seq_along(field_groups)) {
    g_items <- field_groups[[g]]
    if (length(g_items) == 0) next

    if (!is.null(names(field_groups)[g])) {
      cat(sprintf("\n--- %s ---\n", names(field_groups)[g]))
    }

    cat("\nCramer's V\n")
    print(round(x$cramersv_table[g_items, ranks, drop = FALSE], digits))

    cat("\nP-values\n")
    print(signif(x$pvalue_table[g_items, ranks, drop = FALSE], digits))

    cat("\nCategory Proportions by", msg, "\n")
    for (j in g_items) {
      ca <- x$CA[j]
      cat(sprintf("\n  %s (CA = Cat%d)\n", x$ItemLabel[j], ca))
      sub <- x$prop_table[[j]][ranks, , drop = FALSE]
      rownames(sub) <- paste0(msg, ranks)
      print(round(sub, digits))
    }
  }

  invisible(x)
}


# --- Plot method ---------------------------------------------------------

#' @param type Plot type. Currently only "Distractor" is supported.
#' @param nc Number of columns in the plot grid.
#' @param nr Number of rows in the plot grid.
#' @rdname DistractorAnalysis
#' @export
plot.DistractorAnalysis <- function(x, type = "Distractor",
                                    items = NULL, ranks = NULL,
                                    nc = 1, nr = 1, ...) {
  if (is.null(items)) items <- 1:x$nitems
  if (is.null(ranks)) ranks <- 1:x$n_rank

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))
  par(mfrow = c(nr, nc), mar = c(3, 4, 3, 1))

  for (j in items) {
    freq <- x$freq_table[[j]][ranks, , drop = FALSE]
    row_sums <- rowSums(freq)
    row_sums[row_sums == 0] <- 1
    props <- freq / row_sums
    props[is.nan(props)] <- 0

    ca <- x$CA[j]
    colors <- rep("gray70", x$maxQ)
    colors[ca] <- "steelblue"

    msg <- if (!is.null(x$msg)) x$msg else "Rank"

    barplot(t(props),
      beside = FALSE,
      col = colors,
      main = paste0(x$ItemLabel[j], " (CA=Cat", ca, ")"),
      xlab = msg, ylab = "Proportion",
      names.arg = paste0(substr(msg, 1, 1), ranks),
      cex.main = 0.9
    )
  }
}

#' @title Biclustering with Infinite Relational Model
#' @description
#'  This function performs Biclustering structure learning using the Infinite Relational Model (IRM)
#'  to automatically determine the optimal number of classes C and optimal number of
#'  fields F. It dispatches to the appropriate method based on the data type:
#'  binary, ordinal, or nominal. For binary data, see Section 7.8 in Shojima(2022).
#'  For nominal data, a Dirichlet-Multinomial collapsed Gibbs sampler is used.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param ... Additional arguments passed to specific methods.
#' @return An object of class "exametrika" containing the IRM results.
#' See \code{Biclustering_IRM.binary} or \code{Biclustering_IRM.nominal} for details.
#' @importFrom stats rmultinom
#' @examples
#' \donttest{
#' # Fit a Biclustering model with automatic structure learning using IRM
#' # gamma_c and gamma_f are concentration parameters for the Chinese Restaurant Process
#' result <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
#'
#' # Display the Bicluster Reference Matrix (BRM) as a heatmap
#' plot(result, type = "Array")
#'
#' # Plot Field Reference Profiles (FRP) in a 3-column grid
#' plot(result, type = "FRP", nc = 3)
#' }
#'
#' @export

Biclustering_IRM <- function(U, ...) {
  UseMethod("Biclustering_IRM")
}

#' @rdname Biclustering_IRM
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @export
Biclustering_IRM.default <- function(U, na = NULL, Z = NULL, w = NULL, ...) {
  if (inherits(U, "exametrika")) {
    if (U$response.type == "binary") {
      return(Biclustering_IRM.binary(U, ...))
    } else if (U$response.type == "ordinal") {
      return(Biclustering_IRM.ordinal(U, ...))
    } else if (U$response.type == "rated") {
      return(Biclustering_IRM.rated(U, ...))
    } else if (U$response.type == "nominal") {
      return(Biclustering_IRM.nominal(U, ...))
    }
  }

  U <- dataFormat(U, na = na, Z = Z, w = w)
  Biclustering_IRM(U, ...)
}

#' @rdname Biclustering_IRM
#' @param gamma_c \eqn{\gamma_C} is the hyperparameter of the CRP and represents the
#' attractiveness of a new Class. As \eqn{\gamma_C} increases, the student is more likely
#' to be seated at a vacant class. The default is 1.
#' @param gamma_f \eqn{\gamma_F} is the hyperparameter of the CRP and represents the
#' attractiveness of a new Field. The greater this value it more likely to be classified
#' in the new field. The default is 1.
#' @param max_iter A maximum iteration number of IRM process. The default is 100.
#' @param stable_limit The IRM process exits the loop when the FRM stabilizes and no longer
#'  changes significantly. This option sets the maximum number of stable iterations,
#'  with a default of 5.
#' @param minSize A value used for readjusting the number of classes.If the size of each
#' class is less than \code{minSize}, the number of classes will be reduced. Note that this
#' under limit of size is not used for either all correct or all incorrect class.
#' @param EM_limit After IRM process, resizing the number of classes process will starts.
#' This process using EM algorithm,\code{EM_limit} is the maximum number of iteration with
#' default of 20.
#' @param seed Random seed for reproducibility. When a numeric value is provided,
#' \code{set.seed(seed)} is called before the Gibbs sampling begins, ensuring
#' reproducible results. The default is \code{123}, which guarantees deterministic
#' output. Set to \code{NULL} to disable seed setting and let the results depend
#' on the current state of the random number generator.
#' @param verbose verbose output Flag. default is TRUE
#' @return
#' \describe{
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{msg}{A character string indicating the model type. }
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{n_class}{Optimal number of classes (new naming convention).}
#'  \item{n_field}{Optimal number of fields (new naming convention).}
#'  \item{em_cycle}{Number of EM algorithm iterations (new naming convention).}
#'  \item{Nclass}{Optimal number of classes (deprecated, use n_class).}
#'  \item{Nfield}{Optimal number of fields (deprecated, use n_field).}
#'  \item{EM_Cycle}{Number of EM algorithm iterations (deprecated, use em_cycle).}
#'  \item{BRM}{Bicluster Reference Matrix}
#'  \item{FRP}{Field Reference Profile}
#'  \item{FRPIndex}{Index of FFP includes the item location parameters B and Beta,
#'  the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
#'  \item{TRP}{Test Reference Profile}
#'  \item{FMP}{Field Membership Profile}
#'  \item{Students}{Rank Membership Profile matrix.The s-th row vector of \eqn{\hat{M}_R}, \eqn{\hat{m}_R}, is the
#' rank membership profile of Student s, namely the posterior probability distribution representing the student's
#' belonging to the respective latent classes. It also includes the rank with the maximum estimated membership probability,
#' as well as the rank-up odds and rank-down odds.}
#'  \item{LRD}{Latent Rank Distribution. see also [plot.exametrika]}
#'  \item{LFD}{Latent Field Distribution. see also [plot.exametrika]}
#'  \item{RMD}{Rank Membership Distribution.}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#' }
#' @examples
#' \donttest{
#' result <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
#' plot(result, type = "Array")
#' }
#' @export
Biclustering_IRM.binary <- function(U, Z = NULL, w = NULL, na = NULL,
                                    gamma_c = 1, gamma_f = 1,
                                    max_iter = 100, stable_limit = 5, minSize = 20, EM_limit = 20,
                                    seed = 123, verbose = TRUE, ...) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  tmp$U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)
  const <- 1e-100
  gamp <- 1

  # Initialize
  if (!is.null(seed)) {
    set.seed(seed)
  }
  limit_count <- 0
  iter <- 1
  pattern <- sort(unique(nrs(tmp)))
  zeroG <- ifelse(pattern[1] == 0, 1, 0)
  fullG <- ifelse(pattern[length(pattern)] == testlength, 1, 0)

  ## Initial Class
  ncls <- length(pattern)
  cls01 <- matrix(0, ncol = length(pattern), nrow = nobs)
  for (i in 1:nobs) {
    cls01[i, which((pattern == nrs(tmp)[i]))] <- 1
  }
  colnames(cls01) <- paste("Rank", 1:ncls)
  rownames(cls01) <- tmp$ID
  cls <- cls01 %*% (1:ncls)
  Nc <- colSums(cls01)

  ## Initial Field
  nfld <- testlength
  fld <- 1:testlength
  crr <- crr(tmp)
  df_tmp <- data.frame(fld, crr)
  df_tmp <- df_tmp[order(crr, decreasing = T), ]
  sorted_list <- df_tmp$fld
  fld <- order(sorted_list)
  fld01 <- matrix(0, nrow = testlength, ncol = nfld)
  for (i in 1:testlength) {
    fld01[i, fld[i]] <- 1
  }
  colnames(fld01) <- paste("Field", 1:nfld)
  rownames(fld01) <- tmp$ItemLabel
  Nf <- colSums(fld01)

  ## Inner Function for Ccf/Fcf/Ncf/Pcf
  Ccf_Fcf_Pcf <- function(U, Z, cls01, fld01, gamp, zeroG, fullG, type = 1) {
    Ccf <- t(cls01) %*% tmp$U %*% fld01
    Fcf <- t(cls01) %*% (tmp$Z * (1 - tmp$U)) %*% fld01
    Ncf <- Ccf + Fcf
    Pcf <- (Ccf + gamp - 1) / (Ncf + 2 * gamp - (type * 2))
    if (zeroG == 1) {
      Pcf[1, ] <- 0
    }
    if (fullG == 1) {
      Pcf[nrow(Pcf), ] <- 1
    }
    return(list(Ccf = Ccf, Fcf = Fcf, Ncf = Ncf, Pcf = Pcf))
  }
  ## Inner Function for Sorting PCF
  Pcf_sort <- function(Pcf, cls01, fld01, ncls, nfld) {
    sort_list <- order(colSums(Pcf), decreasing = TRUE)
    Pcf <- Pcf[, sort_list]
    fld01 <- fld01[, sort_list]
    colnames(fld01) <- paste("Field", 1:nfld)
    fld <- t((1:nfld) %*% t(fld01))

    sort_list2 <- order(rowSums(Pcf), decreasing = FALSE)
    Pcf <- Pcf[sort_list2, ]
    cls01 <- cls01[, sort_list2]
    colnames(cls01) <- paste("Rank", 1:ncls)
    cls <- cls01 %*% (1:ncls)
    return(list(Pcf = Pcf, cls01 = cls01, fld01 = fld01, cls = cls, fld = fld))
  }

  ### Initial value for parameter set and the model fit
  ret <- Ccf_Fcf_Pcf(tmp$U, tmp$Z, cls01, fld01, gamp, zeroG, fullG, type = 1)
  Ccf <- ret$Ccf
  Fcf <- ret$Fcf
  Ncf <- ret$Ncf
  Pcf <- ret$Pcf

  # IRM Iteration ------------------------------------------------------
  IRM_FLG <- TRUE
  while (IRM_FLG) {
    ### Gibbs sampler i
    Cif <- tmp$U %*% fld01 # SxF. number of correct response Studet S in Field f
    Fif <- (tmp$Z * (1 - tmp$U)) %*% fld01 # SxF. number of incorrect response Studet S in Field f
    iRand <- sample(1:nobs, nobs, replace = F)

    for (loop in 1:nobs) {
      target <- iRand[loop]

      if (cls[target] >= 2 && cls[target] <= ncls - 1) {
        # delete selected member
        Nc[cls[target]] <- Nc[cls[target]] - 1
        Ncf[cls[target], ] <- Ncf[cls[target], ] - Nf
        Ccf[cls[target], ] <- Ccf[cls[target], ] - Cif[target, ]
        Fcf[cls[target], ] <- Fcf[cls[target], ] - Fif[target, ]
        CcfPlus <- Ccf + matrix(rep(Cif[target, ], ncls), nrow = ncls, byrow = T)
        FcfPlus <- Fcf + matrix(rep(Fif[target, ], ncls), nrow = ncls, byrow = T)

        if (Nc[cls[target]] == 0) {
          ## if the class disappeared...
          ncls <- ncls - 1
          delCls <- cls[target]
          cls01 <- cls01[, -delCls]
          cls <- cls01 %*% (1:ncls)
          Nc <- Nc[-delCls]
          Ncf <- Ncf[-delCls, ]
          Ccf <- Ccf[-delCls, ]
          Fcf <- Fcf[-delCls, ]
          CcfPlus <- CcfPlus[-delCls, ]
          FcfPlus <- FcfPlus[-delCls, ]
        }

        # Table Choice!
        ## Likelihood for Existing Table
        Nc_tmp <- Nc[-1]
        Nc_tmp <- Nc_tmp[-(length(Nc_tmp))]
        exist_tab_tmp <- log((Nc_tmp / (nobs - 1 + gamma_c)) + const)
        vec1 <- numeric(ncls - 2)
        vec2 <- numeric(ncls - 2)
        vec3 <- numeric(ncls - 2)
        for (i in 2:(ncls - 1)) {
          Ctmp <- 0
          Ftmp <- 0
          den <- 0
          for (j in 1:nfld) {
            ct1 <- (CcfPlus[i, j] - Ccf[i, j] - 1)
            if (ct1 >= 0) {
              for (s in 0:ct1) {
                Ctmp <- Ctmp + log(CcfPlus[i, j] + gamp - 1 - s + const)
              }
            }
            ct2 <- (FcfPlus[i, j] - Fcf[i, j] - 1)
            if (ct2 >= 0) {
              for (s in 0:ct2) {
                Ftmp <- Ftmp + log(FcfPlus[i, j] + gamp - 1 - s + const)
              }
            }
            ct3 <- CcfPlus[i, j] + FcfPlus[i, j] - Ccf[i, j] - Fcf[i, j] - 1
            if (ct3 >= 0) {
              for (s in 0:ct3) {
                den <- den +
                  (log(CcfPlus[i, j] + FcfPlus[i, j] + 2 * gamp - 1 - s + const))
              }
            }
          }
          vec1[i - 1] <- Ctmp
          vec2[i - 1] <- Ftmp
          vec3[i - 1] <- den
        }
        exist_tab <- exist_tab_tmp + vec1 + vec2 - vec3

        ## Likelihood for New Table
        new_tab_tmp <- log((gamma_c / (nobs - 1 + gamma_c)) + const)
        num <- 0
        den <- 0
        for (j in 1:nfld) {
          maxS <- Cif[target, j] - 1
          if (maxS >= 0) {
            for (s in 0:maxS) {
              num <- num + log(Cif[target, j] + gamp - 1 - s + const)
            }
          }
          maxS <- Nf[j] - Cif[target, j] - 1
          if (maxS >= 0) {
            for (s in 0:maxS) {
              num <- num + log(Nf[j] - Cif[target, j] + gamp - 1 - s + const)
            }
          }
          maxS <- Nf[j] - 1
          if (maxS >= 0) {
            for (s in 0:maxS) {
              den <- den + log(Nf[j] + 2 * gamp - 1 - s + const)
            }
          }
        }
        new_tab <- num - den + new_tab_tmp

        # Likelihood to Probability
        ptab <- c(exist_tab, new_tab)
        maxtab <- max(ptab)
        exptab <- exp(ptab - maxtab)
        ptab <- exptab / sum(exptab)
        ptab <- round(ptab + const, digits = -log10(const))

        sampled_value <- rmultinom(1, 1, ptab)
        newclass01 <- c(0, sampled_value, 0)
        delpos <- length(newclass01) - 1
        if (newclass01[delpos] == 0) {
          # Student s belongs an other class
          cls01[target, ] <- newclass01[-delpos]
          cls[target] <- which.max(cls01[target, ])
        } else {
          # Student s belongs to a new class
          ncls <- ncls + 1
          new_class <- which.max(newclass01)
          cls01_left <- cls01[, 1:(ncls - 2)]
          cls01_right <- cls01[, (ncls - 1)]
          zeros <- rep(0, nrow(cls01))
          cls01 <- cbind(cls01_left, zeros, cls01_right)
          cls01[target, ] <- newclass01

          cls <- cls01 %*% (1:ncls)
          Ccf <- t(cls01) %*% tmp$U %*% fld01
          Fcf <- t(cls01) %*% (tmp$Z * (1 - tmp$U)) %*% fld01
          Ncf <- Ccf + Fcf
        }

        # Update Number of Correct/Incorrect response
        Nc <- colSums(cls01)
        Ccf[cls[target], ] <- Ccf[cls[target], ] + Cif[target, ]
        Fcf[cls[target], ] <- Fcf[cls[target], ] + Fif[target, ]
        Ncf[cls[target], ] <- Ccf[cls[target], ] + Fcf[cls[target], ]
      }
    }

    # Gibbs sampler j
    oldField <- fld # J x C
    Cjc <- t(tmp$U) %*% cls01
    Fjc <- t(tmp$Z * (1 - tmp$U)) %*% cls01
    jRand <- sample(1:testlength, testlength, replace = F)

    for (loop in 1:testlength) {
      target <- jRand[loop]

      # delete selected item
      Nf[fld[target]] <- Nf[fld[target]] - 1
      Ccf[, fld[target]] <- Ccf[, fld[target]] - t(Cjc[target, ])
      Fcf[, fld[target]] <- Fcf[, fld[target]] - t(Fjc[target, ])
      Ncf <- Ccf + Fcf
      CcfPlus <- Ccf + matrix(rep(Cjc[target, ], nfld), nrow = ncls, byrow = F)
      FcfPlus <- Fcf + matrix(rep(Fjc[target, ], nfld), nrow = ncls, byrow = F)

      ## ## if the field disappeared...
      if (Nf[fld[target]] == 0) {
        nfld <- nfld - 1
        delFld <- fld[target]
        fld01 <- fld01[, -delFld]
        fld <- fld01 %*% (1:nfld)
        Nf <- Nf[-delFld]
        Ncf <- Ncf[, -delFld]
        Ccf <- Ccf[, -delFld]
        Fcf <- Fcf[, -delFld]
        CcfPlus <- CcfPlus[, -delFld]
        FcfPlus <- FcfPlus[, -delFld]
      }

      # Table Choice!
      ## Likelihood for each field
      log_tab_tmp <- log((Nf / (testlength - 1 + gamma_f)) + const)
      vec1 <- rep(0, nfld)
      vec2 <- rep(0, nfld)
      vec3 <- rep(0, nfld)
      for (j in 1:nfld) {
        Ctmp <- 0
        Ftmp <- 0
        den <- 0
        for (i in 1:ncls) {
          ct1 <- CcfPlus[i, j] - Ccf[i, j] - 1
          if (ct1 >= 0) {
            for (s in 0:ct1) {
              Ctmp <- Ctmp + log(CcfPlus[i, j] + gamp - 1 - s + const)
            }
          }
          ct2 <- FcfPlus[i, j] - Fcf[i, j] - 1
          if (ct2 >= 0) {
            for (s in 0:ct2) {
              Ftmp <- Ftmp + log(FcfPlus[i, j] + gamp - 1 - s + const)
            }
          }
          ct3 <- CcfPlus[i, j] + FcfPlus[i, j] - Ccf[i, j] - Fcf[i, j] - 1
          if (ct3 >= 0) {
            for (s in 0:ct3) {
              den <- den + (log(CcfPlus[i, j] + FcfPlus[i, j] + 2 * gamp - 1 - s + const))
            }
          }
        }
        vec1[j] <- Ctmp
        vec2[j] <- Ftmp
        vec3[j] <- den
      }
      log_tab <- log_tab_tmp + vec1 + vec2 - vec3
      maxtab <- max(log_tab)
      exptab <- exp(log_tab - maxtab)
      ptab <- exptab / sum(exptab)
      ptab <- round(ptab + const, digits = -log10(const))

      selected_fld <- rmultinom(1, 1, ptab)
      fld01[target, ] <- selected_fld
      fld[target] <- which.max(selected_fld)
      Nf[fld[target]] <- Nf[fld[target]] + 1
      Ccf[, fld[target]] <- Ccf[, fld[target]] + t(Cjc[target, ])
      Fcf[, fld[target]] <- Fcf[, fld[target]] + t(Fjc[target, ])
      Ncf[, fld[target]] <- Ccf[, fld[target]] + Fcf[, fld[target]]
    }

    limit_count <- if (sum(abs(oldField - fld)) == 0) {
      limit_count <- limit_count + 1
    } else {
      limit_count <- 0
    }
    if (verbose) {
      message(
        sprintf(
          "iter %d: match=%d nfld=%d ncls=%d",
          iter, limit_count, nfld, ncls
        )
      )
    }
    if (limit_count == stable_limit || iter == max_iter) {
      IRM_FLG <- FALSE
    } else {
      iter <- iter + 1
    }
  }

  # Pcf update
  ret <- Ccf_Fcf_Pcf(tmp$U, tmp$Z, cls01, fld01, gamp, zeroG, fullG, type = 1)
  Ccf <- ret$Ccf
  Fcf <- ret$Fcf
  Ncf <- ret$Ncf
  Pcf <- ret$Pcf

  # model Fit
  llm <- sum(Ccf * log(Pcf + const) + Fcf * log(1 - Pcf + const))
  df <- nobs * testlength - ncls * nfld
  bic <- -2 * llm - df * log(nobs)
  indices <- c(llm, df, bic)

  # sort
  ret <- Pcf_sort(Pcf, cls01, fld01, ncls, nfld)
  Pcf <- ret$Pcf
  cls01 <- ret$cls01
  fld01 <- ret$fld01
  cls <- ret$cls
  fld <- ret$fld

  ### Pcf Update
  Cif <- tmp$U %*% fld01
  Fif <- (tmp$Z * (1 - tmp$U)) %*% fld01
  ret <- Ccf_Fcf_Pcf(tmp$U, tmp$Z, cls01, fld01, gamp, zeroG, fullG, type = 0)
  Ccf <- ret$Ccf
  Fcf <- ret$Fcf
  Ncf <- ret$Ncf
  Pcf <- ret$Pcf


  # Reorganizing small-sized classses -------------------------------
  zero_position <- which(rowSums(tmp$Z * tmp$U, na.rm = T) == 0)
  full_position <- which(rowSums(tmp$Z * tmp$U, na.rm = T) == testlength)
  delt <- 0
  delrep <- ncls

  DelRepFLG <- TRUE
  while (DelRepFLG) {
    bestfit <- 10^10
    Nc <- colSums(cls01)
    NcTable <- matrix(Nc)
    NcTable <- cbind(NcTable, 1:ncls)
    if (zeroG == 1) {
      NcTable <- NcTable[-1, ]
    }
    if (fullG == 1) {
      NcTable <- NcTable[-nrow(NcTable), ]
    }
    minclass <- NcTable[order(NcTable[, 1]), ][1, ]
    if (minclass[1] < minSize) {
      delt <- delt + 1
      if (verbose) {
        message(
          sprintf(
            "Adjusting classes: BIC=%.1f ncls=%d (min size < %d)",
            bic, ncls, minSize
          )
        )
      }
    } else {
      DelRepFLG <- FALSE
      break
    }

    ncls <- ncls - 1
    delc <- minclass[2]
    Pcf <- Pcf[-delc, ]

    EMt <- 1
    zeta <- log(rep(1, ncls) / ncls)

    EMrepFLG <- TRUE
    while (EMrepFLG) {
      log_num_Zic <- Cif %*% t(log(Pcf + const)) + Fif %*% t(log(1 - Pcf + const)) + matrix(rep(zeta, nobs), nrow = nobs)
      numeZic <- exp(log_num_Zic)
      if (zeroG == 1) {
        numeZic[, 1] <- rep(0, nobs)
      }
      if (fullG == 1) {
        numeZic[, ncol(numeZic)] <- rep(0, nobs)
      }
      denomZi <- rowSums(numeZic)

      Zic <- numeZic / denomZi
      Zic_max_list <- apply(Zic, 1, max)

      cls01 <- sign(Zic - Zic_max_list) + 1
      zero_scorer <- rep(0, ncls)
      zero_scorer[1] <- 1
      full_scorer <- rep(0, ncls)
      full_scorer[ncls] <- 1
      cls01[zero_position, ] <- matrix(rep(zero_scorer, length(zero_position)), nrow = length(zero_position), byrow = T)
      cls01[full_position, ] <- matrix(rep(full_scorer, length(full_position)), nrow = length(full_position), byrow = T)

      cls <- cls01 %*% (1:ncls)

      ret <- Ccf_Fcf_Pcf(tmp$U, tmp$Z, cls01, fld01, gamp, zeroG, fullG, type = 0)
      Ccf <- ret$Ccf
      Fcf <- ret$Fcf
      Ncf <- ret$Ncf
      Pcf <- ret$Pcf

      llm <- sum(Ccf * log(Pcf + const) + Fcf * log(1 - Pcf + const))
      df <- nobs * testlength - ncls * nfld
      bic <- -2 * llm - df * log(nobs)
      indices <- c(llm, df, bic)
      fit <- bic
      if (fit < bestfit) {
        EMt <- EMt + 1
        bestfit <- fit
        best_index <- indices
        bestPcf <- Pcf
        bestclass <- class
      } else {
        Pcf <- bestPcf
        class <- bestclass
        EMrepFLG <- FALSE
        break
      }
      if (EMt >= EM_limit) {
        stop("The EM algorithm has reached its limit. It may not have converged.")
      }
    }
  }


  # Model Fit Finally -----------------------------------------------
  ncls <- max(cls)
  nfld <- max(fld)
  cls01 <- matrix(0, ncol = ncls, nrow = nobs)
  for (i in 1:nobs) {
    cls01[i, cls[i]] <- 1
  }
  colnames(cls01) <- paste("Rank", 1:ncls)
  rownames(cls01) <- tmp$ID

  fld01 <- matrix(0, nrow = testlength, ncol = nfld)
  for (i in 1:testlength) {
    fld01[i, fld[i]] <- 1
  }
  colnames(fld01) <- paste("Field", 1:nfld)
  rownames(fld01) <- tmp$ItemLabel

  ret <- Ccf_Fcf_Pcf(tmp$U, tmp$Z, cls01, fld01, gamp, zeroG, fullG, type = 1)
  Ccf <- ret$Ccf
  Fcf <- ret$Fcf
  Ncf <- ret$Ncf
  Pcf <- ret$Pcf

  llm <- sum(Ccf * log(Pcf + const) + Fcf * log(1 - Pcf + const))
  FitIndices <- TestFit(tmp$U, tmp$Z, llm, ncls * nfld)

  # Sort ------------------------------------------------------------
  ret <- Pcf_sort(Pcf, cls01, fld01, ncls, nfld)
  Pcf <- ret$Pcf
  cls01 <- ret$cls01
  fld01 <- ret$fld01
  cls <- ret$cls
  fld <- ret$fld


  # Output ---------------------------------------------------------
  check_empty_fields(fld, nfld)
  pifr <- t(Pcf)
  rownames(pifr) <- paste0("Field", 1:nfld)
  colnames(pifr) <- paste0("Rank", 1:ncls)
  FRPIndex <- IRPindex(pifr)
  flddist <- colSums(fld01)
  clsdist <- colSums(cls01)
  TRP <- colSums(pifr * flddist)

  ret <- structure(list(
    U = tmp$U, # masked binary matrix (was using dead `U` variable before v1.14.0)
    testlength = testlength,
    msg = "Rank",
    nobs = nobs,
    n_class = ncls, # New naming convention
    n_field = nfld, # New naming convention
    n_cycle = EMt, # New naming convention
    em_cycle = EMt, # IRM-specific naming
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = pifr,
    FRPIndex = FRPIndex,
    TRP = TRP,
    FieldEstimated = fld,
    ClassEstimated = cls,
    TestFitIndices = FitIndices,
    log_lik = llm,
    # Deprecated fields (for backward compatibility)
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = EMt,
    EM_Cycle = EMt
  ), class = c("exametrika", "IRM"))
  return(ret)
}

#' @title IRM (Deprecated)
#' @description
#' This function has been renamed to \code{\link{Biclustering_IRM}}.
#' Please use \code{Biclustering_IRM} instead.
#' @param ... All arguments passed to \code{\link{Biclustering_IRM}}
#' @export
IRM <- function(...) {
  .Deprecated("Biclustering_IRM")
  Biclustering_IRM(...)
}

library(exametrika)

### IRM (Infinite Relational Model) Tests

### Setup - run model once and share across tests.
### Skipped on CRAN because the J35S515 Gibbs run dominates the file's
### runtime; all dependent tests below also call skip_on_cran().
NOT_CRAN <- identical(Sys.getenv("NOT_CRAN"), "true")
if (NOT_CRAN) {
  tmp_irm <- dataFormat(J35S515)
  result_irm <- Biclustering_IRM(tmp_irm, gamma_c = 1, gamma_f = 1, seed = 123, verbose = FALSE)
}

test_that("IRM Basic Execution", {
  skip_on_cran()
  # Basic structure checks
  expect_s3_class(result_irm, "exametrika")
  expect_true("IRM" %in% class(result_irm))

  # Result components exist
  expect_true(!is.null(result_irm$n_class))
  expect_true(!is.null(result_irm$n_field))
  expect_true(!is.null(result_irm$TRP))
  expect_true(!is.null(result_irm$LCD))
  expect_true(!is.null(result_irm$LFD))
  expect_true(!is.null(result_irm$FRP))
  expect_true(!is.null(result_irm$FieldEstimated))
  expect_true(!is.null(result_irm$ClassEstimated))

  # Dimensions are consistent
  expect_equal(length(result_irm$TRP), result_irm$n_class)
  expect_equal(length(result_irm$LCD), result_irm$n_class)
  expect_equal(length(result_irm$LFD), result_irm$n_field)
  expect_equal(length(result_irm$FieldEstimated), result_irm$testlength)
})

test_that("IRM Backward Compatibility", {
  skip_on_cran()
  # Deprecated field names should still work
  expect_equal(result_irm$Nclass, result_irm$n_class)
  expect_equal(result_irm$Nfield, result_irm$n_field)
  expect_equal(result_irm$N_Cycle, result_irm$n_cycle)
  expect_equal(result_irm$EM_Cycle, result_irm$em_cycle)
})

test_that("IRM Test Fit Indices", {
  skip_on_cran()
  # TestFitIndices
  expect_true(!is.null(result_irm$TestFitIndices))
  tfi <- result_irm$TestFitIndices
  expect_true(!is.null(tfi$model_log_like))
  expect_true(!is.null(tfi$AIC))
  expect_true(!is.null(tfi$BIC))

  # Log-likelihood should be negative
  expect_true(tfi$model_log_like < 0)

  # log_lik field should match
  expect_equal(result_irm$log_lik, tfi$model_log_like)
})

test_that("IRM FRP Validity", {
  skip_on_cran()
  # FRP should contain values between 0 and 1
  expect_true(all(result_irm$FRP >= 0 & result_irm$FRP <= 1))

  # FRP dimensions: n_field rows x n_class columns
  expect_equal(nrow(result_irm$FRP), result_irm$n_field)
  expect_equal(ncol(result_irm$FRP), result_irm$n_class)
})

test_that("IRM FRPIndex Exists", {
  skip_on_cran()
  # FRPIndex should exist
  expect_true(!is.null(result_irm$FRPIndex))
})

test_that("IRM Seed Reproducibility", {
  skip_on_cran()
  # Same seed should produce identical results
  result_a <- Biclustering_IRM(tmp_irm, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)
  result_b <- Biclustering_IRM(tmp_irm, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)

  expect_equal(result_a$n_class, result_b$n_class)
  expect_equal(result_a$n_field, result_b$n_field)
  expect_equal(result_a$FRP, result_b$FRP)
  expect_equal(result_a$TRP, result_b$TRP)
  expect_equal(result_a$FieldEstimated, result_b$FieldEstimated)
  expect_equal(result_a$ClassEstimated, result_b$ClassEstimated)
  expect_equal(result_a$log_lik, result_b$log_lik)
})

test_that("IRM Seed NULL does not set seed", {
  skip_on_cran()
  # With seed = NULL, the function should not call set.seed()
  # and the results should depend on the current RNG state.
  # We verify that the function runs without error with seed = NULL.
  result_null <- Biclustering_IRM(tmp_irm, gamma_c = 1, gamma_f = 1, seed = NULL, verbose = FALSE)
  expect_s3_class(result_null, "exametrika")
  expect_true("IRM" %in% class(result_null))
  expect_true(!is.null(result_null$n_class))
  expect_true(!is.null(result_null$n_field))
})

test_that("IRM Default seed is 123", {
  # Verify the default value of seed parameter is 123
  fn_formals <- formals(Biclustering_IRM.binary)
  expect_equal(fn_formals$seed, 123)
})


test_that("Bug #2 regression: Biclustering_IRM.binary handles missing data", {
  # Real-data missingness (HCI, SAT12) used to crash with
  # "確率ベクトル中にNAがあります" because tmp$U retained the -1
  # missingness marker that propagated through tmp$U %*% fld01.
  # Fixed in v1.14.0 by tmp$U <- tmp$U * tmp$Z masking at line 120.
  skip_on_cran()
  set.seed(20260527)
  n <- 500
  J <- 20
  U <- matrix(rbinom(n * J, 1, 0.55), n, J)
  U[sample.int(n * J, size = round(0.02 * n * J))] <- -1L
  df_miss <- dataFormat(data = U, na = -1)
  expect_true(any(df_miss$Z == 0))

  fit <- suppressWarnings(
    Biclustering_IRM(df_miss,
      gamma_c = 1, gamma_f = 1, mic = 10,
      max_iter = 300, seed = 20260527, verbose = FALSE
    )
  )
  expect_s3_class(fit, "exametrika")
  expect_true("IRM" %in% class(fit))
  expect_true(!is.null(fit$n_class) && fit$n_class >= 1)
  expect_true(!is.null(fit$n_field) && fit$n_field >= 1)
})

library(exametrika)

### Ordinal IRM (Biclustering_IRM.ordinal) Tests
### Data: J35S500 (ordinal, 35 items, 500 students, 5 categories)

### Setup - run model once and share across tests
result_oirm <- Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, seed = 123, verbose = FALSE)

test_that("Ordinal IRM Basic Execution", {
  # Basic structure checks
  expect_s3_class(result_oirm, "exametrika")
  expect_true("ordinalBiclustering" %in% class(result_oirm))

  # Result components exist
  expect_true(!is.null(result_oirm$n_class))
  expect_true(!is.null(result_oirm$n_field))
  expect_true(!is.null(result_oirm$n_cycle))
  expect_true(!is.null(result_oirm$FRP))
  expect_true(!is.null(result_oirm$FRPIndex))
  expect_true(!is.null(result_oirm$TRP))
  expect_true(!is.null(result_oirm$BFRP))
  expect_true(!is.null(result_oirm$LCD))
  expect_true(!is.null(result_oirm$LFD))
  expect_true(!is.null(result_oirm$FieldEstimated))
  expect_true(!is.null(result_oirm$ClassEstimated))
  expect_true(!is.null(result_oirm$FieldMembership))
  expect_true(!is.null(result_oirm$ClassMembership))
  expect_true(!is.null(result_oirm$Students))
  expect_true(!is.null(result_oirm$Q))
  expect_true(!is.null(result_oirm$Z))
  expect_true(!is.null(result_oirm$SOACflg))
  expect_true(!is.null(result_oirm$WOACflg))
})

test_that("Ordinal IRM Dimensions Consistency", {
  nfld <- result_oirm$n_field
  ncls <- result_oirm$n_class
  nitems <- result_oirm$testlength
  nobs <- result_oirm$nobs

  # Check basic dimensions
  expect_equal(nitems, 35)
  expect_equal(nobs, 500)
  expect_true(nfld >= 1 && nfld <= nitems)
  expect_true(ncls >= 2)

  # FRP: 3D array (nfld x ncls x maxQ)
  expect_equal(length(dim(result_oirm$FRP)), 3)
  expect_equal(dim(result_oirm$FRP)[1], nfld)
  expect_equal(dim(result_oirm$FRP)[2], ncls)
  expect_equal(dim(result_oirm$FRP)[3], 5) # J35S500 has 5 categories

  # TRP
  expect_equal(length(result_oirm$TRP), ncls)

  # LCD and LFD
  expect_equal(length(result_oirm$LCD), ncls)
  expect_equal(length(result_oirm$LFD), nfld)

  # Field/Class Estimated
  expect_equal(length(result_oirm$FieldEstimated), nitems)
  expect_equal(length(result_oirm$ClassEstimated), nobs)

  # Field/Class Membership
  expect_equal(nrow(result_oirm$FieldMembership), nitems)
  expect_equal(ncol(result_oirm$FieldMembership), nfld)
  expect_equal(nrow(result_oirm$ClassMembership), nobs)
  expect_equal(ncol(result_oirm$ClassMembership), ncls)

  # Students matrix
  expect_equal(nrow(result_oirm$Students), nobs)
  expect_equal(ncol(result_oirm$Students), ncls + 1) # membership columns + Estimate
})

test_that("Ordinal IRM FRP Probabilities Sum to 1", {
  nfld <- dim(result_oirm$FRP)[1]
  ncls <- dim(result_oirm$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_oirm$FRP[f, c, ]), 1.0, tolerance = 1e-10)
    }
  }
})

test_that("Ordinal IRM FRP Values Are Valid Probabilities", {
  # All FRP values should be between 0 and 1
  expect_true(all(result_oirm$FRP >= 0))
  expect_true(all(result_oirm$FRP <= 1))
})

test_that("Ordinal IRM Expected Scores in Valid Range", {
  nfld <- dim(result_oirm$FRP)[1]
  ncls <- dim(result_oirm$FRP)[2]
  maxQ <- dim(result_oirm$FRP)[3]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      esp <- sum((1:maxQ) * result_oirm$FRP[f, c, ])
      expect_gte(esp, 1.0)
      expect_lte(esp, maxQ)
    }
  }
})

test_that("Ordinal IRM Distribution Sums", {
  # LCD (Class Distribution) should sum to nobs
  expect_equal(sum(result_oirm$LCD), result_oirm$nobs)

  # LFD (Field Distribution) should sum to testlength
  expect_equal(sum(result_oirm$LFD), result_oirm$testlength)
})

test_that("Ordinal IRM ClassMembership Rows Sum to 1", {
  row_sums <- unname(rowSums(result_oirm$ClassMembership))
  for (i in seq_along(row_sums)) {
    expect_equal(row_sums[i], 1.0, tolerance = 1e-10)
  }
})

test_that("Ordinal IRM FieldMembership Rows Sum to 1", {
  row_sums <- unname(rowSums(result_oirm$FieldMembership))
  for (i in seq_along(row_sums)) {
    expect_equal(row_sums[i], 1.0, tolerance = 1e-10)
  }
})

test_that("Ordinal IRM ClassEstimated Values Are Valid", {
  expect_true(all(result_oirm$ClassEstimated >= 1))
  expect_true(all(result_oirm$ClassEstimated <= result_oirm$n_class))
})

test_that("Ordinal IRM FieldEstimated Values Are Valid", {
  expect_true(all(result_oirm$FieldEstimated >= 1))
  expect_true(all(result_oirm$FieldEstimated <= result_oirm$n_field))
})

test_that("Ordinal IRM TRP Validity", {
  # TRP is the sum of weighted expected scores across fields
  # Should be positive and increasing (if mic is satisfied)
  expect_true(all(result_oirm$TRP > 0))

  # TRP values should be in a reasonable range
  nfld <- result_oirm$n_field
  maxQ <- 5
  # Each field contributes expected score 1 to maxQ, so total TRP per class
  # should be between nfld and nfld * maxQ
  expect_true(all(result_oirm$TRP >= nfld * 0.5))
})

test_that("Ordinal IRM BFRP Structure", {
  # BFRP should be a list with Weighted and Observed
  expect_true(is.list(result_oirm$BFRP))
  expect_true(!is.null(result_oirm$BFRP$Weighted))
  expect_true(!is.null(result_oirm$BFRP$Observed))

  nfld <- result_oirm$n_field
  ncls <- result_oirm$n_class

  # Both should be nfld x ncls matrices
  expect_equal(dim(result_oirm$BFRP$Weighted), c(nfld, ncls))
  expect_equal(dim(result_oirm$BFRP$Observed), c(nfld, ncls))

  # Values should be positive (expected scores)
  expect_true(all(result_oirm$BFRP$Weighted > 0))
})

test_that("Ordinal IRM FRPIndex Structure and Validity", {
  frpidx <- result_oirm$FRPIndex
  nfld <- result_oirm$n_field

  # FRPIndex dimensions: nfld rows x 6 columns
  expect_equal(nrow(frpidx), nfld)
  expect_equal(ncol(frpidx), 6)
  expect_equal(names(frpidx), c("Alpha", "A", "Beta", "B", "Gamma", "C"))

  # Alpha: location of steepest ascent (integer, 0 to ncls-1)
  expect_true(all(frpidx$Alpha >= 0))
  expect_true(all(frpidx$Alpha < result_oirm$n_class))

  # A: maximum slope (should be positive for meaningful differentiation)
  expect_true(all(frpidx$A > 0))

  # B: profile value at Beta location (0 to 1 for normalized expected scores)
  expect_true(all(frpidx$B >= 0))
  expect_true(all(frpidx$B <= 1))

  # Gamma: non-monotonicity proportion (0 to 1)
  expect_true(all(frpidx$Gamma >= 0))
  expect_true(all(frpidx$Gamma <= 1))

  # C: non-monotonicity sum (should be <= 0)
  expect_true(all(frpidx$C <= 0))
})

test_that("Ordinal IRM SOAC/WOAC Flags Are Logical", {
  expect_true(is.logical(result_oirm$SOACflg))
  expect_true(is.logical(result_oirm$WOACflg))

  # If SOAC is TRUE, WOAC must also be TRUE (SOAC implies WOAC)
  if (result_oirm$SOACflg) {
    expect_true(result_oirm$WOACflg)
  }
})

test_that("Ordinal IRM mic Parameter Stored", {
  # mic parameter should be stored in the result
  expect_true(!is.null(result_oirm$mic))
  expect_true(is.logical(result_oirm$mic))
  expect_true(result_oirm$mic) # default is TRUE
})

test_that("Ordinal IRM Test Fit Indices", {
  # TestFitIndices should exist and have proper components
  expect_true(!is.null(result_oirm$TestFitIndices))
  tfi <- result_oirm$TestFitIndices
  expect_s3_class(tfi, "exametrika")
  expect_true("ModelFit" %in% class(tfi))

  expect_true(!is.null(tfi$model_log_like))
  expect_true(!is.null(tfi$bench_log_like))
  expect_true(!is.null(tfi$null_log_like))
  expect_true(!is.null(tfi$AIC))
  expect_true(!is.null(tfi$BIC))
  expect_true(!is.null(tfi$RMSEA))

  # Log-likelihood should be negative
  expect_true(tfi$model_log_like < 0)

  # Log-likelihood ordering: model < bench
  expect_true(tfi$model_log_like <= tfi$bench_log_like)

  # log_lik field should match
  expect_equal(result_oirm$log_lik, tfi$model_log_like)
})

test_that("Ordinal IRM Backward Compatibility", {
  # Deprecated field names should still work
  expect_equal(result_oirm$Nclass, result_oirm$n_class)
  expect_equal(result_oirm$Nfield, result_oirm$n_field)
  expect_equal(result_oirm$N_Cycle, result_oirm$n_cycle)
  expect_equal(result_oirm$LogLik, result_oirm$log_lik)

  # LRD should equal LCD (alias)
  expect_equal(result_oirm$LRD, result_oirm$LCD)

  # RMD should equal CMD (alias)
  expect_equal(result_oirm$RMD, result_oirm$CMD)
})

test_that("Ordinal IRM Seed Reproducibility", {
  # Same seed should produce identical results
  result_a <- Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)
  result_b <- Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)

  expect_equal(result_a$n_class, result_b$n_class)
  expect_equal(result_a$n_field, result_b$n_field)
  expect_equal(result_a$FRP, result_b$FRP)
  expect_equal(result_a$TRP, result_b$TRP)
  expect_equal(result_a$FieldEstimated, result_b$FieldEstimated)
  expect_equal(result_a$ClassEstimated, result_b$ClassEstimated)
  expect_equal(result_a$log_lik, result_b$log_lik)
  expect_equal(result_a$LCD, result_b$LCD)
  expect_equal(result_a$LFD, result_b$LFD)
  expect_equal(result_a$FRPIndex, result_b$FRPIndex)
  expect_equal(result_a$SOACflg, result_b$SOACflg)
  expect_equal(result_a$WOACflg, result_b$WOACflg)
})

test_that("Ordinal IRM Seed NULL Does Not Set Seed", {
  result_null <- Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, seed = NULL, verbose = FALSE)
  expect_s3_class(result_null, "exametrika")
  expect_true("ordinalBiclustering" %in% class(result_null))
  expect_true(!is.null(result_null$n_class))
  expect_true(!is.null(result_null$n_field))
  expect_true(!is.null(result_null$FRPIndex))
  expect_true(!is.null(result_null$TRP))
})

test_that("Ordinal IRM Default Seed Is 123", {
  fn_formals <- formals(Biclustering_IRM.ordinal)
  expect_equal(fn_formals$seed, 123)
})

test_that("Ordinal IRM S3 Dispatch Works", {
  # J35S500 has response.type == "ordinal"
  expect_equal(J35S500$response.type, "ordinal")
})

test_that("Ordinal IRM Alpha Parameter Validation", {
  # alpha must be positive
  expect_error(
    Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, alpha = 0, seed = 123, verbose = FALSE),
    "alpha must be positive"
  )
  expect_error(
    Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, alpha = -1, seed = 123, verbose = FALSE),
    "alpha must be positive"
  )
})

test_that("Ordinal IRM Q and Z Matrices Stored Correctly", {
  expect_equal(dim(result_oirm$Q), c(500, 35))
  expect_equal(dim(result_oirm$Z), c(500, 35))

  # Z values should be 0 or 1
  expect_true(all(result_oirm$Z %in% c(0, 1)))

  # Q values for observed responses (Z=1) should be integers in 1:maxQ
  observed <- result_oirm$Z == 1
  expect_true(all(result_oirm$Q[observed] >= 1))
  expect_true(all(result_oirm$Q[observed] <= 5))
})

test_that("Ordinal IRM Students Matrix Structure", {
  ncls <- result_oirm$n_class
  students <- result_oirm$Students

  # Last column should be "Estimate"
  expect_equal(colnames(students)[ncol(students)], "Estimate")

  # Membership columns should sum to approximately 1 per student
  membership_sums <- unname(rowSums(students[, 1:ncls]))
  for (i in 1:nrow(students)) {
    expect_equal(membership_sums[i], 1.0, tolerance = 1e-10)
  }

  # Estimate column should match ClassEstimated
  expect_equal(unname(as.numeric(students[, "Estimate"])), unname(result_oirm$ClassEstimated))
})

test_that("Ordinal IRM mic=FALSE Works", {
  # Running with mic=FALSE should still produce valid results
  result_nomic <- Biclustering_IRM(J35S500,
    gamma_c = 1, gamma_f = 1,
    mic = FALSE, seed = 123, verbose = FALSE
  )
  expect_s3_class(result_nomic, "exametrika")
  expect_true("ordinalBiclustering" %in% class(result_nomic))
  expect_false(result_nomic$mic)

  # FRP should still be valid probabilities
  expect_true(all(result_nomic$FRP >= 0))
  expect_true(all(result_nomic$FRP <= 1))

  nfld <- dim(result_nomic$FRP)[1]
  ncls <- dim(result_nomic$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_nomic$FRP[f, c, ]), 1.0, tolerance = 1e-10)
    }
  }
})

library(exametrika)

### Nominal IRM (Biclustering_IRM.nominal) Tests
### Data: J20S600 (nominal, 20 items, 600 students, 4 categories)

### Setup - run model once and share across tests
result_nirm <- Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, seed = 123, verbose = FALSE)

test_that("Nominal IRM Basic Execution", {
  # Basic structure checks
  expect_s3_class(result_nirm, "exametrika")
  expect_true("nominalBiclustering" %in% class(result_nirm))

  # Result components exist
  expect_true(!is.null(result_nirm$n_class))
  expect_true(!is.null(result_nirm$n_field))
  expect_true(!is.null(result_nirm$n_cycle))
  expect_true(!is.null(result_nirm$FRP))
  expect_true(!is.null(result_nirm$LCD))
  expect_true(!is.null(result_nirm$LFD))
  expect_true(!is.null(result_nirm$FieldEstimated))
  expect_true(!is.null(result_nirm$ClassEstimated))
  expect_true(!is.null(result_nirm$FieldMembership))
  expect_true(!is.null(result_nirm$ClassMembership))
  expect_true(!is.null(result_nirm$Students))
  expect_true(!is.null(result_nirm$Q))
  expect_true(!is.null(result_nirm$Z))
})

test_that("Nominal IRM Dimensions Consistency", {
  nfld <- result_nirm$n_field
  ncls <- result_nirm$n_class
  nitems <- result_nirm$testlength
  nobs <- result_nirm$nobs

  # Check basic dimensions

  expect_equal(nitems, 20)
  expect_equal(nobs, 600)
  expect_true(nfld >= 1 && nfld <= nitems)
  expect_true(ncls >= 2)

  # FRP: 3D array (nfld x ncls x maxQ)
  expect_equal(length(dim(result_nirm$FRP)), 3)
  expect_equal(dim(result_nirm$FRP)[1], nfld)
  expect_equal(dim(result_nirm$FRP)[2], ncls)
  expect_equal(dim(result_nirm$FRP)[3], 4) # J20S600 has 4 categories

  # LCD and LFD
  expect_equal(length(result_nirm$LCD), ncls)
  expect_equal(length(result_nirm$LFD), nfld)

  # Field/Class Estimated
  expect_equal(length(result_nirm$FieldEstimated), nitems)
  expect_equal(length(result_nirm$ClassEstimated), nobs)

  # Field/Class Membership
  expect_equal(nrow(result_nirm$FieldMembership), nitems)
  expect_equal(ncol(result_nirm$FieldMembership), nfld)
  expect_equal(nrow(result_nirm$ClassMembership), nobs)
  expect_equal(ncol(result_nirm$ClassMembership), ncls)

  # Students matrix
  expect_equal(nrow(result_nirm$Students), nobs)
  expect_equal(ncol(result_nirm$Students), ncls + 1) # membership columns + Estimate
})

test_that("Nominal IRM FRP Probabilities Sum to 1", {
  nfld <- dim(result_nirm$FRP)[1]
  ncls <- dim(result_nirm$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_nirm$FRP[f, c, ]), 1.0, tolerance = 1e-10)
    }
  }
})

test_that("Nominal IRM FRP Values Are Valid Probabilities", {
  # All FRP values should be between 0 and 1
  expect_true(all(result_nirm$FRP >= 0))
  expect_true(all(result_nirm$FRP <= 1))
})

test_that("Nominal IRM Distribution Sums", {
  # LCD (Class Distribution) should sum to nobs
  expect_equal(sum(result_nirm$LCD), result_nirm$nobs)

  # LFD (Field Distribution) should sum to testlength
  expect_equal(sum(result_nirm$LFD), result_nirm$testlength)
})

test_that("Nominal IRM ClassMembership Rows Sum to 1", {
  row_sums <- unname(rowSums(result_nirm$ClassMembership))
  for (i in seq_along(row_sums)) {
    expect_equal(row_sums[i], 1.0, tolerance = 1e-10)
  }
})

test_that("Nominal IRM FieldMembership Rows Sum to 1", {
  row_sums <- unname(rowSums(result_nirm$FieldMembership))
  for (i in seq_along(row_sums)) {
    expect_equal(row_sums[i], 1.0, tolerance = 1e-10)
  }
})

test_that("Nominal IRM ClassEstimated Values Are Valid", {
  # Each estimated class should be within 1:n_class
  expect_true(all(result_nirm$ClassEstimated >= 1))
  expect_true(all(result_nirm$ClassEstimated <= result_nirm$n_class))
})

test_that("Nominal IRM FieldEstimated Values Are Valid", {
  # Each estimated field should be within 1:n_field
  expect_true(all(result_nirm$FieldEstimated >= 1))
  expect_true(all(result_nirm$FieldEstimated <= result_nirm$n_field))
})

test_that("Nominal IRM Test Fit Indices", {
  # TestFitIndices should exist and have proper components
  expect_true(!is.null(result_nirm$TestFitIndices))
  tfi <- result_nirm$TestFitIndices
  expect_s3_class(tfi, "exametrika")
  expect_true("ModelFit" %in% class(tfi))

  expect_true(!is.null(tfi$model_log_like))
  expect_true(!is.null(tfi$null_log_like))
  expect_true(!is.null(tfi$AIC))
  expect_true(!is.null(tfi$BIC))

  # Nominal data: no benchmark model, chi-square based indices are NA
  expect_true(is.na(tfi$bench_log_like))
  expect_true(is.na(tfi$RMSEA))
  expect_true(is.na(tfi$NFI))
  expect_true(is.na(tfi$CFI))

  # Log-likelihood should be negative
  expect_true(tfi$model_log_like < 0)

  # log_lik field should match
  expect_equal(result_nirm$log_lik, tfi$model_log_like)
})

test_that("Nominal IRM Backward Compatibility", {
  # Deprecated field names should still work
  expect_equal(result_nirm$Nclass, result_nirm$n_class)
  expect_equal(result_nirm$Nfield, result_nirm$n_field)
  expect_equal(result_nirm$N_Cycle, result_nirm$n_cycle)
  expect_equal(result_nirm$LogLik, result_nirm$log_lik)

  # LRD should equal LCD (alias)
  expect_equal(result_nirm$LRD, result_nirm$LCD)

  # RMD should equal CMD (alias)
  expect_equal(result_nirm$RMD, result_nirm$CMD)
})

test_that("Nominal IRM Seed Reproducibility", {
  # Same seed should produce identical results
  result_a <- Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)
  result_b <- Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)

  expect_equal(result_a$n_class, result_b$n_class)
  expect_equal(result_a$n_field, result_b$n_field)
  expect_equal(result_a$FRP, result_b$FRP)
  expect_equal(result_a$FieldEstimated, result_b$FieldEstimated)
  expect_equal(result_a$ClassEstimated, result_b$ClassEstimated)
  expect_equal(result_a$log_lik, result_b$log_lik)
  expect_equal(result_a$LCD, result_b$LCD)
  expect_equal(result_a$LFD, result_b$LFD)
})

test_that("Nominal IRM Seed NULL Does Not Set Seed", {
  # With seed = NULL, the function should not call set.seed()
  # and the results should depend on the current RNG state.
  result_null <- Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, seed = NULL, verbose = FALSE)
  expect_s3_class(result_null, "exametrika")
  expect_true("nominalBiclustering" %in% class(result_null))
  expect_true(!is.null(result_null$n_class))
  expect_true(!is.null(result_null$n_field))
})

test_that("Nominal IRM Default Seed Is 123", {
  # Verify the default value of seed parameter is 123
  fn_formals <- formals(Biclustering_IRM.nominal)
  expect_equal(fn_formals$seed, 123)
})

test_that("Nominal IRM S3 Dispatch Works", {
  # Biclustering_IRM should dispatch to .nominal method for nominal data
  # J20S600 has response.type == "nominal"
  expect_equal(J20S600$response.type, "nominal")
})

test_that("Nominal IRM Alpha Parameter Validation", {
  # alpha must be positive
  expect_error(
    Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, alpha = 0, seed = 123, verbose = FALSE),
    "alpha must be positive"
  )
  expect_error(
    Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, alpha = -1, seed = 123, verbose = FALSE),
    "alpha must be positive"
  )
})

test_that("Nominal IRM Q and Z Matrices Stored Correctly", {
  # Q and Z should be stored in the result
  expect_equal(dim(result_nirm$Q), c(600, 20))
  expect_equal(dim(result_nirm$Z), c(600, 20))

  # Z values should be 0 or 1
  expect_true(all(result_nirm$Z %in% c(0, 1)))

  # Q values for observed responses (Z=1) should be integers in 1:maxQ
  observed <- result_nirm$Z == 1
  expect_true(all(result_nirm$Q[observed] >= 1))
  expect_true(all(result_nirm$Q[observed] <= 4))
})

test_that("Nominal IRM Students Matrix Structure", {
  ncls <- result_nirm$n_class
  students <- result_nirm$Students

  # Last column should be "Estimate"
  expect_equal(colnames(students)[ncol(students)], "Estimate")

  # Membership columns should sum to approximately 1 per student
  membership_sums <- unname(rowSums(students[, 1:ncls]))
  for (i in 1:nrow(students)) {
    expect_equal(membership_sums[i], 1.0, tolerance = 1e-10)
  }

  # Estimate column should match ClassEstimated
  expect_equal(unname(as.numeric(students[, "Estimate"])), unname(result_nirm$ClassEstimated))
})

library(exametrika)

### Rated IRM (Biclustering_IRM.rated) Tests
### Data: J21S300 (rated, 21 items, 300 students, 4 categories)

### Setup - run model once and share across tests
result_rirm <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, seed = 123, verbose = FALSE)

test_that("Rated IRM Basic Execution", {
  # Basic structure checks
  expect_s3_class(result_rirm, "exametrika")
  expect_true("ratedBiclustering" %in% class(result_rirm))

  # Result components exist
  expect_true(!is.null(result_rirm$n_class))
  expect_true(!is.null(result_rirm$n_field))
  expect_true(!is.null(result_rirm$n_cycle))
  expect_true(!is.null(result_rirm$FRP))
  expect_true(!is.null(result_rirm$FieldFRP))
  expect_true(!is.null(result_rirm$quasiFRP))
  expect_true(!is.null(result_rirm$FRPIndex))
  expect_true(!is.null(result_rirm$TRP))
  expect_true(!is.null(result_rirm$LCD))
  expect_true(!is.null(result_rirm$LFD))
  expect_true(!is.null(result_rirm$FieldEstimated))
  expect_true(!is.null(result_rirm$ClassEstimated))
  expect_true(!is.null(result_rirm$FieldMembership))
  expect_true(!is.null(result_rirm$ClassMembership))
  expect_true(!is.null(result_rirm$Students))
  expect_true(!is.null(result_rirm$FieldAnalysis))
  expect_true(!is.null(result_rirm$Q))
  expect_true(!is.null(result_rirm$U))
  expect_true(!is.null(result_rirm$Z))
  expect_true(!is.null(result_rirm$SOACflg))
  expect_true(!is.null(result_rirm$WOACflg))
})

test_that("Rated IRM Dimensions Consistency", {
  nfld <- result_rirm$n_field
  ncls <- result_rirm$n_class
  nitems <- result_rirm$testlength
  nobs <- result_rirm$nobs

  # Check basic dimensions
  expect_equal(nitems, 21)
  expect_equal(nobs, 300)
  expect_true(nfld >= 1 && nfld <= nitems)
  expect_true(ncls >= 2)

  # FRP: 3D array (nfld x ncls x maxQ)
  expect_equal(length(dim(result_rirm$FRP)), 3)
  expect_equal(dim(result_rirm$FRP)[1], nfld)
  expect_equal(dim(result_rirm$FRP)[2], ncls)
  expect_equal(dim(result_rirm$FRP)[3], 4) # J21S300 has 4 categories

  # FieldFRP: nfld x ncls
  expect_equal(dim(result_rirm$FieldFRP), c(nfld, ncls))

  # quasiFRP: nitems x ncls
  expect_equal(dim(result_rirm$quasiFRP), c(nitems, ncls))

  # TRP
  expect_equal(length(result_rirm$TRP), ncls)

  # LCD and LFD
  expect_equal(length(result_rirm$LCD), ncls)
  expect_equal(length(result_rirm$LFD), nfld)

  # Field/Class Estimated
  expect_equal(length(result_rirm$FieldEstimated), nitems)
  expect_equal(length(result_rirm$ClassEstimated), nobs)

  # Field/Class Membership
  expect_equal(nrow(result_rirm$FieldMembership), nitems)
  expect_equal(ncol(result_rirm$FieldMembership), nfld)
  expect_equal(nrow(result_rirm$ClassMembership), nobs)
  expect_equal(ncol(result_rirm$ClassMembership), ncls)

  # Students matrix: membership + Estimate + Rank-Up Odds + Rank-Down Odds
  expect_equal(nrow(result_rirm$Students), nobs)
  expect_equal(ncol(result_rirm$Students), ncls + 3)
})

test_that("Rated IRM FRP Probabilities Sum to 1", {
  nfld <- dim(result_rirm$FRP)[1]
  ncls <- dim(result_rirm$FRP)[2]
  for (f in 1:nfld) {
    for (c in 1:ncls) {
      expect_equal(sum(result_rirm$FRP[f, c, ]), 1.0, tolerance = 1e-10)
    }
  }
})

test_that("Rated IRM FRP Values Are Valid Probabilities", {
  expect_true(all(result_rirm$FRP >= 0))
  expect_true(all(result_rirm$FRP <= 1))
})

test_that("Rated IRM FieldFRP Values Are Valid", {
  expect_true(all(result_rirm$FieldFRP >= 0, na.rm = TRUE))
  expect_true(all(result_rirm$FieldFRP <= 1, na.rm = TRUE))
})

test_that("Rated IRM quasiFRP Values Are Valid", {
  expect_true(all(result_rirm$quasiFRP >= 0))
  expect_true(all(result_rirm$quasiFRP <= 1))
})

test_that("Rated IRM Distribution Sums", {
  expect_equal(sum(result_rirm$LCD), result_rirm$nobs)
  expect_equal(sum(result_rirm$LFD), result_rirm$testlength)
})

test_that("Rated IRM ClassMembership Rows Sum to 1", {
  row_sums <- unname(rowSums(result_rirm$ClassMembership))
  for (i in seq_along(row_sums)) {
    expect_equal(row_sums[i], 1.0, tolerance = 1e-10)
  }
})

test_that("Rated IRM FieldMembership Rows Sum to 1", {
  row_sums <- unname(rowSums(result_rirm$FieldMembership))
  for (i in seq_along(row_sums)) {
    expect_equal(row_sums[i], 1.0, tolerance = 1e-10)
  }
})

test_that("Rated IRM ClassEstimated Values Are Valid", {
  expect_true(all(result_rirm$ClassEstimated >= 1))
  expect_true(all(result_rirm$ClassEstimated <= result_rirm$n_class))
})

test_that("Rated IRM FieldEstimated Values Are Valid", {
  expect_true(all(result_rirm$FieldEstimated >= 1))
  expect_true(all(result_rirm$FieldEstimated <= result_rirm$n_field))
})

test_that("Rated IRM Two-Layer Fit Indices", {
  # Layer 1: Binary fit indices (default)
  tfi <- result_rirm$TestFitIndices
  expect_s3_class(tfi, "exametrika")
  expect_true("ModelFit" %in% class(tfi))
  expect_true(!is.null(tfi$model_log_like))
  expect_true(!is.null(tfi$AIC))
  expect_true(!is.null(tfi$BIC))
  expect_true(!is.null(tfi$CFI))
  expect_true(!is.null(tfi$RMSEA))
  expect_true(tfi$model_log_like < 0)

  # Binary layer: CFI and RMSEA should NOT be NA
  expect_false(is.na(tfi$CFI))
  expect_false(is.na(tfi$RMSEA))

  # Layer 2: Nominal fit indices
  tfi_nom <- result_rirm$TestFitIndices_nominal
  expect_s3_class(tfi_nom, "exametrika")
  expect_true("ModelFit" %in% class(tfi_nom))
  expect_true(!is.null(tfi_nom$AIC))
  expect_true(!is.null(tfi_nom$BIC))
  expect_true(tfi_nom$model_log_like < 0)

  # Nominal layer: CFI and RMSEA should be NA
  expect_true(is.na(tfi_nom$CFI))
  expect_true(is.na(tfi_nom$RMSEA))
  expect_true(is.na(tfi_nom$bench_log_like))

  # log_lik fields should match
  expect_equal(result_rirm$log_lik, tfi$model_log_like)
  expect_equal(result_rirm$log_lik_nominal, tfi_nom$model_log_like)
})

test_that("Rated IRM FRPIndex Structure and Validity", {
  frpidx <- result_rirm$FRPIndex
  nfld <- result_rirm$n_field

  expect_true(nrow(frpidx) >= 1)
  expect_equal(ncol(frpidx), 6)
  expect_equal(names(frpidx), c("Alpha", "A", "Beta", "B", "Gamma", "C"))

  expect_true(all(frpidx$Alpha >= 0))
  expect_true(all(frpidx$A > 0))
  expect_true(all(frpidx$B >= 0))
  expect_true(all(frpidx$B <= 1))
  expect_true(all(frpidx$Gamma >= 0))
  expect_true(all(frpidx$Gamma <= 1))
})

test_that("Rated IRM SOAC/WOAC Flags Are Logical", {
  expect_true(is.logical(result_rirm$SOACflg))
  expect_true(is.logical(result_rirm$WOACflg))

  if (result_rirm$SOACflg) {
    expect_true(result_rirm$WOACflg)
  }
})

test_that("Rated IRM Backward Compatibility", {
  expect_equal(result_rirm$Nclass, result_rirm$n_class)
  expect_equal(result_rirm$Nfield, result_rirm$n_field)
  expect_equal(result_rirm$N_Cycle, result_rirm$n_cycle)
  expect_equal(result_rirm$LogLik, result_rirm$log_lik)

  expect_equal(result_rirm$LRD, result_rirm$LCD)
  expect_equal(result_rirm$RMD, result_rirm$CMD)
})

test_that("Rated IRM Seed Reproducibility", {
  result_a <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)
  result_b <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, seed = 42, verbose = FALSE)

  expect_equal(result_a$n_class, result_b$n_class)
  expect_equal(result_a$n_field, result_b$n_field)
  expect_equal(result_a$FRP, result_b$FRP)
  expect_equal(result_a$FieldFRP, result_b$FieldFRP)
  expect_equal(result_a$TRP, result_b$TRP)
  expect_equal(result_a$FieldEstimated, result_b$FieldEstimated)
  expect_equal(result_a$ClassEstimated, result_b$ClassEstimated)
  expect_equal(result_a$log_lik, result_b$log_lik)
  expect_equal(result_a$log_lik_nominal, result_b$log_lik_nominal)
  expect_equal(result_a$LCD, result_b$LCD)
  expect_equal(result_a$LFD, result_b$LFD)
  expect_equal(result_a$SOACflg, result_b$SOACflg)
  expect_equal(result_a$WOACflg, result_b$WOACflg)
})

test_that("Rated IRM Seed NULL Does Not Set Seed", {
  result_null <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, seed = NULL, verbose = FALSE)
  expect_s3_class(result_null, "exametrika")
  expect_true("ratedBiclustering" %in% class(result_null))
  expect_true(!is.null(result_null$n_class))
  expect_true(!is.null(result_null$n_field))
})

test_that("Rated IRM Default Seed Is 123", {
  fn_formals <- formals(Biclustering_IRM.rated)
  expect_equal(fn_formals$seed, 123)
})

test_that("Rated IRM S3 Dispatch Works", {
  expect_equal(J21S300$response.type, "rated")
})

test_that("Rated IRM Q U Z Matrices Stored Correctly", {
  expect_equal(dim(result_rirm$Q), c(300, 21))
  expect_equal(dim(result_rirm$U), c(300, 21))
  expect_equal(dim(result_rirm$Z), c(300, 21))

  # Z values should be 0 or 1
  expect_true(all(result_rirm$Z %in% c(0, 1)))

  # Q values for observed responses should be in 1:4
  observed <- result_rirm$Z == 1
  expect_true(all(result_rirm$Q[observed] >= 1))
  expect_true(all(result_rirm$Q[observed] <= 4))

  # U values should be 0 or 1 (binary correct/incorrect)
  expect_true(all(result_rirm$U %in% c(0, 1)))
})

test_that("Rated IRM Students Matrix Structure", {
  ncls <- result_rirm$n_class
  students <- result_rirm$Students

  # Should have: membership columns + Estimate + Rank-Up Odds + Rank-Down Odds
  expect_equal(colnames(students)[ncls + 1], "Estimate")
  expect_equal(colnames(students)[ncls + 2], "Rank-Up Odds")
  expect_equal(colnames(students)[ncls + 3], "Rank-Down Odds")

  # Membership columns should sum to approximately 1 per student
  membership_sums <- unname(rowSums(students[, 1:ncls]))
  for (i in 1:nrow(students)) {
    expect_equal(membership_sums[i], 1.0, tolerance = 1e-10)
  }

  # Estimate column should match ClassEstimated
  expect_equal(unname(as.numeric(students[, "Estimate"])), unname(result_rirm$ClassEstimated))
})

test_that("Rated IRM FieldAnalysis Structure", {
  fa <- result_rirm$FieldAnalysis
  nfld <- result_rirm$n_field

  expect_equal(nrow(fa), result_rirm$testlength)
  expect_equal(ncol(fa), 2 + nfld) # CRR + LFE + Field1..FieldN

  # CRR should be between 0 and 1
  expect_true(all(fa[, "CRR"] >= 0))
  expect_true(all(fa[, "CRR"] <= 1))

  # LFE should be valid field indices
  expect_true(all(fa[, "LFE"] >= 1))
  expect_true(all(fa[, "LFE"] <= nfld))
})

test_that("Rated IRM print method runs without error", {
  expect_no_error(capture.output(print(result_rirm)))
})

#' @title Joint Sample Size
#' @description
#' The joint sample size is a matrix whose elements are the number of
#' individuals who responded to each pair of items.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return Returns a matrix of class c("exametrika", "matrix") where each element (i,j)
#'   represents the number of students who responded to both item i and item j. The
#'   diagonal elements represent the total number of responses for each item.
#' @export
JointSampleSize <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("JointSampleSize")
}
#' @rdname JointSampleSize
#' @export
JointSampleSize.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  JointSampleSize.binary(U, na, Z, w)
}

#' @rdname JointSampleSize
#' @export
JointSampleSize.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  S_jk <- t(U$Z) %*% U$Z
  structure(S_jk, class = c("exametrika", "matrix"))
}

#' @title Joint Correct Response Rate
#' @description
#' The joint correct response rate (JCRR) is the rate of students who passed
#' both items. This function is applicable only to binary response data.
#' For non-binary data, it will automatically redirect to the JSR function
#' with an appropriate message.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A matrix of joint correct response rates with exametrika class.
#' Each element (i,j) represents the proportion of students who correctly
#' answered both items i and j.
#' @examples
#' # example code
#' # Calculate JCRR using sample dataset J5S10
#' JCRR(J5S10)
#' @export
JCRR <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("JCRR")
}

#' @rdname JCRR
#' @export
JCRR.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = JCRR.binary(U, na = NULL, Z = NULL, w = NULL),
      "rated" = JCRR.nominal(U, na = NULL, Z = NULL, w = NULL),
      "ordinal" = JCRR.nominal(U, na = NULL, Z = NULL, w = NULL),
      "nominal" = JCRR.nominal(U, na = NULL, Z = NULL, w = NULL)
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    JCRR(U)
  }
}

#' @rdname JCRR
#' @export
JCRR.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  P_J <- t(U$Z * U$U) %*% (U$Z * U$U) / (t(U$Z) %*% U$Z)
  structure(P_J, class = c("exametrika", "matrix"))
}

#' @rdname JCRR
#' @export
JCRR.nominal <- function(U, na = NULL, Z = NULL, w = NULL) {
  message("JCRR is for binary data only. Using Joint Selection Rate for your polytomous data instead.")
  JSR(U)
}

#' @title Joint Selection Rate
#' @description Calculate the Joint Selection Rate (JSR) for polytomous data.
#'   JSR measures the proportion of respondents who selected specific category
#'   combinations between pairs of items. For each pair of items (j,k),
#'   it returns a contingency table showing the joint probability of selecting
#'   each category combination.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @return A list of Joint Selection Rate matrices for each item pair.
#' @examples
#' # example code
#' # Calculate JCRR using sample dataset J5S1000
#' JSR(J5S1000)
#' @export
JSR <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type == "binary") {
    message("JSR is for non-binary data only. Using Joint Correct Response Rate for your binary data instead.")
    JCRR(U)
  }
  nitems <- NCOL(U$Q)
  ncat <- U$categories
  U$Q[U$Z == 0] <- NA
  JSR <- vector("list", nitems)
  for (j in 1:nitems) {
    JSR[[j]] <- vector("list", nitems)
    for (k in 1:nitems) {
      tbl <- table(U$Q[, j], U$Q[, k], useNA = "no") / sum(table(U$Q[, j], U$Q[, k], useNA = "no"))
      colnames(tbl) <- unlist(U$CategoryLabel[k])
      rownames(tbl) <- unlist(U$CategoryLabel[j])
      JSR[[j]][[k]] <- tbl
    }
  }
  return(JSR)
}

#' @title Conditional Selection Rate
#' @description Calculate the Conditional Selection Rate (CSR) for polytomous data.
#'   CSR measures the proportion of respondents who selected a specific category
#'   in item K, given that they selected a particular category in item J.
#' @details
#'   The function returns a nested list structure CSR, where \code{CSR[[j]][[k]]} contains
#'   a matrix of conditional probabilities. In this matrix, the element at row l and
#'   column m represents P(K=m|J=l), which is the probability of selecting category m
#'   for item K, given that category l was selected for item J.
#'
#'   Mathematically, for each cell (l,m) in the \code{CSR[[j]][[k]]} matrix:
#'   \code{CSR[[j]][[k]][l,m] = P(Item K = category m | Item J = category l)}
#'
#'   This is calculated as the number of respondents who selected both category l for
#'   item J and category m for item K, divided by the total number of respondents who
#'   selected category l for item J.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @return A list of Joint Selection Rate matrices for each item pair.
#' @examples
#' # example code
#' # Calculate CSR using sample dataset J5S1000
#' CSR(J5S1000)
#'
#' # Extract the conditional selection rates from item 1 to item 2
#' csr_1_2 <- CSR(J5S1000)[[1]][[2]]
#' # This shows the probability of selecting each category in item 2
#' # given that a specific category was selected in item 1
#' @export
CSR <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type == "binary") {
    message("CSR is for non-binary data only. Using Conditional Correct Response Rate for your binary data instead.")
    CCRR(U)
  }
  nitems <- NCOL(U$Q)
  ncat <- U$categories
  U$Q[U$Z == 0] <- NA
  CSR <- vector("list", nitems)
  for (j in 1:nitems) {
    CSR[[j]] <- vector("list", nitems)
    for (k in 1:nitems) {
      tbl <- table(U$Q[, j], U$Q[, k], useNA = "no")
      ccsr <- matrix(NA, nrow = ncat[j], ncol = ncat[k])
      for (l in 1:ncat[j]) {
        ccsr[l, ] <- tbl[l, ] / rowSums(tbl)[l]
      }
      rownames(ccsr) <- U$CategoryLabel[[j]]
      colnames(ccsr) <- U$CategoryLabel[[k]]
      CSR[[j]][[k]] <- ccsr
    }
  }
  return(CSR)
}

#' @title Conditional Correct Response Rate
#' @description
#' The conditional correct response rate (CCRR) represents the ratio of the students
#' who passed Item C (consequent item) to those who passed Item A (antecedent item).
#' This function is applicable only to binary response data.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A matrix of conditional correct response rates with exametrika class.
#' Each element (i,j) represents the probability of correctly answering item j
#' given that item i was answered correctly.
#' @examples
#' # example code
#' # Calculate CCRR using sample dataset J5S10
#' CCRR(J5S10)
#' @export

CCRR <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("CCRR")
}

#' @rdname CCRR
#' @export
CCRR.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = CCRR.binary(U, na = NULL, Z = NULL, w = NULL),
      "rated" = CCRR.nominal(U, na = NULL, Z = NULL, w = NULL),
      "ordinal" = CCRR.nominal(U, na = NULL, Z = NULL, w = NULL),
      "nominal" = CCRR.nominal(U, na = NULL, Z = NULL, w = NULL)
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    JCRR(U)
  }
}

#' @rdname CCRR
#' @export
CCRR.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  Z <- U$Z
  OneJ <- rep(1, ncol(U$U))
  Pj <- JCRR(U)
  p <- crr(U)
  P_C <- Pj / (p %*% t(OneJ))
  structure(P_C, class = c("exametrika", "matrix"))
}

#' @rdname CCRR
#' @export
CCRR.nominal <- function(U, na = NULL, Z = NULL, w = NULL) {
  message("CCRR is for binary data only. Using Conditional Selection Rate for your polytomous data instead.")
  CSR(U)
}
#' @title Item Lift
#' @description
#' The lift is a commonly used index in a POS data analysis.
#' The item lift of Item k to Item j is defined as follow:
#' \eqn{ l_{jk} = \frac{p_{k\mid j}}{p_k} }
#' This function is applicable only to binary response data.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A matrix of item lift values with exametrika class.
#' Each element (j,k) represents the lift value of item k given item j,
#' which indicates how much more likely item k is to be correct given that
#' item j was answered correctly.
#' @references Brin, S., Motwani, R., Ullman, J., & Tsur, S. (1997). Dynamic itemset counting and
#' implication rules for market basket data. In Proceedings of ACM SIGMOD International Conference
#' on Management of Data (pp. 255–264). https://dl.acm.org/doi/10.1145/253262.253325
#' @examples
#' # example code
#' # Calculate ItemLift using sample dataset J5S10
#' ItemLift(J5S10)
#' @export
ItemLift <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ItemLift")
}

#' @rdname ItemLift
#' @export
ItemLift.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "ItemLift")
  }
  ItemLift.binary(U, na, Z, w)
}

#' @rdname ItemLift
#' @export
ItemLift.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  OneJ <- rep(1, ncol(U$U))
  Pc <- CCRR(U)
  p <- crr(U)
  P_L <- Pc / (OneJ %*% t(p))
  structure(P_L, class = c("exametrika", "matrix"))
}

#' @title Mutual Information
#' @description
#' Mutual Information is a measure that represents the degree of interdependence
#' between two items. This function is applicable to both binary and polytomous response data.
#' The measure is calculated using the joint probability distribution of responses
#' between item pairs and their marginal probabilities.
#' @details
#' For binary data, the following formula is used:
#' \deqn{
#' MI_{jk} = p_{00} \log_2 \frac{p_{00}}{(1-p_j)(1-p_k)} + p_{01} \log_2 \frac{p_{01}}{(1-p_j)p_k}
#'  + p_{10} \log_2 \frac{p_{10}}{p_j(1-p_k)} + p_{11} \log_2 \frac{p_{11}}{p_jp_k}
#' }
#' Where:
#' \itemize{
#'   \item \eqn{p_{00}} is the joint probability of incorrect responses to both items j and k
#'   \item \eqn{p_{01}} is the joint probability of incorrect response to item j and correct to item k
#'   \item \eqn{p_{10}} is the joint probability of correct response to item j and incorrect to item k
#'   \item \eqn{p_{11}} is the joint probability of correct responses to both items j and k
#' }
#'
#' For polytomous data, the following formula is used:
#' \deqn{MI_{jk} = \sum_{j=1}^{C_j}\sum_{k=1}^{C_k}p_{jk}\log \frac{p_{jk}}{p_{j.}p_{.k}}}
#'
#' The base of the logarithm can be the number of rows, number of columns, min(rows, columns),
#' base-10 logarithm, natural logarithm (e), etc.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @param base The base for the logarithm. Default is 2. For polytomous data,
#'   you can use "V" to set the base to min(rows, columns), "e" for natural logarithm (base e),
#'   or any other number to use that specific base.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A matrix of mutual information values with exametrika class.
#' Each element (i,j) represents the mutual information between items i and j,
#' measured in bits. Higher values indicate stronger interdependence between items.
#' @examples
#' # example code
#' # Calculate Mutual Information using sample dataset J15S500
#' MutualInformation(J15S500)
#' @export
MutualInformation <- function(U, na = NULL, Z = NULL, w = NULL, base = 2) {
  UseMethod("MutualInformation")
}

#' @rdname MutualInformation
#' @export
MutualInformation.default <- function(U, na = NULL, Z = NULL, w = NULL, base = 2) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = MutualInformation.binary(U, na, Z, w, base = base),
      "rated" = MutualInformation.ordinal(U, na, Z, w, base = base),
      "ordinal" = MutualInformation.ordinal(U, na, Z, w, base = base),
      "nominal" = MutualInformation.ordinal(U, na, Z, w, base = base)
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    MutualInformation(U, na, Z, w, base = base)
  }
}

#' @rdname MutualInformation
#' @export
MutualInformation.binary <- function(U, na = NULL, Z = NULL, w = NULL, base = 2) {
  p <- crr(U)
  # Calculate joint response matrix
  S <- list()
  S$S_11 <- t(U$Z * U$U) %*% (U$Z * U$U)
  S$S_10 <- t(U$Z * U$U) %*% (U$Z * (1 - U$U))
  S$S_01 <- t(U$Z * (1 - U$U)) %*% (U$Z * U$U)
  S$S_00 <- t(U$Z * (1 - U$U)) %*% (U$Z * (1 - U$U))

  # Calculate joint probability matrix
  P <- lapply(S, function(x) x / (t(U$Z) %*% U$Z))

  # Calculate lift matrix
  L <- list()
  L$L_11 <- P$S_11 / (p %*% t(p))
  L$L_10 <- P$S_10 / (p %*% t(1 - p))
  L$L_01 <- P$S_01 / ((1 - p) %*% t(p))
  L$L_00 <- P$S_00 / ((1 - p) %*% t(1 - p))

  # Calculate mutual information
  MI <- P$S_00 * log(L$L_00, base = 2) +
    P$S_01 * log(L$L_01, base = 2) +
    P$S_10 * log(L$L_10, base = 2) +
    P$S_11 * log(L$L_11, base = 2)

  # Adjust diagonal elements
  diag(MI) <- diag(P$S_00 * log(L$L_00, base = 2) +
    P$S_11 * log(L$L_11, base = 2))

  structure(MI, class = c("exametrika", "matrix"))
}

#' @rdname MutualInformation
#' @export
MutualInformation.ordinal <- function(U, na = NULL, Z = NULL, w = NULL, base = 2) {
  if (is.character(base)) {
    if (base != "V" & base != "v" & base != "e") {
      stop("The base of the logarithm must be a number, 'V', or 'e'.")
    }
  }
  nitems <- NCOL(U$Z)
  ncat <- apply(U$Q, 2, max)
  mat <- matrix(ncol = nitems, nrow = nitems)
  for (i in 1:nitems) {
    for (j in 1:nitems) {
      x <- U$Q[, i]
      y <- U$Q[, j]
      x[x == -1] <- NA
      y[y == -1] <- NA
      pairwise <- !is.na(x + y)
      x <- x[pairwise]
      y <- y[pairwise]
      tbl <- table(x, y) / sum(length(x))

      if (base == "V" | base == "v") {
        tei <- min(ncat[i], ncat[j])
      } else if (base == "e") {
        tei <- exp(1)
      } else {
        tei <- as.numeric(base)
      }
      mi <- 0
      cSum <- colSums(tbl)
      rSum <- rowSums(tbl)
      for (k in 1:NROW(tbl)) {
        for (m in 1:NCOL(tbl)) {
          mi <- mi + (tbl[k, m] * (log(tbl[k, m] / rSum[k] / cSum[m], base = tei)))
        }
      }
      mat[i, j] <- mi
    }
  }
  return(mat)
}

#' @title Phi-Coefficient
#' @description
#' The phi coefficient is the Pearson's product moment correlation coefficient
#' between two binary items. This function is applicable only to binary response data.
#' The coefficient ranges from -1 to 1, where 1 indicates perfect positive correlation,
#' -1 indicates perfect negative correlation, and 0 indicates no correlation.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A matrix of phi coefficients with exametrika class.
#' Each element (i,j) represents the phi coefficient between items i and j.
#' The matrix is symmetric with ones on the diagonal.
#' @examples
#' # example code
#' # Calculate Phi-Coefficient using sample dataset J15S500
#' PhiCoefficient(J15S500)
#' @export
PhiCoefficient <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("PhiCoefficient")
}

#' @rdname PhiCoefficient
#' @export
PhiCoefficient.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "PhiCoefficient")
  }
  PhiCoefficient.binary(U, na, Z, w)
}

#' @rdname PhiCoefficient
#' @export
PhiCoefficient.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  p <- crr(U)
  OneS <- rep(1, nrow(U$U))
  OneJ <- rep(1, ncol(U$U))

  # Calculate centered cross-product matrix
  C <- t(U$Z * (U$U - OneS %*% t(p))) %*%
    (U$Z * (U$U - OneS %*% t(p))) /
    (t(U$Z) %*% U$Z - OneJ %*% t(OneJ))

  # Calculate standard deviations
  v <- diag(C)

  # Calculate correlation matrix
  phi <- C / sqrt(v) %*% t(sqrt(v))

  structure(phi, class = c("exametrika", "matrix"))
}

#' @title Tetrachoric Correlation
#' @description
#' Tetrachoric Correlation is superior to the phi coefficient as a measure of the
#' relation of an item pair. See Divgi, 1979; Olsson, 1979;Harris, 1988.
#' @references Divgi, D. R. (1979). Calculation of the tetrachoric correlation coefficient.
#' Psychometrika, 44, 169–172.
#' @references Olsson, U. (1979). Maximum likelihood estimation of the polychoric correlation
#'  coefficient. Psychometrika,44, 443–460.
#' @references Harris, B. (1988). Tetrachoric correlation coefficient. In L. Kotz, & N. L. Johnson
#'  (Eds.), Encyclopedia of statistical sciences (Vol. 9, pp. 223–225). Wiley.
#' @param x binary vector x
#' @param y binary vector y
#' @importFrom mvtnorm pmvnorm
#' @importFrom stats qnorm
#' @importFrom stats pnorm
#' @importFrom stats optimize
#' @return Returns a single numeric value of class "exametrika" representing the
#'   tetrachoric correlation coefficient between the two binary variables. The value
#'   ranges from -1 to 1, where:
#'   \itemize{
#'     \item 1 indicates perfect positive correlation
#'     \item -1 indicates perfect negative correlation
#'     \item 0 indicates no correlation
#'   }
#' @export

tetrachoric <- function(x, y) {
  pairwise <- !is.na(x + y)
  # count 2x2 cells
  x.fac <- factor(x[pairwise], levels = 0:1)
  y.fac <- factor(y[pairwise], levels = 0:1)
  tbl <- table(x.fac, y.fac)
  S00 <- tbl[1, 1]
  S10 <- tbl[2, 1]
  S01 <- tbl[1, 2]
  S11 <- tbl[2, 2]
  if (S00 == 0) {
    S00 <- 0.5
  }
  if (S10 == 0) {
    S10 <- 0.5
  }
  if (S01 == 0) {
    S01 <- 0.5
  }
  if (S11 == 0) {
    S11 <- 0.5
  }
  # calcs tau
  tau_j <- qnorm(1 - mean(x, na.rm = TRUE))
  tau_k <- qnorm(1 - mean(y, na.rm = TRUE))
  ## BVN funcs
  BVN11 <- function(rho, tau_j, tau_k) {
    pmvnorm(upper = c(-tau_j, -tau_k), corr = matrix(c(1, rho, rho, 1), ncol = 2))
  }
  BVN01 <- function(rho, tau_j, tau_k) {
    pnorm(tau_j) - pmvnorm(upper = c(tau_j, tau_k), corr = matrix(c(1, rho, rho, 1), ncol = 2))
  }
  BVN10 <- function(rho, tau_j, tau_k) {
    pnorm(tau_k) - pmvnorm(upper = c(tau_j, tau_k), corr = matrix(c(1, rho, rho, 1), ncol = 2))
  }
  BVN00 <- function(rho, tau_j, tau_k) {
    pmvnorm(upper = c(tau_j, tau_k), corr = matrix(c(1, rho, rho, 1), ncol = 2))
  }
  ## LL
  log_likelihood_phi <- function(rho, tau_j, tau_k, S00, S11, S10, S01) {
    S00 * log(BVN00(rho, tau_j, tau_k)) + S01 * log(BVN01(rho, tau_j, tau_k)) +
      S10 * log(BVN10(rho, tau_j, tau_k)) + S11 * log(BVN11(rho, tau_j, tau_k))
  }
  ret <- optim(
    par = 0, # initial value
    fn = function(x) {
      -log_likelihood_phi(rho = x, tau_j, tau_k, S00, S11, S10, S01)
    },
    lower = -1, # lower limit
    upper = 1, # upper limit
    method = "Brent" # one-dimensional optimization method
  )
  ret <- structure(ret$par, class = c("exametrika"))
  return(ret)
}

#' @title Polychoric Correlation
#' @description
#' Calculate the polychoric correlation coefficient between two polytomous (categorical ordinal) variables.
#' Polychoric correlation estimates the correlation between two theorized normally distributed
#' continuous latent variables from two observed ordinal variables.
#'
#' @param x A polytomous vector (categorical ordinal variable)
#' @param y A polytomous vector (categorical ordinal variable)
#' @return The polychoric correlation coefficient between x and y
#' @details
#' This function handles missing values (coded as -1 or NA) using pairwise deletion.
#' The estimation uses maximum likelihood approach with Brent's method for optimization.
#' @examples
#' # Example with simulated data
#' set.seed(123)
#' x <- sample(1:5, 100, replace = TRUE)
#' y <- sample(1:4, 100, replace = TRUE)
#' polychoric(x, y)
#'
#' @export
polychoric <- function(x, y) {
  # Use fast C++ implementation
  result <- polychoric_cpp(as.integer(x), as.integer(y))

  # Check for convergence issues
  if (is.na(result)) {
    stop("Failed to converge when calculating polychoric correlation. Try with different initial values or check your data.")
  }

  return(result)
}


#' @title Polyserial Correlation
#' @description Calculates the polyserial correlation coefficient between a continuous variable and an ordinal variable.
#' @details This function implements Olsson et al.'s ad hoc method for estimating the polyserial correlation
#' coefficient. The method assumes that the continuous variable follows a normal distribution and
#' that the ordinal variable is derived from an underlying continuous normal variable through
#' thresholds.
#' @param x A numeric vector representing the continuous variable.
#' @param y A numeric vector representing the ordinal variable (must be integer values).
#' @return A numeric value representing the estimated polyserial correlation coefficient.
#' @references U.Olsson, F.Drasgow, and N.Dorans (1982).
#' The polyserial correlation coefficient. Psychometrika, 47,337-347.
#' @examples
#' n <- 300
#' x <- rnorm(n)
#' y <- sample(1:5, size = n, replace = TRUE)
#' polyserial(x, y)
#' @importFrom stats dnorm
#' @export
#'
polyserial <- function(x, y) {
  x[x == -1] <- NA
  y[y == -1] <- NA
  pairwise <- !is.na(x + y)
  x <- x[pairwise]
  y <- y[pairwise]
  nobs <- length(y)
  ncat <- max(as.numeric(as.factor(y)))

  tbl <- tabulate(y)
  freq <- tbl / sum(tbl)
  cum_freq <- cumsum(freq)

  thresholds <- qnorm(cum_freq)[-ncat]
  tau <- dnorm(thresholds)
  sum_tau <- sum(tau)

  rxy <- cor(x, y)
  sd_y <- sqrt(var(y) * (nobs - 1) / nobs)
  rho <- rxy * sd_y / sum_tau
  rho <- pmin(pmax(rho, -1), 1)
  return(rho)
}

#' @title Polychoric Correlation Matrix
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @return A matrix of polychoric correlations with exametrika class.
#' Each element (i,j) represents the polychoric correlation between items i and j.
#' The matrix is symmetric with ones on the diagonal.
#' @examples
#' \donttest{
#' # example code
#' PolychoricCorrelationMatrix(J5S1000)
#' }
#' @export
#'
PolychoricCorrelationMatrix <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("PolychoricCorrelationMatrix")
}

#' @rdname PolychoricCorrelationMatrix
#' @export
PolychoricCorrelationMatrix.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  if (tmp$response.type != "ordinal") {
    response_type_error(tmp$response.type, "PolychoricCorrelationMatrix")
  }
  PolychoricCorrelationMatrix.ordinal(tmp, na, Z, w)
}

#' @rdname PolychoricCorrelationMatrix
#' @export
PolychoricCorrelationMatrix.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  tmp$Q[tmp$Z == 0] <- NA

  # Use fast C++ matrix implementation
  ret.mat <- polychoric_matrix_cpp(tmp$Q)
  rownames(ret.mat) <- tmp$ItemLabel
  colnames(ret.mat) <- tmp$ItemLabel

  return(ret.mat)
}

#' @title Tetrachoric Correlation Matrix
#' @description
#' Calculates the matrix of tetrachoric correlations between all pairs of items.
#' Tetrachoric Correlation is superior to the phi coefficient as a measure of the
#' relation of an item pair. This function is applicable only to binary response data.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A matrix of tetrachoric correlations with exametrika class.
#' Each element (i,j) represents the tetrachoric correlation between items i and j.
#' The matrix is symmetric with ones on the diagonal.
#' @examples
#' \donttest{
#' # example code
#' TetrachoricCorrelationMatrix(J15S500)
#' }
#' @export
TetrachoricCorrelationMatrix <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("TetrachoricCorrelationMatrix")
}

#' @rdname TetrachoricCorrelationMatrix
#' @export
TetrachoricCorrelationMatrix.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "TetrachoricCorrelationMatrix")
  }
  TetrachoricCorrelationMatrix.binary(U, na, Z, w)
}

#' @rdname TetrachoricCorrelationMatrix
#' @export
TetrachoricCorrelationMatrix.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  tmp$U[tmp$Z == 0] <- NA
  Una <- tmp$U
  m <- ncol(Una)
  mat <- matrix(NA, ncol = m, nrow = m)
  colnames(mat) <- tmp$ItemLabel
  rownames(mat) <- tmp$ItemLabel
  for (i in 1:(m - 1)) {
    for (j in (i + 1):m) {
      x <- Una[, i]
      y <- Una[, j]
      mat[i, j] <- tetrachoric(x, y)
      mat[j, i] <- mat[i, j]
    }
  }
  diag(mat) <- 1
  structure(mat, class = c("exametrika", "matrix"))
}

#' @title Correct Response Rate
#' @description
#' The correct response rate (CRR) is one of the most basic and important
#' statistics for item analysis. This is an index of item difficulty and
#' a measure of how many students out of those who tried an item correctly
#' responded to it. This function is applicable only to binary response data.
#'
#' The CRR for each item is calculated as:
#' \deqn{p_j = \frac{\sum_{i=1}^n z_{ij}u_{ij}}{\sum_{i=1}^n z_{ij}}}
#' where \eqn{z_{ij}} is the missing indicator and \eqn{u_{ij}} is the response.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of weighted correct response rates for each item.
#' Values range from 0 to 1, where higher values indicate easier items
#' (more students answered correctly).
#' @examples
#' # using sample datasaet
#' crr(J15S500)
#' @export
crr <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("crr")
}

#' @rdname crr
#' @export
crr.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "crr")
  }
  crr.binary(U, na, Z, w)
}

#' @rdname crr
#' @export
crr.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Create unit vector for summation
  OneS <- rep(1, length = nrow(U$U))

  # Calculate correct response rate
  # (sum of correct responses) / (sum of non-missing responses)
  p <- t(U$Z * U$U) %*% OneS / t(U$Z) %*% OneS

  # Apply item weights
  pW <- U$w * p

  return(pW)
}

#' @title Item Odds
#' @description
#' Item Odds are defined as the ratio of Correct Response Rate to
#' Incorrect Response Rate:
#' \deqn{O_j = \frac{p_j}{1-p_j}}
#' where \eqn{p_j} is the correct response rate for item j.
#' This function is applicable only to binary response data.
#'
#' The odds value represents how many times more likely a correct response is
#' compared to an incorrect response. For example, an odds of 2 means students
#' are twice as likely to answer correctly as incorrectly.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of odds values for each item. Values range from 0 to infinity,
#' where:
#' \itemize{
#'   \item odds > 1: correct response more likely than incorrect
#'   \item odds = 1: equally likely
#'   \item odds < 1: incorrect response more likely than correct
#' }
#' @examples
#' # using sample dataset
#' ItemOdds(J5S10)
#' @export
ItemOdds <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ItemOdds")
}

#' @rdname ItemOdds
#' @export
ItemOdds.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "ItemOdds")
  }
  ItemOdds.binary(U, na, Z, w)
}

#' @rdname ItemOdds
#' @export
ItemOdds.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate correct response rates
  p <- crr(U)

  # Calculate odds
  o <- p / (1 - p)

  return(o)
}


#' @title Item Threshold
#' @description
#' Item threshold is a measure of difficulty based on a standard normal distribution.
#' This function is applicable only to binary response data.
#'
#' The threshold is calculated as:
#' \deqn{\tau_j = \Phi^{-1}(1-p_j)}
#' where \eqn{\Phi^{-1}} is the inverse standard normal distribution function
#' and \eqn{p_j} is the correct response rate for item j.
#'
#' Higher threshold values indicate more difficult items, as they represent the
#' point on the standard normal scale above which examinees tend to answer incorrectly.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of threshold values for each item on the standard normal scale.
#' Typical values range from about -3 to 3, where:
#' \itemize{
#'   \item Positive values indicate difficult items
#'   \item Zero indicates items of medium difficulty (50% correct)
#'   \item Negative values indicate easy items
#' }
#' @importFrom stats qnorm
#' @examples
#' # using sample dataset
#' ItemThreshold(J5S10)
#' @export
ItemThreshold <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ItemThreshold")
}
#' @export
ItemThreshold.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = ItemThreshold.binary(U, na, Z, w),
      "rated" = ItemThreshold.ordinal(U, na, Z, w),
      "ordinal" = ItemThreshold.ordinal(U, na, Z, w),
      "nominal" = response_type_error(U$response.type, "ItemThreshold")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    ItemThreshold(U)
  }
}

#' @rdname ItemThreshold
#' @export
ItemThreshold.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate correct response rates
  p <- crr(U)
  # Calculate thresholds using inverse normal distribution
  tau <- qnorm(1 - p)
  return(tau)
}

#' @rdname ItemThreshold
#' @export
ItemThreshold.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  nitems <- ncol(U$Q)
  ncat <- U$categories
  U$Q[U$Z == 0] <- NA
  threshold <- vector("list", nitems)
  for (j in 1:nitems) {
    tmp <- qnorm(cumsum(tabulate(U$Q[, j]) / sum(tabulate(U$Q[, j]))))
    threshold[[j]] <- tmp[1:ncat[j] - 1]
  }
  return(threshold)
}


#' @title Item Entropy
#' @description
#' The item entropy is an indicator of the variability or randomness
#' of the responses. This function is applicable only to binary response data.
#'
#' The entropy value represents the uncertainty or information content of the
#' response pattern for each item, measured in bits. Maximum entropy (1 bit)
#' occurs when correct and incorrect responses are equally likely (p = 0.5).
#' @details
#' The item entropy is calculated as:
#' \deqn{e_j = -p_j\log_2p_j-(1-p_j)\log_2(1-p_j)}
#' where \eqn{p_j} is the correct response rate for item j.
#'
#' The entropy value has the following properties:
#' \itemize{
#'   \item Maximum value of 1 bit when p = 0.5 (most uncertainty)
#'   \item Minimum value of 0 bits when p = 0 or 1 (no uncertainty)
#'   \item Higher values indicate more balanced response patterns
#'   \item Lower values indicate more predictable response patterns
#' }
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of entropy values for each item, measured in bits.
#' Values range from 0 to 1, where:
#' \itemize{
#'   \item 1: maximum uncertainty (p = 0.5)
#'   \item 0: complete certainty (p = 0 or 1)
#'   \item Values near 1 indicate items with balanced response patterns
#'   \item Values near 0 indicate items with extreme response patterns
#' }
#' @examples
#' # using sample dataset
#' ItemEntropy(J5S10)
#' @export
ItemEntropy <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ItemEntropy")
}

#' @rdname ItemEntropy
#' @export
ItemEntropy.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = ItemEntropy.binary(U, na, Z, w),
      "rated" = ItemEntropy.ordinal(U, na, Z, w),
      "ordinal" = ItemEntropy.ordinal(U, na, Z, w),
      "nominal" = response_type_error(U$response.type, "ItemEntropy")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    ItemEntropy(U)
  }
}

#' @rdname ItemEntropy
#' @export
ItemEntropy.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate correct response rates
  p <- crr(U)
  # Calculate entropy in bits
  # Using log base 2 for information content in bits
  itemE <- -p * log(p, base = 2) - (1 - p) * log(1 - p, base = 2)
  return(itemE)
}

#' @rdname ItemEntropy
#' @export
ItemEntropy.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  nitems <- ncol(U$Q)
  ncat <- U$categories
  U$Q[U$Z == 0] <- NA
  entropy <- rep(0, nitems)
  for (j in 1:nitems) {
    vec <- tabulate(U$Q[, j]) / sum(tabulate(U$Q[, j]))
    entropy[j] <- sum(vec * log(vec, base = ncat[j])) * -1
  }
  return(entropy)
}


#' @title Item-Total Correlation
#' @description
#' Item-Total correlation (ITC) is a Pearson's correlation of an item with
#' the Number-Right Score (NRS) or total score. This function is applicable
#' only to binary response data.
#'
#' The ITC is a measure of item discrimination, indicating how well an item
#' distinguishes between high and low performing examinees.
#' @details
#' The correlation is calculated between:
#' \itemize{
#'   \item Each item's responses (0 or 1)
#'   \item The total test score (sum of correct responses)
#' }
#' Higher positive correlations indicate items that better discriminate between
#' high and low ability examinees.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of item-total correlations. Values typically range
#' from -1 to 1, where:
#' \itemize{
#'   \item Values near 1: Strong positive discrimination
#'   \item Values near 0: No discrimination
#'   \item Negative values: Potential item problems (lower ability students
#'         performing better than higher ability students)
#' }
#' @note
#' Values below 0.2 might indicate problematic items that should be reviewed.
#' Values above 0.3 are generally considered acceptable.
#' @examples
#' # using sample dataset
#' ItemTotalCorr(J15S500)
#'
#' @export
ItemTotalCorr <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ItemTotalCorr")
}

#' @rdname ItemTotalCorr
#' @export
ItemTotalCorr.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = ItemTotalCorr.binary(U, na, Z, w),
      "rated" = ItemTotalCorr.binary(U, na, Z, w),
      "ordinal" = ItemTotalCorr.ordinal(U, na, Z, w),
      "nominal" = response_type_error(U$response.type, "ItemTotalCorr")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    ItemTotalCorr(U)
  }
}

#' @rdname ItemTotalCorr
#' @export
ItemTotalCorr.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate item probabilities
  p <- crr(U)
  # Calculate total scores
  Zeta <- sscore(U)
  # Create probability matrix (repeating p for each student)
  TBL <- matrix(rep(p, each = NROW(U$U)),
    nrow = NROW(U$U),
    byrow = FALSE
  )
  # Handle missing values in response matrix
  Una <- ifelse(is.na(U$U), 0, U$U)
  # Calculate deviations from expected values
  dev <- U$Z * (Una - TBL)
  # Calculate item variances
  V <- colSums(dev^2) / (colSums(U$Z) - 1)
  SD <- sqrt(V)
  # Calculate correlations
  rho_Zi <- t(dev) %*% Zeta / SD / colSums(U$Z)
  return(rho_Zi)
}

#' @rdname ItemTotalCorr
#' @export
ItemTotalCorr.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  total <- rowSums(U$Q)
  nitems <- NCOL(U$Z)
  rho_Zi <- numeric(nitems)
  for (j in 1:nitems) {
    rho_Zi[j] <- polyserial(total, U$Q[, j])
  }
  return(rho_Zi)
}


#' @title Biserial Correlation
#' @description
#' A biserial correlation is a correlation between dichotomous-ordinal and
#' continuous variables.
#' @param i i is a dichotomous-ordinal variable (0/1). x and y can also be the other way around.
#' @param t t is a continuous variable. x and y can also be the other way around.
#' @return The biserial correlation coefficient between the two variables.
#' @importFrom stats na.omit qnorm pnorm optim
#' @export
BiserialCorrelation <- function(i, t) {
  # Original function remains unchanged
  # Count unique values
  unique_i <- length(unique(na.omit(i)))
  unique_t <- length(unique(na.omit(t)))
  # Check if one is binary and the other is continuous
  if (!((unique_i == 2 && unique_t > 2) | (unique_t == 2 && unique_i > 2))) {
    stop("One argument must be binary and the other must be continuous.")
  }
  ## if switched...
  if (unique_i > 2) {
    tmp <- i
    i <- t
    t <- tmp
  }
  ## calcs correlation
  tau_j <- qnorm(1 - mean(i, na.rm = TRUE))
  ll <- function(rho, tau_j, i, t) {
    tmp <- (1 - i) %*% (log(pnorm(tau_j, mean = rho * t, sd = sqrt(1 - rho^2)))) +
      i %*% (log(1 - pnorm(tau_j, mean = rho * t, sd = sqrt(1 - rho^2))))
  }
  pairwise <- !is.na(i + t)
  ret <- optim(
    par = 0, # initial value
    fn = function(x) {
      -ll(rho = x, tau_j, i[pairwise], t[pairwise])
    },
    lower = -1, # lower limit
    upper = 1, # upper limit
    method = "Brent" # one-dimensional optimization method
  )
  return(ret$par)
}

#' @title Item-Total Biserial Correlation
#' @description
#' The Item-Total Biserial Correlation computes the biserial correlation
#' between each item and the total score. This function is applicable only
#' to binary response data.
#'
#' This correlation provides a measure of item discrimination, indicating how well
#' each item distinguishes between high and low performing examinees.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of item-total biserial correlations. Values range
#' from -1 to 1, where:
#' \itemize{
#'   \item Values near 1: Strong positive discrimination
#'   \item Values near 0: No discrimination
#'   \item Negative values: Potential item problems
#' }
#' @note
#' The biserial correlation is generally preferred over the point-biserial
#' correlation when the dichotomization is artificial (i.e., when the underlying
#' trait is continuous).
#' @examples
#' # using sample dataset
#' ITBiserial(J15S500)
#' @export
ITBiserial <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ITBiserial")
}

#' @rdname ITBiserial
#' @export
ITBiserial.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "ITBiserial")
  }
  ITBiserial.binary(U, na, Z, w)
}

#' @rdname ITBiserial
#' @export
ITBiserial.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate total scores
  Zeta <- sscore(U)

  # Handle missing values
  U_data <- U$U
  U_data[U$Z == 0] <- NA

  # Calculate biserial correlation for each item
  ITB <- vapply(seq_len(ncol(U_data)), function(i) {
    BiserialCorrelation(U_data[, i], Zeta)
  }, numeric(1))

  return(ITB)
}


#' @title Number Right Score
#' @description
#' The Number-Right Score (NRS) function calculates the weighted sum of correct
#' responses for each examinee. This function is applicable only to binary
#' response data.
#'
#' For each examinee, the score is computed as:
#' \deqn{NRS_i = \sum_{j=1}^J z_{ij}u_{ij}w_j}
#' where:
#' \itemize{
#'   \item \eqn{z_{ij}} is the missing response indicator (0/1)
#'   \item \eqn{u_{ij}} is the response (0/1)
#'   \item \eqn{w_j} is the item weight
#' }
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector containing the Number-Right Score for each examinee.
#' The score represents the weighted sum of correct answers, where:
#' \itemize{
#'   \item Maximum score is the sum of all item weights
#'   \item Minimum score is 0
#'   \item Missing responses do not contribute to the score
#' }
#' @examples
#' # using sample dataset
#' nrs(J15S500)
#' @export
nrs <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("nrs")
}

#' @rdname nrs
#' @export
nrs.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "nrs")
  }
  nrs.binary(U, na, Z, w)
}

#' @rdname nrs
#' @export
nrs.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate weighted sum of correct responses
  # Z * U gives correct answers accounting for missing data
  # Multiply by weights and sum across items
  total_weighted_score <- (U$Z * U$U) %*% U$w
  return(total_weighted_score)
}

#' @title Passage Rate of Student
#' @description
#' The Passage Rate for each student is calculated as their Number-Right Score (NRS)
#' divided by the number of items presented to them. This function is applicable
#' only to binary response data.
#'
#' The passage rate is calculated as:
#' \deqn{P_i = \frac{\sum_{j=1}^J z_{ij}u_{ij}w_j}{\sum_{j=1}^J z_{ij}}}
#' where:
#' \itemize{
#'   \item \eqn{z_{ij}} is the missing response indicator (0/1)
#'   \item \eqn{u_{ij}} is the response (0/1)
#'   \item \eqn{w_j} is the item weight
#' }
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector containing the passage rate for each student.
#' Values range from 0 to 1 (or maximum weight) where:
#' \itemize{
#'   \item 1: Perfect score on all attempted items
#'   \item 0: No correct answers
#'   \item NA: No items attempted
#' }
#' @note
#' The passage rate accounts for missing responses by only considering items that
#' were actually presented to each student. This provides a fair comparison
#' between students who attempted different numbers of items.
#' @examples
#' # using sample dataset
#' passage(J15S500)
#'
#' @export
passage <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("passage")
}

#' @rdname passage
#' @export
passage.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "passage")
  }
  passage.binary(U, na, Z, w)
}

#' @rdname passage
#' @export
passage.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate Number-Right Score
  total_score <- nrs(U)

  # Calculate number of items presented to each student
  items_attempted <- NCOL(U$U) - rowSums(1 - U$Z)

  # Calculate passage rate
  # Divide total score by number of items attempted
  passage_rate <- total_score / items_attempted

  return(passage_rate)
}

#' @title Standardized Score
#' @description
#' The standardized score (z-score) indicates how far a student's performance
#' deviates from the mean in units of standard deviation. This function is
#' applicable only to binary response data.
#'
#' The score is calculated by standardizing the passage rates:
#' \deqn{Z_i = \frac{r_i - \bar{r}}{\sigma_r}}
#' where:
#' \itemize{
#'   \item \eqn{r_i} is student i's passage rate
#'   \item \eqn{\bar{r}} is the mean passage rate
#'   \item \eqn{\sigma_r} is the standard deviation of passage rates
#' }
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of standardized scores for each student. The scores follow
#' a standard normal distribution with:
#' \itemize{
#'   \item Mean = 0
#'   \item Standard deviation = 1
#'   \item Approximately 68% of scores between -1 and 1
#'   \item Approximately 95% of scores between -2 and 2
#'   \item Approximately 99% of scores between -3 and 3
#' }
#' @note
#' The standardization allows for comparing student performance across different
#' tests or groups. A positive score indicates above-average performance, while
#' a negative score indicates below-average performance.
#' @examples
#' # using sample dataset
#' sscore(J5S10)
#' @export
sscore <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("sscore")
}

#' @rdname sscore
#' @export
sscore.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "sscore")
  }
  sscore.binary(U, na, Z, w)
}

#' @rdname sscore
#' @export
sscore.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Get number of students
  S <- nrow(U$U)

  # Create unit vector
  OneS <- rep(1, length = S)

  # Calculate passage rates
  passage_rates <- passage(U)

  # Calculate mean passage rate
  mean_rate <- mean(passage_rates, na.rm = TRUE)

  # Calculate variance of passage rates
  centered_rates <- passage_rates - mean_rate
  var_rates <- sum(centered_rates^2, na.rm = TRUE) / (S - 1)

  # Calculate standardized scores
  z_scores <- centered_rates / sqrt(var_rates)

  return(z_scores)
}


#' @title Student Percentile Ranks
#' @description
#' The percentile function calculates each student's relative standing in the group,
#' expressed as a percentile rank (1-100). This function is applicable only to
#' binary response data.
#'
#' The percentile rank indicates the percentage of scores in the distribution
#' that fall below a given score. For example, a percentile rank of 75 means
#' the student performed better than 75% of the group.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A numeric vector of percentile ranks (1-100) for each student, where:
#' \itemize{
#'   \item 100: Highest performing student(s)
#'   \item 50: Median performance
#'   \item 1: Lowest performing student(s)
#' }
#' @note
#' Percentile ranks are calculated using the empirical cumulative distribution
#' function of standardized scores. Tied scores receive the same percentile rank.
#' The values are rounded up to the nearest integer to provide ranks from 1 to 100.
#' @examples
#' # using sample dataset
#' percentile(J5S10)
#'
#' @importFrom stats ecdf
#' @export
percentile <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("percentile")
}

#' @rdname percentile
#' @export
percentile.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "percentile")
  }
  percentile.binary(U, na, Z, w)
}

#' @rdname percentile
#' @export
percentile.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Calculate standardized scores
  standardized_scores <- sscore(U)

  # Calculate empirical cumulative distribution function
  empirical_dist <- ecdf(standardized_scores)

  # Convert to percentiles (1-100)
  # Ceiling function ensures minimum of 1 and handles rounding
  percentile_ranks <- ceiling(empirical_dist(standardized_scores) * 100)

  return(percentile_ranks)
}

#' @title Stanine Scores
#' @description
#' The Stanine (Standard Nine) scoring system divides students into nine groups
#' based on a normalized distribution. This function is applicable only to
#' binary response data.
#'
#' These groups correspond to the following percentile ranges:
#' \itemize{
#'   \item Stanine 1: lowest 4% (percentiles 1-4)
#'   \item Stanine 2: next 7% (percentiles 5-11)
#'   \item Stanine 3: next 12% (percentiles 12-23)
#'   \item Stanine 4: next 17% (percentiles 24-40)
#'   \item Stanine 5: middle 20% (percentiles 41-60)
#'   \item Stanine 6: next 17% (percentiles 61-77)
#'   \item Stanine 7: next 12% (percentiles 78-89)
#'   \item Stanine 8: next 7% (percentiles 90-96)
#'   \item Stanine 9: highest 4% (percentiles 97-100)
#' }
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @note This function is implemented using a binary data compatibility wrapper and
#'       will raise an error if used with polytomous data.
#' @return A list containing two elements:
#' \describe{
#'   \item{stanine}{The score boundaries for each stanine level}
#'   \item{stanineScore}{The stanine score (1-9) for each student}
#' }
#' @note
#' Stanine scores provide a normalized scale with:
#' \itemize{
#'   \item Mean = 5
#'   \item Standard deviation = 2
#'   \item Scores range from 1 to 9
#'   \item Score of 5 represents average performance
#' }
#' @references
#' Angoff, W. H. (1984). Scales, norms, and equivalent scores. Educational Testing Service.
#' (Reprint of chapter in R. L. Thorndike (Ed.) (1971) Educational Measurement (2nd Ed.).
#' American Council on Education.
#' @importFrom stats quantile
#' @examples
#' result <- stanine(J15S500)
#' # View score boundaries
#' result$stanine
#' # View individual scores
#' result$stanineScore
#'
#' @export
stanine <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("stanine")
}

#' @rdname stanine
#' @export
stanine.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (!inherits(U, "exametrika")) {
    U <- dataFormat(U, na = na, Z = Z, w = w)
  }
  if (U$response.type != "binary") {
    response_type_error(U$response.type, "stanine")
  }
  stanine.binary(U, na, Z, w)
}

#' @rdname stanine
#' @export
stanine.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  # Define stanine boundaries (cumulative proportions)
  stanine_bounds <- cumsum(c(0.04, 0.07, 0.12, 0.17, 0.20, 0.17, 0.12, 0.07))

  # Calculate raw scores
  raw_scores <- nrs(U)

  # Calculate score boundaries using raw scores
  stanine_boundaries <- quantile(raw_scores,
    probs = stanine_bounds,
    na.rm = TRUE
  )

  # Calculate percentile scores
  percentile_scores <- percentile(U)

  # Calculate stanine boundaries using percentile scores
  stanine_percentile_bounds <- quantile(percentile_scores,
    probs = stanine_bounds
  )
  stanine_percentile_bounds <- unique(stanine_percentile_bounds)
  if (length(stanine_percentile_bounds) != 8) {
    warning(
      "Standard stanine (9-level) division not possible due to data distribution.\n",
      "Merging duplicate boundaries for adjustment."
    )
  }
  # Assign stanine scores
  stanine_scores <- cut(percentile_scores,
    breaks = c(-Inf, stanine_percentile_bounds, Inf),
    right = FALSE,
    labels = 1:(length(stanine_percentile_bounds) + 1)
  )

  # Return results
  list(
    stanine = stanine_boundaries,
    stanineScore = stanine_scores
  )
}

#' @title Dimensionality
#' @description
#' The dimensionality is the number of components
#' the test is measuring.
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#' @return Returns a list of class c("exametrika", "Dimensionality") containing:
#'   \describe{
#'     \item{Component}{Sequence of component numbers}
#'     \item{Eigenvalue}{Eigenvalues of the tetrachoric correlation matrix}
#'     \item{PerOfVar}{Percentage of variance explained by each component}
#'     \item{CumOfPer}{Cumulative percentage of variance explained}
#'   }
#'
#' @export
Dimensionality <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("Dimensionality")
}

#' @rdname Dimensionality
#' @export
Dimensionality.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = Dimensionality.binary(U, na = NULL, Z = NULL, w = NULL),
      "rated" = Dimensionality.rated(U, na = NULL, Z = NULL, w = NULL),
      "ordinal" = Dimensionality.ordinal(U, na = NULL, Z = NULL, w = NULL),
      "nominal" = response_type_error(U$response.type, "Dimensionality")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    Dimensionality(U)
  }
}

#' @rdname Dimensionality
#' @export
Dimensionality.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  R <- TetrachoricCorrelationMatrix(tmp)
  Esystem <- eigen(R)
  Eval <- Esystem$values
  EvalVariance <- Esystem$values / length(Eval) * 100
  CumVari <- cumsum(EvalVariance)
  ret <-
    structure(
      list(
        Component = seq(1:length(Eval)),
        Eigenvalue = Eval,
        PerOfVar = EvalVariance,
        CumOfPer = CumVari
      ),
      class = c("exametrika", "Dimensionality")
    )

  return(ret)
}

#' @rdname Dimensionality
#' @export
Dimensionality.rated <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  R <- TetrachoricCorrelationMatrix(tmp$U)
  Esystem <- eigen(R)
  Eval <- Esystem$values
  EvalVariance <- Esystem$values / length(Eval) * 100
  CumVari <- cumsum(EvalVariance)
  ret <-
    structure(
      list(
        Component = seq(1:length(Eval)),
        Eigenvalue = Eval,
        PerOfVar = EvalVariance,
        CumOfPer = CumVari
      ),
      class = c("exametrika", "Dimensionality")
    )

  return(ret)
}

#' @rdname Dimensionality
#' @export
Dimensionality.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  R <- PolychoricCorrelationMatrix(tmp)
  Esystem <- eigen(R)
  Eval <- Esystem$values
  EvalVariance <- Esystem$values / length(Eval) * 100
  CumVari <- cumsum(EvalVariance)
  ret <-
    structure(
      list(
        Component = seq(1:length(Eval)),
        Eigenvalue = Eval,
        PerOfVar = EvalVariance,
        CumOfPer = CumVari
      ),
      class = c("exametrika", "Dimensionality")
    )

  return(ret)
}

### Read Output of Mathematica
library(exametrika)

Ch03CTT <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter03CTT_Student.csv"),
  check.names = FALSE
)
Ch03CTT <- Ch03CTT[, sapply(Ch03CTT, is.numeric), drop = FALSE]

## read raw data (J20S400 package data is pre-formatted as nominal)
dat <- read.csv(
  test_path("fixtures", "sample_data", "J20S400.csv"),
  check.names = FALSE
)

test_that("NRS", {
  result <- nrs(dat, na = -99) |> as.vector()
  expect <- Ch03CTT$`Number-Right Score` |> as.vector()
  expect_equal(result, expect)
})

test_that("passage", {
  result <- passage(dat, na = -99) |> as.vector()
  expect <- Ch03CTT$`Passage Rate` |> as.vector()
  expect_equal(result, expect)
})

test_that("sscore", {
  result <- sscore(dat, na = -99) |> as.vector()
  expect <- Ch03CTT$`Standardized Score` |> as.vector()
  expect_equal(result, expect)
})

test_that("percentile", {
  result <- percentile(dat, na = -99) |>
    as.vector() |>
    ceiling()
  expect <- Ch03CTT$`Percentile Rank` |> as.vector()
  expect_equal(result, expect)
})

test_that("stanine", {
  result <- stanine(dat, na = -99)
  test <- result$stanineScore |> as.numeric()
  expect <- Ch03CTT$Stanine |> as.vector()
  expect_equal(test, expect)
})

### Read Output of Mathematica
library(exametrika)

# Mathematica 参照データをシートごとに読み込み
sheets <- c(
  "Item", "JSS", "JCRR",
  "CCRR", "Item_Lift", "Mutual_Information",
  "Phi_Coefficient", "Tetrachoric_Correlation"
)

data_list <- lapply(sheets, function(sheet) {
  df <- read.csv(
    test_path(
      "fixtures", "mathematica_reference",
      paste0("Chapter03CTT_", sheet, ".csv")
    ),
    check.names = FALSE
  )
  # select numeric columns only
  df <- df[, sapply(df, is.numeric), drop = FALSE]
  as.matrix(df)
})

names(data_list) <- c(
  "Ch03Items", "Ch03JSS", "Ch03JCRR", "Ch03CCRR",
  "Ch03IL", "Ch03MI", "Ch03Phi", "Ch03Tet"
)

## read raw data (J20S400 package data is pre-formatted as nominal)
dat <- read.csv(
  test_path("fixtures", "sample_data", "J20S400.csv"),
  check.names = FALSE
)

# Test Section ------------------------------------------------------------

test_that("Correct Response Rate", {
  result <- crr(dat, na = -99) |> as.vector()
  expect <- data_list$Ch03Items[, 2] |> as.vector()
  expect_equal(result, expect)
})


test_that("Item Odds", {
  result <- ItemOdds(dat, na = -99) |> as.vector()
  expect <- data_list$Ch03Items[, 3] |> as.vector()
  expect_equal(result, expect)
})

test_that("Item Threshold", {
  result <- ItemThreshold(dat, na = -99) |> as.vector()
  expect <- data_list$Ch03Items[, 4] |> as.vector()
  expect_equal(result, expect)
})

test_that("Item Entropy", {
  result <- ItemEntropy(dat, na = -99) |> as.vector()
  expect <- data_list$Ch03Items[, 5] |> as.vector()
  expect_equal(result, expect)
})

# Between Items Section ---------------------------------------------------

test_that("Joint sample size", {
  result <- JointSampleSize(dat, na = -99) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03JSS |> unname()
  expect_equal(result, expect)
})

test_that("Joint Correct Response Rate", {
  tmp <- dataFormat(dat, na = -99)
  result <- JCRR(tmp) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03JCRR |> unname()
  expect_equal(result, expect)
})


test_that("Conditional Correct Response Rate", {
  tmp <- dataFormat(dat, na = -99)
  result <- CCRR(tmp) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03CCRR |> unname()
  expect_equal(result, expect)
})


test_that("Item Lift", {
  result <- ItemLift(dat, na = -99) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03IL |> unname()
  expect_equal(result, expect)
})

test_that("Mutual Information", {
  result <- MutualInformation(dat, na = -99) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03MI |> unname()
  expect_equal(result, expect)
})

test_that("Phi Coefficient", {
  result <- PhiCoefficient(dat, na = -99) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03Phi |> unname()
  expect_equal(result, expect)
})

test_that("Tetrachoric Correlation Matrix", {
  result <- TetrachoricCorrelationMatrix(dat, na = -99) |>
    unclass() |>
    unname()
  expect <- data_list$Ch03Tet |> unname()
  expect_equal(expected = expect, object = result, tolerance = 1e-4)
})

test_that("Item Total Correlation", {
  result <- ItemTotalCorr(dat, na = -99)
  expect <- data_list$Ch03Items[, 6]
  expect_equal(expected = expect, object = as.vector(result))
})


test_that("Item Total Biserial Correlation", {
  tmp <- dataFormat(dat, na = -99)
  result <- ITBiserial(tmp) |> as.vector()
  expect <- data_list$Ch03Items[, 7] |> as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})

# Tests for Classical Test Theory Functions

test_that("CTT functions work with binary data", {
  data("J15S500", package = "exametrika")

  # Test correct response rate
  crr_result <- crr(J15S500)
  expect_type(crr_result, "double")
  expect_length(crr_result, 15)
  expect_true(all(crr_result >= 0 & crr_result <= 1))
  expect_true(all(is.finite(crr_result)))

  # Test item-total correlation
  itc_result <- ItemTotalCorr(J15S500)
  expect_type(itc_result, "double")
  expect_length(itc_result, 15)
  expect_true(all(abs(itc_result) <= 1))
  expect_true(all(is.finite(itc_result)))

  # Test number-right score
  nrs_result <- nrs(J15S500)
  expect_type(nrs_result, "double")
  expect_length(nrs_result, 500)
  expect_true(all(nrs_result >= 0))
  expect_true(all(is.finite(nrs_result)))

  # Test standardized scores
  sscore_result <- sscore(J15S500)
  expect_type(sscore_result, "double")
  expect_length(sscore_result, 500)
  expect_true(abs(mean(sscore_result, na.rm = TRUE)) < 0.1) # Should be near 0
  expect_true(abs(sd(sscore_result, na.rm = TRUE) - 1) < 0.1) # Should be near 1

  # Test passage rate
  passage_result <- passage(J15S500)
  expect_type(passage_result, "double")
  expect_length(passage_result, 500)
  expect_true(all(passage_result >= 0 & passage_result <= 1))

  # Test percentiles
  percentile_result <- percentile(J15S500)
  expect_length(percentile_result, 500)
  expect_true(all(percentile_result >= 1 & percentile_result <= 100))

  # Test stanines
  stanine_result <- stanine(J15S500)
  expect_type(stanine_result, "list")
  expect_named(stanine_result, c("stanine", "stanineScore"))
  expect_length(stanine_result$stanineScore, 500)
})

test_that("Item statistics work correctly", {
  data("J15S500", package = "exametrika")

  # Test item odds
  odds_result <- ItemOdds(J15S500)
  expect_type(odds_result, "double")
  expect_length(odds_result, 15)
  expect_true(all(odds_result > 0)) # Odds should be positive

  # Test item thresholds
  threshold_result <- ItemThreshold(J15S500)
  expect_type(threshold_result, "double")
  expect_length(threshold_result, 15)
  expect_true(all(is.finite(threshold_result)))

  # Test item entropy
  entropy_result <- ItemEntropy(J15S500)
  expect_type(entropy_result, "double")
  expect_length(entropy_result, 15)
  expect_true(all(entropy_result >= 0 & entropy_result <= 1))
})

test_that("Inter-item functions work correctly", {
  # Use smaller dataset for speed
  small_data <- J15S500$U[1:100, 1:5]
  formatted_data <- dataFormat(small_data)

  # Test joint sample size
  jss_result <- JointSampleSize(formatted_data)
  expect_s3_class(jss_result, c("exametrika", "matrix"))
  expect_equal(dim(jss_result), c(5, 5))
  expect_true(all(jss_result >= 0))

  # Test joint correct response rate
  jcrr_result <- JCRR(formatted_data)
  expect_s3_class(jcrr_result, c("exametrika", "matrix"))
  expect_equal(dim(jcrr_result), c(5, 5))
  expect_true(all(jcrr_result >= 0 & jcrr_result <= 1))

  # Test conditional correct response rate
  ccrr_result <- CCRR(formatted_data)
  expect_s3_class(ccrr_result, c("exametrika", "matrix"))
  expect_equal(dim(ccrr_result), c(5, 5))

  # Test phi coefficient
  phi_result <- PhiCoefficient(formatted_data)
  expect_s3_class(phi_result, c("exametrika", "matrix"))
  expect_equal(dim(phi_result), c(5, 5))
})

test_that("CTT error handling works", {
  # Test with wrong data type (25 rows)
  set.seed(300)
  ordinal_data <- matrix(sample(1:4, 25 * 4, replace = TRUE), nrow = 25, ncol = 4)
  formatted_ordinal <- dataFormat(ordinal_data)

  # These should fail with ordinal data
  expect_error(crr(formatted_ordinal))
  expect_error(nrs(formatted_ordinal))
  expect_error(passage(formatted_ordinal))
})

test_that("CTT performance is acceptable", {
  data("J15S500", package = "exametrika")

  # Time some key computations
  start_time <- Sys.time()
  crr_result <- crr(J15S500)
  itc_result <- ItemTotalCorr(J15S500)
  sscore_result <- sscore(J15S500)
  elapsed <- as.numeric(Sys.time() - start_time, units = "secs")

  expect_true(elapsed < 1) # Should be very fast
  expect_length(crr_result, 15)
  expect_length(itc_result, 15)
  expect_length(sscore_result, 500)
})

#' @title Chatterjee's xi correlation coefficient
#' @description
#' Computes Chatterjee's (2021) rank-based correlation coefficient xi.
#' Unlike Pearson or Spearman, xi is asymmetric: xi(x, y) and xi(y, x)
#' may differ. This asymmetry is the basis for direction-detection in
#' graphical models.
#' @details
#' For tied x values, ranks are broken using \code{ties_method}. With
#' \code{"random"} (the default), each call may produce a slightly
#' different value due to random tie-breaking. Use \code{xi_stable()}
#' to average over many randomizations.
#' @param x Numeric or ordered factor (predictor)
#' @param y Numeric or ordered factor (response)
#' @param ties_method How to break ties in x. Default "random".
#' @return Numeric scalar: Chatterjee's xi value.
#' @references
#' Chatterjee, S. (2021). A new coefficient of correlation. Journal of
#' the American Statistical Association, 116(536), 2009-2022.
#' @examples
#' \donttest{
#' x <- rnorm(100)
#' y <- x^2 + rnorm(100, sd = 0.1)
#' chatterjee_xi(x, y) # near 1, since y is determined by x
#' chatterjee_xi(y, x) # smaller, since x is not determined by y
#' }
#' @export
chatterjee_xi <- function(x, y, ties_method = "random") {
  n <- as.numeric(length(x))
  ord <- order(rank(x, ties.method = ties_method))
  sorted.y <- y[ord]
  r_i <- rank(sorted.y, ties.method = "max")
  l_i <- rank(-sorted.y, ties.method = "max")
  1 - n * sum(abs(diff(r_i))) / (2 * sum(l_i * (n - l_i)))
}


#' @title Bootstrap-averaged Chatterjee's xi
#' @description
#' Computes Chatterjee's xi B times with random tie-breaking and returns
#' the average. This stabilizes the estimate against tie-breaking
#' variability, which is important for ordinal data with many ties (e.g.,
#' Likert-scale items).
#' @details
#' For each of the B replications, ties in x are broken randomly via
#' \code{rank(x, ties.method = "random")} inside \code{chatterjee_xi()}.
#' The standard error of the mean shrinks as 1 / sqrt(B); B = 1000 typically
#' yields SE around 0.001 for moderately tied data.
#' @param x Numeric or ordered factor (predictor)
#' @param y Numeric or ordered factor (response)
#' @param B Number of bootstrap replications. Default 1000.
#' @param seed Optional integer seed for reproducibility.
#' @return A list with elements:
#' \describe{
#'   \item{xi}{Mean of the B Chatterjee xi values.}
#'   \item{sd}{Standard deviation of the B values (tie-breaking spread).}
#'   \item{se}{Standard error of the mean (sd / sqrt(B)).}
#'   \item{B}{Number of bootstrap replications used.}
#' }
#' @references
#' Chatterjee, S. (2021). A new coefficient of correlation. Journal of
#' the American Statistical Association, 116(536), 2009-2022.
#' @examples
#' \donttest{
#' x <- rnorm(100)
#' y <- x^2 + rnorm(100, sd = 0.1)
#' xi_stable(x, y, B = 500, seed = 42)
#' }
#' @export
xi_stable <- function(x, y, B = 1000, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  xi_vec <- replicate(B, chatterjee_xi(x, y))
  list(
    xi = mean(xi_vec),
    sd = stats::sd(xi_vec),
    se = stats::sd(xi_vec) / sqrt(B),
    B = B
  )
}


#' @title Pairwise Chatterjee's xi correlation matrix
#' @description
#' Computes the p x p asymmetric matrix of Chatterjee's xi correlations
#' between all item pairs. Entry `[j, k]` is xi(item_j, item_k), which in
#' general differs from xi(item_k, item_j); this asymmetry is what
#' enables direction detection in subsequent graph-construction steps.
#' Each off-diagonal entry is computed by \code{xi_stable()} to average
#' over tie-breaking.
#' @details
#' Pairs are computed with pairwise-complete observations: for each
#' pair (j, k), rows where either `Q[i, j]` or `Q[i, k]` is missing are
#' excluded. The diagonal is set to 1.
#' @param U Either an exametrika object or raw data.
#' @param na Values to be treated as missing.
#' @param Z Missing indicator matrix.
#' @param w Item weight vector.
#' @param B Number of bootstrap replications per pair. Default 1000.
#' @param seed Optional integer seed for reproducibility.
#' @param verbose Logical. If TRUE, progress messages are displayed.
#' @return A p x p numeric matrix with item labels as row/column names.
#'   The diagonal is 1; off-diagonal entries are asymmetric.
#' @references
#' Chatterjee, S. (2021). A new coefficient of correlation. Journal of
#' the American Statistical Association, 116(536), 2009-2022.
#' @examples
#' \donttest{
#' xi_mat <- chatterjee_matrix(J15S3810, B = 500, seed = 42)
#' }
#' @export
chatterjee_matrix <- function(U,
                              na = NULL, Z = NULL, w = NULL,
                              B = 1000, seed = NULL,
                              verbose = FALSE) {
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }
  if (tmp$response.type != "ordinal") {
    response_type_error(tmp$response.type, "chatterjee_matrix")
  }

  if (!is.null(seed)) set.seed(seed)
  Q <- tmp$Q
  Q[tmp$Z == 0] <- NA
  p <- ncol(Q)

  if (verbose) {
    message(sprintf("Computing %d x %d Chatterjee xi matrix (B = %d)...", p, p, B))
  }

  M <- matrix(1, nrow = p, ncol = p)
  for (j in seq_len(p)) {
    for (k in seq_len(p)) {
      if (j == k) next
      valid <- !is.na(Q[, j]) & !is.na(Q[, k])
      M[j, k] <- xi_stable(Q[valid, j], Q[valid, k], B = B)$xi
    }
  }

  rownames(M) <- tmp$ItemLabel
  colnames(M) <- tmp$ItemLabel
  M
}

# Tests for Chatterjee's xi correlation

### Setup
set.seed(1234)
test_data <- J15S3810$Q[sample(nrow(J15S3810$Q), 200), ]
tmp <- dataFormat(test_data)


test_that("chatterjee_xi returns a numeric scalar", {
  x <- rnorm(100)
  y <- rnorm(100)
  xi <- chatterjee_xi(x, y)
  expect_length(xi, 1)
  expect_true(is.numeric(xi))
})


test_that("chatterjee_xi detects functional dependence", {
  set.seed(42)
  x <- rnorm(500)
  y <- x^2 + rnorm(500, sd = 0.05)
  # y is nearly determined by x, so xi(x, y) should be high
  expect_gt(chatterjee_xi(x, y), 0.7)
  # x is not determined by y (sign ambiguity), so xi(y, x) should be smaller
  expect_lt(chatterjee_xi(y, x), 0.5)
})


test_that("chatterjee_xi handles large n without integer overflow", {
  set.seed(42)
  n <- 5000
  x <- sample(1:4, n, replace = TRUE)
  y <- sample(1:4, n, replace = TRUE)
  xi <- chatterjee_xi(x, y)
  expect_false(is.na(xi))
  expect_true(is.numeric(xi))
})


test_that("xi_stable returns the expected list structure", {
  x <- rnorm(100)
  y <- rnorm(100)
  res <- xi_stable(x, y, B = 50, seed = 42)
  expect_named(res, c("xi", "sd", "se", "B"))
  expect_equal(res$B, 50)
  expect_equal(res$se, res$sd / sqrt(50))
})


test_that("xi_stable is reproducible with seed", {
  x <- rnorm(100)
  y <- rnorm(100)
  res1 <- xi_stable(x, y, B = 50, seed = 42)
  res2 <- xi_stable(x, y, B = 50, seed = 42)
  expect_equal(res1$xi, res2$xi)
})


test_that("chatterjee_matrix returns a labeled p x p matrix", {
  M <- chatterjee_matrix(tmp, B = 30, seed = 42)
  expect_true(is.matrix(M))
  expect_equal(dim(M), c(15, 15))
  expect_equal(rownames(M), tmp$ItemLabel)
  expect_equal(colnames(M), tmp$ItemLabel)
})


test_that("chatterjee_matrix has unit diagonal", {
  M <- chatterjee_matrix(tmp, B = 30, seed = 42)
  expect_equal(unname(diag(M)), rep(1, 15))
})


test_that("chatterjee_matrix is generally asymmetric", {
  M <- chatterjee_matrix(tmp, B = 30, seed = 42)
  # off-diagonal should not be perfectly symmetric
  off_diff <- max(abs(M - t(M)))
  expect_gt(off_diff, 0.01)
})


test_that("chatterjee_matrix rejects non-ordinal data", {
  expect_error(chatterjee_matrix(J5S10))
})

#' @title Two-Parameter Logistic Model
#' @description
#' The two-parameter logistic model is a classic model that defines
#' the probability of a student with ability theta successfully
#' answering item j, using both a slope parameter and a
#' location parameter.
#' @param a slope parameter
#' @param b location parameter
#' @param theta ability parameter
#' @return Returns a numeric vector of probabilities between 0 and 1, representing
#'   the probability of a correct response given the ability level theta. The probability
#'   is calculated using the formula: \eqn{P(\theta) = \frac{1}{1 + e^{-a(\theta-b)}}}
#' @export

TwoPLM <- function(a, b, theta) {
  p <- 1 / (1 + exp(-a * (theta - b)))
  return(p)
}

#' @title Three-Parameter Logistic Model
#' @description
#' The three-parameter logistic model is a model where the lower
#' asymptote parameter c is added to the 2PLM
#' @param a slope parameter
#' @param b location parameter
#' @param c lower asymptote parameter
#' @param theta ability parameter
#' @return Returns a numeric vector of probabilities between c and 1, representing
#'   the probability of a correct response given the ability level theta. The probability
#'   is calculated using the formula: \eqn{P(\theta) = c + \frac{1-c}{1 + e^{-a(\theta-b)}}}
#' @export

ThreePLM <- function(a, b, c, theta) {
  p <- c + (1 - c) / (1 + exp(-a * (theta - b)))
  return(p)
}

#' @title Four-Parameter Logistic Model
#' @description
#' The four-parameter logistic model is a model where one additional
#' parameter d, called the upper asymptote parameter, is added to the
#' 3PLM.
#' @param a slope parameter
#' @param b location parameter
#' @param c lower asymptote parameter
#' @param d upper asymptote parameter
#' @param theta ability parameter
#' @return Returns a numeric vector of probabilities between c and d, representing
#'   the probability of a correct response given the ability level theta. The probability
#'   is calculated using the formula: \eqn{P(\theta) = c + \frac{(d-c)}{1 + e^{-a(\theta-b)}}}
#' @export

LogisticModel <- function(a = 1, b, c = 0, d = 1, theta) {
  p <- c + ((d - c) / (1 + exp(-a * (theta - b))))
  return(p)
}

#' @title Rasch Model
#' @description
#' The one-parameter logistic model is a model with only one parameter b.
#' This model is a 2PLM model in which a is constrained to 1.
#' This model is also called the Rasch model.
#' @param b slope parameter
#' @param theta ability parameter
#' @return Returns a numeric vector of probabilities between 0 and 1, representing
#'   the probability of a correct response given the ability level theta. The probability
#'   is calculated using the formula: \eqn{P(\theta) = \frac{1}{1 + e^{-(\theta-b)}}}
#' @export

RaschModel <- function(b, theta) {
  p <- 1 / (1 + exp(-1 * (theta - b)))
  return(p)
}

#' @title IIF for 2PLM
#' @description
#' Item Information Function for 2PLM
#' @param a slope parameter
#' @param b location parameter
#' @param theta ability parameter
#' @return Returns a numeric vector representing the item information at each ability
#'   level theta. The information is calculated as:
#'   \eqn{I(\theta) = a^2P(\theta)(1-P(\theta))}
#' @export

IIF2PLM <- function(a, b, theta) {
  a^2 * TwoPLM(a, b, theta) * (1 - TwoPLM(a, b, theta))
}

#' @title IIF for 3PLM
#' @description
#' Item Information Function for 3PLM
#' @param a slope parameter
#' @param b location parameter
#' @param c lower asymptote parameter
#' @param theta ability parameter
#' @return Returns a numeric vector representing the item information at each ability
#'   level theta. The information is calculated as:
#'   \eqn{I(\theta) = \frac{a^2(1-P(\theta))(P(\theta)-c)^2}{(1-c)^2P(\theta)}}
#' @export

IIF3PLM <- function(a, b, c, theta) {
  numerator <- a^2 * (1 - ThreePLM(a, b, c, theta)) * (ThreePLM(a, b, c, theta) - c)^2
  denominator <- (1 - c)^2 * ThreePLM(a, b, c, theta)
  tmp <- numerator / denominator
  return(tmp)
}

#' @title IIF for 4PLM
#' @description
#' Item Information Function for 4PLM
#' @param a slope parameter
#' @param b location parameter
#' @param c lower asymptote parameter
#' @param d upper asymptote parameter
#' @param theta ability parameter
#' @return Returns a numeric vector representing the item information at each ability
#'   level theta. The information is calculated based on the first derivative of
#'   the log-likelihood of the 4PL model with respect to theta.
#' @export

ItemInformationFunc <- function(a = 1, b, c = 0, d = 1, theta) {
  numerator <- a^2 * (LogisticModel(a, b, c, d, theta) - c) * (d - LogisticModel(a, b, c, d, theta)) *
    (LogisticModel(a, b, c, d, theta) * (c + d - LogisticModel(a, b, c, d, theta)) - c * d)
  denominator <- (d - c)^2 * LogisticModel(a, b, c, d, theta) * (1 - LogisticModel(a, b, c, d, theta))
  tmp <- numerator / denominator
  return(tmp)
}


#' @title TIF for IRT
#' @description
#' Test Information Function for 4PLM
#' @param params parameter matrix
#' @param theta ability parameter
#' @return Returns a numeric vector representing the test information at each ability
#'   level theta. The test information is the sum of item information functions for
#'   all items in the test: \eqn{I_{test}(\theta) = \sum_{j=1}^n I_j(\theta)}
#' @export

TestInformationFunc <- function(params, theta) {
  tl <- nrow(params)
  tmp <- 0
  for (i in 1:tl) {
    a <- params[i, 1]
    b <- params[i, 2]
    if (ncol(params) > 2) {
      c <- params[i, 3]
    } else {
      c <- 0
    }
    if (ncol(params) > 3) {
      d <- params[i, 4]
    } else {
      d <- 1
    }
    tmp <- tmp + ItemInformationFunc(a = a, b = b, c = c, d = d, theta)
  }
  return(tmp)
}


#' @title TRF for IRT
#' @description
#' Calculates the expected score across all items on a test for a given ability level (theta)
#' using Item Response Theory. The Test Response Function (TRF) is essentially the sum of
#' the Item Characteristic Curves (ICCs) for all items in the test.
#'
#' @details
#' The Test Response Function computes the expected total score for an examinee with a given
#' ability level (theta) across all items in the test. For each item, the function uses the
#' logistic model with parameters a (discrimination), b (difficulty), c (guessing), and
#' d (upper asymptote).
#' @param params parameter matrix
#' @param theta ability parameter
#' @return A numeric vector with the same length as theta, containing the expected total score
#'   for each ability level.
#' @export
#'
TestResponseFunc <- function(params, theta) {
  tl <- nrow(params)
  tmp <- 0
  for (i in 1:tl) {
    a <- params[i, 1]
    b <- params[i, 2]
    if (ncol(params) > 2) {
      c <- params[i, 3]
    } else {
      c <- 0
    }
    if (ncol(params) > 3) {
      d <- params[i, 4]
    } else {
      d <- 1
    }
    tmp <- tmp + LogisticModel(a = a, b = b, c = c, d = d, theta)
  }
  return(tmp)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)

## ----library, message=FALSE, warning=FALSE------------------------------------
library(exametrika)

## ----model-biclustering, message=FALSE, warning=FALSE-------------------------
Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")

## ----model-ranklustering, message=FALSE, warning=FALSE------------------------
result.Ranklustering <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")

## ----plot-ranklustering, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot(result.Ranklustering, type = "Array")
plot(result.Ranklustering, type = "FRP", nc = 2, nr = 3)
plot(result.Ranklustering, type = "RRV")
plot(result.Ranklustering, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.Ranklustering, type = "LRD")

## ----grid-search, eval=FALSE--------------------------------------------------
# result <- GridSearch(J35S515, method = "R", max_ncls = 10, max_nfld = 10, index = "BIC")
# result$optimal_ncls
# result$optimal_nfld
# result$optimal_result

## ----model-irm, eval=FALSE----------------------------------------------------
# result.IRM <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
# plot(result.IRM, type = "Array")
# plot(result.IRM, type = "FRP", nc = 3)
# plot(result.IRM, type = "TRP")

## ----bic-ord, eval=FALSE------------------------------------------------------
# result.B.ord <- Biclustering(J35S500, ncls = 5, nfld = 5, method = "R")
# result.B.ord
# plot(result.B.ord, type = "Array")

## ----bic-ord-frp, eval=FALSE--------------------------------------------------
# plot(result.B.ord, type = "FRP", nc = 3, nr = 2)

## ----bic-ord-fcrp, eval=FALSE-------------------------------------------------
# plot(result.B.ord, type = "FCRP", nc = 3, nr = 2)
# plot(result.B.ord, type = "FCRP", style = "bar", nc = 3, nr = 2)

## ----bic-ord-fcbr, eval=FALSE-------------------------------------------------
# plot(result.B.ord, type = "FCBR", nc = 3, nr = 2)

## ----bic-ord-scorefield, eval=FALSE-------------------------------------------
# plot(result.B.ord, type = "ScoreField")
# plot(result.B.ord, type = "RRV")

## ----bic-nom, eval=FALSE------------------------------------------------------
# result.B.nom <- Biclustering(J20S600, ncls = 5, nfld = 4)
# result.B.nom
# plot(result.B.nom, type = "Array")

## ----bic-nom-plots, eval=FALSE------------------------------------------------
# plot(result.B.nom, type = "FRP", nc = 2, nr = 2)
# plot(result.B.nom, type = "FCRP", nc = 2, nr = 2)
# plot(result.B.nom, type = "FCRP", style = "bar", nc = 2, nr = 2)
# plot(result.B.nom, type = "ScoreField")
# plot(result.B.nom, type = "RRV")

## ----bic-rated, message=FALSE, warning=FALSE----------------------------------
result.B.rated <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R", maxiter = 300)
result.B.rated
plot(result.B.rated, type = "Array")
plot(result.B.rated, type = "FRP", nc = 3, nr = 1)

## ----irm-rated, message=FALSE, warning=FALSE----------------------------------
result.IRM.rated <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, verbose = FALSE)
plot(result.IRM.rated, type = "Array")
plot(result.IRM.rated, type = "FRP", nc = 3, nr = 1)

## ----distractor, eval=FALSE---------------------------------------------------
# result.B.rated <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R", maxiter = 300)
# da <- DistractorAnalysis(result.B.rated)
# 
# # Full output (grouped by field for Biclustering)
# print(da)
# 
# # Filter by items and/or ranks
# print(da, items = 1:7, ranks = c(1, 5))
# 
# # Plot distractor bar charts
# plot(da, items = 1:6, nc = 3, nr = 2)

## ----distractor-lra, eval=FALSE-----------------------------------------------
# result.LRA.rated <- LRA(J21S300, nrank = 5, mic = TRUE)
# da_lra <- DistractorAnalysis(result.LRA.rated)
# print(da_lra, items = 1:3)
# plot(da_lra, items = 1:6, nc = 3, nr = 2)

#' @rdname Biclustering_IRM
#' @param alpha Dirichlet distribution concentration parameter for the prior density of
#' field reference probabilities (rated/nominal IRM). Must be positive. The default is 1.
#' @return
#' For rated data, the returned list includes:
#' \describe{
#'  \item{Q}{Response matrix.}
#'  \item{U}{Binary correct/incorrect matrix.}
#'  \item{Z}{Missing indicator matrix.}
#'  \item{testlength}{Number of items.}
#'  \item{nobs}{Sample size.}
#'  \item{n_class}{Optimal number of classes.}
#'  \item{n_field}{Optimal number of fields.}
#'  \item{n_cycle}{Number of EM algorithm iterations.}
#'  \item{FRP}{Field Reference Profile (BCRM), a 3D array (nfld x ncls x maxQ).}
#'  \item{FieldFRP}{Field Reference Profile based on binary correct rates (nfld x ncls).}
#'  \item{quasiFRP}{Item-level correct response rate per class (nitems x ncls).}
#'  \item{FRPIndex}{Index of FRP includes the item location parameters B and Beta,
#'  the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
#'  \item{TRP}{Test Reference Profile.}
#'  \item{LFD}{Latent Field Distribution.}
#'  \item{LCD}{Latent Class Distribution.}
#'  \item{FieldMembership}{Field membership probability matrix.}
#'  \item{ClassMembership}{Class membership probability matrix.}
#'  \item{FieldEstimated}{Estimated field assignment for each item.}
#'  \item{ClassEstimated}{Estimated class assignment for each student.}
#'  \item{Students}{Rank Membership Profile matrix with estimated class.}
#'  \item{FieldAnalysis}{Field analysis with CRR and field memberships.}
#'  \item{TestFitIndices}{Overall fit index for the test (binary layer, default).}
#'  \item{TestFitIndices_nominal}{Fit indices for the nominal layer (AIC/BIC/CAIC only).}
#'  \item{log_lik}{Log-likelihood of the binary layer model.}
#'  \item{log_lik_nominal}{Log-likelihood of the nominal layer model.}
#'  \item{SOACflg}{Logical; TRUE if Strongly Ordinal Alignment Condition is satisfied.}
#'  \item{WOACflg}{Logical; TRUE if Weakly Ordinal Alignment Condition is satisfied.}
#' }
#' @examples
#' \donttest{
#' # Fit a rated Biclustering IRM model
#' result <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, verbose = TRUE)
#' plot(result, type = "Array")
#' }
#' @export
Biclustering_IRM.rated <- function(U,
                                   gamma_c = 1, gamma_f = 1, alpha = 1,
                                   max_iter = 100, stable_limit = 5,
                                   minSize = 20, EM_limit = 20,
                                   seed = 123, verbose = TRUE, ...) {
  tmp <- U
  nobs <- NROW(tmp$Q)
  nitems <- NCOL(tmp$Q)
  const <- exp(-nitems)

  # --- Step 1: Run nominal IRM internally ---
  dat_nom <- tmp
  dat_nom$response.type <- "nominal"

  ret_nom <- Biclustering_IRM.nominal(
    dat_nom,
    gamma_c = gamma_c, gamma_f = gamma_f, alpha = alpha,
    max_iter = max_iter, stable_limit = stable_limit,
    minSize = minSize, EM_limit = EM_limit,
    seed = seed, verbose = verbose,
    ...
  )

  # --- Step 2: Retrieve nominal results ---
  ncls <- ret_nom$n_class
  nfld <- ret_nom$n_field
  fld <- ret_nom$FieldEstimated
  fldmemb <- ret_nom$FieldMembership
  fldmemb01 <- sign(fldmemb - apply(fldmemb, 1, max)) + 1
  flddist <- colSums(fldmemb01)
  BCRM <- ret_nom$FRP

  # --- Step 3: Sort classes by correct response rate (Ranklustering) ---
  clsmemb <- ret_nom$ClassMembership
  ord <- order(colSums(t(tmp$U) %*% clsmemb) / colSums(clsmemb))
  clsmemb <- clsmemb[, ord]
  BCRM <- BCRM[, ord, ]
  cls <- apply(clsmemb, 1, which.max)
  clsmemb01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  clsdist <- colSums(clsmemb01)

  # --- Step 4: Student rank membership ---
  StudentRank <- clsmemb
  rownames(StudentRank) <- tmp$ID
  RU <- ifelse(cls + 1 > ncls, NA, cls + 1)
  RD <- ifelse(cls - 1 < 1, NA, cls - 1)
  RUO <- StudentRank[cbind(1:nobs, RU)] / StudentRank[cbind(1:nobs, cls)]
  RDO <- StudentRank[cbind(1:nobs, RD)] / StudentRank[cbind(1:nobs, cls)]
  StudentRank <- cbind(StudentRank, cls, RUO, RDO)
  colnames(StudentRank) <- c(
    paste("Membership", 1:ncls), "Estimate",
    "Rank-Up Odds", "Rank-Down Odds"
  )

  # --- Step 5: Binary fit indices (Layer 1) ---
  PiFR <- matrix(0, nrow = nitems, ncol = ncls)
  for (c in 1:ncls) {
    students_c <- which(cls == c)
    if (length(students_c) == 1) {
      denom <- tmp$Z[students_c, ]
      PiFR[, c] <- (tmp$U[students_c, ] * tmp$Z[students_c, ]) /
        pmax(denom, const)
    } else if (length(students_c) > 1) {
      denom <- colSums(tmp$Z[students_c, ])
      PiFR[, c] <- colSums(tmp$U[students_c, ] * tmp$Z[students_c, ]) /
        pmax(denom, const)
    }
  }

  P_correct <- clsmemb %*% t(PiFR)
  test_log_lik_binary <- sum(tmp$Z * (tmp$U * log(pmax(P_correct, const)) +
    (1 - tmp$U) * log(pmax(1 - P_correct, const))))
  nparam_binary <- nitems * ncls
  FitIndices_binary <- TestFit(tmp$U, tmp$Z, test_log_lik_binary, nparam_binary)

  # --- Step 6: Nominal fit indices (Layer 2) ---
  FitIndices_nominal <- ret_nom$TestFitIndices

  # --- Step 7: FRP labels ---
  msg <- "Rank"
  FRP <- BCRM
  colnames(FRP) <- paste0(msg, 1:ncls)
  rownames(FRP) <- paste0("Field", 1:nfld)
  colnames(fldmemb) <- paste0("Field", 1:nfld)
  rownames(clsmemb) <- tmp$ID
  colnames(clsmemb) <- paste0(msg, 1:ncls)

  # --- Step 8: TRP and FRPIndex (binary-based) ---
  quasiFRP <- PiFR
  rownames(quasiFRP) <- colnames(tmp$U)
  colnames(quasiFRP) <- paste0(msg, 1:ncls)

  FieldFRP <- matrix(0, nrow = nfld, ncol = ncls)
  for (f in 1:nfld) {
    items_f <- which(fld == f)
    if (length(items_f) == 0) {
      FieldFRP[f, ] <- NA
    } else if (length(items_f) == 1) {
      FieldFRP[f, ] <- PiFR[items_f, ]
    } else {
      FieldFRP[f, ] <- colMeans(PiFR[items_f, ])
    }
  }
  rownames(FieldFRP) <- paste0("Field", 1:nfld)
  colnames(FieldFRP) <- paste0(msg, 1:ncls)

  TRP <- colSums(FieldFRP * flddist, na.rm = TRUE)
  valid_fields <- which(!is.na(FieldFRP[, 1]))
  if (length(valid_fields) > 0) {
    FRPIndex <- IRPindex(FieldFRP[valid_fields, , drop = FALSE])
  } else {
    FRPIndex <- data.frame(
      Alpha = numeric(0), A = numeric(0),
      Beta = numeric(0), B = numeric(0),
      Gamma = numeric(0), C = numeric(0)
    )
  }

  TRPlag <- TRP[2:ncls]
  TRPmic <- sum(TRPlag[1:(ncls - 1)] - TRP[1:(ncls - 1)] < 0, na.rm = TRUE)
  FRPmic <- sum(abs(FRPIndex$C))
  SOACflg <- WOACflg <- FALSE
  if (TRPmic == 0) {
    WOACflg <- TRUE
    if (FRPmic == 0) {
      SOACflg <- TRUE
    }
  }
  if (verbose) {
    if (SOACflg & WOACflg) {
      message("\nStrongly ordinal alignment condition was satisfied.")
    }
    if (!SOACflg & WOACflg) {
      message("\nWeakly ordinal alignment condition was satisfied.")
    }
  }

  # --- Step 9: Field Analysis ---
  tmp_binary <- tmp
  tmp_binary$response.type <- "binary"
  crr_vals <- crr(tmp_binary)
  fieldAnalysis <- as.data.frame(fldmemb)
  fieldAnalysis <- cbind(crr_vals, fld, fieldAnalysis)
  colnames(fieldAnalysis) <- c("CRR", "LFE", paste0("Field", 1:nfld))
  fieldAnalysis <- fieldAnalysis[order(fieldAnalysis$CRR, decreasing = TRUE), ]
  fieldAnalysis <- fieldAnalysis[order(fieldAnalysis$LFE), ]
  rownames_tmp <- rownames(fieldAnalysis)
  fieldAnalysis <- matrix(
    as.numeric(as.matrix(fieldAnalysis)),
    ncol = NCOL(fieldAnalysis), nrow = NROW(fieldAnalysis)
  )
  colnames(fieldAnalysis) <- c("CRR", "LFE", paste0("Field", 1:nfld))
  rownames(fieldAnalysis) <- rownames_tmp

  # --- Return ---
  ret <- structure(list(
    model = 2,
    mic = TRUE,
    msg = msg,
    Q = tmp$Q,
    U = tmp$U,
    Z = tmp$Z,
    testlength = nitems,
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    n_cycle = ret_nom$n_cycle,
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = FRP,
    FieldFRP = FieldFRP,
    quasiFRP = quasiFRP,
    FRPIndex = FRPIndex,
    TRP = TRP,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    FieldAnalysis = fieldAnalysis,
    TestFitIndices = FitIndices_binary,
    TestFitIndices_nominal = FitIndices_nominal,
    log_lik = test_log_lik_binary,
    log_lik_nominal = ret_nom$log_lik,
    SOACflg = SOACflg,
    WOACflg = WOACflg,
    # Deprecated fields (for backward compatibility)
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = ret_nom$n_cycle,
    LogLik = test_log_lik_binary
  ), class = c("exametrika", "ratedBiclustering"))

  return(ret)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

simple_quadrature_grm <- function(n_points = 51L) {
    .Call(`_exametrika_simple_quadrature_grm`, n_points)
}

target_to_params_grm <- function(target, nitems, ncat) {
    .Call(`_exametrika_target_to_params_grm`, target, nitems, ncat)
}

log_lik_grm_cpp <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_log_lik_grm_cpp`, target, dat, n_quad_points)
}

score_function_analytical_grm <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_score_function_analytical_grm`, target, dat, n_quad_points)
}

score_function_numerical_grm <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_score_function_numerical_grm`, target, dat, n_quad_points)
}

compare_gradients_grm <- function(target, dat, n_quad_points = 51L) {
    .Call(`_exametrika_compare_gradients_grm`, target, dat, n_quad_points)
}

irm_gibbs_core_cpp <- function(Uq_vec, Uq_dim, Z, cls01_init, fld01_init, gamma_c, gamma_f, alpha_vec_R, max_iter, stable_limit, verbose) {
    .Call(`_exametrika_irm_gibbs_core_cpp`, Uq_vec, Uq_dim, Z, cls01_init, fld01_init, gamma_c, gamma_f, alpha_vec_R, max_iter, stable_limit, verbose)
}

qBiNormal_cpp <- function(a, b, rho) {
    .Call(`_exametrika_qBiNormal_cpp`, a, b, rho)
}

polychoric_likelihood_cpp <- function(rho, mat) {
    .Call(`_exametrika_polychoric_likelihood_cpp`, rho, mat)
}

polychoric_cpp <- function(x, y) {
    .Call(`_exametrika_polychoric_cpp`, x, y)
}

polychoric_matrix_cpp <- function(data) {
    .Call(`_exametrika_polychoric_matrix_cpp`, data)
}

# Tests for GRM C++ Implementation

test_that("GRM helper functions work correctly", {
  # Test target_to_params_grm
  test_target <- c(0, 0.5, -1, 0, log(0.5), 0)
  test_nitems <- 2
  test_ncat <- c(3, 3)

  params_result <- target_to_params_grm(test_target, test_nitems, test_ncat)

  expect_type(params_result, "list")
  expect_named(params_result, c("a", "b"))
  expect_length(params_result$a, 2)
  expect_length(params_result$b, 2)
  expect_true(all(params_result$a > 0)) # Discrimination should be positive

  # Test simple_quadrature_grm
  quad_result <- simple_quadrature_grm(21)

  expect_type(quad_result, "list")
  expect_named(quad_result, c("nodes", "weights"))
  expect_length(quad_result$nodes, 21)
  expect_length(quad_result$weights, 21)
  expect_equal(sum(quad_result$weights), 1, tolerance = 1e-10)
})

test_that("log_lik_grm_cpp works correctly", {
  # Create small test dataset
  set.seed(123)
  small_data <- matrix(sample(1:3, 30, replace = TRUE), nrow = 10, ncol = 3)

  # Generate reasonable starting values
  start_vals <- generate_start_values(small_data)
  target <- params_to_target(start_vals$a, start_vals$b)

  ll_result <- log_lik_grm_cpp(target, small_data, 21)

  expect_type(ll_result, "double")
  expect_true(is.finite(ll_result))
  expect_true(ll_result < 0) # Log-likelihood should be negative
})

test_that("score_function_analytical_grm works correctly", {
  # Create small test dataset
  set.seed(456)
  small_data <- matrix(sample(1:4, 40, replace = TRUE), nrow = 10, ncol = 4)

  # Generate starting values
  start_vals <- generate_start_values(small_data)
  target <- params_to_target(start_vals$a, start_vals$b)

  # Test analytical gradient
  grad_result <- score_function_analytical_grm(target, small_data, 21)

  expect_type(grad_result, "double")
  expect_length(grad_result, length(target))
  expect_true(all(is.finite(grad_result)))
})

test_that("compare_gradients_grm works correctly", {
  # Create small test dataset
  set.seed(789)
  small_data <- matrix(sample(1:3, 24, replace = TRUE), nrow = 8, ncol = 3)

  # Generate starting values
  start_vals <- generate_start_values(small_data)
  target <- params_to_target(start_vals$a, start_vals$b)

  # Compare gradients
  comparison <- compare_gradients_grm(target, small_data, 21)

  expect_type(comparison, "list")
  expect_named(comparison, c("analytical", "numerical", "difference", "max_abs_diff", "rms_diff"))
  expect_length(comparison$analytical, length(target))
  expect_length(comparison$numerical, length(target))
  expect_type(comparison$max_abs_diff, "double")
  expect_type(comparison$rms_diff, "double")

  # Gradients should be reasonably close
  expect_true(comparison$max_abs_diff < 1e-2)
})

test_that("GRM model fitting works correctly", {
  # Use example data and ensure ordinal type
  data("J5S1000", package = "exametrika")
  small_data <- J5S1000$Q[1:50, 1:3]

  # Format data explicitly as ordinal
  formatted_data <- dataFormat(small_data, response.type = "ordinal")

  # Fit GRM model
  grm_result <- GRM(formatted_data, verbose = FALSE)

  expect_s3_class(grm_result, c("exametrika", "GRM"))
  expect_named(grm_result, c(
    "testlength", "nobs", "log_lik", "iterations",
    "params", "EAP", "MAP", "PSD",
    "ItemFitIndices", "TestFitIndices"
  ))

  # Check dimensions
  expect_equal(grm_result$testlength, 3)
  expect_equal(grm_result$nobs, 50)
  expect_equal(nrow(grm_result$params), 3)
  expect_length(grm_result$EAP, 50)
  expect_length(grm_result$MAP, 50)
  expect_length(grm_result$PSD, 50)

  # Check parameter estimates are reasonable
  expect_true(all(grm_result$params$Slope > 0))
  expect_true(all(is.finite(grm_result$params$Slope)))
  expect_true(is.finite(grm_result$log_lik))
  expect_true(grm_result$iterations[[1]] >= 1)
})

test_that("GRM performance is acceptable", {
  # Create medium-sized dataset
  set.seed(999)
  medium_data <- matrix(sample(1:4, 200, replace = TRUE), nrow = 50, ncol = 4)

  # Format as ordinal explicitly
  formatted_medium <- dataFormat(medium_data, response.type = "ordinal")

  # Time the fitting
  start_time <- Sys.time()
  grm_result <- GRM(formatted_medium, verbose = FALSE)
  elapsed <- as.numeric(Sys.time() - start_time, units = "secs")

  expect_true(elapsed < 10) # Should complete within 10 seconds
  expect_s3_class(grm_result, "GRM")
  expect_true(is.finite(grm_result$log_lik))
})

test_that("GRM ItemFitIndices has correct structure and values", {
  # Use example data with mixed category counts
  data("J5S1000", package = "exametrika")
  small_data <- J5S1000$Q[1:50, 1:3]
  formatted_data <- dataFormat(small_data, response.type = "ordinal")
  grm_result <- GRM(formatted_data, verbose = FALSE)

  nitems <- grm_result$testlength

  # ItemFitIndices: each field should be a vector of length nitems
  ifi <- grm_result$ItemFitIndices
  expect_s3_class(ifi, "ModelFit")
  expect_length(ifi$model_log_like, nitems)
  expect_length(ifi$bench_log_like, nitems)
  expect_length(ifi$null_log_like, nitems)
  expect_length(ifi$model_Chi_sq, nitems)
  expect_length(ifi$null_Chi_sq, nitems)
  expect_length(ifi$model_df, nitems)
  expect_length(ifi$null_df, nitems)
  expect_true(all(is.finite(ifi$model_log_like)))
  expect_true(all(is.finite(ifi$null_log_like)))
  expect_true(all(is.finite(ifi$bench_log_like)))

  # Log-likelihoods should be negative (log of probabilities)
  expect_true(all(ifi$model_log_like < 0))
  expect_true(all(ifi$null_log_like < 0))
  expect_true(all(ifi$bench_log_like < 0))

  # Bench (saturated) >= Analysis >= Null in terms of log-likelihood
  expect_true(all(ifi$bench_log_like >= ifi$model_log_like - 1e-10))
  expect_true(all(ifi$model_log_like >= ifi$null_log_like - 1e-10))

  # Chi-square should be non-negative
  expect_true(all(ifi$model_Chi_sq >= -1e-10))
  expect_true(all(ifi$null_Chi_sq >= -1e-10))

  # TestFitIndices: each field should be a scalar
  tfi <- grm_result$TestFitIndices
  expect_s3_class(tfi, "ModelFit")
  expect_length(tfi$model_log_like, 1)
  expect_length(tfi$bench_log_like, 1)
  expect_length(tfi$null_log_like, 1)
  expect_true(is.finite(tfi$model_log_like))
  expect_true(is.finite(tfi$null_log_like))

  # TestFitIndices should be sums of ItemFitIndices
  expect_equal(tfi$model_log_like, sum(ifi$model_log_like), tolerance = 1e-10)
  expect_equal(tfi$bench_log_like, sum(ifi$bench_log_like), tolerance = 1e-10)
  expect_equal(tfi$null_log_like, sum(ifi$null_log_like), tolerance = 1e-10)
})

test_that("GRM works correctly with uniform category counts (apply/table bug regression)", {
  # This test specifically targets the apply()/table() simplification bug:
  # When all items have the same number of categories, apply(X, 2, table)
  # returns a matrix instead of a list, causing response_list[[j]] to fail.
  # The fix uses lapply(seq_len(nitems), function(j) table(X[, j])) instead.

  # Create data where ALL items have the same number of categories (5)
  set.seed(42)
  nobs <- 80
  nitems <- 4
  uniform_data <- matrix(sample(1:5, nobs * nitems, replace = TRUE),
    nrow = nobs, ncol = nitems
  )
  formatted_data <- dataFormat(uniform_data, response.type = "ordinal")

  # This should NOT error (the bug caused errors here)
  grm_result <- GRM(formatted_data, verbose = FALSE)

  expect_s3_class(grm_result, c("exametrika", "GRM"))
  expect_equal(grm_result$testlength, nitems)
  expect_equal(grm_result$nobs, nobs)

  # ItemFitIndices should have correct structure
  ifi <- grm_result$ItemFitIndices
  expect_length(ifi$model_log_like, nitems)
  expect_length(ifi$null_log_like, nitems)
  expect_true(all(is.finite(ifi$model_log_like)))
  expect_true(all(is.finite(ifi$null_log_like)))

  # null_log_like should be properly computed (not NaN or Inf)
  expect_true(all(ifi$null_log_like < 0))

  # TestFitIndices should be scalar sums
  tfi <- grm_result$TestFitIndices
  expect_equal(tfi$model_log_like, sum(ifi$model_log_like), tolerance = 1e-10)
  expect_equal(tfi$null_log_like, sum(ifi$null_log_like), tolerance = 1e-10)
})

test_that("GRM fit indices are finite for moderate item counts (exp(-nitems*100) underflow regression)", {
  skip_on_cran()
  # For nitems >= 8, const <- exp(-nitems * 100) underflowed to 0 and
  # 0 * log(0) poisoned every item/test fit index with NaN.
  set.seed(20260422)
  nitems <- 20
  nobs <- 250
  dat <- matrix(sample(1:3, nobs * nitems, replace = TRUE),
    nrow = nobs, ncol = nitems
  )
  fd <- dataFormat(dat, response.type = "ordinal")
  res <- GRM(fd, verbose = FALSE)

  ifi <- res$ItemFitIndices
  expect_true(all(is.finite(ifi$bench_log_like)))
  expect_true(all(is.finite(ifi$model_Chi_sq)))
  expect_true(all(is.finite(ifi$CFI)))
  expect_true(all(is.finite(ifi$RMSEA)))

  tfi <- res$TestFitIndices
  expect_true(is.finite(tfi$bench_log_like))
  expect_true(is.finite(tfi$RMSEA))
  expect_true(is.finite(tfi$AIC))
  expect_true(is.finite(tfi$BIC))
})

test_that("GRM accepts 0-based ordinal responses (ncat max-value regression)", {
  skip_on_cran()
  # ncat was derived from apply(dat, 2, max), so 0..K-1 coded data and
  # sparse codings (e.g. 1,2,4) silently dropped the top category.
  data("J15S3810", package = "exametrika")
  res <- GRM(J15S3810, verbose = FALSE)

  expect_s3_class(res, c("exametrika", "GRM"))
  expect_equal(res$testlength, 15)
  # J15S3810 items have 4 categories each, so 3 thresholds per item.
  expect_equal(ncol(res$params), 4L)
  expect_true(all(is.finite(res$params$Slope)))
  expect_true(all(is.finite(res$TestFitIndices$RMSEA)))

  # A gapped coding (categories 1, 2, 4) must estimate the same number
  # of thresholds as a contiguous 3-category coding.
  set.seed(1)
  gapped <- matrix(sample(c(1, 2, 4), 200 * 5, replace = TRUE),
    nrow = 200, ncol = 5
  )
  fd <- dataFormat(gapped, response.type = "ordinal")
  res_g <- GRM(fd, verbose = FALSE)
  expect_equal(ncol(res_g$params), 3L)
  expect_true(all(is.finite(res_g$TestFitIndices$AIC)))
})

test_that("GRM error handling works", {
  # Test with inappropriate data
  expect_error(GRM(matrix(letters[1:20], nrow = 5, ncol = 4)))

  # Test with insufficient data
  tiny_data <- matrix(c(1, 2, 1, 2), nrow = 2, ncol = 2)
  expect_error(GRM(tiny_data))
})

# Tests for Polychoric Correlation C++ Implementation

test_that("qBiNormal_cpp works correctly", {
  # Test basic functionality
  expect_equal(qBiNormal_cpp(0, 0, 0), 0.25, tolerance = 1e-10)

  # Test infinite bounds
  expect_equal(qBiNormal_cpp(-Inf, 0, 0.2), 0, tolerance = 1e-10)
  expect_equal(qBiNormal_cpp(0, Inf, 0.4), 0.5, tolerance = 1e-10)
  expect_equal(qBiNormal_cpp(Inf, Inf, 0.5), 1, tolerance = 1e-10)

  # Test edge cases
  expect_equal(qBiNormal_cpp(-1, -1, 0.5), 0.062514, tolerance = 1e-5)
  expect_equal(qBiNormal_cpp(1, 1, -0.3), 0.693007, tolerance = 1e-5)
})

test_that("polychoric_likelihood_cpp works correctly", {
  # Create test contingency table
  test_mat <- matrix(c(
    20, 15, 5,
    10, 25, 15,
    5, 10, 30
  ), nrow = 3, ncol = 3, byrow = TRUE)

  # Test different correlation values
  ll_0 <- polychoric_likelihood_cpp(0, test_mat)
  ll_pos <- polychoric_likelihood_cpp(0.5, test_mat)
  ll_neg <- polychoric_likelihood_cpp(-0.5, test_mat)

  expect_type(ll_0, "double")
  expect_type(ll_pos, "double")
  expect_type(ll_neg, "double")

  expect_true(is.finite(ll_0))
  expect_true(is.finite(ll_pos))
  expect_true(is.finite(ll_neg))
})

test_that("polychoric_cpp individual correlation works", {
  # Test data
  x <- c(1, 2, 3, 2, 1, 3, 2, 2, 1, 3)
  y <- c(2, 3, 1, 2, 3, 1, 2, 1, 3, 2)

  result <- polychoric_cpp(as.integer(x), as.integer(y))

  expect_type(result, "double")
  expect_true(abs(result) <= 1)
  expect_false(is.na(result))

  # Test with insufficient data
  expect_true(is.na(polychoric_cpp(c(1L, 2L), c(1L, 2L))))
})

test_that("polychoric_matrix_cpp works correctly", {
  # Create test matrix
  set.seed(42)
  test_matrix <- matrix(sample(1:3, 30, replace = TRUE), ncol = 3)

  result <- polychoric_matrix_cpp(test_matrix)

  # Check dimensions
  expect_equal(dim(result), c(3, 3))

  # Check diagonal is 1
  expect_equal(diag(result), c(1, 1, 1), tolerance = 1e-10)

  # Check symmetry
  expect_equal(result[1, 2], result[2, 1], tolerance = 1e-10)
  expect_equal(result[1, 3], result[3, 1], tolerance = 1e-10)
  expect_equal(result[2, 3], result[3, 2], tolerance = 1e-10)

  # Check all values are valid correlations
  expect_true(all(abs(result) <= 1))
})

test_that("polychoric function integration works", {
  x <- c(1, 2, 3, 2, 1, 3, 2, 2, 1, 3, 1, 2, 3, 2, 1)
  y <- c(2, 3, 1, 2, 3, 1, 2, 1, 3, 2, 3, 1, 2, 3, 1)

  result <- polychoric(x, y)

  expect_type(result, "double")
  expect_true(abs(result) <= 1)
  expect_false(is.na(result))
})

test_that("Performance is significantly improved", {
  x <- sample(1:4, 50, replace = TRUE)
  y <- sample(1:3, 50, replace = TRUE)

  # Time the C++ implementation
  start_time <- Sys.time()
  result <- polychoric_cpp(as.integer(x), as.integer(y))
  elapsed <- as.numeric(Sys.time() - start_time, units = "secs")

  expect_true(elapsed < 0.1) # Should be very fast
  expect_false(is.na(result))
})

# Basic Package Tests

test_that("Package loads and example data is available", {
  # Test that the package namespace is loaded
  expect_true("exametrika" %in% loadedNamespaces())

  # Test example datasets
  data("J5S1000")
  expect_s3_class(J5S1000, "exametrika")
  expect_equal(nrow(J5S1000$Q), 1000)
  expect_equal(ncol(J5S1000$Q), 5)
  expect_equal(J5S1000$response.type, "ordinal")

  data("J15S500")
  expect_s3_class(J15S500, "exametrika")
  expect_equal(nrow(J15S500$U), 500)
  expect_equal(ncol(J15S500$U), 15)
  expect_equal(J15S500$response.type, "binary")
})

test_that("dataFormat works correctly", {
  # Test with simple binary data (25 rows)
  set.seed(200)
  binary_data <- matrix(sample(c(0, 1), 25 * 4, replace = TRUE), nrow = 25, ncol = 4)
  result <- dataFormat(binary_data)

  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "binary")
  expect_equal(nrow(result$U), 25)
  expect_equal(ncol(result$U), 4)

  # Test with ordinal data (explicitly specified, 25 rows)
  set.seed(201)
  ordinal_data <- matrix(sample(1:4, 25 * 4, replace = TRUE), nrow = 25, ncol = 4)
  result <- dataFormat(ordinal_data, response.type = "ordinal")

  expect_s3_class(result, "exametrika")
  expect_equal(result$response.type, "ordinal")
  expect_equal(nrow(result$Q), 25)
  expect_equal(ncol(result$Q), 4)

  # Test automatic ordinal detection (25 rows)
  set.seed(202)
  ordinal_auto <- matrix(sample(1:4, 25 * 4, replace = TRUE), nrow = 25, ncol = 4)
  result_auto <- dataFormat(ordinal_auto)

  expect_s3_class(result_auto, "exametrika")
  expect_equal(result_auto$response.type, "ordinal")

  # Test with missing values (25 rows)
  set.seed(203)
  data_with_na <- matrix(sample(c(1, 2, 3, NA), 25 * 2, replace = TRUE), nrow = 25, ncol = 2)
  result <- dataFormat(data_with_na)

  expect_s3_class(result, "exametrika")
  expect_true(any(result$Z == 0)) # Should have missing indicators
})

test_that("Essential package functions are exported", {
  exported_functions <- getNamespaceExports("exametrika")

  # Core functions should be exported
  essential_functions <- c(
    "GRM", "polychoric", "PolychoricCorrelationMatrix",
    "dataFormat", "crr", "ItemTotalCorr", "sscore",
    "nrs", "passage", "percentile", "stanine"
  )

  for (func in essential_functions) {
    expect_true(func %in% exported_functions,
      info = paste("Essential function", func, "should be exported")
    )
  }
})

test_that("C++ functions are available", {
  # Test that key C++ functions exist and work
  cpp_functions <- c(
    "qBiNormal_cpp", "polychoric_cpp", "polychoric_matrix_cpp",
    "log_lik_grm_cpp", "score_function_analytical_grm",
    "target_to_params_grm", "simple_quadrature_grm"
  )

  for (func in cpp_functions) {
    expect_true(exists(func, mode = "function"),
      info = paste("C++ function", func, "should be available")
    )
  }

  # Quick functionality test
  expect_type(qBiNormal_cpp(0, 0, 0), "double")
  expect_type(simple_quadrature_grm(5), "list")
})

#' @title Local Dependence Biclustering
#' @description
#' Latent dependence Biclustering, which incorporates biclustering and a Bayesian
#' network model.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param ncls number of latent class(rank). The default is 2.
#' @param method specify the model to analyze the data.Local dependence latent
#' class model is set to "C", latent rank model is set "R". The default is "R".
#' @param conf For the confirmatory parameter, you can input either a vector with
#' items and corresponding fields in sequence, or a field membership profile
#' matrix. In the case of the former, the field membership profile matrix will be generated internally.
#' When providing a membership profile matrix, it needs to be either matrix or data.frame.
#' The number of fields(nfld) will be overwrite to the number of columns of this matrix.
#' @param g_list A list compiling graph-type objects for each rank/class.
#' @param adj_list A list compiling matrix-type adjacency matrices for each rank/class.
#' @param adj_file A file detailing the relationships of the graph for each rank/class,
#' listed in the order of starting point, ending point, and rank(class).
#' @param verbose verbose output Flag. default is TRUE
#' @param beta1 Beta distribution parameter 1 for prior density. Default is 1.
#' @param beta2 Beta distribution parameter 2 for prior density. Default is 1.
#' @return
#' \describe{
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{msg}{A character string indicating the model type. }
#'  \item{Nclass}{Optimal number of classes.}
#'  \item{Nfield}{Optimal number of fields.}
#'  \item{crr}{Correct Response Rate}
#'  \item{ItemLabel}{Label of Items}
#'  \item{FieldLabel}{Label of Fields}
#'  \item{adj_list}{List of Adjacency matrix used in the model}
#'  \item{g_list}{List of graph object used in the model}
#'  \item{IRP}{List of Estimated Parameters. This object is three-dimensional
#'  PIRP array, where each dimension represents the number of rank,number
#'  of field, and Dmax. Dmax denotes the maximum number of correct response
#'  patterns for each field.}
#'  \item{LFD}{Latent Field Distribution. see also [plot.exametrika]}
#'  \item{LRD}{Latent Rank Distribution. see also [plot.exametrika]}
#'  \item{FRP}{Marginal Field Reference Matrix}
#'  \item{FRPIndex}{Index of FFP includes the item location parameters B and Beta,
#'  the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
#'  \item{CCRR_table}{This table is a rearrangement of IRP into a data.frame
#'  format for output, consisting of combinations of rank ,field and PIRP.}
#'  \item{TRP}{Test Reference Profile}
#'  \item{RMD}{Rank Membership Distribution.}
#'  \item{FieldEstimated}{Given vector which correspondence between items
#'  and the fields.}
#'  \item{ClassEstimated}{An index indicating which class a student belongs
#'  to, estimated by confirmatory Ranklustering.}
#'  \item{Students}{Rank Membership Profile matrix.The s-th row vector of \eqn{\hat{M}_R}, \eqn{\hat{m}_R}, is the
#' rank membership profile of Student s, namely the posterior probability distribution representing the student's
#' belonging to the respective latent classes. It also includes the rank with the maximum estimated membership probability,
#' as well as the rank-up odds and rank-down odds.}
#'  \item{TestFitIndices}{Overall fit index for the test.See also [TestFit]}
#' }
#' @examples
#' \donttest{
#' # Example: Latent Dirichlet Bayesian Network model
#' # Create field configuration vector based on field assignments
#' conf <- c(
#'   1, 6, 6, 8, 9, 9, 4, 7, 7, 7, 5, 8, 9, 10, 10, 9, 9,
#'   10, 10, 10, 2, 2, 3, 3, 5, 5, 6, 9, 9, 10, 1, 1, 7, 9, 10
#' )
#'
#' # Create edge data for the network structure between fields
#' edges_data <- data.frame(
#'   "From Field (Parent) >>>" = c(
#'     6, 4, 5, 1, 1, 4, # Class/Rank 2
#'     3, 4, 6, 2, 4, 4, # Class/Rank 3
#'     3, 6, 4, 1, # Class/Rank 4
#'     7, 9, 6, 7 # Class/Rank 5
#'   ),
#'   ">>> To Field (Child)" = c(
#'     8, 7, 8, 7, 2, 5, # Class/Rank 2
#'     5, 8, 8, 4, 6, 7, # Class/Rank 3
#'     5, 8, 5, 8, # Class/Rank 4
#'     10, 10, 8, 9 # Class/Rank 5
#'   ),
#'   "At Class/Rank (Locus)" = c(
#'     2, 2, 2, 2, 2, 2, # Class/Rank 2
#'     3, 3, 3, 3, 3, 3, # Class/Rank 3
#'     4, 4, 4, 4, # Class/Rank 4
#'     5, 5, 5, 5 # Class/Rank 5
#'   )
#' )
#'
#' # Save edge data to temporary CSV file
#' tmp_file <- tempfile(fileext = ".csv")
#' write.csv(edges_data, file = tmp_file, row.names = FALSE)
#'
#' # Fit Latent Dirichlet Bayesian Network model
#' result.LDB <- LDB(
#'   U = J35S515,
#'   ncls = 5, # Number of latent classes
#'   conf = conf, # Field configuration vector
#'   adj_file = tmp_file # Path to the CSV file
#' )
#'
#' # Clean up temporary file
#' unlink(tmp_file)
#'
#' # Display model results
#' print(result.LDB)
#'
#' # Visualize different aspects of the model
#' plot(result.LDB, type = "Array") # Show bicluster structure
#' plot(result.LDB, type = "TRP") # Test Response Profile
#' plot(result.LDB, type = "LRD") # Latent Rank Distribution
#' plot(result.LDB,
#'   type = "RMP", # Rank Membership Profiles
#'   students = 1:9, nc = 3, nr = 3
#' )
#' plot(result.LDB,
#'   type = "FRP", # Field Reference Profiles
#'   nc = 3, nr = 2
#' )
#' # Field PIRP Profile showing correct answer counts for each rank and field
#' plot(result.LDB, type = "FieldPIRP")
#' }
#' @export

LDB <- function(U, Z = NULL, w = NULL, na = NULL,
                ncls = 2, method = "R",
                conf = NULL,
                g_list = NULL, adj_list = NULL, adj_file = NULL,
                verbose = FALSE, beta1 = 1, beta2 = 1) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (U$response.type != "binary") {
    response_type_error(U$response.type, "LDB")
  }

  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  # Check the settings ------------------------------------------------------


  # Check the conf
  if (is.vector(conf)) {
    # check size
    if (length(conf) != NCOL(U)) {
      stop("conf vector size does NOT match with data.")
    }
    conf_mat <- matrix(0, nrow = NCOL(U), ncol = max(conf))
    for (i in 1:NROW(conf_mat)) {
      conf_mat[i, conf[i]] <- 1
    }
  } else if (is.matrix(conf) | is.data.frame(conf)) {
    if (NROW(conf) != NCOL(U)) {
      stop("conf matrix size does NOT match with data.")
    }
    if (any(!conf %in% c(0, 1))) {
      stop("The conf matrix should only contain 0s and 1s.")
    }
    if (any(rowSums(conf) > 1)) {
      stop("The row sums of the conf matrix must be equal to 1.")
    }
  }

  nfld <- NCOL(conf_mat)

  # graph check
  if (is.null(g_list) && is.null(adj_list) && is.null(adj_file)) {
    stop("Specify the graph in either matrix form, CSV file, or as a graph object.")
  }

  FieldLabel <- sprintf("Field%02d", 1:nfld)

  # g_list check
  if (!is.null(g_list)) {
    if (length(g_list) != ncls) {
      stop("The number of classes does not match the length of the list.
           Please specify a graph for all classes.")
    }
    adj_list <- list()
    for (j in 1:ncls) {
      if (!inherits(g_list[[1]], "igraph")) {
        stop("Some items in g_list are not recognized as graph objects.")
      }
      adj_list[[j]] <- fill_adj(g_list[[j]], FieldLabel)
    }
  }
  # adj_list check
  if (!is.null(adj_list)) {
    if (length(adj_list) != ncls) {
      stop("The number of classes does not match the length of the list.
           Please specify a graph for all classes.")
    }
    for (j in 1:ncls) {
      g <- igraph::graph_from_adjacency_matrix(adj_list[[j]])
      adj_list[[j]] <- fill_adj(g, FieldLabel)
    }
  }
  # adj_file check
  if (!is.null(adj_file)) {
    g_csv <- read.csv(adj_file)
    colnames(g_csv) <- c("From", "To", "Rank")
    g_csv$From <- sprintf("Field%02d", g_csv$From)
    g_csv$To <- sprintf("Field%02d", g_csv$To)
    adj_list <- list()
    for (i in 1:ncls) {
      adj_R <- g_csv[g_csv$Rank == i, 1:2]
      g_tmp <- igraph::graph_from_data_frame(adj_R)
      adj_list[[i]] <- fill_adj(g_tmp, FieldLabel)
    }
  }

  ret.Biclustering <- Biclustering(tmp,
    ncls = ncls,
    nfld = nfld,
    method = "R",
    conf = conf,
    verbose = FALSE
  )
  const <- exp(-testlength)


  maxnpa <- 4
  parent <- list()
  nparent <- array(NA, dim = c(nfld, ncls))
  for (i in 1:ncls) {
    adj <- adj_list[[i]]
    npa <- colSums(adj)
    parent[[i]] <- list()
    for (j in 1:nfld) {
      nparent[j, i] <- npa[j]
      if (npa[j] == 0) {
        parent[[i]][[j]] <- ""
      } else {
        parent[[i]][[j]] <- FieldLabel[adj[, j] == 1]
      }
    }
  }

  if (max(nparent) > maxnpa) {
    warning("[Caution!] The maximum number of parents per item is ", maxnpa, ". Please check.")
  }

  flddist <- colSums(ret.Biclustering$FieldMembership)
  fld <- ret.Biclustering$FieldEstimated
  csf <- tmp$U %*% ret.Biclustering$FieldMembership
  colnames(csf) <- FieldLabel
  nsf <- matrix(rep(flddist, nobs), nrow = nobs, byrow = TRUE)
  zeroVecS <- rep(0, nobs)

  #### SxRxF
  pirp_mat <- array(0, dim = c(nobs, ncls, nfld))
  for (i in 1:nobs) {
    for (j in 1:ncls) {
      for (k in 1:nfld) {
        if (parent[[j]][[k]][1] != "") {
          pirp_mat[i, j, k] <- sum(csf[i, parent[[j]][[k]]])
        }
      }
    }
  }
  pattern_max <- max(pirp_mat)
  Dmax <- pattern_max + 1
  #### SxRxFxDmax
  pirp_array <- array(0, dim = c(nobs, ncls, nfld, Dmax))
  for (i in 1:nobs) {
    for (j in 1:ncls) {
      for (k in 1:nfld) {
        vec <- rep(0, Dmax)
        vec[pirp_mat[i, j, k] + 1] <- 1
        pirp_array[i, j, k, ] <- vec
      }
    }
  }

  ### RxFxDmax
  n_pirp <- array(0, dim = c(ncls, nfld, Dmax))
  Smem_expand <- array(replicate(nfld * Dmax, ret.Biclustering$SmoothedMembership),
    dim = dim(pirp_array)
  )
  csf_expand <- array(replicate(ncls * Dmax, csf), dim = c(nobs, nfld, ncls, Dmax))
  n_pirp <- apply(csf_expand * (aperm(Smem_expand * pirp_array, perm = c(1, 3, 2, 4))), 2:4, sum)
  n_pirp <- aperm(n_pirp, perm = c(2, 1, 3))

  ### RxFxDmax
  denom_npirp <- array(0, dim = c(ncls, nfld, Dmax))
  nsf_expand <- array(replicate(ncls * Dmax, nsf), dim = c(nobs, nfld, ncls, Dmax))
  denom_npirp <- apply(nsf_expand * (aperm(Smem_expand * pirp_array, perm = c(1, 3, 2, 4))), 2:4, sum)
  denom_npirp <- aperm(denom_npirp, perm = c(2, 1, 3)) + beta1 + beta2 - 2
  denom0 <- sign(denom_npirp)

  denom_npirp <- denom_npirp + (1 - denom0) * 100
  param <- (n_pirp + beta2 - 1) / denom_npirp

  ### IRP
  pirp_trans <- aperm(pirp_array, perm = c(2, 3, 4, 1))
  param_expand <- array(replicate(nobs, param), dim = dim(pirp_trans))
  denom0_expand <- array(replicate(nobs, denom0), dim = dim(pirp_trans))

  tmp1 <- log(apply(pirp_trans * param_expand * denom0_expand, c(1, 2, 4), sum) + const)
  tmp1 <- aperm(tmp1, perm = c(3, 2, 1))
  csf_expand2 <- array(replicate(ncls, csf), dim = dim(tmp1))
  tmp1 <- csf_expand2 * tmp1

  tmp2 <- log(apply(pirp_trans * (1 - param_expand) * denom0_expand, c(1, 2, 4), sum) + const)
  tmp2 <- aperm(tmp2, perm = c(3, 2, 1))
  nsf_expand2 <- array(replicate(ncls, nsf - csf), dim = dim(tmp2))
  tmp2 <- nsf_expand2 * tmp2

  llsr <- apply(tmp1 + tmp2, c(1, 3), sum)
  minllsr <- apply(llsr, 1, min)
  expllsr <- exp(llsr - minllsr)
  clsmemb <- round(expllsr / rowSums(expllsr), 8)
  cls01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  cls <- apply(clsmemb, 1, which.max)

  StudentRank <- clsmemb
  rownames(StudentRank) <- tmp$ID
  RU <- ifelse(cls + 1 > ncls, NA, cls + 1)
  RD <- ifelse(cls - 1 < 1, NA, cls - 1)
  RUO <- clsmemb[cbind(1:nobs, RU)] / clsmemb[cbind(1:nobs, cls)]
  RDO <- clsmemb[cbind(1:nobs, RD)] / clsmemb[cbind(1:nobs, cls)]
  StudentRank <- cbind(StudentRank, cls, RUO, RDO)
  colnames(StudentRank) <- c(
    paste("Membership", 1:ncls), "Estimate",
    "Rank-Up Odds", "Rank-Down Odds"
  )
  clsdist <- colSums(cls01)
  clsmembdist <- colSums(clsmemb)


  # pifr --------------------------------------------------------------------

  clsmemb_expand <- array(replicate(nfld * Dmax, clsmemb), dim = dim(pirp_array))
  pifr <- param * denom0 * apply(clsmemb_expand * pirp_array, 2:4, sum) / apply(clsmemb, 2, sum)
  pifr <- apply(pifr, c(1, 2), sum)

  # Model Fit ----------------------------------------------------------------

  test_log_lik <- sum(clsmemb * llsr)
  nparam <- sum(denom0)
  FitIndices <- TestFit(tmp$U, tmp$Z, test_log_lik, nparam)

  # Output ------------------------------------------------------------------

  g_list <- list()
  for (i in 1:ncls) {
    g_list[[i]] <- igraph::graph_from_adjacency_matrix(adj_list[[i]])
  }

  CCRR_table <- as.data.frame(matrix(NA, nrow = ncls * nfld, ncol = 2 + Dmax))
  colnames(CCRR_table) <- c(
    "Rank", "Field", paste("PIRP", 0:(Dmax - 1))
  )
  CCRR_table[, 1] <- rep(paste("Rank", 1:ncls), each = nfld)
  CCRR_table[, 2] <- rep(paste("Field", 1:nfld), ncls)
  table_tmp <- param[1, , ]
  for (i in 2:ncls) {
    table_tmp <- rbind(table_tmp, param[i, , ])
  }
  table_tmp[table_tmp == 0] <- NA
  CCRR_table[, 3:NCOL(CCRR_table)] <- table_tmp

  colnames(pifr) <- FieldLabel
  rownames(pifr) <- paste("Rank", 1:NROW(pifr))
  pifr <- t(pifr)

  FRPIndex <- IRPindex(pifr)
  TRP <- as.vector(t(pifr) %*% flddist)
  RMD <- clsmembdist
  TRPlag <- TRP[2:ncls]
  TRPmic <- sum(TRPlag[1:(ncls - 1)] - TRP[1:(ncls - 1)] < 0, na.rm = TRUE)
  FRPmic <- sum(abs(FRPIndex$C))
  SOACflg <- WOACflg <- FALSE
  if (TRPmic == 0) {
    WOACflg <- TRUE
    if (FRPmic == 0) {
      SOACflg <- TRUE
    }
  }

  ret <- structure(list(
    U = U,
    testlength = testlength,
    msg = "Rank",
    nobs = nobs,
    n_rank = ncls,
    n_field = nfld,
    Nrank = ncls,
    Nfield = nfld,
    crr = crr(U),
    ItemLabel = tmp$ItemLabel,
    FieldLabel = FieldLabel,
    adj_list = adj_list,
    g_list = g_list,
    IRP = param,
    LFD = flddist,
    LRD = clsdist,
    FRP = pifr,
    CCRR_table = CCRR_table,
    FRPIndex = FRPIndex,
    TRP = TRP,
    RMD = colSums(clsmemb),
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    TestFitIndices = FitIndices,
    log_lik = test_log_lik,
    SOACflg = SOACflg,
    WOACflg = WOACflg
  ), class = c("exametrika", "LDB"))
  return(ret)
}

library(exametrika)

### GOALS - Mathematica reference data
Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter10LDR_Test.csv"),
  check.names = FALSE
)
CCRR <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter10LDR_CCRR.csv"),
  check.names = FALSE
)
MR <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter10LDR_Marginal_Rankluster.csv"),
  check.names = FALSE
)
Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter10LDR_Item.csv"),
  check.names = FALSE
)
Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter10LDR_Student.csv"),
  check.names = FALSE
)

### Target
fieldFile <- test_path("fixtures", "auxiliary_data", "FixFieldLDB.csv")
ncls <- 5
edgeFile <- test_path("fixtures", "auxiliary_data", "EdgesLDB.csv")
FieldData <- read.csv(fieldFile)
conf <- FieldData[, 2]
adj_file <- edgeFile
tgt <- LDB(U = J35S515, ncls = 5, conf = conf, adj_file = edgeFile)


# test1 Test ------------------------------------------------------

test_that("Test Info", {
  expect <- Test[15:30, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- tgt$TestFitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

# CCRR ------------------------------------------------------------

test_that("Conditional Correct Response Rate", {
  expect <- CCRR[-c(11, 22, 33, 44, 55), 2:14] |>
    unlist() |>
    na.omit() |>
    as.vector() |>
    as.numeric()
  result <- tgt$CCRR_table[, 3:15] |>
    unlist() |>
    na.omit() |>
    as.vector() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Marginal Rankluster ---------------------------------------------

test_that("Marginal Rankluster", {
  expect <- MR[1:10, 2:6] |>
    unlist() |>
    as.vector()
  result <- tgt$FRP |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MR[1:10, 8:13] |>
    unlist() |>
    as.vector()
  result <- tgt$FRPIndex |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MR[11, 2:6] |>
    unlist() |>
    as.vector()
  result <- tgt$TRP |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MR[12, 2:6] |>
    unlist() |>
    as.vector()
  result <- tgt$LRD |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MR[13, 2:6] |>
    unlist() |>
    as.vector()
  result <- tgt$RMD |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Item ------------------------------------------------------------

test_that("Items", {
  expect <- Item$`Latent Field` |>
    unlist() |>
    as.vector()
  result <- tgt$FieldEstimated |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})


# Students ------------------------------------------------------------

test_that("Students", {
  expect <- Student[, 6:13] |>
    unlist() |>
    as.vector()
  result <- tgt$Students |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})

#' @rdname Biclustering_IRM
#' @param mic Logical; if TRUE, forces Field Reference Profiles to be monotonically
#' increasing across classes (ordinal IRM only). Default is TRUE.
#' @return
#' For ordinal data, the returned list includes:
#' \describe{
#'  \item{Q}{Response matrix.}
#'  \item{Z}{Missing indicator matrix.}
#'  \item{testlength}{Number of items.}
#'  \item{nobs}{Sample size.}
#'  \item{n_class}{Optimal number of classes.}
#'  \item{n_field}{Optimal number of fields.}
#'  \item{n_cycle}{Number of EM algorithm iterations.}
#'  \item{FRP}{Field Reference Profile (BCRM), a 3D array (nfld x ncls x maxQ).}
#'  \item{FRPIndex}{Index of FRP includes the item location parameters B and Beta,
#'  the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
#'  \item{TRP}{Test Reference Profile.}
#'  \item{BFRP}{Bicluster Field Reference Profile (expected scores), a list with
#'  Weighted and Observed components.}
#'  \item{LFD}{Latent Field Distribution.}
#'  \item{LCD}{Latent Class Distribution.}
#'  \item{FieldMembership}{Field membership probability matrix.}
#'  \item{ClassMembership}{Class membership probability matrix.}
#'  \item{FieldEstimated}{Estimated field assignment for each item.}
#'  \item{ClassEstimated}{Estimated class assignment for each student.}
#'  \item{Students}{Rank Membership Profile matrix with estimated class.}
#'  \item{TestFitIndices}{Overall fit index for the test.}
#'  \item{log_lik}{Log-likelihood of the model.}
#'  \item{SOACflg}{Logical; TRUE if Strongly Ordinal Alignment Condition is satisfied.}
#'  \item{WOACflg}{Logical; TRUE if Weakly Ordinal Alignment Condition is satisfied.}
#' }
#' @examples
#' \donttest{
#' # Fit an ordinal Biclustering IRM model
#' result <- Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, verbose = TRUE)
#' plot(result, type = "Array")
#' }
#' @export
Biclustering_IRM.ordinal <- function(U,
                                     gamma_c = 1, gamma_f = 1, alpha = 1,
                                     mic = TRUE,
                                     max_iter = 100, stable_limit = 5,
                                     minSize = 20, EM_limit = 100,
                                     seed = 123, verbose = TRUE, ...) {
  tmp <- U

  nitems <- NCOL(tmp$Q)
  nobs <- NROW(tmp$Q)
  const <- exp(-nitems)
  maxQ <- max(tmp$categories)
  alpha_vec <- rep(alpha, maxQ)

  if (alpha <= 0) {
    stop("alpha must be positive (alpha > 0)")
  }

  # One-hot encoding of response matrix: Uq[s, j, q] = 1 if student s chose category q on item j
  Uq <- array(0, dim = c(nobs, nitems, maxQ))
  for (s in 1:nobs) {
    for (j in 1:nitems) {
      Uq[s, j, tmp$Q[s, j]] <- 1
    }
  }

  # Initialize -------------------------------------------------------------
  if (!is.null(seed)) {
    set.seed(seed)
  }

  ## Initial Class: assign by mean response score
  means <- apply(tmp$Q, 1, mean)
  unique_vec <- unique(sort(means))
  ncls <- length(unique_vec)
  cls01 <- matrix(0, ncol = ncls, nrow = nobs)
  for (i in 1:nobs) {
    cls01[i, which(unique_vec == means[i])] <- 1
  }
  colnames(cls01) <- paste("Rank", 1:ncls)
  rownames(cls01) <- tmp$ID

  ## Initial Field: one item per field
  nfld <- nitems
  fld01 <- matrix(0, nrow = nitems, ncol = nfld)
  for (i in 1:nitems) {
    fld01[i, i] <- 1
  }

  # Phase 1: Collapsed Gibbs Sampler (shared core) --------------------------
  gibbs <- irm_gibbs_core(
    Uq = Uq, Z = tmp$Z, cls01 = cls01, fld01 = fld01,
    gamma_c = gamma_c, gamma_f = gamma_f, alpha_vec = alpha_vec,
    max_iter = max_iter, stable_limit = stable_limit, verbose = verbose
  )

  cls01 <- gibbs$cls01
  fld01 <- gibbs$fld01
  ncls <- gibbs$ncls
  nfld <- gibbs$nfld

  # Phase 2: Initial BCRM/BBRM estimation ----------------------------------
  U_fcq <- irm_calc_Ufcq(Uq, tmp$Z, cls01, fld01, maxQ)
  BCRM <- sweep(U_fcq, 3, alpha_vec, "+")
  denom <- apply(BCRM, c(1, 2), sum)
  BCRM <- sweep(BCRM, c(1, 2), denom, "/")

  # Bicluster Boundary Reference Matrix (FxCxQ+1)
  BBRM <- array(NA, dim = c(nfld, ncls, maxQ + 1))
  BBRM[, , 1] <- 1
  BBRM[, , maxQ + 1] <- 0
  for (q in maxQ:2) {
    BBRM[, , q] <- BCRM[, , q] + BBRM[, , q + 1]
  }

  # Phase 3: EM refinement with ordering + small-class adjustment -----------
  bic <- irm_bic_calc(Uq, tmp$Z, fld01, cls01, BCRM, maxQ, const)
  n_cycle <- 0

  ConstFLG <- TRUE
  while (ConstFLG) {
    EMt <- 1
    EMrepFLG <- TRUE
    bestfit <- 10^10

    while (EMrepFLG) {
      # E-step
      V_scq <- array(0, dim = c(nobs, ncls, maxQ))
      for (q in 1:maxQ) {
        V_scq[, , q] <- (Uq[, , q] %*% fld01) %*% log(BCRM[, , q])
      }
      log_S <- apply(V_scq, c(1, 2), sum)
      prob_S <- matrix(0, nrow = nobs, ncol = ncls)
      for (i in seq_len(nobs)) {
        prob_S[i, ] <- irm_log_to_prob(log_S[i, ])
      }
      cls <- apply(prob_S, 1, which.max)
      cls01 <- matrix(0, ncol = ncls, nrow = nobs)
      for (i in 1:nobs) {
        cls01[i, cls[i]] <- 1
      }
      U_fcq <- irm_calc_Ufcq(Uq, tmp$Z, cls01, fld01, maxQ)

      # M-step: cumulative normalization (ordinal)
      Ufcq_prior <- U_fcq + alpha - 1
      Ufcq_prior <- pmax(Ufcq_prior, 1e-10)
      cUfcq <- array(0, dim = c(nfld, ncls, maxQ))
      for (f in seq_len(nfld)) {
        for (cc in seq_len(ncls)) {
          cUfcq[f, cc, ] <- rev(cumsum(rev(Ufcq_prior[f, cc, ])))
        }
      }

      for (q in 1:maxQ) {
        BBRM[, , q] <- cUfcq[, , q] / cUfcq[, , 1]
      }

      # Forced ordering (monotonicity constraint)
      if (mic) {
        overall_order <- array(0, dim = ncls)
        for (i in 1:ncls) {
          total_expected <- 0
          for (j in 1:nfld) {
            field_expected <- sum(BBRM[j, i, 1:maxQ])
            total_expected <- total_expected + field_expected
          }
          overall_order[i] <- total_expected
        }
        overall_order <- order(overall_order)
        BBRM <- BBRM[, overall_order, ]
      }
      for (q in 1:maxQ) {
        BCRM[, , q] <- BBRM[, , q] - BBRM[, , q + 1]
      }

      # Model fit
      bic <- irm_bic_calc(Uq, tmp$Z, fld01, cls01, BCRM, maxQ, const)

      # Converge check
      if (bic < bestfit) {
        EMt <- EMt + 1
        bestfit <- bic
      } else {
        EMrepFLG <- FALSE
      }

      if (EMt >= EM_limit) {
        EMrepFLG <- FALSE
      }
    }

    n_cycle <- n_cycle + EMt

    # Small-class adjustment
    AdjustFLG <- FALSE
    Nc <- colSums(cls01)
    NcTable <- matrix(Nc)
    minclass <- NcTable[order(NcTable[, 1]), ]
    if (minclass[1] < minSize) {
      AdjustFLG <- TRUE
      if (verbose) {
        message(
          sprintf(
            "Adjusting classes: BIC=%.1f ncls=%d (min size < %d)",
            bic, ncls, minSize
          )
        )
      }
      ncls <- ncls - 1
      del_class <- which.min(NcTable)
      BCRM <- BCRM[, -del_class, , drop = FALSE]
      # Rebuild BBRM from BCRM
      BBRM <- array(NA, dim = c(nfld, ncls, maxQ + 1))
      BBRM[, , 1] <- 1
      BBRM[, , maxQ + 1] <- 0
      for (q in maxQ:2) {
        BBRM[, , q] <- BCRM[, , q] + BBRM[, , q + 1]
      }
      cls01 <- cls01[, -del_class, drop = FALSE]
    }

    if (!AdjustFLG) {
      ConstFLG <- FALSE
    }
  }

  # Output -----------------------------------------------------------------

  ## Class membership
  V_scq <- array(0, dim = c(nobs, ncls, maxQ))
  for (q in 1:maxQ) {
    V_scq[, , q] <- (Uq[, , q] %*% fld01) %*% log(BCRM[, , q])
  }
  log_S <- apply(V_scq, c(1, 2), sum)
  clsmemb <- matrix(0, nrow = nobs, ncol = ncls)
  for (i in seq_len(nobs)) {
    clsmemb[i, ] <- irm_log_to_prob(log_S[i, ])
  }
  cls <- apply(clsmemb, 1, which.max)
  cls01 <- matrix(0, ncol = ncls, nrow = nobs)
  for (i in 1:nobs) {
    cls01[i, cls[i]] <- 1
  }

  ## Field membership
  V_jfq <- array(0, dim = c(nitems, nfld, maxQ))
  for (q in 1:maxQ) {
    V_jfq[, , q] <- (t(Uq[, , q]) %*% cls01) %*% log(t(BCRM[, , q]))
  }
  log_J <- apply(V_jfq, c(1, 2), sum)
  fldmemb <- matrix(0, nrow = nitems, ncol = nfld)
  for (i in seq_len(nitems)) {
    fldmemb[i, ] <- irm_log_to_prob(log_J[i, ])
  }
  fld <- apply(fldmemb, 1, which.max)
  check_empty_fields(fld, nfld)
  fld01 <- matrix(0, ncol = nfld, nrow = nitems)
  for (i in 1:nitems) {
    fld01[i, fld[i]] <- 1
  }

  fldmemb01 <- sign(fldmemb - apply(fldmemb, 1, max)) + 1
  flddist <- colSums(fldmemb01)
  clsmemb01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  clsdist <- colSums(clsmemb01)
  StudentRank <- cbind(clsmemb, Estimate = cls)
  rownames(StudentRank) <- tmp$ID
  colnames(StudentRank) <- c(paste("Membership", 1:ncls), "Estimate")

  ## Expected score
  BFRP1 <- BFRP2 <- matrix(0, nrow = nfld, ncol = ncls)
  weights <- matrix(0, nrow = nfld, ncol = ncls)

  for (q in 1:maxQ) {
    contrib <- (t(fldmemb) %*% t(tmp$Z * Uq[, , q]) %*% clsmemb) * (BCRM[, , q])
    BFRP1 <- BFRP1 + q * contrib
    weights <- weights + contrib
  }
  BFRP1 <- BFRP1 / weights

  for (i in 1:ncls) {
    for (j in 1:nfld) {
      BFRP2[j, i] <- mean(tmp$Z[cls == i, fld == j] * tmp$Q[cls == i, fld == j])
    }
  }

  TRP <- colSums(BFRP1)
  TRPlag <- TRP[2:ncls]
  TRPmic <- sum(TRPlag[1:(ncls - 1)] - TRP[1:(ncls - 1)] < 0, na.rm = TRUE)

  ## FRPIndex: normalized expected score -> IRPindex
  model_esp <- matrix(0, nrow = nfld, ncol = ncls)
  for (f in 1:nfld) {
    for (cc in 1:ncls) {
      model_esp[f, cc] <- sum((1:maxQ) * BCRM[f, cc, ])
    }
  }
  norm_frp <- (model_esp - 1) / (maxQ - 1)
  FRPIndex <- IRPindex(norm_frp)
  FRPmic <- sum(abs(FRPIndex$C))

  SOACflg <- WOACflg <- FALSE
  if (TRPmic == 0) {
    WOACflg <- TRUE
    if (FRPmic == 0) {
      SOACflg <- TRUE
    }
  }
  if (verbose) {
    if (SOACflg & WOACflg) {
      message("Strongly ordinal alignment condition was satisfied.")
    }
    if (!SOACflg & WOACflg) {
      message("Weakly ordinal alignment condition was satisfied.")
    }
  }

  # Model fit ---------------------------------------------------------------
  testell <- 0
  for (q in 1:maxQ) {
    pred_prob <- t(fldmemb %*% BCRM[, , q] %*% t(clsmemb))
    observed_mask <- (tmp$Z * Uq[, , q]) == 1
    testell <- testell + sum(log(pmax(pred_prob[observed_mask], const)))
  }
  nparam <- ncls * nfld * (maxQ - 1)

  # Full model
  ptn <- apply(tmp$Q, 1, function(x) paste(x, collapse = ""))
  benchG <- as.numeric(as.factor(ptn))
  fullG <- length(unique(benchG))
  benchmemb <- matrix(0, nrow = nobs, ncol = fullG)
  for (i in 1:nobs) {
    benchmemb[i, benchG[i]] <- 1
  }

  BenchFRQ <- array(NA, dim = c(nitems, fullG, maxQ))
  Bfcq <- array(0, dim = c(nitems, fullG, maxQ))
  for (q in 1:maxQ) {
    Bfcq[, , q] <- (t(tmp$Z * Uq[, , q])) %*% benchmemb
  }

  BenchFRQ <- Bfcq / array(apply(Bfcq, c(1, 2), sum), dim = dim(BenchFRQ))
  BenchFRQ[is.nan(BenchFRQ)] <- const

  ell_B <- 0
  for (q in 1:maxQ) {
    ell_B <- ell_B + sum(t(tmp$Z * Uq[, , q]) %*% benchmemb * log(BenchFRQ[, , q] + const))
  }
  bench_nparam <- nitems * fullG

  # Null model
  Zrep <- replicate(maxQ, tmp$Z)
  NullFRQ <- apply(Zrep * Uq, c(2, 3), sum) / apply(tmp$Z, 2, sum)
  ell_N <- sum(apply(Zrep * Uq, c(2, 3), sum) * log(NullFRQ + const))
  null_nparam <- nitems

  df_B <- bench_nparam - null_nparam
  chi_B <- 2 * (ell_B - ell_N)
  chi_A <- 2 * (ell_B - testell)
  df_A <- bench_nparam - nparam
  FitIndices <- structure(
    c(list(
      model_log_like = testell,
      bench_log_like = ell_B,
      null_log_like = ell_N,
      model_Chi_sq = chi_A,
      null_Chi_sq = chi_B,
      model_df = df_A,
      null_df = df_B
    ), calcFitIndices(chi_A, chi_B, df_A, df_B, nobs)),
    class = c("exametrika", "ModelFit")
  )

  # Return ------------------------------------------------------------------
  msg <- "Rank"
  ret <- structure(list(
    Q = tmp$Q,
    Z = tmp$Z,
    testlength = nitems,
    msg = msg,
    mic = mic,
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    n_cycle = n_cycle,
    Nclass = ncls,
    Nfield = nfld,
    N_Cycle = n_cycle,
    LFD = flddist,
    LRD = clsdist,
    LCD = clsdist,
    FRP = BCRM,
    FRPIndex = FRPIndex,
    TRP = TRP,
    CMD = colSums(clsmemb),
    RMD = colSums(clsmemb),
    FieldMembership = fldmemb,
    ClassMembership = clsmemb,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = StudentRank,
    BFRP = list(Weighted = BFRP1, Observed = BFRP2),
    TestFitIndices = FitIndices,
    log_lik = testell,
    SOACflg = SOACflg,
    WOACflg = WOACflg,
    # Deprecated fields (for backward compatibility)
    LogLik = testell
  ), class = c("exametrika", "ordinalBiclustering"))

  return(ret)
}

#' @title Simple Item Statistics
#' @description
#' This function calculates statistics for each item, with different metrics available
#' depending on the data type (binary, ordinal, or rated).
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class with the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
#'   observed responses, while 0 indicates missing data.
#' @param w Item weight vector specifying the relative importance of each item.
#' @param na Values to be treated as missing values.
#'
#' @return
#' For binary data:
#' \describe{
#' \item{ItemLabel}{Label identifying each item}
#' \item{NR}{Number of Respondents for each item}
#' \item{CRR}{Correct Response Rate denoted as $p_j$.}
#' \item{ODDs}{Item Odds is the ratio of the correct response rate to the incorrect response rate.
#' Defined as \eqn{o_j = \frac{p_j}{1-p_j}}}
#' \item{Threshold}{Item Threshold is a measure of difficulty based on a standard normal distribution.}
#' \item{Entropy}{Item Entropy is an indicator of the variability or randomness of the responses.
#' Defined as \eqn{e_j=-p_j \log_2 p_j - (1-p_j)\log_2(1-p_j)}}
#' \item{ITCrr}{Item-total Correlation is a Pearson's correlation of an item with the Number-Right score.}
#' }
#'
#' For ordinal polytomous data:
#' \describe{
#' \item{ItemLabel}{Label identifying each item}
#' \item{NR}{Number of Respondents for each item}
#' \item{Threshold}{Matrix of threshold values for each item's category boundaries, based on
#' a standard normal distribution. For an item with K categories, there are K-1 thresholds.}
#' \item{Entropy}{Item Entropy calculated using the category probabilities. Unlike binary data,
#' this is calculated using the formula \eqn{e_j = -\sum_{k=1}^{K_j} p_{jk} \log_{K_j} p_{jk}},
#' where \eqn{K_j} is the number of categories for item j.}
#' \item{ITCrr}{Item-total Correlation calculated using polyserial correlation, which
#' accounts for the ordinal nature of the item responses and the continuous total score.}
#' }
#'
#' @note For rated data, the function processes the data as binary, with each response
#' being compared to the correct answer to determine correctness.
#'
#' @examples
#' # using sample dataset(binary)
#' ItemStatistics(J15S500)
#' @export

ItemStatistics <- function(U, na = NULL, Z = NULL, w = NULL) {
  UseMethod("ItemStatistics")
}

#' @rdname ItemStatistics
#' @export

ItemStatistics.default <- function(U, na = NULL, Z = NULL, w = NULL) {
  if (inherits(U, "exametrika")) {
    switch(U$response.type,
      "binary" = ItemStatistics.binary(U, na, Z, w),
      "rated" = ItemStatistics.ordinal(U, na, Z, w),
      "ordinal" = ItemStatistics.ordinal(U, na, Z, w),
      "nominal" = response_type_error(U$response.type, "ItemStatistics")
    )
  } else {
    U <- dataFormat(U, na = na, Z = Z, w = w)
    ItemStatistics(U)
  }
}

#' @rdname ItemStatistics
#' @export
ItemStatistics.binary <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  ret <-
    structure(list(
      ItemLabel = tmp$ItemLabel,
      NR = colSums(tmp$Z),
      CRR = crr(tmp),
      ODDs = ItemOdds(tmp),
      Threshold = ItemThreshold(tmp),
      Entropy = ItemEntropy(tmp),
      ITCrr = ItemTotalCorr(tmp)
    ), class = c("exametrika", "ItemStatistics"))
  return(ret)
}

#' @rdname ItemStatistics
#' @export
ItemStatistics.ordinal <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  Threshold <- ItemThreshold(tmp)
  ncat <- sapply(Threshold, length)
  thr.mat <- matrix(nrow = length(Threshold), ncol = max(ncat))
  colnames(thr.mat) <- 1:max(ncat)
  for (j in 1:length(Threshold)) {
    thr.mat[j, 1:ncat[j]] <- Threshold[[j]]
  }

  ret <-
    structure(list(
      ItemLabel = tmp$ItemLabel,
      NR = colSums(tmp$Z),
      Threshold = thr.mat,
      Entropy = ItemEntropy(tmp),
      ITCrr = ItemTotalCorr(tmp)
    ), class = c("exametrika", "ItemStatistics"))
  return(ret)
}

#' @title Bicluster Network Model
#' @description
#' Bicluster Network Model: BINET is a model that combines the Bayesian
#' network model and Biclustering. BINET is very similar to LDB and LDR.
#' The most significant difference is that in LDB, the nodes represent
#' the fields, whereas in BINET, they represent the class. BINET
#' explores the local dependency structure among latent classes at each
#' latent field, where each field is a locus.
#' @param U U is either a data class of exametrika, or raw data. When raw data is given,
#' it is converted to the exametrika class with the [dataFormat] function.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#' @param ncls number of classes
#' @param nfld number of fields
#' @param conf For the confirmatory parameter, you can input either a vector with
#' items and corresponding fields in sequence, or a field membership profile
#' matrix. In the case of the former, the field membership profile matrix will be generated internally.
#' When providing a membership profile matrix, it needs to be either matrix or data.frame.
#' The number of fields(nfld) will be overwrite to the number of columns of this matrix.
#' @param g_list A list compiling graph-type objects for each rank/class.
#' @param adj_list A list compiling matrix-type adjacency matrices for each rank/class.
#' @param adj_file A file detailing the relationships of the graph for each rank/class,
#' listed in the order of starting point, ending point, and rank(class).
#' @param verbose verbose output Flag. default is TRUE
#' @return
#' \describe{
#'  \item{nobs}{Sample size. The number of rows in the dataset.}
#'  \item{msg}{A character string indicating the model type. }
#'  \item{testlength}{Length of the test. The number of items included in the test.}
#'  \item{Nclass}{Optimal number of classes.}
#'  \item{Nfield}{Optimal number of fields.}
#'  \item{crr}{Correct Response Rate}
#'  \item{ItemLabel}{Label of Items}
#'  \item{FieldLabel}{Label of Fields}
#'  \item{all_adj}{Integrated Adjacency matrix used to plot graph.}
#'  \item{all_g}{Integrated graph object used to plot graph.see also
#'  [plot.exametrika]}
#'  \item{adj_list}{List of Adjacency matrix used in the model}
#'  \item{params}{A list of the estimated conditional probabilities.
#'  It indicates which path was obtained from which parent node(class) to
#'  which child node(class), held by `parent`, `child`, and `field`. The item
#'  Items contained in the field is in `fld`. Named `chap` includes the
#'  conditional correct response answer rate of the child node, while `pap`
#'  contains the pass rate of the parent node.}
#'  \item{PSRP}{Response pattern by the students belonging to the parent
#'  classes of Class c. A more comprehensible arrangement of `params.`}
#'  \item{LCD}{Latent Class Distribution. see also [plot.exametrika]}
#'  \item{LFD}{Latent Field Distribution. see also [plot.exametrika]}
#'  \item{CMD}{Class Membership Distribution.}
#'  \item{FRP}{Marginal bicluster reference matrix.}
#'  \item{FRPIndex}{Index of FFP includes the item location parameters B and Beta,
#'  the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
#'  \item{TRP}{Test Reference Profile}
#'  \item{LDPSR}{A rearranged set of parameters for output. It includes
#'  the field the items contained within that field, and the conditional
#'  correct response rate of parent nodes(class) and child node(class).}
#'  \item{FieldEstimated}{Given vector which correspondence between items
#'  and the fields.}
#'  \item{Students}{Rank Membership Profile matrix.The s-th row vector of \eqn{\hat{M}_R}, \eqn{\hat{m}_R}, is the
#' rank membership profile of Student s, namely the posterior probability distribution representing the student's
#' belonging to the respective latent classes. }
#' \item{NextStage}{The next class that easiest for students to move to,
#' its membership probability, class-up odds, and the field required for
#' more.}
#'  \item{MG_FitIndices}{Multigroup as Null model.See also [TestFit]}
#'  \item{SM_FitIndices}{Saturated Model as Null model.See also [TestFit]}
#' }
#' @examples
#' \donttest{
#' # Example: Bicluster Network Model (BINET)
#' # BINET combines Bayesian network model and Biclustering to explore
#' # local dependency structure among latent classes at each field
#'
#' # Create field configuration vector based on field assignments
#' conf <- c(
#'   1, 5, 5, 5, 9, 9, 6, 6, 6, 6, 2, 7, 7, 11, 11, 7, 7,
#'   12, 12, 12, 2, 2, 3, 3, 4, 4, 4, 8, 8, 12, 1, 1, 6, 10, 10
#' )
#'
#' # Create edge data for network structure between classes
#' edges_data <- data.frame(
#'   "From Class (Parent) >>>" = c(
#'     1, 2, 3, 4, 5, 7, 2, 4, 6, 8, 10, 6, 6, 11, 8, 9, 12
#'   ),
#'   ">>> To Class (Child)" = c(
#'     2, 4, 5, 5, 6, 11, 3, 7, 9, 12, 12, 10, 8, 12, 12, 11, 13
#'   ),
#'   "At Field (Locus)" = c(
#'     1, 2, 2, 3, 4, 4, 5, 5, 5, 5, 5, 7, 8, 8, 9, 9, 12
#'   )
#' )
#'
#' # Save edge data to temporary CSV file
#' tmp_file <- tempfile(fileext = ".csv")
#' write.csv(edges_data, file = tmp_file, row.names = FALSE)
#'
#' # Fit Bicluster Network Model
#' result.BINET <- BINET(
#'   J35S515,
#'   ncls = 13, # Maximum class number from edges (13)
#'   nfld = 12, # Maximum field number from conf (12)
#'   conf = conf, # Field configuration vector
#'   adj_file = tmp_file # Path to the CSV file
#' )
#'
#' # Clean up temporary file
#' unlink(tmp_file)
#'
#' # Display model results
#' print(result.BINET)
#'
#' # Visualize different aspects of the model
#' plot(result.BINET, type = "Array") # Show bicluster structure
#' plot(result.BINET, type = "TRP") # Test Response Profile
#' plot(result.BINET, type = "LRD") # Latent Rank Distribution
#' plot(result.BINET,
#'   type = "RMP", # Rank Membership Profiles
#'   students = 1:9, nc = 3, nr = 3
#' )
#' plot(result.BINET,
#'   type = "FRP", # Field Reference Profiles
#'   nc = 3, nr = 2
#' )
#' plot(result.BINET,
#'   type = "LDPSR", # Locally Dependent Passing Student Rates
#'   nc = 3, nr = 2
#' )
#' }
#' @export

BINET <- function(U, Z = NULL, w = NULL, na = NULL,
                  conf = NULL, ncls = NULL, nfld = NULL,
                  g_list = NULL, adj_list = NULL, adj_file = NULL,
                  verbose = FALSE) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (U$response.type != "binary") {
    response_type_error(U$response.type, "BINET")
  }

  U <- tmp$U * tmp$Z
  testlength <- NCOL(tmp$U)
  nobs <- NROW(tmp$U)

  if (is.null(ncls)) {
    stop("Please specify the appropriate number of classes")
  }
  if (is.null(nfld)) {
    stop("Please specify the appropriate number of fields.")
  }

  if (ncls < 2 | ncls > 20) {
    stop("Please set the number of classes to a number between 2 and less than 20.")
  }

  ret.TS <- TestStatistics(tmp)
  # Check the settings ------------------------------------------------------
  # Check the conf
  if (is.vector(conf)) {
    # check size
    if (length(conf) != NCOL(U)) {
      stop("conf vector size does NOT match with data.")
    }
    conf_mat <- matrix(0, nrow = NCOL(U), ncol = max(conf))
    for (i in 1:NROW(conf_mat)) {
      conf_mat[i, conf[i]] <- 1
    }
  } else if (is.matrix(conf) | is.data.frame(conf)) {
    if (NROW(conf) != NCOL(U)) {
      stop("conf matrix size does NOT match with data.")
    }
    if (any(!conf %in% c(0, 1))) {
      stop("The conf matrix should only contain 0s and 1s.")
    }
    if (any(rowSums(conf) > 1)) {
      stop("The row sums of the conf matrix must be equal to 1.")
    }
  }

  # graph check
  if (is.null(g_list) && is.null(adj_list) && is.null(adj_file)) {
    stop("Specify the graph in either matrix form, CSV file, or as a graph object.")
  }

  FieldLabel <- sprintf("Class%02d", 1:ncls)

  # g_list check
  if (!is.null(g_list)) {
    if (length(g_list) != nfld) {
      stop("The number of fields does not match the length of g_list.
           Please specify a graph for all fields.")
    }
    adj_list <- list()
    for (j in 1:nfld) {
      if (!inherits(g_list[[j]], "igraph")) {
        stop("Some items in g_list are not recognized as graph objects.")
      }
      adj_list[[j]] <- fill_adj(g_list[[j]], FieldLabel)
    }
  }
  # adj_list check
  if (!is.null(adj_list)) {
    if (length(adj_list) != nfld) {
      stop("The number of fields does not match the length of adj_list.
           Please specify an adjacency matrix for all fields.")
    }
    for (j in 1:nfld) {
      g <- igraph::graph_from_adjacency_matrix(adj_list[[j]])
      adj_list[[j]] <- fill_adj(g, FieldLabel)
    }
  }

  # adj_file check
  if (!is.null(adj_file)) {
    g_csv <- read.csv(adj_file)
    colnames(g_csv) <- c("From", "To", "Field")
    g_csv$From <- sprintf("Class%02d", g_csv$From)
    g_csv$To <- sprintf("Class%02d", g_csv$To)
    for (i in 1:nfld) {
      adj_R <- g_csv[g_csv$Field == i, 1:2]
      g_tmp <- igraph::graph_from_data_frame(adj_R)
      adj_list[[i]] <- fill_adj(g_tmp, FieldLabel)
    }
    g_csv$Field <- sprintf("Field%02d", g_csv$Field)
  }

  # Construct g_csv from adj_list for g_list/adj_list input paths
  if (is.null(adj_file)) {
    edge_rows <- list()
    for (k in seq_len(nfld)) {
      edges <- which(adj_list[[k]] == 1, arr.ind = TRUE)
      if (nrow(edges) > 0) {
        edge_rows[[length(edge_rows) + 1]] <- data.frame(
          From = FieldLabel[edges[, 1]],
          To = FieldLabel[edges[, 2]],
          Field = sprintf("Field%02d", k),
          stringsAsFactors = FALSE
        )
      }
    }
    if (length(edge_rows) > 0) {
      g_csv <- do.call(rbind, edge_rows)
    } else {
      g_csv <- data.frame(
        From = character(0),
        To = character(0),
        Field = character(0),
        stringsAsFactors = FALSE
      )
    }
  }

  all_adj <- Reduce("|", adj_list) * 1

  # get the Biclustering structure
  ret.Biclustering <- Biclustering(
    tmp,
    ncls = ncls,
    nfld = nfld,
    conf = conf,
    method = "BINET",
    verbose = FALSE
  )

  ### Adj mat check
  adjU <- all_adj + t(all_adj)
  simpleFLG <- ifelse(max(adjU) <= 1, 1, 0)
  acyclicFLG <- 0
  connectedFLG <- 0
  for (i in 1:(nfld - 1)) {
    acyclicFLG <- acyclicFLG + sum(diag(all_adj^i))
    connectedFLG <- connectedFLG + min(sum(U^i))
  }
  acyclicFLG <- ifelse(acyclicFLG == 0, 1, 0)
  connectedFLG <- ifelse(connectedFLG > 0, 1, 0)
  dag <- simpleFLG * acyclicFLG
  cdag <- dag * connectedFLG

  if (cdag + dag == 0) {
    stop("Your graph is not a DAG")
  }

  ### check the max parent
  maxnpa <- 2
  add_mat <- Reduce("+", adj_list)
  maxna_position <- which(add_mat == max(add_mat), arr.ind = TRUE)
  maxadj <- max(add_mat)
  if (maxadj > maxnpa) {
    warning(
      "[Caution!] The maximum number of fields per edge is ",
      maxnpa, ". Please check the edge from Class ",
      maxna_position[1], " to Class ", maxna_position[2]
    )
  }

  ### Estimation of parameter set
  flddist <- colSums(ret.Biclustering$FieldMembership)
  clsmemb <- ret.Biclustering$ClassMembership
  fldmemb <- ret.Biclustering$FieldMembership
  fld <- ret.Biclustering$FieldEstimated
  cls <- ret.Biclustering$ClassEstimated

  # Estimation for BINET -------------------------------------------------------
  gamp <- 1
  const <- 1e-10
  lls <- 0

  irp <- t(t(clsmemb) %*% tmp$U / colSums(clsmemb))
  Ccj <- t(clsmemb) %*% tmp$U
  Fcj <- t(clsmemb) %*% (tmp$Z * (1 - tmp$U))
  Ncj <- Ccj + Fcj
  Ccf <- Ccj %*% fldmemb
  Fcf <- Fcj %*% fldmemb
  Ncf <- Ccf + Fcf
  Pcf <- (Ccf + gamp - 1) / (Ncf + 2 * gamp - 2)
  Pcf[1, ] <- 0
  Pcf[NROW(Pcf), ] <- 1
  Pcf01 <- matrix(0, ncol = nfld, nrow = ncls)
  Pcf2 <- matrix(list(), nrow = ncls, ncol = nfld)
  Pcj <- matrix(0, nrow = ncls, ncol = testlength)

  param <- list()
  row <- 0

  ### RISP
  for (i in 1:ncls) {
    ## i is parent
    for (j in 1:ncls) {
      ## j is childen
      for (k in 1:nfld) {
        ## k is field
        if (adj_list[[k]][i, j] == 1) {
          Pcf01[j, k] <- 1
          row <- row + 1
          fld_item <- which(fldmemb[, k] == 1)
          paC <- Ccj[i, fld_item]
          paN <- Ncj[i, fld_item]
          pap <- (paC + gamp - 1) / (paN + 2 * gamp - 2)
          if (i == 1) {
            pap <- rep(0, length(pap))
          }
          chC <- Ccj[j, fld_item]
          chN <- Ncj[j, fld_item]
          chp <- (chC + gamp - 1) / (chN + 2 * gamp - 2)
          if (j == ncls) {
            chp <- rep(1, length(chp))
          }
          param[[row]] <- list(
            parent = i,
            child = j,
            field = k,
            fld = fld_item,
            pap = pap,
            chap = chp
          )
          Pcf2[j, k] <- list(chp)
        }
      }
    }
  }

  for (i in 1:nrow(Pcf2)) {
    for (j in 1:ncol(Pcf2)) {
      if (is.numeric(Pcf2[[i, j]][[1]])) {
        Pcf2[[i, j]] <- (1 - Pcf01[i, j]) * Pcf[i, j] + Pcf2[i, j][[1]]
      } else {
        if (is.null(Pcf2[[i, j]])[[1]]) {
          Pcf2[[i, j]] <- (1 - Pcf01[i, j]) * Pcf[i, j]
        }
      }
    }
  }

  for (i in 1:ncls) {
    for (j in 1:nfld) {
      fld_item <- which(fld == j)
      if (Pcf01[i, j] == 1) {
        for (k in 1:length(fld_item)) {
          tg <- fld_item[k]
          Pcj[i, tg] <- Pcf2[i, j][[1]][k]
        }
      } else {
        for (k in fld_item) {
          Pcj[i, k] <- Pcf[i, j]
        }
      }
    }
  }

  log_num_Zic <- matrix(NA, nrow = nobs, ncol = ncls)
  log_num_Zic <- tmp$U %*% log(t(Pcj) + const) +
    (tmp$Z * (1 - tmp$U)) %*% log(t(1 - Pcj) + const)
  num_Zic <- exp(log_num_Zic)
  num_Zic[, 1] <- 0
  num_Zic[, ncls] <- 0
  for (s in 1:nobs) {
    if (cls[s] == 1) {
      num_Zic[s, 1] <- 1
    }
    if (cls[s] == ncls) {
      num_Zic[s, ncls] <- 1
    }
  }

  clsmemb <- num_Zic / rowSums(num_Zic)
  cls <- apply(clsmemb, 1, which.max)
  cls01 <- sign(clsmemb - apply(clsmemb, 1, max)) + 1
  Ccj <- t(clsmemb) %*% tmp$U
  Fcj <- t(clsmemb) %*% (tmp$Z * (1 - tmp$U))
  Ncj <- Ccj + Fcj
  llm <- sum(Ccj * log(Pcj + const) + Fcj * log(1 - Pcj + const))
  chim <- 2 * (lls - llm)
  df <- nobs * testlength - sum(sign(sign(unlist(Pcf2)) + 1))
  aic <- chim - 2 * df
  bic <- chim - df * log(nobs)
  indices <- list(llm = llm, chim = chim, df = df, aic = aic, bic = bic)

  # For output  --------------------------------------------------

  g_list <- list()
  for (i in 1:nfld) {
    g_list[[i]] <- igraph::graph_from_adjacency_matrix(adj_list[[i]])
  }
  all_g <- igraph::graph_from_data_frame(g_csv, directed = TRUE)

  ## Students Info
  next_stage <- matrix(NA, ncol = 3 + maxadj, nrow = nobs)
  colnames(next_stage) <-
    c("NextClass", "Membership", "Class-Up Odds", paste("Recommended Field", 1:maxadj))
  for (s in 1:nobs) {
    path_way <- lapply(adj_list, function(mat) {
      return(which(mat[cls[s], ] == 1))
    })
    path_way <- as.vector(unlist(path_way))
    if (length(path_way) == 0) {
      path_way <- NA
    } else {
      mat <- cbind(
        ns = path_way,
        prob = clsmemb[s, path_way]
      )
      sort_order <- order(mat[, 2], decreasing = TRUE)
      if (length(path_way) == 1) {
        candidate <- mat[1, ]
      } else {
        candidate <- mat[sort_order, ][1, ]
      }
      odds <- candidate[2] / clsmemb[s, cls[s]]
      recommend <- which(sapply(adj_list, function(mat) mat[cls[s], candidate[1]] == 1))
      if (length(recommend) < maxadj) {
        recommend <- c(recommend, rep(NA, maxadj - length(recommend)))
      }
      next_stage[s, ] <- c(candidate, odds, recommend)
    }
  }
  Students <- cbind(clsmemb, cls)
  NextStage <- next_stage
  rownames(Students) <- tmp$ID
  rownames(NextStage) <- tmp$ID
  colnames(Students) <- c(
    paste("Membership", 1:ncls), "Latent Class"
  )
  colnames(NextStage) <- c(
    "Next Class", "Membership in Next Class",
    "Class-up odds", paste("Recommmended Field", 1:maxadj)
  )
  clsdist <- colSums(cls01)
  clsmembdist <- colSums(clsmemb)

  ## PSRP matrix
  Dmax <- max(sapply(param, function(x) length(x$pap)))
  if (Dmax < max(sapply(param, function(x) length(x$chap)))) {
    Dmax <- max(sapply(param, function(x) length(x$chap)))
  }
  PSRP <- list()
  PSRP_tmp <- matrix(ncol = Dmax, nrow = ncls)
  for (i in 1:nfld) {
    for (j in 1:ncls) {
      vec <- Pcf2[j, i][[1]]
      if (length(vec) < Dmax) {
        vec <- c(vec, rep(NA, Dmax - length(vec)))
      }
      PSRP_tmp[j, ] <- vec
    }
    rownames(PSRP_tmp) <- paste("Class", 1:ncls)
    colnames(PSRP_tmp) <- paste("PSRP", 1:Dmax)
    PSRP[[i]] <- PSRP_tmp
  }


  ### Local Dependence Passing Student Rate
  param <- param[order(sapply(param, function(x) x$field))]
  LDPSR_table <- data.frame(matrix(NA, nrow = length(param), ncol = Dmax * 3 + 3))
  names(LDPSR_table) <-
    c(
      "Field", paste("Field Item", 1:Dmax),
      "Parent Class", paste("Parent CCR", 1:Dmax),
      "Child Class", paste("Child CCR", 1:Dmax)
    )
  for (i in 1:length(param)) {
    LDPSR_table[i, 1] <- param[[i]]$field
    item_vec <- tmp$ItemLabel[param[[i]]$fld]
    if (length(item_vec) < Dmax) {
      item_vec <- c(item_vec, rep(NA, Dmax - length(item_vec)))
    }
    LDPSR_table[i, 2:(2 + Dmax - 1)] <- item_vec
    LDPSR_table[i, (2 + Dmax)] <- param[[i]]$parent
    pa_CCR <- param[[i]]$pap
    if (length(pa_CCR) < Dmax) {
      pa_CCR <- c(pa_CCR, rep(NA, Dmax - length(pa_CCR)))
    }
    LDPSR_table[i, (2 + Dmax + 1):(2 * (Dmax + 1))] <- pa_CCR
    LDPSR_table[i, 2 * (Dmax + 1) + 1] <- param[[i]]$child
    ch_CCR <- param[[i]]$chap
    if (length(ch_CCR) < Dmax) {
      ch_CCR <- c(ch_CCR, rep(NA, Dmax - length(ch_CCR)))
    }
    LDPSR_table[i, (2 * (Dmax + 1) + 2):(Dmax * 3 + 3)] <- ch_CCR
  }


  # Marginal Bicluster Ref mat ----------------------------------------------
  pifr <- t(Pcf)
  TRP <- colSums(pifr * flddist)
  LCD <- colSums(cls01)
  CMD <- colSums(clsmemb)


  # Fit indices for two null model ----------------------------------

  indices1 <- TestFit(
    U = tmp$U, Z = tmp$Z,
    ell_A = indices$llm,
    nparam = sum(sign(sign(unlist(Pcf2)) + 1))
  )
  indices2 <- TestFitSaturated(
    U = tmp$U,
    Z = tmp$Z,
    ell_A = indices$llm,
    nparam = sum(sign(sign(unlist(Pcf2)) + 1))
  )


  ret <- structure(list(
    U = U,
    testlength = testlength,
    msg = "Rank",
    nobs = nobs,
    n_class = ncls,
    n_field = nfld,
    Nclass = ncls,
    Nfield = nfld,
    crr = crr(U),
    ItemLabel = tmp$ItemLabel,
    FieldLabel = FieldLabel,
    all_adj = all_adj,
    all_g = all_g,
    adj_list = adj_list,
    params = param,
    FRP = pifr,
    PSRP = PSRP,
    LDPSR = LDPSR_table,
    LFD = flddist,
    LCD = LCD,
    CMD = CMD,
    TRP = TRP,
    FieldEstimated = fld,
    ClassEstimated = cls,
    Students = Students,
    NextStage = NextStage,
    TestFitIndices = indices1,
    MG_FitIndices = indices1,
    SM_FitIndices = indices2,
    log_lik = indices$llm
  ), class = c("exametrika", "BINET"))
  return(ret)
}

library(exametrika)

### GOALS - Mathematica reference data
Test <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter11BINET_Test.csv"),
  check.names = FALSE
)
CCRR <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter11BINET_CCRR.csv"),
  check.names = FALSE
)
LDPSR <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter11BINET_LDPSR.csv"),
  check.names = FALSE
)
MB <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter11BINET_Marginal_Bicluster.csv"),
  check.names = FALSE
)
Item <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter11BINET_Item.csv"),
  check.names = FALSE
)
Student <- read.csv(
  test_path("fixtures", "mathematica_reference", "Chapter11BINET_Student.csv"),
  check.names = FALSE
)

### Target
fieldFile <- test_path("fixtures", "auxiliary_data", "FixFieldBINET.csv")
edgeFile <- test_path("fixtures", "auxiliary_data", "EdgesBINET.csv")
FieldData <- read.csv(fieldFile)
conf <- FieldData[, 2]
tgt <- BINET(
  U = J35S515,
  ncls = 13, nfld = 12,
  conf = conf, adj_file = edgeFile
)

# test1 Test ------------------------------------------------------

test_that("Test Info", {
  expect <- Test[16:31, 2] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- tgt$MG_FitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- Test[16:31, 3] |>
    unlist() |>
    unname() |>
    as.numeric()
  expect <- expect[c(5, 1, 2, 6, 3, 7, 4, 8:16)]
  result <- tgt$SM_FitIndices |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})


# CCRR ------------------------------------------------------

test_that("Conditional Correct Response Rate", {
  for (i in 1:12) {
    st <- (i - 1) * 14 + 1
    ed <- st + 12
    expect <- CCRR[st:ed, 2:5] |>
      unlist() |>
      unname() |>
      na.omit() |>
      as.numeric()
    result <- tgt$PSRP[[i]] |>
      unlist() |>
      as.numeric() |>
      na.omit() |>
      as.vector()
    expect_equal(result, expect, tolerance = 1e-4)
  }
})

# LDPSR ------------------------------------------------------
test_that("LDPSR", {
  expect <- LDPSR[, 1] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$LDPSR[, 1]
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- LDPSR[, 2:4] |>
    unlist() |>
    as.vector()
  result <- tgt$LDPSR[, 2:4] |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- LDPSR[, 6:10] |>
    unlist() |>
    unname() |>
    na.omit() |>
    as.numeric()
  result <- tgt$LDPSR[, 6:10] |>
    unlist() |>
    unname() |>
    na.omit() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- LDPSR[, 11:15] |>
    unlist() |>
    unname() |>
    na.omit() |>
    as.numeric()
  result <- tgt$LDPSR[, 11:15] |>
    unlist() |>
    unname() |>
    na.omit() |>
    as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

# LDPSR ------------------------------------------------------
test_that("Marginal Bicluster", {
  expect <- MB[1:12, 2:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$FRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MB[1:12, 15] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$LFD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MB[13, 2:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$TRP |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MB[14, 2:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$LCD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- MB[15, 2:14] |>
    unlist() |>
    unname() |>
    as.numeric()
  result <- tgt$CMD |> as.numeric()
  expect_equal(result, expect, tolerance = 1e-4)
})

# Students ------------------------------------------------------------

test_that("Students", {
  expect <- Student[, 6:18] |>
    unlist() |>
    as.vector()
  result <- tgt$Students[, 1:13] |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- Student[, 19] |>
    unlist() |>
    as.vector()
  result <- tgt$Students[, 14] |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
  expect <- Student[, 20:24] |>
    unlist() |>
    as.vector()
  result <- tgt$NextStage |>
    unlist() |>
    as.vector()
  expect_equal(result, expect, tolerance = 1e-4)
})

# adj_list input path -------------------------------------------------

test_that("BINET works with adj_list input", {
  # Build adj_list from the same edge data used in adj_file
  edges_data <- data.frame(
    From = c(1, 2, 3, 4, 5, 7, 2, 4, 6, 8, 10, 6, 6, 11, 8, 9, 12),
    To = c(2, 4, 5, 5, 6, 11, 3, 7, 9, 12, 12, 10, 8, 12, 12, 11, 13),
    Field = c(1, 2, 2, 3, 4, 4, 5, 5, 5, 5, 5, 7, 8, 8, 9, 9, 12)
  )
  ncls <- 13
  nfld <- 12
  cls_labels <- sprintf("Class%02d", 1:ncls)

  adj_list_input <- list()
  for (i in 1:nfld) {
    adj_R <- edges_data[edges_data$Field == i, 1:2, drop = FALSE]
    adj <- matrix(0, ncol = ncls, nrow = ncls)
    colnames(adj) <- rownames(adj) <- cls_labels
    if (nrow(adj_R) > 0) {
      for (r in seq_len(nrow(adj_R))) {
        adj[adj_R$From[r], adj_R$To[r]] <- 1
      }
    }
    adj_list_input[[i]] <- adj
  }

  tgt_list <- BINET(
    U = J35S515,
    ncls = ncls, nfld = nfld,
    conf = conf, adj_list = adj_list_input
  )

  # Results should match adj_file version
  expect_equal(tgt_list$log_lik, tgt$log_lik, tolerance = 1e-8)
  expect_equal(tgt_list$FRP, tgt$FRP, tolerance = 1e-8)
  expect_equal(tgt_list$TRP, tgt$TRP, tolerance = 1e-8)
  expect_equal(tgt_list$LCD, tgt$LCD, tolerance = 1e-8)
  expect_equal(tgt_list$CMD, tgt$CMD, tolerance = 1e-8)
  expect_equal(tgt_list$all_adj, tgt$all_adj)
  expect_equal(tgt_list$Students, tgt$Students, tolerance = 1e-8)

  # all_g should have Field edge attribute
  e_df <- igraph::as_data_frame(tgt_list$all_g)
  expect_true("Field" %in% colnames(e_df))
  expect_equal(nrow(e_df), nrow(igraph::as_data_frame(tgt$all_g)))
})

# g_list input path ---------------------------------------------------

test_that("BINET works with g_list input", {
  # Build g_list from the same adj_list used above
  edges_data <- data.frame(
    From = c(1, 2, 3, 4, 5, 7, 2, 4, 6, 8, 10, 6, 6, 11, 8, 9, 12),
    To = c(2, 4, 5, 5, 6, 11, 3, 7, 9, 12, 12, 10, 8, 12, 12, 11, 13),
    Field = c(1, 2, 2, 3, 4, 4, 5, 5, 5, 5, 5, 7, 8, 8, 9, 9, 12)
  )
  ncls <- 13
  nfld <- 12
  cls_labels <- sprintf("Class%02d", 1:ncls)

  g_list_input <- list()
  for (i in 1:nfld) {
    adj_R <- edges_data[edges_data$Field == i, 1:2, drop = FALSE]
    adj <- matrix(0, ncol = ncls, nrow = ncls)
    colnames(adj) <- rownames(adj) <- cls_labels
    if (nrow(adj_R) > 0) {
      for (r in seq_len(nrow(adj_R))) {
        adj[adj_R$From[r], adj_R$To[r]] <- 1
      }
    }
    g_list_input[[i]] <- igraph::graph_from_adjacency_matrix(adj, mode = "directed")
  }

  tgt_glist <- BINET(
    U = J35S515,
    ncls = ncls, nfld = nfld,
    conf = conf, g_list = g_list_input
  )

  # Results should match adj_file version
  expect_equal(tgt_glist$log_lik, tgt$log_lik, tolerance = 1e-8)
  expect_equal(tgt_glist$FRP, tgt$FRP, tolerance = 1e-8)
  expect_equal(tgt_glist$TRP, tgt$TRP, tolerance = 1e-8)
  expect_equal(tgt_glist$all_adj, tgt$all_adj)

  # all_g should have Field edge attribute
  e_df <- igraph::as_data_frame(tgt_glist$all_g)
  expect_true("Field" %in% colnames(e_df))
})

#' @title cumulative probability of GRM
#' @param theta latent score of subject
#' @param a discriminant parameter of IRF
#' @param b difficulty parameter of IRF
#' @keywords internal

grm_cumprob <- function(theta, a, b) {
  return(1 / (1 + exp(-a * (theta - b))))
}

#' @title Probability function for GRM
#' @description
#' Calculates the probability of selecting each category given a latent trait value and item parameters.
#' @param theta Latent trait value of the subject
#' @param a Discrimination parameter of IRF
#' @param b Vector of difficulty parameters (thresholds) of IRF
#' @return Vector of category selection probabilities
#' @examples
#' \dontrun{
#' # Example for an item with 3 categories
#' a <- 1.5
#' b <- c(-1.0, 1.0)
#' theta <- 0
#' grm_prob(theta, a, b)
#' }
#' @export

grm_prob <- function(theta, a, b) {
  K <- length(b) + 1
  cum_p <- numeric(K + 1)
  cum_p[1] <- 1
  cum_p[K + 1] <- 0

  for (k in 1:(K - 1)) {
    cum_p[k + 1] <- grm_cumprob(theta, a, b[k])
  }
  p <- numeric(K)
  for (k in 1:K) {
    p[k] <- cum_p[k] - cum_p[k + 1]
  }
  return(p)
}

#' @title Item Information Function for GRM
#' @description
#' Calculates the value of the Item Information Function for the Graded Response Model.
#' @param theta Latent trait value of the subject
#' @param a Discrimination parameter of IRF
#' @param b Vector of difficulty parameters (thresholds) of IRF
#' @return Value of the Item Information Function
#' @examples
#' \dontrun{
#' # Example for an item with 3 categories
#' a <- 1.5
#' b <- c(-1.0, 1.0)
#' thetas <- seq(-3, 3, by = 0.1)
#' info <- sapply(thetas, function(t) grm_iif(t, a, b))
#' plot(thetas, info, type = "l", xlab = "Theta", ylab = "Information")
#' }
#' @export


grm_iif <- function(theta, a, b) {
  K <- length(b) + 1
  p <- grm_prob(theta, a, b)
  info <- 1
  for (c in 1:(K - 1)) {
    p_c <- grm_cumprob(theta, a, b)[c]
    q_c <- 1 - p_c

    p_c_plus1 <- grm_cumprob(theta, a, b)[c + 1]
    if (is.na(p_c_plus1)) {
      p_c_plus1 <- 0
    }
    q_c_plus1 <- 1 - p_c_plus1

    num <- (p_c * q_c - p_c_plus1 * q_c_plus1)^2
    den <- p_c
    info <- info + (1.702^2 * a^2 * num) / den
  }
  return(unname(info))
}

#' @title generate start values for optimize
#' @param tmp dataset
#' @keywords internal
generate_start_values <- function(tmp) {
  nobs <- nrow(tmp)
  nitems <- ncol(tmp)
  ncat <- apply(tmp, 2, max, na.rm = TRUE)

  cor_mat <- cor(tmp, use = "pairwise.complete.obs")
  diag(cor_mat) <- 1
  eigen_result <- eigen(cor_mat)
  loadings <- abs(eigen_result$vectors[, 1]) * sqrt(abs(eigen_result$values[1]))
  a_init <- loadings / sqrt(1 - loadings^2)
  a_init <- pmax(pmin(a_init, 2.5), 0.3)

  b_init <- vector("list", nitems)
  for (j in 1:nitems) {
    valid_resp <- tmp[!is.na(tmp[, j]), j]
    n_thresh <- ncat[j] - 1
    if (n_thresh == 1) {
      b_init[[j]] <- quantile(valid_resp, 0.5, na.rm = TRUE) - mean(valid_resp, na.rm = TRUE)
    } else {
      probs <- seq(0.1, 0.9, length.out = n_thresh)
      thresholds <- quantile(valid_resp, probs, na.rm = TRUE)
      thresholds <- thresholds - median(thresholds)
      thresholds <- sort(thresholds)
      for (k in 2:length(thresholds)) {
        if (thresholds[k] <= thresholds[k - 1]) {
          thresholds[k] <- thresholds[k - 1] + 0.1
        }
      }
      b_init[[j]] <- thresholds
    }
  }
  return(list(a = a_init, b = b_init))
}


#' @title convert parameters to optimization target
#' @param a_vec discrimination parameter vector
#' @param b_list threshold parameter list
#' @keywords internal
params_to_target <- function(a_vec, b_list) {
  nitems <- length(a_vec)
  target_a <- log(a_vec)
  target_b <- c()
  for (j in 1:nitems) {
    thresholds <- b_list[[j]]
    n_thresholds <- length(thresholds)
    if (n_thresholds == 1) {
      target_b <- c(target_b, thresholds)
    } else {
      target_b <- c(target_b, thresholds[1])
      for (k in 2:n_thresholds) {
        delta <- max(thresholds[k] - thresholds[k - 1], 0.01)
        target_b <- c(target_b, log(delta))
      }
    }
  }
  return(c(target_a, target_b))
}

#' @title Graded Response Model (GRM)
#' @description
#' Implements Samejima's (1969) Graded Response Model (GRM), which is an Item Response Theory
#' model for ordered categorical response data. The model estimates discrimination parameters
#' and category threshold parameters for each item. It is widely used in psychological measurement,
#' educational assessment, and other fields that deal with multi-step rating scales.
#'
#' @param U Either an object of class "exametrika" or raw data. When raw data is given,
#'   it is converted to the exametrika class using the \code{\link{dataFormat}} function.
#' @param Z Missing indicator matrix of type matrix or data.frame. 1 indicates observed values, 0 indicates missing values.
#' @param w Item weight vector
#' @param na Specifies numbers or characters to be treated as missing values.
#' @param verbose Logical; if TRUE, shows progress of iterations (default: TRUE)
#'
#' @return A list of class "exametrika" and "GRM" containing the following elements:
#' \describe{
#'   \item{testlength}{Length of the test (number of items)}
#'   \item{nobs}{Sample size (number of rows in the dataset)}
#'   \item{log_lik}{Log-likelihood value at convergence}
#'   \item{iterations}{Number of iterations and function evaluations from optimization}
#'   \item{params}{Matrix containing the estimated item parameters}
#'   \item{EAP}{Ability parameters of examinees estimated by EAP method}
#'   \item{MAP}{Ability parameters of examinees estimated by MAP method}
#'   \item{PSD}{Posterior standard deviation of the ability parameters}
#'   \item{ItemFitIndices}{Fit indices for each item. See also \code{\link{ItemFit}}}
#'   \item{TestFitIndices}{Overall fit indices for the test. See also \code{\link{TestFit}}}
#' }
#'
#' @references
#' Samejima, F. (1969). Estimation of latent ability using a response pattern of graded scores.
#' Psychometrika Monograph Supplement, 34(4, Pt. 2), 1-100.
#'
#' @importFrom stats optim dnorm
#' @useDynLib exametrika, .registration=TRUE
#' @importFrom Rcpp evalCpp
#'
#' @examples
#' \dontrun{
#' # Apply GRM to example data
#' result <- GRM(J5S1000)
#' print(result)
#' plot(result, type = "IRF")
#' plot(result, type = "IIF")
#' plot(result, type = "TIF")
#' }
#' @export
#'
#'
GRM <- function(U, na = NULL, Z = NULL, w = NULL, verbose = TRUE) {
  # data format
  if (!inherits(U, "exametrika")) {
    tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  } else {
    tmp <- U
  }

  if (tmp$response.type != "ordinal") {
    response_type_error(tmp$response.type, "GRM")
  }

  tmp$Q[tmp$Z == 0] <- NA
  dat <- tmp$Q
  nitems <- NCOL(tmp$Z)
  nobs <- NROW(tmp$Z)

  # GRM internals (R and C++) index categories from 1..K, so remap each item's
  # responses to contiguous 1-based codes. Handles 0-based coding (e.g., 0..3)
  # and gaps (e.g., 1,2,4). ncat becomes the count of distinct valid responses.
  cat_levels <- lapply(seq_len(nitems), function(j) {
    sort(unique(dat[!is.na(dat[, j]), j]))
  })
  ncat <- vapply(cat_levels, length, integer(1))
  for (j in seq_len(nitems)) {
    dat[, j] <- match(dat[, j], cat_levels[[j]])
  }
  n_quad_points <- 51

  start_vals <- generate_start_values(dat)
  target <- params_to_target(start_vals$a, start_vals$b)
  if (verbose) {
    cat(
      "Parameters:", length(target), "| Initial LL:",
      round(log_lik_grm_cpp(target, dat, n_quad_points), 3), "\n"
    )
  }

  result <- optim(
    par = target,
    fn = function(x) -log_lik_grm_cpp(x, dat, n_quad_points),
    gr = function(x) -score_function_analytical_grm(x, dat, n_quad_points),
    method = "BFGS",
    control = list(
      maxit = 500,
      reltol = 1e-8,
      trace = if (verbose) 1 else 0
    )
  )
  final_params <- target_to_params_grm(result$par, nitems, ncat)

  est_a <- final_params$a
  est_b <- final_params$b
  est_mat <- matrix(NA, nrow = nitems, ncol = max(ncat))
  est_mat[, 1] <- est_a
  for (j in 1:nitems) {
    est_mat[j, 2:ncat[j]] <- est_b[[j]]
  }
  colnames(est_mat) <- c("Slope", paste0("Threshold", 1:(max(ncat) - 1)))
  rownames(est_mat) <- tmp$ItemLabel

  # subject params
  quadrature <- seq(-6, 6, 0.01)
  quad_weight <- dnorm(quadrature)
  quad_weight <- quad_weight / sum(quad_weight)
  nq <- length(quadrature)
  # W(SxP), L(PxQ)
  W <- matrix(nrow = nobs, ncol = sum(ncat))
  L <- matrix(nrow = sum(ncat), ncol = nq)

  for (i in 1:nobs) {
    vec <- c()
    for (j in 1:nitems) {
      resp <- dat[i, j]
      v <- rep(0, ncat[j])
      v[resp] <- 1
      vec <- c(vec, v)
    }
    W[i, ] <- vec
  }

  for (j in 1:nitems) {
    a <- est_a[j]
    b <- est_b[[j]]
    start_pos <- c(0, cumsum(ncat)) + 1
    end_pos <- start_pos - 1
    start_pos <- start_pos[1:nitems]
    end_pos <- end_pos[-1]
    for (q in 1:nq) {
      L[start_pos[j]:end_pos[j], q] <- grm_prob(quadrature[q], a, b)
    }
  }

  L_weighted <- t(t(L) * quad_weight)
  post_theta <- W %*% L_weighted
  post_theta <- post_theta / rowSums(post_theta)
  score_idx <- apply(post_theta, 1, which.max)
  MAP <- quadrature[score_idx]
  # EAP
  theta_matrix <- matrix(quadrature, nrow = nobs, ncol = nq, byrow = TRUE)
  EAP <- rowSums(post_theta * theta_matrix)
  # PSD
  theta_squared_matrix <- matrix(quadrature^2, nrow = nobs, ncol = nq, byrow = TRUE)
  second_moment <- rowSums(post_theta * theta_squared_matrix)
  posterior_variance <- second_moment - EAP^2
  PSD <- sqrt(posterior_variance)

  # model fit
  const <- 1e-16
  ### Analysis model
  ell_A <- rep(0, nitems)
  for (j in 1:nitems) {
    for (i in 1:nobs) {
      theta_i <- EAP[i]
      resp <- dat[i, j]
      if (!is.na(resp)) {
        prob <- grm_prob(theta_i, est_a[j], est_b[[j]])[resp]
        ell_A[j] <- ell_A[j] + log(prob + const)
      }
    }
  }
  ### Null model
  response_list <- lapply(seq_len(nitems), function(j) {
    table(factor(dat[, j], levels = seq_len(ncat[j])))
  })
  valid_response <- apply(tmp$Z, 2, sum)
  ell_N <- rep(0, nitems)
  for (j in 1:nitems) {
    cat_probs <- response_list[[j]] / valid_response[j]
    nz <- response_list[[j]] > 0
    if (any(nz)) {
      ell_N[j] <- sum(response_list[[j]][nz] * log(cat_probs[nz]))
    }
  }
  ### Bench model
  patterns <- rowSums(dat, na.rm = TRUE)
  unique_ptn <- unique(patterns)
  n_pattern <- length(unique_ptn)
  pattern_groups <- match(patterns, unique_ptn)

  MsG <- matrix(0, nrow = nobs, ncol = n_pattern)
  for (i in 1:nobs) {
    MsG[i, pattern_groups[i]] <- 1
  }
  group_size <- colSums(MsG)
  ell_B <- rep(0, nitems)
  for (j in 1:nitems) {
    for (g in 1:n_pattern) {
      group_memb <- which(pattern_groups == g)
      valid_memb <- group_memb[tmp$Z[group_memb, j] == 1]
      if (length(valid_memb) > 0) {
        cat_counts <- table(factor(dat[valid_memb, j], levels = seq_len(ncat[j])))
        cat_probs <- cat_counts / sum(cat_counts)
        nz <- cat_counts > 0
        if (any(nz)) {
          ell_B[j] <- ell_B[j] + sum(cat_counts[nz] * log(cat_probs[nz]))
        }
      }
    }
  }

  chi_A <- 2 * (ell_B - ell_A)
  chi_B <- 2 * (ell_B - ell_N)
  # Free-parameter counts per item (matches exametrika convention in
  # Biclustering.ordinal: df = bench_nparam - restricted_nparam):
  #   bench: per raw-score group a multinomial over ncat categories
  #          -> n_pattern * (ncat - 1)
  #   null:  single multinomial across all examinees
  #          -> (ncat - 1)
  #   GRM:   one slope plus (ncat - 1) thresholds
  #          -> ncat
  bench_nparam <- n_pattern * (ncat - 1)
  null_nparam <- ncat - 1
  model_nparam <- ncat
  df_A <- bench_nparam - model_nparam
  df_B <- bench_nparam - null_nparam
  ItemFitIndices <- structure(
    c(list(
      model_log_like = ell_A,
      bench_log_like = ell_B,
      null_log_like = ell_N,
      model_Chi_sq = chi_A,
      null_Chi_sq = chi_B,
      model_df = df_A,
      null_df = df_B
    ), calcFitIndices(chi_A, chi_B, df_A, df_B, nobs)),
    class = c("exametrika", "ModelFit")
  )
  TestFitIndices <- structure(
    c(list(
      model_log_like = sum(ell_A),
      bench_log_like = sum(ell_B),
      null_log_like = sum(ell_N),
      model_Chi_sq = sum(chi_A),
      null_Chi_sq = sum(chi_B),
      model_df = sum(df_A),
      null_df = sum(df_B)
    ), calcFitIndices(sum(chi_A), sum(chi_B), sum(df_A), sum(df_B), nobs)),
    class = c("exametrika", "ModelFit")
  )

  ret <- structure(list(
    testlength = nitems,
    nobs = nobs,
    log_lik = -result$value,
    iterations = result$counts,
    params = as.data.frame(est_mat),
    EAP = EAP,
    MAP = MAP,
    PSD = PSD,
    ItemFitIndices = ItemFitIndices,
    TestFitIndices = TestFitIndices
  ), class = c("exametrika", "GRM"))
  return(ret)
}

#' @rdname LRA
#' @section Rated Data Method:
#' \code{LRA.rated} analyzes data with ratings assigned to each response, such as
#' partially-credited items or preference scales where response categories have different weights.
#'
#' @param minFreqRatio Minimum frequency ratio for response categories (default = 0).
#' Categories with occurrence rates below this threshold will be excluded from analysis.
#' For example, if set to 0.1, response categories that appear in less than 10% of
#' responses for an item will be omitted.
#'
#' @return
#' For rated data (\code{LRA.rated}), the returned list additionally includes:
#' \describe{
#' \item{msg}{A character string indicating the model type. }
#' \item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
#' \item{ScoreReport}{Descriptive statistics of test performance, including sample size,
#'   test length, central tendency, variability, distribution characteristics, and reliability.}
#' \item{ItemReport}{Basic statistics for each item including category proportions and item-total correlations.}
#' \item{ICRP}{Item Category Reference Profile matrix showing probability of response in each category by rank.}
#' \item{ScoreRankCorr}{Spearman's correlation between test scores and estimated ranks.}
#' \item{RankQuantCorr}{Spearman's correlation between estimated ranks and quantile groups.}
#' \item{ScoreRank}{Contingency table of raw scores by estimated ranks.}
#' \item{ScoreMembership}{Expected rank memberships for each raw score.}
#' \item{RankQuantile}{Cross-tabulation of rank frequencies and quantile groups.}
#' \item{MembQuantile}{Cross-tabulation of rank membership probabilities and quantile groups.}
#' \item{ItemQuantileRef}{Reference values for each item across quantile groups.}
#' \item{CatQuant}{Response patterns across item categories and quantile groups.}
#' }
#'
#' @examples
#' \donttest{
#' # Rated data example
#' # Fit a Latent Rank Analysis model with 10 ranks to rated data
#' result.LRArated <- LRA(J35S5000, nrank = 10, mic = TRUE)
#'
#' # Plot score distributions
#' plot(result.LRArated, type = "ScoreFreq")
#' plot(result.LRArated, type = "ScoreRank")
#'
#' # Plot category response patterns for items 1-6
#' plot(result.LRArated, type = "ICRP", items = 1:6, nc = 3, nr = 2)
#' }
#'
#' @importFrom stats cor
#' @importFrom utils head
#' @export
#'
LRA.rated <- function(U,
                      nrank = 2,
                      mic = FALSE,
                      maxiter = 100,
                      trapezoidal = 0,
                      eps = 1e-4,
                      minFreqRatio = 0,
                      verbose = FALSE, ...) {
  ## check trapezoidal prior
  if (trapezoidal > 0) {
    if (trapezoidal > 1 / nrank) {
      stop("Trapezoidal prior hem value must be smaller than 1/nrank")
    }
  }

  ScoreReport <- ScoreReport(U)
  ItemReport <- ItemReport(U)
  score <- rowSums(U$Z * U$U)
  maxscore <- max(score)
  zzzTotal <- apply(U$Z, 2, sum)
  scoredist <- table(factor(score, levels = 0:maxscore))
  nobs <- NROW(U$Q)
  nitems <- NCOL(U$Q)
  correct_answer <- U$CA
  nquan <- nrank
  missing1error0 <- 0
  collapse_indicator <- 100
  const <- 1e-6

  # Making Filter
  Fil <- create_filter_matrix(nrank)


  # Prepare ---------------------------------------------------------


  # Calc Frequency
  # NOTE: Use lapply(seq_len, ...) instead of lapply(apply(..., 2, unique), ...)
  # because apply() returns a matrix (not a list) when all columns have the
  # same number of unique values, causing lapply to iterate over individual
  # elements rather than per-column vectors.
  category999 <- lapply(seq_len(nitems), function(j) sort(unique(U$Q[, j])))
  # Calc Frequency excluding missing
  category <- lapply(
    category999,
    function(x) x[x != -1]
  )
  # number of categories
  ncat999 <- sapply(category999, length)
  # number of categories excluding missing
  ncat <- sapply(category, length)

  # Frequency table of categories
  catfreq999 <- lapply(seq_len(nitems), function(j) table(U$Q[, j]))
  catfreq <- lapply(catfreq999, function(x) x[names(x) != "-1"])

  ## Count Correct Ans
  correctFreq <- rep(0, nitems)
  for (j in 1:nitems) {
    correctFreq[j] <- sum(U$Q[, j] == U$CA[j])
  }

  ### MinFreq Check
  usecategory <- list()
  for (j in 1:nitems) {
    temp <- sapply(1:ncat[j], function(k) {
      if (catfreq[[j]][k] >= nobs * minFreqRatio || catfreq[[j]][k] != U$CA[j]) {
        category[[j]][k]
      } else {
        collapse_indicator
      }
    })
    usecategory[[j]] <- sort(unique(temp))
  }

  usecategory999 <- list()
  for (j in 1:nitems) {
    if (-1 %in% category999[[j]]) {
      usecategory999[[j]] <- c(usecategory[[j]], -1)
    } else {
      usecategory999[[j]] <- usecategory[[j]]
    }
  }

  # useNcat,useNcat999
  useNcat <- sapply(usecategory, length)
  useNcat999 <- sapply(usecategory999, length)

  # Expand data matrix
  expand_Zij <- matrix(0, nrow = nobs, ncol = sum(useNcat))
  current_col <- 1

  for (j in 1:ncol(U$Z)) {
    for (k in 1:useNcat[j]) {
      expand_Zij[, current_col] <- U$Z[, j]
      current_col <- current_col + 1
    }
  }

  # use_Xij
  use_Xij <- matrix(0, nrow = nobs, ncol = nitems)
  for (i in 1:nobs) {
    for (j in 1:nitems) {
      if (U$Q[i, j] %in% usecategory[[j]]) {
        use_Xij[i, j] <- U$Q[i, j]
      } else {
        use_Xij[i, j] <- collapse_indicator
      }
    }
  }

  usecatfreq <- list()
  for (j in 1:nitems) {
    usecatfreq[[j]] <- sapply(usecategory[[j]], function(cat) {
      sum(use_Xij[, j] == cat)
    })
  }

  Cijk999 <- array(NA, dim = c(nobs, nitems, max(ncat999)))
  for (j in 1:nitems) {
    for (k in 1:ncat999[j]) {
      Cijk999[, j, k] <- 1 * (U$Q[, j] == category999[[j]][k])
    }
  }


  use_Cijk <- array(0, dim = c(nobs, nitems, max(sapply(usecategory, length))))
  for (j in 1:nitems) {
    for (k in 1:useNcat[j]) {
      use_Cijk[, j, k] <- 1 * (use_Xij[, j] == usecategory[[j]][k])
    }
  }

  CijkMat <- matrix(0, nrow = nobs, ncol = sum(useNcat))
  for (i in 1:nobs) {
    tmp <- c()
    for (j in 1:nitems) {
      tmp <- c(tmp, use_Cijk[i, j, 1:useNcat[j]])
    }
    CijkMat[i, ] <- tmp
  }

  quantileScore <- quantile(score, probs = (1:(nquan - 1)) / nquan)
  quantileRank <- rowSums(outer(score, quantileScore, ">")) + 1
  quantileRefVec <- matrix(0, nrow = nitems, ncol = nquan)
  for (j in 1:nitems) {
    for (k in 1:nquan) {
      quantileRefVec[j, k] <- mean(U$U[quantileRank == k, j])
    }
  }

  QRVdf <- data.frame(quantileRefVec)
  names(QRVdf) <- paste0("Q", 1:nquan)
  rownames(QRVdf) <- U$ItemLabel

  ## Category Quantile Report
  SelectRatio <- NULL
  ref_mat <- matrix(NA, nrow = sum(ncat999), ncol = nquan)
  catquanRefbox999 <- list()
  quanRefbox <- list()
  for (j in 1:nitems) {
    catquanRefbox999[[j]] <- matrix(NA, nrow = ncat999[j], ncol = nquan)
    quanRefbox[[j]] <- matrix(NA, nrow = useNcat[j], ncol = nquan)
    place0 <- (cumsum(ncat999) - ncat999 + 1)[j]
    place1 <- cumsum(ncat999[1:j])[j]
    for (q in 1:nquan) {
      catquanRefbox999[[j]][, q] <- apply(Cijk999[quantileRank == q, j, ][, 1:ncat999[j]], 2, sum) / sum(U$Z[quantileRank == q, j])
      quanRefbox[[j]][, q] <- apply(use_Cijk[quantileRank == q, j, ][, 1:useNcat[j]], 2, sum) / sum(U$Z[quantileRank == q, j])
      ref_mat[place0:place1, q] <- catquanRefbox999[[j]][, q]
    }
    category_labels <- unlist(U$CategoryLabel[j])
    if (any(names(catfreq999[[j]]) == "-1")) {
      category_labels <- c(paste0(U$ItemLabel[j], "-X"), category_labels)
    }
    item_data <- data.frame(
      item = rep(U$ItemLabel[j], ncat999[j]),
      cateogry = category_labels,
      selRatio = catfreq999[[j]] / zzzTotal[j],
      ref_mat = ref_mat[place0:place1, ]
    )
    if (any(names(catfreq999[[j]]) == "-1")) {
      item_data <- rbind(
        item_data[names(catfreq999[[j]]) != "-1", ],
        item_data[names(catfreq999[[j]]) == "-1", ]
      )
    }
    SelectRatio <- rbind(SelectRatio, item_data[, -3])
  }
  rownames(SelectRatio) <- NULL
  names(SelectRatio) <- c("Item", "Category", "SLCT Ratio", paste0("Q", 1:nquan))


  # Algorithm -------------------------------------------------------

  ## Prior
  if (trapezoidal > 0) {
    log_prior <- c(
      trapezoidal,
      rep((1 - 2 * trapezoidal) / (nrank - 2), nrank - 2),
      trapezoidal
    )
  } else {
    log_prior <- rep(1, nrank)
  }
  lp2 <- logprior_NQmat <- matrix(rep(log(log_prior), nobs), byrow = TRUE, nrow = nobs)


  # Design Vector/Matrix
  designfunc <- function(x, y) 1

  create_upper_diagonal_matrix <- function(func, n) {
    matrix <- matrix(0, n, n)
    for (i in 1:n) {
      for (j in 1:n) {
        if (i <= j) matrix[i, j] <- func(i, j)
      }
    }
    return(matrix)
  }

  design0 <- sapply(1:nitems, function(j) sum(useNcat[1:j]))
  design1 <- cbind(head(c(0, design0) + 1, -1), design0)

  design2 <- lapply(1:nitems, function(j) {
    seq(design1[j, 1], design1[j, 2])
  })

  design3 <- do.call(rbind, lapply(1:nitems, function(j) {
    cbind(rep(j, useNcat[j]), design2[[j]])
  }))

  design4 <- matrix(0, nrow = nitems, ncol = sum(useNcat))
  for (i in 1:nrow(design3)) {
    design4[design3[i, 1], design3[i, 2]] <- 1
  }

  design5 <- do.call(rbind, lapply(1:nitems, function(j) {
    # For each row j of design4, repeat ncat[j] times
    matrix(rep(design4[j, ], useNcat[j]), nrow = useNcat[j], byrow = TRUE)
  }))

  design6 <- rep(NA, nitems)
  for (j in 1:nitems) {
    pos <- which(usecategory[[j]] == correct_answer[j])
    design6[j] <- design2[[j]][pos]
  }
  # Saturation Model -------------------------------------------------
  correctCRPsatu <- rep(NA, nitems)
  for (i in 1:nitems) {
    correctCRPsatu[i] <- 0.1 + 0.8 / (1 + exp(-(i - nitems / 2) / 3))
  }

  catRefbox_satu <- vector("list", nitems)
  for (j in 1:nitems) {
    catRefbox_satu[[j]] <- matrix(NA, nrow = useNcat[j], ncol = length(correctCRPsatu))
    for (k in 1:useNcat[j]) {
      if (usecategory[[j]][[k]] == correct_answer[j]) {
        catRefbox_satu[[j]][k, ] <- correctCRPsatu
      } else {
        catRefbox_satu[[j]][k, ] <- (1 - correctCRPsatu) / (useNcat[j] - 1)
      }
    }
  }

  catRefmat_satu <- do.call(rbind, catRefbox_satu)
  ## EM Algorithm
  if (verbose) {
    message("Starting EM estimation for Saturation Model...")
  }

  ij_log_lik_satu <- -10
  iter_satu <- 0
  converge_satu <- TRUE
  FLG <- TRUE
  while (FLG) {
    old_log_lik_satu <- ij_log_lik_satu

    rankProf_num_satu <- exp(CijkMat %*% log(catRefmat_satu + const))
    rankProf_den_satu <- rowSums(rankProf_num_satu)
    rankProf_satu <- rankProf_num_satu / rankProf_den_satu

    refMatcore_satu <- t(CijkMat) %*% rankProf_satu

    catRefmat_satu <- refMatcore_satu / design5 %*% refMatcore_satu
    log_lik_satu <- sum(rankProf_satu * (log(rankProf_num_satu + const)))
    ij_log_lik_satu <- log_lik_satu / nitems / nobs

    iter_satu <- iter_satu + 1
    if (verbose) {
      message(
        sprintf(
          "Saturation Model - iter %d: log_lik=%.6f",
          iter_satu, ij_log_lik_satu
        )
      )
    }

    if (abs(old_log_lik_satu - ij_log_lik_satu) < eps * abs(old_log_lik_satu)) {
      FLG <- FALSE
    }
    if (iter_satu > maxiter) {
      converge_satu <- FALSE
      FLG <- FALSE
      message("\nReached the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
    }
  }

  first_sum <- sum(catRefmat_satu[design6, 1])
  last_sum <- sum(catRefmat_satu[design6, ncol(catRefmat_satu)])


  if (first_sum > last_sum) {
    catRefmat_satu <- catRefmat_satu[, ncol(catRefmat_satu):1]
  }

  catRefbox_satu <- vector("list", nitems)
  for (j in 1:nitems) {
    catRefbox_satu[[j]] <- catRefmat_satu[design1[j, 1]:design1[j, 2], ]
  }

  if (mic) {
    catRefbox_satu2 <- vector("list", nitems)
    for (j in 1:nitems) {
      target_row <- catRefmat_satu[design6[j], ]
      current_matrix <- catRefbox_satu[[j]]
      augmented_matrix <- t(rbind(target_row, current_matrix))
      sorted <- augmented_matrix[do.call(order, as.data.frame(augmented_matrix)), ]
      back_matrix <- t(sorted)
      catRefbox_satu2[[j]] <- back_matrix[-1, ]
    }
    catRefmat_satu <- do.call(rbind, catRefbox_satu2)
  }


  # Restricted Model ------------------------------------------------

  catRefbox <- quanRefbox
  catRefmat <- do.call(rbind, catRefbox)

  ij_log_lik <- -10
  iter <- 0
  if (verbose) {
    message("\nStarting EM estimation for Restricted Model...")
  }
  converge <- TRUE
  FLG <- TRUE
  while (FLG) {
    old_log_lik <- ij_log_lik
    rankProf_num <- exp(CijkMat %*% log(catRefmat + const) + logprior_NQmat)
    rankProf_den <- rowSums(rankProf_num)
    rankProf <- rankProf_num / rankProf_den

    refMatcore <- t(CijkMat) %*% rankProf %*% Fil
    catRefmat <- refMatcore / design5 %*% refMatcore

    first_col_sum <- sum(catRefmat[design6, 1])
    last_col_sum <- sum(catRefmat[design6, ncol(catRefmat)])
    if (first_col_sum > last_col_sum) {
      catRefmat <- catRefmat[, ncol(catRefmat):1]
    }
    for (j in 1:nitems) {
      catRefbox[[j]] <- catRefmat[design1[j, 1]:design1[j, 2], ]
    }

    if (mic == 1) {
      for (j in 1:nitems) {
        sort_order <- order(catRefmat[design6[[j]], ])
        catRefbox[[j]] <- catRefbox[[j]][, sort_order]
      }
    }

    catRefmat <- do.call(rbind, catRefbox)

    log_lik <- sum(rankProf * log(rankProf_num))
    ij_log_lik <- log_lik / nitems / nobs

    iter <- iter + 1
    if (verbose) {
      message(
        sprintf(
          "Restricted Model - iter %d: log_lik=%.6f",
          iter, ij_log_lik
        )
      )
    }

    if (abs(old_log_lik - ij_log_lik) < eps * abs(old_log_lik)) {
      FLG <- FALSE
    }
    if (iter > maxiter) {
      converge <- FALSE
      FLG <- FALSE
      message("\nReached the maximum number of iterations.")
      message("Warning: Algorithm may not have converged. Interpret results with caution.")
    }
  }


  # results ---------------------------------------------------------

  category_report <- as.data.frame(catRefmat)
  colnames(category_report) <- paste0("rank", 1:nrank)

  cat_labelXXX <- vector("list", length(category))
  for (i in seq_along(category)) {
    original_labels <- U$CategoryLabel[[i]]
    used_cats <- usecategory[[i]]

    is_dropped <- used_cats == 100

    if (any(is_dropped)) {
      used_labels <- original_labels[used_cats[!is_dropped]]
      cat_labelXXX[[i]] <- c(used_labels, "CatX")
    } else {
      cat_labelXXX[[i]] <- original_labels
    }
  }
  category_report$ItemLabel <- rep(U$ItemLabel, U$categories)
  category_report$CategoryLabel <- unlist(U$CategoryLabel)
  ## Item-Category Reference Profile
  ICRP <- category_report[, c("ItemLabel", "CategoryLabel", paste0("rank", 1:nrank))]

  #
  testRefVec <- apply(catRefmat[design6, ], 2, sum)

  ## rank Profile
  rankProf_num <- exp(CijkMat %*% log(catRefmat + const) + logprior_NQmat)
  rankProf_den <- rowSums(rankProf_num)
  rankProf <- rankProf_num / rankProf_den

  rankmemb <- apply(rankProf, 1, which.max)
  rankmemb01 <- sign(rankProf - apply(rankProf, 1, max)) + 1
  rankdist <- colSums(rankmemb01)
  RMD <- colSums(rankProf)

  StudentRank <- rankProf
  RU <- ifelse(rankmemb + 1 > nrank, NA, rankmemb + 1)
  RD <- ifelse(rankmemb - 1 < 1, NA, rankmemb - 1)
  RUO <- StudentRank[cbind(1:nobs, RU)] / StudentRank[cbind(1:nobs, rankmemb)]
  RDO <- StudentRank[cbind(1:nobs, RD)] / StudentRank[cbind(1:nobs, rankmemb)]
  StudentRank <- cbind(StudentRank, score, rankmemb, quantileRank, RUO, RDO)
  colnames(StudentRank) <- c(
    paste("Membership", 1:nrank), "Score", "Estimate",
    "Quantile Rank",
    "Rank-Up Odds", "Rank-Down Odds"
  )
  rownames(StudentRank) <- U$ID

  rho1 <- cor(score, rankmemb, method = "spearman")

  ## score-rank dist
  scoreRankDist <- matrix(0, nrow = maxscore + 1, ncol = nrank)
  scoreMembDist <- matrix(0, nrow = maxscore + 1, ncol = nrank)
  rownames(scoreRankDist) <- rownames(scoreMembDist) <- 0:maxscore
  colnames(scoreRankDist) <- colnames(scoreMembDist) <- paste0("Rank", 1:nrank)
  for (s in 0:maxscore) {
    if (scoredist[s + 1] > 0) {
      ranks_s <- rankmemb[score == s]
      scoreRankDist[s + 1, ] <- sapply(1:nrank, function(r) sum(ranks_s == r))
      scoreMembDist[s + 1, ] <- colSums(rankProf[score == s, , drop = FALSE])
    }
  }
  rankQuanDist <- unname(table(rankmemb, quantileRank))
  membQuanDist <- matrix(0, nrow = nrank, ncol = nquan)
  rho2 <- cor(rankmemb, quantileRank, method = "spearman")
  for (q in 1:nquan) {
    membQuanDist[, q] <- colSums(rankProf[quantileRank == q, , drop = FALSE])
  }
  rownames(rankQuanDist) <- rownames(membQuanDist) <- paste0("Rank", 1:nrank)
  colnames(rankQuanDist) <- colnames(membQuanDist) <- paste0("Quantile", 1:nrank)

  # Fit indices -----------------------------------------------------
  itemdf <- (useNcat - 1) * (nitems - sum(diag(Fil)))
  testdf <- sum(itemdf)
  null_itemdf <- (useNcat - 1) * (nitems - 1)
  null_testdf <- sum(null_itemdf)

  rankProf_num_satu <- exp(CijkMat %*% log(catRefmat_satu + const))
  rankProf_den_satu <- rowSums(rankProf_num_satu)
  rankProf_satu <- rankProf_num_satu / rankProf_den_satu
  Rank_satu <- apply(rankProf_satu, 1, which.max)
  Rank_satu01 <- sign(rankProf_satu - apply(rankProf_satu, 1, max)) + 1

  # modelfff_jq1 <- t(dat$Z) %*% rankmemb01
  # modelggg_jq1 <- t(CijkMat) %*% rankmemb01
  # satufff_jq1 <- t(dat$Z) %*% Rank_satu01
  # satuggg_jq1 <- t(CijkMat) %*% Rank_satu01
  modelfff_jq2 <- t(U$Z) %*% rankProf
  modelggg_jq2 <- t(CijkMat) %*% rankProf
  satufff_jq2 <- t(U$Z) %*% rankProf_satu
  satuggg_jq2 <- t(CijkMat) %*% rankProf_satu

  # Model item log-likelihood
  # model_itemll1 <- design4 %*% colSums(t(modelggg_jq1 * log(catRefmat + const)))
  model_itemll2 <- design4 %*% colSums(t(modelggg_jq2 * log(catRefmat + const)))

  # Saturated item log-likelihood
  # satu_itemll1 <- design4 %*% colSums(t(satuggg_jq1 * log(catRefmat_satu + const)))
  satu_itemll2 <- design4 %*% colSums(t(satuggg_jq2 * log(catRefmat_satu + const)))

  null_itemll <- rep(0, nitems)
  zzzTotal <- apply(U$Z, 2, sum)
  for (j in 1:nitems) {
    null_itemll[j] <- sum(usecatfreq[[j]] * log(usecatfreq[[j]] / zzzTotal[j] + const))
  }


  # Model chi-square
  # model_itemchisq1 <- pmax(0.000001, pmin(2 * (satu_itemll1 - model_itemll1), 1000000000))
  model_itemchisq2 <- pmax(0.000001, pmin(2 * (satu_itemll2 - model_itemll2), 1000000000))

  # Null chi-square
  # null_itemchisq1 <- pmax(0.000001, pmin(2 * (satu_itemll1 - null_itemll), 1000000000)) + 0.000001
  null_itemchisq2 <- pmax(0.000001, pmin(2 * (satu_itemll2 - null_itemll), 1000000000)) + 0.000001

  # Item Fit Indices
  # ItemFitIndices1 <- calcFitIndices(model_itemchisq1, null_itemchisq1, itemdf, null_itemdf, colSums(dat$Z))
  ItemFits <- calcFitIndices(model_itemchisq2, null_itemchisq2, itemdf, null_itemdf, colSums(U$Z))
  ItemFitIndices <- structure(
    c(list(
      model_log_like = model_itemll2,
      bench_log_like = satu_itemll2,
      null_log_like = null_itemll,
      model_Chi_sq = model_itemchisq2,
      null_Chi_sq = null_itemchisq2,
      model_df = itemdf,
      null_df = null_itemdf
    ), ItemFits),
    class = c("exametrika", "ModelFit")
  )

  TestFits <- calcFitIndices(
    sum(model_itemchisq2),
    sum(null_itemchisq2) + const,
    sum(itemdf),
    sum(null_itemdf),
    sum(colSums(U$Z))
  )

  TestFitIndices <- structure(
    c(list(
      model_log_like = log_lik,
      bench_log_like = sum(satu_itemll2),
      null_log_like = sum(null_itemll),
      model_Chi_sq = sum(model_itemchisq2),
      null_Chi_sq = sum(null_itemchisq2),
      model_df = sum(itemdf),
      null_df = sum(null_itemdf)
    ), TestFits),
    class = c("exametrika", "ModelFit")
  )

  # output ----------------------------------------------------------

  ret <- structure(list(
    U = U,
    mic = mic,
    msg = "Rank",
    converge = converge,
    testlength = NCOL(U$Q),
    nobs = NROW(U$Q),
    n_rank = nrank, # New naming convention
    n_cycle = iter, # New naming convention
    TRP = as.vector(testRefVec),
    LRD = rankdist,
    RMD = RMD,
    ICRP = ICRP,
    ScoreRankCorr = rho1,
    RankQuantCorr = rho2,
    Students = StudentRank,
    ScoreRank = scoreRankDist,
    ScoreMembership = scoreMembDist,
    RankQuantile = rankQuanDist,
    MembQuantile = membQuanDist,
    ItemFitIndices = ItemFitIndices,
    TestFitIndices = TestFitIndices,
    ScoreReport = ScoreReport,
    ItemReport = ItemReport,
    ItemQuantileRef = QRVdf,
    CatQuant = SelectRatio,
    # Deprecated fields (for backward compatibility)
    Nrank = nrank,
    N_Cycle = iter
  ), class = c("exametrika", "LRArated"))
  return(ret)
}

#' @title StudentAnalysis
#' @description
#' The StudentAnalysis function returns descriptive statistics for each individual student.
#' Specifically, it provides the number of responses, the number of correct answers,
#' the passage rate, the standardized score, the percentile, and the stanine.
#' @param U U is a data matrix of the type matrix or data.frame.
#' @param Z Z is a missing indicator matrix of the type matrix or data.frame
#' @param w w is item weight vector
#' @param na na argument specifies the numbers or characters to be treated as missing values.
#'   \itemize{
#'     \item ID: Student identifier
#'     \item NR: Number of responses
#'     \item NRS: Number-right score (total correct answers)
#'     \item PR: Passage rate (proportion correct)
#'     \item SS: Standardized score (z-score)
#'     \item Percentile: Student's percentile rank
#'     \item Stanine: Student's stanine score (1-9)
#'   }
#' @return Returns a data frame containing the following columns for each student:
#'   \itemize{
#'     \item ID: Student identifier
#'     \item NR: Number of responses
#'     \item NRS: Number-right score (total correct answers)
#'     \item PR: Passage rate (proportion correct)
#'     \item SS: Standardized score (z-score)
#'     \item Percentile: Student's percentile rank
#'     \item Stanine: Student's stanine score (1-9)
#'   }
#' @examples
#' # using sample dataset
#' StudentAnalysis(J15S500)
#' @export
#'

StudentAnalysis <- function(U, na = NULL, Z = NULL, w = NULL) {
  tmp <- dataFormat(data = U, na = na, Z = Z, w = w)
  NRS <- nrs(tmp)
  NR <- NCOL(tmp$U) - rowSums(is.na(tmp$U))
  PR <- passage(tmp)
  SS <- sscore(tmp)
  Ptile <- percentile(tmp)
  ST <- stanine(tmp)
  ret <- data.frame(
    ID = tmp$ID,
    NR = NR,
    NRS = NRS,
    PR = PR,
    SS = SS,
    Percentile = Ptile,
    Stanine = ST$stanineScore
  )
  return(ret)
}

library(exametrika)

### StudentAnalysis Tests

test_that("StudentAnalysis Basic Execution", {
  result <- StudentAnalysis(J15S500)

  # Result should be a data.frame
  expect_true(is.data.frame(result))

  # Expected columns: ID, NR, NRS, PR, SS, Percentile, Stanine
  expected_cols <- c("ID", "NR", "NRS", "PR", "SS", "Percentile", "Stanine")
  expect_equal(names(result), expected_cols)
})

test_that("StudentAnalysis Correct Dimensions", {
  result <- StudentAnalysis(J15S500)

  # J15S500 has 500 students and 15 items
  expect_equal(nrow(result), 500)
  expect_equal(ncol(result), 7)

  # NR should equal number of items (15) for complete-response students
  expect_true(all(result$NR <= 15))
  expect_true(all(result$NR > 0))
})

test_that("StudentAnalysis Value Ranges", {
  result <- StudentAnalysis(J15S500)

  # Passage Rate should be between 0 and 1
  expect_true(all(result$PR >= 0 & result$PR <= 1, na.rm = TRUE))

  # NRS should be non-negative and at most equal to NR
  expect_true(all(result$NRS >= 0, na.rm = TRUE))
  expect_true(all(result$NRS <= result$NR, na.rm = TRUE))

  # Stanine should be between 1 and 9 (returned as factor)
  expect_true(is.factor(result$Stanine))
  stanine_vals <- as.numeric(as.character(result$Stanine))
  expect_true(all(stanine_vals >= 1 & stanine_vals <= 9, na.rm = TRUE))

  # Percentile should be between 0 and 100
  expect_true(all(result$Percentile >= 0 & result$Percentile <= 100, na.rm = TRUE))
})

#' @title Initialize field membership profile probabilities
#' @description
#' Calculates initial probabilities for the field membership profile matrix
#' in biclustering analysis. This function generates probability distributions
#' based on class and field effects to initialize the profile matrix.
#' @param Q Number of categories
#' @param class_effect Class effect parameter (numeric value influencing class-based probabilities)
#' @param field_effect Field effect parameter (numeric value influencing field-based probabilities)
#' @return A normalized probability vector of length Q for initializing the field membership profile matrix
#' @noRd
init_field_membership_probs <- function(Q, class_effect, field_effect) {
  q_vals <- 1:Q / Q
  high_influence <- q_vals * class_effect
  low_influence <- q_vals[order(q_vals, decreasing = TRUE)] * field_effect
  probs <- high_influence + low_influence
  return(probs / sum(probs))
}

#' @title Create filter matrix for clustering
#' @description
#' Generates a filter matrix used in clustering algorithms. The matrix is constructed
#' with diagonal elements based on the number of classes, and off-diagonal elements
#' that create a tridiagonal structure. The first and last columns are normalized.
#' @param ncls Number of classes (integer value determining the matrix dimensions)
#' @return A square filter matrix of size ncls x ncls with normalized first and last columns
#' @noRd
#' @title Check for empty fields in Biclustering
#' @description
#' Checks if any fields have no items assigned after Biclustering estimation.
#' This can occur when the specified number of fields is too large for the data,
#' causing some fields to be empty. Emits a warning message if empty fields are detected.
#' @param fld Vector of field assignments for each item
#' @param nfld Number of specified fields
#' @noRd
check_empty_fields <- function(fld, nfld) {
  empty_fields <- setdiff(1:nfld, unique(fld))
  if (length(empty_fields) > 0) {
    n_effective <- nfld - length(empty_fields)
    warning(
      sprintf(
        "Only %d of %d specified fields have items assigned. Fields %s are empty. Consider reducing nfld.",
        n_effective, nfld, paste(empty_fields, collapse = ", ")
      ),
      call. = FALSE
    )
  }
}

create_filter_matrix <- function(ncls) {
  f0 <- ifelse(ncls < 5, 1.05 - 0.05 * ncls,
    ifelse(ncls < 10, 1.00 - 0.04 * ncls,
      0.80 - 0.02 * ncls
    )
  )
  f1 <- diag(0, ncls)
  f1[row(f1) == col(f1) - 1] <- (1 - f0) / 2
  Fil <- diag(rep(f0, ncls)) + t(f1) + f1
  Fil[, 1] <- Fil[, 1] / sum(Fil[, 1])
  Fil[, ncls] <- Fil[, ncls] / sum(Fil[, ncls])

  return(Fil)
}

# Helper functions for exametrika test comparisons
# This file is automatically sourced by testthat before test execution.

#' Load a Mathematica reference CSV fixture
#'
#' @param filename CSV filename (without path)
#' @return data.frame with check.names = FALSE
load_ref <- function(filename) {
  read.csv(
    test_path("fixtures", "mathematica_reference", filename),
    check.names = FALSE
  )
}

#' Load an auxiliary data CSV fixture
#'
#' @param filename CSV filename (without path)
#' @return data.frame with check.names = FALSE
load_aux <- function(filename) {
  read.csv(
    test_path("fixtures", "auxiliary_data", filename),
    check.names = FALSE
  )
}

#' Compare matrices with cleaned row/col names
#'
#' Strips rownames and colnames from both matrices before comparison.
#' @param actual Actual matrix or data.frame
#' @param expected Expected matrix or data.frame
#' @param tolerance Numeric tolerance (default 1e-4)
expect_matrix_equal <- function(actual, expected, tolerance = 1e-4) {
  actual <- as.matrix(actual)
  expected <- as.matrix(expected)
  rownames(actual) <- rownames(expected) <- NULL
  colnames(actual) <- colnames(expected) <- NULL
  expect_equal(actual, expected, tolerance = tolerance)
}

#' Compare numeric vectors extracted from data structures
#'
#' Unlist and convert both values to numeric vectors before comparison.
#' @param actual Actual value (vector, list, or data.frame)
#' @param expected Expected value (vector, list, or data.frame)
#' @param tolerance Numeric tolerance (default 1e-4)
expect_numeric_equal <- function(actual, expected, tolerance = 1e-4) {
  actual <- actual |>
    unlist() |>
    unname() |>
    as.numeric()
  expected <- expected |>
    unlist() |>
    unname() |>
    as.numeric()
  expect_equal(actual, expected, tolerance = tolerance)
}

---
title: "exametrika 日本語ガイド"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{exametrika 日本語ガイド}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

> 日本語チュートリアルは [こちら](https://kosugitti.github.io/slides/BMS2025spring/docs/) もご参照ください。
>
> **注意**: CRANビルド時間短縮のため、計算負荷の高いコード例は `eval=FALSE` としています。完全な実行結果は [pkgdownサイト](https://kosugitti.github.io/exametrika/articles/guide-ja.html) をご覧ください。

## 概要

`exametrika`パッケージは、教育テストデータを分析するための包括的なテストデータエンジニアリングツールを提供します。荘島（2022）で説明されている方法に基づき、このパッケージは研究者と実務家に以下の機能を提供します：

- テスト応答パターンと項目特性の分析
- 様々な心理測定モデルを用いた受験者の分類
- テストデータにおける潜在構造の調査
- 項目間の局所依存性の検討
- 項目関係のネットワーク分析

## 機能

本パッケージは以下の心理測定モデルと技法を実装しています：

#### 古典的手法
- 古典的テスト理論（CTT）
  - 項目の難易度と識別力
  - テストの信頼性と妥当性
- 項目反応理論（IRT）
  - 2PL、3PL、4PLモデル
  - 項目特性曲線
  - テスト情報関数

#### 潜在構造分析
- 潜在クラス分析（LCA）
  - クラスメンバーシップの推定
  - 項目応答プロファイル
- 潜在ランク分析（LRA）
  - 順序付き潜在クラス
  - ランク遷移確率
- バイクラスタリングとランククラスタリング
  - 項目と受験者の同時クラスタリング
  - フィールド固有の応答パターン
- 無限関係モデル（IRM）
  - 最適なクラス/フィールド数の決定
  - ノンパラメトリッククラスタリング

## モデルの概要

### 局所依存モデル

本パッケージは、テストデータにおける局所依存性をモデル化する3つの相補的なアプローチを実装しています：

1. **局所依存潜在ランク分析（LDLRA）**
   - 項目間の依存関係が異なる習熟度ランクでどのように変化するかを分析
   - 項目間の関係が学習者の能力レベルによって変化すると予想される場合に適している
   - LRAとベイジアンネットワークの長所を組み合わせたモデル

2. **局所依存バイクラスタリング（LDB）**
   - 各ランク内における項目フィールド間の関係に焦点
   - 項目が自然なグループ（フィールド）を形成し、階層的な関係を持つ場合に最適
   - バイクラスタリングとフィールドレベルの依存構造を統合

3. **バイクラスターネットワークモデル（BINET）**
   - 各フィールド内でのクラス遷移を検討
   - 複雑なクラス進行パターンの理解に最適
   - バイクラスタリングとクラスレベルのネットワーク分析を組み合わせたモデル

### モデル選択ガイド

| モデル | 主な焦点 | 適用場面 |
|--------|----------|----------|
| LDLRA  | 項目レベルの依存関係 | 項目関係が習熟度によって変化する場合 |
| LDB    | フィールドレベルの構造 | 項目が自然なグループを形成し依存関係を持つ場合 |
| BINET  | クラス進行 | フィールド内に複雑な学習パターンが存在する場合 |

## インストール

```{r install, eval=FALSE}
# CRANからインストール
install.packages("exametrika")

# または開発版をGitHubからインストール
if (!require("devtools")) install.packages("devtools")
devtools::install_github("kosugitti/exametrika")
```

### 依存パッケージ

本パッケージには以下が必要です：

+ R (>= 4.1.0)
+ igraph (ネットワーク分析用)
+ その他の依存パッケージは自動的にインストールされます

## データ形式と使用方法

### 基本的な使用方法

```{r setup-library, message=FALSE, warning=FALSE}
library(exametrika)
```

### データ要件

Exametrikaは2値データと多値データの両方に対応しています：

- 2値データ（0/1）
  - 0：誤答
  - 1：正答
- 多値データ
  - 順序のある反応カテゴリ
  - 複数の得点レベル
- 欠測値
  - NA値がサポートされています
  - カスタムの欠測値コードを指定可能

### データフォーマット

`dataFormat`関数は分析用の入力データを前処理します：

```{r example-data-format}
data <- dataFormat(J15S500)
str(data)
```

### サンプルデータセット

本パッケージには、荘島（2022）からの様々なサンプルデータセットが学習用に含まれています：

- **命名規則**：JxxSxxx形式（J：項目数、S：サンプルサイズ）

| データセット | 項目数 | 受験者数 | 種別 | 用途 |
|-------------|-------:|---------:|------|------|
| J5S10 | 5 | 10 | 2値 | クイックテスト |
| J5S1000 | 5 | 1,000 | 順序 | GRM |
| J12S5000 | 12 | 5,000 | 2値 | LDLRA |
| J15S500 | 15 | 500 | 2値 | IRT, LCA |
| J15S3810 | 15 | 3,810 | 順序(4件法) | 順序LRA |
| J20S400 | 20 | 400 | 2値 | BNM |
| J20S600 | 20 | 600 | 名義(4cat) | 名義Biclustering |
| J35S500 | 35 | 500 | 順序(5件法) | 順序Biclustering |
| J35S515 | 35 | 515 | 2値 | Biclustering, Network |
| J35S5000 | 35 | 5,000 | 多肢選択 | 評定LRA |

## 使用例

### テスト統計量

```{r results-test-statistics, message=FALSE, warning=FALSE}
TestStatistics(J15S500)
```

### 項目統計量

```{r results-item-statistics, message=FALSE, warning=FALSE}
ItemStatistics(J15S500)
```

### CTT

```{r results-ctt, message=FALSE, warning=FALSE}
CTT(J15S500)
```

### IRT

IRT関数は、ロジスティックモデルを用いてパラメータを推定します。`model`オプションで指定でき、2PL、3PL、4PLモデルに対応しています。

```{r model-irt, eval=FALSE}
result.IRT <- IRT(J15S500, model = 3)
result.IRT
```

```{r results-irt-ability, eval=FALSE}
head(result.IRT$ability)
```

```{r plot-irt-curves, eval=FALSE}
plot(result.IRT, type = "IRF", items = 1:6, nc = 2, nr = 3)
plot(result.IRT, type = "IRF", overlay = TRUE)
plot(result.IRT, type = "IIC", items = 1:6, nc = 2, nr = 3)
plot(result.IRT, type = "TRF")
plot(result.IRT, type = "TIC")
```

### GRM

Graded Response Model（Samejima,1969）はIRTを多値反応モデルに展開したものです。

```{r GRM, eval=FALSE}
result.GRM <- GRM(J5S1000)
result.GRM
```

```{r GRM-plot, eval=FALSE}
plot(result.GRM, type = "IRF", nc = 2)
plot(result.GRM, type = "IIF", nc = 2)
plot(result.GRM, type = "TIF")
```

### LCA

潜在クラス分析では、データセットとクラス数の指定が必要です。

```{r model-lca, eval=FALSE}
result.LCA <- LCA(J15S500, ncls = 5)
head(result.LCA$Students)
```

```{r plot-lca, eval=FALSE}
plot(result.LCA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LCA, type = "CMP", students = 1:9, nc = 3, nr = 3)
plot(result.LCA, type = "TRP")
plot(result.LCA, type = "LCD")
```

### LRA

潜在ランク分析では、データセットとランク数の指定が必要です。

```{r model-lra, eval=FALSE}
result.LRA <- LRA(J15S500, nrank = 6)
head(result.LRA$Students)
```

```{r plot-lra, eval=FALSE}
plot(result.LRA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LRA, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.LRA, type = "TRP")
plot(result.LRA, type = "LRD")
```

### LRA 順序尺度データへの適用

```{r model-lra-ordinal, eval=FALSE}
result.LRAord <- LRA(J15S3810, nrank = 3, mic = TRUE)
```

```{r plot-lra-ordinal-score, eval=FALSE}
plot(result.LRAord, type = "ScoreFreq")
plot(result.LRAord, type = "ScoreRank")
```

```{r plot-lra-ordinal-icbr-icrp, eval=FALSE}
plot(result.LRAord, type = "ICBR", items = 1:4, nc = 2, nr = 2)
plot(result.LRAord, type = "ICRP", items = 1:4, nc = 2, nr = 2)
```

```{r plot-lra-ordinal-rmp, eval=FALSE}
plot(result.LRAord, type = "RMP", students = 1:9, nc = 3, nr = 3)
```

### LRA 名義尺度データへの適用

```{r model-lra-rated, eval=FALSE}
result.LRArated <- LRA(J35S5000, nrank = 10, mic = TRUE)
```

```{r plot-lra-rated, eval=FALSE}
plot(result.LRArated, type = "ScoreFreq")
plot(result.LRArated, type = "ScoreRank")
plot(result.LRArated, type = "ICRP", items = 1:4, nc = 2, nr = 2)
```

### バイクラスタリング

```{r model-biclustering, eval=FALSE}
Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
```

```{r model-ranklustering, eval=FALSE}
result.Ranklustering <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")
plot(result.Ranklustering, type = "Array")
plot(result.Ranklustering, type = "FRP", nc = 2, nr = 3)
plot(result.Ranklustering, type = "RRV")
plot(result.Ranklustering, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.Ranklustering, type = "LRD")
```

### グリッドサーチ

```{r grid-search, eval=FALSE}
result <- GridSearch(J35S515, method = "R", max_ncls = 10, max_nfld = 10, index = "BIC")
result$optimal_ncls
result$optimal_nfld
plot(result$optimal_result, type = "Array")
```

### 無限関係モデル

```{r model-irm, eval=FALSE}
result.IRM <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result.IRM, type = "Array")
plot(result.IRM, type = "FRP", nc = 3)
plot(result.IRM, type = "TRP")
```

### 多段階反応データ用のバイクラスタリング

#### 順序尺度データ

```{r bic-poly-ord, eval=FALSE}
result.B.ord <- Biclustering(J35S500, ncls = 5, nfld = 5, method = "R")
result.B.ord
plot(result.B.ord, type = "Array")
plot(result.B.ord, type = "FRP", nc = 3, nr = 2)
plot(result.B.ord, type = "FCRP", nc = 3, nr = 2)
plot(result.B.ord, type = "FCRP", style = "bar", nc = 3, nr = 2)
plot(result.B.ord, type = "FCBR", nc = 3, nr = 2)
plot(result.B.ord, type = "ScoreField")
plot(result.B.ord, type = "RRV")
```

#### 名義尺度データ

```{r bic-poly-nom, eval=FALSE}
result.B.nom <- Biclustering(J20S600, ncls = 5, nfld = 4)
result.B.nom
plot(result.B.nom, type = "Array")
plot(result.B.nom, type = "FRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", style = "bar", nc = 2, nr = 2)
plot(result.B.nom, type = "ScoreField")
plot(result.B.nom, type = "RRV")
```

### ベイジアンネットワークモデル

```{r setup-igraph, eval=FALSE}
library(igraph)
DAG <- matrix(
  c(
    "Item01", "Item02", "Item02", "Item03", "Item02", "Item04",
    "Item03", "Item05", "Item04", "Item05"
  ),
  ncol = 2, byrow = TRUE
)
g <- igraph::graph_from_data_frame(DAG)
adj_mat <- as.matrix(igraph::get.adjacency(g))
```

```{r model-bnm, eval=FALSE}
result.BNM <- BNM(J5S10, adj_matrix = adj_mat)
result.BNM
```

#### 遺伝的アルゴリズムによる構造学習

```{r model-ga-bnm, eval=FALSE}
BNM_GA(J5S10,
  population = 20, Rs = 0.5, Rm = 0.002, maxParents = 2,
  maxGeneration = 100, crossover = 2, elitism = 2
)
```

#### PBILによる構造学習

```{r model-pbil-bnm, eval=FALSE}
BNM_PBIL(J5S10,
  population = 20, Rs = 0.5, Rm = 0.005, maxParents = 2,
  alpha = 0.05, estimate = 4
)
```

### 局所依存潜在ランク分析

```{r setup-ldlra-dag, eval=FALSE}
DAG_dat <- matrix(c(
  "From", "To", "Rank",
  "Item01", "Item02", 1, "Item04", "Item05", 1,
  "Item01", "Item02", 2, "Item02", "Item03", 2,
  "Item04", "Item05", 2, "Item08", "Item09", 2,
  "Item08", "Item10", 2, "Item09", "Item10", 2, "Item08", "Item11", 2,
  "Item01", "Item02", 3, "Item02", "Item03", 3,
  "Item04", "Item05", 3, "Item08", "Item09", 3,
  "Item08", "Item10", 3, "Item09", "Item10", 3, "Item08", "Item11", 3,
  "Item02", "Item03", 4, "Item04", "Item06", 4,
  "Item04", "Item07", 4, "Item05", "Item06", 4,
  "Item05", "Item07", 4, "Item08", "Item10", 4,
  "Item08", "Item11", 4, "Item09", "Item11", 4,
  "Item02", "Item03", 5, "Item04", "Item06", 5,
  "Item04", "Item07", 5, "Item05", "Item06", 5,
  "Item05", "Item07", 5, "Item09", "Item11", 5,
  "Item10", "Item11", 5, "Item10", "Item12", 5
), ncol = 3, byrow = TRUE)

edgeFile <- tempfile(fileext = ".csv")
write.csv(DAG_dat, edgeFile, row.names = FALSE, quote = TRUE)
```

```{r model-ldlra, eval=FALSE}
result.LDLRA <- LDLRA(J12S5000, ncls = 5, adj_file = edgeFile)
result.LDLRA
```

```{r plot-ldlra, eval=FALSE}
plot(result.LDLRA, type = "IRP", nc = 4, nr = 3)
plot(result.LDLRA, type = "TRP")
plot(result.LDLRA, type = "LRD")
```

```{r cleanup-ldlra, include=FALSE}
if (exists("edgeFile")) unlink(edgeFile)
```

### 局所依存バイクラスタリング

```{r setup-ldb, eval=FALSE}
conf <- c(
  1, 6, 6, 8, 9, 9, 4, 7, 7, 7, 5, 8, 9, 10, 10,
  9, 9, 10, 10, 10, 2, 2, 3, 3, 5, 5, 6, 9, 9, 10,
  1, 1, 7, 9, 10
)

edges_data <- data.frame(
  "From Field (Parent) >>>" = c(6, 4, 5, 1, 1, 4, 3, 4, 6, 2, 4, 4, 3, 6, 4, 1, 7, 9, 6, 7),
  ">>> To Field (Child)" = c(8, 7, 8, 7, 2, 5, 5, 8, 8, 4, 6, 7, 5, 8, 5, 8, 10, 10, 8, 9),
  "At Class/Rank (Locus)" = c(2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5)
)

edgeFile <- tempfile(fileext = ".csv")
write.csv(edges_data, file = edgeFile, row.names = FALSE)
```

```{r model-ldb, eval=FALSE}
result.LDB <- LDB(U = J35S515, ncls = 5, conf = conf, adj_file = edgeFile)
result.LDB
```

```{r plot-ldb, eval=FALSE}
plot(result.LDB, type = "Array")
plot(result.LDB, type = "TRP")
plot(result.LDB, type = "LRD")
plot(result.LDB, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.LDB, type = "FRP", nc = 3, nr = 2)
plot(result.LDB, type = "FieldPIRP")
```

```{r cleanup-ldb, include=FALSE}
if (exists("edgeFile")) unlink(edgeFile)
```

### バイクラスターネットワークモデル

```{r setup-binet, eval=FALSE}
conf <- c(
  1, 5, 5, 5, 9, 9, 6, 6, 6, 6, 2, 7, 7, 11, 11,
  7, 7, 12, 12, 12, 2, 2, 3, 3, 4, 4, 4, 8, 8, 12,
  1, 1, 6, 10, 10
)

edges_data <- data.frame(
  "From Class (Parent) >>>" = c(1, 2, 3, 4, 5, 7, 2, 4, 6, 8, 10, 6, 6, 11, 8, 9, 12),
  ">>> To Class (Child)" = c(2, 4, 5, 5, 6, 11, 3, 7, 9, 12, 12, 10, 8, 12, 12, 11, 13),
  "At Field (Locus)" = c(1, 2, 2, 3, 4, 4, 5, 5, 5, 5, 5, 7, 8, 8, 9, 9, 12)
)

edgeFile <- tempfile(fileext = ".csv")
write.csv(edges_data, file = edgeFile, row.names = FALSE)
```

```{r model-binet, eval=FALSE}
result.BINET <- BINET(
  U = J35S515, ncls = 13, nfld = 12,
  conf = conf, adj_file = edgeFile
)
print(result.BINET)
```

```{r plot-binet, eval=FALSE}
plot(result.BINET, type = "Array")
plot(result.BINET, type = "TRP")
plot(result.BINET, type = "LRD")
plot(result.BINET, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.BINET, type = "FRP", nc = 3, nr = 2)
plot(result.BINET, type = "LDPSR", nc = 3, nr = 2)
```

```{r cleanup-binet, include=FALSE}
if (exists("edgeFile")) unlink(edgeFile)
```

## 参考文献

Shojima, K. (2022) *Test Data Engineering: Latent Rank Analysis, Biclustering, and Bayesian Network* (Behaviormetrics: Quantitative Approaches to Human Behavior, 13), Springer.

---
title: "Biclustering and Ranklustering"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Biclustering and Ranklustering}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

> **Note**: Some computationally intensive examples below are shown with `eval=FALSE` to keep CRAN build times short. For full rendered output, see the [pkgdown site](https://kosugitti.github.io/exametrika/articles/biclustering.html).

```{r library, message=FALSE, warning=FALSE}
library(exametrika)
```

## Biclustering and Ranklustering

Biclustering and Ranklustering simultaneously cluster items into fields and examinees into classes/ranks. The difference is specified via the `method` option:

- `method = "B"`: Biclustering (no filtering matrix)
- `method = "R"`: Ranklustering (with filtering matrix for ordered ranks)

### Biclustering

```{r model-biclustering, message=FALSE, warning=FALSE}
Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
```

### Ranklustering

```{r model-ranklustering, message=FALSE, warning=FALSE}
result.Ranklustering <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")
```

```{r plot-ranklustering, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.Ranklustering, type = "Array")
plot(result.Ranklustering, type = "FRP", nc = 2, nr = 3)
plot(result.Ranklustering, type = "RRV")
plot(result.Ranklustering, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.Ranklustering, type = "LRD")
```

## Finding Optimal Number of Classes and Fields

### Grid Search

`GridSearch()` systematically evaluates multiple parameter combinations and selects the best-fitting model:

```{r grid-search, eval=FALSE}
result <- GridSearch(J35S515, method = "R", max_ncls = 10, max_nfld = 10, index = "BIC")
result$optimal_ncls
result$optimal_nfld
result$optimal_result
```

### Infinite Relational Model (IRM)

The IRM uses the Chinese Restaurant Process to automatically determine the optimal number of fields and classes:

```{r model-irm, eval=FALSE}
result.IRM <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result.IRM, type = "Array")
plot(result.IRM, type = "FRP", nc = 3)
plot(result.IRM, type = "TRP")
```

## Biclustering for Polytomous Data

### Ordinal Data

```{r bic-ord, eval=FALSE}
result.B.ord <- Biclustering(J35S500, ncls = 5, nfld = 5, method = "R")
result.B.ord
plot(result.B.ord, type = "Array")
```

FRP (Field Reference Profile) shows the expected score per field across latent ranks:

```{r bic-ord-frp, eval=FALSE}
plot(result.B.ord, type = "FRP", nc = 3, nr = 2)
```

FCRP (Field Category Response Profile) shows category probabilities across ranks. The `style` parameter can be `"line"` or `"bar"`:

```{r bic-ord-fcrp, eval=FALSE}
plot(result.B.ord, type = "FCRP", nc = 3, nr = 2)
plot(result.B.ord, type = "FCRP", style = "bar", nc = 3, nr = 2)
```

FCBR (Field Cumulative Boundary Reference) shows cumulative boundary probabilities (ordinal only):

```{r bic-ord-fcbr, eval=FALSE}
plot(result.B.ord, type = "FCBR", nc = 3, nr = 2)
```

ScoreField and RRV plots:

```{r bic-ord-scorefield, eval=FALSE}
plot(result.B.ord, type = "ScoreField")
plot(result.B.ord, type = "RRV")
```

### Nominal Data

```{r bic-nom, eval=FALSE}
result.B.nom <- Biclustering(J20S600, ncls = 5, nfld = 4)
result.B.nom
plot(result.B.nom, type = "Array")
```

Nominal Biclustering supports FRP, FCRP, ScoreField, and RRV (but not FCBR):

```{r bic-nom-plots, eval=FALSE}
plot(result.B.nom, type = "FRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", style = "bar", nc = 2, nr = 2)
plot(result.B.nom, type = "ScoreField")
plot(result.B.nom, type = "RRV")
```

### Rated Data (Multiple-Choice with Correct Answers)

Rated data has both multiple response categories and correct answers. `Biclustering.rated()` internally runs nominal Biclustering, then sorts classes by correct response rate:

```{r bic-rated, eval=FALSE}
result.B.rated <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R", maxiter = 300)
result.B.rated
plot(result.B.rated, type = "Array")
plot(result.B.rated, type = "FRP", nc = 3, nr = 1)
```

Two layers of fit indices are reported: binary (with CFI/RMSEA) and nominal (AIC/BIC/CAIC only). Access them via `result$TestFitIndices` and `result$TestFitIndices_nominal`.

### Rated IRM

The IRM also supports rated data:

```{r irm-rated, eval=FALSE}
result.IRM.rated <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result.IRM.rated, type = "Array")
plot(result.IRM.rated, type = "FRP", nc = 3, nr = 1)
```

## Distractor Analysis

`DistractorAnalysis()` examines how examinees in each rank/class respond to each item's categories. It computes observed frequencies, proportions, chi-square statistics, p-values, and Cramer's V (effect size) for each item-by-rank cell.

```{r distractor, eval=FALSE}
result.B.rated <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R", maxiter = 300)
da <- DistractorAnalysis(result.B.rated)

# Full output (grouped by field for Biclustering)
print(da)

# Filter by items and/or ranks
print(da, items = 1:7, ranks = c(1, 5))

# Plot distractor bar charts
plot(da, items = 1:6, nc = 3, nr = 2)
```

`DistractorAnalysis()` also works with `LRA()` results for rated data:

```{r distractor-lra, eval=FALSE}
result.LRA.rated <- LRA(J21S300, nrank = 5, mic = TRUE)
da_lra <- DistractorAnalysis(result.LRA.rated)
print(da_lra, items = 1:3)
plot(da_lra, items = 1:6, nc = 3, nr = 2)
```

## Reference

Shojima, K. (2022). *Test Data Engineering*. Springer.

---
title: "Biclustering and Ranklustering"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Biclustering and Ranklustering}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

> **Note**: Some computationally intensive examples below are shown with `eval=FALSE` to keep CRAN build times short. For full rendered output, see the [pkgdown site](https://kosugitti.github.io/exametrika/articles/biclustering.html).

```{r library, message=FALSE, warning=FALSE}
library(exametrika)
```

## Biclustering and Ranklustering

Biclustering and Ranklustering simultaneously cluster items into fields and examinees into classes/ranks. The difference is specified via the `method` option:

- `method = "B"`: Biclustering (no filtering matrix)
- `method = "R"`: Ranklustering (with filtering matrix for ordered ranks)

### Biclustering

```{r model-biclustering, message=FALSE, warning=FALSE}
Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
```

### Ranklustering

```{r model-ranklustering, message=FALSE, warning=FALSE}
result.Ranklustering <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")
```

```{r plot-ranklustering, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.Ranklustering, type = "Array")
plot(result.Ranklustering, type = "FRP", nc = 2, nr = 3)
plot(result.Ranklustering, type = "RRV")
plot(result.Ranklustering, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.Ranklustering, type = "LRD")
```

## Finding Optimal Number of Classes and Fields

### Grid Search

`GridSearch()` systematically evaluates multiple parameter combinations and selects the best-fitting model:

```{r grid-search, eval=FALSE}
result <- GridSearch(J35S515, method = "R", max_ncls = 10, max_nfld = 10, index = "BIC")
result$optimal_ncls
result$optimal_nfld
result$optimal_result
```

### Infinite Relational Model (IRM)

The IRM uses the Chinese Restaurant Process to automatically determine the optimal number of fields and classes:

```{r model-irm, eval=FALSE}
result.IRM <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result.IRM, type = "Array")
plot(result.IRM, type = "FRP", nc = 3)
plot(result.IRM, type = "TRP")
```

## Biclustering for Polytomous Data

### Ordinal Data

```{r bic-ord, eval=FALSE}
result.B.ord <- Biclustering(J35S500, ncls = 5, nfld = 5, method = "R")
result.B.ord
plot(result.B.ord, type = "Array")
```

FRP (Field Reference Profile) shows the expected score per field across latent ranks:

```{r bic-ord-frp, eval=FALSE}
plot(result.B.ord, type = "FRP", nc = 3, nr = 2)
```

FCRP (Field Category Response Profile) shows category probabilities across ranks. The `style` parameter can be `"line"` or `"bar"`:

```{r bic-ord-fcrp, eval=FALSE}
plot(result.B.ord, type = "FCRP", nc = 3, nr = 2)
plot(result.B.ord, type = "FCRP", style = "bar", nc = 3, nr = 2)
```

FCBR (Field Cumulative Boundary Reference) shows cumulative boundary probabilities (ordinal only):

```{r bic-ord-fcbr, eval=FALSE}
plot(result.B.ord, type = "FCBR", nc = 3, nr = 2)
```

ScoreField and RRV plots:

```{r bic-ord-scorefield, eval=FALSE}
plot(result.B.ord, type = "ScoreField")
plot(result.B.ord, type = "RRV")
```

### Nominal Data

```{r bic-nom, eval=FALSE}
result.B.nom <- Biclustering(J20S600, ncls = 5, nfld = 4)
result.B.nom
plot(result.B.nom, type = "Array")
```

Nominal Biclustering supports FRP, FCRP, ScoreField, and RRV (but not FCBR):

```{r bic-nom-plots, eval=FALSE}
plot(result.B.nom, type = "FRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", nc = 2, nr = 2)
plot(result.B.nom, type = "FCRP", style = "bar", nc = 2, nr = 2)
plot(result.B.nom, type = "ScoreField")
plot(result.B.nom, type = "RRV")
```

### Rated Data (Multiple-Choice with Correct Answers)

Rated data has both multiple response categories and correct answers. `Biclustering.rated()` internally runs nominal Biclustering, then sorts classes by correct response rate:

```{r bic-rated, message=FALSE, warning=FALSE}
result.B.rated <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R", maxiter = 300)
result.B.rated
plot(result.B.rated, type = "Array")
plot(result.B.rated, type = "FRP", nc = 3, nr = 1)
```

Two layers of fit indices are reported: binary (with CFI/RMSEA) and nominal (AIC/BIC/CAIC only). Access them via `result$TestFitIndices` and `result$TestFitIndices_nominal`.

### Rated IRM

The IRM also supports rated data:

```{r irm-rated, message=FALSE, warning=FALSE}
result.IRM.rated <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, verbose = FALSE)
plot(result.IRM.rated, type = "Array")
plot(result.IRM.rated, type = "FRP", nc = 3, nr = 1)
```

## Distractor Analysis

`DistractorAnalysis()` examines how examinees in each rank/class respond to each item's categories. It computes observed frequencies, proportions, chi-square statistics, p-values, and Cramer's V (effect size) for each item-by-rank cell.

```{r distractor, eval=FALSE}
result.B.rated <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R", maxiter = 300)
da <- DistractorAnalysis(result.B.rated)

# Full output (grouped by field for Biclustering)
print(da)

# Filter by items and/or ranks
print(da, items = 1:7, ranks = c(1, 5))

# Plot distractor bar charts
plot(da, items = 1:6, nc = 3, nr = 2)
```

`DistractorAnalysis()` also works with `LRA()` results for rated data:

```{r distractor-lra, eval=FALSE}
result.LRA.rated <- LRA(J21S300, nrank = 5, mic = TRUE)
da_lra <- DistractorAnalysis(result.LRA.rated)
print(da_lra, items = 1:3)
plot(da_lra, items = 1:6, nc = 3, nr = 2)
```

## Reference

Shojima, K. (2022). *Test Data Engineering*. Springer.

---
title: "Item Response Theory (IRT)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Item Response Theory (IRT)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

```{r library, message=FALSE, warning=FALSE}
library(exametrika)
```

## IRT for Binary Data

The `IRT()` function estimates item parameters using logistic models. It supports 2PL, 3PL, and 4PL models via the `model` option.

```{r model-irt, message=FALSE, warning=FALSE}
result.IRT <- IRT(J15S500, model = 3)
result.IRT
```

The estimated ability parameters for each examinee are included in the returned object:

```{r results-irt-ability, message=FALSE, warning=FALSE}
head(result.IRT$ability)
```

### Plot Types

IRT provides several plot types:

- **IRF**: Item Response Function (Item Characteristic Curves)
- **IIC**: Item Information Curves
- **TRF**: Test Response Function
- **TIC**: Test Information Curve

Items can be specified using the `items` argument. The layout is controlled by `nr` (rows) and `nc` (columns).

```{r plot-irt-irf, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.IRT, type = "IRF", items = 1:6, nc = 2, nr = 3)
```

```{r plot-irt-overlay, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.IRT, type = "IRF", overlay = TRUE)
```

```{r plot-irt-iic, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.IRT, type = "IIC", items = 1:6, nc = 2, nr = 3)
```

```{r plot-irt-trf, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.IRT, type = "TRF")
```

```{r plot-irt-tic, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.IRT, type = "TIC")
```

## GRM: Graded Response Model

The Graded Response Model (Samejima, 1969) extends IRT to polytomous response data. It can be applied using the `GRM()` function.

```{r grm, message=FALSE, warning=FALSE}
result.GRM <- GRM(J5S1000)
result.GRM
```

GRM supports similar plot types as IRT:

```{r grm-irf, fig.width=7, fig.height=8, message=FALSE, warning=FALSE}
plot(result.GRM, type = "IRF", nc = 2)
```

```{r grm-iif, fig.width=7, fig.height=8, message=FALSE, warning=FALSE}
plot(result.GRM, type = "IIF", nc = 2)
```

```{r grm-tif, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.GRM, type = "TIF")
```

## References

- Shojima, K. (2022). *Test Data Engineering*. Springer.
- Samejima, F. (1969). Estimation of latent ability using a response pattern of graded scores. *Psychometrika*, 34(S1), 1--97.

---
title: "Getting Started with exametrika"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with exametrika}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

## Overview

The `exametrika` package provides comprehensive Test Data Engineering tools for analyzing educational test data. Based on the methods described in Shojima (2022), this package enables researchers and practitioners to:

- Analyze test response patterns and item characteristics
- Classify respondents using various psychometric models
- Investigate latent structures in test data
- Examine local dependencies between items
- Perform network analysis of item relationships

## Installation

```{r install, eval=FALSE}
# Install from CRAN
install.packages("exametrika")

# Or install the development version from GitHub
if (!require("devtools")) install.packages("devtools")
devtools::install_github("kosugitti/exametrika")
```

## Data Format

```{r library, message=FALSE, warning=FALSE}
library(exametrika)
```

### Data Requirements

Exametrika accepts both binary and polytomous response data:

- **Binary data** (0/1): 0 = incorrect, 1 = correct
- **Polytomous data**: ordinal response categories or multiple score levels
- **Missing values**: NA values supported; custom missing value codes can be specified

### Data Formatting

The `dataFormat()` function processes input data before analysis:

```{r example-data-format}
# Format raw data for analysis
data <- dataFormat(J15S500)
str(data)
```

### Sample Datasets

The package includes sample datasets from Shojima (2022). The naming convention is `JxxSxxx` where J = number of items and S = sample size.

| Dataset | Items | Examinees | Type | Use Case |
|---------|------:|----------:|------|----------|
| J5S10 | 5 | 10 | Binary | Quick testing |
| J5S1000 | 5 | 1,000 | Ordinal | GRM examples |
| J12S5000 | 12 | 5,000 | Binary | LDLRA examples |
| J15S500 | 15 | 500 | Binary | IRT, LCA examples |
| J15S3810 | 15 | 3,810 | Ordinal (4-point) | Ordinal LRA |
| J20S400 | 20 | 400 | Binary | BNM examples |
| J20S600 | 20 | 600 | Nominal (4-cat) | Nominal Biclustering |
| J35S500 | 35 | 500 | Ordinal (5-cat) | Ordinal Biclustering |
| J35S515 | 35 | 515 | Binary | Biclustering, network models |
| J35S5000 | 35 | 5,000 | Multiple-choice | Rated LRA |
| J50S100 | 50 | 100 | Binary | Small sample testing |

## Basic Statistics

### Test Statistics

```{r test-statistics, message=FALSE, warning=FALSE}
TestStatistics(J15S500)
```

### Item Statistics

```{r item-statistics, message=FALSE, warning=FALSE}
ItemStatistics(J15S500)
```

### Classical Test Theory

```{r ctt, message=FALSE, warning=FALSE}
CTT(J15S500)
```

## Next Steps

- [Item Response Theory](irt.html): IRT and GRM models
- [Latent Class and Rank Analysis](latent-class-rank.html): LCA and LRA
- [Biclustering](biclustering.html): Biclustering, Ranklustering, and IRM
- [Network Models](network-models.html): BNM, LDLRA, LDB, and BINET

## Reference

Shojima, Kojiro (2022) *Test Data Engineering: Latent Rank Analysis, Biclustering, and Bayesian Network* (Behaviormetrics: Quantitative Approaches to Human Behavior, 13), Springer.

---
title: "Latent Class and Rank Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Latent Class and Rank Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

> **Note**: Some computationally intensive examples below are shown with `eval=FALSE` to keep CRAN build times short. For full rendered output, see the [pkgdown site](https://kosugitti.github.io/exametrika/articles/latent-class-rank.html).

```{r library, message=FALSE, warning=FALSE}
library(exametrika)
```

## Latent Class Analysis (LCA)

LCA classifies examinees into unordered latent classes. Specify the dataset and the number of classes.

```{r model-lca, message=FALSE, warning=FALSE}
LCA(J15S500, ncls = 5)
```

The Class Membership Matrix indicates which latent class each examinee belongs to:

```{r results-lca-membership, message=FALSE, warning=FALSE}
result.LCA <- LCA(J15S500, ncls = 5)
head(result.LCA$Students)
```

### LCA Plot Types

- **IRP**: Item Reference Profile
- **CMP**: Class Membership Profile
- **TRP**: Test Reference Profile
- **LCD**: Latent Class Distribution

```{r plot-lca, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.LCA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LCA, type = "CMP", students = 1:9, nc = 3, nr = 3)
plot(result.LCA, type = "TRP")
plot(result.LCA, type = "LCD")
```

## Latent Rank Analysis (LRA)

LRA is similar to LCA but assumes an ordering among the latent classes (ranks). Specify the dataset and the number of ranks.

```{r model-lra, message=FALSE, warning=FALSE}
LRA(J15S500, nrank = 6)
```

Rank membership probabilities and rank-up/rank-down odds are calculated:

```{r results-lra-membership, message=FALSE, warning=FALSE}
result.LRA <- LRA(J15S500, nrank = 6)
head(result.LRA$Students)
```

```{r plot-lra, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
plot(result.LRA, type = "IRP", items = 1:6, nc = 2, nr = 3)
plot(result.LRA, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.LRA, type = "TRP")
plot(result.LRA, type = "LRD")
```

## LRA for Ordinal Data

LRA can also handle ordinal scale data. The `mic` option enforces monotonic increasing constraints.

```{r model-lra-ordinal, eval=FALSE}
result.LRAord <- LRA(J15S3810, nrank = 3, mic = TRUE)
```

Score-rank relationship visualizations:

```{r plot-lra-ordinal-score, eval=FALSE}
plot(result.LRAord, type = "ScoreFreq")
plot(result.LRAord, type = "ScoreRank")
```

Item-rank relationship plots:

- **ICBR**: Item Category Boundary Reference -- cumulative probability curves for each category threshold
- **ICRP**: Item Category Response Profile -- probability of each response category across ranks

```{r plot-lra-ordinal-icbr, eval=FALSE}
plot(result.LRAord, type = "ICBR", items = 1:4, nc = 2, nr = 2)
plot(result.LRAord, type = "ICRP", items = 1:4, nc = 2, nr = 2)
```

Rank membership profiles for individual examinees:

```{r plot-lra-ordinal-rmp, eval=FALSE}
plot(result.LRAord, type = "RMP", students = 1:9, nc = 3, nr = 3)
```

## LRA for Rated/Nominal Data

For multiple-choice tests (nominal scale), LRA can analyze response patterns including distractor choices.

```{r lra-rated, eval=FALSE}
result.LRArated <- LRA(J35S5000, nrank = 10, mic = TRUE)
```

```{r plot-lra-rated-score, eval=FALSE}
plot(result.LRArated, type = "ScoreFreq")
plot(result.LRArated, type = "ScoreRank")
```

```{r plot-lra-rated-icrp, eval=FALSE}
plot(result.LRArated, type = "ICRP", items = 1:4, nc = 2, nr = 2)
```

## Reference

Shojima, K. (2022). *Test Data Engineering*. Springer.

---
title: "Bayesian Network and Local Dependence Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Bayesian Network and Local Dependence Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  out.width = "100%"
)
```

> **Note**: Most examples in this vignette are shown with `eval=FALSE` to keep CRAN build times short. For full rendered output, see the [pkgdown site](https://kosugitti.github.io/exametrika/articles/network-models.html).

```{r library, message=FALSE, warning=FALSE}
library(exametrika)
library(igraph)
```

## Bayesian Network Model (BNM)

BNM represents conditional probabilities between items in a network format. A Directed Acyclic Graph (DAG) must be provided via `adj_matrix`, `adj_file`, or `g` (igraph object).

### Creating the Graph

```{r setup-igraph, message=FALSE, warning=FALSE}
DAG <- matrix(
  c(
    "Item01", "Item02",
    "Item02", "Item03",
    "Item02", "Item04",
    "Item03", "Item05",
    "Item04", "Item05"
  ),
  ncol = 2, byrow = TRUE
)

# Graph object
g <- igraph::graph_from_data_frame(DAG)
g

# Adjacency matrix
adj_mat <- as.matrix(igraph::get.adjacency(g))
print(adj_mat)
```

### Running BNM

```{r model-bnm, message=FALSE, warning=FALSE}
result.BNM <- BNM(J5S10, adj_matrix = adj_mat)
result.BNM
```

### Structure Learning with Genetic Algorithm

`BNM_GA()` searches for a DAG suitable for the data using a genetic algorithm:

```{r model-ga-bnm, eval=FALSE}
BNM_GA(J5S10,
  population = 20, Rs = 0.5, Rm = 0.002, maxParents = 2,
  maxGeneration = 100, crossover = 2, elitism = 2
)
```

### Structure Learning with PBIL

```{r model-pbil-bnm, eval=FALSE}
BNM_PBIL(J5S10,
  population = 20, Rs = 0.5, Rm = 0.005, maxParents = 2,
  alpha = 0.05, estimate = 4
)
```

## Local Dependence Latent Rank Analysis (LDLRA)

LDLRA combines LRA and BNM to analyze how item dependency networks change across latent ranks. A graph must be specified for each rank.

### Setting Up Rank-Specific Graphs

Graphs can be provided via CSV file, adjacency matrix list, or igraph object list:

```{r setup-ldlra-dag, message=FALSE, warning=FALSE}
DAG_dat <- matrix(c(
  "From", "To", "Rank",
  "Item01", "Item02", 1,
  "Item04", "Item05", 1,
  "Item01", "Item02", 2,
  "Item02", "Item03", 2,
  "Item04", "Item05", 2,
  "Item08", "Item09", 2,
  "Item08", "Item10", 2,
  "Item09", "Item10", 2,
  "Item08", "Item11", 2,
  "Item01", "Item02", 3,
  "Item02", "Item03", 3,
  "Item04", "Item05", 3,
  "Item08", "Item09", 3,
  "Item08", "Item10", 3,
  "Item09", "Item10", 3,
  "Item08", "Item11", 3,
  "Item02", "Item03", 4,
  "Item04", "Item06", 4,
  "Item04", "Item07", 4,
  "Item05", "Item06", 4,
  "Item05", "Item07", 4,
  "Item08", "Item10", 4,
  "Item08", "Item11", 4,
  "Item09", "Item11", 4,
  "Item02", "Item03", 5,
  "Item04", "Item06", 5,
  "Item04", "Item07", 5,
  "Item05", "Item06", 5,
  "Item05", "Item07", 5,
  "Item09", "Item11", 5,
  "Item10", "Item11", 5,
  "Item10", "Item12", 5
), ncol = 3, byrow = TRUE)

edgeFile <- tempfile(fileext = ".csv")
write.csv(DAG_dat, edgeFile, row.names = FALSE, quote = TRUE)
```

### Running LDLRA

```{r model-ldlra, eval=FALSE}
result.LDLRA <- LDLRA(J12S5000, ncls = 5, adj_file = edgeFile)
result.LDLRA
```

```{r plot-ldlra, eval=FALSE}
plot(result.LDLRA, type = "IRP", nc = 4, nr = 3)
plot(result.LDLRA, type = "TRP")
plot(result.LDLRA, type = "LRD")
```

```{r cleanup-ldlra, include=FALSE}
if (exists("edgeFile")) unlink(edgeFile)
```

### Structure Learning for LDLRA with PBIL

`LDLRA_PBIL()` learns item-interaction graphs for each rank automatically:

```{r model-pbil-ldlra, eval=FALSE}
result.LDLRA.PBIL <- LDLRA_PBIL(J35S515,
  seed = 123, ncls = 5, method = "R",
  elitism = 1, successiveLimit = 15
)
result.LDLRA.PBIL
```

## Local Dependence Biclustering (LDB)

LDB combines biclustering with Bayesian network models, analyzing relationships between item fields within each rank.

```{r setup-ldb, message=FALSE, warning=FALSE}
conf <- c(
  1, 6, 6, 8, 9, 9, 4, 7, 7, 7, 5, 8, 9, 10, 10,
  9, 9, 10, 10, 10, 2, 2, 3, 3, 5, 5, 6, 9, 9, 10,
  1, 1, 7, 9, 10
)

edges_data <- data.frame(
  "From Field (Parent) >>>" = c(
    6, 4, 5, 1, 1, 4,
    3, 4, 6, 2, 4, 4,
    3, 6, 4, 1,
    7, 9, 6, 7
  ),
  ">>> To Field (Child)" = c(
    8, 7, 8, 7, 2, 5,
    5, 8, 8, 4, 6, 7,
    5, 8, 5, 8,
    10, 10, 8, 9
  ),
  "At Class/Rank (Locus)" = c(
    2, 2, 2, 2, 2, 2,
    3, 3, 3, 3, 3, 3,
    4, 4, 4, 4,
    5, 5, 5, 5
  )
)

edgeFile <- tempfile(fileext = ".csv")
write.csv(edges_data, file = edgeFile, row.names = FALSE)
```

```{r model-ldb, eval=FALSE}
result.LDB <- LDB(U = J35S515, ncls = 5, conf = conf, adj_file = edgeFile)
result.LDB
```

```{r plot-ldb, eval=FALSE}
plot(result.LDB, type = "Array")
plot(result.LDB, type = "TRP")
plot(result.LDB, type = "LRD")
plot(result.LDB, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.LDB, type = "FRP", nc = 3, nr = 2)
```

FieldPIRP visualizes correct answer counts for each rank and field:

```{r plot-ldb-fieldpirp, eval=FALSE}
plot(result.LDB, type = "FieldPIRP")
```

```{r cleanup-ldb, include=FALSE}
if (exists("edgeFile")) unlink(edgeFile)
```

## Bicluster Network Model (BINET)

BINET combines biclustering with class-level network analysis. Unlike LDB where nodes are fields, in BINET the nodes represent classes.

```{r setup-binet, message=FALSE, warning=FALSE}
conf <- c(
  1, 5, 5, 5, 9, 9, 6, 6, 6, 6, 2, 7, 7, 11, 11,
  7, 7, 12, 12, 12, 2, 2, 3, 3, 4, 4, 4, 8, 8, 12,
  1, 1, 6, 10, 10
)

edges_data <- data.frame(
  "From Class (Parent) >>>" = c(
    1, 2, 3, 4, 5, 7,
    2, 4, 6, 8, 10,
    6, 6, 11, 8, 9, 12
  ),
  ">>> To Class (Child)" = c(
    2, 4, 5, 5, 6, 11,
    3, 7, 9, 12, 12,
    10, 8, 12, 12, 11, 13
  ),
  "At Field (Locus)" = c(
    1, 2, 2, 3, 4, 4,
    5, 5, 5, 5, 5,
    7, 8, 8, 9, 9, 12
  )
)

edgeFile <- tempfile(fileext = ".csv")
write.csv(edges_data, file = edgeFile, row.names = FALSE)
```

```{r model-binet, eval=FALSE}
result.BINET <- BINET(
  U = J35S515, ncls = 13, nfld = 12,
  conf = conf, adj_file = edgeFile
)
print(result.BINET)
```

```{r plot-binet, eval=FALSE}
plot(result.BINET, type = "Array")
plot(result.BINET, type = "TRP")
plot(result.BINET, type = "LRD")
plot(result.BINET, type = "RMP", students = 1:9, nc = 3, nr = 3)
plot(result.BINET, type = "FRP", nc = 3, nr = 2)
```

LDPSR shows Passing Student Rates for locally dependent classes:

```{r plot-binet-ldpsr, eval=FALSE}
plot(result.BINET, type = "LDPSR", nc = 3, nr = 2)
```

```{r cleanup-binet, include=FALSE}
if (exists("edgeFile")) unlink(edgeFile)
```

## Model Comparison

| Model | Nodes | Locus | Key Feature |
|-------|-------|-------|-------------|
| **LDLRA** | Items | Rank | Item dependencies change across ranks |
| **LDB** | Fields | Rank | Field-level dependencies within ranks |
| **BINET** | Classes | Field | Class transitions within fields |

## Reference

Shojima, K. (2022). *Test Data Engineering*. Springer.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_grm.R
\name{grm_iif}
\alias{grm_iif}
\title{Item Information Function for GRM}
\usage{
grm_iif(theta, a, b)
}
\arguments{
\item{theta}{Latent trait value of the subject}

\item{a}{Discrimination parameter of IRF}

\item{b}{Vector of difficulty parameters (thresholds) of IRF}
}
\value{
Value of the Item Information Function
}
\description{
Calculates the value of the Item Information Function for the Graded Response Model.
}
\examples{
\dontrun{
# Example for an item with 3 categories
a <- 1.5
b <- c(-1.0, 1.0)
thetas <- seq(-3, 3, by = 0.1)
info <- sapply(thetas, function(t) grm_iif(t, a, b))
plot(thetas, info, type = "l", xlab = "Theta", ylab = "Information")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_grm.R
\name{GRM}
\alias{GRM}
\title{Graded Response Model (GRM)}
\usage{
GRM(U, na = NULL, Z = NULL, w = NULL, verbose = TRUE)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class using the \code{\link{dataFormat}} function.}

\item{na}{Specifies numbers or characters to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. 1 indicates observed values, 0 indicates missing values.}

\item{w}{Item weight vector}

\item{verbose}{Logical; if TRUE, shows progress of iterations (default: TRUE)}
}
\value{
A list of class "exametrika" and "GRM" containing the following elements:
\describe{
\item{testlength}{Length of the test (number of items)}
\item{nobs}{Sample size (number of rows in the dataset)}
\item{log_lik}{Log-likelihood value at convergence}
\item{iterations}{Number of iterations and function evaluations from optimization}
\item{params}{Matrix containing the estimated item parameters}
\item{EAP}{Ability parameters of examinees estimated by EAP method}
\item{MAP}{Ability parameters of examinees estimated by MAP method}
\item{PSD}{Posterior standard deviation of the ability parameters}
\item{ItemFitIndices}{Fit indices for each item. See also \code{\link{ItemFit}}}
\item{TestFitIndices}{Overall fit indices for the test. See also \code{\link{TestFit}}}
}
}
\description{
Implements Samejima's (1969) Graded Response Model (GRM), which is an Item Response Theory
model for ordered categorical response data. The model estimates discrimination parameters
and category threshold parameters for each item. It is widely used in psychological measurement,
educational assessment, and other fields that deal with multi-step rating scales.
}
\examples{
\dontrun{
# Apply GRM to example data
result <- GRM(J5S1000)
print(result)
plot(result, type = "IRF")
plot(result, type = "IIF")
plot(result, type = "TIF")
}
}
\references{
Samejima, F. (1969). Estimation of latent ability using a response pattern of graded scores.
Psychometrika Monograph Supplement, 34(4, Pt. 2), 1-100.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{polyserial}
\alias{polyserial}
\title{Polyserial Correlation}
\usage{
polyserial(x, y)
}
\arguments{
\item{x}{A numeric vector representing the continuous variable.}

\item{y}{A numeric vector representing the ordinal variable (must be integer values).}
}
\value{
A numeric value representing the estimated polyserial correlation coefficient.
}
\description{
Calculates the polyserial correlation coefficient between a continuous variable and an ordinal variable.
}
\details{
This function implements Olsson et al.'s ad hoc method for estimating the polyserial correlation
coefficient. The method assumes that the continuous variable follows a normal distribution and
that the ordinal variable is derived from an underlying continuous normal variable through
thresholds.
}
\examples{
n <- 300
x <- rnorm(n)
y <- sample(1:5, size = n, replace = TRUE)
polyserial(x, y)
}
\references{
U.Olsson, F.Drasgow, and N.Dorans (1982).
The polyserial correlation coefficient. Psychometrika, 47,337-347.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_GridSearch.R
\name{GridSearch}
\alias{GridSearch}
\title{Grid Search for Optimal Parameters}
\usage{
GridSearch(
  obj,
  max_ncls = 10,
  max_nfld = 10,
  fun = "Biclustering",
  index = "BIC",
  verbose = TRUE,
  ...
)
}
\arguments{
\item{obj}{Input data matrix or object to be analyzed}

\item{max_ncls}{Maximum number of classes/clusters to test (default: 10)}

\item{max_nfld}{Maximum number of fields to test for
Biclustering (default: 10)}

\item{fun}{Function name to use for analysis.
Options: "Biclustering", "LCA", "LRA" (default: "Biclustering")}

\item{index}{Fit index to optimize from TestFitIndices returned by
each function. Valid options: "BIC" (default), "AIC", "CAIC",
"model_log_like", "model_Chi_sq", "RMSEA", "NFI", "RFI", "IFI",
"TLI", "CFI". Aliases are also accepted: "loglik", "log_lik",
"LogLik", "LL" (all map to "model_log_like"), "Chi_sq", "chi_sq"
(map to "model_Chi_sq").}

\item{verbose}{Logical; if TRUE, displays detailed progress messages during grid search. Default is TRUE.}

\item{...}{Additional arguments passed to the analysis function}
}
\value{
A list containing:
For Biclustering:
\item{index_matrix}{Matrix of fit indices for each
ncls/nfld combination}
\item{optimal_ncls}{Optimal number of classes/clusters}
\item{optimal_nfld}{Optimal number of fields}
\item{optimal_result}{Analysis result using optimal parameters}
\item{failed_settings}{List of parameter combinations that failed to converge}

For LCA/LRA:
\item{index_vec}{Vector of fit indices for each ncls}
\item{optimal_ncls}{Optimal number of classes/clusters}
\item{optimal_result}{Analysis result using optimal parameters}
\item{failed_settings}{List of parameter combinations that failed to converge}
}
\description{
Performs a grid search to find optimal parameters for different
analysis methods.
Supports Biclustering, LCA (Latent Class Analysis),
and LRA (Latent Rank Analysis).
}
\examples{
\dontrun{
# Grid search for Biclustering
result <- grid_serch(data_matrix, max_ncls = 5, max_nfld = 5)

# Grid search for LCA
result <- grid_serch(data_matrix, max_ncls = 8, fun = "LCA")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_ModelFitModule.R
\name{ItemFit}
\alias{ItemFit}
\title{Model Fit Functions for Items}
\usage{
ItemFit(U, Z, ell_A, nparam)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{ell_A}{log likelihood of this model}

\item{nparam}{number of parameters for this model}
}
\value{
\describe{
\item{model_log_like}{log likelihood of analysis model}
\item{bench_log_like}{log likelihood of benchmark model}
\item{null_log_like}{log likelihood of null model}
\item{model_Chi_sq}{Chi-Square statistics for analysis model}
\item{null_Chi_sq}{Chi-Square statistics for null model}
\item{model_df}{degrees of freedom of analysis model}
\item{null_df}{degrees of freedom of null model}
\item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
\item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
\item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
\item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
\item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
}
}
\description{
A general function that returns the model fit indices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J20S400}
\alias{J20S400}
\title{J20S400}
\format{
An exametrika class object with 400 students and 20 items containing binary (0/1) responses
}
\source{
\url{http://sh0j1ma.stars.ne.jp/exmk/}
}
\usage{
J20S400
}
\description{
A binary response dataset for test analysis
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09B_LDLRA_GA.R
\name{StrLearningPBIL_LDLRA}
\alias{StrLearningPBIL_LDLRA}
\title{StrLearningPBIL_LDLRA (Deprecated)}
\usage{
StrLearningPBIL_LDLRA(...)
}
\arguments{
\item{...}{All arguments passed to \code{\link{LDLRA_PBIL}}}
}
\description{
This function has been renamed to \code{\link{LDLRA_PBIL}}.
Please use \code{LDLRA_PBIL} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_ModelFitModule.R
\name{calcFitIndices}
\alias{calcFitIndices}
\title{calc Fit Indices}
\usage{
calcFitIndices(chi_A, chi_B, df_A, df_B, nobs)
}
\arguments{
\item{chi_A}{chi-squares for this model}

\item{chi_B}{chi-squares for compared model}

\item{df_A}{degrees of freedom for this model}

\item{df_B}{degrees of freedom for compared model}

\item{nobs}{number of observations for Information criteria}
}
\value{
\describe{
\item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
\item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
\item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
\item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
\item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
}
}
\description{
A general function that returns the model fit indices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{JCRR}
\alias{JCRR}
\alias{JCRR.default}
\alias{JCRR.binary}
\alias{JCRR.nominal}
\title{Joint Correct Response Rate}
\usage{
JCRR(U, na = NULL, Z = NULL, w = NULL)

\method{JCRR}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{JCRR}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{JCRR}{nominal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A matrix of joint correct response rates with exametrika class.
Each element (i,j) represents the proportion of students who correctly
answered both items i and j.
}
\description{
The joint correct response rate (JCRR) is the rate of students who passed
both items. This function is applicable only to binary response data.
For non-binary data, it will automatically redirect to the JSR function
with an appropriate message.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# example code
# Calculate JCRR using sample dataset J5S10
JCRR(J5S10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/23_Chatterjee.R
\name{chatterjee_xi}
\alias{chatterjee_xi}
\title{Chatterjee's xi correlation coefficient}
\usage{
chatterjee_xi(x, y, ties_method = "random")
}
\arguments{
\item{x}{Numeric or ordered factor (predictor)}

\item{y}{Numeric or ordered factor (response)}

\item{ties_method}{How to break ties in x. Default "random".}
}
\value{
Numeric scalar: Chatterjee's xi value.
}
\description{
Computes Chatterjee's (2021) rank-based correlation coefficient xi.
Unlike Pearson or Spearman, xi is asymmetric: xi(x, y) and xi(y, x)
may differ. This asymmetry is the basis for direction-detection in
graphical models.
}
\details{
For tied x values, ranks are broken using \code{ties_method}. With
\code{"random"} (the default), each call may produce a slightly
different value due to random tie-breaking. Use \code{xi_stable()}
to average over many randomizations.
}
\examples{
\donttest{
x <- rnorm(100)
y <- x^2 + rnorm(100, sd = 0.1)
chatterjee_xi(x, y) # near 1, since y is determined by x
chatterjee_xi(y, x) # smaller, since x is not determined by y
}
}
\references{
Chatterjee, S. (2021). A new coefficient of correlation. Journal of
the American Statistical Association, 116(536), 2009-2022.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_exametrikaPrint.R
\name{print.exametrika}
\alias{print.exametrika}
\title{Print Method for Exametrika Objects}
\usage{
\method{print}{exametrika}(x, digits = 3, ...)
}
\arguments{
\item{x}{An object of class "exametrika" with various possible subclasses}

\item{digits}{Integer indicating the number of decimal places to display. Default is 3.}

\item{...}{Additional arguments passed to print methods (not currently used)}
}
\value{
Prints a formatted summary of the exametrika object to the console, with content
varying by object subclass:

\describe{
\item{TestStatistics}{Basic descriptive statistics of the test}
\item{Dimensionality}{Eigenvalue analysis results with scree plot}
\item{ItemStatistics}{Item-level statistics and psychometric properties}
\item{QitemStatistics}{Item statistics for polytomous items}
\item{exametrikaData}{Data structure details including response patterns and weights}
\item{IIAnalysis}{Item-item relationship measures (tetrachoric correlations, etc.)}
\item{CTT}{Classical Test Theory reliability measures}
\item{IRT/GRM}{Item parameters, ability estimates, and fit indices}
\item{LCA/LRA}{Class/Rank profiles, distribution information, and model fit statistics}
\item{Biclustering/Biclustering_IRM}{Cluster profiles, field distributions, and model diagnostics}
\item{LDLRA/LDB/BINET}{Network visualizations, parameter estimates, and conditional probabilities}
}
}
\description{
S3 method for printing objects of class "exametrika". This function formats and displays
appropriate summary information based on the specific subclass of the exametrika object.
Different types of analysis results (IRT, LCA, network models, etc.) are presented
with customized formatting to highlight the most relevant information.
}
\details{
The function identifies the specific subclass of the exametrika object and tailors the
output accordingly. For most analysis types, the function displays:
\itemize{
\item Basic model description and parameters
\item Estimation results (e.g., item parameters, latent class profiles)
\item Model fit statistics and diagnostics
\item Visual representations where appropriate (e.g., graphs for network models, scree plots
for dimensionality analysis)
}

When printing network-based models (LDLRA, LDB, BINET), this function visualizes
the network structure using graphs, which can help in interpreting complex relationships
between items or latent variables.
}
\examples{
\donttest{
# Print IRT analysis results with 4 decimal places
result <- IRT(J15S500)
print(result, digits = 4)

# Print Latent Class Analysis results
result_lca <- LCA(J15S500, ncls = 3)
print(result_lca)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_grm.R
\name{grm_cumprob}
\alias{grm_cumprob}
\title{cumulative probability of GRM}
\usage{
grm_cumprob(theta, a, b)
}
\arguments{
\item{theta}{latent score of subject}

\item{a}{discriminant parameter of IRF}

\item{b}{difficulty parameter of IRF}
}
\description{
cumulative probability of GRM
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{BiserialCorrelation}
\alias{BiserialCorrelation}
\title{Biserial Correlation}
\usage{
BiserialCorrelation(i, t)
}
\arguments{
\item{i}{i is a dichotomous-ordinal variable (0/1). x and y can also be the other way around.}

\item{t}{t is a continuous variable. x and y can also be the other way around.}
}
\value{
The biserial correlation coefficient between the two variables.
}
\description{
A biserial correlation is a correlation between dichotomous-ordinal and
continuous variables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_CTT.R
\name{AlphaCoefficient}
\alias{AlphaCoefficient}
\title{Alpha Coefficient}
\usage{
AlphaCoefficient(x, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{x}{This should be a data matrix or a Covariance/Phi/Tetrachoric matrix.}

\item{na}{This parameter identifies the numbers or characters that should be treated as missing values when 'x' is a data matrix.}

\item{Z}{This parameter represents a missing indicator matrix. It is only needed if 'x' is a data matrix.}

\item{w}{This parameter is an item weight vector. It is only required if 'x' is a data matrix.}
}
\value{
For a correlation/covariance matrix input, returns a single numeric value
representing the alpha coefficient. For a data matrix input, returns a list with
three components:
\describe{
\item{AlphaCov}{Alpha coefficient calculated from covariance matrix}
\item{AlphaPhi}{Alpha coefficient calculated from phi coefficient matrix}
\item{AlphaTetrachoric}{Alpha coefficient calculated from tetrachoric correlation matrix}
}
}
\description{
This function computes Tau-Equivalent Measurement, also known as Cronbach's alpha coefficient, for a given data set.
}
\references{
Cronbach, L. J. (1951). Coefficient alpha and the internal structure of a test. Psychometrika, 16,297–334.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{LogisticModel}
\alias{LogisticModel}
\title{Four-Parameter Logistic Model}
\usage{
LogisticModel(a = 1, b, c = 0, d = 1, theta)
}
\arguments{
\item{a}{slope parameter}

\item{b}{location parameter}

\item{c}{lower asymptote parameter}

\item{d}{upper asymptote parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector of probabilities between c and d, representing
the probability of a correct response given the ability level theta. The probability
is calculated using the formula: \eqn{P(\theta) = c + \frac{(d-c)}{1 + e^{-a(\theta-b)}}}
}
\description{
The four-parameter logistic model is a model where one additional
parameter d, called the upper asymptote parameter, is added to the
3PLM.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{stanine}
\alias{stanine}
\alias{stanine.default}
\alias{stanine.binary}
\title{Stanine Scores}
\usage{
stanine(U, na = NULL, Z = NULL, w = NULL)

\method{stanine}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{stanine}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A list containing two elements:
\describe{
\item{stanine}{The score boundaries for each stanine level}
\item{stanineScore}{The stanine score (1-9) for each student}
}
}
\description{
The Stanine (Standard Nine) scoring system divides students into nine groups
based on a normalized distribution. This function is applicable only to
binary response data.

These groups correspond to the following percentile ranges:
\itemize{
\item Stanine 1: lowest 4\% (percentiles 1-4)
\item Stanine 2: next 7\% (percentiles 5-11)
\item Stanine 3: next 12\% (percentiles 12-23)
\item Stanine 4: next 17\% (percentiles 24-40)
\item Stanine 5: middle 20\% (percentiles 41-60)
\item Stanine 6: next 17\% (percentiles 61-77)
\item Stanine 7: next 12\% (percentiles 78-89)
\item Stanine 8: next 7\% (percentiles 90-96)
\item Stanine 9: highest 4\% (percentiles 97-100)
}
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.

Stanine scores provide a normalized scale with:
\itemize{
\item Mean = 5
\item Standard deviation = 2
\item Scores range from 1 to 9
\item Score of 5 represents average performance
}
}
\examples{
result <- stanine(J15S500)
# View score boundaries
result$stanine
# View individual scores
result$stanineScore

}
\references{
Angoff, W. H. (1984). Scales, norms, and equivalent scores. Educational Testing Service.
(Reprint of chapter in R. L. Thorndike (Ed.) (1971) Educational Measurement (2nd Ed.).
American Council on Education.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{TestInformationFunc}
\alias{TestInformationFunc}
\title{TIF for IRT}
\usage{
TestInformationFunc(params, theta)
}
\arguments{
\item{params}{parameter matrix}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector representing the test information at each ability
level theta. The test information is the sum of item information functions for
all items in the test: \eqn{I_{test}(\theta) = \sum_{j=1}^n I_j(\theta)}
}
\description{
Test Information Function for 4PLM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{ItemEntropy}
\alias{ItemEntropy}
\alias{ItemEntropy.default}
\alias{ItemEntropy.binary}
\alias{ItemEntropy.ordinal}
\title{Item Entropy}
\usage{
ItemEntropy(U, na = NULL, Z = NULL, w = NULL)

\method{ItemEntropy}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemEntropy}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemEntropy}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of entropy values for each item, measured in bits.
Values range from 0 to 1, where:
\itemize{
\item 1: maximum uncertainty (p = 0.5)
\item 0: complete certainty (p = 0 or 1)
\item Values near 1 indicate items with balanced response patterns
\item Values near 0 indicate items with extreme response patterns
}
}
\description{
The item entropy is an indicator of the variability or randomness
of the responses. This function is applicable only to binary response data.

The entropy value represents the uncertainty or information content of the
response pattern for each item, measured in bits. Maximum entropy (1 bit)
occurs when correct and incorrect responses are equally likely (p = 0.5).
}
\details{
The item entropy is calculated as:
\deqn{e_j = -p_j\log_2p_j-(1-p_j)\log_2(1-p_j)}
where \eqn{p_j} is the correct response rate for item j.

The entropy value has the following properties:
\itemize{
\item Maximum value of 1 bit when p = 0.5 (most uncertainty)
\item Minimum value of 0 bits when p = 0 or 1 (no uncertainty)
\item Higher values indicate more balanced response patterns
\item Lower values indicate more predictable response patterns
}
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# using sample dataset
ItemEntropy(J5S10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_typeErrorMessage.R
\name{response_type_error}
\alias{response_type_error}
\title{Generate Error Message for Invalid Response Type}
\usage{
response_type_error(response_type, fun_name)
}
\arguments{
\item{response_type}{character. One of "binary", "rated", "ordinal", or "nominal"}

\item{fun_name}{character. Name of the calling function}
}
\value{
Never returns; always stops with an error message
}
\description{
Internal function to generate standardized error messages when a function
is called with an incompatible response type.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{TwoPLM}
\alias{TwoPLM}
\title{Two-Parameter Logistic Model}
\usage{
TwoPLM(a, b, theta)
}
\arguments{
\item{a}{slope parameter}

\item{b}{location parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector of probabilities between 0 and 1, representing
the probability of a correct response given the ability level theta. The probability
is calculated using the formula: \eqn{P(\theta) = \frac{1}{1 + e^{-a(\theta-b)}}}
}
\description{
The two-parameter logistic model is a classic model that defines
the probability of a student with ability theta successfully
answering item j, using both a slope parameter and a
location parameter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{RaschModel}
\alias{RaschModel}
\title{Rasch Model}
\usage{
RaschModel(b, theta)
}
\arguments{
\item{b}{slope parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector of probabilities between 0 and 1, representing
the probability of a correct response given the ability level theta. The probability
is calculated using the formula: \eqn{P(\theta) = \frac{1}{1 + e^{-(\theta-b)}}}
}
\description{
The one-parameter logistic model is a model with only one parameter b.
This model is a 2PLM model in which a is constrained to 1.
This model is also called the Rasch model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{passage}
\alias{passage}
\alias{passage.default}
\alias{passage.binary}
\title{Passage Rate of Student}
\usage{
passage(U, na = NULL, Z = NULL, w = NULL)

\method{passage}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{passage}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector containing the passage rate for each student.
Values range from 0 to 1 (or maximum weight) where:
\itemize{
\item 1: Perfect score on all attempted items
\item 0: No correct answers
\item NA: No items attempted
}
}
\description{
The Passage Rate for each student is calculated as their Number-Right Score (NRS)
divided by the number of items presented to them. This function is applicable
only to binary response data.

The passage rate is calculated as:
\deqn{P_i = \frac{\sum_{j=1}^J z_{ij}u_{ij}w_j}{\sum_{j=1}^J z_{ij}}}
where:
\itemize{
\item \eqn{z_{ij}} is the missing response indicator (0/1)
\item \eqn{u_{ij}} is the response (0/1)
\item \eqn{w_j} is the item weight
}
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.

The passage rate accounts for missing responses by only considering items that
were actually presented to each student. This provides a fair comparison
between students who attempted different numbers of items.
}
\examples{
# using sample dataset
passage(J15S500)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{TestResponseFunc}
\alias{TestResponseFunc}
\title{TRF for IRT}
\usage{
TestResponseFunc(params, theta)
}
\arguments{
\item{params}{parameter matrix}

\item{theta}{ability parameter}
}
\value{
A numeric vector with the same length as theta, containing the expected total score
for each ability level.
}
\description{
Calculates the expected score across all items on a test for a given ability level (theta)
using Item Response Theory. The Test Response Function (TRF) is essentially the sum of
the Item Characteristic Curves (ICCs) for all items in the test.
}
\details{
The Test Response Function computes the expected total score for an examinee with a given
ability level (theta) across all items in the test. For each item, the function uses the
logistic model with parameters a (discrimination), b (difficulty), c (guessing), and
d (upper asymptote).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_QitemFunctions.R
\name{ItemReport}
\alias{ItemReport}
\title{Generate Item Report for Non-Binary Test Data}
\usage{
ItemReport(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item}
}
\value{
An object of class "exametrika" and "QitemStatistics" containing:
\describe{
\item{ItemLabel}{Labels identifying each item}
\item{Obs}{Number of valid responses for each item}
\item{ObsRatio}{Proportion of valid responses for each item (range: 0-1)}
\item{ItemMean}{Mean score of each item}
\item{ItemSD}{Standard deviation of each item score}
\item{ItemCORR}{Item-total correlation coefficients - correlation between
item scores and total test scores}
\item{ItemCORR_R}{Corrected item-total correlation coefficients - correlation between
item scores and total test scores excluding the target item}
}
}
\description{
Calculates item-level statistics for non-binary test data, including response rates,
basic descriptive statistics, and item-total correlations.
}
\details{
This function is intended for non-binary (ordinal or rated) response data. It provides
detailed statistics for each item in the test, focusing on response patterns and
the relationship between individual items and overall test performance.
If binary data is provided, an error message will be displayed.
}
\examples{
\donttest{
# Generate item report for sample ordinal data
item_stats <- ItemReport(J15S3810)

# View first few rows of the item report
head(item_stats)

# Example with rated data including custom missing value indicator
item_stats2 <- ItemReport(J35S5000, na = -99)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_dataFormat.R
\name{dataFormat}
\alias{dataFormat}
\title{dataFormat}
\usage{
dataFormat(
  data,
  na = NULL,
  id = NULL,
  Z = NULL,
  w = NULL,
  response.type = NULL,
  CA = NULL
)
}
\arguments{
\item{data}{is a data matrix of the type matrix or data.frame.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{id}{id indicates the column number containing the examinee ID.
If NULL (default), the first column is auto-detected as ID or response data.
If a column number is specified, that column is always used as the ID column.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{response.type}{Character string specifying the type of response data:
"binary" for dichotomous data,
"ordinal" for ordered polytomous data,
"rated" for polytomous data with correct answers,
"nominal" for unordered polytomous data.
If NULL (default), the type is automatically detected.}

\item{CA}{A numeric vector specifying the correct answers for rated polytomous data.
Required when response.type is "rated".}
}
\value{
\describe{
\item{U}{For binary response data. A matrix with rows representing the sample size and columns
representing the number of items, where elements are either 0 or 1. \eqn{u_{ij}=1} indicates
that student i correctly answered item j, while \eqn{u_{ij}=0} means that student i answered
item j incorrectly.}
\item{Q}{For polytomous response data. A matrix with rows representing the sample size and columns
representing the number of items, where elements are non-negative integers. When input data is
in factor format, the factor levels are converted to consecutive integers starting from 1.}
\item{ID}{The ID label given by the designated column or function.}
\item{ItemLabel}{The item names given by the provided column names or function.}
\item{Z}{Missing indicator matrix. \eqn{z_{ij}=1} indicates that item j is presented to Student i,
while \eqn{z_{ij}=0} indicates item j is NOT presented to Student i.
If the data contains NA values, -1 is assigned.}
\item{w}{Item weight vector}
\item{response.type}{Character string indicating the type of response data:
"binary", "ordinal", "rated", or "nominal"}
\item{CategoryLabel}{List containing the original factor labels when polytomous responses
are provided as factors. NULL if no factor data is present.}
\item{categories}{Numeric vector containing the number of response categories for each item.}
\item{CA}{For rated polytomous data, a numeric vector of correct answers. NULL for other types.}
}
}
\description{
This function serves the role of formatting the data prior to the analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08B_GA_utils.R
\name{maxParents_penalty}
\alias{maxParents_penalty}
\title{Utility function for searching DAG}
\usage{
maxParents_penalty(vec, testlength, maxParents)
}
\arguments{
\item{vec}{gene Vector corresponding to the upper triangular of the adjacency matrix}

\item{testlength}{test length. In this context it means a number of nodes.}

\item{maxParents}{Upper limit of number of nodes.}
}
\description{
Function to limit the number of parent nodes
}
\details{
When generating an adjacency matrix using GA, the number of edges coming
from a single node should be limited to 2 or 3. This is because if
there are too many edges, it becomes difficult to interpret in practical
applications. This function works to adjust the sampling of the randomly
generated adjacency matrix so that the column sum of the upper triangular
elements fits within the set limit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_InterItemAnalysis.R
\name{InterItemAnalysis}
\alias{InterItemAnalysis}
\title{Inter-Item Analysis for Psychometric Data}
\usage{
InterItemAnalysis(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
For binary data, an object of class "exametrika" and "IIAnalysis" containing:
\describe{
\item{JSS}{Joint Sample Size matrix - N(i,j) shows number of examinees who responded to both items i and j}
\item{JCRR}{Joint Correct Response Rate matrix - P(Xi=1, Xj=1) shows probability of correct responses to both items}
\item{CCRR}{Conditional Correct Response Rate matrix - P(Xi=1|Xj=1) shows probability of correct response to item i
given correct response to item j}
\item{IL}{Item Lift matrix - P(Xi=1, Xj=1)/(P(Xi=1)*P(Xj=1)) measures association strength}
\item{MI}{Mutual Information matrix - measures information shared between items}
\item{Phi}{Phi Coefficient matrix - correlation coefficient between binary variables}
\item{Tetrachoric}{Tetrachoric Correlation matrix - correlation between underlying continuous variables}
}

For ordinal/rated data, an object of class "exametrika" and "IIAnalysis.ordinal" containing:
\describe{
\item{JSS}{Joint Sample Size matrix}
\item{JSR}{Joint Selection Rate matrix - frequencies of joint category selections}
\item{CSR}{Conditional Selection Rate matrix - probabilities of response categories conditional on other items}
\item{MI}{Mutual Information matrix}
\item{Polychoric}{Polychoric Correlation matrix - correlations between underlying continuous variables}
}
}
\description{
Calculates various relationship metrics between pairs of items in test data. This analysis
helps identify item interdependencies, content overlaps, and potential local dependence.
For binary data, metrics include joint response rates, conditional probabilities,
and several correlation measures. For ordinal/rated data, appropriate correlation
measures are calculated.

The following metrics are calculated for binary data:
\itemize{
\item JSS: Joint Sample Size - number of examinees responding to both items
\item JCRR: Joint Correct Response Rate - proportion of examinees answering both items correctly
\item CCRR: Conditional Correct Response Rate - probability of answering one item correctly
given a correct response to another item
\item IL: Item Lift - ratio of joint correct response rate to the product of marginal rates
\item MI: Mutual Information - measure of mutual dependence between items
\item Phi: Phi Coefficient - correlation coefficient for binary variables
\item Tetrachoric: Tetrachoric Correlation - estimate of Pearson correlation for underlying
continuous variables
}

For ordinal/rated data, the function calculates:
\itemize{
\item JSS: Joint Sample Size
\item JSR: Joint Selection Rate
\item CSR: Conditional Selection Rate
\item MI: Mutual Information
\item Polychoric: Polychoric Correlation - extension of tetrachoric correlation for ordinal data
}
}
\details{
This function automatically detects the data type and applies appropriate analysis methods:
\itemize{
\item For binary data: Calculates tetrachoric correlations and related statistics
\item For ordinal/rated data: Calculates polychoric correlations and related statistics
\item For nominal data: Returns an error (not supported)
}

Inter-item analysis is useful for:
\itemize{
\item Identifying groups of highly related items
\item Detecting local dependence between items
\item Evaluating test dimensionality
\item Informing item selection and test construction
}
}
\examples{
\donttest{
# Basic usage with binary data
ii_analysis <- InterItemAnalysis(J15S500)

# View joint sample sizes
head(ii_analysis$JSS)

# View tetrachoric correlations
head(ii_analysis$Tetrachoric)

# Find pairs of items with high mutual information (potential local dependence)
high_MI <- which(ii_analysis$MI > 0.2 & upper.tri(ii_analysis$MI), arr.ind = TRUE)
if (nrow(high_MI) > 0) {
  print("Item pairs with high mutual information:")
  print(high_MI)
}

# Example with ordinal data
ordinal_analysis <- InterItemAnalysis(J15S3810)

# View polychoric correlations for ordinal data
head(ordinal_analysis$Polychoric)
}

}
\seealso{
\code{\link{dataFormat}} for data preparation, \code{\link{CTT}} for
Classical Test Theory analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{ThreePLM}
\alias{ThreePLM}
\title{Three-Parameter Logistic Model}
\usage{
ThreePLM(a, b, c, theta)
}
\arguments{
\item{a}{slope parameter}

\item{b}{location parameter}

\item{c}{lower asymptote parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector of probabilities between c and 1, representing
the probability of a correct response given the ability level theta. The probability
is calculated using the formula: \eqn{P(\theta) = c + \frac{1-c}{1 + e^{-a(\theta-b)}}}
}
\description{
The three-parameter logistic model is a model where the lower
asymptote parameter c is added to the 2PLM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{IIF3PLM}
\alias{IIF3PLM}
\title{IIF for 3PLM}
\usage{
IIF3PLM(a, b, c, theta)
}
\arguments{
\item{a}{slope parameter}

\item{b}{location parameter}

\item{c}{lower asymptote parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector representing the item information at each ability
level theta. The information is calculated as:
\eqn{I(\theta) = \frac{a^2(1-P(\theta))(P(\theta)-c)^2}{(1-c)^2P(\theta)}}
}
\description{
Item Information Function for 3PLM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08C_BNM_GA.R
\name{BNM_PBIL}
\alias{BNM_PBIL}
\title{Structure Learning for BNM by PBIL}
\usage{
BNM_PBIL(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  seed = 123,
  population = 20,
  Rs = 0.5,
  Rm = 0.002,
  maxParents = 2,
  maxGeneration = 100,
  successiveLimit = 5,
  elitism = 0,
  alpha = 0.05,
  estimate = 1,
  filename = NULL,
  verbose = TRUE
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{seed}{seed for random.}

\item{population}{Population size. The default is 20}

\item{Rs}{Survival Rate. The default is 0.5}

\item{Rm}{Mutation Rate. The default is 0.002}

\item{maxParents}{Maximum number of edges emanating from a single node. The default is 2.}

\item{maxGeneration}{Maximum number of generations.}

\item{successiveLimit}{Termination conditions. If the optimal individual does not change
for this number of generations, it is considered to have converged.}

\item{elitism}{Number of elites that remain without crossover when transitioning to
the next generation.}

\item{alpha}{Learning rate. The default is 0.05}

\item{estimate}{In PBIL for estimating the adjacency matrix, specify by number from the
following four methods: 1. Optimal adjacency matrix, 2. Rounded average of individuals in
the last generation, 3. Rounded average of survivors in the last generation, 4. Rounded
generational gene of the last generation. The default is 1.}

\item{filename}{Specify the filename when saving the generated adjacency matrix in CSV format.
The default is null, and no output is written to the file.}

\item{verbose}{verbose output Flag. default is TRUE}
}
\value{
\describe{
\item{adj}{Optimal adjacency matrix}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{crr}{correct response ratio}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{param}{Learned Parameters}
\item{CCRR_table}{Correct Response Rate tables}
}
}
\description{
Generating a DAG from data using a Population-Based Incremental Learning
}
\details{
This function performs structural learning using the Population-Based
Incremental Learning model(PBIL) proposed by Fukuda et al.(2014) within
the genetic algorithm framework. Instead of learning the adjacency matrix
itself, the 'genes of genes' that generate the adjacency matrix are updated
with each generation. For more details, please refer to Fukuda(2014) and Section
8.5.2 of the text(Shojima,2022).
}
\examples{
\donttest{
# Perform Structure Learning for Bayesian Network Model using PBIL
# (Population-Based Incremental Learning)
BNM_PBIL(J5S10,
  population = 20, # Size of population in each generation
  Rs = 0.5, # 50\% survival rate for next generation
  Rm = 0.005, # 0.5\% mutation rate for genetic diversity
  maxParents = 2, # Maximum of 2 parent nodes per item
  alpha = 0.05, # Learning rate for probability update
  estimate = 4 # Use rounded generational gene method
)
}
}
\references{
Fukuda, S., Yamanaka, Y., & Yoshihiro, T. (2014). A Probability-based evolutionary
algorithm with mutations to learn Bayesian networks. International Journal of Artificial
Intelligence and Interactive Multimedia, 3, 7–13. DOI: 10.9781/ijimai.2014.311
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_LCA.R
\name{LCA}
\alias{LCA}
\title{Latent Class Analysis}
\usage{
LCA(
  U,
  ncls = 2,
  na = NULL,
  Z = NULL,
  w = NULL,
  maxiter = 100,
  verbose = TRUE,
  beta1 = 1,
  beta2 = 1,
  conf = NULL
)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{ncls}{Number of latent classes to identify (between 2 and 20). Default is 2.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}

\item{maxiter}{Maximum number of EM algorithm iterations. Default is 100.}

\item{verbose}{Logical; if TRUE, displays progress during estimation. Default is TRUE.}

\item{beta1}{Beta distribution parameter 1 for prior density of class reference matrix. Default is 1.}

\item{beta2}{Beta distribution parameter 2 for prior density of class reference matrix. Default is 1.}

\item{conf}{Confirmatory IRP matrix (items x ncls) for test equating.
Same format as the IRP output. Non-NA values are fixed throughout estimation,
NA values are freely estimated. Fixed values must be in the open interval (0, 1).
When row names are present, items are matched by label; otherwise by position.
Default is NULL (fully exploratory).}
}
\value{
An object of class "exametrika" and "LCA" containing:
\describe{
\item{msg}{A character string indicating the model type. }
\item{testlength}{Length of the test (number of items).}
\item{nobs}{Sample size (number of rows in the dataset).}
\item{Nclass}{Number of latent classes specified.}
\item{N_Cycle}{Number of EM algorithm iterations performed.}
\item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
\item{TRP}{Test Reference Profile vector showing expected scores for each latent class.
Calculated as the column sum of the estimated class reference matrix.}
\item{LCD}{Latent Class Distribution vector showing the number of examinees assigned to each latent class.}
\item{CMD}{Class Membership Distribution vector showing the sum of membership probabilities for each latent class.}
\item{Students}{Class Membership Profile matrix showing the posterior probability of each examinee
belonging to each latent class. The last column ("Estimate") indicates the most likely class assignment.}
\item{IRP}{Item Reference Profile matrix where each row represents an item and each column
represents a latent class. Values indicate the probability of a correct response
for members of that class.}
\item{ItemFitIndices}{Fit indices for each item. See also \code{\link{ItemFit}}.}
\item{TestFitIndices}{Overall fit indices for the test. See also \code{\link{TestFit}}.}
}
}
\description{
Performs Latent Class Analysis (LCA) on binary response data using the Expectation-Maximization (EM) algorithm.
LCA identifies unobserved (latent) subgroups of examinees with similar response patterns,
and estimates both the class characteristics and individual membership probabilities.
}
\details{
Latent Class Analysis is a statistical method for identifying unobserved subgroups within
a population based on observed response patterns. It assumes that examinees belong to one
of several distinct latent classes, and that the probability of a correct response to each
item depends on class membership.

The algorithm proceeds by:
\enumerate{
\item Initializing class reference probabilities
\item Computing posterior class membership probabilities for each examinee (E-step)
\item Re-estimating class reference probabilities based on these memberships (M-step)
\item Iterating until convergence or reaching the maximum number of iterations
}

Unlike Item Response Theory (IRT), LCA treats latent variables as categorical rather than
continuous, identifying distinct profiles rather than positions on a continuum.
}
\examples{
\donttest{
# Fit a Latent Class Analysis model with 5 classes to the sample dataset
result.LCA <- LCA(J15S500, ncls = 5)

# Display the first few rows of student class membership probabilities
head(result.LCA$Students)

# Plot Item Response Profiles (IRP) for items 1-6 in a 2x3 grid
# Shows probability of correct response for each item across classes
plot(result.LCA, type = "IRP", items = 1:6, nc = 2, nr = 3)

# Plot Class Membership Probabilities (CMP) for students 1-9 in a 3x3 grid
# Shows probability distribution of class membership for each student
plot(result.LCA, type = "CMP", students = 1:9, nc = 3, nr = 3)

# Plot Test Response Profile (TRP) showing expected scores for each class
plot(result.LCA, type = "TRP")

# Plot Latent Class Distribution (LCD) showing class sizes
plot(result.LCA, type = "LCD")

# Compare models with different numbers of classes
# (In practice, you might try more class counts)
lca2 <- LCA(J15S500, ncls = 2)
lca3 <- LCA(J15S500, ncls = 3)
lca4 <- LCA(J15S500, ncls = 4)
lca5 <- LCA(J15S500, ncls = 5)

# Compare BIC values to select optimal number of classes
# (Lower BIC indicates better fit)
data.frame(
  Classes = 2:5,
  BIC = c(
    lca2$TestFitIndices$BIC,
    lca3$TestFitIndices$BIC,
    lca4$TestFitIndices$BIC,
    lca5$TestFitIndices$BIC
  )
)
}

}
\references{
Goodman, L. A. (1974). Exploratory latent structure analysis using both identifiable and
unidentifiable models. Biometrika, 61(2), 215-231.

Lazarsfeld, P. F., & Henry, N. W. (1968). Latent structure analysis.
Boston: Houghton Mifflin.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08A_BNM.R
\name{BNM}
\alias{BNM}
\title{Bayesian Network Model}
\usage{
BNM(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  g = NULL,
  adj_file = NULL,
  adj_matrix = NULL,
  beta1 = 1,
  beta2 = 1
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{g}{Specify a graph object suitable for the igraph class.}

\item{adj_file}{specify CSV file where the graph structure is specified.}

\item{adj_matrix}{specify adjacency matrix.}

\item{beta1}{Beta distribution parameter 1 (for correct responses). Default is 1.}

\item{beta2}{Beta distribution parameter 2 (for incorrect responses). Default is 1. Note: referred to as beta0 internally.}
}
\value{
\describe{
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{crr}{correct response ratio}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{adj}{Adjacency matrix}\
\item{param}{Learned Parameters}
\item{CCRR_table}{Correct Response Rate tables}
}
}
\description{
performs Bayesian Network Model with specified graph structure
}
\details{
This function performs a Bayesian network analysis on the relationships
between items. This corresponds to Chapter 8 of the text. It uses the igraph
package for graph visualization and checking the adjacency matrix.
You need to provide either a graph object or a CSV file where the graph
structure is specified.
}
\examples{
\donttest{
# Create a Directed Acyclic Graph (DAG) structure for item relationships
# Each row represents a directed edge from one item to another
DAG <-
  matrix(
    c(
      "Item01", "Item02", # Item01 influences Item02
      "Item02", "Item03", # Item02 influences Item03
      "Item02", "Item04", # Item02 influences Item04
      "Item03", "Item05", # Item03 influences Item05
      "Item04", "Item05" # Item04 influences Item05
    ),
    ncol = 2, byrow = TRUE
  )

# Convert the DAG matrix to an igraph object for network analysis
g <- igraph::graph_from_data_frame(DAG)
g

# Create adjacency matrix from the graph
# Shows direct connections between items (1 for connection, 0 for no connection)
adj_mat <- as.matrix(igraph::as_adjacency_matrix(g))
print(adj_mat)

# Fit Bayesian Network Model using the specified adjacency matrix
# Analyzes probabilistic relationships between items based on the graph structure
result.BNM <- BNM(J5S10, adj_matrix = adj_mat)
result.BNM
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{polychoric}
\alias{polychoric}
\title{Polychoric Correlation}
\usage{
polychoric(x, y)
}
\arguments{
\item{x}{A polytomous vector (categorical ordinal variable)}

\item{y}{A polytomous vector (categorical ordinal variable)}
}
\value{
The polychoric correlation coefficient between x and y
}
\description{
Calculate the polychoric correlation coefficient between two polytomous (categorical ordinal) variables.
Polychoric correlation estimates the correlation between two theorized normally distributed
continuous latent variables from two observed ordinal variables.
}
\details{
This function handles missing values (coded as -1 or NA) using pairwise deletion.
The estimation uses maximum likelihood approach with Brent's method for optimization.
}
\examples{
# Example with simulated data
set.seed(123)
x <- sample(1:5, 100, replace = TRUE)
y <- sample(1:4, 100, replace = TRUE)
polychoric(x, y)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04C_ParameterEstimation.R
\name{slopeprior}
\alias{slopeprior}
\title{Prior distribution function with respect to the slope.}
\usage{
slopeprior(a, m, s, const = 1e-15)
}
\arguments{
\item{a}{slope coefficient}

\item{m}{prior parameter to be set}

\item{s}{prior parameter to be set}

\item{const}{A very small constant}
}
\description{
Prior distribution function with respect to the slope.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/22_GlassoUnit.R
\name{Glasso}
\alias{Glasso}
\title{Graphical Lasso for Gaussian Graphical Models}
\usage{
Glasso(
  U,
  na = NULL,
  Z = NULL,
  w = NULL,
  gamma = 0.5,
  n_lambda = 50,
  lambda_ratio = 0.01,
  penalize_diagonal = TRUE,
  max_iter = 100,
  eps = 1e-06,
  edge_tol = 1e-08,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{gamma}{EBIC tuning parameter in \verb{[0, 1]}. Default is 0.5.}

\item{n_lambda}{Number of lambda values in the search grid. Default is 50.}

\item{lambda_ratio}{Ratio of lambda_min to lambda_max. Default is 0.01.}

\item{penalize_diagonal}{Logical. If TRUE, the L1 penalty is applied to
the diagonal of Theta. Default is TRUE.}

\item{max_iter}{Maximum number of outer iterations for the block
coordinate descent algorithm. Default is 100.}

\item{eps}{Convergence tolerance. Default is 1e-6.}

\item{edge_tol}{Threshold below which an off-diagonal element of Theta is
considered zero (no edge). Default is 1e-8.}

\item{verbose}{Logical. If TRUE, progress messages are displayed.}

\item{...}{Additional arguments (currently unused; reserved for future extensions).}
}
\value{
\describe{
\item{theta}{Estimated precision matrix at the optimal lambda.}
\item{W}{Working covariance matrix at the optimal lambda.}
\item{lambda_opt}{Selected lambda value.}
\item{gamma}{EBIC tuning parameter used.}
\item{ebic_opt}{Minimum EBIC value.}
\item{n_edge}{Number of edges in the selected model.}
\item{path}{Data frame with columns lambda, ebic, n_edge over the search grid.}
}
}
\description{
Estimates a sparse precision matrix from ordinal item response data via
the Graphical Lasso (Friedman, Hastie, Tibshirani 2008). The polychoric
correlation matrix is used as input. The optimal regularization parameter
lambda is selected by the Extended Bayesian Information Criterion
(Foygel and Drton 2010).
}
\details{
The Graphical Lasso estimates the precision matrix Theta by maximizing
the penalized log-likelihood:

log det(Theta) - tr(S Theta) - lambda * ||Theta||_1

subject to Theta being positive definite. Optimization is performed by
block coordinate descent (Algorithm 17.2 of Hastie, Tibshirani, Friedman
2009) with cyclical coordinate descent for the inner lasso step.

Lambda is searched on a log-scale grid from lambda_max (the maximum
absolute off-diagonal of S) down to lambda_max * lambda_ratio. For each
lambda, the EBIC is computed and the lambda minimizing EBIC is returned.
Warm-starting (W and beta cache from the previous lambda) is used to
accelerate the search.
}
\examples{
\donttest{
# Estimate a sparse precision matrix from ordinal data
result.Glasso <- Glasso(J15S3810)
result.Glasso$lambda_opt
result.Glasso$n_edge
}
}
\references{
Friedman, J., Hastie, T., and Tibshirani, R. (2008). Sparse inverse
covariance estimation with the graphical lasso. Biostatistics, 9(3), 432-441.

Foygel, R., and Drton, M. (2010). Extended Bayesian Information Criteria
for Gaussian Graphical Models. Advances in Neural Information Processing
Systems 23.
}
\seealso{
\link{PolychoricCorrelationMatrix}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{nrs}
\alias{nrs}
\alias{nrs.default}
\alias{nrs.binary}
\title{Number Right Score}
\usage{
nrs(U, na = NULL, Z = NULL, w = NULL)

\method{nrs}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{nrs}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector containing the Number-Right Score for each examinee.
The score represents the weighted sum of correct answers, where:
\itemize{
\item Maximum score is the sum of all item weights
\item Minimum score is 0
\item Missing responses do not contribute to the score
}
}
\description{
The Number-Right Score (NRS) function calculates the weighted sum of correct
responses for each examinee. This function is applicable only to binary
response data.

For each examinee, the score is computed as:
\deqn{NRS_i = \sum_{j=1}^J z_{ij}u_{ij}w_j}
where:
\itemize{
\item \eqn{z_{ij}} is the missing response indicator (0/1)
\item \eqn{u_{ij}} is the response (0/1)
\item \eqn{w_j} is the item weight
}
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# using sample dataset
nrs(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08C_BNM_GA.R
\name{StrLearningPBIL_BNM}
\alias{StrLearningPBIL_BNM}
\title{StrLearningPBIL_BNM (Deprecated)}
\usage{
StrLearningPBIL_BNM(...)
}
\arguments{
\item{...}{All arguments passed to \code{\link{BNM_PBIL}}}
}
\description{
This function has been renamed to \code{\link{BNM_PBIL}}.
Please use \code{BNM_PBIL} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08A_BNM.R
\name{BitRespPtn}
\alias{BitRespPtn}
\title{Binary pattern maker}
\usage{
BitRespPtn(n)
}
\arguments{
\item{n}{decimal numbers}
}
\value{
binary patterns
}
\description{
Binary pattern maker
}
\details{
if n <- 1, return 0,1
if n <- 2, return 00,01,10,11
and so on.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_LDLRA.R
\name{LDLRA}
\alias{LDLRA}
\title{Local Dependence Latent Rank Analysis}
\usage{
LDLRA(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  ncls = 2,
  method = "R",
  g_list = NULL,
  adj_list = NULL,
  adj_file = NULL,
  verbose = FALSE,
  beta1 = 2,
  beta2 = 2
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{ncls}{number of latent class(rank). The default is 2.}

\item{method}{specify the model to analyze the data.Local dependence latent
class model is set to "C", latent rank model is set "R". The default is "R".}

\item{g_list}{A list compiling graph-type objects for each rank/class.}

\item{adj_list}{A list compiling matrix-type adjacency matrices for each rank/class.}

\item{adj_file}{A file detailing the relationships of the graph for each rank/class,
listed in the order of starting point, ending point, and rank(class).}

\item{verbose}{verbose output Flag. default is TRUE}

\item{beta1}{Beta distribution parameter 1 for prior density of rank reference matrix. Default is 2.}

\item{beta2}{Beta distribution parameter 2 for prior density of rank reference matrix. Default is 2.}
}
\value{
\describe{
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{msg}{A character string indicating the model type. }
\item{testlength}{Length of the test. The number of items included in the test.}
\item{crr}{correct response ratio}
\item{adj_list}{adjacency matrix list}
\item{g_list}{graph list}
\item{referenceMatrix}{Learned Parameters.A three-dimensional array of patterns where
item x rank x pattern.}
\item{IRP}{Marginal Item Reference Matrix}
\item{IRPIndex}{IRP Indices which include Alpha, Beta, Gamma.}
\item{TRP}{Test Reference Profile matrix.}
\item{LRD}{latent Rank/Class Distribution}
\item{RMD}{Rank/Class Membership Distribution}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{Estimation_table}{Estimated parameters tables.}
\item{CCRR_table}{Correct Response Rate tables}
\item{Studens}{Student information. It includes estimated class
membership, probability of class membership, RUO, and RDO.}
}
}
\description{
performs local dependence latent lank analysis(LD_LRA) by Shojima(2011)
}
\details{
This function is intended to perform LD-LRA. LD-LRA is an analysis that
combines LRA and BNM, and it is used to analyze the network structure among
items in the latent rank. In this function, structural learning is not
performed, so you need to provide item graphs for each rank as separate files.
The file format for this is plain text CSV that includes edges (From, To) and
rank numbers.
}
\examples{
\donttest{
# Create sample DAG structure with different rank levels
# Format: From, To, Rank
DAG_dat <- matrix(c(
  "From", "To", "Rank",
  "Item01", "Item02", "1", # Simple structure for Rank 1
  "Item01", "Item02", "2", # More complex structure for Rank 2
  "Item02", "Item03", "2",
  "Item01", "Item02", "3", # Additional connections for Rank 3
  "Item02", "Item03", "3",
  "Item03", "Item04", "3"
), ncol = 3, byrow = TRUE)

# Method 1: Directly use graph and adjacency lists
g_list <- list()
adj_list <- list()

for (i in 1:3) {
  adj_R <- DAG_dat[DAG_dat[, 3] == as.character(i), 1:2, drop = FALSE]
  g_tmp <- igraph::graph_from_data_frame(
    d = data.frame(
      From = adj_R[, 1],
      To = adj_R[, 2]
    ),
    directed = TRUE
  )
  adj_tmp <- igraph::as_adjacency_matrix(g_tmp)
  g_list[[i]] <- g_tmp
  adj_list[[i]] <- adj_tmp
}

# Fit Local Dependence Latent Rank Analysis
result.LDLRA1 <- LDLRA(J12S5000,
  ncls = 3,
  g_list = g_list,
  adj_list = adj_list
)

# Plot Item Reference Profiles (IRP) in a 4x3 grid
# Shows the probability patterns of correct responses for each item across ranks
plot(result.LDLRA1, type = "IRP", nc = 4, nr = 3)

# Plot Test Reference Profile (TRP)
# Displays the overall pattern of correct response probabilities across ranks
plot(result.LDLRA1, type = "TRP")

# Plot Latent Rank Distribution (LRD)
# Shows the distribution of students across different ranks
plot(result.LDLRA1, type = "LRD")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_QitemFunctions.R
\name{generate_category_labels}
\alias{generate_category_labels}
\title{Generate category labels for response data}
\usage{
generate_category_labels(data_column, item_name)
}
\arguments{
\item{data_column}{A vector containing response data for a single item}

\item{item_name}{Character string of the item name}
}
\value{
A character vector of category labels
}
\description{
Generate category labels for response data
}
\details{
If the input is a factor, returns its levels.
Otherwise, generates labels in the format "Item name-Category-N"
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J50S100}
\alias{J50S100}
\title{J50S100}
\format{
An exametrika class object with 100 students and 50 items containing binary responses
}
\usage{
J50S100
}
\description{
A simulated binary dataset for test analysis. This is a synthetic dataset
generated using random number generation for demonstration and testing purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{sscore}
\alias{sscore}
\alias{sscore.default}
\alias{sscore.binary}
\title{Standardized Score}
\usage{
sscore(U, na = NULL, Z = NULL, w = NULL)

\method{sscore}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{sscore}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of standardized scores for each student. The scores follow
a standard normal distribution with:
\itemize{
\item Mean = 0
\item Standard deviation = 1
\item Approximately 68\% of scores between -1 and 1
\item Approximately 95\% of scores between -2 and 2
\item Approximately 99\% of scores between -3 and 3
}
}
\description{
The standardized score (z-score) indicates how far a student's performance
deviates from the mean in units of standard deviation. This function is
applicable only to binary response data.

The score is calculated by standardizing the passage rates:
\deqn{Z_i = \frac{r_i - \bar{r}}{\sigma_r}}
where:
\itemize{
\item \eqn{r_i} is student i's passage rate
\item \eqn{\bar{r}} is the mean passage rate
\item \eqn{\sigma_r} is the standard deviation of passage rates
}
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.

The standardization allows for comparing student performance across different
tests or groups. A positive score indicates above-average performance, while
a negative score indicates below-average performance.
}
\examples{
# using sample dataset
sscore(J5S10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_ModelFitModule.R
\name{TestFitSaturated}
\alias{TestFitSaturated}
\title{Model Fit Functions for saturated model}
\usage{
TestFitSaturated(U, Z, ell_A, nparam)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{ell_A}{log likelihood of this model}

\item{nparam}{number of parameters for this model}
}
\value{
\describe{
\item{model_log_like}{log likelihood of analysis model}
\item{bench_log_like}{log likelihood of benchmark model}
\item{null_log_like}{log likelihood of null model}
\item{model_Chi_sq}{Chi-Square statistics for analysis model}
\item{null_Chi_sq}{Chi-Square statistics for null model}
\item{model_df}{degrees of freedom of analysis model}
\item{null_df}{degrees of freedom of null model}
\item{NFI}{Normed Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{RFI}{Relative Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{IFI}{Incremental Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{TLI}{Tucker-Lewis Index. Lager values closer to 1.0 indicate a better fit.}
\item{CFI}{Comparative Fit Index. Lager values closer to 1.0 indicate a better fit.}
\item{RMSEA}{Root Mean Square Error of Approximation. Smaller values closer to 0.0 indicate a better fit.}
\item{AIC}{Akaike Information Criterion. A lower value indicates a better fit.}
\item{CAIC}{Consistent AIC.A lower value indicates a better fit.}
\item{BIC}{Bayesian Information Criterion. A lower value indicates a better fit.}
}
}
\description{
A general function that returns the model fit indices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{percentile}
\alias{percentile}
\alias{percentile.default}
\alias{percentile.binary}
\title{Student Percentile Ranks}
\usage{
percentile(U, na = NULL, Z = NULL, w = NULL)

\method{percentile}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{percentile}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of percentile ranks (1-100) for each student, where:
\itemize{
\item 100: Highest performing student(s)
\item 50: Median performance
\item 1: Lowest performing student(s)
}
}
\description{
The percentile function calculates each student's relative standing in the group,
expressed as a percentile rank (1-100). This function is applicable only to
binary response data.

The percentile rank indicates the percentage of scores in the distribution
that fall below a given score. For example, a percentile rank of 75 means
the student performed better than 75\% of the group.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.

Percentile ranks are calculated using the empirical cumulative distribution
function of standardized scores. Tied scores receive the same percentile rank.
The values are rounded up to the nearest integer to provide ranks from 1 to 100.
}
\examples{
# using sample dataset
percentile(J5S10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_BINET.R
\name{BINET}
\alias{BINET}
\title{Bicluster Network Model}
\usage{
BINET(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  conf = NULL,
  ncls = NULL,
  nfld = NULL,
  g_list = NULL,
  adj_list = NULL,
  adj_file = NULL,
  verbose = FALSE
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{conf}{For the confirmatory parameter, you can input either a vector with
items and corresponding fields in sequence, or a field membership profile
matrix. In the case of the former, the field membership profile matrix will be generated internally.
When providing a membership profile matrix, it needs to be either matrix or data.frame.
The number of fields(nfld) will be overwrite to the number of columns of this matrix.}

\item{ncls}{number of classes}

\item{nfld}{number of fields}

\item{g_list}{A list compiling graph-type objects for each rank/class.}

\item{adj_list}{A list compiling matrix-type adjacency matrices for each rank/class.}

\item{adj_file}{A file detailing the relationships of the graph for each rank/class,
listed in the order of starting point, ending point, and rank(class).}

\item{verbose}{verbose output Flag. default is TRUE}
}
\value{
\describe{
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{msg}{A character string indicating the model type. }
\item{testlength}{Length of the test. The number of items included in the test.}
\item{Nclass}{Optimal number of classes.}
\item{Nfield}{Optimal number of fields.}
\item{crr}{Correct Response Rate}
\item{ItemLabel}{Label of Items}
\item{FieldLabel}{Label of Fields}
\item{all_adj}{Integrated Adjacency matrix used to plot graph.}
\item{all_g}{Integrated graph object used to plot graph.see also
\link{plot.exametrika}}
\item{adj_list}{List of Adjacency matrix used in the model}
\item{params}{A list of the estimated conditional probabilities.
It indicates which path was obtained from which parent node(class) to
which child node(class), held by \code{parent}, \code{child}, and \code{field}. The item
Items contained in the field is in \code{fld}. Named \code{chap} includes the
conditional correct response answer rate of the child node, while \code{pap}
contains the pass rate of the parent node.}
\item{PSRP}{Response pattern by the students belonging to the parent
classes of Class c. A more comprehensible arrangement of \code{params.}}
\item{LCD}{Latent Class Distribution. see also \link{plot.exametrika}}
\item{LFD}{Latent Field Distribution. see also \link{plot.exametrika}}
\item{CMD}{Class Membership Distribution.}
\item{FRP}{Marginal bicluster reference matrix.}
\item{FRPIndex}{Index of FFP includes the item location parameters B and Beta,
the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
\item{TRP}{Test Reference Profile}
\item{LDPSR}{A rearranged set of parameters for output. It includes
the field the items contained within that field, and the conditional
correct response rate of parent nodes(class) and child node(class).}
\item{FieldEstimated}{Given vector which correspondence between items
and the fields.}
\item{Students}{Rank Membership Profile matrix.The s-th row vector of \eqn{\hat{M}_R}, \eqn{\hat{m}_R}, is the
rank membership profile of Student s, namely the posterior probability distribution representing the student's
belonging to the respective latent classes. }
\item{NextStage}{The next class that easiest for students to move to,
its membership probability, class-up odds, and the field required for
more.}
\item{MG_FitIndices}{Multigroup as Null model.See also \link{TestFit}}
\item{SM_FitIndices}{Saturated Model as Null model.See also \link{TestFit}}
}
}
\description{
Bicluster Network Model: BINET is a model that combines the Bayesian
network model and Biclustering. BINET is very similar to LDB and LDR.
The most significant difference is that in LDB, the nodes represent
the fields, whereas in BINET, they represent the class. BINET
explores the local dependency structure among latent classes at each
latent field, where each field is a locus.
}
\examples{
\donttest{
# Example: Bicluster Network Model (BINET)
# BINET combines Bayesian network model and Biclustering to explore
# local dependency structure among latent classes at each field

# Create field configuration vector based on field assignments
conf <- c(
  1, 5, 5, 5, 9, 9, 6, 6, 6, 6, 2, 7, 7, 11, 11, 7, 7,
  12, 12, 12, 2, 2, 3, 3, 4, 4, 4, 8, 8, 12, 1, 1, 6, 10, 10
)

# Create edge data for network structure between classes
edges_data <- data.frame(
  "From Class (Parent) >>>" = c(
    1, 2, 3, 4, 5, 7, 2, 4, 6, 8, 10, 6, 6, 11, 8, 9, 12
  ),
  ">>> To Class (Child)" = c(
    2, 4, 5, 5, 6, 11, 3, 7, 9, 12, 12, 10, 8, 12, 12, 11, 13
  ),
  "At Field (Locus)" = c(
    1, 2, 2, 3, 4, 4, 5, 5, 5, 5, 5, 7, 8, 8, 9, 9, 12
  )
)

# Save edge data to temporary CSV file
tmp_file <- tempfile(fileext = ".csv")
write.csv(edges_data, file = tmp_file, row.names = FALSE)

# Fit Bicluster Network Model
result.BINET <- BINET(
  J35S515,
  ncls = 13, # Maximum class number from edges (13)
  nfld = 12, # Maximum field number from conf (12)
  conf = conf, # Field configuration vector
  adj_file = tmp_file # Path to the CSV file
)

# Clean up temporary file
unlink(tmp_file)

# Display model results
print(result.BINET)

# Visualize different aspects of the model
plot(result.BINET, type = "Array") # Show bicluster structure
plot(result.BINET, type = "TRP") # Test Response Profile
plot(result.BINET, type = "LRD") # Latent Rank Distribution
plot(result.BINET,
  type = "RMP", # Rank Membership Profiles
  students = 1:9, nc = 3, nr = 3
)
plot(result.BINET,
  type = "FRP", # Field Reference Profiles
  nc = 3, nr = 2
)
plot(result.BINET,
  type = "LDPSR", # Locally Dependent Passing Student Rates
  nc = 3, nr = 2
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_CTT.R
\name{CTT}
\alias{CTT}
\title{Classical Test Theory}
\usage{
CTT(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{U is a data matrix of the type matrix or data.frame.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}
}
\value{
Returns a list of class c("exametrika", "CTT") containing two data frames:
\describe{
\item{Reliability}{A data frame with overall reliability coefficients
(Alpha and Omega) calculated using different correlation matrices
(Covariance, Phi, and Tetrachoric)}
\item{ReliabilityExcludingItem}{A data frame showing alpha coefficients
when each item is excluded, calculated using different correlation matrices}
}
}
\description{
This function calculates the overall alpha and omega coefficients for
the given data matrix. It also computes the alpha coefficient for
each item, assuming that item is excluded.
}
\examples{
\donttest{
# using sample dataset
CTT(J15S500)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/21_DistractorAnalysis.R
\name{DistractorAnalysis}
\alias{DistractorAnalysis}
\alias{DistractorAnalysis.LRArated}
\alias{DistractorAnalysis.ratedBiclustering}
\alias{print.DistractorAnalysis}
\alias{plot.DistractorAnalysis}
\title{Distractor Analysis}
\usage{
DistractorAnalysis(x, ...)

\method{DistractorAnalysis}{LRArated}(x, ...)

\method{DistractorAnalysis}{ratedBiclustering}(x, ...)

\method{print}{DistractorAnalysis}(x, items = NULL, ranks = NULL, digits = 4, ...)

\method{plot}{DistractorAnalysis}(x, type = "Distractor", items = NULL, ranks = NULL, nc = 1, nr = 1, ...)
}
\arguments{
\item{x}{A result object from a rated model (class \code{LRArated} or
\code{ratedBiclustering}).}

\item{...}{Additional arguments (currently unused).}

\item{items}{Integer vector of item indices to display. NULL for all items.}

\item{ranks}{Integer vector of rank/class indices to display. NULL for all ranks.}

\item{digits}{Number of digits for rounding. Default is 4.}

\item{type}{Plot type. Currently only "Distractor" is supported.}

\item{nc}{Number of columns in the plot grid.}

\item{nr}{Number of rows in the plot grid.}
}
\value{
An object of class \code{c("exametrika", "DistractorAnalysis")} containing:
\describe{
\item{freq_table}{List of frequency matrices (nrank x maxQ), one per item.}
\item{prop_table}{List of proportion matrices (nrank x maxQ), one per item.}
\item{chisq_table}{Matrix (nitems x nrank) of chi-square statistics.}
\item{pvalue_table}{Matrix (nitems x nrank) of p-values.}
\item{cramersv_table}{Matrix (nitems x nrank) of Cramer's V effect sizes.}
\item{CA}{Correct answer vector.}
\item{n_rank}{Number of ranks/classes.}
\item{maxQ}{Number of response categories.}
\item{nitems}{Number of items.}
\item{ItemLabel}{Item label vector.}
\item{n_field}{Number of fields (Biclustering only).}
\item{FieldEstimated}{Field assignment vector (Biclustering only).}
\item{field_items}{List of item indices per field (Biclustering only).}
}
}
\description{
Performs distractor analysis for rated (multiple-choice) models.
For each item and rank/class, computes observed category frequency tables,
chi-square tests against chance level (uniform distribution), and
Cramer's V as an effect size measure.

This function works with results from \code{\link{LRA}} (rated data) and
\code{\link{Biclustering}} / \code{\link{Biclustering_IRM}} (rated data).
}
\examples{
\donttest{
# LRA.rated example
result_lra <- LRA(J21S300, nrank = 5, mic = TRUE)
da <- DistractorAnalysis(result_lra)
print(da)
print(da, items = 1:3)
print(da, ranks = c(1, 5))
plot(da)
plot(da, items = 1:6, nc = 3, nr = 2)

# Biclustering.rated example
result_bic <- Biclustering(J21S300, ncls = 5, nfld = 3, method = "R")
da_bic <- DistractorAnalysis(result_bic)
print(da_bic, items = 1:7)
plot(da_bic, items = 1:6, nc = 3, nr = 2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04C_ParameterEstimation.R
\name{asymprior}
\alias{asymprior}
\title{Prior distribution function with guessing parameter}
\usage{
asymprior(c, alp, bet)
}
\arguments{
\item{c}{guessing parameter}

\item{alp}{prior to be set}

\item{bet}{prior to be set}
}
\description{
Prior distribution function with guessing parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{CSR}
\alias{CSR}
\title{Conditional Selection Rate}
\usage{
CSR(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A list of Joint Selection Rate matrices for each item pair.
}
\description{
Calculate the Conditional Selection Rate (CSR) for polytomous data.
CSR measures the proportion of respondents who selected a specific category
in item K, given that they selected a particular category in item J.
}
\details{
The function returns a nested list structure CSR, where \code{CSR[[j]][[k]]} contains
a matrix of conditional probabilities. In this matrix, the element at row l and
column m represents P(K=m|J=l), which is the probability of selecting category m
for item K, given that category l was selected for item J.

Mathematically, for each cell (l,m) in the \code{CSR[[j]][[k]]} matrix:
\code{CSR[[j]][[k]][l,m] = P(Item K = category m | Item J = category l)}

This is calculated as the number of respondents who selected both category l for
item J and category m for item K, divided by the total number of respondents who
selected category l for item J.
}
\examples{
# example code
# Calculate CSR using sample dataset J5S1000
CSR(J5S1000)

# Extract the conditional selection rates from item 1 to item 2
csr_1_2 <- CSR(J5S1000)[[1]][[2]]
# This shows the probability of selecting each category in item 2
# given that a specific category was selected in item 1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_LDLRA.R
\name{LD_param_est}
\alias{LD_param_est}
\title{LDparam set}
\usage{
LD_param_est(
  tmp,
  adj_list,
  classRefMat,
  ncls,
  smoothpost,
  beta1 = 2,
  beta2 = 2
)
}
\arguments{
\item{tmp}{tmp}

\item{adj_list}{adj_list}

\item{classRefMat}{values returned from emclus}

\item{ncls}{ncls}

\item{smoothpost}{smoothpost}

\item{beta1}{Beta distribution parameter 1 for prior density. Default is 2.}

\item{beta2}{Beta distribution parameter 2 for prior density. Default is 2.}
}
\description{
A function that extracts only the estimation of graph parameters
after the rank estimation is completed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02D_StudentAnalysis.R
\name{StudentAnalysis}
\alias{StudentAnalysis}
\title{StudentAnalysis}
\usage{
StudentAnalysis(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{U is a data matrix of the type matrix or data.frame.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.
\itemize{
\item ID: Student identifier
\item NR: Number of responses
\item NRS: Number-right score (total correct answers)
\item PR: Passage rate (proportion correct)
\item SS: Standardized score (z-score)
\item Percentile: Student's percentile rank
\item Stanine: Student's stanine score (1-9)
}}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}
}
\value{
Returns a data frame containing the following columns for each student:
\itemize{
\item ID: Student identifier
\item NR: Number of responses
\item NRS: Number-right score (total correct answers)
\item PR: Passage rate (proportion correct)
\item SS: Standardized score (z-score)
\item Percentile: Student's percentile rank
\item Stanine: Student's stanine score (1-9)
}
}
\description{
The StudentAnalysis function returns descriptive statistics for each individual student.
Specifically, it provides the number of responses, the number of correct answers,
the passage rate, the standardized score, the percentile, and the stanine.
}
\examples{
# using sample dataset
StudentAnalysis(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{MutualInformation}
\alias{MutualInformation}
\alias{MutualInformation.default}
\alias{MutualInformation.binary}
\alias{MutualInformation.ordinal}
\title{Mutual Information}
\usage{
MutualInformation(U, na = NULL, Z = NULL, w = NULL, base = 2)

\method{MutualInformation}{default}(U, na = NULL, Z = NULL, w = NULL, base = 2)

\method{MutualInformation}{binary}(U, na = NULL, Z = NULL, w = NULL, base = 2)

\method{MutualInformation}{ordinal}(U, na = NULL, Z = NULL, w = NULL, base = 2)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}

\item{base}{The base for the logarithm. Default is 2. For polytomous data,
you can use "V" to set the base to min(rows, columns), "e" for natural logarithm (base e),
or any other number to use that specific base.}
}
\value{
A matrix of mutual information values with exametrika class.
Each element (i,j) represents the mutual information between items i and j,
measured in bits. Higher values indicate stronger interdependence between items.
}
\description{
Mutual Information is a measure that represents the degree of interdependence
between two items. This function is applicable to both binary and polytomous response data.
The measure is calculated using the joint probability distribution of responses
between item pairs and their marginal probabilities.
}
\details{
For binary data, the following formula is used:
\deqn{
MI_{jk} = p_{00} \log_2 \frac{p_{00}}{(1-p_j)(1-p_k)} + p_{01} \log_2 \frac{p_{01}}{(1-p_j)p_k}
 + p_{10} \log_2 \frac{p_{10}}{p_j(1-p_k)} + p_{11} \log_2 \frac{p_{11}}{p_jp_k}
}
Where:
\itemize{
\item \eqn{p_{00}} is the joint probability of incorrect responses to both items j and k
\item \eqn{p_{01}} is the joint probability of incorrect response to item j and correct to item k
\item \eqn{p_{10}} is the joint probability of correct response to item j and incorrect to item k
\item \eqn{p_{11}} is the joint probability of correct responses to both items j and k
}

For polytomous data, the following formula is used:
\deqn{MI_{jk} = \sum_{j=1}^{C_j}\sum_{k=1}^{C_k}p_{jk}\log \frac{p_{jk}}{p_{j.}p_{.k}}}

The base of the logarithm can be the number of rows, number of columns, min(rows, columns),
base-10 logarithm, natural logarithm (e), etc.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# example code
# Calculate Mutual Information using sample dataset J15S500
MutualInformation(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J12S5000}
\alias{J12S5000}
\title{J12S5000}
\format{
An exametrika class object with 5000 students and 12 items containing binary (0/1) responses
}
\source{
\url{http://sh0j1ma.stars.ne.jp/exmk/}
}
\usage{
J12S5000
}
\description{
A binary response dataset for test analysis
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{ITBiserial}
\alias{ITBiserial}
\alias{ITBiserial.default}
\alias{ITBiserial.binary}
\title{Item-Total Biserial Correlation}
\usage{
ITBiserial(U, na = NULL, Z = NULL, w = NULL)

\method{ITBiserial}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{ITBiserial}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of item-total biserial correlations. Values range
from -1 to 1, where:
\itemize{
\item Values near 1: Strong positive discrimination
\item Values near 0: No discrimination
\item Negative values: Potential item problems
}
}
\description{
The Item-Total Biserial Correlation computes the biserial correlation
between each item and the total score. This function is applicable only
to binary response data.

This correlation provides a measure of item discrimination, indicating how well
each item distinguishes between high and low performing examinees.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.

The biserial correlation is generally preferred over the point-biserial
correlation when the dichotomization is artificial (i.e., when the underlying
trait is continuous).
}
\examples{
# using sample dataset
ITBiserial(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_QitemFunctions.R
\name{polychoric_likelihood}
\alias{polychoric_likelihood}
\title{Calculate Polychoric Correlation Likelihood}
\usage{
polychoric_likelihood(rho, mat)
}
\arguments{
\item{rho}{Numeric value between -1 and 1, the correlation coefficient}

\item{mat}{A contingency table matrix for two ordinal variables}
}
\value{
The negative log-likelihood value for the given correlation coefficient
}
\description{
Calculates the negative log-likelihood for estimating polychoric correlation
from a contingency table of two ordinal variables.
}
\details{
The function estimates thresholds from the marginal distributions and
calculates the expected probabilities based on a bivariate normal distribution.
It then computes the log-likelihood by comparing observed and expected frequencies.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/23_Chatterjee.R
\name{chatterjee_matrix}
\alias{chatterjee_matrix}
\title{Pairwise Chatterjee's xi correlation matrix}
\usage{
chatterjee_matrix(
  U,
  na = NULL,
  Z = NULL,
  w = NULL,
  B = 1000,
  seed = NULL,
  verbose = FALSE
)
}
\arguments{
\item{U}{Either an exametrika object or raw data.}

\item{na}{Values to be treated as missing.}

\item{Z}{Missing indicator matrix.}

\item{w}{Item weight vector.}

\item{B}{Number of bootstrap replications per pair. Default 1000.}

\item{seed}{Optional integer seed for reproducibility.}

\item{verbose}{Logical. If TRUE, progress messages are displayed.}
}
\value{
A p x p numeric matrix with item labels as row/column names.
The diagonal is 1; off-diagonal entries are asymmetric.
}
\description{
Computes the p x p asymmetric matrix of Chatterjee's xi correlations
between all item pairs. Entry \verb{[j, k]} is xi(item_j, item_k), which in
general differs from xi(item_k, item_j); this asymmetry is what
enables direction detection in subsequent graph-construction steps.
Each off-diagonal entry is computed by \code{xi_stable()} to average
over tie-breaking.
}
\details{
Pairs are computed with pairwise-complete observations: for each
pair (j, k), rows where either \code{Q[i, j]} or \code{Q[i, k]} is missing are
excluded. The diagonal is set to 1.
}
\examples{
\donttest{
xi_mat <- chatterjee_matrix(J15S3810, B = 500, seed = 42)
}
}
\references{
Chatterjee, S. (2021). A new coefficient of correlation. Journal of
the American Statistical Association, 116(536), 2009-2022.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_IRM.R, R/17_Biclustering_nominal_IRM.R,
%   R/18_Biclustering_ordinal_IRM.R, R/20_Biclustering_rated_IRM.R
\name{Biclustering_IRM}
\alias{Biclustering_IRM}
\alias{Biclustering_IRM.default}
\alias{Biclustering_IRM.binary}
\alias{Biclustering_IRM.nominal}
\alias{Biclustering_IRM.ordinal}
\alias{Biclustering_IRM.rated}
\title{Biclustering with Infinite Relational Model}
\usage{
Biclustering_IRM(U, ...)

\method{Biclustering_IRM}{default}(U, na = NULL, Z = NULL, w = NULL, ...)

\method{Biclustering_IRM}{binary}(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  gamma_c = 1,
  gamma_f = 1,
  max_iter = 100,
  stable_limit = 5,
  minSize = 20,
  EM_limit = 20,
  seed = 123,
  verbose = TRUE,
  ...
)

\method{Biclustering_IRM}{nominal}(
  U,
  gamma_c = 1,
  gamma_f = 1,
  alpha = 1,
  max_iter = 100,
  stable_limit = 5,
  minSize = 20,
  EM_limit = 20,
  seed = 123,
  verbose = TRUE,
  ...
)

\method{Biclustering_IRM}{ordinal}(
  U,
  gamma_c = 1,
  gamma_f = 1,
  alpha = 1,
  mic = TRUE,
  max_iter = 100,
  stable_limit = 5,
  minSize = 20,
  EM_limit = 100,
  seed = 123,
  verbose = TRUE,
  ...
)

\method{Biclustering_IRM}{rated}(
  U,
  gamma_c = 1,
  gamma_f = 1,
  alpha = 1,
  max_iter = 100,
  stable_limit = 5,
  minSize = 20,
  EM_limit = 20,
  seed = 123,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{...}{Additional arguments passed to specific methods.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{gamma_c}{\eqn{\gamma_C} is the hyperparameter of the CRP and represents the
attractiveness of a new Class. As \eqn{\gamma_C} increases, the student is more likely
to be seated at a vacant class. The default is 1.}

\item{gamma_f}{\eqn{\gamma_F} is the hyperparameter of the CRP and represents the
attractiveness of a new Field. The greater this value it more likely to be classified
in the new field. The default is 1.}

\item{max_iter}{A maximum iteration number of IRM process. The default is 100.}

\item{stable_limit}{The IRM process exits the loop when the FRM stabilizes and no longer
changes significantly. This option sets the maximum number of stable iterations,
with a default of 5.}

\item{minSize}{A value used for readjusting the number of classes.If the size of each
class is less than \code{minSize}, the number of classes will be reduced. Note that this
under limit of size is not used for either all correct or all incorrect class.}

\item{EM_limit}{After IRM process, resizing the number of classes process will starts.
This process using EM algorithm,\code{EM_limit} is the maximum number of iteration with
default of 20.}

\item{seed}{Random seed for reproducibility. When a numeric value is provided,
\code{set.seed(seed)} is called before the Gibbs sampling begins, ensuring
reproducible results. The default is \code{123}, which guarantees deterministic
output. Set to \code{NULL} to disable seed setting and let the results depend
on the current state of the random number generator.}

\item{verbose}{verbose output Flag. default is TRUE}

\item{alpha}{Dirichlet distribution concentration parameter for the prior density of
field reference probabilities (rated/nominal IRM). Must be positive. The default is 1.}

\item{mic}{Logical; if TRUE, forces Field Reference Profiles to be monotonically
increasing across classes (ordinal IRM only). Default is TRUE.}
}
\value{
An object of class "exametrika" containing the IRM results.
See \code{Biclustering_IRM.binary} or \code{Biclustering_IRM.nominal} for details.

\describe{
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{msg}{A character string indicating the model type. }
\item{testlength}{Length of the test. The number of items included in the test.}
\item{n_class}{Optimal number of classes (new naming convention).}
\item{n_field}{Optimal number of fields (new naming convention).}
\item{em_cycle}{Number of EM algorithm iterations (new naming convention).}
\item{Nclass}{Optimal number of classes (deprecated, use n_class).}
\item{Nfield}{Optimal number of fields (deprecated, use n_field).}
\item{EM_Cycle}{Number of EM algorithm iterations (deprecated, use em_cycle).}
\item{BRM}{Bicluster Reference Matrix}
\item{FRP}{Field Reference Profile}
\item{FRPIndex}{Index of FFP includes the item location parameters B and Beta,
the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
\item{TRP}{Test Reference Profile}
\item{FMP}{Field Membership Profile}
\item{Students}{Rank Membership Profile matrix.The s-th row vector of \eqn{\hat{M}_R}, \eqn{\hat{m}_R}, is the
rank membership profile of Student s, namely the posterior probability distribution representing the student's
belonging to the respective latent classes. It also includes the rank with the maximum estimated membership probability,
as well as the rank-up odds and rank-down odds.}
\item{LRD}{Latent Rank Distribution. see also \link{plot.exametrika}}
\item{LFD}{Latent Field Distribution. see also \link{plot.exametrika}}
\item{RMD}{Rank Membership Distribution.}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
}

For nominal data, the returned list includes:
\describe{
\item{Q}{Response matrix.}
\item{Z}{Missing indicator matrix.}
\item{testlength}{Number of items.}
\item{nobs}{Sample size.}
\item{n_class}{Optimal number of classes.}
\item{n_field}{Optimal number of fields.}
\item{n_cycle}{Number of EM algorithm iterations.}
\item{FRP}{Field Reference Profile, a 3D array (nfld x ncls x maxQ).}
\item{LFD}{Latent Field Distribution.}
\item{LCD}{Latent Class Distribution.}
\item{FieldMembership}{Field membership probability matrix.}
\item{ClassMembership}{Class membership probability matrix.}
\item{FieldEstimated}{Estimated field assignment for each item.}
\item{ClassEstimated}{Estimated class assignment for each student.}
\item{Students}{Rank Membership Profile matrix with estimated class.}
\item{TestFitIndices}{Overall fit index for the test.}
\item{log_lik}{Log-likelihood of the model.}
}

For ordinal data, the returned list includes:
\describe{
\item{Q}{Response matrix.}
\item{Z}{Missing indicator matrix.}
\item{testlength}{Number of items.}
\item{nobs}{Sample size.}
\item{n_class}{Optimal number of classes.}
\item{n_field}{Optimal number of fields.}
\item{n_cycle}{Number of EM algorithm iterations.}
\item{FRP}{Field Reference Profile (BCRM), a 3D array (nfld x ncls x maxQ).}
\item{FRPIndex}{Index of FRP includes the item location parameters B and Beta,
the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
\item{TRP}{Test Reference Profile.}
\item{BFRP}{Bicluster Field Reference Profile (expected scores), a list with
Weighted and Observed components.}
\item{LFD}{Latent Field Distribution.}
\item{LCD}{Latent Class Distribution.}
\item{FieldMembership}{Field membership probability matrix.}
\item{ClassMembership}{Class membership probability matrix.}
\item{FieldEstimated}{Estimated field assignment for each item.}
\item{ClassEstimated}{Estimated class assignment for each student.}
\item{Students}{Rank Membership Profile matrix with estimated class.}
\item{TestFitIndices}{Overall fit index for the test.}
\item{log_lik}{Log-likelihood of the model.}
\item{SOACflg}{Logical; TRUE if Strongly Ordinal Alignment Condition is satisfied.}
\item{WOACflg}{Logical; TRUE if Weakly Ordinal Alignment Condition is satisfied.}
}

For rated data, the returned list includes:
\describe{
\item{Q}{Response matrix.}
\item{U}{Binary correct/incorrect matrix.}
\item{Z}{Missing indicator matrix.}
\item{testlength}{Number of items.}
\item{nobs}{Sample size.}
\item{n_class}{Optimal number of classes.}
\item{n_field}{Optimal number of fields.}
\item{n_cycle}{Number of EM algorithm iterations.}
\item{FRP}{Field Reference Profile (BCRM), a 3D array (nfld x ncls x maxQ).}
\item{FieldFRP}{Field Reference Profile based on binary correct rates (nfld x ncls).}
\item{quasiFRP}{Item-level correct response rate per class (nitems x ncls).}
\item{FRPIndex}{Index of FRP includes the item location parameters B and Beta,
the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
\item{TRP}{Test Reference Profile.}
\item{LFD}{Latent Field Distribution.}
\item{LCD}{Latent Class Distribution.}
\item{FieldMembership}{Field membership probability matrix.}
\item{ClassMembership}{Class membership probability matrix.}
\item{FieldEstimated}{Estimated field assignment for each item.}
\item{ClassEstimated}{Estimated class assignment for each student.}
\item{Students}{Rank Membership Profile matrix with estimated class.}
\item{FieldAnalysis}{Field analysis with CRR and field memberships.}
\item{TestFitIndices}{Overall fit index for the test (binary layer, default).}
\item{TestFitIndices_nominal}{Fit indices for the nominal layer (AIC/BIC/CAIC only).}
\item{log_lik}{Log-likelihood of the binary layer model.}
\item{log_lik_nominal}{Log-likelihood of the nominal layer model.}
\item{SOACflg}{Logical; TRUE if Strongly Ordinal Alignment Condition is satisfied.}
\item{WOACflg}{Logical; TRUE if Weakly Ordinal Alignment Condition is satisfied.}
}
}
\description{
This function performs Biclustering structure learning using the Infinite Relational Model (IRM)
to automatically determine the optimal number of classes C and optimal number of
fields F. It dispatches to the appropriate method based on the data type:
binary, ordinal, or nominal. For binary data, see Section 7.8 in Shojima(2022).
For nominal data, a Dirichlet-Multinomial collapsed Gibbs sampler is used.
}
\examples{
\donttest{
# Fit a Biclustering model with automatic structure learning using IRM
# gamma_c and gamma_f are concentration parameters for the Chinese Restaurant Process
result <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)

# Display the Bicluster Reference Matrix (BRM) as a heatmap
plot(result, type = "Array")

# Plot Field Reference Profiles (FRP) in a 3-column grid
plot(result, type = "FRP", nc = 3)
}

\donttest{
result <- Biclustering_IRM(J35S515, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result, type = "Array")
}
\donttest{
# Fit a nominal Biclustering IRM model
result <- Biclustering_IRM(J20S600, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result, type = "Array")
}
\donttest{
# Fit an ordinal Biclustering IRM model
result <- Biclustering_IRM(J35S500, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result, type = "Array")
}
\donttest{
# Fit a rated Biclustering IRM model
result <- Biclustering_IRM(J21S300, gamma_c = 1, gamma_f = 1, verbose = TRUE)
plot(result, type = "Array")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{ItemTotalCorr}
\alias{ItemTotalCorr}
\alias{ItemTotalCorr.default}
\alias{ItemTotalCorr.binary}
\alias{ItemTotalCorr.ordinal}
\title{Item-Total Correlation}
\usage{
ItemTotalCorr(U, na = NULL, Z = NULL, w = NULL)

\method{ItemTotalCorr}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemTotalCorr}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemTotalCorr}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of item-total correlations. Values typically range
from -1 to 1, where:
\itemize{
\item Values near 1: Strong positive discrimination
\item Values near 0: No discrimination
\item Negative values: Potential item problems (lower ability students
performing better than higher ability students)
}
}
\description{
Item-Total correlation (ITC) is a Pearson's correlation of an item with
the Number-Right Score (NRS) or total score. This function is applicable
only to binary response data.

The ITC is a measure of item discrimination, indicating how well an item
distinguishes between high and low performing examinees.
}
\details{
The correlation is calculated between:
\itemize{
\item Each item's responses (0 or 1)
\item The total test score (sum of correct responses)
}
Higher positive correlations indicate items that better discriminate between
high and low ability examinees.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.

Values below 0.2 might indicate problematic items that should be reviewed.
Values above 0.3 are generally considered acceptable.
}
\examples{
# using sample dataset
ItemTotalCorr(J15S500)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{JointSampleSize}
\alias{JointSampleSize}
\alias{JointSampleSize.default}
\alias{JointSampleSize.binary}
\title{Joint Sample Size}
\usage{
JointSampleSize(U, na = NULL, Z = NULL, w = NULL)

\method{JointSampleSize}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{JointSampleSize}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
Returns a matrix of class c("exametrika", "matrix") where each element (i,j)
represents the number of students who responded to both item i and item j. The
diagonal elements represent the total number of responses for each item.
}
\description{
The joint sample size is a matrix whose elements are the number of
individuals who responded to each pair of items.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{IIF2PLM}
\alias{IIF2PLM}
\title{IIF for 2PLM}
\usage{
IIF2PLM(a, b, theta)
}
\arguments{
\item{a}{slope parameter}

\item{b}{location parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector representing the item information at each ability
level theta. The information is calculated as:
\eqn{I(\theta) = a^2P(\theta)(1-P(\theta))}
}
\description{
Item Information Function for 2PLM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/23_Chatterjee.R
\name{xi_stable}
\alias{xi_stable}
\title{Bootstrap-averaged Chatterjee's xi}
\usage{
xi_stable(x, y, B = 1000, seed = NULL)
}
\arguments{
\item{x}{Numeric or ordered factor (predictor)}

\item{y}{Numeric or ordered factor (response)}

\item{B}{Number of bootstrap replications. Default 1000.}

\item{seed}{Optional integer seed for reproducibility.}
}
\value{
A list with elements:
\describe{
\item{xi}{Mean of the B Chatterjee xi values.}
\item{sd}{Standard deviation of the B values (tie-breaking spread).}
\item{se}{Standard error of the mean (sd / sqrt(B)).}
\item{B}{Number of bootstrap replications used.}
}
}
\description{
Computes Chatterjee's xi B times with random tie-breaking and returns
the average. This stabilizes the estimate against tie-breaking
variability, which is important for ordinal data with many ties (e.g.,
Likert-scale items).
}
\details{
For each of the B replications, ties in x are broken randomly via
\code{rank(x, ties.method = "random")} inside \code{chatterjee_xi()}.
The standard error of the mean shrinks as 1 / sqrt(B); B = 1000 typically
yields SE around 0.001 for moderately tied data.
}
\examples{
\donttest{
x <- rnorm(100)
y <- x^2 + rnorm(100, sd = 0.1)
xi_stable(x, y, B = 500, seed = 42)
}
}
\references{
Chatterjee, S. (2021). A new coefficient of correlation. Journal of
the American Statistical Association, 116(536), 2009-2022.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_LRA.R, R/12_LRA_ordinal.R, R/13_LRA_rated.R
\name{LRA}
\alias{LRA}
\alias{LRA.default}
\alias{LRA.binary}
\alias{LRA.ordinal}
\alias{LRA.rated}
\title{Latent Rank Analysis}
\usage{
LRA(U, ...)

\method{LRA}{default}(U, na = NULL, Z = NULL, w = NULL, ...)

\method{LRA}{binary}(
  U,
  nrank = 2,
  method = "GTM",
  mic = FALSE,
  maxiter = 100,
  BIC.check = FALSE,
  seed = NULL,
  verbose = FALSE,
  beta1 = 1,
  beta2 = 1,
  conf = NULL,
  ...
)

\method{LRA}{ordinal}(
  U,
  nrank = 2,
  mic = FALSE,
  maxiter = 100,
  trapezoidal = 0,
  eps = 1e-04,
  verbose = FALSE,
  ...
)

\method{LRA}{rated}(
  U,
  nrank = 2,
  mic = FALSE,
  maxiter = 100,
  trapezoidal = 0,
  eps = 1e-04,
  minFreqRatio = 0,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{...}{Additional arguments passed to specific methods.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. 1 indicates observed values, 0 indicates missing values.}

\item{w}{Item weight vector.}

\item{nrank}{Number of latent ranks to estimate. Must be between 2 and 20.}

\item{method}{For binary data only. Either "SOM" (Self-Organizing Maps) or "GTM" (Gaussian Topographic Mapping). Default is "GTM".}

\item{mic}{Logical; if TRUE, forces Item Reference Profiles to be monotonically increasing. Default is FALSE.}

\item{maxiter}{Maximum number of iterations for estimation. Default is 100.}

\item{BIC.check}{For binary data with SOM method only. If TRUE, convergence is checked using BIC values. Default is FALSE.}

\item{seed}{For binary data with SOM method only. Random seed for reproducibility.}

\item{verbose}{Logical; if TRUE, displays detailed progress during estimation. Default is TRUE.}

\item{beta1}{Beta distribution parameter 1 for prior density of rank reference matrix (GTM method only). Default is 1.}

\item{beta2}{Beta distribution parameter 2 for prior density of rank reference matrix (GTM method only). Default is 1.}

\item{conf}{Confirmatory IRP matrix (items x nrank) for test equating.
Same format as the IRP output. Non-NA values are fixed throughout estimation,
NA values are freely estimated. Fixed values must be in the open interval (0, 1).
When row names are present, items are matched by label; otherwise by position.
Default is NULL (fully exploratory).}

\item{trapezoidal}{Specifies the height of both tails when using a trapezoidal
prior distribution. Must be less than 1/nrank. The default value is 0, which
results in a uniform prior distribution.}

\item{eps}{Convergence threshold for parameter updates. Default is 1e-4.}

\item{minFreqRatio}{Minimum frequency ratio for response categories (default = 0).
Categories with occurrence rates below this threshold will be excluded from analysis.
For example, if set to 0.1, response categories that appear in less than 10\% of
responses for an item will be omitted.}
}
\value{
A list of class "exametrika" and the specific subclass (e.g., "LRA", "LRAordinal", "LRArated")
containing the following common elements:
\describe{
\item{msg}{A character string indicating the model type. }
\item{testlength}{Length of the test (number of items).}
\item{nobs}{Sample size (number of rows in the dataset).}
\item{Nrank}{Number of latent ranks specified.}
\item{N_Cycle}{Number of EM algorithm iterations performed.}
\item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
\item{TRP}{Test Reference Profile vector showing expected scores at each rank.}
\item{LRD}{Latent Rank Distribution vector showing the number of examinees at each rank.}
\item{RMD}{Rank Membership Distribution vector showing the sum of probabilities for each rank.}
\item{Students}{Rank Membership Profile matrix showing the posterior probabilities of
examinees belonging to each rank, along with their estimated ranks and odds ratios.}
\item{ItemFitIndices}{Fit indices for each item. See also \code{\link{ItemFit}}.}
\item{TestFitIndices}{Overall fit indices for the test. See also \code{\link{TestFit}}.}
}

Each subclass returns additional specific elements, detailed in their respective documentation.

For binary data (\code{LRA.binary}), the returned list additionally includes:
\describe{
\item{IRP}{Item Reference Profile matrix showing the probability of correct response for each item across different ranks.}
\item{IRPIndex}{Item Response Profile indices including the location parameters B and Beta,
slope parameters A and Alpha, and monotonicity indices C and Gamma.}
}

For ordinal data (\code{LRA.ordinal}), the returned list additionally includes:
\describe{
\item{msg}{A character string indicating the model type. }
\item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
\item{ScoreReport}{Descriptive statistics of test performance, including sample size,
test length, central tendency, variability, distribution characteristics, and reliability.}
\item{ItemReport}{Basic statistics for each item including category proportions and item-total correlations.}
\item{ICBR}{Item Category Boundary Reference matrix showing cumulative probabilities for rank-category combinations.}
\item{ICRP}{Item Category Reference Profile matrix showing probability of response in each category by rank.}
\item{ScoreRankCorr}{Spearman's correlation between test scores and estimated ranks.}
\item{RankQuantCorr}{Spearman's correlation between estimated ranks and quantile groups.}
\item{ScoreRank}{Contingency table of raw scores by estimated ranks.}
\item{ScoreMembership}{Expected rank memberships for each raw score.}
\item{RankQuantile}{Cross-tabulation of rank frequencies and quantile groups.}
\item{MembQuantile}{Cross-tabulation of rank membership probabilities and quantile groups.}
\item{CatQuant}{Response patterns across item categories and quantile groups.}
}

For rated data (\code{LRA.rated}), the returned list additionally includes:
\describe{
\item{msg}{A character string indicating the model type. }
\item{converge}{Logical value indicating whether the algorithm converged within maxiter iterations}
\item{ScoreReport}{Descriptive statistics of test performance, including sample size,
test length, central tendency, variability, distribution characteristics, and reliability.}
\item{ItemReport}{Basic statistics for each item including category proportions and item-total correlations.}
\item{ICRP}{Item Category Reference Profile matrix showing probability of response in each category by rank.}
\item{ScoreRankCorr}{Spearman's correlation between test scores and estimated ranks.}
\item{RankQuantCorr}{Spearman's correlation between estimated ranks and quantile groups.}
\item{ScoreRank}{Contingency table of raw scores by estimated ranks.}
\item{ScoreMembership}{Expected rank memberships for each raw score.}
\item{RankQuantile}{Cross-tabulation of rank frequencies and quantile groups.}
\item{MembQuantile}{Cross-tabulation of rank membership probabilities and quantile groups.}
\item{ItemQuantileRef}{Reference values for each item across quantile groups.}
\item{CatQuant}{Response patterns across item categories and quantile groups.}
}
}
\description{
A general function for estimating Latent Rank Analysis across different response types.
This function automatically dispatches to the appropriate method based on the response type:

\itemize{
\item For binary data (\code{LRA.binary}): Analysis using either SOM or GTM method
\item For ordinal data (\code{LRA.ordinal}): Analysis using the GTM method with category thresholds
\item For rated data (\code{LRA.rated}): Analysis using the GTM method with rating categories
}

Latent Rank Analysis identifies underlying rank structures in test data and assigns
examinees to these ranks based on their response patterns.
}
\section{Binary Data Method}{

\code{LRA.binary} analyzes dichotomous (0/1) response data using either Self-Organizing Maps (SOM)
or Gaussian Topographic Mapping (GTM).
}

\section{Ordinal Data Method}{

\code{LRA.ordinal} analyzes ordered categorical data with multiple thresholds,
such as Likert-scale responses or graded items.
}

\section{Rated Data Method}{

\code{LRA.rated} analyzes data with ratings assigned to each response, such as
partially-credited items or preference scales where response categories have different weights.
}

\examples{
\donttest{
# Binary data example
# Fit a Latent Rank Analysis model with 6 ranks to binary data
result.LRA <- LRA(J15S500, nrank = 6)

# Display the first few rows of student rank membership profiles
head(result.LRA$Students)

# Plot Item Reference Profiles (IRP) for the first 6 items
plot(result.LRA, type = "IRP", items = 1:6, nc = 2, nr = 3)

# Plot Test Reference Profile (TRP) showing expected scores at each rank
plot(result.LRA, type = "TRP")
}

\donttest{
# Ordinal data example
# Fit a Latent Rank Analysis model with 3 ranks to ordinal data
result.LRAord <- LRA(J15S3810, nrank = 3, mic = TRUE)

# Plot score distributions
plot(result.LRAord, type = "ScoreFreq")
plot(result.LRAord, type = "ScoreRank")

# Plot category response patterns for items 1-6
plot(result.LRAord, type = "ICBR", items = 1:6, nc = 3, nr = 2)
plot(result.LRAord, type = "ICRP", items = 1:6, nc = 3, nr = 2)
}

\donttest{
# Rated data example
# Fit a Latent Rank Analysis model with 10 ranks to rated data
result.LRArated <- LRA(J35S5000, nrank = 10, mic = TRUE)

# Plot score distributions
plot(result.LRArated, type = "ScoreFreq")
plot(result.LRArated, type = "ScoreRank")

# Plot category response patterns for items 1-6
plot(result.LRArated, type = "ICRP", items = 1:6, nc = 3, nr = 2)
}

}
\seealso{
\code{\link{plot.exametrika}} for visualizing LRA results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J35S500}
\alias{J35S500}
\title{J35S500}
\format{
An exametrika class object with 500 students and 35 items containing ordinal responses with 5 categories
}
\usage{
J35S500
}
\description{
A simulated ordinal dataset for polytomous Biclustering analysis.
This is a synthetic dataset generated using random number generation
with a cumulative staircase pattern (5 latent classes, 5 fields,
5 response categories) for demonstration and testing purposes.
Contains approximately 0.5\\% missing values.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08C_BNM_GA.R
\name{BNM_GA}
\alias{BNM_GA}
\title{Structure Learning for BNM by simple GA}
\usage{
BNM_GA(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  seed = 123,
  population = 20,
  Rs = 0.5,
  Rm = 0.005,
  maxParents = 2,
  maxGeneration = 100,
  successiveLimit = 5,
  crossover = 0,
  elitism = 0,
  filename = NULL,
  verbose = TRUE
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{seed}{seed for random.}

\item{population}{Population size. The default is 20}

\item{Rs}{Survival Rate. The default is 0.5}

\item{Rm}{Mutation Rate. The default is 0.005}

\item{maxParents}{Maximum number of edges emanating from a single node. The default is 2.}

\item{maxGeneration}{Maximum number of generations.}

\item{successiveLimit}{Termination conditions. If the optimal individual does not change
for this number of generations, it is considered to have converged.}

\item{crossover}{Configure crossover using numerical values. Specify 0 for uniform
crossover, where bits are randomly copied from both parents. Choose 1 for single-point
crossover with one crossover point, and 2 for two-point crossover with two crossover points.
The default is 0.}

\item{elitism}{Number of elites that remain without crossover when transitioning to
the next generation.}

\item{filename}{Specify the filename when saving the generated adjacency matrix in CSV format.
The default is null, and no output is written to the file.}

\item{verbose}{verbose output Flag. default is TRUE}
}
\value{
\describe{
\item{adj}{Optimal adjacency matrix}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{crr}{correct response ratio}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{adj}{Adjacency matrix}
\item{param}{Learned Parameters}
\item{CCRR_table}{Correct Response Rate tables}
}
}
\description{
Generating a DAG from data using a genetic algorithm.
}
\details{
This function generates a DAG from data using a genetic algorithm.
Depending on the size of the data and the settings, the computation may
take a significant amount of computational time. For details on the
settings or algorithm, see Shojima(2022), section 8.5
}
\examples{
\donttest{
# Perform Structure Learning for Bayesian Network Model using Genetic Algorithm
# Parameters are set for balanced exploration and computational efficiency
BNM_GA(J5S10,
  population = 20, # Size of population in each generation
  Rs = 0.5, # 50\% survival rate for next generation
  Rm = 0.002, # 0.2\% mutation rate for genetic diversity
  maxParents = 2, # Maximum of 2 parent nodes per item
  maxGeneration = 100, # Maximum number of evolutionary steps
  crossover = 2, # Use two-point crossover method
  elitism = 2 # Keep 2 best solutions in each generation
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{ItemLift}
\alias{ItemLift}
\alias{ItemLift.default}
\alias{ItemLift.binary}
\title{Item Lift}
\usage{
ItemLift(U, na = NULL, Z = NULL, w = NULL)

\method{ItemLift}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemLift}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A matrix of item lift values with exametrika class.
Each element (j,k) represents the lift value of item k given item j,
which indicates how much more likely item k is to be correct given that
item j was answered correctly.
}
\description{
The lift is a commonly used index in a POS data analysis.
The item lift of Item k to Item j is defined as follow:
\eqn{ l_{jk} = \frac{p_{k\mid j}}{p_k} }
This function is applicable only to binary response data.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# example code
# Calculate ItemLift using sample dataset J5S10
ItemLift(J5S10)
}
\references{
Brin, S., Motwani, R., Ullman, J., & Tsur, S. (1997). Dynamic itemset counting and
implication rules for market basket data. In Proceedings of ACM SIGMOD International Conference
on Management of Data (pp. 255–264). https://dl.acm.org/doi/10.1145/253262.253325
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_IRM.R
\name{IRM}
\alias{IRM}
\title{IRM (Deprecated)}
\usage{
IRM(...)
}
\arguments{
\item{...}{All arguments passed to \code{\link{Biclustering_IRM}}}
}
\description{
This function has been renamed to \code{\link{Biclustering_IRM}}.
Please use \code{Biclustering_IRM} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02B_TestStatistics.R
\name{TestStatistics}
\alias{TestStatistics}
\alias{TestStatistics.default}
\alias{TestStatistics.binary}
\alias{TestStatistics.ordinal}
\title{Simple Test Statistics}
\usage{
TestStatistics(U, na = NULL, Z = NULL, w = NULL)

\method{TestStatistics}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{TestStatistics}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{TestStatistics}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
The returned object depends on the data type:

For binary data, a list of class c("exametrika", "TestStatistics") containing:
\describe{
\item{TestLength}{Length of the test. The number of items included in the test.}
\item{SampleSize}{Sample size. The number of rows in the dataset.}
\item{Mean}{Average number of correct answers.}
\item{SEofMean}{Standard error of mean.}
\item{Variance}{Variance of test scores.}
\item{SD}{Standard Deviation of test scores.}
\item{Skewness}{Skewness of score distribution (measure of asymmetry).}
\item{Kurtosis}{Kurtosis of score distribution (measure of tail extremity).}
\item{Min}{Minimum score.}
\item{Max}{Maximum score.}
\item{Range}{Range of scores (Max - Min).}
\item{Q1}{First quartile. Same as the 25th percentile.}
\item{Median}{Median. Same as the 50th percentile.}
\item{Q3}{Third quartile. Same as the 75th percentile.}
\item{IQR}{Interquartile range. Calculated by subtracting Q1 from Q3.}
\item{Stanine}{Stanine score boundaries, see \code{\link{stanine}}.}
}

For ordinal and rated data, the function calls \code{\link{ScoreReport}} and returns
its result. See \code{\link{ScoreReport}} for details of the returned object.

For nominal data, an error is returned as this function does not support nominal data.
}
\description{
Calculates descriptive statistics for test scores, providing a comprehensive
summary of central tendency, variability, and distribution shape.
Different statistics are calculated based on the data type (binary, ordinal, rated, or nominal).
}
\examples{
# Basic usage
stats <- TestStatistics(J15S500)
print(stats)

# Extract specific statistics
cat("Mean score:", stats$Mean, "\n")
cat("Standard deviation:", stats$SD, "\n")

# View score distribution summary
summary_stats <- data.frame(
  Min = stats$Min,
  Q1 = stats$Q1,
  Median = stats$Median,
  Mean = stats$Mean,
  Q3 = stats$Q3,
  Max = stats$Max
)
print(summary_stats)

# Stem-and-leaf plot of score distribution
stem(nrs(dataFormat(J15S500)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09B_LDLRA_GA.R
\name{LDLRA_PBIL}
\alias{LDLRA_PBIL}
\title{Structure Learning for LDLRA by PBIL algorithm}
\usage{
LDLRA_PBIL(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  seed = 123,
  ncls = 2,
  method = "R",
  population = 20,
  Rs = 0.5,
  Rm = 0.002,
  maxParents = 2,
  maxGeneration = 100,
  successiveLimit = 5,
  elitism = 0,
  alpha = 0.05,
  estimate = 1,
  filename = NULL,
  verbose = TRUE,
  beta1 = 2,
  beta2 = 2
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{seed}{seed for random.}

\item{ncls}{number of latent class(rank). The default is 2.}

\item{method}{specify the model to analyze the data.Local dependence latent
class model is set to "C", latent rank model is set "R". The default is "R".}

\item{population}{Population size. The default is 20}

\item{Rs}{Survival Rate. The default is 0.5}

\item{Rm}{Mutation Rate. The default is 0.002}

\item{maxParents}{Maximum number of edges emanating from a single node. The default is 2.}

\item{maxGeneration}{Maximum number of generations.}

\item{successiveLimit}{Termination conditions. If the optimal individual does not change
for this number of generations, it is considered to have converged.}

\item{elitism}{Number of elites that remain without crossover when transitioning to
the next generation.}

\item{alpha}{Learning rate. The default is 0.05}

\item{estimate}{In PBIL for estimating the adjacency matrix, specify by number from the
following four methods: 1. Optimal adjacency matrix, 2. Rounded average of individuals in
the last generation, 3. Rounded average of survivors in the last generation, 4. Rounded
generational gene of the last generation. The default is 1.}

\item{filename}{Specify the filename when saving the generated adjacency matrix in CSV format.
The default is null, and no output is written to the file.}

\item{verbose}{verbose output Flag. default is TRUE}

\item{beta1}{Beta distribution parameter 1 for prior density. Default is 2.}

\item{beta2}{Beta distribution parameter 2 for prior density. Default is 2.}
}
\value{
\describe{
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{crr}{correct response ratio}
\item{adj_list}{adjacency matrix list}
\item{g_list}{graph list}
\item{referenceMatrix}{Learned Parameters.A three-dimensional array of patterns where
item x rank x pattern.}
\item{IRP}{Marginal Item Reference Matrix}
\item{IRPIndex}{IRP Indices which include Alpha, Beta, Gamma.}
\item{TRP}{Test Reference Profile matrix.}
\item{LRD}{latent Rank/Class Distribution}
\item{RMD}{Rank/Class Membership Distribution}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
\item{Estimation_table}{Estimated parameters tables.}
\item{CCRR_table}{Correct Response Rate tables}
\item{Studens}{Student information. It includes estimated class
membership, probability of class membership, RUO, and RDO.}
}
}
\description{
Generating DAG list from data using Population-Based Incremental learning
}
\details{
This function performs structural learning for each classes by using
the Population-Based Incremental Learning model(PBIL) proposed by
Fukuda et al.(2014) within the genetic algorithm framework.
Instead of learning the adjacency matrix itself, the 'genes of genes'
that generate the adjacency matrix are updated with each generation.
For more details, please refer to Fukuda(2014) and Section
9.4.3 of the text(Shojima,2022).
}
\examples{
\donttest{
# Perform Structure Learning for LDLRA using PBIL algorithm
# This process may take considerable time due to evolutionary optimization
result.LDLRA.PBIL <- LDLRA_PBIL(J35S515,
  seed = 123, # Set random seed for reproducibility
  ncls = 5, # Number of latent ranks
  maxGeneration = 10,
  method = "R", # Use rank model (vs. class model)
  elitism = 1, # Keep best solution in each generation
  successiveLimit = 15 # Convergence criterion
)

# Examine the learned network structure
# Plot Item Response Profiles showing item patterns across ranks
plot(result.LDLRA.PBIL, type = "IRP", nc = 4, nr = 3)

# Plot Test Response Profile showing overall response patterns
plot(result.LDLRA.PBIL, type = "TRP")

# Plot Latent Rank Distribution showing student distribution
plot(result.LDLRA.PBIL, type = "LRD")
}
}
\references{
Fukuda, S., Yamanaka, Y., & Yoshihiro, T. (2014). A Probability-based evolutionary
algorithm with mutations to learn Bayesian networks. International Journal of Artificial
Intelligence and Interactive Multimedia, 3, 7–13. DOI: 10.9781/ijimai.2014.311
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_QitemFunctions.R
\name{ScoreReport}
\alias{ScoreReport}
\title{Generate Score Report for Non-Binary Test Data}
\usage{
ScoreReport(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item}
}
\value{
An object of class "exametrika" and "TestStatistics" containing:
\describe{
\item{TestLength}{Number of items included in the test}
\item{SampleSize}{Number of examinees (rows) in the dataset}
\item{Mean}{Average score across all examinees}
\item{Median}{Median score}
\item{SD}{Standard deviation of test scores}
\item{Variance}{Variance of test scores}
\item{Skewness}{Skewness of the score distribution (measure of asymmetry)}
\item{Kurtosis}{Kurtosis of the score distribution (measure of tail extremity)}
\item{Min}{Minimum score obtained}
\item{Max}{Maximum score obtained}
\item{Range}{Difference between maximum and minimum scores}
\item{Alpha}{Cronbach's alpha coefficient, a measure of internal consistency reliability}
}
}
\description{
Calculates comprehensive descriptive statistics for a test, including measures of
central tendency, variability, distribution shape, and reliability.
}
\details{
This function is intended for non-binary (ordinal or rated) response data. It calculates
descriptive statistics for the overall test performance. If binary data is provided,
an error message will be displayed.
}
\examples{
\donttest{
# Generate score report for sample ordinal data
ScoreReport(J15S3810)

# Example with rated data
ScoreReport(J35S5000)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J15S3810}
\alias{J15S3810}
\title{J15S3810}
\format{
An exametrika class object with 3810 students and 15 items containing ordinal responses with 4 categories
}
\usage{
J15S3810
}
\description{
An ordinal response dataset for test analysis
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_Biclustering.R
\name{softmax}
\alias{softmax}
\title{softmax function}
\usage{
softmax(x)
}
\arguments{
\item{x}{numeric vector}
}
\description{
to avoid overflow
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{tetrachoric}
\alias{tetrachoric}
\title{Tetrachoric Correlation}
\usage{
tetrachoric(x, y)
}
\arguments{
\item{x}{binary vector x}

\item{y}{binary vector y}
}
\value{
Returns a single numeric value of class "exametrika" representing the
tetrachoric correlation coefficient between the two binary variables. The value
ranges from -1 to 1, where:
\itemize{
\item 1 indicates perfect positive correlation
\item -1 indicates perfect negative correlation
\item 0 indicates no correlation
}
}
\description{
Tetrachoric Correlation is superior to the phi coefficient as a measure of the
relation of an item pair. See Divgi, 1979; Olsson, 1979;Harris, 1988.
}
\references{
Divgi, D. R. (1979). Calculation of the tetrachoric correlation coefficient.
Psychometrika, 44, 169–172.

Olsson, U. (1979). Maximum likelihood estimation of the polychoric correlation
coefficient. Psychometrika,44, 443–460.

Harris, B. (1988). Tetrachoric correlation coefficient. In L. Kotz, & N. L. Johnson
(Eds.), Encyclopedia of statistical sciences (Vol. 9, pp. 223–225). Wiley.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{JSR}
\alias{JSR}
\title{Joint Selection Rate}
\usage{
JSR(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A list of Joint Selection Rate matrices for each item pair.
}
\description{
Calculate the Joint Selection Rate (JSR) for polytomous data.
JSR measures the proportion of respondents who selected specific category
combinations between pairs of items. For each pair of items (j,k),
it returns a contingency table showing the joint probability of selecting
each category combination.
}
\examples{
# example code
# Calculate JCRR using sample dataset J5S1000
JSR(J5S1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04C_ParameterEstimation.R
\name{PSD_item_params}
\alias{PSD_item_params}
\title{internal functions for PSD of Item parameters}
\usage{
PSD_item_params(model, Lambda, quadrature, marginal_posttheta)
}
\arguments{
\item{model}{2,3,or 4PL}

\item{Lambda}{item parameters Matrix}

\item{quadrature}{quads}

\item{marginal_posttheta}{marginal post theta}
}
\description{
internal functions for PSD of Item parameters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{crr}
\alias{crr}
\alias{crr.default}
\alias{crr.binary}
\title{Correct Response Rate}
\usage{
crr(U, na = NULL, Z = NULL, w = NULL)

\method{crr}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{crr}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of weighted correct response rates for each item.
Values range from 0 to 1, where higher values indicate easier items
(more students answered correctly).
}
\description{
The correct response rate (CRR) is one of the most basic and important
statistics for item analysis. This is an index of item difficulty and
a measure of how many students out of those who tried an item correctly
responded to it. This function is applicable only to binary response data.

The CRR for each item is calculated as:
\deqn{p_j = \frac{\sum_{i=1}^n z_{ij}u_{ij}}{\sum_{i=1}^n z_{ij}}}
where \eqn{z_{ij}} is the missing indicator and \eqn{u_{ij}} is the response.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# using sample datasaet
crr(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{PhiCoefficient}
\alias{PhiCoefficient}
\alias{PhiCoefficient.default}
\alias{PhiCoefficient.binary}
\title{Phi-Coefficient}
\usage{
PhiCoefficient(U, na = NULL, Z = NULL, w = NULL)

\method{PhiCoefficient}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{PhiCoefficient}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A matrix of phi coefficients with exametrika class.
Each element (i,j) represents the phi coefficient between items i and j.
The matrix is symmetric with ones on the diagonal.
}
\description{
The phi coefficient is the Pearson's product moment correlation coefficient
between two binary items. This function is applicable only to binary response data.
The coefficient ranges from -1 to 1, where 1 indicates perfect positive correlation,
-1 indicates perfect negative correlation, and 0 indicates no correlation.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# example code
# Calculate Phi-Coefficient using sample dataset J15S500
PhiCoefficient(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_grm.R
\name{generate_start_values}
\alias{generate_start_values}
\title{generate start values for optimize}
\usage{
generate_start_values(tmp)
}
\arguments{
\item{tmp}{dataset}
}
\description{
generate start values for optimize
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_QitemFunctions.R
\name{qBiNormal}
\alias{qBiNormal}
\title{bivariate normal CDF}
\usage{
qBiNormal(a, b, rho)
}
\arguments{
\item{a}{Numeric value, the upper limit for the first variable.}

\item{b}{Numeric value, the upper limit for the second variable.}

\item{rho}{Numeric value between -1 and 1, the correlation coefficient.}
}
\value{
The probability P(X <= a, Y <= b), a value between 0 and 1.
}
\description{
Calculates the cumulative distribution function (CDF) of a bivariate normal distribution.
This function computes P(X <= a, Y <= b) where X and Y follow a bivariate normal
distribution with correlation coefficient rho.
}
\details{
The implementation uses numerical integration with Gauss-Legendre quadrature
for accurate computation. Special cases for infinite bounds are handled separately.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J21S300}
\alias{J21S300}
\title{J21S300}
\format{
An exametrika class object with 300 students and 21 items containing
rated responses (multiple-choice with correct answers) with 4 categories
}
\usage{
J21S300
}
\description{
A simulated rated dataset for Rated IRM and Biclustering analysis.
This is a synthetic dataset generated with a known structure
(5 latent classes, 3 fields, 4 response categories with correct answers)
for demonstration and testing purposes. Each field contains 7 items.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_dataFormat.R
\name{longdataFormat}
\alias{longdataFormat}
\title{Long Format Data Conversion}
\usage{
longdataFormat(
  data,
  na = NULL,
  Sid = NULL,
  Qid = NULL,
  Resp = NULL,
  w = NULL,
  response.type = NULL,
  CA = NULL
)
}
\arguments{
\item{data}{is a data matrix of the type matrix or data.frame. This must
contain at least three columns to identify the student, the item, and
the response. Additionally, it can include a column for the weight of
the items.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{Sid}{Specify the column number containing the student ID label vector.}

\item{Qid}{Specify the column number containing the Question label vector.}

\item{Resp}{Specify the column number containing the Response value vector.}

\item{w}{Specify the column number containing the weight vector.}

\item{response.type}{Character string specifying the type of response data:
"binary" for dichotomous data,
"ordinal" for ordered polytomous data,
"rated" for polytomous data with correct answers,
"nominal" for unordered polytomous data.
If NULL (default), the type is automatically detected.}

\item{CA}{A numeric vector specifying the correct answers for rated polytomous data.
Required when response.type is "rated".}
}
\value{
\describe{
\item{U}{For binary response data. A matrix with rows representing the sample size and columns
representing the number of items, where elements are either 0 or 1. \eqn{u_{ij}=1} indicates
that student i correctly answered item j, while \eqn{u_{ij}=0} means that student i answered
item j incorrectly.}
\item{Q}{For polytomous response data. A matrix with rows representing the sample size and columns
representing the number of items, where elements are non-negative integers. When input data is
in factor format, the factor levels are converted to consecutive integers starting from 1.}
\item{ID}{The ID label given by the designated column or function.}
\item{ItemLabel}{The item names given by the provided column names or function.}
\item{Z}{Missing indicator matrix. \eqn{z_{ij}=1} indicates that item j is presented to Student i,
while \eqn{z_{ij}=0} indicates item j is NOT presented to Student i.}
\item{w}{Item weight vector}
\item{response.type}{Character string indicating the type of response data:
"binary", "ordinal", "rated", or "nominal"}
\item{CategoryLabel}{List containing the original factor labels when polytomous responses
are provided as factors. NULL if no factor data is present.}
\item{categories}{Numeric vector containing the number of response categories for each item.}
\item{CA}{For rated polytomous data, a numeric vector of correct answers. NULL for other types.}
}
}
\description{
A function to reshape long data into a dataset suitable for exametrika.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{PolychoricCorrelationMatrix}
\alias{PolychoricCorrelationMatrix}
\alias{PolychoricCorrelationMatrix.default}
\alias{PolychoricCorrelationMatrix.ordinal}
\title{Polychoric Correlation Matrix}
\usage{
PolychoricCorrelationMatrix(U, na = NULL, Z = NULL, w = NULL)

\method{PolychoricCorrelationMatrix}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{PolychoricCorrelationMatrix}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A matrix of polychoric correlations with exametrika class.
Each element (i,j) represents the polychoric correlation between items i and j.
The matrix is symmetric with ones on the diagonal.
}
\description{
Polychoric Correlation Matrix
}
\examples{
\donttest{
# example code
PolychoricCorrelationMatrix(J5S1000)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_grm.R
\name{grm_prob}
\alias{grm_prob}
\title{Probability function for GRM}
\usage{
grm_prob(theta, a, b)
}
\arguments{
\item{theta}{Latent trait value of the subject}

\item{a}{Discrimination parameter of IRF}

\item{b}{Vector of difficulty parameters (thresholds) of IRF}
}
\value{
Vector of category selection probabilities
}
\description{
Calculates the probability of selecting each category given a latent trait value and item parameters.
}
\examples{
\dontrun{
# Example for an item with 3 categories
a <- 1.5
b <- c(-1.0, 1.0)
theta <- 0
grm_prob(theta, a, b)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/10_LDB.R
\name{LDB}
\alias{LDB}
\title{Local Dependence Biclustering}
\usage{
LDB(
  U,
  Z = NULL,
  w = NULL,
  na = NULL,
  ncls = 2,
  method = "R",
  conf = NULL,
  g_list = NULL,
  adj_list = NULL,
  adj_file = NULL,
  verbose = FALSE,
  beta1 = 1,
  beta2 = 1
)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \link{dataFormat} function.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{ncls}{number of latent class(rank). The default is 2.}

\item{method}{specify the model to analyze the data.Local dependence latent
class model is set to "C", latent rank model is set "R". The default is "R".}

\item{conf}{For the confirmatory parameter, you can input either a vector with
items and corresponding fields in sequence, or a field membership profile
matrix. In the case of the former, the field membership profile matrix will be generated internally.
When providing a membership profile matrix, it needs to be either matrix or data.frame.
The number of fields(nfld) will be overwrite to the number of columns of this matrix.}

\item{g_list}{A list compiling graph-type objects for each rank/class.}

\item{adj_list}{A list compiling matrix-type adjacency matrices for each rank/class.}

\item{adj_file}{A file detailing the relationships of the graph for each rank/class,
listed in the order of starting point, ending point, and rank(class).}

\item{verbose}{verbose output Flag. default is TRUE}

\item{beta1}{Beta distribution parameter 1 for prior density. Default is 1.}

\item{beta2}{Beta distribution parameter 2 for prior density. Default is 1.}
}
\value{
\describe{
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{msg}{A character string indicating the model type. }
\item{Nclass}{Optimal number of classes.}
\item{Nfield}{Optimal number of fields.}
\item{crr}{Correct Response Rate}
\item{ItemLabel}{Label of Items}
\item{FieldLabel}{Label of Fields}
\item{adj_list}{List of Adjacency matrix used in the model}
\item{g_list}{List of graph object used in the model}
\item{IRP}{List of Estimated Parameters. This object is three-dimensional
PIRP array, where each dimension represents the number of rank,number
of field, and Dmax. Dmax denotes the maximum number of correct response
patterns for each field.}
\item{LFD}{Latent Field Distribution. see also \link{plot.exametrika}}
\item{LRD}{Latent Rank Distribution. see also \link{plot.exametrika}}
\item{FRP}{Marginal Field Reference Matrix}
\item{FRPIndex}{Index of FFP includes the item location parameters B and Beta,
the slope parameters A and Alpha, and the monotonicity indices C and Gamma.}
\item{CCRR_table}{This table is a rearrangement of IRP into a data.frame
format for output, consisting of combinations of rank ,field and PIRP.}
\item{TRP}{Test Reference Profile}
\item{RMD}{Rank Membership Distribution.}
\item{FieldEstimated}{Given vector which correspondence between items
and the fields.}
\item{ClassEstimated}{An index indicating which class a student belongs
to, estimated by confirmatory Ranklustering.}
\item{Students}{Rank Membership Profile matrix.The s-th row vector of \eqn{\hat{M}_R}, \eqn{\hat{m}_R}, is the
rank membership profile of Student s, namely the posterior probability distribution representing the student's
belonging to the respective latent classes. It also includes the rank with the maximum estimated membership probability,
as well as the rank-up odds and rank-down odds.}
\item{TestFitIndices}{Overall fit index for the test.See also \link{TestFit}}
}
}
\description{
Latent dependence Biclustering, which incorporates biclustering and a Bayesian
network model.
}
\examples{
\donttest{
# Example: Latent Dirichlet Bayesian Network model
# Create field configuration vector based on field assignments
conf <- c(
  1, 6, 6, 8, 9, 9, 4, 7, 7, 7, 5, 8, 9, 10, 10, 9, 9,
  10, 10, 10, 2, 2, 3, 3, 5, 5, 6, 9, 9, 10, 1, 1, 7, 9, 10
)

# Create edge data for the network structure between fields
edges_data <- data.frame(
  "From Field (Parent) >>>" = c(
    6, 4, 5, 1, 1, 4, # Class/Rank 2
    3, 4, 6, 2, 4, 4, # Class/Rank 3
    3, 6, 4, 1, # Class/Rank 4
    7, 9, 6, 7 # Class/Rank 5
  ),
  ">>> To Field (Child)" = c(
    8, 7, 8, 7, 2, 5, # Class/Rank 2
    5, 8, 8, 4, 6, 7, # Class/Rank 3
    5, 8, 5, 8, # Class/Rank 4
    10, 10, 8, 9 # Class/Rank 5
  ),
  "At Class/Rank (Locus)" = c(
    2, 2, 2, 2, 2, 2, # Class/Rank 2
    3, 3, 3, 3, 3, 3, # Class/Rank 3
    4, 4, 4, 4, # Class/Rank 4
    5, 5, 5, 5 # Class/Rank 5
  )
)

# Save edge data to temporary CSV file
tmp_file <- tempfile(fileext = ".csv")
write.csv(edges_data, file = tmp_file, row.names = FALSE)

# Fit Latent Dirichlet Bayesian Network model
result.LDB <- LDB(
  U = J35S515,
  ncls = 5, # Number of latent classes
  conf = conf, # Field configuration vector
  adj_file = tmp_file # Path to the CSV file
)

# Clean up temporary file
unlink(tmp_file)

# Display model results
print(result.LDB)

# Visualize different aspects of the model
plot(result.LDB, type = "Array") # Show bicluster structure
plot(result.LDB, type = "TRP") # Test Response Profile
plot(result.LDB, type = "LRD") # Latent Rank Distribution
plot(result.LDB,
  type = "RMP", # Rank Membership Profiles
  students = 1:9, nc = 3, nr = 3
)
plot(result.LDB,
  type = "FRP", # Field Reference Profiles
  nc = 3, nr = 2
)
# Field PIRP Profile showing correct answer counts for each rank and field
plot(result.LDB, type = "FieldPIRP")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J15S500}
\alias{J15S500}
\title{J15S500}
\format{
An exametrika class object with 500 students and 15 items containing binary (0/1) responses
}
\source{
\url{http://sh0j1ma.stars.ne.jp/exmk/}
}
\usage{
J15S500
}
\description{
A binary response dataset for test analysis
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08C_BNM_GA.R
\name{StrLearningGA_BNM}
\alias{StrLearningGA_BNM}
\title{StrLearningGA_BNM (Deprecated)}
\usage{
StrLearningGA_BNM(...)
}
\arguments{
\item{...}{All arguments passed to \code{\link{BNM_GA}}}
}
\description{
This function has been renamed to \code{\link{BNM_GA}}.
Please use \code{BNM_GA} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J5S1000}
\alias{J5S1000}
\title{J5S1000}
\format{
An exametrika class object with 1000 students and 5 items containing ordinal responses
}
\usage{
J5S1000
}
\description{
A simulated ordinal dataset for test analysis. This is a synthetic dataset
generated using random number generation for demonstration and testing purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_Biclustering.R, R/15_Biclustering_nominal.R,
%   R/16_Biclustering_ordinal.R, R/19_Biclustering_rated.R
\name{Biclustering}
\alias{Biclustering}
\alias{Biclustering.default}
\alias{Biclustering.binary}
\alias{Biclustering.nominal}
\alias{Biclustering.ordinal}
\alias{Biclustering.rated}
\title{Biclustering and Ranklustering Analysis}
\usage{
Biclustering(U, ...)

\method{Biclustering}{default}(U, na = NULL, Z = NULL, w = NULL, ...)

\method{Biclustering}{binary}(
  U,
  ncls = 2,
  nfld = 2,
  method = "B",
  conf = NULL,
  conf_class = NULL,
  mic = FALSE,
  maxiter = 100,
  verbose = TRUE,
  beta1 = 1,
  beta2 = 1,
  ...
)

\method{Biclustering}{nominal}(
  U,
  ncls = 2,
  nfld = 2,
  conf = NULL,
  conf_class = NULL,
  mic = FALSE,
  maxiter = 100,
  verbose = TRUE,
  alpha = 1,
  ...
)

\method{Biclustering}{ordinal}(
  U,
  ncls = 2,
  nfld = 2,
  method = "B",
  conf = NULL,
  conf_class = NULL,
  mic = FALSE,
  maxiter = 100,
  verbose = TRUE,
  alpha = 1,
  ...
)

\method{Biclustering}{rated}(
  U,
  ncls = 2,
  nfld = 2,
  method = "R",
  conf = NULL,
  maxiter = 100,
  verbose = TRUE,
  alpha = 1,
  ...
)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{...}{Additional arguments passed to specific methods.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}

\item{ncls}{Number of latent classes/ranks to identify (between 2 and 20).}

\item{nfld}{Number of latent fields (item clusters) to identify.}

\item{method}{Analysis method to use (character string):
\itemize{
\item "B" or "Biclustering": Standard biclustering (default)
\item "R" or "Ranklustering": Ranklustering with ordered class structure
}}

\item{conf}{Confirmatory parameter for pre-specified field assignments. Can be either:
\itemize{
\item A vector with items and corresponding fields in sequence
\item A field membership profile matrix (items x fields) with 0/1 values
\item NULL (default) for exploratory analysis where field memberships are estimated
}}

\item{conf_class}{Confirmatory parameter for pre-specified class assignments
on the respondent side. Mirrors \code{conf} but operates on classes. Can be:
\itemize{
\item A vector with respondents and corresponding class indices
\item A class membership profile matrix (respondents × classes) with 0/1 values
\item NULL (default) for exploratory class estimation
When supplied, class memberships are held fixed during EM. For Ranklustering
(\code{method = "R"}), the neighbour-smoothing step is skipped because smoothing
pre-specified labels would defeat the purpose of fixing them.
}}

\item{mic}{Logical; if TRUE, forces Field Reference Profiles to be monotonically
increasing. Default is FALSE.}

\item{maxiter}{Maximum number of EM algorithm iterations. Default is 100.}

\item{verbose}{Logical; if TRUE, displays progress during estimation. Default is TRUE.}

\item{beta1}{Beta distribution parameter 1 for prior density of field reference matrix. Default is 1.}

\item{beta2}{Beta distribution parameter 2 for prior density of field reference matrix. Default is 1.}

\item{alpha}{Dirichlet distribution concentration parameter for prior density of field reference probabilities. Default is 1.}
}
\value{
An object of class "exametrika" and "Biclustering" containing:
\describe{
\item{model}{Model type indicator (1 for biclustering, 2 for ranklustering)}
\item{msg}{A character string indicating the model type. }
\item{mic}{Logical value indicating whether monotonicity constraint was applied}
\item{testlength}{Number of items in the test}
\item{nobs}{Number of examinees in the dataset}
\item{Nclass}{Number of latent classes/ranks specified}
\item{Nfield}{Number of latent fields specified}
\item{N_Cycle}{Number of EM iterations performed}
\item{converge}{Logical value indicating wheter the algorithm converged within maxiter iterasions}
\item{LFD}{Latent Field Distribution - counts of items assigned to each field}
\item{LRD/LCD}{Latent Rank/Class Distribution - counts of examinees assigned to each class/rank}
\item{FRP}{Field Reference Profile matrix - probability of correct response for each field-class combination}
\item{FRPIndex}{Field Reference Profile indices (Kumagai, 2007) — data.frame with 6 columns:
\describe{
\item{Alpha}{Maximum-slope location: class transition where the largest increase occurs}
\item{A}{Maximum slope: largest consecutive-class difference in the profile}
\item{Beta}{Location parameter: class whose FRP value is closest to 0.5}
\item{B}{FRP value at the Beta position}
\item{Gamma}{Non-monotonicity ratio: proportion of class transitions that decrease}
\item{C}{Monotonicity violation: sum of negative consecutive-class differences (C=0 means perfectly monotone)}
}
For binary data, computed directly from the FRP matrix (correct response rates).
For ordinal data, computed from normalized expected scores: (E[score]-1)/(maxQ-1),
which maps expected scores from [1, maxQ] to [0, 1] for comparability with the binary case.
Not computed for nominal data (no natural category ordering).}
\item{TRP}{Test Reference Profile - expected score for examinees in each class/rank}
\item{CMD/RMD}{Class/Rank Membership Distribution - sum of membership probabilities across examinees}
\item{FieldMembership}{Matrix showing the probabilities of each item belonging to each field}
\item{ClassMembership}{Matrix showing the probabilities of each examinee belonging to each class/rank}
\item{SmoothedMembership}{Matrix of smoothed class membership probabilities after filtering}
\item{FieldEstimated}{Vector of the most likely field assignments for each item}
\item{ClassEstimated}{Vector of the most likely class/rank assignments for each examinee}
\item{Students}{Data frame containing membership probabilities and classification information for each examinee}
\item{FieldAnalysis}{Matrix showing field analysis results with item-level information}
\item{TestFitIndices}{Model fit indices for evaluating the quality of the clustering solution}
\item{SOACflg}{Logical flag indicating whether Strongly Ordinal Alignment Condition is satisfied}
\item{WOACflg}{Logical flag indicating whether Weakly Ordinal Alignment Condition is satisfied}
}
}
\description{
Performs biclustering, ranklustering, or their confirmatory variants on binary response data.
These methods simultaneously cluster both examinees and items into homogeneous groups
(or ordered ranks for ranklustering). The analysis reveals latent structures and patterns
in the data by creating a matrix with rows and columns arranged to highlight block structures.
}
\details{
Biclustering simultaneously clusters both rows (examinees) and columns (items) of a data matrix.
Unlike traditional clustering that groups either rows or columns, biclustering identifies
submatrices with similar patterns. Ranklustering is a variant that imposes an ordinal
structure on the classes, making it suitable for proficiency scaling.

The algorithm uses an Expectation-Maximization approach to iteratively estimate:
\enumerate{
\item Field membership of items (which items belong to which fields)
\item Class/rank membership of examinees (which examinees belong to which classes)
\item Field Reference Profiles (probability patterns for each field-class combination)
}

The confirmatory option allows for pre-specified field assignments, which is useful
when there is prior knowledge about item groupings or for testing hypothesized structures.
}
\examples{
\donttest{
# Perform Biclustering with Binary method (B)
# Analyze data with 5 fields and 6 classes
result.Bi <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")

# Perform Biclustering with Rank method (R)
# Store results for further analysis and visualization
result.Rank <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "R")

# Display the Bicluster Reference Matrix (BRM) as a heatmap
plot(result.Rank, type = "Array")

# Plot Field Reference Profiles (FRP) in a 2x3 grid
# Shows the probability patterns for each field
plot(result.Rank, type = "FRP", nc = 2, nr = 3)

# Plot Rank Membership Profiles (RMP) for students 1-9 in a 3x3 grid
# Shows posterior probability distribution of rank membership
plot(result.Rank, type = "RMP", students = 1:9, nc = 3, nr = 3)

# Example of confirmatory analysis with pre-specified fields
# Assign items 1-10 to field 1, 11-20 to field 2, etc.
field_assignments <- c(rep(1, 10), rep(2, 10), rep(3, 15))
result.Conf <- Biclustering(J35S515, nfld = 3, ncls = 5, conf = field_assignments)
}
\donttest{
# Perform Biclustering for nominal sample data()
# Analyze data with 5 fields and 6 classes
result.Bi <- Biclustering(J35S515, nfld = 5, ncls = 6, method = "B")
}
\donttest{
# Perform Biclustering for rated sample data
# Analyze data with 5 fields and 6 classes
result.Bi <- Biclustering(J35S5000, nfld = 5, ncls = 6, method = "R")
}
}
\references{
Shojima, K. (2012). Biclustering of binary data matrices using bilinear models.
Behaviormetrika, 39(2), 161-178.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_exametrikaPlot.R
\name{plot.exametrika}
\alias{plot.exametrika}
\title{Plot Method for Objects of Class "exametrika"}
\usage{
\method{plot}{exametrika}(
  x,
  type = c("IRF", "TRF", "IIF", "TIF", "IIC", "ICC", "TIC", "IRP", "TRP", "LCD", "CMP",
    "FRP", "RMP", "LRD", "Array", "CRV", "RRV", "FCRP", "FCBR", "ScoreField",
    "FieldPIRP", "LDPSR", "ScoreFreq", "ScoreRank", "ICRP", "ICBR"),
  items = NULL,
  students = NULL,
  nc = 1,
  nr = 1,
  overlay = FALSE,
  colors = NULL,
  cell_width = 3,
  cell_height = 1,
  filename = NULL,
  width = 800,
  height = 600,
  dpi = 300,
  stat = "mean",
  style = "line",
  ...
)
}
\arguments{
\item{x}{An object of class "exametrika"}

\item{type}{Character string specifying the plot type. Available types vary by model:
\describe{
\item{IRF, ICC}{Item Response Function. Also known as 'ICC' (Item Characteristic Curve).}
\item{TRF}{Test Response Function.}
\item{IIF, IIC}{Item Information Function. Also known as 'IIC' (Item Information Curve).}
\item{TIF, TIC}{Test Information Function. Also known as 'TIC' (Test Information Curve).}
\item{IRP}{Item Reference Profile. Line graph with items and latent classes/ranks
on the horizontal axis, and membership probability on the vertical axis.}
\item{CRV, RRV}{Class/Rank Reference Vector. Plots correct answer rates for each class or rank,
with fields on the horizontal axis and correct answer rates on the vertical axis.}
\item{TRP}{Test Reference Profile. Shows latent classes/ranks on the horizontal axis,
displaying members per class/rank as a bar graph and expected test scores as a line graph.}
\item{LCD}{Latent Class Distribution. Displays latent classes on the horizontal axis,
showing members per class as a bar graph and cumulative membership probability as a line.}
\item{LRD}{Latent Rank Distribution. Similar to LCD but with ranks instead of classes on the horizontal axis.}
\item{CMP}{Class Membership Profile. Line graph showing class membership probabilities of students.}
\item{RMP}{Rank Membership Profile. Similar to CMP but with ranks instead of classes.}
\item{ScoreFreq}{Frequency polygon of score distribution with rank thresholds.}
\item{ScoreRank}{Heatmap of score membership probabilities for each rank.}
\item{ICRP}{Visualizes ranks (x-axis) versus category response probabilities (y-axis).}
\item{ICBR}{Visualizes ranks (x-axis) versus cumulative category probabilities (y-axis).}
\item{FRP}{Field Reference Profile. Shows correspondence between fields and latent classes/ranks.}
\item{Array}{Array plot for Biclustering/Ranklustering. Colored matrix cells where darker cells
indicate larger values.}
\item{FieldPIRP}{Shows correct response rates by number of correct answers in parent fields.
Only available for LDB model.}
\item{LDPSR}{Latent Dependence Passing Student Rate. Compares passing rates of parent and child classes.}
\item{FCRP}{Field Category Response Profile. Category probability plot per field for polytomous Biclustering.
Use \code{style} parameter to choose between "line" and "bar" display.}
\item{FCBR}{Field Cumulative Boundary Reference. Boundary probability plot per field
(ordinal Biclustering only).}
\item{ScoreField}{Heatmap of expected scores across fields and latent classes/ranks
for polytomous Biclustering.}
}}

\item{items}{Numeric vector specifying which items to plot. If NULL, all items are included.
When type is "IIF"/"IIC", specifying 0 will produce a TIF/TIC for the entire test.}

\item{students}{Numeric vector specifying which students to plot. If NULL, all students are included.}

\item{nc}{Integer specifying the number of columns for multiple plots. Default is 1.}

\item{nr}{Integer specifying the number of rows for multiple plots. Default is 1.}

\item{overlay}{Logical. If TRUE, elements such as IRFs will be overlaid on a single plot. Default is FALSE.}

\item{colors}{Character vector specifying custom color palette. If NULL, default colorblind-friendly palette is used.
For array plots, the first color should be for missing values, followed by response category colors.}

\item{cell_width}{Numeric value specifying the width of each cell in array plots. Default is 3.}

\item{cell_height}{Numeric value specifying the height of each cell in array plots. Default is 1.}

\item{filename}{Character string specifying output filename. If NULL, plot is displayed on screen.
Supported formats: png, pdf, jpeg, jpg. Format determined by file extension.}

\item{width}{Numeric value specifying plot width in pixels (for png/jpeg) or inches (for pdf). Default is 800.}

\item{height}{Numeric value specifying plot height in pixels (for png/jpeg) or inches (for pdf). Default is 600.}

\item{dpi}{Numeric value specifying resolution in dots per inch for raster formats (png/jpeg). Default is 300.}

\item{stat}{Character string specifying the summary statistic for polytomous FRP and RRV plots.
One of "mean" (default), "median", or "mode".}

\item{style}{Character string specifying the display style for FCRP plots.
One of "line" (default) or "bar" (stacked bar chart).}

\item{...}{Additional graphical parameters passed to the underlying base R
plotting functions. Standard parameters such as \code{pch}, \code{las},
\code{cex}, \code{col}, \code{lty}, and \code{lwd} are forwarded
consistently to all plot types (including manually drawn axes), so they
can be used to customise point symbols, axis-label orientation, text
sizes, and other graphical elements. User-supplied values take precedence
over the package defaults (e.g. \code{xlab}, \code{ylab}, \code{main} can
be overridden).}
}
\value{
Produces visualizations based on the model class and specified type:

\describe{
\item{IRT models}{IRF (Item Response Function), TRF (Test Response Function),
IIF (Item Information Function), TIF (Test Information Function)}
\item{LCA/LRA models}{IRP (Item Reference Profile), TRP (Test Reference Profile),
LCD/LRD (Latent Class/Rank Distribution), CMP/RMP (Class/Rank Membership Profile)}
\item{Biclustering/Biclustering_IRM models}{Array plots showing clustering patterns, FRP, TRP, etc.}
\item{LDLRA/LDB/BINET models}{Network and profile plots specific to each model}
}
}
\description{
Creates visualizations for objects with class "exametrika".
The calculation results of the exametrika package have an exametrika class attribute,
along with the specific analysis model class (IRT, GRM, LCA, LRA, Biclustering, Biclustering_IRM, LDLRA, LDB,
BINET). Each model has its own compatible plot types, accessible by specifying the 'type' parameter.
}
\details{
Each model class supports specific plot types:

\describe{
\item{IRT}{Supports "IRF"/"ICC", "TRF", "IIF"/"IIC", "TIF"/"TIC"}
\item{GRM}{Supports "IRF"/"ICC", "IIF"/"IIC", "TIF"/"TIC"}
\item{LCA}{Supports "IRP", "FRP", "TRP", "LCD", "CMP"}
\item{LRA}{Supports "IRP", "FRP", "TRP", "LRD", "RMP"}
\item{LRAordinal}{Supports "ScoreFreq", "ScoreRank", "ICRP", "ICBR", "RMP"}
\item{LRArated}{Supports "ScoreFreq", "ScoreRank", "ICRP", "RMP"}
\item{Biclustering}{Supports "FRP", "TRP", "LCD", "LRD", "CMP", "RMP", "CRV", "RRV", "Array"}
\item{ordinalBiclustering}{Supports "FRP", "FCRP", "FCBR", "LCD", "LRD", "CMP", "RMP", "Array", "ScoreField", "RRV"}
\item{nominalBiclustering}{Supports "FRP", "FCRP", "LCD", "LRD", "CMP", "Array", "ScoreField", "RRV"}
\item{Biclustering_IRM}{Supports "FRP", "TRP", "Array"}
\item{LDLRA}{Supports "IRP", "TRP", "LRD", "RMP"}
\item{LDB}{Supports "FRP", "TRP", "LRD", "RMP", "Array", "FieldPIRP"}
\item{BINET}{Supports "FRP", "TRP", "LRD", "RMP", "Array", "LDPSR"}
}
}
\examples{
\dontrun{
# IRT model example
irt_result <- exametrika::IRT(J15S500)
plot(irt_result, type = "IRF", items = 1:5)
plot(irt_result, type = "TIF")

# LCA model example
lca_result <- exametrika::LCA(U)
plot(lca_result, type = "IRP")
plot(lca_result, type = "LCD")
}

# Array plot with custom output
biclustering_result <- exametrika::Biclustering(J35S515)
# Custom colors and file output
my_colors <- c("#404040", "#E69F00", "#56B4E9", "#009E73", "#F0E442")
plot(biclustering_result, type = "Array", colors = my_colors)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04A_IRTbaseFunctions.R
\name{ItemInformationFunc}
\alias{ItemInformationFunc}
\title{IIF for 4PLM}
\usage{
ItemInformationFunc(a = 1, b, c = 0, d = 1, theta)
}
\arguments{
\item{a}{slope parameter}

\item{b}{location parameter}

\item{c}{lower asymptote parameter}

\item{d}{upper asymptote parameter}

\item{theta}{ability parameter}
}
\value{
Returns a numeric vector representing the item information at each ability
level theta. The information is calculated based on the first derivative of
the log-likelihood of the 4PL model with respect to theta.
}
\description{
Item Information Function for 4PLM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04C_ParameterEstimation.R
\name{objective_function_IRT}
\alias{objective_function_IRT}
\title{Log-likelihood function used in the Maximization Step (M-Step).}
\usage{
objective_function_IRT(lambda, model, qjtrue, qjfalse, quadrature)
}
\arguments{
\item{lambda}{item parameter vector}

\item{model}{2,3,or 4 PL}

\item{qjtrue}{correct resp pattern}

\item{qjfalse}{incorrect resp pattern}

\item{quadrature}{Pattern of a segmented normal distribution.}
}
\description{
Log-likelihood function used in the Maximization Step (M-Step).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{ItemOdds}
\alias{ItemOdds}
\alias{ItemOdds.default}
\alias{ItemOdds.binary}
\title{Item Odds}
\usage{
ItemOdds(U, na = NULL, Z = NULL, w = NULL)

\method{ItemOdds}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemOdds}{binary}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of odds values for each item. Values range from 0 to infinity,
where:
\itemize{
\item odds > 1: correct response more likely than incorrect
\item odds = 1: equally likely
\item odds < 1: incorrect response more likely than correct
}
}
\description{
Item Odds are defined as the ratio of Correct Response Rate to
Incorrect Response Rate:
\deqn{O_j = \frac{p_j}{1-p_j}}
where \eqn{p_j} is the correct response rate for item j.
This function is applicable only to binary response data.

The odds value represents how many times more likely a correct response is
compared to an incorrect response. For example, an odds of 2 means students
are twice as likely to answer correctly as incorrectly.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# using sample dataset
ItemOdds(J5S10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_CTT.R
\name{AlphaIfDel}
\alias{AlphaIfDel}
\title{Alpha Coefficient if Item removed}
\usage{
AlphaIfDel(x, delItem = NULL, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{x}{This should be a data matrix or a Covariance/Phi/Tetrachoric matrix.}

\item{delItem}{Specify the item to be deleted. If NULL, calculations are performed for all cases.}

\item{na}{This parameter identifies the numbers or characters that should be treated as missing values when 'x' is a data matrix.}

\item{Z}{This parameter represents a missing indicator matrix. It is only needed if 'x' is a data matrix.}

\item{w}{This parameter is an item weight vector. It is only required if 'x' is a data matrix.}
}
\description{
This function returns the alpha coefficient when the specified item is excluded.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04C_ParameterEstimation.R
\name{IRT}
\alias{IRT}
\title{Estimating Item parameters using EM algorithm}
\usage{
IRT(U, model = 2, na = NULL, Z = NULL, w = NULL, verbose = TRUE)
}
\arguments{
\item{U}{U is either a data class of exametrika, or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{model}{This argument takes the number of item parameters to be
estimated in the logistic model. It is limited to values 2, 3, or 4.}

\item{na}{na argument specifies the numbers or characters to be treated as missing values.}

\item{Z}{Z is a missing indicator matrix of the type matrix or data.frame}

\item{w}{w is item weight vector}

\item{verbose}{logical; if TRUE, shows progress of iterations (default: TRUE)}
}
\value{
\describe{
\item{model}{number of item parameters you set.}
\item{testlength}{Length of the test. The number of items included in the test.}
\item{nobs}{Sample size. The number of rows in the dataset.}
\item{params}{Matrix containing the estimated item parameters}
\item{Q3mat}{Q3-matrix developed by Yen(1984)}
\item{itemPSD}{Posterior standard deviation of the item parameters}
\item{ability}{Estimated parameters of students ability}
\item{ItemFitIndices}{Fit index for each item.See also \code{\link{ItemFit}}}
\item{TestFitIndices}{Overall fit index for the test.See also \code{\link{TestFit}}}
}
}
\description{
A function for estimating item parameters using the EM algorithm.
}
\details{
Apply the 2, 3, and 4 parameter logistic models to estimate the item and subject populations.
The 4PL model can be described as follows.
\deqn{P(\theta,a_j,b_j,c_j,d_j)= c_j + \frac{d_j -c_j}{1+exp\{-a_j(\theta - b_j)\}}}
\eqn{a_j, b_j, c_j}, and \eqn{d_j} are parameters related to item j, and are parameters that
adjust the logistic curve.
\eqn{a_j} is called the slope parameter, \eqn{b_j} is the location, \eqn{c_j} is the lower asymptote,
and \eqn{d_j} is the upper asymptote parameter.
The model includes lower models, and among the 4PL models, the case where \eqn{d=1} is the 3PL model,
and among the 3PL models, the case where \eqn{c=0} is the 2PL model.
}
\examples{
\donttest{
# Fit a 3-parameter IRT model to the sample dataset
result.IRT <- IRT(J15S500, model = 3)

# Display the first few rows of estimated student abilities
head(result.IRT$ability)

# Plot Item Response Function (IRF) for items 1-6 in a 2x3 grid
plot(result.IRT, type = "IRF", items = 1:6, nc = 2, nr = 3)

# Plot Item Information Function (IIF) for items 1-6 in a 2x3 grid
plot(result.IRT, type = "IIF", items = 1:6, nc = 2, nr = 3)

# Plot the Test Information Function (TIF) for all items
plot(result.IRT, type = "TIF")
}
}
\references{
Yen, W. M. (1984) Applied Psychological Measurement, 8, 125-145.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_CTT.R
\name{OmegaCoefficient}
\alias{OmegaCoefficient}
\title{Omega Coefficient}
\usage{
OmegaCoefficient(x, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{x}{This should be a data matrix or a Covariance/Phi/Tetrachoric matrix.}

\item{na}{This parameter identifies the numbers or characters that should be treated as missing values when 'x' is a data matrix.}

\item{Z}{This parameter represents a missing indicator matrix. It is only needed if 'x' is a data matrix.}

\item{w}{This parameter is an item weight vector. It is only required if 'x' is a data matrix.}
}
\value{
For a correlation/covariance matrix input, returns a single numeric value
representing the omega coefficient. For a data matrix input, returns a list with
three components:
\describe{
\item{OmegaCov}{Omega coefficient calculated from covariance matrix}
\item{OmegaPhi}{Omega coefficient calculated from phi coefficient matrix}
\item{OmegaTetrachoric}{Omega coefficient calculated from tetrachoric correlation matrix}
}
}
\description{
This function computes Tau-Congeneric Measurement, also known as McDonald's tau coefficient, for a given data set.
}
\references{
McDonald, R. P. (1999). Test theory: A unified treatment. Erlbaum.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{Dimensionality}
\alias{Dimensionality}
\alias{Dimensionality.default}
\alias{Dimensionality.binary}
\alias{Dimensionality.rated}
\alias{Dimensionality.ordinal}
\title{Dimensionality}
\usage{
Dimensionality(U, na = NULL, Z = NULL, w = NULL)

\method{Dimensionality}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{Dimensionality}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{Dimensionality}{rated}(U, na = NULL, Z = NULL, w = NULL)

\method{Dimensionality}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
Returns a list of class c("exametrika", "Dimensionality") containing:
\describe{
\item{Component}{Sequence of component numbers}
\item{Eigenvalue}{Eigenvalues of the tetrachoric correlation matrix}
\item{PerOfVar}{Percentage of variance explained by each component}
\item{CumOfPer}{Cumulative percentage of variance explained}
}
}
\description{
The dimensionality is the number of components
the test is measuring.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_grm.R
\name{params_to_target}
\alias{params_to_target}
\title{convert parameters to optimization target}
\usage{
params_to_target(a_vec, b_list)
}
\arguments{
\item{a_vec}{discrimination parameter vector}

\item{b_list}{threshold parameter list}
}
\description{
convert parameters to optimization target
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02C_ItemStatistics.R
\name{ItemStatistics}
\alias{ItemStatistics}
\alias{ItemStatistics.default}
\alias{ItemStatistics.binary}
\alias{ItemStatistics.ordinal}
\title{Simple Item Statistics}
\usage{
ItemStatistics(U, na = NULL, Z = NULL, w = NULL)

\method{ItemStatistics}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemStatistics}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemStatistics}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
For binary data:
\describe{
\item{ItemLabel}{Label identifying each item}
\item{NR}{Number of Respondents for each item}
\item{CRR}{Correct Response Rate denoted as $p_j$.}
\item{ODDs}{Item Odds is the ratio of the correct response rate to the incorrect response rate.
Defined as \eqn{o_j = \frac{p_j}{1-p_j}}}
\item{Threshold}{Item Threshold is a measure of difficulty based on a standard normal distribution.}
\item{Entropy}{Item Entropy is an indicator of the variability or randomness of the responses.
Defined as \eqn{e_j=-p_j \log_2 p_j - (1-p_j)\log_2(1-p_j)}}
\item{ITCrr}{Item-total Correlation is a Pearson's correlation of an item with the Number-Right score.}
}

For ordinal polytomous data:
\describe{
\item{ItemLabel}{Label identifying each item}
\item{NR}{Number of Respondents for each item}
\item{Threshold}{Matrix of threshold values for each item's category boundaries, based on
a standard normal distribution. For an item with K categories, there are K-1 thresholds.}
\item{Entropy}{Item Entropy calculated using the category probabilities. Unlike binary data,
this is calculated using the formula \eqn{e_j = -\sum_{k=1}^{K_j} p_{jk} \log_{K_j} p_{jk}},
where \eqn{K_j} is the number of categories for item j.}
\item{ITCrr}{Item-total Correlation calculated using polyserial correlation, which
accounts for the ordinal nature of the item responses and the continuous total score.}
}
}
\description{
This function calculates statistics for each item, with different metrics available
depending on the data type (binary, ordinal, or rated).
}
\note{
For rated data, the function processes the data as binary, with each response
being compared to the correct answer to determine correctness.
}
\examples{
# using sample dataset(binary)
ItemStatistics(J15S500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J35S5000}
\alias{J35S5000}
\title{J35S5000}
\format{
An exametrika class object with 5000 students and 35 items containing polytomous responses with correct answers
}
\usage{
J35S5000
}
\description{
A rated response dataset for test analysis
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{CCRR}
\alias{CCRR}
\alias{CCRR.default}
\alias{CCRR.binary}
\alias{CCRR.nominal}
\title{Conditional Correct Response Rate}
\usage{
CCRR(U, na = NULL, Z = NULL, w = NULL)

\method{CCRR}{default}(U, na = NULL, Z = NULL, w = NULL)

\method{CCRR}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{CCRR}{nominal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A matrix of conditional correct response rates with exametrika class.
Each element (i,j) represents the probability of correctly answering item j
given that item i was answered correctly.
}
\description{
The conditional correct response rate (CCRR) represents the ratio of the students
who passed Item C (consequent item) to those who passed Item A (antecedent item).
This function is applicable only to binary response data.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# example code
# Calculate CCRR using sample dataset J5S10
CCRR(J5S10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_TestItemFunctions.R
\name{ItemThreshold}
\alias{ItemThreshold}
\alias{ItemThreshold.binary}
\alias{ItemThreshold.ordinal}
\title{Item Threshold}
\usage{
ItemThreshold(U, na = NULL, Z = NULL, w = NULL)

\method{ItemThreshold}{binary}(U, na = NULL, Z = NULL, w = NULL)

\method{ItemThreshold}{ordinal}(U, na = NULL, Z = NULL, w = NULL)
}
\arguments{
\item{U}{Either an object of class "exametrika" or raw data. When raw data is given,
it is converted to the exametrika class with the \code{\link{dataFormat}} function.}

\item{na}{Values to be treated as missing values.}

\item{Z}{Missing indicator matrix of type matrix or data.frame. Values of 1 indicate
observed responses, while 0 indicates missing data.}

\item{w}{Item weight vector specifying the relative importance of each item.}
}
\value{
A numeric vector of threshold values for each item on the standard normal scale.
Typical values range from about -3 to 3, where:
\itemize{
\item Positive values indicate difficult items
\item Zero indicates items of medium difficulty (50\% correct)
\item Negative values indicate easy items
}
}
\description{
Item threshold is a measure of difficulty based on a standard normal distribution.
This function is applicable only to binary response data.

The threshold is calculated as:
\deqn{\tau_j = \Phi^{-1}(1-p_j)}
where \eqn{\Phi^{-1}} is the inverse standard normal distribution function
and \eqn{p_j} is the correct response rate for item j.

Higher threshold values indicate more difficult items, as they represent the
point on the standard normal scale above which examinees tend to answer incorrectly.
}
\note{
This function is implemented using a binary data compatibility wrapper and
will raise an error if used with polytomous data.
}
\examples{
# using sample dataset
ItemThreshold(J5S10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_dataset_explain.R
\docType{data}
\name{J20S600}
\alias{J20S600}
\title{J20S600}
\format{
An exametrika class object with 600 students and 20 items containing nominal responses with 4 categories
}
\usage{
J20S600
}
\description{
A simulated nominal dataset for polytomous Biclustering analysis.
This is a synthetic dataset generated using random number generation
with a cyclic category preference pattern (5 latent classes, 4 fields,
4 response categories) for demonstration and testing purposes.
Contains approximately 0.5\\% missing values.
}
\keyword{datasets}
